# Copyright 2020-2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Modelarts notebook ignore_warning."""
from mindinsight.modelarts.adapter_notebook.base_notebook import BaseNotebook

def ignore_warning(exception):
    """Ignore warning."""
    return exception

def get_notebook_registry():
    """Get notebook registry."""
    is_adapter_success = False

    try:
        from mindinsight.modelarts.base.get_env import build_adapter
    except ModuleNotFoundError as e:
        ignore_warning(e)
    else:
        is_adapter_success = True

    return build_adapter() if is_adapter_success else BaseNotebook()
