# Copyright 2019 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Base processor, and init data manager parameter."""


class BaseProcessor:
    """Base processors processor. All processors should inherit this class."""

    def __init__(self, data_manager):
        """
        Init image processor.

        Args:
            data_manager (DataManager): A DataManager instance.
        """
        self._data_manager = data_manager
