.profiling-dashboard{height:100%;background-color:var(--bg-color);--tabs-height:48px;padding:0 32px 24px 32px}.profiling-dashboard .profiling-dashboard-head{height:var(--tabs-height);position:relative}.profiling-dashboard .profiling-dashboard-head .path{position:absolute;font-weight:700;top:12px}.profiling-dashboard .profiling-dashboard-head .el-tabs{height:100%}.profiling-dashboard .profiling-dashboard-head .el-tabs__item{font-size:18px;font-weight:700}.profiling-dashboard .profiling-dashboard-content{height:calc(100% - var(--tabs-height))}.profiling-dashboard .profiling-dashboard-content .profiling-content-title{display:-webkit-box;display:-ms-flexbox;display:flex;font-size:16px;font-weight:700;line-height:24px;-webkit-box-align:center;-ms-flex-align:center;align-items:center;margin-bottom:6px}.profiling-dashboard .profiling-dashboard-tab{display:-webkit-box;display:-ms-flexbox;display:flex;height:100%;background:var(--bg-color);position:relative}.profiling-dashboard .profiling-dashboard-tab .el-tabs__item{height:30px;line-height:normal}.profiling-dashboard .profiling-dashboard-tab .dashboard-close-detail{position:absolute;top:6px;right:0;cursor:pointer}.profiling-dashboard .profiling-dashboard-tab .dashboard-content{-webkit-box-flex:1;-ms-flex-positive:1;flex-grow:1;overflow:hidden}