.operator-shape-step{width:100%;height:100%}.operator-shape-step .shape-step{margin:20px 0}.operator-shape-step .operator-shape-option{line-height:25px;height:25px;margin:0 auto;text-align:center}.operator-shape-step .operator-shape-option .operator-filter-title{color:#00a5a7;margin:0 10px}.operator-shape-step .operator-shape-select .operator-detail-select{border-radius:10%;width:22%;line-height:30px;height:30px;margin:0 auto}.operator-shape-step .shape-step .shape-step-title{height:40px;line-height:40px;font-size:16px;font-weight:700;display:-webkit-box;display:-ms-flexbox;display:flex}.operator-shape-step .shape-step .shape-step-title .el-icon-info{line-height:40px;margin:0 5px}.operator-shape-step .shape-step .item{margin-right:10px;font-size:20px;color:#6c7280;cursor:pointer}.operator-shape-step .shape-step .el-input{width:30%;margin:0 50px}.operator-shape-step .shape-step .shape-step-right{margin-left:35px}.operator-shape-step .operator-shape-detail{display:-webkit-box;display:-ms-flexbox;display:flex;width:100%;margin:50px 0;height:calc(100% - 200px);border:1px solid var(--border-color);-webkit-box-flex:1;-ms-flex-positive:1;flex-grow:1;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;overflow-x:auto;overflow-y:auto}#operatorShapeDetailChart{height:80%;width:100%;margin:0 auto}.operator-shape-step .operator-shape-title{height:20px;font-size:15px;font-weight:700;margin-top:12px;padding:16px}.define-chart-tip{display:inline-block;margin-right:5px;width:10px;height:10px}.operator-shape-step .operator-shape-dialog .el-dialog{position:relative;top:14%;height:42%;overflow-y:hidden}.operator-shape-step .image-noData{width:100%;height:calc(100% - 10px);display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:top;-ms-flex-pack:top;justify-content:top;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.operator-shape-step .image-noData p{font-size:16px;padding-top:10px;text-align:center}