.cl-cluster{height:100%;background-color:var(--bg-color)}.cl-cluster .cl-cluster-bk{height:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.step-trace-tooltip-contain .step-trace-tooltip-contain-item{line-height:20px;margin-bottom:3px}.step-trace-tooltip-contain .step-trace-tooltip-contain-item span.label{font-weight:700;margin-right:3px}.cl-cluster .no-data-img{background:var(--bg-color);text-align:center;position:absolute;top:56px;bottom:0;width:100%;z-index:999;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.cl-cluster .no-data-img p{font-size:16px;padding-top:10px}.cl-cluster .el-table th>.cell{color:var(--font-color)}.cl-cluster .el-table td>.cell{margin-left:10px}.cl-cluster .el-table td:first-child .cell{margin-left:0}.cl-cluster .el-table th{-webkit-user-select:auto;-moz-user-select:auto;-ms-user-select:auto;user-select:auto}.cl-cluster .thSpan{color:#d4d9e6;margin-right:8px}.cl-cluster .col-name{max-width:calc(100% - 25px);vertical-align:middle}.cl-cluster .cl-cluster-chart{height:280px;margin-top:5px;-ms-flex-negative:0;flex-shrink:0}.cl-cluster .cl-cluster-table{-webkit-box-flex:1;-ms-flex:1;flex:1;margin-top:20px;overflow:hidden}.cl-cluster .cl-cluster-page{padding-top:10px;text-align:right}.cl-cluster .cl-step-filter .cl-step-filter{display:inline-block}.cl-cluster .cl-step-filter .el-input{width:120px;margin:0 20px}.cl-cluster .cl-step-filter .el-button{border:1px solid #00a5a7;border-radius:2px;background-color:var(--bg-color);color:var(--theme-color);padding:7px 15px;margin-right:20px}