# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mindinsight_lineage.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='mindinsight_lineage.proto',
  package='mindinsight',
  syntax='proto2',
  serialized_pb=_b('\n\x19mindinsight_lineage.proto\x12\x0bmindinsight\"\xab\x02\n\x0cLineageEvent\x12\x11\n\twall_time\x18\x01 \x02(\x01\x12\x0c\n\x04step\x18\x02 \x01(\x03\x12\x11\n\x07version\x18\x03 \x01(\tH\x00\x12\x32\n\rtrain_lineage\x18\x06 \x01(\x0b\x32\x19.mindinsight.TrainLineageH\x00\x12<\n\x12\x65valuation_lineage\x18\x07 \x01(\x0b\x32\x1e.mindinsight.EvaluationLineageH\x00\x12\x32\n\rdataset_graph\x18\t \x01(\x0b\x32\x19.mindinsight.DatasetGraphH\x00\x12\x39\n\x11user_defined_info\x18\n \x01(\x0b\x32\x1c.mindinsight.UserDefinedInfoH\x00\x42\x06\n\x04what\"\x9a\x04\n\x0fUserDefinedInfo\x12/\n\tuser_info\x18\x01 \x03(\x0b\x32\x1c.mindinsight.UserDefinedInfo\x12;\n\x08map_dict\x18\x02 \x03(\x0b\x32).mindinsight.UserDefinedInfo.MapDictEntry\x12=\n\tmap_int32\x18\x03 \x03(\x0b\x32*.mindinsight.UserDefinedInfo.MapInt32Entry\x12\x39\n\x07map_str\x18\x04 \x03(\x0b\x32(.mindinsight.UserDefinedInfo.MapStrEntry\x12?\n\nmap_double\x18\x05 \x03(\x0b\x32+.mindinsight.UserDefinedInfo.MapDoubleEntry\x1aL\n\x0cMapDictEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12+\n\x05value\x18\x02 \x01(\x0b\x32\x1c.mindinsight.UserDefinedInfo:\x02\x38\x01\x1a/\n\rMapInt32Entry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a-\n\x0bMapStrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x30\n\x0eMapDoubleEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\"\xb6\x04\n\x0cTrainLineage\x12\x43\n\x10hyper_parameters\x18\x01 \x01(\x0b\x32).mindinsight.TrainLineage.HyperParameters\x12=\n\rtrain_dataset\x18\x02 \x01(\x0b\x32&.mindinsight.TrainLineage.TrainDataset\x12\x36\n\talgorithm\x18\x03 \x01(\x0b\x32#.mindinsight.TrainLineage.Algorithm\x12.\n\x05model\x18\x04 \x01(\x0b\x32\x1f.mindinsight.TrainLineage.Model\x1a\xa0\x01\n\x0fHyperParameters\x12\x11\n\toptimizer\x18\x01 \x01(\t\x12\x15\n\rlearning_rate\x18\x02 \x01(\x02\x12\x15\n\rloss_function\x18\x03 \x01(\t\x12\r\n\x05\x65poch\x18\x04 \x01(\x05\x12\x15\n\rparallel_mode\x18\x05 \x01(\t\x12\x12\n\ndevice_num\x18\x06 \x01(\x05\x12\x12\n\nbatch_size\x18\x08 \x01(\x05\x1a\x46\n\x0cTrainDataset\x12\x1a\n\x12train_dataset_path\x18\x01 \x01(\t\x12\x1a\n\x12train_dataset_size\x18\x02 \x01(\x05\x1a*\n\tAlgorithm\x12\x0f\n\x07network\x18\x01 \x01(\t\x12\x0c\n\x04loss\x18\x02 \x01(\x02\x1a#\n\x05Model\x12\x0c\n\x04path\x18\x03 \x01(\t\x12\x0c\n\x04size\x18\x04 \x01(\x03\"\xaf\x01\n\x11\x45valuationLineage\x12\x0e\n\x06metric\x18\x02 \x01(\t\x12\x42\n\rvalid_dataset\x18\x03 \x01(\x0b\x32+.mindinsight.EvaluationLineage.ValidDataset\x1a\x46\n\x0cValidDataset\x12\x1a\n\x12valid_dataset_path\x18\x01 \x01(\t\x12\x1a\n\x12valid_dataset_size\x18\x02 \x01(\x05\"\xc4\x01\n\x0c\x44\x61tasetGraph\x12+\n\x08\x63hildren\x18\x01 \x03(\x0b\x32\x19.mindinsight.DatasetGraph\x12\x32\n\tparameter\x18\x02 \x01(\x0b\x32\x1f.mindinsight.OperationParameter\x12*\n\noperations\x18\x03 \x03(\x0b\x32\x16.mindinsight.Operation\x12\'\n\x07sampler\x18\x04 \x01(\x0b\x32\x16.mindinsight.Operation\"c\n\tOperation\x12\x37\n\x0eoperationParam\x18\x01 \x01(\x0b\x32\x1f.mindinsight.OperationParameter\x12\x0c\n\x04size\x18\x02 \x03(\x05\x12\x0f\n\x07weights\x18\x03 \x03(\x02\"\xde\x04\n\x12OperationParameter\x12;\n\x06mapStr\x18\x01 \x03(\x0b\x32+.mindinsight.OperationParameter.MapStrEntry\x12\x43\n\nmapStrList\x18\x02 \x03(\x0b\x32/.mindinsight.OperationParameter.MapStrListEntry\x12=\n\x07mapBool\x18\x03 \x03(\x0b\x32,.mindinsight.OperationParameter.MapBoolEntry\x12;\n\x06mapInt\x18\x04 \x03(\x0b\x32+.mindinsight.OperationParameter.MapIntEntry\x12\x41\n\tmapDouble\x18\x05 \x03(\x0b\x32..mindinsight.OperationParameter.MapDoubleEntry\x1a-\n\x0bMapStrEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1aG\n\x0fMapStrListEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.mindinsight.StrList:\x02\x38\x01\x1a.\n\x0cMapBoolEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x08:\x02\x38\x01\x1a-\n\x0bMapIntEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\x1a\x30\n\x0eMapDoubleEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x01:\x02\x38\x01\"\x1b\n\x07StrList\x12\x10\n\x08strValue\x18\x01 \x03(\tB\x03\xf8\x01\x01')
)
_sym_db.RegisterFileDescriptor(DESCRIPTOR)




_LINEAGEEVENT = _descriptor.Descriptor(
  name='LineageEvent',
  full_name='mindinsight.LineageEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='wall_time', full_name='mindinsight.LineageEvent.wall_time', index=0,
      number=1, type=1, cpp_type=5, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='step', full_name='mindinsight.LineageEvent.step', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='version', full_name='mindinsight.LineageEvent.version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='train_lineage', full_name='mindinsight.LineageEvent.train_lineage', index=3,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='evaluation_lineage', full_name='mindinsight.LineageEvent.evaluation_lineage', index=4,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='dataset_graph', full_name='mindinsight.LineageEvent.dataset_graph', index=5,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='user_defined_info', full_name='mindinsight.LineageEvent.user_defined_info', index=6,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='what', full_name='mindinsight.LineageEvent.what',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=43,
  serialized_end=342,
)


_USERDEFINEDINFO_MAPDICTENTRY = _descriptor.Descriptor(
  name='MapDictEntry',
  full_name='mindinsight.UserDefinedInfo.MapDictEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.UserDefinedInfo.MapDictEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.UserDefinedInfo.MapDictEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=661,
  serialized_end=737,
)

_USERDEFINEDINFO_MAPINT32ENTRY = _descriptor.Descriptor(
  name='MapInt32Entry',
  full_name='mindinsight.UserDefinedInfo.MapInt32Entry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.UserDefinedInfo.MapInt32Entry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.UserDefinedInfo.MapInt32Entry.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=739,
  serialized_end=786,
)

_USERDEFINEDINFO_MAPSTRENTRY = _descriptor.Descriptor(
  name='MapStrEntry',
  full_name='mindinsight.UserDefinedInfo.MapStrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.UserDefinedInfo.MapStrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.UserDefinedInfo.MapStrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=788,
  serialized_end=833,
)

_USERDEFINEDINFO_MAPDOUBLEENTRY = _descriptor.Descriptor(
  name='MapDoubleEntry',
  full_name='mindinsight.UserDefinedInfo.MapDoubleEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.UserDefinedInfo.MapDoubleEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.UserDefinedInfo.MapDoubleEntry.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=835,
  serialized_end=883,
)

_USERDEFINEDINFO = _descriptor.Descriptor(
  name='UserDefinedInfo',
  full_name='mindinsight.UserDefinedInfo',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_info', full_name='mindinsight.UserDefinedInfo.user_info', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map_dict', full_name='mindinsight.UserDefinedInfo.map_dict', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map_int32', full_name='mindinsight.UserDefinedInfo.map_int32', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map_str', full_name='mindinsight.UserDefinedInfo.map_str', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='map_double', full_name='mindinsight.UserDefinedInfo.map_double', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_USERDEFINEDINFO_MAPDICTENTRY, _USERDEFINEDINFO_MAPINT32ENTRY, _USERDEFINEDINFO_MAPSTRENTRY, _USERDEFINEDINFO_MAPDOUBLEENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=345,
  serialized_end=883,
)


_TRAINLINEAGE_HYPERPARAMETERS = _descriptor.Descriptor(
  name='HyperParameters',
  full_name='mindinsight.TrainLineage.HyperParameters',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='optimizer', full_name='mindinsight.TrainLineage.HyperParameters.optimizer', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='learning_rate', full_name='mindinsight.TrainLineage.HyperParameters.learning_rate', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='loss_function', full_name='mindinsight.TrainLineage.HyperParameters.loss_function', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='epoch', full_name='mindinsight.TrainLineage.HyperParameters.epoch', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parallel_mode', full_name='mindinsight.TrainLineage.HyperParameters.parallel_mode', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='device_num', full_name='mindinsight.TrainLineage.HyperParameters.device_num', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='batch_size', full_name='mindinsight.TrainLineage.HyperParameters.batch_size', index=6,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1139,
  serialized_end=1299,
)

_TRAINLINEAGE_TRAINDATASET = _descriptor.Descriptor(
  name='TrainDataset',
  full_name='mindinsight.TrainLineage.TrainDataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='train_dataset_path', full_name='mindinsight.TrainLineage.TrainDataset.train_dataset_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='train_dataset_size', full_name='mindinsight.TrainLineage.TrainDataset.train_dataset_size', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1301,
  serialized_end=1371,
)

_TRAINLINEAGE_ALGORITHM = _descriptor.Descriptor(
  name='Algorithm',
  full_name='mindinsight.TrainLineage.Algorithm',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='network', full_name='mindinsight.TrainLineage.Algorithm.network', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='loss', full_name='mindinsight.TrainLineage.Algorithm.loss', index=1,
      number=2, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1373,
  serialized_end=1415,
)

_TRAINLINEAGE_MODEL = _descriptor.Descriptor(
  name='Model',
  full_name='mindinsight.TrainLineage.Model',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='path', full_name='mindinsight.TrainLineage.Model.path', index=0,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='size', full_name='mindinsight.TrainLineage.Model.size', index=1,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1417,
  serialized_end=1452,
)

_TRAINLINEAGE = _descriptor.Descriptor(
  name='TrainLineage',
  full_name='mindinsight.TrainLineage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hyper_parameters', full_name='mindinsight.TrainLineage.hyper_parameters', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='train_dataset', full_name='mindinsight.TrainLineage.train_dataset', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='algorithm', full_name='mindinsight.TrainLineage.algorithm', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='model', full_name='mindinsight.TrainLineage.model', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_TRAINLINEAGE_HYPERPARAMETERS, _TRAINLINEAGE_TRAINDATASET, _TRAINLINEAGE_ALGORITHM, _TRAINLINEAGE_MODEL, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=886,
  serialized_end=1452,
)


_EVALUATIONLINEAGE_VALIDDATASET = _descriptor.Descriptor(
  name='ValidDataset',
  full_name='mindinsight.EvaluationLineage.ValidDataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='valid_dataset_path', full_name='mindinsight.EvaluationLineage.ValidDataset.valid_dataset_path', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='valid_dataset_size', full_name='mindinsight.EvaluationLineage.ValidDataset.valid_dataset_size', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1560,
  serialized_end=1630,
)

_EVALUATIONLINEAGE = _descriptor.Descriptor(
  name='EvaluationLineage',
  full_name='mindinsight.EvaluationLineage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='metric', full_name='mindinsight.EvaluationLineage.metric', index=0,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='valid_dataset', full_name='mindinsight.EvaluationLineage.valid_dataset', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_EVALUATIONLINEAGE_VALIDDATASET, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1455,
  serialized_end=1630,
)


_DATASETGRAPH = _descriptor.Descriptor(
  name='DatasetGraph',
  full_name='mindinsight.DatasetGraph',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='children', full_name='mindinsight.DatasetGraph.children', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='parameter', full_name='mindinsight.DatasetGraph.parameter', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='operations', full_name='mindinsight.DatasetGraph.operations', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='sampler', full_name='mindinsight.DatasetGraph.sampler', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1633,
  serialized_end=1829,
)


_OPERATION = _descriptor.Descriptor(
  name='Operation',
  full_name='mindinsight.Operation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='operationParam', full_name='mindinsight.Operation.operationParam', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='size', full_name='mindinsight.Operation.size', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='weights', full_name='mindinsight.Operation.weights', index=2,
      number=3, type=2, cpp_type=6, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1831,
  serialized_end=1930,
)


_OPERATIONPARAMETER_MAPSTRENTRY = _descriptor.Descriptor(
  name='MapStrEntry',
  full_name='mindinsight.OperationParameter.MapStrEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.OperationParameter.MapStrEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.OperationParameter.MapStrEntry.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=788,
  serialized_end=833,
)

_OPERATIONPARAMETER_MAPSTRLISTENTRY = _descriptor.Descriptor(
  name='MapStrListEntry',
  full_name='mindinsight.OperationParameter.MapStrListEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.OperationParameter.MapStrListEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.OperationParameter.MapStrListEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2323,
  serialized_end=2394,
)

_OPERATIONPARAMETER_MAPBOOLENTRY = _descriptor.Descriptor(
  name='MapBoolEntry',
  full_name='mindinsight.OperationParameter.MapBoolEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.OperationParameter.MapBoolEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.OperationParameter.MapBoolEntry.value', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2396,
  serialized_end=2442,
)

_OPERATIONPARAMETER_MAPINTENTRY = _descriptor.Descriptor(
  name='MapIntEntry',
  full_name='mindinsight.OperationParameter.MapIntEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.OperationParameter.MapIntEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.OperationParameter.MapIntEntry.value', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2444,
  serialized_end=2489,
)

_OPERATIONPARAMETER_MAPDOUBLEENTRY = _descriptor.Descriptor(
  name='MapDoubleEntry',
  full_name='mindinsight.OperationParameter.MapDoubleEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='mindinsight.OperationParameter.MapDoubleEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='value', full_name='mindinsight.OperationParameter.MapDoubleEntry.value', index=1,
      number=2, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=_descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001')),
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=835,
  serialized_end=883,
)

_OPERATIONPARAMETER = _descriptor.Descriptor(
  name='OperationParameter',
  full_name='mindinsight.OperationParameter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='mapStr', full_name='mindinsight.OperationParameter.mapStr', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mapStrList', full_name='mindinsight.OperationParameter.mapStrList', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mapBool', full_name='mindinsight.OperationParameter.mapBool', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mapInt', full_name='mindinsight.OperationParameter.mapInt', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    _descriptor.FieldDescriptor(
      name='mapDouble', full_name='mindinsight.OperationParameter.mapDouble', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[_OPERATIONPARAMETER_MAPSTRENTRY, _OPERATIONPARAMETER_MAPSTRLISTENTRY, _OPERATIONPARAMETER_MAPBOOLENTRY, _OPERATIONPARAMETER_MAPINTENTRY, _OPERATIONPARAMETER_MAPDOUBLEENTRY, ],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1933,
  serialized_end=2539,
)


_STRLIST = _descriptor.Descriptor(
  name='StrList',
  full_name='mindinsight.StrList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='strValue', full_name='mindinsight.StrList.strValue', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2541,
  serialized_end=2568,
)

_LINEAGEEVENT.fields_by_name['train_lineage'].message_type = _TRAINLINEAGE
_LINEAGEEVENT.fields_by_name['evaluation_lineage'].message_type = _EVALUATIONLINEAGE
_LINEAGEEVENT.fields_by_name['dataset_graph'].message_type = _DATASETGRAPH
_LINEAGEEVENT.fields_by_name['user_defined_info'].message_type = _USERDEFINEDINFO
_LINEAGEEVENT.oneofs_by_name['what'].fields.append(
  _LINEAGEEVENT.fields_by_name['version'])
_LINEAGEEVENT.fields_by_name['version'].containing_oneof = _LINEAGEEVENT.oneofs_by_name['what']
_LINEAGEEVENT.oneofs_by_name['what'].fields.append(
  _LINEAGEEVENT.fields_by_name['train_lineage'])
_LINEAGEEVENT.fields_by_name['train_lineage'].containing_oneof = _LINEAGEEVENT.oneofs_by_name['what']
_LINEAGEEVENT.oneofs_by_name['what'].fields.append(
  _LINEAGEEVENT.fields_by_name['evaluation_lineage'])
_LINEAGEEVENT.fields_by_name['evaluation_lineage'].containing_oneof = _LINEAGEEVENT.oneofs_by_name['what']
_LINEAGEEVENT.oneofs_by_name['what'].fields.append(
  _LINEAGEEVENT.fields_by_name['dataset_graph'])
_LINEAGEEVENT.fields_by_name['dataset_graph'].containing_oneof = _LINEAGEEVENT.oneofs_by_name['what']
_LINEAGEEVENT.oneofs_by_name['what'].fields.append(
  _LINEAGEEVENT.fields_by_name['user_defined_info'])
_LINEAGEEVENT.fields_by_name['user_defined_info'].containing_oneof = _LINEAGEEVENT.oneofs_by_name['what']
_USERDEFINEDINFO_MAPDICTENTRY.fields_by_name['value'].message_type = _USERDEFINEDINFO
_USERDEFINEDINFO_MAPDICTENTRY.containing_type = _USERDEFINEDINFO
_USERDEFINEDINFO_MAPINT32ENTRY.containing_type = _USERDEFINEDINFO
_USERDEFINEDINFO_MAPSTRENTRY.containing_type = _USERDEFINEDINFO
_USERDEFINEDINFO_MAPDOUBLEENTRY.containing_type = _USERDEFINEDINFO
_USERDEFINEDINFO.fields_by_name['user_info'].message_type = _USERDEFINEDINFO
_USERDEFINEDINFO.fields_by_name['map_dict'].message_type = _USERDEFINEDINFO_MAPDICTENTRY
_USERDEFINEDINFO.fields_by_name['map_int32'].message_type = _USERDEFINEDINFO_MAPINT32ENTRY
_USERDEFINEDINFO.fields_by_name['map_str'].message_type = _USERDEFINEDINFO_MAPSTRENTRY
_USERDEFINEDINFO.fields_by_name['map_double'].message_type = _USERDEFINEDINFO_MAPDOUBLEENTRY
_TRAINLINEAGE_HYPERPARAMETERS.containing_type = _TRAINLINEAGE
_TRAINLINEAGE_TRAINDATASET.containing_type = _TRAINLINEAGE
_TRAINLINEAGE_ALGORITHM.containing_type = _TRAINLINEAGE
_TRAINLINEAGE_MODEL.containing_type = _TRAINLINEAGE
_TRAINLINEAGE.fields_by_name['hyper_parameters'].message_type = _TRAINLINEAGE_HYPERPARAMETERS
_TRAINLINEAGE.fields_by_name['train_dataset'].message_type = _TRAINLINEAGE_TRAINDATASET
_TRAINLINEAGE.fields_by_name['algorithm'].message_type = _TRAINLINEAGE_ALGORITHM
_TRAINLINEAGE.fields_by_name['model'].message_type = _TRAINLINEAGE_MODEL
_EVALUATIONLINEAGE_VALIDDATASET.containing_type = _EVALUATIONLINEAGE
_EVALUATIONLINEAGE.fields_by_name['valid_dataset'].message_type = _EVALUATIONLINEAGE_VALIDDATASET
_DATASETGRAPH.fields_by_name['children'].message_type = _DATASETGRAPH
_DATASETGRAPH.fields_by_name['parameter'].message_type = _OPERATIONPARAMETER
_DATASETGRAPH.fields_by_name['operations'].message_type = _OPERATION
_DATASETGRAPH.fields_by_name['sampler'].message_type = _OPERATION
_OPERATION.fields_by_name['operationParam'].message_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER_MAPSTRENTRY.containing_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER_MAPSTRLISTENTRY.fields_by_name['value'].message_type = _STRLIST
_OPERATIONPARAMETER_MAPSTRLISTENTRY.containing_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER_MAPBOOLENTRY.containing_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER_MAPINTENTRY.containing_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER_MAPDOUBLEENTRY.containing_type = _OPERATIONPARAMETER
_OPERATIONPARAMETER.fields_by_name['mapStr'].message_type = _OPERATIONPARAMETER_MAPSTRENTRY
_OPERATIONPARAMETER.fields_by_name['mapStrList'].message_type = _OPERATIONPARAMETER_MAPSTRLISTENTRY
_OPERATIONPARAMETER.fields_by_name['mapBool'].message_type = _OPERATIONPARAMETER_MAPBOOLENTRY
_OPERATIONPARAMETER.fields_by_name['mapInt'].message_type = _OPERATIONPARAMETER_MAPINTENTRY
_OPERATIONPARAMETER.fields_by_name['mapDouble'].message_type = _OPERATIONPARAMETER_MAPDOUBLEENTRY
DESCRIPTOR.message_types_by_name['LineageEvent'] = _LINEAGEEVENT
DESCRIPTOR.message_types_by_name['UserDefinedInfo'] = _USERDEFINEDINFO
DESCRIPTOR.message_types_by_name['TrainLineage'] = _TRAINLINEAGE
DESCRIPTOR.message_types_by_name['EvaluationLineage'] = _EVALUATIONLINEAGE
DESCRIPTOR.message_types_by_name['DatasetGraph'] = _DATASETGRAPH
DESCRIPTOR.message_types_by_name['Operation'] = _OPERATION
DESCRIPTOR.message_types_by_name['OperationParameter'] = _OPERATIONPARAMETER
DESCRIPTOR.message_types_by_name['StrList'] = _STRLIST

LineageEvent = _reflection.GeneratedProtocolMessageType('LineageEvent', (_message.Message,), dict(
  DESCRIPTOR = _LINEAGEEVENT,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.LineageEvent)
  ))
_sym_db.RegisterMessage(LineageEvent)

UserDefinedInfo = _reflection.GeneratedProtocolMessageType('UserDefinedInfo', (_message.Message,), dict(

  MapDictEntry = _reflection.GeneratedProtocolMessageType('MapDictEntry', (_message.Message,), dict(
    DESCRIPTOR = _USERDEFINEDINFO_MAPDICTENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.UserDefinedInfo.MapDictEntry)
    ))
  ,

  MapInt32Entry = _reflection.GeneratedProtocolMessageType('MapInt32Entry', (_message.Message,), dict(
    DESCRIPTOR = _USERDEFINEDINFO_MAPINT32ENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.UserDefinedInfo.MapInt32Entry)
    ))
  ,

  MapStrEntry = _reflection.GeneratedProtocolMessageType('MapStrEntry', (_message.Message,), dict(
    DESCRIPTOR = _USERDEFINEDINFO_MAPSTRENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.UserDefinedInfo.MapStrEntry)
    ))
  ,

  MapDoubleEntry = _reflection.GeneratedProtocolMessageType('MapDoubleEntry', (_message.Message,), dict(
    DESCRIPTOR = _USERDEFINEDINFO_MAPDOUBLEENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.UserDefinedInfo.MapDoubleEntry)
    ))
  ,
  DESCRIPTOR = _USERDEFINEDINFO,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.UserDefinedInfo)
  ))
_sym_db.RegisterMessage(UserDefinedInfo)
_sym_db.RegisterMessage(UserDefinedInfo.MapDictEntry)
_sym_db.RegisterMessage(UserDefinedInfo.MapInt32Entry)
_sym_db.RegisterMessage(UserDefinedInfo.MapStrEntry)
_sym_db.RegisterMessage(UserDefinedInfo.MapDoubleEntry)

TrainLineage = _reflection.GeneratedProtocolMessageType('TrainLineage', (_message.Message,), dict(

  HyperParameters = _reflection.GeneratedProtocolMessageType('HyperParameters', (_message.Message,), dict(
    DESCRIPTOR = _TRAINLINEAGE_HYPERPARAMETERS,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.TrainLineage.HyperParameters)
    ))
  ,

  TrainDataset = _reflection.GeneratedProtocolMessageType('TrainDataset', (_message.Message,), dict(
    DESCRIPTOR = _TRAINLINEAGE_TRAINDATASET,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.TrainLineage.TrainDataset)
    ))
  ,

  Algorithm = _reflection.GeneratedProtocolMessageType('Algorithm', (_message.Message,), dict(
    DESCRIPTOR = _TRAINLINEAGE_ALGORITHM,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.TrainLineage.Algorithm)
    ))
  ,

  Model = _reflection.GeneratedProtocolMessageType('Model', (_message.Message,), dict(
    DESCRIPTOR = _TRAINLINEAGE_MODEL,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.TrainLineage.Model)
    ))
  ,
  DESCRIPTOR = _TRAINLINEAGE,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.TrainLineage)
  ))
_sym_db.RegisterMessage(TrainLineage)
_sym_db.RegisterMessage(TrainLineage.HyperParameters)
_sym_db.RegisterMessage(TrainLineage.TrainDataset)
_sym_db.RegisterMessage(TrainLineage.Algorithm)
_sym_db.RegisterMessage(TrainLineage.Model)

EvaluationLineage = _reflection.GeneratedProtocolMessageType('EvaluationLineage', (_message.Message,), dict(

  ValidDataset = _reflection.GeneratedProtocolMessageType('ValidDataset', (_message.Message,), dict(
    DESCRIPTOR = _EVALUATIONLINEAGE_VALIDDATASET,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.EvaluationLineage.ValidDataset)
    ))
  ,
  DESCRIPTOR = _EVALUATIONLINEAGE,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.EvaluationLineage)
  ))
_sym_db.RegisterMessage(EvaluationLineage)
_sym_db.RegisterMessage(EvaluationLineage.ValidDataset)

DatasetGraph = _reflection.GeneratedProtocolMessageType('DatasetGraph', (_message.Message,), dict(
  DESCRIPTOR = _DATASETGRAPH,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.DatasetGraph)
  ))
_sym_db.RegisterMessage(DatasetGraph)

Operation = _reflection.GeneratedProtocolMessageType('Operation', (_message.Message,), dict(
  DESCRIPTOR = _OPERATION,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.Operation)
  ))
_sym_db.RegisterMessage(Operation)

OperationParameter = _reflection.GeneratedProtocolMessageType('OperationParameter', (_message.Message,), dict(

  MapStrEntry = _reflection.GeneratedProtocolMessageType('MapStrEntry', (_message.Message,), dict(
    DESCRIPTOR = _OPERATIONPARAMETER_MAPSTRENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter.MapStrEntry)
    ))
  ,

  MapStrListEntry = _reflection.GeneratedProtocolMessageType('MapStrListEntry', (_message.Message,), dict(
    DESCRIPTOR = _OPERATIONPARAMETER_MAPSTRLISTENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter.MapStrListEntry)
    ))
  ,

  MapBoolEntry = _reflection.GeneratedProtocolMessageType('MapBoolEntry', (_message.Message,), dict(
    DESCRIPTOR = _OPERATIONPARAMETER_MAPBOOLENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter.MapBoolEntry)
    ))
  ,

  MapIntEntry = _reflection.GeneratedProtocolMessageType('MapIntEntry', (_message.Message,), dict(
    DESCRIPTOR = _OPERATIONPARAMETER_MAPINTENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter.MapIntEntry)
    ))
  ,

  MapDoubleEntry = _reflection.GeneratedProtocolMessageType('MapDoubleEntry', (_message.Message,), dict(
    DESCRIPTOR = _OPERATIONPARAMETER_MAPDOUBLEENTRY,
    __module__ = 'mindinsight_lineage_pb2'
    # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter.MapDoubleEntry)
    ))
  ,
  DESCRIPTOR = _OPERATIONPARAMETER,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.OperationParameter)
  ))
_sym_db.RegisterMessage(OperationParameter)
_sym_db.RegisterMessage(OperationParameter.MapStrEntry)
_sym_db.RegisterMessage(OperationParameter.MapStrListEntry)
_sym_db.RegisterMessage(OperationParameter.MapBoolEntry)
_sym_db.RegisterMessage(OperationParameter.MapIntEntry)
_sym_db.RegisterMessage(OperationParameter.MapDoubleEntry)

StrList = _reflection.GeneratedProtocolMessageType('StrList', (_message.Message,), dict(
  DESCRIPTOR = _STRLIST,
  __module__ = 'mindinsight_lineage_pb2'
  # @@protoc_insertion_point(class_scope:mindinsight.StrList)
  ))
_sym_db.RegisterMessage(StrList)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\370\001\001'))
_USERDEFINEDINFO_MAPDICTENTRY.has_options = True
_USERDEFINEDINFO_MAPDICTENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_USERDEFINEDINFO_MAPINT32ENTRY.has_options = True
_USERDEFINEDINFO_MAPINT32ENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_USERDEFINEDINFO_MAPSTRENTRY.has_options = True
_USERDEFINEDINFO_MAPSTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_USERDEFINEDINFO_MAPDOUBLEENTRY.has_options = True
_USERDEFINEDINFO_MAPDOUBLEENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_OPERATIONPARAMETER_MAPSTRENTRY.has_options = True
_OPERATIONPARAMETER_MAPSTRENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_OPERATIONPARAMETER_MAPSTRLISTENTRY.has_options = True
_OPERATIONPARAMETER_MAPSTRLISTENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_OPERATIONPARAMETER_MAPBOOLENTRY.has_options = True
_OPERATIONPARAMETER_MAPBOOLENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_OPERATIONPARAMETER_MAPINTENTRY.has_options = True
_OPERATIONPARAMETER_MAPINTENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
_OPERATIONPARAMETER_MAPDOUBLEENTRY.has_options = True
_OPERATIONPARAMETER_MAPDOUBLEENTRY._options = _descriptor._ParseOptions(descriptor_pb2.MessageOptions(), _b('8\001'))
# @@protoc_insertion_point(module_scope)
