# Copyright 2019 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""Mindinsight init module."""

from mindinsight._version import VERSION
from mindinsight.modelarts import notebook

__version__ = VERSION
__version_info__ = tuple(VERSION.split('.'))

__all__ = [
    '__version__',
    '__version_info__'
]


def load_ipython_extension(ipython):
    """
    IPython API entry point.

    Only intended to be called by the IPython runtime.

    See:
       The IPython extension guide.
    """
    notebook.notebook_load_ipython_extension(ipython)
