# Copyright 2019 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""base_notebook"""

class BaseNotebook:
    """
    Base notebook.
    """
    def __init__(self, **kwargs):
        pass

    def start(self, args_string, **kwargs):
        kwargs = kwargs
        return args_string

    def stop(self, port, **kwargs):
        pass

    def handle(self, **kwargs):
        pass
