.empty-container[data-v-fc3dc28a]{height:100%;width:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;background-color:var(--bg-color)}.empty-container img[data-v-fc3dc28a]{margin-bottom:10px;-o-object-fit:none;object-fit:none}