.cpu-utilization-wrap{height:100%}.cpu-utilization-wrap .title{font-size:16px;font-weight:700;text-align:left}.cpu-utilization-wrap .image-noData{width:100%;height:calc(100% - 37px);display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.cpu-utilization-wrap .image-noData p{font-size:16px;padding-top:10px}.cpu-utilization-wrap .el-button{border:1px solid #00a5a7;border-radius:2px;background-color:var(--bg-color);color:#00a5a7;padding:7px 15px}.cpu-utilization-wrap .el-button:hover{background:var(--button-hover-tip)}.cpu-info{height:calc(100% - 30px);display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.cpu-info .step-filter{min-height:44px}.cpu-info .step-filter .step-input{width:100px;margin-right:20px}.cpu-info .step-filter:first-child label{margin-right:10px}.cpu-info .cpu-detail{height:calc(100% - 44px);-webkit-box-flex:1;-ms-flex-positive:1;flex-grow:1;overflow-x:hidden;overflow-y:scroll}.cpu-info .cpu-detail .cpu-detail-item{padding:16px;border:1px solid var(--border-color);border-radius:4px;margin-right:8px}.cpu-info .cpu-detail .cpu-detail-item-top{margin-top:16px}.cpu-info .cpu-detail .detail-item-title{height:20px;font-size:15px;font-weight:700}.cpu-info .cpu-detail .detail-item-graph{height:120px;background-color:var(--graph-bg-color);margin:10px 0}.cpu-info .cpu-detail .detail-item-graph svg path{fill:var(--data-process-operator-color);stroke:#e6ebf5}.cpu-info .cpu-detail .detail-item-graph svg g.selected path{stroke:red}.cpu-info .cpu-detail .detail-item-graph svg text{fill:var(--font-color)}.cpu-info .cpu-detail .detail-item-graph .is-active{stroke:red}.cpu-info .cpu-detail .detail-item{height:400px;display:-webkit-box;display:-ms-flexbox;display:flex}.cpu-info .cpu-detail .detail-item:last-of-type{padding-bottom:0;border-bottom:none}.cpu-info .cpu-detail .detail-item .cpu-chart{height:100%;-webkit-box-flex:1;-ms-flex-positive:1;flex-grow:1}.cpu-info .cpu-detail .detail-item .cpu-chart-info{height:100%;width:300px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;padding-left:20px;background-color:var(--module-bg-color)}.cpu-info .cpu-detail .detail-item .cpu-chart-info .info-title{font-size:14px;font-weight:700;line-height:30px}.cpu-info .cpu-detail .detail-item .cpu-chart-info .info-line{line-height:30px}.cpu-chart-tip{display:inline-block;margin-right:5px;border-radius:10px;width:10px;height:10px}