.cl-memory-heatmap-dasnhoard[data-v-128c5444]{height:100%;width:100%;background:var(--bg-color)}.cl-memory-heatmap-dasnhoard .dashboard-item[data-v-128c5444]{width:100%;height:100%;padding:15px;border:solid 1px var(--border-color);border-radius:4px;min-height:284px}.cluster-wrap[data-v-128c5444]{height:calc(50% - 28px);border-top:1px solid var(--item-split-line-color)}.cl-memory-heatmap-dasnhoard .dashboard-item .title-item[data-v-128c5444]{display:-webkit-box;display:-ms-flexbox;display:flex;height:24px;margin-top:5px}.cl-memory-heatmap-dasnhoard .dashboard-item .title-item .title-text[data-v-128c5444]{-webkit-box-flex:1;-ms-flex:1;flex:1;font-size:16px;font-weight:700;line-height:24px}.cl-memory-heatmap-dasnhoard .dashboard-item .title-item .detail-link[data-v-128c5444]{cursor:pointer;font-size:12px;height:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.cl-memory-heatmap-dasnhoard .dashboard-item .title-item .detail-link a[data-v-128c5444]{color:var(--theme-color);padding-right:6px}.cl-memory-heatmap-dasnhoard .dashboard-item .title-item .detail-link button[data-v-128c5444]{color:var(--theme-color);border:none;background-color:var(--bg-color);cursor:pointer;padding:0}.cl-memory-heatmap-dasnhoard .dashboard-item .title-item .detail-link.disabled button[data-v-128c5444]{color:var(--button-disabled-font-color);cursor:not-allowed}.cl-memory-heatmap-dasnhoard .dashboard-item .content-item[data-v-128c5444]{height:calc(100% - 49px);margin-top:20px}.cl-memory-heatmap-dasnhoard .dashboard-item .content-item .dashboard-chart-content[data-v-128c5444]{width:100%;height:100%}.cl-memory-heatmap-dasnhoard .margin-item[data-v-128c5444]{margin-top:20px}.cl-memory-heatmap-dasnhoard .noData-content[data-v-128c5444]{width:100%;height:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.cl-memory-heatmap-dasnhoard .noData-content .noData-text[data-v-128c5444],.cl-memory-heatmap-dasnhoard .noData-content p[data-v-128c5444]{font-size:16px}.legend-content[data-v-128c5444]{height:56px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:start;-ms-flex-pack:start;justify-content:flex-start}.legend-content .legend-item[data-v-128c5444]{width:26px;height:100%;padding:5px 0;margin-left:10px}.legend-content .legend-item .color-item[data-v-128c5444]{width:100%;padding:0 1px;height:calc(100% - 19px)}.legend-content .legend-item .color-item div[data-v-128c5444]{width:100%;height:100%;border:1px solid #e6ebf5}.legend-content .legend-item .value-item[data-v-128c5444]{margin-top:4px}.heatmap-content[data-v-128c5444]{-webkit-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:grid;grid-template-columns:repeat(8,1fr);overflow-y:auto}.heatmap-content .heatmap-item[data-v-128c5444]{height:120px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;gap:10px}.heatmap-content .heatmap-item .color-item[data-v-128c5444]{width:80px;height:80px;border-radius:8px;cursor:pointer}