.router-container .profiling-content-title-gpu{font-size:18px;font-weight:700}.prof-wrap{height:100%;background:var(--bg-color)}.prof-wrap .prof-head{height:50px;line-height:50px;display:inline-block}.prof-wrap .prof-head .path-message{display:inline-block;line-height:20px;padding:18px 0;font-weight:700;margin-left:5px}.prof-wrap .prof-content{height:calc(100% - 50px);padding:0 24px 24px 0}.prof-wrap .prof-content>div{float:left;height:100%}.prof-wrap .prof-content .prof-content-left{width:22%;-webkit-transition:width .2s;transition:width .2s;position:relative}.prof-wrap .prof-content .prof-content-left .el-input__inner{padding:0 10px}.prof-wrap .prof-content .prof-content-left .helper{padding:32px;padding-top:20px;height:100%;overflow-y:auto;margin-left:24px;background:var(--module-bg-color);word-wrap:break-word}.prof-wrap .prof-content .prof-content-left .helper .nowrap-style{white-space:nowrap}.prof-wrap .prof-content .prof-content-left .helper .cur-card{padding-bottom:20px;border-bottom:1px solid var(--border-color)}.prof-wrap .prof-content .prof-content-left .helper .cur-card .card-select{width:calc(100% - 120px)}.prof-wrap .prof-content .prof-content-left .helper .cur-card>label{margin-right:14px}.prof-wrap .prof-content .prof-content-left .helper .helper-title{font-size:18px;font-weight:700;margin:24px 0}.prof-wrap .prof-content .prof-content-left .helper .helper-title .el-icon-rank{float:right;cursor:pointer}.prof-wrap .prof-content .prof-content-left .helper .helper-container-title{display:inline-block;padding:0 6px}.prof-wrap .prof-content .prof-content-left .helper .helper-icon{display:inline-block;width:6px;height:6px;margin-top:6px;border-radius:3px;background-color:var(--theme-color)}.prof-wrap .prof-content .prof-content-left .helper .suggested-title{font-weight:700;margin-bottom:20px;font-size:16px}.prof-wrap .prof-content .prof-content-left .helper .link-title{cursor:pointer}.prof-wrap .prof-content .prof-content-left .helper .container-bottom{margin-bottom:16px}.prof-wrap .prof-content .prof-content-left .helper .suggested-items-style{display:-webkit-box;display:-ms-flexbox;display:flex;font-weight:700;margin-bottom:6px;margin-top:10px}.prof-wrap .prof-content .prof-content-left .helper .helper-content-style{margin-left:6px;line-height:20px;word-break:break-all;text-overflow:ellipsis;overflow:hidden;display:-webkit-box;-webkit-box-orient:vertical;-webkit-line-clamp:8}.prof-wrap .prof-content .prof-content-left .content-icon{color:var(--theme-color);padding-top:3px}.prof-wrap .prof-content .prof-content-left .content-style{display:-webkit-box;display:-ms-flexbox;display:flex}.prof-wrap .prof-content .prof-content-left .collapse-btn{position:absolute;right:-21px;width:31px;height:100px;top:50%;margin-top:-50px;cursor:pointer;line-height:86px;z-index:1;text-align:center}.prof-wrap .prof-content .prof-content-left .collapse-btn-0{background-image:url(../../static/img/collapse-left.16dd02f7.svg)}.prof-wrap .prof-content .prof-content-left .collapse-btn-1{background-image:url(../../static/img/collapse-left.523794c7.svg)}.prof-wrap .prof-content .prof-content-left .collapse-btn-0.collapse{background-image:url(../../static/img/collapse-right.4e288788.svg)}.prof-wrap .prof-content .prof-content-left .collapse-btn-1.collapse{background-image:url(../../static/img/collapse-right.281a4cc4.svg)}.prof-wrap .prof-content .prof-content-left.collapse{width:0}.prof-wrap .prof-content .prof-content-right{width:78%;padding-left:20px;-webkit-transition:width .2s;transition:width .2s;position:relative}.prof-content-right .tab-container{width:100%;padding-bottom:5px}.prof-content-right .tab-container .el-tabs__item{font-size:14px;line-height:14px;height:27px}.prof-content-right .tab-container .el-tabs__item.is-active{color:var(--theme-color);font-weight:700}.prof-content-right .router-container.detail{height:100%}.prof-content-right .router-container.dashboard{height:calc(100% - 46px)}.prof-wrap .prof-content .prof-content-right .close{position:absolute;right:0;top:-10px;cursor:pointer}.prof-wrap .prof-content .prof-content-right.collapse{width:100%}