.cl-profiler-wrap{height:calc(100% - 56px)}.flops-info{line-height:30px;background:var(--module-bg-color);margin-bottom:8px}.flops-info span{margin-right:15px;font-weight:700}.flops-info .view-detail{float:right;cursor:pointer;font-size:12px;height:30px;line-height:30px}.flops-info .view-detail button{color:var(--theme-color);border:none;background-color:var(--module-bg-color);cursor:pointer}.flops-info .view-detail button.disabled{cursor:not-allowed;color:var(--button-disabled-font-color)}.cl-search-box{float:right;margin-bottom:10px;margin-right:20px}.cl-search-box .el-input{width:300px}.core-search-type{float:right;width:130px;margin-right:10px}.cl-profiler-top .chart-title{float:left;font-weight:700;height:32px;font-size:16px}.cl-profiler-top.fullScreen{display:none}.cl-profiler-bottom{height:53%;padding-top:10px}.cl-profiler-bottom.flops{height:calc(53% - 38px)}.cl-profiler-bottom .fullScreen{float:right;margin-top:5px;cursor:pointer}.cl-profiler-bottom.fullScreen{height:100%}.cl-profiler-bottom.fullScreen.flops{height:calc(100% - 38px)}.cl-profiler-echarts{width:100%;height:calc(100% - 32px);display:inline-block;position:relative;overflow:auto}.cl-profiler-echarts .chart{width:100%;height:100%;min-width:1300px;min-height:321px;overflow:hidden}.chart-radio-group{float:right}.el-radio-group .el-radio-button--small .el-radio-button__inner{height:30px;width:80px;font-size:14px;line-height:10px}.cl-profiler-bar{padding:20px}.cl-profiler-bar,.cl-profiler-table-type{display:inline-block;width:calc(100% - 400px);vertical-align:top;height:100%}.el-pagination{margin:7px 0;float:right}.details-data-list .el-table th{padding:10px 0;border-top:1px solid #ebeef5}.details-data-list .el-table th .cell{border-left:1px solid #d9d8dd;height:14px;line-height:14px}.details-data-list .el-table th:first-child .cell{border-left:none}.details-data-list .el-table td:nth-child(2),.details-data-list .el-table th:nth-child(2){max-width:30%}.details-data-list .el-table td{padding:8px 0}.details-data-list .el-table__row--level-0 td:first-child:after{width:20px;height:1px;background:#ebeef5;z-index:11;position:absolute;left:0;bottom:-1px;content:"";display:block}.details-data-list .el-table__row--level-1 td{padding:4px 0;position:relative}.details-data-list .el-table__row--level-1 td:first-child:before{width:42px;background:#f0fdfd;border-right:2px solid #00a5a7;z-index:10;position:absolute;left:0;top:-1px;bottom:0;content:"";display:block}.details-data-list .el-table__row--level-1:first-child td:first-child:before{bottom:0}.details-data-list .el-dialog__title{font-weight:700}.details-data-list .el-dialog__body{max-height:500px;padding-top:10px;overflow:auto}.flops-data-list .el-dialog{margin-top:64px!important;height:calc(100% - 64px);margin-bottom:0}.flops-data-list .el-dialog__body{padding-top:10px;overflow:auto;text-align:center;height:calc(100% - 54px)}.flops-data-list .reset{text-align:right}.details-data-list .el-dialog__body .details-data-title{margin-bottom:20px}.el-table__expanded-cell[class*=cell]{padding:0}.expand-table{position:relative;padding-left:44px}.expand-table:before{content:"";position:absolute;left:0;top:0;height:100%;background:#f0fdfd;width:42px;border-right:2px solid #00a5a7}.el-radio-button:first-child .el-radio-button__inner,.el-radio-button:last-child .el-radio-button__inner{border-radius:0}.image-noData{width:100%;height:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.image-noData p{font-size:16px;padding-top:10px}#flopsChart{display:inline-block;min-height:90%}