.cluster-comm-op-details-modal .modal-content .content-header{height:36px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.cluster-comm-op-details-modal .modal-content .content-header .el-input{width:240px}.cluster-comm-op-details-modal .modal-content .content-header .el-radio-button__inner{padding:10px 12px}.cluster-comm-op-details-modal .modal-content .content-container{height:400px}.cluster-comm-op-details-modal .modal-content .content-pagination{height:36px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:end;-ms-flex-pack:end;justify-content:flex-end}.communication-container{height:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.communication-sub-title{width:100%;height:75px;padding-top:5px;margin-left:16px}.communication-view{height:100%}#networkPlot{position:relative}#networkPlot,.communication-graph-box{width:100%;height:100%}.el-icon-magic-stick{position:absolute;top:0;right:5%;z-index:999}.lasso path{stroke:#505050;stroke-width:2px}.lasso .drawn{fill-opacity:.05}.lasso .loop_close{fill:none;stroke-dasharray:4,4}.lasso .origin{fill:#39f;fill-opacity:.5}.cls-1,.cls-2{fill:none;stroke:#cbcbcb;stroke-miterlimit:10}.cls-2{stroke-dasharray:2}.cl-comm .el-dialog__body{padding:20px}.cl-comm .el-tabs__item{font-size:14px;line-height:14px;height:27px}.cl-comm .el-tabs__item .is-active{color:var(--theme-color);font-weight:700}.comm-tooltip-container{padding:10px}.comm-tooltip-container .comm-tooltip-cell{width:100%;line-height:24px}.comm-tooltip-container .comm-tooltip-cell span{word-break:normal}.comm-tooltip-container .comm-tooltip-cell span:first-of-type{font-weight:700}.cl-comm[data-v-655a8126]{height:100%;background-color:var(--bg-color);display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.cl-comm .comm-tip[data-v-655a8126]{margin-left:6px}.cl-comm .el-select[data-v-655a8126]{width:120px;margin:0 20px}.cl-comm .comm-content[data-v-655a8126]{-webkit-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;width:100%;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;overflow:hidden}.comm-content .content-filter[data-v-655a8126]{height:32px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;margin-bottom:5px;-ms-flex-negative:0;flex-shrink:0;padding-left:10px}.comm-content .content-filter .el-input[data-v-655a8126]{width:120px;margin:0 20px}.comm-content .content-filter .el-button[data-v-655a8126]{padding:7px 15px;color:var(--theme-color);border-color:var(--theme-color)}.comm-content .content-chart[data-v-655a8126]{height:280px;margin-bottom:20px;-ms-flex-negative:0;flex-shrink:0}.comm-content .comm-container[data-v-655a8126]{height:calc(100% - 383px);margin-bottom:10px;display:-webkit-box;display:-ms-flexbox;display:flex}.comm-content .content-link-table[data-v-655a8126]{height:calc(100% - 84px);margin-bottom:10px}.comm-container .content-table[data-v-655a8126]{width:calc(100% - 500px);height:100%}.comm-container .comm-graph[data-v-655a8126]{width:500px;height:100%}.content-table .table-button[data-v-655a8126]{color:var(--theme-color);cursor:pointer}.comm-content .content-pagination[data-v-655a8126]{height:36px;-ms-flex-negative:0;flex-shrink:0}.cl-comm .dialog-pagination[data-v-655a8126],.comm-content .content-pagination[data-v-655a8126]{display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:end;-ms-flex-pack:end;justify-content:flex-end}.cl-comm .dialog-pagination[data-v-655a8126]{height:60px}.cl-comm .dialog-content[data-v-655a8126]{max-height:600px}