.empty-container[data-v-fc3dc28a]{height:100%;width:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;background-color:var(--bg-color)}.empty-container img[data-v-fc3dc28a]{margin-bottom:10px;-o-object-fit:none;object-fit:none}.performance-dashboard[data-v-735a22b0]{display:grid;grid-template-rows:calc(50% - 10px) calc(50% - 10px);grid-template-columns:100%;height:100%;row-gap:20px}.performance-dashboard .container[data-v-735a22b0]{width:100%;height:100%;border:1px solid var(--border-color);border-radius:4px;padding:15px;position:relative}.performance-dashboard .content[data-v-735a22b0]{width:100%;height:calc(100% - 24px)}.performance-dashboard .header[data-v-735a22b0]{width:100%;height:24px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:justify;-ms-flex-pack:justify;justify-content:space-between}.performance-dashboard .header .title[data-v-735a22b0]{font-size:16px;font-weight:700}.performance-dashboard .header .jump[data-v-735a22b0]{cursor:not-allowed;color:var(--button-disabled-font-color);font-size:12px}.performance-dashboard .header .is-effective[data-v-735a22b0]{color:var(--theme-color);cursor:pointer}