.mi-contour-map{height:100%;position:relative;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;overflow:hidden}.mi-contour-map .map-content{-webkit-box-flex:1;-ms-flex-positive:1;flex-grow:1}.mi-contour-map .map-animation{height:30px;padding-right:20px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-pack:end;-ms-flex-pack:end;justify-content:flex-end}.mi-contour-map .map-animation .animation-button{cursor:pointer}.mi-contour-map .map-legend{position:absolute;bottom:48px;right:0;display:grid;height:240px;width:100px}.mi-contour-map .map-legend .legend-item{display:grid;grid-template-columns:20px 1fr;-webkit-box-align:center;-ms-flex-align:center;align-items:center;gap:10px;position:relative}.mi-contour-map .map-legend .legend-item .legend-color{height:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-align:center;-ms-flex-align:center;align-items:center}.mi-contour-map .map-legend .legend-item .contour-color{border-right:1px solid grey;border-left:1px solid grey}.mi-contour-map .map-legend .legend-item:first-of-type .contour-color{border-top:1px solid grey}.mi-contour-map .map-legend .legend-item:last-of-type .contour-color{border-bottom:1px solid grey}.mi-contour-map .map-legend .legend-item .contour-color .color{height:2px;width:100%}.mi-contour-map .map-legend .legend-item .topographic-color{border:1px solid grey;border-bottom:none}.mi-contour-map .map-legend .legend-item:last-of-type .topographic-color{border-bottom:1px solid grey}.mi-contour-map .map-legend .legend-item .topographic-color .color{height:100%;width:100%}.mi-contour-map .map-legend .legend-item .text{position:absolute;left:24px}.map-legend .legend-title{height:30px;line-height:30px}