.operator{height:100%}.operator .clear{clear:both}.operator .el-tabs__item{color:#6c7280;line-height:36px;height:36px}.operator .el-tabs__item.is-active{color:#00a5a7;font-weight:700}.operator .operator-title{padding:0 15px;font-size:18px;font-weight:700}.operator .cl-profiler{height:calc(100% - 24px);overflow-y:auto;width:100%;padding:0 16px;overflow:hidden}.operator .cl-profiler .custom-label{max-width:calc(100% - 25px);padding:0;vertical-align:middle}.operator .cl-profiler .el-tabs{height:100%}.operator .cl-profiler .el-tabs .el-tabs__header{margin-bottom:10px}.operator .cl-profiler .el-tabs__content{height:calc(100% - 46px)}.operator .cl-profiler .el-tab-pane{height:100%}