# Copyright 2020 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""
MindWizard.

MindWizard is a tool for quickly generating classic network scripts.
It collects user preference on the combination of network parameters,
such models, hyperparameters and datasets, then automatically generates target network scripts.
The generated scripts can be used for training and evaluation in the Ascend or GPU environment.
"""
