.cl-memory-heatmap[data-v-3cbf8c04]{height:100%;width:100%;background:var(--bg-color);display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.cl-memory-heatmap .el-icon-info[data-v-3cbf8c04]{color:#6c7280}.el-tooltip__popper .heatmap-tooltip-container .cl-memory-heatmap-tip[data-v-3cbf8c04]{padding:10px}.el-tooltip__popper .heatmap-tooltip-container .cl-memory-heatmap-tip .tip-title[data-v-3cbf8c04]{font-size:16px;font-weight:700;padding:0 0 3px 0}.el-tooltip__popper .heatmap-tooltip-container .tag-tip .tip-title[data-v-3cbf8c04]{color:#333}.el-tooltip__popper .heatmap-tooltip-container .cl-memory-heatmap-tip .tip-part[data-v-3cbf8c04]{line-height:20px;word-break:normal}.cl-memory-heatmap .content[data-v-3cbf8c04]{display:grid;grid-template-columns:1fr auto;width:100%;border-bottom:solid 1px var(--border-color);padding:8px 0}.cl-memory-heatmap .content .select-granuLarity[data-v-3cbf8c04]{width:100px}.cl-memory-heatmap .content .legend-content[data-v-3cbf8c04]{display:-webkit-box;display:-ms-flexbox;display:flex;-ms-flex-wrap:wrap;flex-wrap:wrap;gap:12px;padding-left:4px}.content .legend-content .legend-item[data-v-3cbf8c04]{position:relative;height:36px}.content .legend-content .legend-item .color-item[data-v-3cbf8c04]{width:24px;height:24px;border:1px solid #e6ebf5}.content .legend-content .legend-item .text-item[data-v-3cbf8c04]{top:24px;position:absolute;left:50%;-webkit-transform:translateX(-50%);transform:translateX(-50%)}.cl-memory-heatmap .heatmap-content[data-v-3cbf8c04]{-webkit-box-flex:1;-ms-flex-positive:1;flex-grow:1;display:grid;grid-template-columns:repeat(8,1fr);overflow-y:auto}.heatmap-content .heatmap-item[data-v-3cbf8c04]{height:200px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;gap:10px}.heatmap-content .heatmap-item .color-item[data-v-3cbf8c04]{width:80px;height:80px;border-radius:8px;cursor:pointer}