.cl-memory-detail{width:100%;height:100%;display:-webkit-box;display:-ms-flexbox;display:flex}.cl-memory-detail .chart-container.full-screen,.cl-memory-detail .top-content.full-screen{display:none}.cl-memory-detail .table-container.full-screen{margin-top:0;height:100%}.cl-memory-detail .memory-bk{width:100%;height:100%;overflow:hidden;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.cl-memory-detail .title-content{height:44px;padding-bottom:20px;color:var(--font-color);font-weight:700;font-size:20px}.cl-memory-detail .title-content .title-text{font-size:16px}.cl-memory-detail .top-content .content-item{display:-webkit-box;display:-ms-flexbox;display:flex;height:calc(100% - 21px);width:100%}.cl-memory-detail .main-content{height:calc(100% - 30px);overflow-y:auto}.cl-memory-detail .noData-content{width:100%;height:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.cl-memory-detail .top-content{width:100%;height:137px;padding:24px;border:1px solid #d9d9d9;border-radius:4px;background:var(--item-bg-color)}.cl-memory-detail .top-content .detail-item{width:16.6%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;padding:24px 5px 0 5px;text-align:center}.cl-memory-detail .top-content .label-item,.cl-memory-detail .top-content .unit-item{font-size:12px;line-height:12px;color:var(--icon-info-color)}.cl-memory-detail .top-content .value-item{font-size:24px;font-weight:700}.cl-memory-detail .chart-container,.cl-memory-detail .table-container{width:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;border:1px solid #ccc;border-radius:4px;padding:20px 24px;background:var(--item-bg-color)}.cl-memory-detail .chart-container{background:var(--bg-color)}.cl-memory-detail .table-container .fullScreen-icon{float:right;margin-top:2px;cursor:pointer}.cl-memory-detail .chart-container .chart-content,.cl-memory-detail .table-container .table-content{margin-top:16px;height:calc(100% - 37px)}.cl-memory-detail .chart-container{height:calc(60% - 106px);margin-top:20px;min-height:280px}.cl-memory-detail .table-container{margin-top:20px;height:calc(40% - 71px);min-height:160px}.cl-memory-detail .title-item{font-size:16px;font-weight:700}.cl-memory-detail .el-table td,.cl-memory-detail .el-table th{border:1px solid #ebeef5}