.helper{-webkit-box-sizing:border-box;box-sizing:border-box;padding:32px;padding-top:20px;height:100%;overflow-y:auto;background:var(--module-bg-color);word-wrap:break-word}.helper .nowrap-style{white-space:nowrap}.helper .helper-title{font-size:18px;font-weight:700;margin:24px 0}.helper .helper-title .el-icon-rank{float:right;cursor:pointer}.helper .helper-container-title{display:inline-block;padding:0 6px}.helper .helper-icon{display:inline-block;width:6px;height:6px;margin-top:6px;border-radius:3px;background-color:var(--theme-color)}.helper .suggested-title{font-weight:700;margin-bottom:20px;font-size:16px}.helper .link-title{cursor:pointer}.helper .container-bottom{margin-bottom:16px}.helper .suggested-items-style{display:-webkit-box;display:-ms-flexbox;display:flex;font-weight:700;margin-bottom:6px;margin-top:10px}.helper .helper-content-style{margin-left:6px;line-height:20px;word-break:break-all;text-overflow:ellipsis;overflow:hidden;display:-webkit-box;-webkit-box-orient:vertical;-webkit-line-clamp:8}.helper .content-icon{color:var(--theme-color);padding-top:3px}.dashboard-content .cl-cluster-title,.helper .content-style{display:-webkit-box;display:-ms-flexbox;display:flex}.dashboard-content .cl-cluster-title{height:36px;line-height:36px;position:relative;font-size:16px;font-weight:700;-ms-flex-negative:0;flex-shrink:0;-webkit-box-align:center;-ms-flex-align:center;align-items:center;gap:4px}.cluster-dashboard-wrap[data-v-4b0c7ba0]{height:100%;display:-webkit-box;display:-ms-flexbox;display:flex}.dashboard-left[data-v-4b0c7ba0]{width:22%;-ms-flex-negative:0;flex-shrink:0;padding-right:20px;overflow:visible;-webkit-transition:width .1s;transition:width .1s;position:relative}.is-hidden[data-v-4b0c7ba0]{width:0;padding-right:0}.helper-control[data-v-4b0c7ba0]{position:absolute;right:0;width:31px;height:100px;top:50%;margin-top:-50px;cursor:pointer;line-height:86px;z-index:1;text-align:center}.collapse-btn-0[data-v-4b0c7ba0]{background-image:url(../../static/img/collapse-left.16dd02f7.svg)}.collapse-btn-1[data-v-4b0c7ba0]{background-image:url(../../static/img/collapse-left.523794c7.svg)}.collapse-btn-0.collapse[data-v-4b0c7ba0]{background-image:url(../../static/img/collapse-right.4e288788.svg)}.collapse-btn-1.collapse[data-v-4b0c7ba0]{background-image:url(../../static/img/collapse-right.281a4cc4.svg)}.cluster-dashboard[data-v-4b0c7ba0]{-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-flex:1;-ms-flex-positive:1;flex-grow:1;overflow:hidden}.cluster-dashboard .dashboard-content .item-container[data-v-4b0c7ba0]{height:calc(100% - 47px)}.cluster-dashboard .tab-container[data-v-4b0c7ba0]{width:100%;padding-bottom:5px}.cluster-dashboard .tab-container .el-tabs__item[data-v-4b0c7ba0]{font-size:14px;line-height:14px;height:27px}.cluster-dashboard .tab-container .el-tabs__item.is-active[data-v-4b0c7ba0]{color:var(--theme-color);font-weight:700}.cluster-dashboard-tabs[data-v-4b0c7ba0]{position:relative}.custer-dashboard-tabs-link[data-v-4b0c7ba0]{z-index:99;position:absolute;top:2px}