.step-trace{width:100%;height:100%}.step-trace .step-trace-title{height:32px}.step-trace .step-trace-title .el-icon-question{cursor:pointer}.step-trace .step-trace-title .pf-content-right{display:inline-block;margin-left:35px}.step-trace .step-trace-title .pf-content-right .input-wrap{font-weight:400}.step-trace .step-trace-title .pf-content-right .input-wrap label{margin-right:20px}.step-trace .step-trace-title .pf-content-right .input-wrap .el-input{width:150px;margin-right:16px}.step-trace .step-trace-title .el-button{border:1px solid var(--theme-color);border-radius:2px;background-color:var(--bg-color);color:var(--theme-color);padding:7px 15px}.step-trace .step-trace-title .el-button:hover{background:var(--button-hover-color)}.step-trace .step-trace-title .show-average{position:absolute;right:36px}.step-trace .step-message{height:calc(10% + 10px);line-height:16px;margin-top:8px;overflow-y:auto}.step-trace .step-padding-right{padding-right:20px;display:inline-block}.step-trace .step-left-padding-right{padding-right:30px;display:inline-block}.step-trace .font-weight-style{font-weight:700}.step-trace .pf-content-middle{display:grid;grid-template-rows:40% calc(50% - 10px);gap:20px;padding-top:10px;height:calc(100% - 72px)}.step-trace .pf-content-middle #trace-container{width:100%;height:100%;border:1px solid var(--border-color);overflow:auto}.step-trace .pf-content-middle #trace-container .training-trace{position:relative;height:0}.step-trace .pf-content-middle #trace-container .training-trace .content{overflow:hidden;text-align:center;text-overflow:ellipsis;white-space:nowrap;font-size:12px;line-height:40px}.step-trace .pf-content-middle #trace-container .training-trace .content-mini{overflow:visible}.step-trace .chart-container{display:grid;grid-template-columns:repeat(3,1fr);gap:15px;height:100%}.step-trace .pf-content-middle .chart-wrap{height:100%;width:100%;border:1px solid var(--border-color);padding-top:20px;border-radius:1px;overflow:auto}.step-trace .pf-content-middle .chart-wrap .chart{height:calc(100% - 110px);min-height:180px;min-width:250px;overflow:hidden}.step-trace .pf-content-middle .chart-wrap .title{margin:0 0 15px 20px;font-weight:700;font-size:16px}.step-trace .pf-content-middle .chart-wrap .rate-wrap{font-size:12px;padding-left:20px}.step-trace .pf-content-middle .chart-wrap .rate-wrap>div{display:inline-block;margin:0 15px 5px 0;color:var(--step-trace-chart-text-color)}.step-trace .pf-content-middle .chart-wrap .rate-wrap>div span{margin-right:10px;color:var(--step-trace-chart-label-color)}.step-trace .pf-content-middle .chart-wrap.chart-show{width:calc(50% - 7.5px)}.step-trace .image-noData{width:100%;height:calc(100% - 52px);display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.step-trace .image-noData p{font-size:16px;padding-top:10px}.step-trace .image-noData.svg{height:100%}.step-trace .el-icon-info{font-size:18px;color:#6c7280}