# Copyright 2020-2021 Huawei Technologies Co., Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ============================================================================
"""The analyser module."""
from . import analyser, minddata_pipeline_analyser, step_trace_analyser, minddata_analyser, \
    timeline_analyser, msprof_timeline_analyser, cpu_analyser, gpu_analyser, memory_usage_analyser, \
    minddata_cpu_utilization_analyser, cluster_analyser, flops_analyser, parallel_strategy_analyser, parallel_analyser
