.cl-resource-content[data-v-68d95561]{height:100%;overflow-y:auto}.cl-resource-content .dashboard-item[data-v-68d95561]{width:100%;height:calc(50% - 10px);padding:15px;border:solid 1px var(--border-color);background:var(--bg-color);border-radius:4px;min-height:284px}.cl-resource-content .dashboard-item .title-item[data-v-68d95561]{display:-webkit-box;display:-ms-flexbox;display:flex;height:24px}.cl-resource-content .dashboard-item .title-item .title-text[data-v-68d95561]{-webkit-box-flex:1;-ms-flex:1;flex:1;font-size:16px;font-weight:700;line-height:24px}.cl-resource-content .dashboard-item .title-item .detail-link[data-v-68d95561]{cursor:pointer;font-size:12px;height:18px;line-height:12px;padding-top:2px}.cl-resource-content .dashboard-item .title-item .detail-link a[data-v-68d95561]{color:var(--theme-color);padding-right:6px}.cl-resource-content .dashboard-item .title-item .detail-link button[data-v-68d95561]{color:var(--theme-color);border:none;background-color:var(--bg-color);cursor:pointer}.cl-resource-content .dashboard-item .title-item .detail-link.disabled button[data-v-68d95561]{color:#c0c4cc;cursor:not-allowed}.cl-resource-content .dashboard-item .content-item[data-v-68d95561]{height:calc(100% - 44px);margin-top:20px}.cl-resource-content .margin-item[data-v-68d95561]{margin-top:20px}.cl-resource-content .noData-content[data-v-68d95561]{width:100%;height:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.cl-resource-content .noData-content .noData-text[data-v-68d95561],.cl-resource-content .noData-content p[data-v-68d95561]{font-size:16px}