(window["webpackJsonp"]=window["webpackJsonp"]||[]).push([["chunk-d393e822"],{"698f":function(a,t,n){"use strict";n("deda")},"7e92":function(a,t,n){"use strict";n.r(t);var e=function(){var a=this,t=a.$createElement,n=a._self._c||t;return n("div",{staticClass:"mi-common-full-page compare-dashboard"},[n("div",{staticClass:"compare-tabs"},[n("el-tabs",{on:{"tab-click":a.onTabClick},model:{value:a.tab,callback:function(t){a.tab=t},expression:"tab"}},a._l(a.tabs,(function(a){return n("el-tab-pane",{key:a.name,attrs:{label:a.label,name:a.name}})})),1)],1),n("div",{staticClass:"dashboard-content"},[n("router-view")],1)])},s=[],c=(n("caad"),n("b0c0"),n("2532"),"scalar"),i="loss",o={data:function(){return{tabs:[{label:this.$t("summaryManage.scalarComparison"),name:c},{label:this.$t("summaryManage.lossComparison"),name:i}],tab:null}},created:function(){var a=this.$route.path;this.tab=a.includes("scalar")?this.tabs[0].name:this.tabs[1].name},methods:{onTabClick:function(a){this.$router.push({path:a.name})}}},l=o,r=(n("698f"),n("2877")),u=Object(r["a"])(l,e,s,!1,null,null,null);t["default"]=u.exports},b0c0:function(a,t,n){var e=n("83ab"),s=n("9bf2").f,c=Function.prototype,i=c.toString,o=/^\s*function ([^ (]*)/,l="name";e&&!(l in c)&&s(c,l,{configurable:!0,get:function(){try{return i.call(this).match(o)[1]}catch(a){return""}}})},deda:function(a,t,n){}}]);