.cl-operator[data-v-04caa587]{height:100%;background-color:var(--bg-color);overflow:auto}.operator-tab[data-v-04caa587]{height:51px;line-height:56px;padding:0 34px;border-bottom:1px solid var(--table-border-color)}.cl-operator .operator-path[data-v-04caa587]{display:inline-block;line-height:20px;padding:0 4px 15px 4px;font-weight:700;vertical-align:bottom}.cl-operator .cl-title-right .cl-close-btn[data-v-04caa587]{position:sticky}.cl-title-right[data-v-04caa587]{height:51px;line-height:56px;padding-right:20px;position:absolute;right:60px}.cl-operator .table-container[data-v-04caa587]{height:80%;width:95%;margin:0 auto;overflow:hidden}.cl-operator .table-container #exportTable[data-v-04caa587]{height:55%;overflow-y:hidden}.cl-operator .table-container .pagination-container[data-v-04caa587]{margin:10px 0}.cl-operator .operator-container[data-v-04caa587]{margin-top:30px;width:100%;margin-right:20px}.cl-operator .operator-name-input[data-v-04caa587]{margin-top:30px;padding:6px 32px 10px 2.5%;width:300px}.cl-operator .search[data-v-04caa587]{width:300px}.cl-operator .cl-operator-file[data-v-04caa587]{float:right;margin-top:40px;margin-right:2.6vw}.cl-operator .cl-operator-file .el-icon-info[data-v-04caa587]{font-size:24px;height:40px;line-height:40px;padding-right:5px;vertical-align:bottom}.operator-tab-item[data-v-04caa587]{padding:0 10px;height:48px;-webkit-box-sizing:border-box;box-sizing:border-box;line-height:48px;display:inline-block;list-style:none;font-size:18px;color:var(--el-tabs-item-color);position:relative}.item-active[data-v-04caa587]{color:var(--theme-color);font-weight:700;border-bottom:3px solid var(--theme-color)}.operator-tab-item[data-v-04caa587]:hover{color:var(--theme-color);cursor:pointer}.cl-operator .el-tag[data-v-04caa587]{background-color:#ecf5ff;display:inline-block;padding:0 10px;font-size:12px;color:#409eff;border:1px solid #d9ecff;border-radius:4px;-webkit-box-sizing:border-box;box-sizing:border-box;white-space:nowrap}.cl-operator .el-icon-bottom[data-v-04caa587]{padding-left:2px;color:red}.cl-operator .el-icon-top[data-v-04caa587]{padding-left:2px;color:#409eff}.cl-operator .operator-container .el-input__inner{height:42px;line-height:42px;border-radius:6px}.el-pagination.is-background .el-pager li:not(.disabled).active{color:#fff;background-color:#409eff}.el-icon-arrow-left:before,.el-pagination.is-background .btn-next,.el-pagination.is-background .btn-prev,.el-pagination.is-background .el-icon-arrow-right:before,.el-pagination.is-background .el-pager li{color:#000}