#cl-summary-manage{height:100%;width:100%;background-color:var(--bg-color)}#cl-summary-manage .no-data-page{width:100%;height:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center}#cl-summary-manage .no-data-page .no-data-img{text-align:center;height:200px;width:310px;margin:auto}#cl-summary-manage .no-data-page .no-data-img img{max-width:100%}#cl-summary-manage .no-data-page .no-data-img p{font-size:16px;padding-top:10px}#cl-summary-manage .cl-summary-manage-container{height:100%;padding:14px 32px 32px}#cl-summary-manage .cl-title{border:none;height:55px;line-height:75px}#cl-summary-manage .cl-title-left{padding-left:0;height:55px;line-height:55px}#cl-summary-manage .cl-title-left .btn-wrap{float:right}#cl-summary-manage .summary-title{font-size:20px;font-weight:700;margin-right:15px}#cl-summary-manage .summary-subtitle{margin-left:20px}#cl-summary-manage .container{height:calc(100% - 97px);overflow-y:auto}#cl-summary-manage .container .list-table{height:100%}#cl-summary-manage .container .list-table .operate-container{padding-right:32px}#cl-summary-manage .pagination-content{margin-top:16px;text-align:right}#cl-summary-manage .operate-btn{margin-left:20px;padding:12px 0}#cl-summary-manage .el-dialog{min-width:500px;padding-bottom:30px}#cl-summary-manage .operate-btn.button-disable{-moz-user-select:none;-webkit-user-select:none;-ms-user-select:none;user-select:none;color:#c0c4cc;cursor:not-allowed}#cl-summary-manage .menu-item{color:var(--theme-color);cursor:pointer}#cl-summary-manage .menu-item.operate-btn.first-btn{margin-left:0}#cl-summary-manage #contextMenu{position:absolute;min-width:150px;border:1px solid #d4d4d4}#cl-summary-manage #contextMenu ul{background-color:var(--bg-color);border-radius:2px}#cl-summary-manage #contextMenu ul li{padding:5px 18px;cursor:pointer}#cl-summary-manage #contextMenu ul li:hover{background-color:#a7a7a7;color:#fff}#cl-summary-manage .details-data-list .el-table td,#cl-summary-manage .details-data-list .el-table th.is-leaf{border:none;border-top:1px solid var(--table-border-color)}#cl-summary-manage .details-data-list .el-table th{padding:10px 0;border-top:1px solid var(--table-border-color)}#cl-summary-manage .details-data-list .el-table th .cell{border-left:1px solid #d9d8dd;height:14px;line-height:14px}#cl-summary-manage .details-data-list .el-table th:first-child .cell{border-left:none}#cl-summary-manage .details-data-list .el-table td:nth-child(2),#cl-summary-manage .details-data-list .el-table th:nth-child(2){max-width:30%}#cl-summary-manage .details-data-list .el-table td{padding:8px 0}#cl-summary-manage .details-data-list .el-table__row--level-0 td:first-child:after{width:20px;height:1px;background:var(--table-border-color);z-index:11;position:absolute;left:0;bottom:-1px;content:"";display:block}#cl-summary-manage .details-data-list .el-table__row--level-1 td{padding:4px 0;position:relative}#cl-summary-manage .details-data-list .el-table__row--level-1 td:first-child:before{width:42px;background:var(--expand-table-before-bg-color);border-right:2px var(--theme-color) solid;z-index:10;position:absolute;left:0;top:-1px;bottom:0;content:"";display:block}#cl-summary-manage .details-data-list .el-table__row--level-1:first-child td:first-child:before{bottom:0}#cl-summary-manage .details-data-list .el-dialog__title{font-weight:700;display:inline-block;width:calc(100% - 20px);word-break:break-all}#cl-summary-manage .details-data-list .el-dialog__body{max-height:500px;padding-top:10px;padding-bottom:0;overflow:auto}#cl-summary-manage .details-data-list .el-dialog__body .details-data-title{margin-bottom:20px}#cl-summary-manage .details-data-list .sessionMsg{color:var(--font-color);font-weight:700;font-size:16px;margin-right:5px}#cl-summary-manage .details-data-list .session-title{margin-bottom:10px;color:var(--font-color)}#cl-summary-manage .is-disabled.custom-btn{border:1px solid var(--table-border-color)!important;color:#adb0b8}#cl-summary-manage .is-disabled.custom-btn,#cl-summary-manage .is-disabled.custom-btn:hover{background-color:var(--button-disabled-bg-color)}#cl-summary-manage .custom-btn{border:1px solid var(--theme-color);border-radius:2px}#cl-summary-manage .white{background-color:var(--bg-color);color:var(--theme-color)}#cl-summary-manage .white:hover{background-color:var(--button-hover-color)}