(window["webpackJsonp"]=window["webpackJsonp"]||[]).push([["chunk-51ae1590"],{"1abe":function(t,i,s){},"50af":function(t,i,s){"use strict";s("1abe")},9485:function(t,i,s){t.exports=s.p+"static/img/nodata.8290bed9.png"},f517:function(t,i,s){"use strict";s.r(i);var a=function(){var t=this,i=t.$createElement,a=t._self._c||i;return a("div",{staticClass:"cl-resource-content"},[a("div",{staticClass:"dashboard-item"},[a("div",{staticClass:"title-item"},[a("div",{staticClass:"title-text"},[t._v(" "+t._s(t.$t("profiling.structuralCpuUtil"))+" ")]),a("div",{staticClass:"detail-link",class:{disabled:!0}},[a("button",{attrs:{disabled:!0},on:{click:function(i){return t.viewDetail("cpu-utilization")}}},[t._v(" "+t._s(t.$t("profiling.viewDetail"))+" "),a("i",{staticClass:"el-icon-d-arrow-right"})])])]),a("div",{staticClass:"content-item"},[a("div",{staticClass:"noData-content"},[a("img",{attrs:{src:s("9485")}}),a("p",[t._v(t._s(t.$t("public.noSupport")))])])])]),a("div",{staticClass:"dashboard-item margin-item"},[a("div",{staticClass:"title-item"},[a("div",{staticClass:"title-text"},[t._v(" "+t._s(t.$t("profiling.memory.usedMemory"))+" ")]),a("div",{staticClass:"detail-link",class:{disabled:!0}},[a("button",{attrs:{disabled:!0},on:{click:function(i){return t.viewDetail("memory-utilization")}}},[t._v(" "+t._s(t.$t("profiling.viewDetail"))+" "),a("i",{staticClass:"el-icon-d-arrow-right"})])])]),a("div",{staticClass:"content-item"},[a("div",{staticClass:"noData-content"},[a("div",[a("img",{attrs:{src:s("9485")}})]),a("p",[t._v(t._s(t.$t("public.noSupport")))])])])])])},e=[],n={props:{rankID:String},data:function(){return{trainInfo:{id:this.$route.query.id,path:this.$route.query.path,dir:this.$route.query.dir}}}},r=n,l=(s("50af"),s("2877")),c=Object(l["a"])(r,a,e,!1,null,"68d95561",null);i["default"]=c.exports}}]);