.el-tooltip-popper{max-width:500px}.tooltip-container .pro-dash-tooltip{line-height:20px;padding:10px}.tooltip-container .pro-dash-tooltip .font-style{font-weight:700}.tooltip-container .pro-dash-tooltip .font-size-style{font-weight:700;font-size:16px}.tooltip-container .pro-dash-tooltip .indent{padding-left:30px}.pro-router-wrap{height:100%}.pro-router-wrap>div{float:left;height:100%}.pro-router-wrap>div>div{border:1px solid var(--border-color);border-radius:1px}.pro-router-wrap>div .title-wrap{padding:15px}.pro-router-wrap>div .title-wrap .title{float:left;font-weight:700;font-size:18px;max-width:300px}.pro-router-wrap>div .title-wrap .tip-icon{float:right;margin-right:10px;font-size:20px;color:#6c7280}.pro-router-wrap>div .title-wrap .tip-icon .el-icon-warning{cursor:pointer}.pro-router-wrap>div .title-wrap .tip-icon .el-icon-warning:hover:before{color:var(--theme-color)}.pro-router-wrap>div .title-wrap .view-detail{float:right;cursor:pointer;font-size:12px;height:24px;line-height:24px}.pro-router-wrap>div .title-wrap .view-detail a{color:var(--theme-color)!important;padding-right:6px}.pro-router-wrap>div .title-wrap .view-detail button{color:var(--theme-color);border:none;background-color:var(--bg-color);cursor:pointer}.pro-router-wrap>div .title-wrap .view-detail button.disabled{cursor:not-allowed;color:var(--button-disabled-font-color)}.pro-router-wrap>div .title-wrap:after{content:"";clear:both;display:block}.pro-router-wrap>div .loading-icon{margin-left:5px}.pro-router-wrap .pro-router-left{width:calc(100% - 400px);padding-right:15px}.pro-router-wrap .pro-router-left .step-trace{height:calc(100% - 275px);margin-bottom:15px}.pro-router-wrap .pro-router-left .step-trace .trace-container{width:100%;height:calc(100% - 54px);overflow:auto}.pro-router-wrap .pro-router-left .minddata{height:260px}.pro-router-wrap .pro-router-right{width:400px}.pro-router-wrap .pro-router-right .op-time-consume{height:calc(100% - 275px);margin-bottom:15px}.pro-router-wrap .pro-router-right .op-time-consume .time-list{height:calc(40% - 52px)}.pro-router-wrap .pro-router-right .op-time-consume .time-list .item{height:25px;line-height:25px;padding:0 20px}.pro-router-wrap .pro-router-right .op-time-consume .time-list .item>span{display:inline-block;height:100%;vertical-align:middle}.pro-router-wrap .pro-router-right .op-time-consume .time-list .item .index{color:#fff;background-color:#6c92fa;width:20px;height:20px;border-radius:20px;text-align:center;vertical-align:middle;line-height:20px}.pro-router-wrap .pro-router-right .op-time-consume .time-list .item .name{margin-left:10px;width:calc(50% - 30px);text-overflow:ellipsis;overflow:hidden}.pro-router-wrap .pro-router-right .op-time-consume .time-list .item .num{width:20%}.pro-router-wrap .pro-router-right .op-time-consume .time-list .item .time{width:30%;position:relative}.pro-router-wrap .pro-router-right .op-time-consume .time-list .item .time span{display:inline-block;position:absolute;left:0;height:20px}.pro-router-wrap .pro-router-right .op-time-consume .time-list .item .time .bar{background-color:var(--operator-bar-bg-color);top:2px}.pro-router-wrap .pro-router-right .op-time-consume .time-list .item .time .value{line-height:25px;height:25px}.pro-router-wrap .pro-router-right .time-line{height:260px}.pro-router-wrap .op-time-content{height:calc(100% - 72px);overflow:auto}.pro-router-wrap .pie-chart{width:100%;height:260px;overflow:hidden}.pro-router-wrap .image-noData{width:100%;height:calc(100% - 52px);min-height:194px;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.pro-router-wrap .image-noData p{font-size:16px;padding-top:10px}