.cl-resource-content[data-v-3ee07de5]{height:100%}.cl-resource-content .dashboard-item[data-v-3ee07de5]{width:100%;height:100%;padding:15px;border:solid 1px var(--border-color);border-radius:4px}.cl-resource-content .dashboard-item .title-item[data-v-3ee07de5]{display:-webkit-box;display:-ms-flexbox;display:flex;height:24px}.cl-resource-content .dashboard-item .title-item .title-text[data-v-3ee07de5]{-webkit-box-flex:1;-ms-flex:1;flex:1;font-size:18px;font-weight:700;line-height:24px}.cl-resource-content .dashboard-item .title-item .detail-link[data-v-3ee07de5]{cursor:pointer;font-size:12px;height:18px;line-height:12px;padding-top:2px}.cl-resource-content .dashboard-item .title-item .detail-link a[data-v-3ee07de5]{color:#00a5a7;padding-right:6px}.cl-resource-content .dashboard-item .title-item .detail-link button[data-v-3ee07de5]{color:#00a5a7;border:none;background-color:var(--bg-color);cursor:pointer}.cl-resource-content .dashboard-item .title-item .detail-link.disabled button[data-v-3ee07de5]{color:#c0c4cc;cursor:not-allowed}.cl-resource-content .dashboard-item .content-item[data-v-3ee07de5]{height:calc(100% - 38px);margin-top:20px}.cl-resource-content .dashboard-item .content-item .dashboard-chart-content[data-v-3ee07de5]{width:100%;height:100%}.cl-resource-content .margin-item[data-v-3ee07de5]{margin-top:20px}.cl-resource-content .noData-content[data-v-3ee07de5]{width:100%;height:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;-webkit-box-align:center;-ms-flex-align:center;align-items:center;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column}.cl-resource-content .noData-content .noData-text[data-v-3ee07de5],.cl-resource-content .noData-content p[data-v-3ee07de5]{font-size:16px}.content-item .cpu-info[data-v-3ee07de5]{display:grid;grid-template-columns:1fr 300px;height:100%}.content-item .cpu-info .cpu-chart[data-v-3ee07de5]{height:100%;width:100%}.content-item .cpu-info .cpu-chart-info[data-v-3ee07de5]{height:100%;width:100%;display:-webkit-box;display:-ms-flexbox;display:flex;-webkit-box-orient:vertical;-webkit-box-direction:normal;-ms-flex-direction:column;flex-direction:column;-webkit-box-pack:center;-ms-flex-pack:center;justify-content:center;padding-left:20px;background-color:var(--module-bg-color)}.content-item .cpu-chart-info .info-title[data-v-3ee07de5]{font-size:14px;font-weight:700;line-height:30px}.content-item .cpu-chart-info .info-line[data-v-3ee07de5]{line-height:30px}.cpu-chart-tip,.cpu-chart-tip[data-v-3ee07de5]{display:inline-block;margin-right:5px;border-radius:10px;width:10px;height:10px}