(window["webpackJsonp"]=window["webpackJsonp"]||[]).push([["chunk-f4faca80"],{"014a":function(e,t,i){e.exports=i.p+"static/img/icon2.43db58cc.svg"},"0210":function(e,t,i){"use strict";i("f848")},"04c5":function(e,t,i){e.exports=i.p+"static/img/graph-step2.b46dacf6.svg"},"0946":function(e,t,i){e.exports=i.p+"static/img/deb-operator.6c48bc92.png"},"0a03":function(e,t,i){e.exports=i.p+"static/img/dynamic-shape-node.2b7eae99.svg"},"0a6a":function(e,t,i){e.exports=i.p+"static/img/icon6.55a35373.svg"},"129f":function(e,t){e.exports=Object.is||function(e,t){return e===t?0!==e||1/e===1/t:e!=e&&t!=t}},1300:function(e,t,i){e.exports=i.p+"static/img/map-dataset.00aaf154.svg"},1684:function(e,t,i){e.exports=i.p+"static/img/explain-tn.bfc8a666.svg"},"18bc":function(e,t,i){e.exports=i.p+"static/img/graph-step3-en.91fc058d.svg"},"1a09":function(e,t,i){var n={"./0/name-scope.svg":"77fb","./1/name-scope.svg":"9e69"};function a(e){var t=o(e);return i(t)}function o(e){if(!i.o(n,e)){var t=new Error("Cannot find module '"+e+"'");throw t.code="MODULE_NOT_FOUND",t}return n[e]}a.keys=function(){return Object.keys(n)},a.resolve=o,e.exports=a,a.id="1a09"},"1b54":function(e,t,i){e.exports=i.p+"static/img/queue.9041b2bc.svg"},"1cac":function(e,t,i){e.exports=i.p+"static/img/virtual-node.09ce0c16.svg"},"1db6":function(e,t,i){e.exports=i.p+"static/img/collapse-left.16dd02f7.svg"},"1ec7":function(e,t,i){e.exports=i.p+"static/img/graph-step2-en.e8383e96.svg"},"241b":function(e,t,i){e.exports=i.p+"static/img/guideIconHover.944fb0d3.svg"},"26b4":function(e,t,i){e.exports=i.p+"static/img/graph-step3.dc430e83.svg"},"29c4":function(e,t,i){e.exports=i.p+"static/img/polymetric.34b3b326.svg"},"2b98":function(e,t,i){e.exports=i.p+"static/img/shuffle-dataset.328078ac.svg"},"2ca0":function(e,t,i){"use strict";var n=i("23e7"),a=i("06cf").f,o=i("50c4"),s=i("5a34"),r=i("1d80"),l=i("ab13"),c=i("c430"),h="".startsWith,d=Math.min,u=l("startsWith"),p=!c&&!u&&!!function(){var e=a(String.prototype,"startsWith");return e&&!e.writable}();n({target:"String",proto:!0,forced:!p&&!u},{startsWith:function(e){var t=String(r(this));s(e);var i=o(d(arguments.length>1?arguments[1]:void 0,t.length)),n=String(e);return h?h.call(t,n,i):t.slice(i,i+n.length)===n}})},"2cd9":function(e,t,i){e.exports=i.p+"static/img/virtual-node.09ce0c16.svg"},"2de0":function(e,t,i){e.exports=i.p+"static/img/pipeline-close.85b13a23.svg"},"2df0":function(e,t,i){var n={"./0/dynamic-shape-node.svg":"0a03","./1/dynamic-shape-node.svg":"cb78"};function a(e){var t=o(e);return i(t)}function o(e){if(!i.o(n,e)){var t=new Error("Cannot find module '"+e+"'");throw t.code="MODULE_NOT_FOUND",t}return n[e]}a.keys=function(){return Object.keys(n)},a.resolve=o,e.exports=a,a.id="2df0"},"2eb2":function(e,t,i){e.exports=i.p+"static/img/icon5.543e067c.svg"},"2fcd":function(e,t,i){e.exports=i.p+"static/img/repeat-dataset.129b2a7f.svg"},3015:function(e,t,i){"use strict";i("bb51")},"30d6":function(e,t){e.exports="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAAyZpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADw/eHBhY2tldCBiZWdpbj0i77u/IiBpZD0iVzVNME1wQ2VoaUh6cmVTek5UY3prYzlkIj8+IDx4OnhtcG1ldGEgeG1sbnM6eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IkFkb2JlIFhNUCBDb3JlIDUuNi1jMTM4IDc5LjE1OTgyNCwgMjAxNi8wOS8xNC0wMTowOTowMSAgICAgICAgIj4gPHJkZjpSREYgeG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4gPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIgeG1sbnM6eG1wPSJodHRwOi8vbnMuYWRvYmUuY29tL3hhcC8xLjAvIiB4bWxuczp4bXBNTT0iaHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wL21tLyIgeG1sbnM6c3RSZWY9Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEuMC9zVHlwZS9SZXNvdXJjZVJlZiMiIHhtcDpDcmVhdG9yVG9vbD0iQWRvYmUgUGhvdG9zaG9wIENDIDIwMTcgKFdpbmRvd3MpIiB4bXBNTTpJbnN0YW5jZUlEPSJ4bXAuaWlkOkMyMzY4N0NDQUQyMzExRTk5OEVEQzJBOEUzMkNBMTQyIiB4bXBNTTpEb2N1bWVudElEPSJ4bXAuZGlkOkMyMzY4N0NEQUQyMzExRTk5OEVEQzJBOEUzMkNBMTQyIj4gPHhtcE1NOkRlcml2ZWRGcm9tIHN0UmVmOmluc3RhbmNlSUQ9InhtcC5paWQ6QzIzNjg3Q0FBRDIzMTFFOTk4RURDMkE4RTMyQ0ExNDIiIHN0UmVmOmRvY3VtZW50SUQ9InhtcC5kaWQ6QzIzNjg3Q0JBRDIzMTFFOTk4RURDMkE4RTMyQ0ExNDIiLz4gPC9yZGY6RGVzY3JpcHRpb24+IDwvcmRmOlJERj4gPC94OnhtcG1ldGE+IDw/eHBhY2tldCBlbmQ9InIiPz60vH4DAAAAw0lEQVR42mL8//8/AyWAkVIDWGbNmkWRAUwMFAKYAVeAeAMQcxFQzwbEK4H4DLoBIAF/IN4OxHw4NIMM3wTEYUB8Cd2AZCCeAcR2QLwXiIXRNPNCDXcHYlCgpaAb8BeIs4C4C4hNgPggEEtC5QSBeA/U8D4gzgDif9gCERSf5UBcA8TaQHwYiM2AeB+UbgbiYqg6RDRi8WsrEH8C4olAfBIqVg51HdHROBmI44H4LRBn49KMywUwsBiKaZuUh4EBAAEGAI/gMqWZGi6/AAAAAElFTkSuQmCC"},"326d":function(e,t,i){},3325:function(e,t,i){e.exports=i.p+"static/img/constant-node.8e641e28.svg"},"341a":function(e,t,i){e.exports=i.p+"static/img/explain-fp.2a3ac1ea.svg"},"344d":function(e,t,i){e.exports=i.p+"static/img/graph-step3-en.e2fbd792.svg"},3581:function(e,t,i){e.exports=i.p+"static/img/graph-stepimg2.8f67ead9.svg"},3805:function(e,t,i){e.exports=i.p+"static/img/collapse-right.281a4cc4.svg"},"385b":function(e,t,i){e.exports=i.p+"static/img/graph-step1.bcb87343.svg"},"3a3f":function(module,exports){
/**
 * @license
 * (c) 2009-2016 Michael Leibman
 * michael{dot}leibman{at}gmail{dot}com
 * http://github.com/mleibman/slickgrid
 *
 * Distributed under MIT license.
 * All rights reserved.
 *
 * SlickGrid v2.4
 *
 * NOTES:
 *     Cell/row DOM manipulations are done directly bypassing jQuery's DOM manipulation methods.
 *     This increases the speed dramatically, but can only be done safely because there are no event handlers
 *     or data associated with any cell/row DOM nodes.  Cell editors must make sure they implement .destroy()
 *     and do proper cleanup.
 */
if("undefined"===typeof jQuery)throw new Error("SlickGrid requires jquery module to be loaded");if(!jQuery.fn.drag)throw new Error("SlickGrid requires jquery.event.drag module to be loaded");if("undefined"===typeof Slick)throw new Error("slick.core.js not loaded");(function($){"use strict";var scrollbarDimensions,maxSupportedCssHeight;function SlickGrid(container,data,columns,options){var defaults={alwaysShowVerticalScroll:!1,alwaysAllowHorizontalScroll:!1,explicitInitialization:!1,rowHeight:25,defaultColumnWidth:80,enableAddRow:!1,leaveSpaceForNewRows:!1,editable:!1,autoEdit:!0,suppressActiveCellChangeOnEdit:!1,enableCellNavigation:!0,enableColumnReorder:!0,asyncEditorLoading:!1,asyncEditorLoadDelay:100,forceFitColumns:!1,enableAsyncPostRender:!1,asyncPostRenderDelay:50,enableAsyncPostRenderCleanup:!1,asyncPostRenderCleanupDelay:40,autoHeight:!1,editorLock:Slick.GlobalEditorLock,showColumnHeader:!0,showHeaderRow:!1,headerRowHeight:25,createFooterRow:!1,showFooterRow:!1,footerRowHeight:25,createPreHeaderPanel:!1,showPreHeaderPanel:!1,preHeaderPanelHeight:25,showTopPanel:!1,topPanelHeight:25,formatterFactory:null,editorFactory:null,cellFlashingCssClass:"flashing",selectedCellCssClass:"selected",multiSelect:!0,enableTextSelectionOnCells:!1,dataItemColumnValueExtractor:null,frozenBottom:!1,frozenColumn:-1,frozenRow:-1,fullWidthRows:!1,multiColumnSort:!1,numberedMultiColumnSort:!1,tristateMultiColumnSort:!1,sortColNumberInSeparateSpan:!1,defaultFormatter:defaultFormatter,forceSyncScrolling:!1,addNewRowCssClass:"new-row",preserveCopiedSelectionOnPaste:!1,showCellSelection:!0,viewportClass:null,minRowBuffer:3,emulatePagingWhenScrolling:!0,editorCellNavOnLRKeys:!1,doPaging:!0,autosizeColsMode:Slick.GridAutosizeColsMode.LegacyOff,autosizeColPaddingPx:4,autosizeTextAvgToMWidthRatio:.75,viewportSwitchToScrollModeWidthPercent:void 0,viewportMinWidthPx:void 0,viewportMaxWidthPx:void 0},columnDefaults={name:"",resizable:!0,sortable:!1,minWidth:30,maxWidth:void 0,rerenderOnResize:!1,headerCssClass:null,defaultSortAsc:!0,focusable:!0,selectable:!0},columnAutosizeDefaults={ignoreHeaderText:!1,colValueArray:void 0,allowAddlPercent:void 0,formatterOverride:void 0,autosizeMode:Slick.ColAutosizeMode.ContentIntelligent,rowSelectionModeOnInit:void 0,rowSelectionMode:Slick.RowSelectionMode.FirstNRows,rowSelectionCount:100,valueFilterMode:Slick.ValueFilterMode.None,widthEvalMode:Slick.WidthEvalMode.CanvasTextSize,sizeToRemaining:void 0,widthPx:void 0,colDataTypeOf:void 0},th,h,ph,n,cj,page=0,offset=0,vScrollDir=1,initialized=!1,$container,uid="slickgrid_"+Math.round(1e6*Math.random()),self=this,$focusSink,$focusSink2,$groupHeaders=$(),$headerScroller,$headers,$headerRow,$headerRowScroller,$headerRowSpacerL,$headerRowSpacerR,$footerRow,$footerRowScroller,$footerRowSpacerL,$footerRowSpacerR,$preHeaderPanel,$preHeaderPanelScroller,$preHeaderPanelSpacer,$preHeaderPanelR,$preHeaderPanelScrollerR,$preHeaderPanelSpacerR,$topPanelScroller,$topPanel,$viewport,$canvas,$style,$boundAncestors,treeColumns,stylesheet,columnCssRulesL,columnCssRulesR,viewportH,viewportW,canvasWidth,canvasWidthL,canvasWidthR,headersWidth,headersWidthL,headersWidthR,viewportHasHScroll,viewportHasVScroll,headerColumnWidthDiff=0,headerColumnHeightDiff=0,cellWidthDiff=0,cellHeightDiff=0,jQueryNewWidthBehaviour=!1,absoluteColumnMinWidth,hasFrozenRows=!1,frozenRowsHeight=0,actualFrozenRow=-1,paneTopH=0,paneBottomH=0,viewportTopH=0,viewportBottomH=0,topPanelH=0,headerRowH=0,footerRowH=0,tabbingDirection=1,$activeCanvasNode,$activeViewportNode,activePosX,activeRow,activeCell,activeCellNode=null,currentEditor=null,serializedEditorValue,editController,rowsCache={},renderedRows=0,numVisibleRows=0,prevScrollTop=0,scrollTop=0,lastRenderedScrollTop=0,lastRenderedScrollLeft=0,prevScrollLeft=0,scrollLeft=0,selectionModel,selectedRows=[],plugins=[],cellCssClasses={},columnsById={},sortColumns=[],columnPosLeft=[],columnPosRight=[],pagingActive=!1,pagingIsLastPage=!1,scrollThrottle=ActionThrottle(render,50),h_editorLoader=null,h_render=null,h_postrender=null,h_postrenderCleanup=null,postProcessedRows={},postProcessToRow=null,postProcessFromRow=null,postProcessedCleanupQueue=[],postProcessgroupId=0,counter_rows_rendered=0,counter_rows_removed=0,rowNodeFromLastMouseWheelEvent,zombieRowNodeFromLastMouseWheelEvent,zombieRowCacheFromLastMouseWheelEvent,zombieRowPostProcessedFromLastMouseWheelEvent,$paneHeaderL,$paneHeaderR,$paneTopL,$paneTopR,$paneBottomL,$paneBottomR,$headerScrollerL,$headerScrollerR,$headerL,$headerR,$groupHeadersL,$groupHeadersR,$headerRowScrollerL,$headerRowScrollerR,$footerRowScrollerL,$footerRowScrollerR,$headerRowL,$headerRowR,$footerRowL,$footerRowR,$topPanelScrollerL,$topPanelScrollerR,$topPanelL,$topPanelR,$viewportTopL,$viewportTopR,$viewportBottomL,$viewportBottomR,$canvasTopL,$canvasTopR,$canvasBottomL,$canvasBottomR,$viewportScrollContainerX,$viewportScrollContainerY,$headerScrollContainer,$headerRowScrollContainer,$footerRowScrollContainer,cssShow={position:"absolute",visibility:"hidden",display:"block"},$hiddenParents,oldProps=[],columnResizeDragging=!1;function init(){if($container=container instanceof jQuery?container:$(container),$container.length<1)throw new Error("SlickGrid requires a valid container, "+container+" does not exist in the DOM.");if(cacheCssForHiddenInit(),maxSupportedCssHeight=maxSupportedCssHeight||getMaxSupportedCssHeight(),options=$.extend({},defaults,options),validateAndEnforceOptions(),columnDefaults.width=options.defaultColumnWidth,treeColumns=new Slick.TreeColumns(columns),columns=treeColumns.extractColumns(),updateColumnProps(),options.enableColumnReorder&&!$.fn.sortable)throw new Error("SlickGrid's 'enableColumnReorder = true' option requires jquery-ui.sortable module to be loaded");if(editController={commitCurrentEdit:commitCurrentEdit,cancelCurrentEdit:cancelCurrentEdit},$container.empty().css("overflow","hidden").css("outline",0).addClass(uid).addClass("ui-widget"),/relative|absolute|fixed/.test($container.css("position"))||$container.css("position","relative"),$focusSink=$("<div tabIndex='0' hideFocus style='position:fixed;width:0;height:0;top:0;left:0;outline:0;'></div>").appendTo($container),$paneHeaderL=$("<div class='slick-pane slick-pane-header slick-pane-left' tabIndex='0' />").appendTo($container),$paneHeaderR=$("<div class='slick-pane slick-pane-header slick-pane-right' tabIndex='0' />").appendTo($container),$paneTopL=$("<div class='slick-pane slick-pane-top slick-pane-left' tabIndex='0' />").appendTo($container),$paneTopR=$("<div class='slick-pane slick-pane-top slick-pane-right' tabIndex='0' />").appendTo($container),$paneBottomL=$("<div class='slick-pane slick-pane-bottom slick-pane-left' tabIndex='0' />").appendTo($container),$paneBottomR=$("<div class='slick-pane slick-pane-bottom slick-pane-right' tabIndex='0' />").appendTo($container),options.createPreHeaderPanel&&($preHeaderPanelScroller=$("<div class='slick-preheader-panel ui-state-default' style='overflow:hidden;position:relative;' />").appendTo($paneHeaderL),$preHeaderPanel=$("<div />").appendTo($preHeaderPanelScroller),$preHeaderPanelSpacer=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($preHeaderPanelScroller),$preHeaderPanelScrollerR=$("<div class='slick-preheader-panel ui-state-default' style='overflow:hidden;position:relative;' />").appendTo($paneHeaderR),$preHeaderPanelR=$("<div />").appendTo($preHeaderPanelScrollerR),$preHeaderPanelSpacerR=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($preHeaderPanelScrollerR),options.showPreHeaderPanel||($preHeaderPanelScroller.hide(),$preHeaderPanelScrollerR.hide())),$headerScrollerL=$("<div class='slick-header ui-state-default slick-header-left' />").appendTo($paneHeaderL),$headerScrollerR=$("<div class='slick-header ui-state-default slick-header-right' />").appendTo($paneHeaderR),$headerScroller=$().add($headerScrollerL).add($headerScrollerR),treeColumns.hasDepth()){$groupHeadersL=[],$groupHeadersR=[];for(var e=0;e<treeColumns.getDepth()-1;e++)$groupHeadersL[e]=$("<div class='slick-group-header-columns slick-group-header-columns-left' style='left:-1000px' />").appendTo($headerScrollerL),$groupHeadersR[e]=$("<div class='slick-group-header-columns slick-group-header-columns-right' style='left:-1000px' />").appendTo($headerScrollerR);$groupHeaders=$().add($groupHeadersL).add($groupHeadersR)}$headerL=$("<div class='slick-header-columns slick-header-columns-left' style='left:-1000px' />").appendTo($headerScrollerL),$headerR=$("<div class='slick-header-columns slick-header-columns-right' style='left:-1000px' />").appendTo($headerScrollerR),$headers=$().add($headerL).add($headerR),$headerRowScrollerL=$("<div class='slick-headerrow ui-state-default' />").appendTo($paneTopL),$headerRowScrollerR=$("<div class='slick-headerrow ui-state-default' />").appendTo($paneTopR),$headerRowScroller=$().add($headerRowScrollerL).add($headerRowScrollerR),$headerRowSpacerL=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($headerRowScrollerL),$headerRowSpacerR=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($headerRowScrollerR),$headerRowL=$("<div class='slick-headerrow-columns slick-headerrow-columns-left' />").appendTo($headerRowScrollerL),$headerRowR=$("<div class='slick-headerrow-columns slick-headerrow-columns-right' />").appendTo($headerRowScrollerR),$headerRow=$().add($headerRowL).add($headerRowR),$topPanelScrollerL=$("<div class='slick-top-panel-scroller ui-state-default' />").appendTo($paneTopL),$topPanelScrollerR=$("<div class='slick-top-panel-scroller ui-state-default' />").appendTo($paneTopR),$topPanelScroller=$().add($topPanelScrollerL).add($topPanelScrollerR),$topPanelL=$("<div class='slick-top-panel' style='width:10000px' />").appendTo($topPanelScrollerL),$topPanelR=$("<div class='slick-top-panel' style='width:10000px' />").appendTo($topPanelScrollerR),$topPanel=$().add($topPanelL).add($topPanelR),options.showColumnHeader||$headerScroller.hide(),options.showTopPanel||$topPanelScroller.hide(),options.showHeaderRow||$headerRowScroller.hide(),$viewportTopL=$("<div class='slick-viewport slick-viewport-top slick-viewport-left' tabIndex='0' hideFocus />").appendTo($paneTopL),$viewportTopR=$("<div class='slick-viewport slick-viewport-top slick-viewport-right' tabIndex='0' hideFocus />").appendTo($paneTopR),$viewportBottomL=$("<div class='slick-viewport slick-viewport-bottom slick-viewport-left' tabIndex='0' hideFocus />").appendTo($paneBottomL),$viewportBottomR=$("<div class='slick-viewport slick-viewport-bottom slick-viewport-right' tabIndex='0' hideFocus />").appendTo($paneBottomR),$viewport=$().add($viewportTopL).add($viewportTopR).add($viewportBottomL).add($viewportBottomR),$activeViewportNode=$viewportTopL,$canvasTopL=$("<div class='grid-canvas grid-canvas-top grid-canvas-left' tabIndex='0' hideFocus />").appendTo($viewportTopL),$canvasTopR=$("<div class='grid-canvas grid-canvas-top grid-canvas-right' tabIndex='0' hideFocus />").appendTo($viewportTopR),$canvasBottomL=$("<div class='grid-canvas grid-canvas-bottom grid-canvas-left' tabIndex='0' hideFocus />").appendTo($viewportBottomL),$canvasBottomR=$("<div class='grid-canvas grid-canvas-bottom grid-canvas-right' tabIndex='0' hideFocus />").appendTo($viewportBottomR),options.viewportClass&&$viewport.toggleClass(options.viewportClass,!0),$canvas=$().add($canvasTopL).add($canvasTopR).add($canvasBottomL).add($canvasBottomR),scrollbarDimensions=scrollbarDimensions||measureScrollbar(),$activeCanvasNode=$canvasTopL,$preHeaderPanelSpacer&&$preHeaderPanelSpacer.css("width",getCanvasWidth()+scrollbarDimensions.width+"px"),$headers.width(getHeadersWidth()),$headerRowSpacerL.css("width",getCanvasWidth()+scrollbarDimensions.width+"px"),$headerRowSpacerR.css("width",getCanvasWidth()+scrollbarDimensions.width+"px"),options.createFooterRow&&($footerRowScrollerR=$("<div class='slick-footerrow ui-state-default' />").appendTo($paneTopR),$footerRowScrollerL=$("<div class='slick-footerrow ui-state-default' />").appendTo($paneTopL),$footerRowScroller=$().add($footerRowScrollerL).add($footerRowScrollerR),$footerRowSpacerL=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").css("width",getCanvasWidth()+scrollbarDimensions.width+"px").appendTo($footerRowScrollerL),$footerRowSpacerR=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").css("width",getCanvasWidth()+scrollbarDimensions.width+"px").appendTo($footerRowScrollerR),$footerRowL=$("<div class='slick-footerrow-columns slick-footerrow-columns-left' />").appendTo($footerRowScrollerL),$footerRowR=$("<div class='slick-footerrow-columns slick-footerrow-columns-right' />").appendTo($footerRowScrollerR),$footerRow=$().add($footerRowL).add($footerRowR),options.showFooterRow||$footerRowScroller.hide()),$focusSink2=$focusSink.clone().appendTo($container),options.explicitInitialization||finishInitialization()}function finishInitialization(){initialized||(initialized=!0,getViewportWidth(),getViewportHeight(),measureCellPaddingAndBorder(),disableSelection($headers),options.enableTextSelectionOnCells||$viewport.on("selectstart.ui",(function(e){return $(e.target).is("input,textarea")})),setFrozenOptions(),setPaneVisibility(),setScroller(),setOverflow(),updateColumnCaches(),createColumnHeaders(),createColumnGroupHeaders(),createColumnFooter(),setupColumnSort(),createCssRules(),resizeCanvas(),bindAncestorScrollEvents(),$container.on("resize.slickgrid",resizeCanvas),$viewport.on("scroll",handleScroll),jQuery.fn.mousewheel&&$viewport.on("mousewheel",handleMouseWheel),$headerScroller.on("contextmenu",handleHeaderContextMenu).on("click",handleHeaderClick).on("mouseenter",".slick-header-column",handleHeaderMouseEnter).on("mouseleave",".slick-header-column",handleHeaderMouseLeave),$headerRowScroller.on("scroll",handleHeaderRowScroll),options.createFooterRow&&($footerRow.on("contextmenu",handleFooterContextMenu).on("click",handleFooterClick),$footerRowScroller.on("scroll",handleFooterRowScroll)),options.createPreHeaderPanel&&$preHeaderPanelScroller.on("scroll",handlePreHeaderPanelScroll),$focusSink.add($focusSink2).on("keydown",handleKeyDown),$canvas.on("keydown",handleKeyDown).on("click",handleClick).on("dblclick",handleDblClick).on("contextmenu",handleContextMenu).on("draginit",handleDragInit).on("dragstart",{distance:3},handleDragStart).on("drag",handleDrag).on("dragend",handleDragEnd).on("mouseenter",".slick-cell",handleMouseEnter).on("mouseleave",".slick-cell",handleMouseLeave),restoreCssFromHiddenInit())}function cacheCssForHiddenInit(){$hiddenParents=$container.parents().addBack().not(":visible"),$hiddenParents.each((function(){var e={};for(var t in cssShow)e[t]=this.style[t],this.style[t]=cssShow[t];oldProps.push(e)}))}function restoreCssFromHiddenInit(){$hiddenParents.each((function(e){var t=oldProps[e];for(var i in cssShow)this.style[i]=t[i]}))}function hasFrozenColumns(){return options.frozenColumn>-1}function registerPlugin(e){plugins.unshift(e),e.init(self)}function unregisterPlugin(e){for(var t=plugins.length;t>=0;t--)if(plugins[t]===e){plugins[t].destroy&&plugins[t].destroy(),plugins.splice(t,1);break}}function getPluginByName(e){for(var t=plugins.length-1;t>=0;t--)if(plugins[t].pluginName===e)return plugins[t]}function setSelectionModel(e){selectionModel&&(selectionModel.onSelectedRangesChanged.unsubscribe(handleSelectedRangesChanged),selectionModel.destroy&&selectionModel.destroy()),selectionModel=e,selectionModel&&(selectionModel.init(self),selectionModel.onSelectedRangesChanged.subscribe(handleSelectedRangesChanged))}function getSelectionModel(){return selectionModel}function getCanvasNode(e,t){e||(e=0),t||(t=0);var i="number"===typeof e?e:getColumnIndex(e);return hasFrozenRows&&t>=actualFrozenRow+(options.frozenBottom?0:1)?hasFrozenColumns()&&i>options.frozenColumn?$canvasBottomR[0]:$canvasBottomL[0]:hasFrozenColumns()&&i>options.frozenColumn?$canvasTopR[0]:$canvasTopL[0]}function getActiveCanvasNode(e){return setActiveCanvasNode(e),$activeCanvasNode[0]}function getCanvases(){return $canvas}function setActiveCanvasNode(e){e&&($activeCanvasNode=$(e.target).closest(".grid-canvas"))}function getViewportNode(){return $viewport[0]}function getActiveViewportNode(e){return setActiveViewPortNode(e),$activeViewportNode[0]}function setActiveViewportNode(e){e&&($activeViewportNode=$(e.target).closest(".slick-viewport"))}function measureScrollbar(){var e=$('<div class="'+$viewport.className+'" style="position:absolute; top:-10000px; left:-10000px; overflow:auto; width:100px; height:100px;"></div>').appendTo("body"),t=$('<div style="width:200px; height:200px; overflow:auto;"></div>').appendTo(e),i={width:e[0].offsetWidth-e[0].clientWidth,height:e[0].offsetHeight-e[0].clientHeight};return t.remove(),e.remove(),i}function getHeadersWidth(){headersWidth=headersWidthL=headersWidthR=0;for(var e=!options.autoHeight,t=0,i=columns.length;t<i;t++){var n=columns[t].width;options.frozenColumn>-1&&t>options.frozenColumn?headersWidthR+=n:headersWidthL+=n}return e&&(options.frozenColumn>-1&&t>options.frozenColumn?headersWidthR+=scrollbarDimensions.width:headersWidthL+=scrollbarDimensions.width),hasFrozenColumns()?(headersWidthL+=1e3,headersWidthR=Math.max(headersWidthR,viewportW)+headersWidthL,headersWidthR+=scrollbarDimensions.width):(headersWidthL+=scrollbarDimensions.width,headersWidthL=Math.max(headersWidthL,viewportW)+1e3),headersWidth=headersWidthL+headersWidthR,Math.max(headersWidth,viewportW)+1e3}function getHeadersWidthL(){return headersWidthL=0,columns.forEach((function(e,t){options.frozenColumn>-1&&t>options.frozenColumn||(headersWidthL+=e.width)})),hasFrozenColumns()?headersWidthL+=1e3:(headersWidthL+=scrollbarDimensions.width,headersWidthL=Math.max(headersWidthL,viewportW)+1e3),headersWidthL}function getHeadersWidthR(){return headersWidthR=0,columns.forEach((function(e,t){options.frozenColumn>-1&&t>options.frozenColumn&&(headersWidthR+=e.width)})),hasFrozenColumns()&&(headersWidthR=Math.max(headersWidthR,viewportW)+getHeadersWidthL(),headersWidthR+=scrollbarDimensions.width),headersWidthR}function getCanvasWidth(){var e=viewportHasVScroll?viewportW-scrollbarDimensions.width:viewportW,t=columns.length;canvasWidthL=canvasWidthR=0;while(t--)hasFrozenColumns()&&t>options.frozenColumn?canvasWidthR+=columns[t].width:canvasWidthL+=columns[t].width;var i=canvasWidthL+canvasWidthR;return options.fullWidthRows?Math.max(i,e):i}function updateCanvasWidth(e){var t,i=canvasWidth,n=canvasWidthL,a=canvasWidthR;canvasWidth=getCanvasWidth(),t=canvasWidth!==i||canvasWidthL!==n||canvasWidthR!==a,(t||hasFrozenColumns()||hasFrozenRows)&&($canvasTopL.width(canvasWidthL),getHeadersWidth(),$headerL.width(headersWidthL),$headerR.width(headersWidthR),hasFrozenColumns()?($canvasTopR.width(canvasWidthR),$paneHeaderL.width(canvasWidthL),$paneHeaderR.css("left",canvasWidthL),$paneHeaderR.css("width",viewportW-canvasWidthL),$paneTopL.width(canvasWidthL),$paneTopR.css("left",canvasWidthL),$paneTopR.css("width",viewportW-canvasWidthL),$headerRowScrollerL.width(canvasWidthL),$headerRowScrollerR.width(viewportW-canvasWidthL),$headerRowL.width(canvasWidthL),$headerRowR.width(canvasWidthR),options.createFooterRow&&($footerRowScrollerL.width(canvasWidthL),$footerRowScrollerR.width(viewportW-canvasWidthL),$footerRowL.width(canvasWidthL),$footerRowR.width(canvasWidthR)),options.createPreHeaderPanel&&$preHeaderPanel.width(canvasWidth),$viewportTopL.width(canvasWidthL),$viewportTopR.width(viewportW-canvasWidthL),hasFrozenRows&&($paneBottomL.width(canvasWidthL),$paneBottomR.css("left",canvasWidthL),$viewportBottomL.width(canvasWidthL),$viewportBottomR.width(viewportW-canvasWidthL),$canvasBottomL.width(canvasWidthL),$canvasBottomR.width(canvasWidthR))):($paneHeaderL.width("100%"),$paneTopL.width("100%"),$headerRowScrollerL.width("100%"),$headerRowL.width(canvasWidth),options.createFooterRow&&($footerRowScrollerL.width("100%"),$footerRowL.width(canvasWidth)),options.createPreHeaderPanel&&($preHeaderPanel.width("100%"),$preHeaderPanel.width(canvasWidth)),$viewportTopL.width("100%"),hasFrozenRows&&($viewportBottomL.width("100%"),$canvasBottomL.width(canvasWidthL))),viewportHasHScroll=canvasWidth>viewportW-scrollbarDimensions.width),$headerRowSpacerL.width(canvasWidth+(viewportHasVScroll?scrollbarDimensions.width:0)),$headerRowSpacerR.width(canvasWidth+(viewportHasVScroll?scrollbarDimensions.width:0)),options.createFooterRow&&($footerRowSpacerL.width(canvasWidth+(viewportHasVScroll?scrollbarDimensions.width:0)),$footerRowSpacerR.width(canvasWidth+(viewportHasVScroll?scrollbarDimensions.width:0))),(t||e)&&applyColumnWidths()}function disableSelection(e){e&&e.jquery&&e.attr("unselectable","on").css("MozUserSelect","none").on("selectstart.ui",(function(){return!1}))}function getMaxSupportedCssHeight(){var e=1e6,t=navigator.userAgent.toLowerCase().match(/firefox/)?6e6:1e9,i=$("<div style='display:none' />").appendTo(document.body);while(1){var n=2*e;if(i.css("height",n),n>t||i.height()!==n)break;e=n}return i.remove(),e}function getUID(){return uid}function getHeaderColumnWidthDiff(){return headerColumnWidthDiff}function getScrollbarDimensions(){return scrollbarDimensions}function bindAncestorScrollEvents(){var e=hasFrozenRows&&!options.frozenBottom?$canvasBottomL[0]:$canvasTopL[0];while((e=e.parentNode)!=document.body&&null!=e)if(e==$viewportTopL[0]||e.scrollWidth!=e.clientWidth||e.scrollHeight!=e.clientHeight){var t=$(e);$boundAncestors=$boundAncestors?$boundAncestors.add(t):t,t.on("scroll."+uid,handleActiveCellPositionChange)}}function unbindAncestorScrollEvents(){$boundAncestors&&($boundAncestors.off("scroll."+uid),$boundAncestors=null)}function updateColumnHeader(e,t,i){if(initialized){var n=getColumnIndex(e);if(null!=n){var a=columns[n],o=$headers.children().eq(n);o&&(void 0!==t&&(columns[n].name=t),void 0!==i&&(columns[n].toolTip=i),trigger(self.onBeforeHeaderCellDestroy,{node:o[0],column:a,grid:self}),o.attr("title",i||"").children().eq(0).html(t),trigger(self.onHeaderCellRendered,{node:o[0],column:a,grid:self}))}}}function getHeader(e){if(!e)return hasFrozenColumns()?$headers:$headerL;var t=getColumnIndex(e.id);return hasFrozenColumns()?t<=options.frozenColumn?$headerL:$headerR:$headerL}function getHeaderColumn(e){var t="number"===typeof e?e:getColumnIndex(e),i=hasFrozenColumns()?t<=options.frozenColumn?$headerL:$headerR:$headerL,n=hasFrozenColumns()?t<=options.frozenColumn?t:t-options.frozenColumn-1:t,a=i.children().eq(n);return a&&a[0]}function getHeaderRow(){return hasFrozenColumns()?$headerRow:$headerRow[0]}function getFooterRow(){return hasFrozenColumns()?$footerRow:$footerRow[0]}function getPreHeaderPanel(){return $preHeaderPanel[0]}function getPreHeaderPanelRight(){return $preHeaderPanelR[0]}function getHeaderRowColumn(e){var t,i="number"===typeof e?e:getColumnIndex(e);hasFrozenColumns()?i<=options.frozenColumn?t=$headerRowL:(t=$headerRowR,i-=options.frozenColumn+1):t=$headerRowL;var n=t.children().eq(i);return n&&n[0]}function getFooterRowColumn(e){var t,i="number"===typeof e?e:getColumnIndex(e);hasFrozenColumns()?i<=options.frozenColumn?t=$footerRowL:(t=$footerRowR,i-=options.frozenColumn+1):t=$footerRowL;var n=t&&t.children().eq(i);return n&&n[0]}function createColumnFooter(){if(options.createFooterRow){$footerRow.find(".slick-footerrow-column").each((function(){var e=$(this).data("column");e&&trigger(self.onBeforeFooterRowCellDestroy,{node:this,column:e,grid:self})})),$footerRowL.empty(),$footerRowR.empty();for(var e=0;e<columns.length;e++){var t=columns[e],i=$("<div class='ui-state-default slick-footerrow-column l"+e+" r"+e+"'></div>").data("column",t).addClass(hasFrozenColumns()&&e<=options.frozenColumn?"frozen":"").appendTo(hasFrozenColumns()&&e>options.frozenColumn?$footerRowR:$footerRowL);trigger(self.onFooterRowCellRendered,{node:i[0],column:t,grid:self})}}}function createColumnGroupHeaders(){var e=0,t=!1;if(treeColumns.hasDepth()){for(var i=0;i<$groupHeadersL.length;i++){$groupHeadersL[i].empty(),$groupHeadersR[i].empty();var n=treeColumns.getColumnsInDepth(i);for(var a in n){var o=n[a];e+=o.extractColumns().length,hasFrozenColumns()&&0===i&&e-1===options.frozenColumn&&(t=!0),$("<div class='ui-state-default slick-group-header-column' />").html("<span class='slick-column-name'>"+o.name+"</span>").attr("id",""+uid+o.id).attr("title",o.toolTip||"").data("column",o).addClass(o.headerCssClass||"").addClass(hasFrozenColumns()&&e-1>options.frozenColumn?"frozen":"").appendTo(hasFrozenColumns()&&e-1>options.frozenColumn?$groupHeadersR[i]:$groupHeadersL[i])}if(hasFrozenColumns()&&0===i&&!t){$groupHeadersL[i].empty(),$groupHeadersR[i].empty(),alert("All columns of group should to be grouped!");break}}applyColumnGroupHeaderWidths()}}function createColumnHeaders(){function e(){$(this).addClass("ui-state-hover")}function t(){$(this).removeClass("ui-state-hover")}$headers.find(".slick-header-column").each((function(){var e=$(this).data("column");e&&trigger(self.onBeforeHeaderCellDestroy,{node:this,column:e,grid:self})})),$headerL.empty(),$headerR.empty(),getHeadersWidth(),$headerL.width(headersWidthL),$headerR.width(headersWidthR),$headerRow.find(".slick-headerrow-column").each((function(){var e=$(this).data("column");e&&trigger(self.onBeforeHeaderRowCellDestroy,{node:this,column:e,grid:self})})),$headerRowL.empty(),$headerRowR.empty(),options.createFooterRow&&($footerRowL.find(".slick-footerrow-column").each((function(){var e=$(this).data("column");e&&trigger(self.onBeforeFooterRowCellDestroy,{node:this,column:e,grid:self})})),$footerRowL.empty(),hasFrozenColumns()&&($footerRowR.find(".slick-footerrow-column").each((function(){var e=$(this).data("column");e&&trigger(self.onBeforeFooterRowCellDestroy,{node:this,column:e,grid:self})})),$footerRowR.empty()));for(var i=0;i<columns.length;i++){var n=columns[i],a=hasFrozenColumns()?i<=options.frozenColumn?$headerL:$headerR:$headerL,o=hasFrozenColumns()?i<=options.frozenColumn?$headerRowL:$headerRowR:$headerRowL,s=$("<div class='ui-state-default slick-header-column' />").html("<span class='slick-column-name'>"+n.name+"</span>").width(n.width-headerColumnWidthDiff).attr("id",""+uid+n.id).attr("title",n.toolTip||"").data("column",n).addClass(n.headerCssClass||"").addClass(hasFrozenColumns()&&i<=options.frozenColumn?"frozen":"").appendTo(a);if((options.enableColumnReorder||n.sortable)&&s.on("mouseenter",e).on("mouseleave",t),n.hasOwnProperty("headerCellAttrs")&&n.headerCellAttrs instanceof Object)for(var r in n.headerCellAttrs)n.headerCellAttrs.hasOwnProperty(r)&&s.attr(r,n.headerCellAttrs[r]);if(n.sortable&&(s.addClass("slick-header-sortable"),s.append("<span class='slick-sort-indicator"+(options.numberedMultiColumnSort&&!options.sortColNumberInSeparateSpan?" slick-sort-indicator-numbered":"")+"' />"),options.numberedMultiColumnSort&&options.sortColNumberInSeparateSpan&&s.append("<span class='slick-sort-indicator-numbered' />")),trigger(self.onHeaderCellRendered,{node:s[0],column:n,grid:self}),options.showHeaderRow){var l=$("<div class='ui-state-default slick-headerrow-column l"+i+" r"+i+"'></div>").data("column",n).addClass(hasFrozenColumns()&&i<=options.frozenColumn?"frozen":"").appendTo(o);trigger(self.onHeaderRowCellRendered,{node:l[0],column:n,grid:self})}if(options.createFooterRow&&options.showFooterRow){var c=$("<div class='ui-state-default slick-footerrow-column l"+i+" r"+i+"'></div>").data("column",n).appendTo($footerRow);trigger(self.onFooterRowCellRendered,{node:c[0],column:n,grid:self})}}setSortColumns(sortColumns),setupColumnResize(),options.enableColumnReorder&&("function"==typeof options.enableColumnReorder?options.enableColumnReorder(self,$headers,headerColumnWidthDiff,setColumns,setupColumnResize,columns,getColumnIndex,uid,trigger):setupColumnReorder())}function setupColumnSort(){$headers.click((function(e){if(!columnResizeDragging&&(e.metaKey=e.metaKey||e.ctrlKey,!$(e.target).hasClass("slick-resizable-handle"))){var t=$(e.target).closest(".slick-header-column");if(t.length){var i=t.data("column");if(i.sortable){if(!getEditorLock().commitCurrentEdit())return;for(var n=null,a=0;a<sortColumns.length;a++)if(sortColumns[a].columnId==i.id){n=sortColumns[a],n.sortAsc=!n.sortAsc;break}var o=!!n;options.tristateMultiColumnSort?(n||(n={columnId:i.id,sortAsc:i.defaultSortAsc}),o&&n.sortAsc&&(sortColumns.splice(a,1),n=null),options.multiColumnSort||(sortColumns=[]),!n||o&&options.multiColumnSort||sortColumns.push(n)):e.metaKey&&options.multiColumnSort?n&&sortColumns.splice(a,1):((e.shiftKey||e.metaKey)&&options.multiColumnSort||(sortColumns=[]),n?0===sortColumns.length&&sortColumns.push(n):(n={columnId:i.id,sortAsc:i.defaultSortAsc},sortColumns.push(n))),setSortColumns(sortColumns),options.multiColumnSort?trigger(self.onSort,{multiColumnSort:!0,sortCols:$.map(sortColumns,(function(e){return{columnId:columns[getColumnIndex(e.columnId)].id,sortCol:columns[getColumnIndex(e.columnId)],sortAsc:e.sortAsc}}))},e):trigger(self.onSort,{multiColumnSort:!1,columnId:sortColumns.length>0?i.id:null,sortCol:sortColumns.length>0?i:null,sortAsc:!(sortColumns.length>0)||sortColumns[0].sortAsc},e)}}}}))}function currentPositionInHeader(e){var t=0;return $headers.find(".slick-header-column").each((function(i){if(this.id==e)return t=i,!1})),t}function limitPositionInGroup(e){var t,i=0,n=0;return treeColumns.getColumnsInDepth($groupHeadersL.length-1).some((function(a){return i=n,n+=a.columns.length,a.columns.some((function(i){return i.id===e&&(t=a),t})),t})),n--,{start:i,end:n,group:t}}function remove(e,t){var i=e.lastIndexOf(t);i>-1&&(e.splice(i,1),remove(e,t))}function columnPositionValidInGroup(e){var t=currentPositionInHeader(e[0].id),i=limitPositionInGroup(e.data("column").id),n=i.start<=t&&t<=i.end;return{limit:i,valid:n,message:n?"":'Column "'.concat(e.text(),'" can be reordered only within the "',i.group.name,'" group!')}}function setupColumnReorder(){$headers.filter(":ui-sortable").sortable("destroy");var e,t=null;function i(){$viewportScrollContainerX[0].scrollLeft=$viewportScrollContainerX[0].scrollLeft+10}function n(){$viewportScrollContainerX[0].scrollLeft=$viewportScrollContainerX[0].scrollLeft-10}$headers.sortable({containment:"parent",distance:3,axis:"x",cursor:"default",tolerance:"intersection",helper:"clone",placeholder:"slick-sortable-placeholder ui-state-default slick-header-column",start:function(t,i){i.placeholder.width(i.helper.outerWidth()-headerColumnWidthDiff),e=!hasFrozenColumns()||i.placeholder.offset().left+i.placeholder.width()>$viewportScrollContainerX.offset().left,$(i.helper).addClass("slick-header-column-active")},beforeStop:function(e,t){$(t.helper).removeClass("slick-header-column-active")},sort:function(a,o){e&&a.originalEvent.pageX>$container[0].clientWidth?t||(t=setInterval(i,100)):e&&a.originalEvent.pageX<$viewportScrollContainerX.offset().left?t||(t=setInterval(n,100)):(clearInterval(t),t=null)},stop:function(e,i){var n=!1;clearInterval(t),t=null;var a=null;if(treeColumns.hasDepth()){var o=columnPositionValidInGroup(i.item);a=o.limit,n=!o.valid,n&&alert(o.message)}if(!n&&getEditorLock().commitCurrentEdit()){var s=$headerL.sortable("toArray");s=s.concat($headerR.sortable("toArray"));for(var r=[],l=0;l<s.length;l++)r.push(columns[getColumnIndex(s[l].replace(uid,""))]);setColumns(r),trigger(self.onColumnsReordered,{impactedColumns:getImpactedColumns(a)}),e.stopPropagation(),setupColumnResize()}else $(this).sortable("cancel")}})}function getImpactedColumns(e){var t=[];if(e)for(var i=e.start;i<=e.end;i++)t.push(columns[i]);else t=columns;return t}function setupColumnResize(){var e,t,i,n,a,o,s,r,l;a=$headers.children(),a.find(".slick-resizable-handle").remove(),a.each((function(e,t){e>=columns.length||columns[e].resizable&&(void 0===r&&(r=e),l=e)})),void 0!==r&&a.each((function(c,h){c>=columns.length||c<r||options.forceFitColumns&&c>=l||($(h),$("<div class='slick-resizable-handle' />").appendTo(h).on("dragstart",(function(t,r){if(!getEditorLock().commitCurrentEdit())return!1;n=t.pageX,$(this).parent().addClass("slick-header-column-active");var l=null,h=null;if(a.each((function(e,t){e>=columns.length||(columns[e].previousWidth=$(t).outerWidth())})),options.forceFitColumns)for(l=0,h=0,e=c+1;e<columns.length;e++)i=columns[e],i.resizable&&(null!==h&&(i.maxWidth?h+=i.maxWidth-i.previousWidth:h=null),l+=i.previousWidth-Math.max(i.minWidth||0,absoluteColumnMinWidth));var d=0,u=0;for(e=0;e<=c;e++)i=columns[e],i.resizable&&(null!==u&&(i.maxWidth?u+=i.maxWidth-i.previousWidth:u=null),d+=i.previousWidth-Math.max(i.minWidth||0,absoluteColumnMinWidth));null===l&&(l=1e5),null===d&&(d=1e5),null===h&&(h=1e5),null===u&&(u=1e5),s=n+Math.min(l,u),o=n-Math.min(d,h)})).on("drag",(function(a,r){columnResizeDragging=!0;var l,h,d=Math.min(s,Math.max(o,a.pageX))-n,u=0;if(d<0){for(h=d,e=c;e>=0;e--)i=columns[e],i.resizable&&(l=Math.max(i.minWidth||0,absoluteColumnMinWidth),h&&i.previousWidth+h<l?(h+=i.previousWidth-l,i.width=l):(i.width=i.previousWidth+h,h=0));for(t=0;t<=c;t++)i=columns[t],hasFrozenColumns()&&t>options.frozenColumn?i.width:u+=i.width;if(options.forceFitColumns)for(h=-d,e=c+1;e<columns.length;e++)i=columns[e],i.resizable&&(h&&i.maxWidth&&i.maxWidth-i.previousWidth<h?(h-=i.maxWidth-i.previousWidth,i.width=i.maxWidth):(i.width=i.previousWidth+h,h=0),hasFrozenColumns()&&e>options.frozenColumn?i.width:u+=i.width);else for(e=c+1;e<columns.length;e++)i=columns[e],hasFrozenColumns()&&e>options.frozenColumn?i.width:u+=i.width;if(options.forceFitColumns)for(h=-d,e=c+1;e<columns.length;e++)i=columns[e],i.resizable&&(h&&i.maxWidth&&i.maxWidth-i.previousWidth<h?(h-=i.maxWidth-i.previousWidth,i.width=i.maxWidth):(i.width=i.previousWidth+h,h=0))}else{for(h=d,u=0,0,e=c;e>=0;e--)i=columns[e],i.resizable&&(h&&i.maxWidth&&i.maxWidth-i.previousWidth<h?(h-=i.maxWidth-i.previousWidth,i.width=i.maxWidth):(i.width=i.previousWidth+h,h=0));for(t=0;t<=c;t++)i=columns[t],hasFrozenColumns()&&t>options.frozenColumn?i.width:u+=i.width;if(options.forceFitColumns)for(h=-d,e=c+1;e<columns.length;e++)i=columns[e],i.resizable&&(l=Math.max(i.minWidth||0,absoluteColumnMinWidth),h&&i.previousWidth+h<l?(h+=i.previousWidth-l,i.width=l):(i.width=i.previousWidth+h,h=0),hasFrozenColumns()&&e>options.frozenColumn?i.width:u+=i.width);else for(e=c+1;e<columns.length;e++)i=columns[e],hasFrozenColumns()&&e>options.frozenColumn?i.width:u+=i.width}hasFrozenColumns()&&u!=canvasWidthL&&($headerL.width(u+1e3),$paneHeaderR.css("left",u)),applyColumnHeaderWidths(),applyColumnGroupHeaderWidths(),options.syncColumnCellResize&&applyColumnWidths()})).on("dragend",(function(t,n){var o;for($(this).parent().removeClass("slick-header-column-active"),e=0;e<columns.length;e++)i=columns[e],o=$(a[e]).outerWidth(),i.previousWidth!==o&&i.rerenderOnResize&&invalidateAllRows();updateCanvasWidth(!0),render(),trigger(self.onColumnsResized,{triggeredByColumn:$(this).parent().attr("id").replace(uid,"")}),setTimeout((function(){columnResizeDragging=!1}),300)})))}))}function getVBoxDelta(e){var t=["borderTopWidth","borderBottomWidth","paddingTop","paddingBottom"],i=0;return e&&"function"===typeof e.css&&$.each(t,(function(t,n){i+=parseFloat(e.css(n))||0})),i}function setFrozenOptions(){if(options.frozenColumn=options.frozenColumn>=0&&options.frozenColumn<columns.length?parseInt(options.frozenColumn):-1,options.frozenRow>-1){hasFrozenRows=!0,frozenRowsHeight=options.frozenRow*options.rowHeight;var e=getDataLength();actualFrozenRow=options.frozenBottom?e-options.frozenRow:options.frozenRow}else hasFrozenRows=!1}function setPaneVisibility(){hasFrozenColumns()?($paneHeaderR.show(),$paneTopR.show(),hasFrozenRows?($paneBottomL.show(),$paneBottomR.show()):($paneBottomR.hide(),$paneBottomL.hide())):($paneHeaderR.hide(),$paneTopR.hide(),$paneBottomR.hide(),hasFrozenRows?$paneBottomL.show():($paneBottomR.hide(),$paneBottomL.hide()))}function setOverflow(){$viewportTopL.css({"overflow-x":hasFrozenColumns()?hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"hidden":"scroll":hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"hidden":"auto","overflow-y":options.alwaysShowVerticalScroll?"scroll":hasFrozenColumns()?"hidden":hasFrozenRows?"scroll":"auto"}),$viewportTopR.css({"overflow-x":hasFrozenColumns()?hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"hidden":"scroll":hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"hidden":"auto","overflow-y":options.alwaysShowVerticalScroll?"scroll":(hasFrozenColumns(),hasFrozenRows?"scroll":"auto")}),$viewportBottomL.css({"overflow-x":hasFrozenColumns()?hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"scroll":"auto":(hasFrozenRows&&options.alwaysAllowHorizontalScroll,"auto"),"overflow-y":options.alwaysShowVerticalScroll?"scroll":hasFrozenColumns()?"hidden":hasFrozenRows?"scroll":"auto"}),$viewportBottomR.css({"overflow-x":hasFrozenColumns()?hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"scroll":"auto":(hasFrozenRows&&options.alwaysAllowHorizontalScroll,"auto"),"overflow-y":options.alwaysShowVerticalScroll?"scroll":(hasFrozenColumns(),"auto")}),options.viewportClass&&($viewportTopL.toggleClass(options.viewportClass,!0),$viewportTopR.toggleClass(options.viewportClass,!0),$viewportBottomL.toggleClass(options.viewportClass,!0),$viewportBottomR.toggleClass(options.viewportClass,!0))}function setScroller(){hasFrozenColumns()?($headerScrollContainer=$headerScrollerR,$headerRowScrollContainer=$headerRowScrollerR,$footerRowScrollContainer=$footerRowScrollerR,hasFrozenRows?options.frozenBottom?($viewportScrollContainerX=$viewportBottomR,$viewportScrollContainerY=$viewportTopR):$viewportScrollContainerX=$viewportScrollContainerY=$viewportBottomR:$viewportScrollContainerX=$viewportScrollContainerY=$viewportTopR):($headerScrollContainer=$headerScrollerL,$headerRowScrollContainer=$headerRowScrollerL,$footerRowScrollContainer=$footerRowScrollerL,hasFrozenRows?options.frozenBottom?($viewportScrollContainerX=$viewportBottomL,$viewportScrollContainerY=$viewportTopL):$viewportScrollContainerX=$viewportScrollContainerY=$viewportBottomL:$viewportScrollContainerX=$viewportScrollContainerY=$viewportTopL)}function measureCellPaddingAndBorder(){var e,t=["borderLeftWidth","borderRightWidth","paddingLeft","paddingRight"],i=["borderTopWidth","borderBottomWidth","paddingTop","paddingBottom"],n=$.fn.jquery.split(".");jQueryNewWidthBehaviour=1==n[0]&&n[1]>=8||n[0]>=2,e=$("<div class='ui-state-default slick-header-column' style='visibility:hidden'>-</div>").appendTo($headers),headerColumnWidthDiff=headerColumnHeightDiff=0,"border-box"!=e.css("box-sizing")&&"border-box"!=e.css("-moz-box-sizing")&&"border-box"!=e.css("-webkit-box-sizing")&&($.each(t,(function(t,i){headerColumnWidthDiff+=parseFloat(e.css(i))||0})),$.each(i,(function(t,i){headerColumnHeightDiff+=parseFloat(e.css(i))||0}))),e.remove();var a=$("<div class='slick-row' />").appendTo($canvas);e=$("<div class='slick-cell' id='' style='visibility:hidden'>-</div>").appendTo(a),cellWidthDiff=cellHeightDiff=0,"border-box"!=e.css("box-sizing")&&"border-box"!=e.css("-moz-box-sizing")&&"border-box"!=e.css("-webkit-box-sizing")&&($.each(t,(function(t,i){cellWidthDiff+=parseFloat(e.css(i))||0})),$.each(i,(function(t,i){cellHeightDiff+=parseFloat(e.css(i))||0}))),a.remove(),absoluteColumnMinWidth=Math.max(headerColumnWidthDiff,cellWidthDiff)}function createCssRules(){$style=$("<style type='text/css' rel='stylesheet' />").appendTo($("head"));for(var e=options.rowHeight-cellHeightDiff,t=["."+uid+" .slick-group-header-column { left: 1000px; }","."+uid+" .slick-header-column { left: 1000px; }","."+uid+" .slick-top-panel { height:"+options.topPanelHeight+"px; }","."+uid+" .slick-preheader-panel { height:"+options.preHeaderPanelHeight+"px; }","."+uid+" .slick-headerrow-columns { height:"+options.headerRowHeight+"px; }","."+uid+" .slick-footerrow-columns { height:"+options.footerRowHeight+"px; }","."+uid+" .slick-cell { height:"+e+"px; }","."+uid+" .slick-row { height:"+options.rowHeight+"px; }"],i=0;i<columns.length;i++)t.push("."+uid+" .l"+i+" { }"),t.push("."+uid+" .r"+i+" { }");$style[0].styleSheet?$style[0].styleSheet.cssText=t.join(" "):$style[0].appendChild(document.createTextNode(t.join(" ")))}function getColumnCssRules(e){var t;if(!stylesheet){var i=document.styleSheets;for(t=0;t<i.length;t++)if((i[t].ownerNode||i[t].owningElement)==$style[0]){stylesheet=i[t];break}if(!stylesheet)throw new Error("Cannot find stylesheet.");columnCssRulesL=[],columnCssRulesR=[];var n,a,o=stylesheet.cssRules||stylesheet.rules;for(t=0;t<o.length;t++){var s=o[t].selectorText;(n=/\.l\d+/.exec(s))?(a=parseInt(n[0].substr(2,n[0].length-2),10),columnCssRulesL[a]=o[t]):(n=/\.r\d+/.exec(s))&&(a=parseInt(n[0].substr(2,n[0].length-2),10),columnCssRulesR[a]=o[t])}}return{left:columnCssRulesL[e],right:columnCssRulesR[e]}}function removeCssRules(){$style.remove(),stylesheet=null}function destroy(){getEditorLock().cancelCurrentEdit(),trigger(self.onBeforeDestroy,{});var e=plugins.length;while(e--)unregisterPlugin(plugins[e]);options.enableColumnReorder&&$headers.filter(":ui-sortable").sortable("destroy"),unbindAncestorScrollEvents(),$container.off(".slickgrid"),removeCssRules(),$canvas.off("draginit dragstart dragend drag"),$container.empty().removeClass(uid)}var canvas=null,canvas_context=null;function autosizeColumn(e,t){var n=e;if("number"===typeof e)n=columns[e];else if("string"===typeof e)for(i=0;i<columns.length;i++)columns[i].Id===e&&(n=columns[i]);var a=$(getCanvasNode(0,0));getColAutosizeWidth(n,a,t)}function autosizeColumns(e,t){if(e=e||options.autosizeColsMode,e!==Slick.GridAutosizeColsMode.LegacyForceFit&&e!==Slick.GridAutosizeColsMode.LegacyOff){if(e!==Slick.GridAutosizeColsMode.None){canvas=document.createElement("canvas"),canvas&&canvas.getContext&&(canvas_context=canvas.getContext("2d"));var i,n,a,o,s=$(getCanvasNode(0,0)),r=viewportHasVScroll?viewportW-scrollbarDimensions.width:viewportW,l=0,c=0,h=0,d=0,u=0;for(i=0;i<columns.length;i++)n=columns[i],getColAutosizeWidth(n,s,t),u+=n.autoSize.autosizeMode===Slick.ColAutosizeMode.Locked?n.width:0,d+=n.autoSize.autosizeMode===Slick.ColAutosizeMode.Locked?n.width:n.minWidth,l+=n.autoSize.widthPx,c+=n.autoSize.sizeToRemaining?0:n.autoSize.widthPx,h+=n.autoSize.sizeToRemaining&&n.minWidth||0;var p=l-c;if(e===Slick.GridAutosizeColsMode.FitViewportToCols){var f=l+scrollbarDimensions.width;e=Slick.GridAutosizeColsMode.IgnoreViewport,options.viewportMaxWidthPx&&f>options.viewportMaxWidthPx?(f=options.viewportMaxWidthPx,e=Slick.GridAutosizeColsMode.FitColsToViewport):options.viewportMinWidthPx&&f<options.viewportMinWidthPx&&(f=options.viewportMinWidthPx,e=Slick.GridAutosizeColsMode.FitColsToViewport),$container.width(f)}if(e===Slick.GridAutosizeColsMode.FitColsToViewport)if(p>0&&c<r-h)for(i=0;i<columns.length;i++){n=columns[i];var g=r-c;a=n.autoSize.sizeToRemaining?g*n.autoSize.widthPx/p:n.autoSize.widthPx,n.rerenderOnResize&&n.width!=a&&(o=!0),n.width=a}else if(options.viewportSwitchToScrollModeWidthPercent&&c+h>r*options.viewportSwitchToScrollModeWidthPercent/100||d>r)e=Slick.GridAutosizeColsMode.IgnoreViewport;else{var m=c-u,v=r-u-h;for(i=0;i<columns.length;i++)n=columns[i],a=n.width,n.autoSize.autosizeMode!==Slick.ColAutosizeMode.Locked&&(n.autoSize.sizeToRemaining?a=n.minWidth:(a=v/m*n.autoSize.widthPx,a<n.minWidth&&(a=n.minWidth),m-=n.autoSize.widthPx,v-=a)),n.rerenderOnResize&&n.width!=a&&(o=!0),n.width=a}if(e===Slick.GridAutosizeColsMode.IgnoreViewport)for(i=0;i<columns.length;i++)a=columns[i].autoSize.widthPx,columns[i].rerenderOnResize&&columns[i].width!=a&&(o=!0),columns[i].width=a;reRenderColumns(o)}}else legacyAutosizeColumns()}function LogColWidths(){var e="Col Widths:";for(i=0;i<columns.length;i++)e+=" "+columns[i].width;console.log(e)}function getColAutosizeWidth(e,t,i){var n=e.autoSize;if(n.widthPx=e.width,n.autosizeMode!==Slick.ColAutosizeMode.Locked&&n.autosizeMode!==Slick.ColAutosizeMode.Guide){var a=getDataLength();if(n.autosizeMode===Slick.ColAutosizeMode.ContentIntelligent){var o,s=n.colDataTypeOf;if(a>0){var r=getDataItem(0);r&&(o=r[e.field],s=typeof o,"object"===s&&(o instanceof Date&&(s="date"),"undefined"!==typeof moment&&o instanceof moment&&(s="moment")))}"boolean"===s&&(n.colValueArray=[!0,!1]),"number"===s&&(n.valueFilterMode=Slick.ValueFilterMode.GetGreatestAndSub,n.rowSelectionMode=Slick.RowSelectionMode.AllRows),"string"===s&&(n.valueFilterMode=Slick.ValueFilterMode.GetLongestText,n.rowSelectionMode=Slick.RowSelectionMode.AllRows,n.allowAddlPercent=5),"date"===s&&(n.colValueArray=[new Date(2009,8,30,12,20,20)]),"moment"===s&&"undefined"!==typeof moment&&(n.colValueArray=[moment([2009,8,30,12,20,20])])}var l=getColContentSize(e,t,i),c=n.allowAddlPercent?1+n.allowAddlPercent/100:1;l=l*c+options.autosizeColPaddingPx,e.minWidth&&l<e.minWidth&&(l=e.minWidth),e.maxWidth&&l>e.maxWidth&&(l=e.maxWidth),n.widthPx=l}}function getColContentSize(e,t,i){var n,a=e.autoSize,o=1,s=0,r=0;if(a.ignoreHeaderText||(r=getColHeaderWidth(e)),a.colValueArray)return s=getColWidth(e,t,a.colValueArray),Math.max(r,s);var l=getData();l.getItems&&(l=l.getItems());var c=(i?a.rowSelectionModeOnInit:void 0)||a.rowSelectionMode;if(c===Slick.RowSelectionMode.FirstRow&&(l=l.slice(0,1)),c===Slick.RowSelectionMode.LastRow&&(l=l.slice(l.length-1,l.length)),c===Slick.RowSelectionMode.FirstNRows&&(l=l.slice(0,a.rowSelectionCount)),a.valueFilterMode===Slick.ValueFilterMode.DeDuplicate){var h={};for(d=0,n=l.length;d<n;d++)h[l[d][e.field]]=!0;if(Object.keys)l=Object.keys(h);else for(var d in l=[],h)l.push(d)}if(a.valueFilterMode===Slick.ValueFilterMode.GetGreatestAndSub){var u,p=0;for(d=0,n=l.length;d<n;d++)g=l[d][e.field],Math.abs(g)>p&&(u=g,p=Math.abs(g));u=""+u,u=Array(u.length+1).join("9"),u=+u,l=[u]}if(a.valueFilterMode===Slick.ValueFilterMode.GetLongestTextAndSub){var f=0;for(d=0,n=l.length;d<n;d++)g=l[d][e.field],(g||"").length>f&&(f=g.length);g=Array(f+1).join("m"),o=options.autosizeTextAvgToMWidthRatio,l=[g]}if(a.valueFilterMode===Slick.ValueFilterMode.GetLongestText){f=0;var g,m=0;for(d=0,n=l.length;d<n;d++)g=l[d][e.field],(g||"").length>f&&(f=g.length,m=d);g=l[m][e.field],l=[g]}return s=getColWidth(e,t,l)*o,Math.max(r,s)}function getColWidth(e,t,i){var n=getColumnIndex(e.id),a=$('<div class="slick-row ui-widget-content"></div>'),o=$('<div class="slick-cell"></div>');o.css({position:"absolute",visibility:"hidden","text-overflow":"initial","white-space":"nowrap"}),a.append(o),t.append(a);var s,r,l,c,h,d=0;return canvas_context&&e.autoSize.widthEvalMode===Slick.WidthEvalMode.CanvasTextSize?(canvas_context.font=o.css("font-size")+" "+o.css("font-family"),$(i).each((function(t,i){h=Array.isArray(i)?i[e.field]:i,r=""+h,s=r?canvas_context.measureText(r).width:0,s>d&&(d=s,l=r)})),o.html(l),s=o.outerWidth(),a.remove(),s):($(i).each((function(t,i){h=Array.isArray(i)?i[e.field]:i,c=e.formatterOverride?e.formatterOverride(t,n,h,e,i):e.formatter?e.formatter(t,n,h,e,i):""+h,applyFormatResultToCellNode(c,o[0]),s=o.outerWidth(),s>d&&(d=s)})),a.remove(),d)}function getColHeaderWidth(e){var t=0,i=getUID()+e.id,n=document.getElementById(i),a=i+"_";if(n){var o=n.cloneNode(!0);o.id=a,o.style.cssText="position: absolute; visibility: hidden;right: auto;text-overflow: initial;white-space: nowrap;",n.parentNode.insertBefore(o,n),t=o.offsetWidth,o.parentNode.removeChild(o)}else{var s=getHeader(e);n=$("<div class='ui-state-default slick-header-column' />").html("<span class='slick-column-name'>"+e.name+"</span>").attr("id",a).css({position:"absolute",visibility:"hidden",right:"auto","text-overflow:":"initial","white-space":"nowrap"}).addClass(e.headerCssClass||"").appendTo(s),t=n[0].offsetWidth,s[0].removeChild(n[0])}return t}function legacyAutosizeColumns(){var e,t,i,n=[],a=0,o=0,s=viewportHasVScroll?viewportW-scrollbarDimensions.width:viewportW;for(e=0;e<columns.length;e++)t=columns[e],n.push(t.width),o+=t.width,t.resizable&&(a+=t.width-Math.max(t.minWidth,absoluteColumnMinWidth));i=o;while(o>s&&a){var r=(o-s)/a;for(e=0;e<columns.length&&o>s;e++){t=columns[e];var l=n[e];if(!(!t.resizable||l<=t.minWidth||l<=absoluteColumnMinWidth)){var c=Math.max(t.minWidth,absoluteColumnMinWidth),h=Math.floor(r*(l-c))||1;h=Math.min(h,l-c),o-=h,a-=h,n[e]-=h}}if(i<=o)break;i=o}i=o;while(o<s){var d=s/o;for(e=0;e<columns.length&&o<s;e++){t=columns[e];var u,p=n[e];u=!t.resizable||t.maxWidth<=p?0:Math.min(Math.floor(d*p)-p,t.maxWidth-p||1e6)||1,o+=u,n[e]+=o<=s?u:0}if(i>=o)break;i=o}var f=!1;for(e=0;e<columns.length;e++)columns[e].rerenderOnResize&&columns[e].width!=n[e]&&(f=!0),columns[e].width=n[e];reRenderColumns(f)}function reRenderColumns(e){applyColumnHeaderWidths(),applyColumnGroupHeaderWidths(),updateCanvasWidth(!0),trigger(self.onAutosizeColumns,{columns:columns}),e&&(invalidateAllRows(),render())}function trigger(e,t,i){return i=i||new Slick.EventData,t=t||{},t.grid=self,e.notify(t,i,self)}function getEditorLock(){return options.editorLock}function getEditController(){return editController}function getColumnIndex(e){return columnsById[e]}function applyColumnGroupHeaderWidths(){if(treeColumns.hasDepth())for(var e=$groupHeadersL.length-1;e>=0;e--){treeColumns.getColumnsInDepth(e);$().add($groupHeadersL[e]).add($groupHeadersR[e]).each((function(e){var t=$(this),i=0;t.width(0===e?getHeadersWidthL():getHeadersWidthR()),t.children().each((function(){var e=$(this),n=$(this).data("column");n.width=0,n.columns.forEach((function(){var e=t.next().children(":eq("+i+++")");n.width+=e.outerWidth()})),e.width(n.width-headerColumnWidthDiff)}))}))}}function applyColumnHeaderWidths(){if(initialized){for(var e,t=0,i=$headers.children(),n=columns.length;t<n;t++)e=$(i[t]),jQueryNewWidthBehaviour?e.outerWidth()!==columns[t].width&&e.outerWidth(columns[t].width):e.width()!==columns[t].width-headerColumnWidthDiff&&e.width(columns[t].width-headerColumnWidthDiff);updateColumnCaches()}}function applyColumnWidths(){for(var e,t,i=0,n=0;n<columns.length;n++)e=columns[n].width,t=getColumnCssRules(n),t.left.style.left=i+"px",t.right.style.right=(-1!=options.frozenColumn&&n>options.frozenColumn?canvasWidthR:canvasWidthL)-i-e+"px",options.frozenColumn==n?i=0:i+=columns[n].width}function setSortColumn(e,t){setSortColumns([{columnId:e,sortAsc:t}])}function setSortColumns(e){sortColumns=e;var t=options.numberedMultiColumnSort&&sortColumns.length>1,i=$headers.children();i.removeClass("slick-header-column-sorted").find(".slick-sort-indicator").removeClass("slick-sort-indicator-asc slick-sort-indicator-desc"),i.find(".slick-sort-indicator-numbered").text(""),$.each(sortColumns,(function(e,n){null==n.sortAsc&&(n.sortAsc=!0);var a=getColumnIndex(n.columnId);null!=a&&(i.eq(a).addClass("slick-header-column-sorted").find(".slick-sort-indicator").addClass(n.sortAsc?"slick-sort-indicator-asc":"slick-sort-indicator-desc"),t&&i.eq(a).find(".slick-sort-indicator-numbered").text(e+1))}))}function getSortColumns(){return sortColumns}function handleSelectedRangesChanged(e,t){var i=selectedRows.slice(0);selectedRows=[];for(var n={},a=0;a<t.length;a++)for(var o=t[a].fromRow;o<=t[a].toRow;o++){n[o]||(selectedRows.push(o),n[o]={});for(var s=t[a].fromCell;s<=t[a].toCell;s++)canCellBeSelected(o,s)&&(n[o][columns[s].id]=options.selectedCellCssClass)}setCellCssStyles(options.selectedCellCssClass,n),simpleArrayEquals(i,selectedRows)&&trigger(self.onSelectedRowsChanged,{rows:getSelectedRows(),previousSelectedRows:i},e)}function simpleArrayEquals(e,t){return Array.isArray(e)&&Array.isArray(t)&&t.sort().toString()!==e.sort().toString()}function getColumns(){return columns}function updateColumnCaches(){columnPosLeft=[],columnPosRight=[];for(var e=0,t=0,i=columns.length;t<i;t++)columnPosLeft[t]=e,columnPosRight[t]=e+columns[t].width,options.frozenColumn==t?e=0:e+=columns[t].width}function updateColumnProps(){columnsById={};for(var e=0;e<columns.length;e++){columns[e].width&&(columns[e].widthRequest=columns[e].width);var t=columns[e]=$.extend({},columnDefaults,columns[e]);t.autoSize=$.extend({},columnAutosizeDefaults,t.autoSize),columnsById[t.id]=e,t.minWidth&&t.width<t.minWidth&&(t.width=t.minWidth),t.maxWidth&&t.width>t.maxWidth&&(t.width=t.maxWidth),t.resizable}}function setColumns(e){var t=new Slick.TreeColumns(e);t.hasDepth()?(treeColumns=t,columns=treeColumns.extractColumns()):columns=e,updateColumnProps(),updateColumnCaches(),initialized&&(setPaneVisibility(),setOverflow(),invalidateAllRows(),createColumnHeaders(),createColumnGroupHeaders(),createColumnFooter(),removeCssRules(),createCssRules(),resizeCanvas(),updateCanvasWidth(),applyColumnHeaderWidths(),applyColumnWidths(),handleScroll())}function getOptions(){return options}function setOptions(e,t){getEditorLock().commitCurrentEdit()&&(makeActiveCellNormal(),void 0!==e.showColumnHeader&&setColumnHeaderVisibility(e.showColumnHeader),options.enableAddRow!==e.enableAddRow&&invalidateRow(getDataLength()),options=$.extend(options,e),validateAndEnforceOptions(),$viewport.css("overflow-y",options.autoHeight?"hidden":"auto"),t||render(),setFrozenOptions(),setScroller(),zombieRowNodeFromLastMouseWheelEvent=null,setColumns(treeColumns.extractColumns()))}function validateAndEnforceOptions(){options.autoHeight&&(options.leaveSpaceForNewRows=!1),options.forceFitColumns&&(options.autosizeColsMode=Slick.GridAutosizeColsMode.LegacyForceFit,console.log("forceFitColumns option is deprecated - use autosizeColsMode"))}function setData(e,t){data=e,invalidateAllRows(),updateRowCount(),t&&scrollTo(0)}function getData(){return data}function getDataLength(){return data.getLength?data.getLength():data&&data.length||0}function getDataLengthIncludingAddNew(){return getDataLength()+(options.enableAddRow&&(!pagingActive||pagingIsLastPage)?1:0)}function getDataItem(e){return data.getItem?data.getItem(e):data[e]}function getTopPanel(){return $topPanel[0]}function setTopPanelVisibility(e,t){var i=!1!==t;options.showTopPanel!=e&&(options.showTopPanel=e,e?i?$topPanelScroller.slideDown("fast",resizeCanvas):($topPanelScroller.show(),resizeCanvas()):i?$topPanelScroller.slideUp("fast",resizeCanvas):($topPanelScroller.hide(),resizeCanvas()))}function setHeaderRowVisibility(e,t){var i=!1!==t;options.showHeaderRow!=e&&(options.showHeaderRow=e,e?i?$headerRowScroller.slideDown("fast",resizeCanvas):($headerRowScroller.show(),resizeCanvas()):i?$headerRowScroller.slideUp("fast",resizeCanvas):($headerRowScroller.hide(),resizeCanvas()))}function setColumnHeaderVisibility(e,t){options.showColumnHeader!=e&&(options.showColumnHeader=e,e?t?$headerScroller.slideDown("fast",resizeCanvas):($headerScroller.show(),resizeCanvas()):t?$headerScroller.slideUp("fast",resizeCanvas):($headerScroller.hide(),resizeCanvas()))}function setFooterRowVisibility(e,t){var i=!1!==t;options.showFooterRow!=e&&(options.showFooterRow=e,e?i?$footerRowScroller.slideDown("fast",resizeCanvas):($footerRowScroller.show(),resizeCanvas()):i?$footerRowScroller.slideUp("fast",resizeCanvas):($footerRowScroller.hide(),resizeCanvas()))}function setPreHeaderPanelVisibility(e,t){var i=!1!==t;options.showPreHeaderPanel!=e&&(options.showPreHeaderPanel=e,e?i?$preHeaderPanelScroller.slideDown("fast",resizeCanvas):($preHeaderPanelScroller.show(),resizeCanvas()):i?$preHeaderPanelScroller.slideUp("fast",resizeCanvas):($preHeaderPanelScroller.hide(),resizeCanvas()))}function getContainerNode(){return $container.get(0)}function getRowTop(e){return options.rowHeight*e-offset}function getRowFromPosition(e){return Math.floor((e+offset)/options.rowHeight)}function scrollTo(e){e=Math.max(e,0),e=Math.min(e,th-$viewportScrollContainerY.height()+(viewportHasHScroll||hasFrozenColumns()?scrollbarDimensions.height:0));var t=offset;page=Math.min(n-1,Math.floor(e/ph)),offset=Math.round(page*cj);var i=e-offset;if(offset!=t){var a=getVisibleRange(i);cleanupRows(a),updateRowPositions()}prevScrollTop!=i&&(vScrollDir=prevScrollTop+t<i+offset?1:-1,lastRenderedScrollTop=scrollTop=prevScrollTop=i,hasFrozenColumns()&&($viewportTopL[0].scrollTop=i),hasFrozenRows&&($viewportBottomL[0].scrollTop=$viewportBottomR[0].scrollTop=i),$viewportScrollContainerY[0].scrollTop=i,trigger(self.onViewportChanged,{}))}function defaultFormatter(e,t,i,n,a,o){return null==i?"":(i+"").replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;")}function getFormatter(e,t){var i=data.getItemMetadata&&data.getItemMetadata(e),n=i&&i.columns&&(i.columns[t.id]||i.columns[getColumnIndex(t.id)]);return n&&n.formatter||i&&i.formatter||t.formatter||options.formatterFactory&&options.formatterFactory.getFormatter(t)||options.defaultFormatter}function callFormatter(e,t,i,n,a,o){var s,r=data.getItemMetadata&&data.getItemMetadata(e);if(r=r&&r.columns,r){var l=r[n.id]||r[t];s=getFormatter(e,n)(e,t,i,n,a,l)}else s=getFormatter(e,n)(e,t,i,n,a);return s}function getEditor(e,t){var i=columns[t],n=data.getItemMetadata&&data.getItemMetadata(e),a=n&&n.columns;return a&&a[i.id]&&void 0!==a[i.id].editor?a[i.id].editor:a&&a[t]&&void 0!==a[t].editor?a[t].editor:i.editor||options.editorFactory&&options.editorFactory.getEditor(i)}function getDataItemValueForColumn(e,t){return options.dataItemColumnValueExtractor?options.dataItemColumnValueExtractor(e,t):e[t.field]}function appendRowHtml(e,t,i,n,a){var o=getDataItem(i),s=i<a&&!o,r="slick-row"+(hasFrozenRows&&i<=options.frozenRow?" frozen":"")+(s?" loading":"")+(i===activeRow&&options.showCellSelection?" active":"")+(i%2==1?" odd":" even");o||(r+=" "+options.addNewRowCssClass);var l=data.getItemMetadata&&data.getItemMetadata(i);l&&l.cssClasses&&(r+=" "+l.cssClasses);var c,h,d=getFrozenRowOffset(i),u="<div class='ui-widget-content "+r+"' style='top:"+(getRowTop(i)-d)+"px'>";e.push(u),hasFrozenColumns()&&t.push(u);for(var p=0,f=columns.length;p<f;p++){if(h=columns[p],c=1,l&&l.columns){var g=l.columns[h.id]||l.columns[p];c=g&&g.colspan||1,"*"===c&&(c=f-p)}if(columnPosRight[Math.min(f-1,p+c-1)]>n.leftPx){if(!h.alwaysRenderColumn&&columnPosLeft[p]>n.rightPx)break;hasFrozenColumns()&&p>options.frozenColumn?appendCellHtml(t,i,p,c,o):appendCellHtml(e,i,p,c,o)}else(h.alwaysRenderColumn||hasFrozenColumns()&&p<=options.frozenColumn)&&appendCellHtml(e,i,p,c,o);c>1&&(p+=c-1)}e.push("</div>"),hasFrozenColumns()&&t.push("</div>")}function appendCellHtml(e,t,i,n,a){var o=columns[i],s="slick-cell l"+i+" r"+Math.min(columns.length-1,i+n-1)+(o.cssClass?" "+o.cssClass:"");for(var r in hasFrozenColumns()&&i<=options.frozenColumn&&(s+=" frozen"),t===activeRow&&i===activeCell&&options.showCellSelection&&(s+=" active"),cellCssClasses)cellCssClasses[r][t]&&cellCssClasses[r][t][o.id]&&(s+=" "+cellCssClasses[r][t][o.id]);var l=null,c="";a&&(l=getDataItemValueForColumn(a,o),c=getFormatter(t,o)(t,i,l,o,a,self),null!==c&&void 0!==c||(c=""));var h=trigger(self.onBeforeAppendCell,{row:t,cell:i,value:l,dataContext:a})||"";h+=c&&c.addClasses?(h?" ":"")+c.addClasses:"";var d=c&&c.toolTip?"title='"+c.toolTip+"'":"",u="";if(o.hasOwnProperty("cellAttrs")&&o.cellAttrs instanceof Object)for(var r in o.cellAttrs)o.cellAttrs.hasOwnProperty(r)&&(u+=" "+r+'="'+o.cellAttrs[r]+'" ');e.push("<div class='"+s+(h?" "+h:"")+"' "+d+u+">"),a&&e.push("[object Object]"!==Object.prototype.toString.call(c)?c:c.text),e.push("</div>"),rowsCache[t].cellRenderQueue.push(i),rowsCache[t].cellColSpans[i]=n}function cleanupRows(e){for(var t in rowsCache){var i=!0;hasFrozenRows&&(options.frozenBottom&&t>=actualFrozenRow||!options.frozenBottom&&t<=actualFrozenRow)&&(i=!1),(t=parseInt(t,10))!==activeRow&&(t<e.top||t>e.bottom)&&i&&removeRowFromCache(t)}options.enableAsyncPostRenderCleanup&&startPostProcessingCleanup()}function invalidate(){updateRowCount(),invalidateAllRows(),render()}function invalidateAllRows(){for(var e in currentEditor&&makeActiveCellNormal(),rowsCache)removeRowFromCache(e);options.enableAsyncPostRenderCleanup&&startPostProcessingCleanup()}function queuePostProcessedRowForCleanup(e,t,i){for(var n in postProcessgroupId++,t)t.hasOwnProperty(n)&&postProcessedCleanupQueue.push({actionType:"C",groupId:postProcessgroupId,node:e.cellNodesByColumnIdx[0|n],columnIdx:0|n,rowIdx:i});postProcessedCleanupQueue.push({actionType:"R",groupId:postProcessgroupId,node:e.rowNode}),$(e.rowNode).detach()}function queuePostProcessedCellForCleanup(e,t,i){postProcessedCleanupQueue.push({actionType:"C",groupId:postProcessgroupId,node:e,columnIdx:t,rowIdx:i}),$(e).detach()}function removeRowFromCache(e){var t=rowsCache[e];t&&(rowNodeFromLastMouseWheelEvent==t.rowNode[0]||hasFrozenColumns()&&rowNodeFromLastMouseWheelEvent==t.rowNode[1]?(t.rowNode.hide(),zombieRowNodeFromLastMouseWheelEvent=t.rowNode):t.rowNode.each((function(){this.parentElement.removeChild(this)})),delete rowsCache[e],delete postProcessedRows[e],renderedRows--,counter_rows_removed++)}function invalidateRows(e){var t,i;if(e&&e.length){for(vScrollDir=0,i=e.length,t=0;t<i;t++)currentEditor&&activeRow===e[t]&&makeActiveCellNormal(),rowsCache[e[t]]&&removeRowFromCache(e[t]);options.enableAsyncPostRenderCleanup&&startPostProcessingCleanup()}}function invalidateRow(e){(e||0===e)&&invalidateRows([e])}function applyFormatResultToCellNode(e,t,i){null!==e&&void 0!==e||(e=""),"[object Object]"===Object.prototype.toString.call(e)?(t.innerHTML=e.text,e.removeClasses&&!i&&$(t).removeClass(e.removeClasses),e.addClasses&&$(t).addClass(e.addClasses),e.toolTip&&$(t).attr("title",e.toolTip)):t.innerHTML=e}function updateCell(e,t){var i=getCellNode(e,t);if(i){var n=columns[t],a=getDataItem(e);if(currentEditor&&activeRow===e&&activeCell===t)currentEditor.loadValue(a);else{var o=a?getFormatter(e,n)(e,t,getDataItemValueForColumn(a,n),n,a,self):"";applyFormatResultToCellNode(o,i),invalidatePostProcessingResults(e)}}}function updateRow(e){var t=rowsCache[e];if(t){ensureCellNodesInRowsCache(e);var i,n=getDataItem(e);for(var a in t.cellNodesByColumnIdx)if(t.cellNodesByColumnIdx.hasOwnProperty(a)){a|=0;var o=columns[a],s=t.cellNodesByColumnIdx[a][0];e===activeRow&&a===activeCell&&currentEditor?currentEditor.loadValue(n):n?(i=getFormatter(e,o)(e,a,getDataItemValueForColumn(n,o),o,n,self),applyFormatResultToCellNode(i,s)):s.innerHTML=""}invalidatePostProcessingResults(e)}}function getViewportHeight(){if(options.autoHeight){var e=$paneHeaderL.outerHeight();e+=options.showHeaderRow?options.headerRowHeight+getVBoxDelta($headerRowScroller):0,e+=options.showFooterRow?options.footerRowHeight+getVBoxDelta($footerRowScroller):0,e+=getCanvasWidth()>viewportW?scrollbarDimensions.height:0,viewportH=options.rowHeight*getDataLengthIncludingAddNew()+(-1==options.frozenColumn?e:0)}else{var t=options.showColumnHeader?parseFloat($.css($headerScroller[0],"height"))+getVBoxDelta($headerScroller):0;topPanelH=options.showTopPanel?options.topPanelHeight+getVBoxDelta($topPanelScroller):0,headerRowH=options.showHeaderRow?options.headerRowHeight+getVBoxDelta($headerRowScroller):0,footerRowH=options.showFooterRow?options.footerRowHeight+getVBoxDelta($footerRowScroller):0;var i=options.createPreHeaderPanel&&options.showPreHeaderPanel?options.preHeaderPanelHeight+getVBoxDelta($preHeaderPanelScroller):0;viewportH=parseFloat($.css($container[0],"height",!0))-parseFloat($.css($container[0],"paddingTop",!0))-parseFloat($.css($container[0],"paddingBottom",!0))-t-topPanelH-headerRowH-footerRowH-i}return numVisibleRows=Math.ceil(viewportH/options.rowHeight),viewportH}function getViewportWidth(){viewportW=parseFloat($container.width())}function resizeCanvas(){if(initialized){paneTopH=0,paneBottomH=0,viewportTopH=0,viewportBottomH=0,getViewportWidth(),getViewportHeight(),hasFrozenRows?options.frozenBottom?(paneTopH=viewportH-frozenRowsHeight-scrollbarDimensions.height,paneBottomH=frozenRowsHeight+scrollbarDimensions.height):(paneTopH=frozenRowsHeight,paneBottomH=viewportH-frozenRowsHeight):paneTopH=viewportH,paneTopH+=topPanelH+headerRowH+footerRowH,hasFrozenColumns()&&options.autoHeight&&(paneTopH+=scrollbarDimensions.height),viewportTopH=paneTopH-topPanelH-headerRowH-footerRowH,options.autoHeight&&(hasFrozenColumns()&&$container.height(paneTopH+parseFloat($.css($headerScrollerL[0],"height"))),$paneTopL.css("position","relative")),$paneTopL.css({top:$paneHeaderL.height(),height:paneTopH});var e=$paneTopL.position().top+paneTopH;options.autoHeight||$viewportTopL.height(viewportTopH),hasFrozenColumns()?($paneTopR.css({top:$paneHeaderL.height(),height:paneTopH}),$viewportTopR.height(viewportTopH),hasFrozenRows&&($paneBottomL.css({top:e,height:paneBottomH}),$paneBottomR.css({top:e,height:paneBottomH}),$viewportBottomR.height(paneBottomH))):hasFrozenRows&&($paneBottomL.css({width:"100%",height:paneBottomH}),$paneBottomL.css("top",e)),hasFrozenRows?($viewportBottomL.height(paneBottomH),options.frozenBottom?($canvasBottomL.height(frozenRowsHeight),hasFrozenColumns()&&$canvasBottomR.height(frozenRowsHeight)):($canvasTopL.height(frozenRowsHeight),hasFrozenColumns()&&$canvasTopR.height(frozenRowsHeight))):$viewportTopR.height(viewportTopH),scrollbarDimensions&&scrollbarDimensions.width||(scrollbarDimensions=measureScrollbar()),options.autosizeColsMode===Slick.GridAutosizeColsMode.LegacyForceFit&&autosizeColumns(),updateRowCount(),handleScroll(),lastRenderedScrollLeft=-1,render()}}function updatePagingStatusFromView(e){pagingActive=0!==e.pageSize,pagingIsLastPage=e.pageNum==e.totalPages-1}function updateRowCount(){if(initialized){var e=getDataLength(),t=getDataLengthIncludingAddNew(),i=0,a=hasFrozenRows&&!options.frozenBottom?$canvasBottomL.height():$canvasTopL.height();if(hasFrozenRows)i=getDataLength()-options.frozenRow;else i=t+(options.leaveSpaceForNewRows?numVisibleRows-1:0);var o=$viewportScrollContainerY.height(),s=viewportHasVScroll;viewportHasVScroll=options.alwaysShowVerticalScroll||!options.autoHeight&&i*options.rowHeight>o,makeActiveCellNormal();var r=e-1;for(var l in rowsCache)l>r&&removeRowFromCache(l);options.enableAsyncPostRenderCleanup&&startPostProcessingCleanup(),activeCellNode&&activeRow>r&&resetActiveCell();a=h;options.autoHeight?h=options.rowHeight*i:(th=Math.max(options.rowHeight*i,o-scrollbarDimensions.height),th<maxSupportedCssHeight?(h=ph=th,n=1,cj=0):(h=maxSupportedCssHeight,ph=h/100,n=Math.floor(th/ph),cj=(th-h)/(n-1))),h!==a&&(hasFrozenRows&&!options.frozenBottom?($canvasBottomL.css("height",h),hasFrozenColumns()&&$canvasBottomR.css("height",h)):($canvasTopL.css("height",h),$canvasTopR.css("height",h)),scrollTop=$viewportScrollContainerY[0].scrollTop);var c=scrollTop+offset<=th-o;0==th||0==scrollTop?page=offset=0:scrollTo(c?scrollTop+offset:th-o),h!=a&&options.autoHeight&&resizeCanvas(),options.autosizeColsMode===Slick.GridAutosizeColsMode.LegacyForceFit&&s!=viewportHasVScroll&&autosizeColumns(),updateCanvasWidth(!1)}}function getVisibleRange(e,t){return null==e&&(e=scrollTop),null==t&&(t=scrollLeft),{top:getRowFromPosition(e),bottom:getRowFromPosition(e+viewportH)+1,leftPx:t,rightPx:t+viewportW}}function getRenderedRange(e,t){var i=getVisibleRange(e,t),n=Math.round(viewportH/options.rowHeight),a=options.minRowBuffer;return-1==vScrollDir?(i.top-=n,i.bottom+=a):1==vScrollDir?(i.top-=a,i.bottom+=n):(i.top-=a,i.bottom+=a),i.top=Math.max(0,i.top),i.bottom=Math.min(getDataLengthIncludingAddNew()-1,i.bottom),i.leftPx-=viewportW,i.rightPx+=viewportW,i.leftPx=Math.max(0,i.leftPx),i.rightPx=Math.min(canvasWidth,i.rightPx),i}function ensureCellNodesInRowsCache(e){var t=rowsCache[e];if(t&&t.cellRenderQueue.length){var i=t.rowNode.children().last();while(t.cellRenderQueue.length){var n=t.cellRenderQueue.pop();t.cellNodesByColumnIdx[n]=i,i=i.prev(),0===i.length&&(i=$(t.rowNode[0]).children().last())}}}function cleanUpCells(e,t){if(!hasFrozenRows||!(options.frozenBottom&&t>actualFrozenRow||t<=actualFrozenRow)){var i,n=rowsCache[t],a=[];for(var o in n.cellNodesByColumnIdx)if(n.cellNodesByColumnIdx.hasOwnProperty(o)&&(o|=0,!(o<=options.frozenColumn)&&!(Array.isArray(columns)&&columns[o]&&columns[o].alwaysRenderColumn))){var s=n.cellColSpans[o];(columnPosLeft[o]>e.rightPx||columnPosRight[Math.min(columns.length-1,o+s-1)]<e.leftPx)&&(t==activeRow&&o==activeCell||a.push(o))}while(null!=(i=a.pop()))n.cellNodesByColumnIdx[i][0].parentElement.removeChild(n.cellNodesByColumnIdx[i][0]),delete n.cellColSpans[i],delete n.cellNodesByColumnIdx[i],postProcessedRows[t]&&delete postProcessedRows[t][i]}}function cleanUpAndRenderCells(e){for(var t,i,n,a=[],o=[],s=e.top,r=e.bottom;s<=r;s++)if(t=rowsCache[s],t){ensureCellNodesInRowsCache(s),cleanUpCells(e,s),i=0;var l=data.getItemMetadata&&data.getItemMetadata(s);l=l&&l.columns;for(var c=getDataItem(s),h=0,d=columns.length;h<d;h++){if(columnPosLeft[h]>e.rightPx)break;if(null==(n=t.cellColSpans[h])){if(n=1,l){var u=l[columns[h].id]||l[h];n=u&&u.colspan||1,"*"===n&&(n=d-h)}columnPosRight[Math.min(d-1,h+n-1)]>e.leftPx&&(appendCellHtml(a,s,h,n,c),i++),h+=n>1?n-1:0}else h+=n>1?n-1:0}i&&(i,o.push(s))}if(a.length){var p,f,g=document.createElement("div");g.innerHTML=a.join("");while(null!=(p=o.pop())){var m;t=rowsCache[p];while(null!=(m=t.cellRenderQueue.pop()))f=g.lastChild,hasFrozenColumns()&&m>options.frozenColumn?t.rowNode[1].appendChild(f):t.rowNode[0].appendChild(f),t.cellNodesByColumnIdx[m]=$(f)}}}function renderRows(e){for(var t=[],i=[],n=[],a=!1,o=getDataLength(),s=e.top,r=e.bottom;s<=r;s++)rowsCache[s]||hasFrozenRows&&options.frozenBottom&&s==getDataLength()||(renderedRows++,n.push(s),rowsCache[s]={rowNode:null,cellColSpans:[],cellNodesByColumnIdx:[],cellRenderQueue:[]},appendRowHtml(t,i,s,e,o),activeCellNode&&activeRow===s&&(a=!0),counter_rows_rendered++);if(n.length){var l=document.createElement("div"),c=document.createElement("div");l.innerHTML=t.join(""),c.innerHTML=i.join("");for(s=0,r=n.length;s<r;s++)hasFrozenRows&&n[s]>=actualFrozenRow?hasFrozenColumns()?rowsCache[n[s]].rowNode=$().add($(l.firstChild).appendTo($canvasBottomL)).add($(c.firstChild).appendTo($canvasBottomR)):rowsCache[n[s]].rowNode=$().add($(l.firstChild).appendTo($canvasBottomL)):hasFrozenColumns()?rowsCache[n[s]].rowNode=$().add($(l.firstChild).appendTo($canvasTopL)).add($(c.firstChild).appendTo($canvasTopR)):rowsCache[n[s]].rowNode=$().add($(l.firstChild).appendTo($canvasTopL));a&&(activeCellNode=getCellNode(activeRow,activeCell))}}function startPostProcessing(){options.enableAsyncPostRender&&(clearTimeout(h_postrender),h_postrender=setTimeout(asyncPostProcessRows,options.asyncPostRenderDelay))}function startPostProcessingCleanup(){options.enableAsyncPostRenderCleanup&&(clearTimeout(h_postrenderCleanup),h_postrenderCleanup=setTimeout(asyncPostProcessCleanupRows,options.asyncPostRenderCleanupDelay))}function invalidatePostProcessingResults(e){for(var t in postProcessedRows[e])postProcessedRows[e].hasOwnProperty(t)&&(postProcessedRows[e][t]="C");postProcessFromRow=Math.min(postProcessFromRow,e),postProcessToRow=Math.max(postProcessToRow,e),startPostProcessing()}function updateRowPositions(){for(var e in rowsCache){var t=e?parseInt(e):0;rowsCache[t].rowNode[0].style.top=getRowTop(t)+"px"}}function render(){if(initialized){scrollThrottle.dequeue();var e=getVisibleRange(),t=getRenderedRange();if(cleanupRows(t),lastRenderedScrollLeft!=scrollLeft){if(hasFrozenRows){var i=jQuery.extend(!0,{},t);options.frozenBottom?(i.top=actualFrozenRow,i.bottom=getDataLength()):(i.top=0,i.bottom=options.frozenRow),cleanUpAndRenderCells(i)}cleanUpAndRenderCells(t)}renderRows(t),hasFrozenRows&&(options.frozenBottom?renderRows({top:actualFrozenRow,bottom:getDataLength()-1,leftPx:t.leftPx,rightPx:t.rightPx}):renderRows({top:0,bottom:options.frozenRow-1,leftPx:t.leftPx,rightPx:t.rightPx})),postProcessFromRow=e.top,postProcessToRow=Math.min(getDataLengthIncludingAddNew()-1,e.bottom),startPostProcessing(),lastRenderedScrollTop=scrollTop,lastRenderedScrollLeft=scrollLeft,h_render=null,trigger(self.onRendered,{startRow:e.top,endRow:e.bottom,grid:self})}}function handleHeaderScroll(){handleElementScroll($headerScrollContainer[0])}function handleHeaderRowScroll(){var e=$headerRowScrollContainer[0].scrollLeft;e!=$viewportScrollContainerX[0].scrollLeft&&($viewportScrollContainerX[0].scrollLeft=e)}function handleFooterRowScroll(){var e=$footerRowScrollContainer[0].scrollLeft;e!=$viewportScrollContainerX[0].scrollLeft&&($viewportScrollContainerX[0].scrollLeft=e)}function handlePreHeaderPanelScroll(){handleElementScroll($preHeaderPanelScroller[0])}function handleElementScroll(e){var t=e.scrollLeft;t!=$viewportScrollContainerX[0].scrollLeft&&($viewportScrollContainerX[0].scrollLeft=t)}function handleScroll(){return scrollTop=$viewportScrollContainerY[0].scrollTop,scrollLeft=$viewportScrollContainerX[0].scrollLeft,_handleScroll(!1)}function _handleScroll(e){var t=$viewportScrollContainerY[0].scrollHeight-$viewportScrollContainerY[0].clientHeight,i=$viewportScrollContainerY[0].scrollWidth-$viewportScrollContainerY[0].clientWidth;t=Math.max(0,t),i=Math.max(0,i),scrollTop>t&&(scrollTop=t),scrollLeft>i&&(scrollLeft=i);var a=Math.abs(scrollTop-prevScrollTop),o=Math.abs(scrollLeft-prevScrollLeft);if(o&&(prevScrollLeft=scrollLeft,$viewportScrollContainerX[0].scrollLeft=scrollLeft,$headerScrollContainer[0].scrollLeft=scrollLeft,$topPanelScroller[0].scrollLeft=scrollLeft,$headerRowScrollContainer[0].scrollLeft=scrollLeft,options.createFooterRow&&($footerRowScrollContainer[0].scrollLeft=scrollLeft),options.createPreHeaderPanel&&(hasFrozenColumns()?$preHeaderPanelScrollerR[0].scrollLeft=scrollLeft:$preHeaderPanelScroller[0].scrollLeft=scrollLeft),hasFrozenColumns()?hasFrozenRows&&($viewportTopR[0].scrollLeft=scrollLeft):hasFrozenRows&&($viewportTopL[0].scrollLeft=scrollLeft)),a)if(vScrollDir=prevScrollTop<scrollTop?1:-1,prevScrollTop=scrollTop,e&&($viewportScrollContainerY[0].scrollTop=scrollTop),hasFrozenColumns()&&(hasFrozenRows&&!options.frozenBottom?$viewportBottomL[0].scrollTop=scrollTop:$viewportTopL[0].scrollTop=scrollTop),a<viewportH)scrollTo(scrollTop+offset);else{var s=offset;page=h==viewportH?0:Math.min(n-1,Math.floor(scrollTop*((th-viewportH)/(h-viewportH))*(1/ph))),offset=Math.round(page*cj),s!=offset&&invalidateAllRows()}if(o||a){var r=Math.abs(lastRenderedScrollLeft-scrollLeft),l=Math.abs(lastRenderedScrollTop-scrollTop);(r>20||l>20)&&(options.forceSyncScrolling||l<viewportH&&r<viewportW?render():scrollThrottle.enqueue(),trigger(self.onViewportChanged,{}))}return trigger(self.onScroll,{scrollLeft:scrollLeft,scrollTop:scrollTop}),!(!o&&!a)}function ActionThrottle(e,t){var i=!1,n=!1;function a(){i?n=!0:s()}function o(){n=!1}function s(){i=!0,setTimeout(r,t),e()}function r(){n?(o(),s()):i=!1}return{enqueue:a,dequeue:o}}function asyncPostProcessRows(){var e=getDataLength();while(postProcessFromRow<=postProcessToRow){var t=vScrollDir>=0?postProcessFromRow++:postProcessToRow--,i=rowsCache[t];if(i&&!(t>=e)){for(var n in postProcessedRows[t]||(postProcessedRows[t]={}),ensureCellNodesInRowsCache(t),i.cellNodesByColumnIdx)if(i.cellNodesByColumnIdx.hasOwnProperty(n)){n|=0;var a=columns[n],o=postProcessedRows[t][n];if(a.asyncPostRender&&"R"!==o){var s=i.cellNodesByColumnIdx[n];s&&a.asyncPostRender(s,t,getDataItem(t),a,"C"===o),postProcessedRows[t][n]="R"}}return void(h_postrender=setTimeout(asyncPostProcessRows,options.asyncPostRenderDelay))}}}function asyncPostProcessCleanupRows(){if(postProcessedCleanupQueue.length>0){var e=postProcessedCleanupQueue[0].groupId;while(postProcessedCleanupQueue.length>0&&postProcessedCleanupQueue[0].groupId==e){var t=postProcessedCleanupQueue.shift();if("R"==t.actionType&&$(t.node).remove(),"C"==t.actionType){var i=columns[t.columnIdx];i.asyncPostRenderCleanup&&t.node&&i.asyncPostRenderCleanup(t.node,t.rowIdx,i)}}h_postrenderCleanup=setTimeout(asyncPostProcessCleanupRows,options.asyncPostRenderCleanupDelay)}}function updateCellCssStylesOnRenderedRows(e,t){var i,n,a,o;for(var s in rowsCache){if(o=t&&t[s],a=e&&e[s],o)for(n in o)a&&o[n]==a[n]||(i=getCellNode(s,getColumnIndex(n)),i&&$(i).removeClass(o[n]));if(a)for(n in a)o&&o[n]==a[n]||(i=getCellNode(s,getColumnIndex(n)),i&&$(i).addClass(a[n]))}}function addCellCssStyles(e,t){if(cellCssClasses[e])throw new Error("addCellCssStyles: cell CSS hash with key '"+e+"' already exists.");cellCssClasses[e]=t,updateCellCssStylesOnRenderedRows(t,null),trigger(self.onCellCssStylesChanged,{key:e,hash:t,grid:self})}function removeCellCssStyles(e){cellCssClasses[e]&&(updateCellCssStylesOnRenderedRows(null,cellCssClasses[e]),delete cellCssClasses[e],trigger(self.onCellCssStylesChanged,{key:e,hash:null,grid:self}))}function setCellCssStyles(e,t){var i=cellCssClasses[e];cellCssClasses[e]=t,updateCellCssStylesOnRenderedRows(t,i),trigger(self.onCellCssStylesChanged,{key:e,hash:t,grid:self})}function getCellCssStyles(e){return cellCssClasses[e]}function flashCell(e,t,i){function n(e,t){t&&setTimeout((function(){e.queue((function(){e.toggleClass(options.cellFlashingCssClass).dequeue(),n(e,t-1)}))}),i)}if(i=i||100,rowsCache[e]){var a=$(getCellNode(e,t));n(a,4)}}function handleMouseWheel(e,t,i,n){var a=$(e.target).closest(".slick-row"),o=a[0];if(o!=rowNodeFromLastMouseWheelEvent){var s=a.parents(".grid-canvas"),r=s.hasClass("grid-canvas-left");if(zombieRowNodeFromLastMouseWheelEvent&&zombieRowNodeFromLastMouseWheelEvent[r?0:1]!=o){var l=zombieRowNodeFromLastMouseWheelEvent[r||1==zombieRowNodeFromLastMouseWheelEvent.length?0:1];l.parentElement.removeChild(l),zombieRowNodeFromLastMouseWheelEvent=null}rowNodeFromLastMouseWheelEvent=o}scrollTop=Math.max(0,$viewportScrollContainerY[0].scrollTop-n*options.rowHeight),scrollLeft=$viewportScrollContainerX[0].scrollLeft+10*i;var c=_handleScroll(!0);c&&e.preventDefault()}function handleDragInit(e,t){var i=getCellFromEvent(e);if(!i||!cellExists(i.row,i.cell))return!1;var n=trigger(self.onDragInit,t,e);return!!e.isImmediatePropagationStopped()&&n}function handleDragStart(e,t){var i=getCellFromEvent(e);if(!i||!cellExists(i.row,i.cell))return!1;var n=trigger(self.onDragStart,t,e);return!!e.isImmediatePropagationStopped()&&n}function handleDrag(e,t){return trigger(self.onDrag,t,e)}function handleDragEnd(e,t){trigger(self.onDragEnd,t,e)}function handleKeyDown(e){trigger(self.onKeyDown,{row:activeRow,cell:activeCell},e);var t=e.isImmediatePropagationStopped(),i=Slick.keyCode;if(!t&&!e.shiftKey&&!e.altKey){if(options.editable&&currentEditor&&currentEditor.keyCaptureList&&currentEditor.keyCaptureList.indexOf(e.which)>-1)return;e.which==i.HOME?t=e.ctrlKey?navigateTop():navigateRowStart():e.which==i.END&&(t=e.ctrlKey?navigateBottom():navigateRowEnd())}if(!t)if(e.shiftKey||e.altKey||e.ctrlKey)e.which!=i.TAB||!e.shiftKey||e.ctrlKey||e.altKey||(t=navigatePrev());else{if(options.editable&&currentEditor&&currentEditor.keyCaptureList&&currentEditor.keyCaptureList.indexOf(e.which)>-1)return;if(e.which==i.ESCAPE){if(!getEditorLock().isActive())return;cancelEditAndSetFocus()}else e.which==i.PAGE_DOWN?(navigatePageDown(),t=!0):e.which==i.PAGE_UP?(navigatePageUp(),t=!0):e.which==i.LEFT?t=navigateLeft():e.which==i.RIGHT?t=navigateRight():e.which==i.UP?t=navigateUp():e.which==i.DOWN?t=navigateDown():e.which==i.TAB?t=navigateNext():e.which==i.ENTER&&(options.editable&&(currentEditor?activeRow===getDataLength()?navigateDown():commitEditAndSetFocus():getEditorLock().commitCurrentEdit()&&makeActiveCellEditable(void 0,void 0,e)),t=!0)}if(t){e.stopPropagation(),e.preventDefault();try{e.originalEvent.keyCode=0}catch(n){}}}function handleClick(e){currentEditor||(e.target!=document.activeElement||$(e.target).hasClass("slick-cell"))&&setFocus();var t=getCellFromEvent(e);if(t&&(null===currentEditor||activeRow!=t.row||activeCell!=t.cell)&&(trigger(self.onClick,{row:t.row,cell:t.cell},e),!e.isImmediatePropagationStopped()&&canCellBeActive(t.row,t.cell)&&(!getEditorLock().isActive()||getEditorLock().commitCurrentEdit()))){scrollRowIntoView(t.row,!1);var i=e.target&&e.target.className===Slick.preClickClassName,n=columns[t.cell],a=!!(options.editable&&n&&n.editor&&options.suppressActiveCellChangeOnEdit);setActiveCellInternal(getCellNode(t.row,t.cell),null,i,a,e)}}function handleContextMenu(e){var t=$(e.target).closest(".slick-cell",$canvas);0!==t.length&&(activeCellNode===t[0]&&null!==currentEditor||trigger(self.onContextMenu,{},e))}function handleDblClick(e){var t=getCellFromEvent(e);!t||null!==currentEditor&&activeRow==t.row&&activeCell==t.cell||(trigger(self.onDblClick,{row:t.row,cell:t.cell},e),e.isImmediatePropagationStopped()||options.editable&&gotoCell(t.row,t.cell,!0,e))}function handleHeaderMouseEnter(e){trigger(self.onHeaderMouseEnter,{column:$(this).data("column"),grid:self},e)}function handleHeaderMouseLeave(e){trigger(self.onHeaderMouseLeave,{column:$(this).data("column"),grid:self},e)}function handleHeaderContextMenu(e){var t=$(e.target).closest(".slick-header-column",".slick-header-columns"),i=t&&t.data("column");trigger(self.onHeaderContextMenu,{column:i},e)}function handleHeaderClick(e){if(!columnResizeDragging){var t=$(e.target).closest(".slick-header-column",".slick-header-columns"),i=t&&t.data("column");i&&trigger(self.onHeaderClick,{column:i},e)}}function handleFooterContextMenu(e){var t=$(e.target).closest(".slick-footerrow-column",".slick-footerrow-columns"),i=t&&t.data("column");trigger(self.onFooterContextMenu,{column:i},e)}function handleFooterClick(e){var t=$(e.target).closest(".slick-footerrow-column",".slick-footerrow-columns"),i=t&&t.data("column");trigger(self.onFooterClick,{column:i},e)}function handleMouseEnter(e){trigger(self.onMouseEnter,{},e)}function handleMouseLeave(e){trigger(self.onMouseLeave,{},e)}function cellExists(e,t){return!(e<0||e>=getDataLength()||t<0||t>=columns.length)}function getCellFromPoint(e,t){for(var i=getRowFromPosition(t),n=0,a=0,o=0;o<columns.length&&a<e;o++)a+=columns[o].width,n++;return n<0&&(n=0),{row:i,cell:n-1}}function getCellFromNode(e){var t=/l\d+/.exec(e.className);if(!t)throw new Error("getCellFromNode: cannot get cell - "+e.className);return parseInt(t[0].substr(1,t[0].length-1),10)}function getRowFromNode(e){for(var t in rowsCache)for(var i in rowsCache[t].rowNode)if(rowsCache[t].rowNode[i]===e)return t?parseInt(t):0;return null}function getFrozenRowOffset(e){var t=hasFrozenRows?options.frozenBottom?e>=actualFrozenRow?h<viewportTopH?actualFrozenRow*options.rowHeight:h:0:e>=actualFrozenRow?frozenRowsHeight:0:0;return t}function getCellFromEvent(e){var t,i,n=$(e.target).closest(".slick-cell",$canvas);if(!n.length)return null;if(t=getRowFromNode(n[0].parentNode),hasFrozenRows){var a=n.parents(".grid-canvas").offset(),o=0,s=n.parents(".grid-canvas-bottom").length;s&&(o=options.frozenBottom?$canvasTopL.height():frozenRowsHeight),t=getCellFromPoint(e.clientX-a.left,e.clientY-a.top+o+$(document).scrollTop()).row}return i=getCellFromNode(n[0]),null==t||null==i?null:{row:t,cell:i}}function getCellNodeBox(e,t){if(!cellExists(e,t))return null;for(var i=getFrozenRowOffset(e),n=getRowTop(e)-i,a=n+options.rowHeight-1,o=0,s=0;s<t;s++)o+=columns[s].width,options.frozenColumn==s&&(o=0);var r=o+columns[t].width;return{top:n,left:o,bottom:a,right:r}}function resetActiveCell(){setActiveCellInternal(null,!1)}function setFocus(){-1==tabbingDirection?$focusSink[0].focus():$focusSink2[0].focus()}function scrollCellIntoView(e,t,i){if(scrollRowIntoView(e,i),!(t<=options.frozenColumn)){var n=getColspan(e,t);internalScrollColumnIntoView(columnPosLeft[t],columnPosRight[t+(n>1?n-1:0)])}}function internalScrollColumnIntoView(e,t){var i=scrollLeft+$viewportScrollContainerX.width();e<scrollLeft?($viewportScrollContainerX.scrollLeft(e),handleScroll(),render()):t>i&&($viewportScrollContainerX.scrollLeft(Math.min(e,t-$viewportScrollContainerX[0].clientWidth)),handleScroll(),render())}function scrollColumnIntoView(e){internalScrollColumnIntoView(columnPosLeft[e],columnPosRight[e])}function setActiveCellInternal(e,t,i,n,a){null!==activeCellNode&&(makeActiveCellNormal(),$(activeCellNode).removeClass("active"),rowsCache[activeRow]&&$(rowsCache[activeRow].rowNode).removeClass("active"));if(activeCellNode=e,null!=activeCellNode){var o=$(activeCellNode),s=o.offset(),r=Math.floor(o.parents(".grid-canvas").offset().top),l=o.parents(".grid-canvas-bottom").length;hasFrozenRows&&l&&(r-=options.frozenBottom?$canvasTopL.height():frozenRowsHeight);var c=getCellFromPoint(s.left,Math.ceil(s.top)-r);activeRow=c.row,activeCell=activePosX=activeCell=activePosX=getCellFromNode(activeCellNode),null==t&&(t=activeRow==getDataLength()||options.autoEdit),options.showCellSelection&&(o.addClass("active"),rowsCache[activeRow]&&$(rowsCache[activeRow].rowNode).addClass("active")),options.editable&&t&&isCellPotentiallyEditable(activeRow,activeCell)&&(clearTimeout(h_editorLoader),options.asyncEditorLoading?h_editorLoader=setTimeout((function(){makeActiveCellEditable(void 0,i,a)}),options.asyncEditorLoadDelay):makeActiveCellEditable(void 0,i,a))}else activeRow=activeCell=null;n||trigger(self.onActiveCellChanged,getActiveCell())}function clearTextSelection(){if(document.selection&&document.selection.empty)try{document.selection.empty()}catch(t){}else if(window.getSelection){var e=window.getSelection();e&&e.removeAllRanges&&e.removeAllRanges()}}function isCellPotentiallyEditable(e,t){var i=getDataLength();return!(e<i&&!getDataItem(e))&&(!(columns[t].cannotTriggerInsert&&e>=i)&&!!getEditor(e,t))}function makeActiveCellNormal(){if(currentEditor){if(trigger(self.onBeforeCellEditorDestroy,{editor:currentEditor}),currentEditor.destroy(),currentEditor=null,activeCellNode){var e=getDataItem(activeRow);if($(activeCellNode).removeClass("editable invalid"),e){var t=columns[activeCell],i=getFormatter(activeRow,t),n=i(activeRow,activeCell,getDataItemValueForColumn(e,t),t,e,self);applyFormatResultToCellNode(n,activeCellNode),invalidatePostProcessingResults(activeRow)}}navigator.userAgent.toLowerCase().match(/msie/)&&clearTextSelection(),getEditorLock().deactivate(editController)}}function makeActiveCellEditable(e,t,i){if(activeCellNode){if(!options.editable)throw new Error("Grid : makeActiveCellEditable : should never get called when options.editable is false");if(clearTimeout(h_editorLoader),isCellPotentiallyEditable(activeRow,activeCell)){var n=columns[activeCell],a=getDataItem(activeRow);if(!1!==trigger(self.onBeforeEditCell,{row:activeRow,cell:activeCell,item:a,column:n})){getEditorLock().activate(editController),$(activeCellNode).addClass("editable");var o=e||getEditor(activeRow,activeCell);e||o.suppressClearOnEdit||(activeCellNode.innerHTML="");var s=data.getItemMetadata&&data.getItemMetadata(activeRow);s=s&&s.columns;var r=s&&(s[n.id]||s[activeCell]);currentEditor=new o({grid:self,gridPosition:absBox($container[0]),position:absBox(activeCellNode),container:activeCellNode,column:n,columnMetaData:r,item:a||{},event:i,commitChanges:commitEditAndSetFocus,cancelChanges:cancelEditAndSetFocus}),a&&(currentEditor.loadValue(a),t&&currentEditor.preClick&&currentEditor.preClick()),serializedEditorValue=currentEditor.serializeValue(),currentEditor.position&&handleActiveCellPositionChange()}else setFocus()}}}function commitEditAndSetFocus(){getEditorLock().commitCurrentEdit()&&(setFocus(),options.autoEdit&&navigateDown())}function cancelEditAndSetFocus(){getEditorLock().cancelCurrentEdit()&&setFocus()}function absBox(e){var t={top:e.offsetTop,left:e.offsetLeft,bottom:0,right:0,width:$(e).outerWidth(),height:$(e).outerHeight(),visible:!0};t.bottom=t.top+t.height,t.right=t.left+t.width;var i=e.offsetParent;while((e=e.parentNode)!=document.body){if(null==e)break;t.visible&&e.scrollHeight!=e.offsetHeight&&"visible"!=$(e).css("overflowY")&&(t.visible=t.bottom>e.scrollTop&&t.top<e.scrollTop+e.clientHeight),t.visible&&e.scrollWidth!=e.offsetWidth&&"visible"!=$(e).css("overflowX")&&(t.visible=t.right>e.scrollLeft&&t.left<e.scrollLeft+e.clientWidth),t.left-=e.scrollLeft,t.top-=e.scrollTop,e===i&&(t.left+=e.offsetLeft,t.top+=e.offsetTop,i=e.offsetParent),t.bottom=t.top+t.height,t.right=t.left+t.width}return t}function getActiveCellPosition(){return absBox(activeCellNode)}function getGridPosition(){return absBox($container[0])}function handleActiveCellPositionChange(){if(activeCellNode&&(trigger(self.onActiveCellPositionChanged,{}),currentEditor)){var e=getActiveCellPosition();currentEditor.show&&currentEditor.hide&&(e.visible?currentEditor.show():currentEditor.hide()),currentEditor.position&&currentEditor.position(e)}}function getCellEditor(){return currentEditor}function getActiveCell(){return activeCellNode?{row:activeRow,cell:activeCell}:null}function getActiveCellNode(){return activeCellNode}function scrollRowIntoView(e,t){if(!hasFrozenRows||!options.frozenBottom&&e>actualFrozenRow-1||options.frozenBottom&&e<actualFrozenRow-1){var i=$viewportScrollContainerY.height(),n=hasFrozenRows&&!options.frozenBottom?e-options.frozenRow:e,a=n*options.rowHeight,o=(n+1)*options.rowHeight-i+(viewportHasHScroll?scrollbarDimensions.height:0);(n+1)*options.rowHeight>scrollTop+i+offset?(scrollTo(t?a:o),render()):n*options.rowHeight<scrollTop+offset&&(scrollTo(t?o:a),render())}}function scrollRowToTop(e){scrollTo(e*options.rowHeight),render()}function scrollPage(e){var t=e*numVisibleRows,i=scrollTop+options.rowHeight-1;if(scrollTo((getRowFromPosition(i)+t)*options.rowHeight),render(),options.enableCellNavigation&&null!=activeRow){var n=activeRow+t,a=getDataLengthIncludingAddNew();n>=a&&(n=a-1),n<0&&(n=0);var o=0,s=null,r=activePosX;while(o<=activePosX)canCellBeActive(n,o)&&(s=o),o+=getColspan(n,o);null!==s?(setActiveCellInternal(getCellNode(n,s)),activePosX=r):resetActiveCell()}}function navigatePageDown(){scrollPage(1)}function navigatePageUp(){scrollPage(-1)}function navigateTop(){navigateToRow(0)}function navigateBottom(){navigateToRow(getDataLength()-1)}function navigateToRow(e){var t=getDataLength();if(!t)return!0;if(e<0?e=0:e>=t&&(e=t-1),scrollCellIntoView(e,0,!0),options.enableCellNavigation&&null!=activeRow){var i=0,n=null,a=activePosX;while(i<=activePosX)canCellBeActive(e,i)&&(n=i),i+=getColspan(e,i);null!==n?(setActiveCellInternal(getCellNode(e,n)),activePosX=a):resetActiveCell()}return!0}function getColspan(e,t){var i=data.getItemMetadata&&data.getItemMetadata(e);if(!i||!i.columns)return 1;var n=i.columns[columns[t].id]||i.columns[t],a=n&&n.colspan;return a="*"===a?columns.length-t:a||1,a}function findFirstFocusableCell(e){var t=0;while(t<columns.length){if(canCellBeActive(e,t))return t;t+=getColspan(e,t)}return null}function findLastFocusableCell(e){var t=0,i=null;while(t<columns.length)canCellBeActive(e,t)&&(i=t),t+=getColspan(e,t);return i}function gotoRight(e,t,i){if(t>=columns.length)return null;do{t+=getColspan(e,t)}while(t<columns.length&&!canCellBeActive(e,t));return t<columns.length?{row:e,cell:t,posX:t}:null}function gotoLeft(e,t,i){if(t<=0)return null;var n=findFirstFocusableCell(e);if(null===n||n>=t)return null;var a,o={row:e,cell:n,posX:n};while(1){if(a=gotoRight(o.row,o.cell,o.posX),!a)return null;if(a.cell>=t)return o;o=a}}function gotoDown(e,t,i){var n,a=getDataLengthIncludingAddNew();while(1){if(++e>=a)return null;n=t=0;while(t<=i)n=t,t+=getColspan(e,t);if(canCellBeActive(e,n))return{row:e,cell:n,posX:i}}}function gotoUp(e,t,i){var n;while(1){if(--e<0)return null;n=t=0;while(t<=i)n=t,t+=getColspan(e,t);if(canCellBeActive(e,n))return{row:e,cell:n,posX:i}}}function gotoNext(e,t,i){if(null==e&&null==t&&(e=t=i=0,canCellBeActive(e,t)))return{row:e,cell:t,posX:t};var n=gotoRight(e,t,i);if(n)return n;var a=null,o=getDataLengthIncludingAddNew();e===o-1&&e--;while(++e<o)if(a=findFirstFocusableCell(e),null!==a)return{row:e,cell:a,posX:a};return null}function gotoPrev(e,t,i){if(null==e&&null==t&&(e=getDataLengthIncludingAddNew()-1,t=i=columns.length-1,canCellBeActive(e,t)))return{row:e,cell:t,posX:t};var n,a;while(!n){if(n=gotoLeft(e,t,i),n)break;if(--e<0)return null;t=0,a=findLastFocusableCell(e),null!==a&&(n={row:e,cell:a,posX:a})}return n}function gotoRowStart(e,t,i){var n=findFirstFocusableCell(e);return null===n?null:{row:e,cell:n,posX:n}}function gotoRowEnd(e,t,i){var n=findLastFocusableCell(e);return null===n?null:{row:e,cell:n,posX:n}}function navigateRight(){return navigate("right")}function navigateLeft(){return navigate("left")}function navigateDown(){return navigate("down")}function navigateUp(){return navigate("up")}function navigateNext(){return navigate("next")}function navigatePrev(){return navigate("prev")}function navigateRowStart(){return navigate("home")}function navigateRowEnd(){return navigate("end")}function navigate(e){if(!options.enableCellNavigation)return!1;if(!activeCellNode&&"prev"!=e&&"next"!=e)return!1;if(!getEditorLock().commitCurrentEdit())return!0;setFocus();var t={up:-1,down:1,left:-1,right:1,prev:-1,next:1,home:-1,end:1};tabbingDirection=t[e];var i={up:gotoUp,down:gotoDown,left:gotoLeft,right:gotoRight,prev:gotoPrev,next:gotoNext,home:gotoRowStart,end:gotoRowEnd},n=i[e],a=n(activeRow,activeCell,activePosX);if(a){if(hasFrozenRows&&options.frozenBottom&a.row==getDataLength())return;var o=a.row==getDataLength();return(!options.frozenBottom&&a.row>=actualFrozenRow||options.frozenBottom&&a.row<actualFrozenRow)&&scrollCellIntoView(a.row,a.cell,!o&&options.emulatePagingWhenScrolling),setActiveCellInternal(getCellNode(a.row,a.cell)),activePosX=a.posX,!0}return setActiveCellInternal(getCellNode(activeRow,activeCell)),!1}function getCellNode(e,t){if(rowsCache[e]){ensureCellNodesInRowsCache(e);try{return rowsCache[e].cellNodesByColumnIdx.length>t?rowsCache[e].cellNodesByColumnIdx[t][0]:null}catch(i){return rowsCache[e].cellNodesByColumnIdx[t]}}return null}function setActiveCell(e,t,i,n,a){initialized&&(e>getDataLength()||e<0||t>=columns.length||t<0||options.enableCellNavigation&&(scrollCellIntoView(e,t,!1),setActiveCellInternal(getCellNode(e,t),i,n,a)))}function canCellBeActive(e,t){if(!options.enableCellNavigation||e>=getDataLengthIncludingAddNew()||e<0||t>=columns.length||t<0)return!1;var i=data.getItemMetadata&&data.getItemMetadata(e);if(i&&"undefined"!==typeof i.focusable)return!!i.focusable;var n=i&&i.columns;return n&&n[columns[t].id]&&"undefined"!==typeof n[columns[t].id].focusable?!!n[columns[t].id].focusable:n&&n[t]&&"undefined"!==typeof n[t].focusable?!!n[t].focusable:!!columns[t].focusable}function canCellBeSelected(e,t){if(e>=getDataLength()||e<0||t>=columns.length||t<0)return!1;var i=data.getItemMetadata&&data.getItemMetadata(e);if(i&&"undefined"!==typeof i.selectable)return!!i.selectable;var n=i&&i.columns&&(i.columns[columns[t].id]||i.columns[t]);return n&&"undefined"!==typeof n.selectable?!!n.selectable:!!columns[t].selectable}function gotoCell(e,t,i,n){if(initialized&&canCellBeActive(e,t)&&getEditorLock().commitCurrentEdit()){scrollCellIntoView(e,t,!1);var a=getCellNode(e,t),o=columns[t],s=!!(options.editable&&o&&o.editor&&options.suppressActiveCellChangeOnEdit);setActiveCellInternal(a,i||e===getDataLength()||options.autoEdit,null,s,n),currentEditor||setFocus()}}function commitCurrentEdit(){var e=getDataItem(activeRow),t=columns[activeCell];if(currentEditor){if(currentEditor.isValueChanged()){var i=currentEditor.validate();if(i.valid){if(activeRow<getDataLength()){var n={row:activeRow,cell:activeCell,editor:currentEditor,serializedValue:currentEditor.serializeValue(),prevSerializedValue:serializedEditorValue,execute:function(){this.editor.applyValue(e,this.serializedValue),updateRow(this.row),trigger(self.onCellChange,{row:this.row,cell:this.cell,item:e})},undo:function(){this.editor.applyValue(e,this.prevSerializedValue),updateRow(this.row),trigger(self.onCellChange,{row:this.row,cell:this.cell,item:e})}};options.editCommandHandler?(makeActiveCellNormal(),options.editCommandHandler(e,t,n)):(n.execute(),makeActiveCellNormal())}else{var a={};currentEditor.applyValue(a,currentEditor.serializeValue()),makeActiveCellNormal(),trigger(self.onAddNewRow,{item:a,column:t})}return!getEditorLock().isActive()}return $(activeCellNode).removeClass("invalid"),$(activeCellNode).width(),$(activeCellNode).addClass("invalid"),trigger(self.onValidationError,{editor:currentEditor,cellNode:activeCellNode,validationResults:i,row:activeRow,cell:activeCell,column:t}),currentEditor.focus(),!1}makeActiveCellNormal()}return!0}function cancelCurrentEdit(){return makeActiveCellNormal(),!0}function rowsToRanges(e){for(var t=[],i=columns.length-1,n=0;n<e.length;n++)t.push(new Slick.Range(e[n],0,e[n],i));return t}function getSelectedRows(){if(!selectionModel)throw new Error("Selection model is not set");return selectedRows.slice(0)}function setSelectedRows(e){if(!selectionModel)throw new Error("Selection model is not set");self&&self.getEditorLock&&!self.getEditorLock().isActive()&&selectionModel.setSelectedRanges(rowsToRanges(e))}this.debug=function(){var e="";e+="\ncounter_rows_rendered:  "+counter_rows_rendered,e+="\ncounter_rows_removed:  "+counter_rows_removed,e+="\nrenderedRows:  "+renderedRows,e+="\nnumVisibleRows:  "+numVisibleRows,e+="\nmaxSupportedCssHeight:  "+maxSupportedCssHeight,e+="\nn(umber of pages):  "+n,e+="\n(current) page:  "+page,e+="\npage height (ph):  "+ph,e+="\nvScrollDir:  "+vScrollDir,alert(e)},this.eval=function(expr){return eval(expr)},$.extend(this,{slickGridVersion:"2.4.22",onScroll:new Slick.Event,onSort:new Slick.Event,onHeaderMouseEnter:new Slick.Event,onHeaderMouseLeave:new Slick.Event,onHeaderContextMenu:new Slick.Event,onHeaderClick:new Slick.Event,onHeaderCellRendered:new Slick.Event,onBeforeHeaderCellDestroy:new Slick.Event,onHeaderRowCellRendered:new Slick.Event,onFooterRowCellRendered:new Slick.Event,onFooterContextMenu:new Slick.Event,onFooterClick:new Slick.Event,onBeforeHeaderRowCellDestroy:new Slick.Event,onBeforeFooterRowCellDestroy:new Slick.Event,onMouseEnter:new Slick.Event,onMouseLeave:new Slick.Event,onClick:new Slick.Event,onDblClick:new Slick.Event,onContextMenu:new Slick.Event,onKeyDown:new Slick.Event,onAddNewRow:new Slick.Event,onBeforeAppendCell:new Slick.Event,onValidationError:new Slick.Event,onViewportChanged:new Slick.Event,onColumnsReordered:new Slick.Event,onColumnsResized:new Slick.Event,onCellChange:new Slick.Event,onBeforeEditCell:new Slick.Event,onBeforeCellEditorDestroy:new Slick.Event,onBeforeDestroy:new Slick.Event,onActiveCellChanged:new Slick.Event,onActiveCellPositionChanged:new Slick.Event,onDragInit:new Slick.Event,onDragStart:new Slick.Event,onDrag:new Slick.Event,onDragEnd:new Slick.Event,onSelectedRowsChanged:new Slick.Event,onCellCssStylesChanged:new Slick.Event,onAutosizeColumns:new Slick.Event,onRendered:new Slick.Event,registerPlugin:registerPlugin,unregisterPlugin:unregisterPlugin,getPluginByName:getPluginByName,getColumns:getColumns,setColumns:setColumns,getColumnIndex:getColumnIndex,updateColumnHeader:updateColumnHeader,setSortColumn:setSortColumn,setSortColumns:setSortColumns,getSortColumns:getSortColumns,autosizeColumns:autosizeColumns,autosizeColumn:autosizeColumn,getOptions:getOptions,setOptions:setOptions,getData:getData,getDataLength:getDataLength,getDataItem:getDataItem,setData:setData,getSelectionModel:getSelectionModel,setSelectionModel:setSelectionModel,getSelectedRows:getSelectedRows,setSelectedRows:setSelectedRows,getContainerNode:getContainerNode,updatePagingStatusFromView:updatePagingStatusFromView,applyFormatResultToCellNode:applyFormatResultToCellNode,render:render,invalidate:invalidate,invalidateRow:invalidateRow,invalidateRows:invalidateRows,invalidateAllRows:invalidateAllRows,updateCell:updateCell,updateRow:updateRow,getViewport:getVisibleRange,getRenderedRange:getRenderedRange,resizeCanvas:resizeCanvas,updateRowCount:updateRowCount,scrollRowIntoView:scrollRowIntoView,scrollRowToTop:scrollRowToTop,scrollCellIntoView:scrollCellIntoView,scrollColumnIntoView:scrollColumnIntoView,getCanvasNode:getCanvasNode,getUID:getUID,getHeaderColumnWidthDiff:getHeaderColumnWidthDiff,getScrollbarDimensions:getScrollbarDimensions,getHeadersWidth:getHeadersWidth,getCanvasWidth:getCanvasWidth,getCanvases:getCanvases,getActiveCanvasNode:getActiveCanvasNode,setActiveCanvasNode:setActiveCanvasNode,getViewportNode:getViewportNode,getActiveViewportNode:getActiveViewportNode,setActiveViewportNode:setActiveViewportNode,focus:setFocus,scrollTo:scrollTo,getCellFromPoint:getCellFromPoint,getCellFromEvent:getCellFromEvent,getActiveCell:getActiveCell,setActiveCell:setActiveCell,getActiveCellNode:getActiveCellNode,getActiveCellPosition:getActiveCellPosition,resetActiveCell:resetActiveCell,editActiveCell:makeActiveCellEditable,getCellEditor:getCellEditor,getCellNode:getCellNode,getCellNodeBox:getCellNodeBox,canCellBeSelected:canCellBeSelected,canCellBeActive:canCellBeActive,navigatePrev:navigatePrev,navigateNext:navigateNext,navigateUp:navigateUp,navigateDown:navigateDown,navigateLeft:navigateLeft,navigateRight:navigateRight,navigatePageUp:navigatePageUp,navigatePageDown:navigatePageDown,navigateTop:navigateTop,navigateBottom:navigateBottom,navigateRowStart:navigateRowStart,navigateRowEnd:navigateRowEnd,gotoCell:gotoCell,getTopPanel:getTopPanel,setTopPanelVisibility:setTopPanelVisibility,getPreHeaderPanel:getPreHeaderPanel,getPreHeaderPanelLeft:getPreHeaderPanel,getPreHeaderPanelRight:getPreHeaderPanelRight,setPreHeaderPanelVisibility:setPreHeaderPanelVisibility,getHeader:getHeader,getHeaderColumn:getHeaderColumn,setHeaderRowVisibility:setHeaderRowVisibility,getHeaderRow:getHeaderRow,getHeaderRowColumn:getHeaderRowColumn,setFooterRowVisibility:setFooterRowVisibility,getFooterRow:getFooterRow,getFooterRowColumn:getFooterRowColumn,getGridPosition:getGridPosition,flashCell:flashCell,addCellCssStyles:addCellCssStyles,setCellCssStyles:setCellCssStyles,removeCellCssStyles:removeCellCssStyles,getCellCssStyles:getCellCssStyles,getFrozenRowOffset:getFrozenRowOffset,setColumnHeaderVisibility:setColumnHeaderVisibility,init:finishInitialization,destroy:destroy,getEditorLock:getEditorLock,getEditController:getEditController}),init()}$.extend(!0,window,{Slick:{Grid:SlickGrid}})})(jQuery)},"3d78":function(e,t,i){e.exports=i.p+"static/img/batch-img.9f7f171d.svg"},"3e9a":function(e,t,i){e.exports=i.p+"static/img/polymetric.bf83d2e0.svg"},"3f34":function(e,t,i){e.exports=i.p+"static/img/explain-forecast.7634af20.svg"},"408a":function(e,t,i){var n=i("c6b6");e.exports=function(e){if("number"!=typeof e&&"Number"!=n(e))throw TypeError("Incorrect invocation");return+e}},"428b":function(e,t,i){e.exports=i.p+"static/img/graph-step1.488b8525.svg"},4425:function(e,t,i){e.exports=i.p+"static/img/icon1.effeb908.svg"},"44b8":function(e,t,i){"use strict";i("6780")},4936:function(e,t,i){e.exports=i.p+"static/img/histogram-guid.57843081.svg"},"498a":function(e,t,i){"use strict";var n=i("23e7"),a=i("58a8").trim,o=i("c8d2");n({target:"String",proto:!0,forced:o("trim")},{trim:function(){return a(this)}})},"4ab4":function(e,t,i){e.exports=i.p+"static/img/polymetric.bf83d2e0.svg"},"4d06":function(e,t){e.exports="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABQAAAAUCAYAAACNiR0NAAAAAXNSR0IArs4c6QAAAptJREFUOBGllEFoE0EUhudtbRUvRuhFcognz4V6sSCSg7Ee2kNpoyhYAt1GULEr3YhWsL0oZrWJoOBmK7GCoov2oAWpRSJ4UZDi2ZM9BC8FgweRtmZ8f9wZpprSgAObee+f/32Z2Zkdyhf9IZLithDidWes43Qmk/nJccutXC7vWKmt3uOClCRx3hKSbCnFHn6GV2prc2EYdrRKgxc1qAUDLItnN0PEv9yklEeXq9+e+b7fvhUUHnhRAy8YYFnuhdFQCjqnAPxPfd9/iCeVSmWb0v7uMQYPvGoMDLBICV4hGJOyXlA5/2OYiMdOpNPpX0pDz8tsW67WHvPM0konshzXsYvILSU2BBIXVY6CL9XaLPfagzjSNExwjYJtACLJOdk8kbiioPxST94qBvcZRHgQQ1Pj8KJG5ej1kk3RK/iT/H6uao3ETCOWYkRpDJtyneykylXfFIhBr1C6xrO6pIxmz+/3uuuMXjY1FW8KhIGhHkPHlRk9w24yzDU1M9Yv3BT/J27brDhast51w9eT6u3fvrjw8o2h6bApMD9dmuLvRu82CQr4WeKq7qjy4OEj/RZD32pSFPwD5B2e4DEG/mkW0YNxx7Z7DnTPv/+wlOBvtCsaOpTq7VtfXJh/p7zoN2xKvuDnhBQ3tIHokTtmn+KNqEPjDbK8YvDQPIs42OZZ1JuCT8+E4dPbG48NKxiAiCMtRN5oPIFGbZQ2ZpgvBGeErN+JNC4Uc/u79h1LJpPrSjN7XA4fP31+yod/QOtknc059l3ypktpnjbfHHw9ohG92LVTDmaz2TVtbhLg+uIb57m6cXj2kld43GLMiIKx+IqXNLQVDHx4EvHdg6hBDgZYliAZ8BK/8jPbGWsf4OtqFYZWGryoQS0YYP0GSgsoUPv9lyUAAAAASUVORK5CYII="},"4de4":function(e,t,i){"use strict";var n=i("23e7"),a=i("b727").filter,o=i("1dde"),s=i("ae40"),r=o("filter"),l=s("filter");n({target:"Array",proto:!0,forced:!r||!l},{filter:function(e){return a(this,e,arguments.length>1?arguments[1]:void 0)}})},5185:function(e,t,i){e.exports=i.p+"static/img/operator-node.e4e15cb8.svg"},"532e":function(e,t,i){e.exports=i.p+"static/img/data-process-guid.beda9514.svg"},"54ae":function(e,t,i){e.exports=i.p+"static/img/constant-node.8e641e28.svg"},5530:function(e,t,i){"use strict";i.d(t,"a",(function(){return o}));i("4de4"),i("e439"),i("dbb4"),i("b64b"),i("d3b7"),i("159b");var n=i("ade3");function a(e,t){var i=Object.keys(e);if(Object.getOwnPropertySymbols){var n=Object.getOwnPropertySymbols(e);t&&(n=n.filter((function(t){return Object.getOwnPropertyDescriptor(e,t).enumerable}))),i.push.apply(i,n)}return i}function o(e){for(var t=1;t<arguments.length;t++){var i=null!=arguments[t]?arguments[t]:{};t%2?a(Object(i),!0).forEach((function(t){Object(n["a"])(e,t,i[t])})):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(i)):a(Object(i)).forEach((function(t){Object.defineProperty(e,t,Object.getOwnPropertyDescriptor(i,t))}))}return e}},"564b":function(e,t,i){e.exports=i.p+"static/img/collapse-right.4e288788.svg"},"56de":function(e,t,i){e.exports=i.p+"static/img/graph-guid.2fe2af69.svg"},5801:function(e,t,i){e.exports=i.p+"static/img/explain-fn.51490881.svg"},"5d69":function(e,t,i){"use strict";i.r(t);var n,a,o=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"deb-wrap"},[n("FlexibleGrid",{attrs:{areas:e.areas,rowSize:e.rowSize,columnSize:e.columnSize,hideAreas:e.hideAreas,showFixed:e.showFixed,gridStyleKey:e.gridStyleKey},on:{resizeGridStyle:e.resizeGridStyle}},[n("div",{staticClass:"left-wrap",attrs:{slot:"left"},slot:"left"},[n("div",{staticClass:"left"},[n("div",{staticClass:"header"},[e._v(" "+e._s("tree"===e.radio1?e.$t("debugger.nodeList"):"stack"===e.radio1?e.$t("debugger.stackList"):e.$t("debugger.curHitNode")+"("+e.pagination.total+")")+" "),e.hitsOutdated&&"hit"===e.radio1?n("div",{staticClass:"outdate-tip"},[n("el-tooltip",{staticClass:"item",attrs:{effect:"light",content:e.$t("debugger.outdateTip"),placement:"top"}},[n("i",{staticClass:"el-icon-warning"})])],1):e._e(),n("div",{staticClass:"radio-tabs"},[n("el-radio-group",{attrs:{size:"mini"},on:{change:function(t){return e.searchWatchpointHits(!0)}},model:{value:e.radio1,callback:function(t){e.radio1=t},expression:"radio1"}},[n("el-radio-button",{attrs:{label:"hit"}},[n("i",{staticClass:"el-icon-s-fold"})]),n("el-radio-button",{attrs:{label:"tree"}},[n("i",{staticClass:"el-icon-s-grid"})]),n("el-radio-button",{attrs:{label:"stack"}},[n("i",{staticClass:"el-icon-s-unfold"})])],1)],1)]),n("div",{directives:[{name:"show",rawName:"v-show",value:"tree"===e.radio1,expression:"radio1==='tree'"}],staticClass:"content"},[n("div",{staticClass:"search-conditions",class:e.collapseConditions?"collapse-top":""},[n("div",{staticClass:"condition-title"},[e._v(" "+e._s(e.$t("debugger.searchConditions"))+" "),n("div",{staticClass:"img-btn",class:e.collapseConditions?"collapse":"",on:{click:function(t){e.collapseConditions=!e.collapseConditions}}})]),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.collapseConditions,expression:"!collapseConditions"}],staticClass:"condition-container"},[n("div",{staticClass:"node-type"},[n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.logicCard")))]),n("el-select",{attrs:{disabled:!e.trainId},on:{change:e.logicCardChange},model:{value:e.logicCard.value,callback:function(t){e.$set(e.logicCard,"value",t)},expression:"logicCard.value"}},e._l(e.logicCard.options,(function(e){return n("el-option",{key:e,attrs:{value:e}})})),1)],1),n("div",{staticClass:"node-type"},[n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.graphName")))]),n("el-select",{on:{change:e.queryGraphByFile},model:{value:e.graphFiles.value,callback:function(t){e.$set(e.graphFiles,"value",t)},expression:"graphFiles.value"}},e._l(e.graphFiles.options,(function(e){return n("el-option",{key:e,attrs:{value:e}})})),1)],1),n("div",{staticClass:"node-type"},[n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.nodeTypes")))]),n("el-select",{on:{change:e.nodeTypesChange},model:{value:e.nodeTypes.value,callback:function(t){e.$set(e.nodeTypes,"value",t)},expression:"nodeTypes.value"}},e._l(e.nodeTypes.options,(function(t){return n("el-option",{key:t,class:{"deb-indent":"all"!=t},attrs:{label:e.nodeTypes.label[t],value:t}})})),1)],1),n("div",{staticClass:"select-wrap"},[n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.nodeName")))]),n("el-input",{staticClass:"input-with-select",attrs:{placeholder:e.$t("graph.inputNodeName"),title:e.searchWord,clearable:""},on:{input:e.filterChange,clear:e.filter},nativeOn:{keyup:function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"enter",13,t.key,"Enter")?null:e.filter.apply(null,arguments)}},model:{value:e.searchWord,callback:function(t){e.searchWord=t},expression:"searchWord"}})],1),n("div",{staticClass:"select-wrap"},[n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.stackInfo")))]),n("el-input",{staticClass:"input-with-select",attrs:{placeholder:e.$t("debugger.inputStack"),title:e.searchStackContent,clearable:""},on:{input:e.filterChange,clear:e.filter},nativeOn:{keyup:function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"enter",13,t.key,"Enter")?null:e.filter.apply(null,arguments)}},model:{value:e.searchStackContent,callback:function(t){e.searchStackContent=t},expression:"searchStackContent"}})],1)])]),n("div",{staticClass:"tree-wrap",class:e.collapseConditions?"":"collapse"},[e.curWatchPointId&&e.treeFlag?n("div",{staticClass:"select-all-files"}):e._e(),n("tree",{directives:[{name:"show",rawName:"v-show",value:e.treeFlag,expression:"treeFlag"}],ref:"tree",attrs:{props:e.props,load:e.loadNode,"node-key":"name","expand-on-click-node":!1,lazy:e.lazy,"highlight-current":!0,"show-checkbox":!!e.curWatchPointId,disabled:e.treeDisabled},on:{"node-collapse":e.nodeCollapse,"node-click":e.handleNodeClick,check:e.check},scopedSlots:e._u([{key:"default",fn:function(t){var a=t.node,o=t.data;return n("span",{staticClass:"custom-tree-node"},[n("span",{class:{const:"Const"===o.type&&e.curWatchPointId}},["name_scope"===o.type?n("img",{staticClass:"image-type",attrs:{src:i("9b3c")}}):"Const"===o.type?n("img",{staticClass:"image-type",attrs:{src:i("3325")}}):"aggregation_scope"===o.type?n("img",{staticClass:"image-type",attrs:{src:i("3e9a")}}):e.nodeIsDynamicShape(a.data)?n("img",{staticClass:"image-type",attrs:{src:i("8c7d")}}):n("img",{staticClass:"image-type",attrs:{src:i("5185")}})]),n("span",{staticClass:"custom-tree-node",attrs:{title:a.label}},[e._v(e._s(a.label))])])}}])}),n("tree",{directives:[{name:"show",rawName:"v-show",value:!e.treeFlag,expression:"!treeFlag"}],ref:"searchTree",attrs:{props:e.defaultProps,load:e.loadSearchNode,lazy:!0,"node-key":"name","expand-on-click-node":!1,"show-checkbox":!!e.curWatchPointId,disabled:e.treeDisabled},on:{"node-click":e.handleNodeClick,check:e.searchCheck},scopedSlots:e._u([{key:"default",fn:function(t){var a=t.node,o=t.data;return n("span",{staticClass:"custom-tree-node"},[n("span",{class:{const:"Const"===o.type&&e.curWatchPointId}},["name_scope"===o.type?n("img",{staticClass:"image-type",attrs:{src:i("9b3c")}}):"Const"===o.type?n("img",{staticClass:"image-type",attrs:{src:i("3325")}}):"aggregation_scope"===o.type?n("img",{staticClass:"image-type",attrs:{src:i("3e9a")}}):e.nodeIsDynamicShape(a.data)?n("img",{staticClass:"image-type",attrs:{src:i("8c7d")}}):n("img",{staticClass:"image-type",attrs:{src:i("5185")}})]),n("span",{staticClass:"custom-tree-node",attrs:{title:a.label}},[e._v(e._s(a.label))])])}}])})],1),n("div",{staticClass:"watch-point-wrap"},[n("div",{staticClass:"title-wrap"},[e._v(" "+e._s(e.$t("debugger.watchList"))+" "),n("el-tooltip",{staticClass:"tooltip",attrs:{effect:"light",placement:"right"}},[n("div",{attrs:{slot:"content"},slot:"content"},[e._v(" "+e._s(e.$t("debugger.createWpTip.explain"))+" "),n("div",[e._v(e._s(e.$t("debugger.createWpTip.example")))])]),n("i",{staticClass:"el-icon-info"})]),n("div",{staticClass:"check-wrap"},[n("i",{staticClass:"el-icon-circle-check",class:{disable:!e.enableRecheck},attrs:{title:e.$t("debugger.recheck")},on:{click:function(t){return e.recheckWatchpoint()}}})]),n("div",{staticClass:"delete-wrap"},[n("i",{staticClass:"el-icon-delete",class:{disable:!(e.watchPointArr.length&&e.metadata.state!==e.state.running&&e.metadata.state!==e.state.sending)},attrs:{title:e.$t("debugger.clearWatchpoint")},on:{click:function(t){return e.deleteWatchpoint()}}})]),n("div",{staticClass:"add-wrap"},[n("i",{staticClass:"el-icon-circle-plus",class:{disable:e.metadata.state===e.state.running||e.metadata.state===e.state.sending},attrs:{title:e.$t("debugger.createWP")},on:{click:e.initCondition}})])],1),n("div",{staticClass:"content-wrap"},[n("ul",{directives:[{name:"show",rawName:"v-show",value:e.allWatchPointFlag,expression:"allWatchPointFlag"}],staticClass:"list-wrap",attrs:{id:"watch-point-list"}},e._l(e.watchPointArr,(function(t,i){return n("li",{key:i,staticClass:"list",attrs:{title:e.getWatchPointContent(t)}},[n("div",{staticClass:"name",class:{selected:t.selected},on:{click:function(t){return e.selectWatchPoint(i)}}},[n("div",{staticClass:"item-content"},[e._v(" "+e._s(e.getWatchPointContent(t))+" ")]),t.selected?n("i",{staticClass:"el-icon-check icon",on:{click:function(t){return t.stopPropagation(),e.showOrigin()}}}):e._e(),t.selected?n("i",{staticClass:"el-icon-close icon",class:{disabled:e.metadata.state===e.state.running||e.metadata.state===e.state.sending},on:{click:function(i){return i.stopPropagation(),e.deleteWatchpoint(t)}}}):e._e()])])})),0)])])]),n("div",{directives:[{name:"show",rawName:"v-show",value:"hit"===e.radio1,expression:"radio1==='hit'"}],staticClass:"content"},[n("div",{staticClass:"node-type"},[n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.logicCard")))]),n("el-select",{attrs:{disabled:!e.trainId||e.metadata.state===e.state.running||e.metadata.state===e.state.sending},on:{change:function(t){e.logicCardChange(),e.searchWatchpointHits(!0,!0)}},model:{value:e.logicCard.value,callback:function(t){e.$set(e.logicCard,"value",t)},expression:"logicCard.value"}},e._l(e.logicCard.options,(function(e){return n("el-option",{key:e,attrs:{value:e}})})),1)],1),n("div",{staticClass:"node-type"},[n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.graphName")))]),n("el-select",{attrs:{disabled:e.metadata.state===e.state.running||e.metadata.state===e.state.sending},on:{change:function(t){return e.searchWatchpointHits(!0,!0)}},model:{value:e.hitWpCondition.graphFile,callback:function(t){e.$set(e.hitWpCondition,"graphFile",t)},expression:"hitWpCondition.graphFile"}},e._l(e.graphFiles.options,(function(e){return n("el-option",{key:e,attrs:{value:e}})})),1)],1),n("div",{staticClass:"node-type"},[n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.watchPoint")))]),n("el-select",{attrs:{disabled:e.metadata.state===e.state.running||e.metadata.state===e.state.sending},on:{change:function(t){return e.searchWatchpointHits(!0,!0)}},model:{value:e.hitWpCondition.watchPoint,callback:function(t){e.$set(e.hitWpCondition,"watchPoint",t)},expression:"hitWpCondition.watchPoint"}},e._l(e.hitWatchPointArr,(function(e){return n("el-option",{key:e.value,attrs:{label:e.label,value:e.value}})})),1)],1),n("div",{staticClass:"hit-list-wrap"},[n("el-table",{staticClass:"watchpoint-table",attrs:{data:e.watchPointHits,"row-key":"id","expand-row-keys":e.expandKeys}},[n("el-table-column",{attrs:{type:"expand",width:"40"},scopedSlots:e._u([{key:"default",fn:function(t){return[n("ul",e._l(t.row.lists,(function(t,i){return n("li",{key:i},[e._v(e._s(t.name)+" "),e._l(t.params,(function(t,i){return n("div",{key:i,staticClass:"param"},[n("div",{staticClass:"tensor-icon"}),e._v(" "+e._s(t.content)+" ")])})),t.tip?n("div",{staticClass:"hit-tip"},[n("i",{staticClass:"el-icon-warning"}),e._v(e._s(t.tip)+" ")]):e._e()],2)})),0)]}}])}),n("el-table-column",{attrs:{prop:"name",label:e.$t("graph.name")},scopedSlots:e._u([{key:"default",fn:function(t){return[n("div",{staticClass:"hit-item",class:{selected:t.row.selected},on:{click:function(i){return e.updateTensorValue(t.$index)}}},[e._v(" "+e._s(t.row.graph_name)+"/"+e._s(t.row.name)+" ")])]}}])})],1),n("el-pagination",{directives:[{name:"show",rawName:"v-show",value:e.pagination.total,expression:"pagination.total"}],staticClass:"watchpoint-page",attrs:{small:"","current-page":e.pagination.currentPage,"page-size":e.pagination.pageSize,"pager-count":e.pagination.pageCount,layout:"prev, pager, next, jumper",total:e.pagination.total},on:{"current-change":e.handleCurrentChange}})],1)]),n("div",{directives:[{name:"show",rawName:"v-show",value:"stack"===e.radio1,expression:"radio1==='stack'"}],staticClass:"content"},[n("div",{staticClass:"stack-search"},[n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.stackInfo")))]),n("el-input",{attrs:{placeholder:e.$t("debugger.inputStack"),title:e.stacks.searchContent,clearable:""},on:{clear:function(t){return e.stackPageChange(1)}},nativeOn:{keyup:function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"enter",13,t.key,"Enter")?null:e.queryStacks.apply(null,arguments)}},model:{value:e.stacks.searchContent,callback:function(t){e.$set(e.stacks,"searchContent",t)},expression:"stacks.searchContent"}})],1),n("div",{staticClass:"hit-list-wrap"},[n("el-table",{staticClass:"watchpoint-table stack",attrs:{data:e.stacks.list}},[n("el-table-column",{attrs:{type:"expand"},scopedSlots:e._u([{key:"default",fn:function(t){return[n("ul",{staticClass:"stack-info"},e._l(t.row.items,(function(t){return n("li",{key:t.line_no},[e._v(" "+e._s(t.line_no)+": "+e._s(t.code_line)+" "),n("el-button",{staticClass:"btn",attrs:{type:"text"},on:{click:function(i){return e.stackOperator(t)}}},[e._v(e._s(e.$t("debugger.search")))])],1)})),0)]}}])}),n("el-table-column",{attrs:{prop:"file_path",label:e.$t("graph.name")}})],1),n("el-pagination",{directives:[{name:"show",rawName:"v-show",value:e.stacks.total,expression:"stacks.total"}],staticClass:"watchpoint-page",attrs:{small:"","current-page":e.stacks.currentPage,"page-size":e.stacks.pageSize.size,"pager-count":e.pagination.pageCount,layout:"prev, pager, next, jumper",total:e.stacks.total},on:{"current-change":e.stackPageChange}})],1)]),n("div",{staticClass:"btn-wrap"},[n("div",{staticClass:"step"},[n("el-tooltip",{staticClass:"item",attrs:{effect:"light",content:e.$t("debugger.inputTip",{total_step_num:e.metadata.total_step_num}),placement:"top-start"}},[n("el-input",{attrs:{placeholder:e.$t("debugger.inputStep"),disabled:e.metadata.step===e.metadata.total_step_num&&!!e.trainId},on:{input:e.stepChange},nativeOn:{keyup:function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"enter",13,t.key,"Enter")?null:e.control(0)}},model:{value:e.step,callback:function(t){e.step=t},expression:"step"}})],1),n("el-button",{staticClass:"custom-btn green",attrs:{type:"primary",size:"mini",disabled:!(e.step&&e.metadata.state===e.state.waiting)||e.metadata.step===e.metadata.total_step_num&&!!e.trainId},on:{click:function(t){return e.control(0)}}},[e._v(e._s(e.$t("public.sure")))])],1),n("div",{staticClass:"btn-two"},[n("el-button",{staticClass:"custom-btn white",attrs:{size:"mini",disabled:e.metadata.state!==e.state.waiting||e.metadata.step===e.metadata.total_step_num&&!!e.trainId},on:{click:function(t){return e.control(1)}}},[e._v(e._s(e.$t("debugger.continue")))]),n("el-button",{staticClass:"custom-btn white",attrs:{size:"mini",disabled:e.metadata.state!==e.state.running},on:{click:function(t){return e.control(3)}}},[e._v(e._s(e.$t("debugger.pause")))]),n("el-button",{staticClass:"custom-btn white",attrs:{size:"mini",disabled:e.metadata.state===e.state.pending||e.metadata.state===e.state.sending},on:{click:e.terminate}},[e._v(e._s(e.$t("debugger.terminate")))])],1)])])]),n("div",{staticClass:"rightTop",attrs:{slot:"rightTop"},slot:"rightTop"},[n("div",{staticClass:"header"},[n("span",{staticClass:"item"},[e._v(" "+e._s(e.$t("debugger.clientIp")+e.$t("symbols.colon"))+" "),n("span",{staticClass:"content"},[e._v(" "+e._s(void 0!==e.metadata.ip?e.metadata.ip:"--")+" ")])]),n("span",{staticClass:"item"},[e._v(e._s(e.$t("debugger.deviceId")+e.$t("symbols.colon"))+" "),n("span",{staticClass:"content"},[e._v(" "+e._s(void 0!==e.metadata.device_name?e.metadata.device_name:"--")+" ")])]),n("span",{staticClass:"item"},[e._v(e._s(e.$t("debugger.currentStep")+e.$t("symbols.colon"))+" "),n("span",{staticClass:"content"},[e._v(" "+e._s(void 0!==e.metadata.step?e.metadata.step:"--")+" ")]),e.trainId&&!e.isShowInp?n("i",{staticClass:"el-icon-edit",class:{disabled:e.metadata.state===e.state.running||e.metadata.state===e.state.sending},attrs:{title:e.$t("debugger.inpStepTip",{total_step_num:e.metadata.total_step_num})},on:{click:e.editStep}}):e._e(),e.trainId&&e.isShowInp?n("el-tooltip",{staticClass:"item",attrs:{effect:"light",content:e.$t("debugger.inputTip",{total_step_num:e.metadata.total_step_num}),placement:"top-start"}},[n("el-input",{attrs:{type:"text"},on:{input:e.newStepChange},model:{value:e.newStep,callback:function(t){e.newStep=t},expression:"newStep"}})],1):e._e(),e.trainId&&e.isShowInp?n("i",{staticClass:"el-icon-check",on:{click:e.saveStepValue}}):e._e(),e.trainId&&e.isShowInp?n("i",{staticClass:"el-icon-close",on:{click:function(t){e.isShowInp=!1}}}):e._e(),n("el-tooltip",{staticClass:"tooltip",attrs:{effect:"light",content:e.$t("debugger.stepTip")+(e.trainId?e.$t("debugger.stepTipOffline"):""),placement:"top"}},[n("i",{staticClass:"el-icon-info"})])],1),n("span",{staticClass:"item"},[e._v(e._s(e.$t("debugger.state")+e.$t("symbols.colon"))+" "),n("span",{staticClass:"content"},[n("i",{directives:[{name:"show",rawName:"v-show",value:e.metadata.state===e.state.running,expression:"metadata.state === state.running"}],staticClass:"el-icon-loading"}),n("i",{directives:[{name:"show",rawName:"v-show",value:e.metadata.state===e.state.sending,expression:"metadata.state === state.sending"}],staticClass:"el-icon-time"}),e._v(" "+e._s(e.metadata.state?e.$t("debugger.stateTips."+e.metadata.state):"")+" ")]),n("el-tooltip",{staticClass:"tooltip",attrs:{effect:"light",placement:"right"}},[n("div",{attrs:{slot:"content"},slot:"content"},[n("div",{staticClass:"tooltip-item"},[n("div",{staticClass:"item"},[n("span",[e._v("1.")]),n("span",[e._v(e._s(e.$t("debugger.stateInfo.waiting")))])]),n("div",{staticClass:"item"},[n("span",[e._v("2.")]),n("span",[e._v(e._s(e.$t("debugger.stateInfo.running")))])]),n("div",{staticClass:"item"},[n("span",[e._v("3.")]),n("span",[e._v(e._s(e.$t("debugger.stateInfo.sending")))])]),n("div",{staticClass:"item"},[n("span",[e._v("4.")]),n("span",[e._v(e._s(e.$t("debugger.stateInfo.pending")))])])])]),n("i",{staticClass:"el-icon-info"})])],1)]),n("div",{staticClass:"svg-wrap",class:{collapse:e.collapseTable}},[n("div",{staticClass:"graph-container"},[n("div",{attrs:{id:"graph"}}),n("div",{attrs:{id:"contextMenu"}},[n("ul",[n("li",[e._v(e._s(e.$t("debugger.continueTo")))])])])]),n("div",{staticClass:"btn-wrap"},["Ascend"===e.version?n("el-button",{staticClass:"custom-btn green notShow",attrs:{type:"primary",size:"mini"},on:{click:function(t){return e.getNodeByBfs(!1)}}},[e._v(" "+e._s(e.$t("debugger.previousNode"))+" ")]):e._e(),"GPU"!==e.version||e.trainId?e._e():n("el-button",{staticClass:"custom-btn green",class:{disabled:!e.currentNodeName},attrs:{type:"primary",size:"mini",disabled:!e.currentNodeName},on:{click:e.getCurrentNodeInfo}},[e._v(" "+e._s(e.$t("debugger.currentNode"))+" ")]),"Ascend"===e.version?n("el-button",{staticClass:"custom-btn white notShow",attrs:{type:"primary",size:"mini"},on:{click:function(t){return e.getNodeByBfs(!0)}}},[e._v(" "+e._s(e.$t("debugger.nextNode"))+" ")]):e._e(),"GPU"!==e.version||e.trainId?e._e():n("el-button",{staticClass:"custom-btn white",class:{disabled:e.metadata.state===e.state.running||e.metadata.state===e.state.sending},attrs:{type:"primary",size:"mini",disabled:e.metadata.state===e.state.running||e.metadata.state===e.state.sending},on:{click:e.getNextNodeInfo}},[e._v(" "+e._s(e.$t("debugger.nextNode"))+" ")])],1),n("div",{staticClass:"legend"},[n("div",{staticClass:"title"},[e._v(" "+e._s(e.$t("graph.legend"))+" "),n("img",{directives:[{name:"show",rawName:"v-show",value:!e.showLegend,expression:"!showLegend"}],attrs:{src:i("e08c"),alt:""},on:{click:function(t){e.showLegend=!e.showLegend}}}),n("img",{directives:[{name:"show",rawName:"v-show",value:e.showLegend,expression:"showLegend"}],attrs:{src:i("30d6"),alt:""},on:{click:function(t){e.showLegend=!e.showLegend}}})]),n("div",{directives:[{name:"show",rawName:"v-show",value:e.showLegend,expression:"showLegend"}],staticClass:"legend-content"},[n("div",{staticClass:"legend-item"},[n("div",{staticClass:"pic"},[n("img",{attrs:{src:i("1a09")("./"+e.themeIndex+"/name-scope.svg"),alt:""}})]),n("div",{staticClass:"legend-text",attrs:{title:e.$t("graph.nameSpace")}},[e._v(" "+e._s(e.$t("graph.nameSpace"))+" ")])]),n("div",{staticClass:"legend-item"},[n("div",{staticClass:"pic"},[n("img",{attrs:{src:i("bc0b")("./"+e.themeIndex+"/polymetric.svg"),alt:""}})]),n("div",{staticClass:"legend-text",attrs:{title:e.$t("graph.polymetric")}},[e._v(" "+e._s(e.$t("graph.polymetric"))+" ")])]),n("div",{staticClass:"legend-item"},[n("div",{staticClass:"pic"},[n("img",{attrs:{src:i("d73b")("./"+e.themeIndex+"/virtual-node.svg"),alt:""}})]),n("div",{staticClass:"legend-text",attrs:{title:e.$t("graph.virtualNode")}},[e._v(" "+e._s(e.$t("graph.virtualNode"))+" ")])]),n("div",{staticClass:"legend-item"},[n("div",{staticClass:"pic"},[n("img",{attrs:{src:i("e057")("./"+e.themeIndex+"/operator-node.svg"),alt:""}})]),n("div",{staticClass:"legend-text",attrs:{title:e.$t("graph.operatorNode")}},[e._v(" "+e._s(e.$t("graph.operatorNode"))+" ")])]),n("div",{staticClass:"legend-item"},[n("div",{staticClass:"pic"},[n("img",{attrs:{src:i("e7e6")("./"+e.themeIndex+"/constant-node.svg"),alt:""}})]),n("div",{staticClass:"legend-text",attrs:{title:e.$t("graph.constantNode")}},[e._v(" "+e._s(e.$t("graph.constantNode"))+" ")])]),n("div",{staticClass:"legend-item"},[n("div",{staticClass:"pic"},[n("img",{attrs:{src:i("2df0")("./"+e.themeIndex+"/dynamic-shape-node.svg"),alt:""}})]),n("div",{staticClass:"legend-text",attrs:{title:e.$t("graph.dynamicShapeNode")}},[e._v(" "+e._s(e.$t("graph.dynamicShapeNode"))+" ")])]),n("br"),n("div",{staticClass:"legend-item"},[n("div",{staticClass:"pic"},[n("img",{attrs:{src:i("7012"),alt:""}})]),n("div",{staticClass:"legend-text",attrs:{title:e.$t("graph.dataFlowEdge")}},[e._v(" "+e._s(e.$t("graph.dataFlowEdge"))+" ")])]),n("div",{staticClass:"legend-item"},[n("div",{staticClass:"pic"},[n("img",{attrs:{src:i("c37b"),alt:""}})]),n("div",{staticClass:"legend-text",attrs:{title:e.$t("graph.controllDepEdge")}},[e._v(" "+e._s(e.$t("graph.controllDepEdge"))+" ")])])])]),e.trainId?n("div",{staticClass:"graph-count-wrap"},[n("div",{staticClass:"title"},[e._v(" "+e._s(e.$t("debugger.graphExecutionHistory"))+" "),n("el-tooltip",{staticClass:"tooltip",attrs:{effect:"light",placement:"right"}},[n("div",{attrs:{slot:"content"},slot:"content"},[n("div",{staticClass:"graph-count-info"},[n("div",{staticClass:"graph-execution-history"},[e._v(e._s(e.$t("debugger.graphExecutionHistoryTip")))])])]),n("i",{staticClass:"el-icon-info"})]),n("div",{staticClass:"select-wrap"},[n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.hasData")))]),n("el-select",{on:{change:e.graphExecutionSelect},model:{value:e.hasDataObj.value,callback:function(t){e.$set(e.hasDataObj,"value",t)},expression:"hasDataObj.value"}},e._l(e.hasDataObj.options,(function(e){return n("el-option",{key:e.value,attrs:{label:e.label,value:e.value}})})),1),n("div",{staticClass:"label"},[e._v(e._s(e.$t("debugger.graphName"))+" "),n("el-tooltip",{staticClass:"tooltip",attrs:{effect:"light",content:e.$t("debugger.graphNameExplain"),placement:"right"}},[n("i",{staticClass:"el-icon-info"})])],1),n("el-select",{on:{change:e.graphExecutionSelect},model:{value:e.graphNameObj.value,callback:function(t){e.$set(e.graphNameObj,"value",t)},expression:"graphNameObj.value"}},e._l(e.graphNameObj.options,(function(e){return n("el-option",{key:e,attrs:{value:e}})})),1)],1),n("div",{staticClass:"has-data"}),e._v(e._s(this.$t("debugger.hasDataTip"))+" "),n("div",{staticClass:"current-step"}),e._v(e._s(this.$t("debugger.currentStepTip"))+" "),n("img",{directives:[{name:"show",rawName:"v-show",value:!e.showGraphCount,expression:"!showGraphCount"}],attrs:{src:i("e08c"),alt:""},on:{click:function(t){e.showGraphCount=!e.showGraphCount,e.scrollGraphCount()}}}),n("img",{directives:[{name:"show",rawName:"v-show",value:e.showGraphCount,expression:"showGraphCount"}],attrs:{src:i("30d6"),alt:""},on:{click:function(t){e.showGraphCount=!e.showGraphCount}}})],1),n("div",{directives:[{name:"show",rawName:"v-show",value:e.showGraphCount,expression:"showGraphCount"}],staticClass:"content"},[n("div",{staticClass:"left"},[n("div",{staticClass:"graph-count"},[e._v(" "+e._s(e.$t("debugger.graphExecutionStep"))+" "),n("el-tooltip",{staticClass:"tooltip",attrs:{effect:"light",placement:"right-start"}},[n("div",{attrs:{slot:"content"},slot:"content"},[n("div",{staticClass:"graph-execute"},[n("div",{staticClass:"graph-execute-step"},[e._v(e._s(e.$t("debugger.graphExecutionCountTip")))])])]),n("i",{staticClass:"el-icon-info"})])],1),n("div",{staticClass:"graph-id"},[e._v(e._s(e.$t("debugger.graphName")))])]),n("div",{staticClass:"right",attrs:{id:"graph-count-container"},on:{scroll:e.handleScroll}},[e.graphIdArr.length?n("div",{staticClass:"all-content-wrap",style:{width:e.graphIdArr.length*e.graphCountWidth+"px"}},[n("div",{ref:"content",staticClass:"content-wrap"},e._l(e.visibleGraphIdArr,(function(t,i){return n("div",{key:i,staticClass:"value-wrap",class:{highLight:e.metadata.step===t.count,hasData:t.has_data}},[n("div",{staticClass:"count"},[e._v(e._s(t.count))]),t.sub_graph_names.length?n("el-tooltip",{staticClass:"item",attrs:{effect:"light",content:"sub("+t.sub_graph_names.join()+")",placement:"top"}},[n("div",{staticClass:"graph-name"},[e._v(e._s(t.graph_name))])]):n("div",{staticClass:"graph-name"},[e._v(e._s(t.graph_name))])],1)})),0)]):n("div",{staticClass:"no-data"},[e._v(" "+e._s(e.isGraphRunsInit?e.$t("public.noData")+(e.allGraphIdArr.length?"":"("+e.$t("debugger.noExecutionHistoryFile")+")"):e.$t("public.dataLoading"))+" ")])])])]):e._e()])]),n("div",{staticClass:"rightBottom",attrs:{slot:"rightBottom"},slot:"rightBottom"},[n("div",{staticClass:"table-container"},[n("el-tabs",{on:{"tab-click":e.tabsChange},model:{value:e.tabs.activeName,callback:function(t){e.$set(e.tabs,"activeName",t)},expression:"tabs.activeName"}},[n("el-tab-pane",{attrs:{name:"tensor"}},[n("span",{attrs:{slot:"label"},slot:"label"},[e._v(" "+e._s(e.$t("debugger.tensorMsg"))+" "),e.tableData.length&&e.tableData.find((function(e){return void 0===e.value}))?n("el-tooltip",{staticClass:"item",attrs:{effect:"dark",content:e.$t("public.dataLoading"),placement:"top"}},[n("i",{staticClass:"el-icon-loading"})]):e._e()],1),n("div",{staticClass:"table-content"},[n("div",{staticClass:"table-wrap"},[n("el-table",{ref:"singleTable",attrs:{data:e.tableData,"header-cell-style":e.discountHeaderStyle,"span-method":e.objectSpanMethod,"row-class-name":e.tableRowClassName,"tooltip-effect":"light"}},[n("el-table-column",{attrs:{label:e.$t("graph.name")}},[n("el-table-column",{attrs:{property:"type",width:"80"}}),n("el-table-column",{attrs:{label:"","show-overflow-tooltip":""},scopedSlots:e._u([{key:"default",fn:function(t){return[n("span",{staticClass:"value",on:{click:function(i){return e.queryAllTreeData(t.row.name,!1,t.row.graph_name,!0)}}},[e._v(" "+e._s(t.row.name)+" ")])]}}])})],1),n("el-table-column",{attrs:{property:"step",label:e.$t("debugger.step"),width:"80"}}),n("el-table-column",{attrs:{property:"dtype",label:e.$t("debugger.dType"),width:"200"}}),n("el-table-column",{attrs:{property:"shape",label:e.$t("debugger.shape"),width:"120"}}),n("el-table-column",{attrs:{label:e.$t("debugger.value"),width:"260"},scopedSlots:e._u([{key:"default",fn:function(t){return[n("div",{staticClass:"value-wrap"},["click to view"===t.row.value?n("el-button",{attrs:{size:"mini",type:"text",disabled:e.metadata.state===e.state.running||e.metadata.state===e.state.sending},on:{click:function(i){return e.showTensor(t.row,"value")}}},[e._v(" "+e._s(e.$t("debugger.view"))+" ")]):n("el-button",{staticClass:"value-tip",attrs:{size:"mini",type:"text",disabled:e.metadata.state===e.state.running||e.metadata.state===e.state.sending,title:isNaN(t.row.value)?"":t.row.value},on:{click:function(i){return e.showTensor(t.row,"value")}}},[e._v(" "+e._s(t.row.value))]),n("el-button",{attrs:{size:"mini",type:"text",disabled:e.metadata.state===e.state.running||e.metadata.state===e.state.sending||"null"===t.row.value||!t.row.value||t.row.oversized},on:{click:function(i){return e.loadTensor(t.row)}}},[e._v(" "+e._s(e.$t("graph.downloadPic"))+" ")]),n("el-button",{attrs:{size:"mini",type:"text",disabled:e.metadata.state===e.state.running||e.metadata.state===e.state.sending||!t.row.has_prev_step||"oversize"===t.row.tensor_status},on:{click:function(i){return e.showTensor(t.row,"compare")}}},[e._v(" "+e._s(e.$t("debugger.compareToPre"))+" ")])],1)]}}])},[n("template",{slot:"header"},[n("span",{staticClass:"center"},[e._v(e._s(e.$t("debugger.value")))])])],2)],1)],1)])]),n("el-tab-pane",{attrs:{label:e.$t("graph.nodeInfo"),name:"detail"}},[n("div",{staticClass:"table-content"},[n("div",{staticClass:"table-wrap"},[n("el-table",{ref:"nodeInfo",attrs:{data:e.selectedNode.IOInfo,"header-cell-style":e.discountHeaderStyle,"span-method":e.objectSpanMethod,"row-class-name":e.tableRowClassName,"tooltip-effect":"light"}},[n("el-table-column",{attrs:{label:e.$t("graph.name")}},[n("el-table-column",{attrs:{property:"IOType",width:"80"}}),n("el-table-column",{attrs:{label:"","show-overflow-tooltip":""},scopedSlots:e._u([{key:"default",fn:function(t){return[n("span",{staticClass:"value",on:{click:function(i){return e.queryAllTreeData(t.row.name,!1,t.row.graph_name,!0)}}},[e._v(" "+e._s(t.row.name)+" ")])]}}])})],1)],1)],1)])]),n("el-tab-pane",{attrs:{name:"stack"}},[n("span",{attrs:{slot:"label"},slot:"label"},[e._v(" "+e._s(e.$t("debugger.stackInfo"))+" "),n("el-tooltip",{staticClass:"item",attrs:{effect:"light",content:e.$t("debugger.stackTip"),placement:"top"}},[n("i",{staticClass:"el-icon-info"})])],1),n("div",{staticClass:"table-content"},[n("ul",{staticClass:"stack-content"},[e._l(e.nodeStackContent,(function(t,i){return n("li",{directives:[{name:"show",rawName:"v-show",value:e.nodeStackContent.length,expression:"nodeStackContent.length"}],key:i,staticClass:"stack-item",class:{"sub-stack":t.has_substack}},[e._v(" "+e._s(t.file_path?t.file_path+":"+t.line_no:"")+" "),t.file_path?n("br"):e._e(),e._v(" "+e._s(t.code_line)+" "),n("div",{staticClass:"operator-btns"},[t.file_path?n("el-button",{attrs:{type:"text"},on:{click:function(i){return e.stackOperator(t)}}},[e._v(e._s(e.$t("debugger.search")))]):e._e()],1)])})),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.nodeStackContent.length,expression:"!nodeStackContent.length"}],staticClass:"noData-text"},[e._v(" "+e._s(e.$t("public.noData"))+" ")])],2)])])],1)],1)])]),e.tensorCompareFlag?n("div",{staticClass:"deb-con"},[n("debugger-tensor",{ref:"deb-tensor",attrs:{row:e.curRowObj,formateWatchpointParams:e.formateWatchpointParams},on:{close:e.closeTensor}})],1):e._e(),n("el-dialog",{staticClass:"creat-watch-point-dialog",attrs:{title:e.$t("debugger.createWP"),visible:e.createWPDialogVisible,"show-close":!1,"close-on-click-modal":!1,"modal-append-to-body":!1,width:"930px"},on:{"update:visible":function(t){e.createWPDialogVisible=t}}},[n("div",{staticClass:"conditions-container"},[e._l(e.createWatchPointArr,(function(t,i){return n("div",{key:i,staticClass:"condition-item"},[n("el-select",{staticClass:"collection",on:{change:function(i){return e.collectionChange(t)}},model:{value:t.collection.selectedId,callback:function(i){e.$set(t.collection,"selectedId",i)},expression:"item.collection.selectedId"}},e._l(e.conditionCollections,(function(t){return n("el-option",{key:t.id,class:{"deb-indent":"tensor_condition_collection"!=t.id},attrs:{label:e.transCondition(t.id),value:t.id}})})),1),n("el-select",{staticClass:"condition",on:{change:function(i){return e.conditionChange(t)}},model:{value:t.condition.selectedId,callback:function(i){e.$set(t.condition,"selectedId",i)},expression:"item.condition.selectedId"}},e._l(t.condition.options,(function(t){return n("el-option",{key:t.id,attrs:{label:e.transCondition(t.id),value:t.id}})})),1),t.param.options.length?n("el-select",{staticClass:"param",on:{change:function(i){return e.paramChange(t)}},model:{value:t.param.name,callback:function(i){e.$set(t.param,"name",i)},expression:"item.param.name"}},e._l(t.param.options,(function(t){return n("el-option",{directives:[{name:"show",rawName:"v-show",value:"SUPPORT_PARAM"!==t.paran_type,expression:"i.paran_type !== 'SUPPORT_PARAM'"}],key:t.name,attrs:{label:e.transCondition(t.name),value:t.name}})})),1):e._e(),n("el-tooltip",{staticClass:"item",attrs:{effect:"light",content:e.$t("debugger.paramValueTip",{value:/^(\-|\+)?\d+(\.\d+)?$/.test(t.param.value)?Number(t.param.value):""}),placement:"top"}},[n("el-input",{directives:[{name:"show",rawName:"v-show",value:t.param.options.length&&"BOOL"!==t.param.type,expression:"item.param.options.length &&\n                              item.param.type !== 'BOOL'"}],staticClass:"param-value",attrs:{placeholder:e.$t("scalar.placeHolderNumber")},on:{input:function(i){return e.validateParam(t)}},model:{value:t.param.value,callback:function(i){e.$set(t.param,"value",i)},expression:"item.param.value"}})],1),t.param.options.length&&"BOOL"===t.param.type?n("el-select",{staticClass:"param-value",model:{value:t.param.value,callback:function(i){e.$set(t.param,"value",i)},expression:"item.param.value"}},[n("el-option",{key:!0,attrs:{label:"true",value:!0}}),n("el-option",{key:!1,attrs:{label:"false",value:!1}})],1):e._e(),n("div",{directives:[{name:"show",rawName:"v-show",value:e.percentParams.includes(t.param.name),expression:"percentParams.includes(item.param.name)"}],staticClass:"percent-sign"},[e._v("%")]),t.compositeParams.selections.length?n("div",{staticClass:"inclusive-param"},e._l(t.compositeParams.selections,(function(i,a){return n("div",{key:a,staticClass:"item"},[e._v(" "+e._s(e.transCondition(i.name))+" "),n("el-tooltip",{staticClass:"item",attrs:{effect:"light",content:e.$t("debugger.paramValueTip",{value:/^(\-|\+)?\d+(\.\d+)?$/.test(t.compositeParams.selections[a].value)?Number(t.compositeParams.selections[a].value):""}),placement:"top"}},[n("el-input",{directives:[{name:"show",rawName:"v-show",value:"BOOL"!==i.type,expression:"i.type !== 'BOOL'"}],staticClass:"param-value",attrs:{placeholder:e.$t("scalar.placeHolderNumber")},on:{input:function(i){return e.validateParam(t)}},model:{value:t.compositeParams.selections[a].value,callback:function(i){e.$set(t.compositeParams.selections[a],"value",i)},expression:"item.compositeParams.selections[index].value"}})],1),"BOOL"===i.type?n("el-select",{staticClass:"param-value",model:{value:i.value,callback:function(t){e.$set(i,"value",t)},expression:"i.value"}},[n("el-option",{key:!0,attrs:{label:"true",value:!0}}),n("el-option",{key:!1,attrs:{label:"false",value:!1}})],1):e._e()],1)})),0):e._e()],1)})),n("div",{staticClass:"watch-point-tip"},[n("i",{staticClass:"el-icon-warning"}),e._v(" "+e._s(e.$t("debugger.createWpTip.explain"))+" "),n("div",[e._v(e._s(e.$t("debugger.createWpTip.example")))])])],2),n("span",{staticClass:"dialog-footer",attrs:{slot:"footer"},slot:"footer"},[n("div",{directives:[{name:"show",rawName:"v-show",value:!e.validPram,expression:"!validPram"}],staticClass:"error-msg"},[n("i",{staticClass:"el-icon-warning"}),e._v(" "+e._s(e.$t("public.notice")+e.$t("symbols.colon")+e.paramErrorMsg)+" ")]),e.createWPDialogVisible&&"operator_overflow"===e.createWatchPointArr[0].condition.selectedId?n("div",{staticClass:"error-msg"},[n("i",{staticClass:"el-icon-warning"}),e._v(" "+e._s(e.$t("public.notice")+e.$t("symbols.colon")+e.$t("debugger.paramErrorMsg.watchOverflow"))+" ")]):e._e(),n("el-button",{staticClass:"custom-btn green",attrs:{type:"primary",size:"mini",disabled:!e.validPram},on:{click:function(t){return e.createWatchPoint(!0)}}},[e._v(e._s(e.$t("public.sure")))]),n("el-button",{staticClass:"custom-btn",attrs:{type:"primary",size:"mini"},on:{click:function(t){return e.createWatchPoint(!1)}}},[e._v(" "+e._s(e.$t("public.cancel"))+" ")])],1)]),n("el-dialog",{staticClass:"pendingTips",attrs:{title:e.$t("public.notice"),visible:e.dialogVisible,"show-close":!1,"close-on-click-modal":!1,"modal-append-to-body":!1,width:"420px"},on:{"update:visible":function(t){e.dialogVisible=t}}},[n("span",{staticClass:"dialog-icon"},[n("span",{staticClass:"el-icon-warning"})]),e.noOfflineGraphName?n("span",{staticClass:"dialog-content"},[e._v(e._s(e.$t("debugger.noOfflineGraphName")))]):e.noOfflineGraph?n("span",{staticClass:"dialog-content"},[e._v(e._s(e.$t("debugger.noOfflineGraphData")))]):e.nodeDataIsLarge?n("span",{staticClass:"dialog-content"},[e._v(e._s(e.$t("debugger.nodeIsLargeTip",{maxGraphNodeSize:e.maxGraphNodeSize})))]):e.initFail?n("span",{staticClass:"dialog-content"},[e._v(e._s(e.$t("debugger.debuggerError")))]):n("span",{staticClass:"dialog-content"},[e._v(e._s("offline-debugger"==e.gridStyleKey?e.$t("debugger.pendingTipsOff"):e.$t("debugger.pendingTips")))]),n("span",{staticClass:"dialog-footer",attrs:{slot:"footer"},slot:"footer"},[n("el-button",{staticClass:"custom-btn green",attrs:{type:"primary",size:"mini"},on:{click:function(t){return e.toSummeryList()}}},[e._v(e._s(e.$t("debugger.toSummeryList")))])],1)]),n("el-dialog",{staticClass:"pendingTips",attrs:{title:e.$t("debugger.recommendTip"),visible:e.recommendWatchPointDialog,"show-close":!1,"close-on-click-modal":!1,"modal-append-to-body":!1,width:"420px"},on:{"update:visible":function(t){e.recommendWatchPointDialog=t}}},[n("span",{staticClass:"dialog-icon"},[n("span",{staticClass:"el-icon-warning"})]),n("span",{staticClass:"dialog-content"},[e._v(e._s(e.$t("debugger.recommendDetail")))]),n("span",{staticClass:"dialog-footer",attrs:{slot:"footer"},slot:"footer"},[n("el-button",{staticClass:"custom-btn green",attrs:{type:"primary",size:"mini"},on:{click:function(t){return e.initRecommendWatchPoints(!0)}}},[e._v(" "+e._s(e.$t("debugger.use"))+" ")]),n("el-button",{staticClass:"custom-btn",attrs:{type:"primary",size:"mini"},on:{click:function(t){return e.initRecommendWatchPoints(!1)}}},[e._v(" "+e._s(e.$t("debugger.notUse"))+" ")])],1)]),n("el-dialog",{staticClass:"pendingTips",attrs:{title:e.$t("public.notice"),visible:e.conflictFlag,"show-close":!1,"close-on-click-modal":!1,"modal-append-to-body":!1,width:"420px"},on:{"update:visible":function(t){e.conflictFlag=t}}},[n("span",{staticClass:"dialog-icon"},[n("span",{staticClass:"el-icon-warning"})]),n("span",{staticClass:"dialog-content"},[e._v(" "+e._s(e.$t("debugger.versionConflictTip",{msv:e.debuggerVersion.ms,miv:e.debuggerVersion.mi}))+" ")]),n("span",{staticClass:"dialog-footer",attrs:{slot:"footer"},slot:"footer"},[n("el-button",{staticClass:"custom-btn green",attrs:{type:"primary",size:"mini"},on:{click:function(t){e.trainId?e.deleteSession():e.control(2)}}},[e._v(e._s(e.$t("public.sure")))])],1)])],1)},s=[],r=i("5530"),l=i("2909"),c=(i("99af"),i("4de4"),i("7db0"),i("c740"),i("caad"),i("e260"),i("a15b"),i("d81d"),i("fb6a"),i("b0c0"),i("e9c4"),i("b64b"),i("d3b7"),i("ac1f"),i("00b4"),i("6062"),i("8a79"),i("2532"),i("3ca3"),i("5319"),i("1276"),i("159b"),i("ddb0"),i("5698")),h=(i("6d21"),i("3778")),d=i("8d38"),u=i("3835"),p=(i("a434"),i("a9e3"),i("b680"),i("25f0"),i("841c"),i("498a"),i("f756")),f={select:c["select"],selectAll:c["selectAll"],zoom:c["zoom"],dispatch:c["dispatch"]},g={data:function(){return{conditionRulesMap:this.$t("debugger.tensorTuningRule"),downloadedTensor:{},maxFileSize:[2,"GB"],maxSupportNodesNumber:1e3,maxGraphNodeSize:1e5}},methods:{editStep:function(){this.metadata.state!==this.state.running&&this.metadata.state!==this.state.sending&&(this.isShowInp=!0,this.newStep=this.metadata.step)},newStepChange:function(e){""!==e&&(e=e.replace(/[^0-9]+/g,""),Number(e)<=this.metadata.total_step_num?this.newStep=Number(e):this.newStep=this.metadata.total_step_num)},saveStepValue:function(){var e=this;if(this.isShowInp=!1,""!==this.newStep&&this.newStep!==this.metadata.step){this.metadata.step=this.newStep;var t={mode:"reset",level:"step",steps:this.metadata.step,graph_name:this.graphFiles.value,rank_id:this.logicCard.value};this.graphFiles.value===this.$t("debugger.all")&&delete t.graph_name,h["a"].control(t,this.sessionId).then((function(t){var i;t&&t.data&&t.data.metadata&&void 0!==t.data.metadata.enable_recheck&&(e.enableRecheck=t.data.metadata.enable_recheck),null!==(i=t.data.metadata)&&void 0!==i&&i.state&&(e.metadata.state=t.data.metadata.state),e.queryTensorHistory(),"hit"===e.radio1&&e.searchWatchpointHits(!0)}),(function(t){e.showErrorMsg(t)}))}},getSession:function(){var e=this,t={dump_dir:null,session_type:"ONLINE"};h["a"].getSession(t).then((function(t){t&&(e.sessionId=t.data,e.retrieveAll(),e.queryStacks())}))},deleteSession:function(){var e=this;h["a"].deleteSession(this.sessionId).then((function(t){e.$router.push({path:"/summary-manage"})}))},handleCurrentChange:function(e){this.pagination.currentPage=e,this.searchWatchpointHits(!1)},showOrigin:function(){this.loadOriginalTree(),this.queryWatchPoints()},initCondition:function(){var e=this;this.metadata.state!==this.state.running&&this.metadata.state!==this.state.sending&&h["a"].queryConditions(this.sessionId).then((function(t){t&&t.data&&(e.conditionCollections=t.data,e.addWatchPoint())}))},transCondition:function(e){return e?(t=this.conditionRulesMap[e]?this.conditionRulesMap[e]:e.endsWith("_lt")?e.replace(/_lt$/," <"):e.endsWith("_gt")?e.replace(/_gt$/," >"):e.endsWith("_ge")?e.replace(/_ge$/," >="):e,t.includes("max_min")&&(t=t.replace("max_min","max-min")),t):"";var t},collapseBtnClick:function(){this.leftShow=!this.leftShow,this.initSvgSize()},stepChange:function(){if(""!==this.step){var e=this.metadata.total_step_num;this.step=this.step.toString().replace(/[^\.\d]/g,"").replace(/\./g,""),this.step=Number(this.step),0===this.step&&(this.step=1),this.step>=e&&(this.step=e)}},getCurrentNodeInfo:function(){var e=this;this.loadingInstance=this.$loading(this.loadingOption);var t=this.currentNodeName,i={mode:"node",params:{watch_point_id:this.curWatchPointId?this.curWatchPointId:0,name:t,single_node:!0,node_type:"leaf"}};this.graphFiles.value===this.$t("debugger.all")?(t="".concat(this.metadata.graph_name,"/").concat(this.currentNodeName),i.params.name=t):(this.metadata.graph_name!==this.graphFiles.value&&(this.graphFiles.value=this.metadata.graph_name,this.isCurrentGraph=!1),i.params.graph_name=this.graphFiles.value),h["a"].retrieve(i,this.sessionId).then((function(i){if(i.data&&(i.data.metadata&&e.dealMetadata(i.data.metadata),i.data.graph)){var n=i.data.graph;n.nodes&&!e.isCurrentGraph?(e.node.childNodes=[],e.origialTree=n.nodes.map((function(e){return Object(r["a"])(Object(r["a"])({label:e.name.split("/").pop(),leaf:"name_scope"!==e.type&&"aggregation_scope"!==e.type},e),{},{showCheckbox:-1!==e.watched})})),e.resolve(e.origialTree),e.node.childNodes.forEach((function(t){t.data.watched===e.checkboxStatus.checked&&(t.checked=!0),t.data.watched===e.checkboxStatus.indeterminate&&(t.indeterminate=!0)})),e.isCurrentGraph=!0,e.firstFloorNodes=[],e.allGraphData={},f.select("#graph svg").remove(),e.selectedNode.name="",e.packageDataToObject("",!0,JSON.parse(JSON.stringify(n.nodes))),e.querySingleNode(JSON.parse(JSON.stringify(n)),t,!0)):e.querySingleNode(JSON.parse(JSON.stringify(n)),t,!0),n.children&&(e.dealTreeData(n.children,t),e.defaultCheckedArr=e.$refs.tree.getCheckedKeys())}e.loadingInstance.close()}),(function(t){e.showErrorMsg(t),e.loadingInstance.close()}))},getNextNodeInfo:function(){var e=this;this.loadingInstance=this.$loading(this.loadingOption),this.pagination.currentPage=1,this.watchPointHits=[],this.pagination.total=0;var t={mode:"continue",level:"node",name:"",graph_name:this.graphFiles.value,rank_id:this.logicCard.value};this.graphFiles.value===this.$t("debugger.all")&&delete t.graph_name,h["a"].control(t,this.sessionId).then((function(e){}),(function(t){e.showErrorMsg(t)}))},getNodeByBfs:function(e){var t=this,i=this.$refs.tree.getCurrentNode(),n=this.$refs.tree.getCurrentKey();(i&&("name_scope"===i.type||"aggregation_scope"===i.type)||null===this.curLeafNodeName)&&(n=this.curLeafNodeName);var a={ascend:e,name:n,watch_point_id:this.curWatchPointId?this.curWatchPointId:0,graph_name:this.graphFiles.value};this.graphFiles.value===this.$t("debugger.all")&&delete a.graph_name,h["a"].retrieveNodeByBfs(a).then((function(i){if(i.data&&i.data.graph&&i.data.name){t.retrieveTensorHistory({name:i.data.name});var a=i.data.graph;t.curLeafNodeName=i.data.name,t.nodeName=i.data.name,a.children&&(t.dealTreeData(a.children,n),t.defaultCheckedArr=t.$refs.tree.getCheckedKeys()),t.querySingleNode(JSON.parse(JSON.stringify(a)),i.data.name)}else e?t.$message.success(t.$t("debugger.nextNodeTip")):t.$message.success(t.$t("debugger.previousNodeTip"))}),(function(e){t.showErrorMsg(e)}))},terminate:function(){var e=this;this.$confirm(this.$t("debugger.ternimateConfirm"),this.$t("public.notice"),{confirmButtonText:this.$t("public.sure"),cancelButtonText:this.$t("public.cancel"),type:"warning"}).then((function(){e.trainId?e.deleteSession():e.control(2)}),(function(t){e.showErrorMsg(t)}))},tableRowClassName:function(e){var t=e.row;return t.is_hit?"success-row":""},objectSpanMethod:function(e){e.row,e.column;var t=e.rowIndex,i=e.columnIndex,n=0,a=0;if("tensor"===this.tabs.activeName?(n=this.inputLength,a=this.outputLength):(n=this.selectedNode.inputNum,a=this.selectedNode.outputNum),0===i&&a>0)return 0===t?{rowspan:a,colspan:1}:t>0&&t<a?{rowspan:0,colspan:0}:t===a?{rowspan:n,colspan:1}:{rowspan:0,colspan:0}},discountHeaderStyle:function(e){e.row,e.column;var t=e.rowIndex;e.columnIndex;if(1===t)return{display:"none"}},handleNodeClick:function(e){if(this.isIntoView=!1,this.selectedNode.name=e.name,this.treeFlag&&this.allGraphData[e.name])this.querySingleNode({},e.name,!0);else if(this.graphFiles.value===this.$t("debugger.all"))if(e.name.includes("/")){var t=e.name.split("/")[0];this.queryAllTreeData(e.name.replace("".concat(t,"/"),""),!0,t,!0)}else this.queryAllTreeData(e.name,!0,e.name,!0);else this.queryAllTreeData(e.name,!0,this.graphFiles.value,!0)},retrieveTensorHistory:function(e,t){var i=this,n={name:e.name,rank_id:this.logicCard.value};this.graphFiles.value===this.$t("debugger.all")?n.name="".concat(t,"/").concat(e.name):n.graph_name=t,h["a"].retrieveTensorHistory(n,this.sessionId).then((function(t){if(t.data&&t.data.metadata&&i.dealMetadata(t.data.metadata),e.name===i.nodeName){if(t.data&&t.data.tensor_history.length>i.maxSupportNodesNumber)return void i.$message.error(i.$t("graph.tooManyNodes"));t.data&&t.data.tensor_history?(i.tableData=t.data.tensor_history,i.dealTableData(i.tableData)):i.tableData=[]}}),(function(e){i.showErrorMsg(e)}))},dealTableData:function(e){var t=this,i=e.filter((function(e){return"output"===e.type})),n=e.filter((function(e){return"input"===e.type}));this.outputLength=i.length,this.inputLength=n.length,e.splice.apply(e,[0,e.length].concat(Object(l["a"])(i.concat(n)))),e.forEach((function(e){if(Array.isArray(e.shape)?e.shape="[".concat(e.shape.toString(),"]"):void 0!==e.shape&&(e.shape=e.shape+""),"click to view"===e.value||(Array.isArray(e.value)?e.value="[".concat(e.value.toString(),"]"):void 0!==e.value&&(e.value=e.value+"")),void 0!==e.dtype&&(e.dtype=e.dtype+""),e.bytes){var i=t.fileSizeConversion(e.bytes),n=Object(u["a"])(i,2),a=n[0],o=n[1];e.oversized=o===t.maxFileSize[1]&&a>t.maxFileSize[0]}}))},dealMetadata:function(e){e.graph_name&&e.graph_name!==this.metadata.graph_name&&this.graphFiles.value!==this.$t("debugger.all")&&(this.graphFiles.value=e.graph_name,this.isCurrentGraph=!1),e.graph_name&&void 0!==e.step&&e.step!==this.metadata.step&&e.graph_name===this.metadata.graph_name&&this.graphFiles.value!==e.graph_name&&this.graphFiles.value!==this.$t("debugger.all")&&(this.graphFiles.value=e.graph_name,this.isCurrentGraph=!1,this.queryGraphByFile()),this.metadata.pos=e.pos,void 0!==e.enable_recheck&&(this.enableRecheck=e.enable_recheck);var t=this.metadata.state;if(e.state&&(this.metadata.state=e.state),e.debugger_version&&(this.debuggerVersion=e.debugger_version),void 0!==e.node_name&&void 0!==e.step){var i=e.node_name;if(i!==this.currentNodeName&&""!==i||this.metadata.step!==e.step||this.metadata.state===this.state.waiting&&(t===this.state.sending||t===this.state.running)){i&&(this.metadata.state!==this.state.running&&(this.nodeName=i),this.currentNodeName=i),this.metadata.step=e.step;var n=this.graphFiles.value===this.$t("debugger.all")?"":this.graphFiles.value;this.graphFiles.value===this.$t("debugger.all")&&this.selectedNode.name&&(n=this.selectedNode.name.split("/")[0]),e.graph_name&&(this.metadata.graph_name=e.graph_name,n=e.graph_name),i&&this.metadata.state!==this.state.running?this.selectedNode.name&&i===this.selectedNode.name?this.queryTensorHistory():this.queryAllTreeData(i,!0,n):this.selectedNode.name&&this.queryTensorHistory()}else this.loadingInstance.close()}e.step&&e.step>this.metadata.step&&(this.metadata.step=e.step)},queryTensorHistory:function(){if(this.selectedNode.name){var e=this.selectedNode.name.split("^"),t=e[0].replace("_unfold","");if(!t||!this.allGraphData[t])return;var i=this.allGraphData[t].type,n=["name_scope","aggregation_scope"];if(!this.selectedNode.name.includes("more...")&&!n.includes(i)){var a=e[0].replace("_unfold","");this.graphFiles.value===this.$t("debugger.all")?this.retrieveTensorHistory({name:a.replace("".concat(a.split("/")[0],"/"),"")},a.split("/")[0]):this.retrieveTensorHistory({name:a},this.graphFiles.value)}}},pollData:function(){var e=this,t={pos:this.metadata.pos,graph_name:this.graphFiles.value,rank_id:this.logicCard.value};this.graphFiles.value===this.$t("debugger.all")&&delete t.graph_name,h["a"].pollData(t,this.sessionId).then((function(t){if(t.data){var i;if(t.data.metadata){if(t.data.metadata.state===e.state.node_too_large)return e.dialogVisible=!0,e.nodeDataIsLarge=!0,void(e.maxGraphNodeSize=t.data.metadata.max_graph_node_size);e.dealMetadata(t.data.metadata)}if(null!==(i=t.data.graph)&&void 0!==i&&null!==(i=i.graph_names)&&void 0!==i&&i.length&&e.graphFiles.options.length){var n=t.data.graph.graph_names.filter((function(t){return!e.graphFiles.options.includes(t)}));n.length&&(e.$message.success(e.$t("debugger.newGraphName",{graphNames:n})),setTimeout((function(){location.reload()}),3e3))}var a=null;e.selectedNode.name&&(a=e.selectedNode.name.replace("_unfold",""));var o=e.graphFiles.value;if(t.data.receive_tensor&&t.data.metadata&&t.data.metadata.step>=e.metadata.step&&t.data.receive_tensor.node_name===a&&(e.graphFiles.value===e.$t("debugger.all")&&(o=a.split("/")[0],a=a.replace("".concat(o,"/"),"")),e.retrieveTensorHistory({name:a},o)),t.data.receive_watchpoint_hits&&(e.radio1="hit",e.pagination.currentPage=1,e.watchPointHits=[],e.pagination.total=0,e.searchWatchpointHits(!0)),t.data.receive_tensor&&t.data.receive_tensor.graph_name&&t.data.receive_tensor.tensor_name&&e.tensorCompareFlag){var s=e.$refs["deb-tensor"];s&&("stats"===t.data.receive_tensor.level?s.updateGraphData(t.data.receive_tensor.graph_name,t.data.receive_tensor.tensor_name):void 0===t.data.receive_tensor.level&&s.tabChange(s.gridType))}t.data.tensor_file&&(e.downloadedTensor.name===t.data.node_name?e.downloadTensor():e.downloadedTensor.name=t.data.node_name),e.pollData()}}),(function(t){(!t||t&&"routeJump"!==t.message)&&(e.initFail=!0,e.dialogVisible=!0)}))},control:function(e){var t=this;3!==e&&(this.pagination.currentPage=1,this.watchPointHits=[],this.pagination.total=0);var i={};if(0===e){if(!this.step)return;i.mode="continue",i.steps=parseInt(this.step)}else 1===e?(i.mode="continue",i.steps=-1):2===e?i.mode="terminate":3===e&&(i.mode="pause");h["a"].control(i,this.sessionId).then((function(e){e.data&&e.data.metadata&&(setTimeout((function(){var e="";e=t.metadata.state===t.state.sending?t.$t("debugger.stateMsg.sending"):t.metadata.state===t.state.running?t.$t("debugger.stateMsg.running"):"".concat(t.$t("debugger.backstageStatus")).concat(t.metadata.state),t.$message.success(e)}),500),t.metadata.state=e.data.metadata.state,void 0!==e.data.metadata.enable_recheck&&(t.enableRecheck=e.data.metadata.enable_recheck))}),(function(e){t.showErrorMsg(e)}))},loadOriginalTree:function(){this.node.childNodes=[],this.curWatchPointId=null,this.defaultCheckedArr=[],this.resolve(JSON.parse(JSON.stringify(this.origialTree))),this.resetGraph(),this.tabledata=[]},recheckWatchpoint:function(){var e=this;this.enableRecheck&&h["a"].recheckWatchPoints(this.sessionId).then((function(t){t&&t.data&&t.data.metadata&&(void 0!==t.data.metadata.enable_recheck&&(e.enableRecheck=t.data.metadata.enable_recheck,e.pagination.currentPage=1,e.watchPointHits=[],e.pagination.total=0),t.data.metadata.state&&(e.metadata.state=t.data.metadata.state)),e.$message.success(e.$t("debugger.recheckSuccess"))}),(function(e){}))},addWatchPoint:function(){this.paramErrorMsg=this.$t("debugger.paramErrorMsg.errorType"),this.createWatchPointArr=[],this.createWatchPointArr.push({collection:{selectedId:this.conditionCollections[0].id},condition:{selectedId:"",options:[]},param:{options:[],name:"",value:"",type:""},compositeParams:{options:[],selections:[]}}),this.collectionChange(this.createWatchPointArr[0]),this.createWPDialogVisible=!0},deleteWatchpoint:function(e){var t=this;if(this.watchPointArr.length&&this.metadata.state!==this.state.running&&this.metadata.state!==this.state.sending)if(e&&e.id||!e){var i=e?this.$t("debugger.deleteWatchpointConfirm"):this.$t("debugger.clearWatchpointConfirm");this.$confirm(i,this.$t("public.notice"),{confirmButtonText:this.$t("public.sure"),cancelButtonText:this.$t("public.cancel"),type:"warning"}).then((function(){var i={watch_point_id:e?e.id:null};h["a"].deleteWatchpoint(i,t.sessionId).then((function(i){e||(t.curWatchPointId=null,t.watchPointArr=[]),t.loadOriginalTree(),t.queryWatchPoints(),t.$message.success(t.$t("debugger.successDeleteWP")),i&&i.data&&i.data.metadata&&void 0!==i.data.metadata.enable_recheck&&(t.enableRecheck=i.data.metadata.enable_recheck),t.curWatchPointId=null}),(function(e){t.showErrorMsg(e)}))}))}else this.curWatchPointId=null},validateParam:function(e){var t=this,i=/^(\-|\+)?\d+(\.\d+)?$/;if(this.validPram=i.test(e.param.value),e.compositeParams.selections.forEach((function(e){t.validPram=t.validPram&&i.test(e.value)})),this.validPram){this.paramErrorMsg="";var n=parseFloat(e.param.value),a=["abs_mean_gt","abs_mean_lt","rtol","abs_mean_update_ratio_gt","abs_mean_update_ratio_lt"];a.includes(e.param.name)&&n<0&&(this.validPram=!1,this.paramErrorMsg=this.$t("debugger.paramErrorMsg.nonnegative"));var o=["max_min_lt","max_min_gt"];if(o.includes(e.param.name)&&n<=0&&(this.validPram=!1,this.paramErrorMsg=this.$t("debugger.paramErrorMsg.allPositive")),this.percentParams.includes(e.param.name)){var s={min:0,max:100};(n<s.min||n>s.max)&&(this.validPram=!1,this.paramErrorMsg=this.$t("debugger.paramErrorMsg.percentError"))}if(this.validPram&&e.compositeParams.selections.length){var r=["range_start_inclusive","range_end_inclusive"],l=e.compositeParams.selections.filter((function(e){return e.name===r[0]})),c=e.compositeParams.selections.filter((function(e){return e.name===r[1]}));if(l.length&&c.length){var h=parseFloat(l[0].value),d=parseFloat(c[0].value);h>d&&(this.validPram=!1,this.paramErrorMsg=this.$t("debugger.paramErrorMsg.rangeError"))}}}else this.paramErrorMsg=this.$t("debugger.paramErrorMsg.errorType");this.$forceUpdate()},createWatchPoint:function(e){var t=this;if(e){var i=this.createWatchPointArr[0],n={condition:{id:i.condition.selectedId,params:[]},watch_nodes:[],graph_name:this.graphFiles.value,rank_id:this.logicCard.value};this.graphFiles.value===this.$t("debugger.all")&&delete n.graph_name,i.param.options.length&&(n.condition.params=[{name:i.param.name,value:"BOOL"===i.param.type?Boolean(i.param.value):Number(i.param.value)}]),i.compositeParams.selections.length&&i.compositeParams.selections.forEach((function(e){n.condition.params.push({name:e.name,value:"BOOL"===e.type?Boolean(e.value):Number(e.value)})})),h["a"].createWatchpoint(n,this.sessionId).then((function(e){t.createWatchPointArr=[],t.createWPDialogVisible=!1,t.$message.success(t.$t("debugger.successCreateWP")),e&&e.data&&e.data.metadata&&void 0!==e.data.metadata.enable_recheck&&(t.enableRecheck=e.data.metadata.enable_recheck),t.queryWatchPoints(!0)}),(function(e){t.loadOriginalTree(),t.queryWatchPoints(),t.showErrorMsg(e)}))}else this.createWatchPointArr=[],this.createWPDialogVisible=!1},collectionChange:function(e){var t=this.conditionCollections.filter((function(t){return t.id===e.collection.selectedId}))[0];t.conditions&&t.conditions.length?(e.condition.options=t.conditions,e.condition.selectedId=e.condition.options[0].id,this.conditionChange(e)):(e.condition.options=[],e.condition.selectedId="",e.param.options=[],e.param.name="",e.param.type="",e.param.value="",this.validPram=!1,this.paramErrorMsg=this.$t("debugger.paramErrorMsg.errorType"))},conditionChange:function(e){var t=e.condition.options.filter((function(t){return t.id===e.condition.selectedId}))[0];t.parameters&&t.parameters.length?(e.param.options=t.parameters.filter((function(e){return"SUPPORT_PARAM"!==e.param_type})),e.compositeParams.options=t.parameters.filter((function(e){return"SUPPORT_PARAM"===e.param_type})),e.param.name=e.param.options[0].name,this.paramChange(e)):(e.param.options=[],e.param.name="",e.param.type="",e.param.value="",this.validPram=!0,e.compositeParams.options=[],e.compositeParams.selections=[])},paramChange:function(e){var t=e.param.options.filter((function(t){return t.name===e.param.name}))[0];t.required_params&&t.required_params.length?(e.compositeParams.selections=e.compositeParams.options.filter((function(e){return t.required_params.includes(e.name)})),e.compositeParams.selections.forEach((function(e){e.value="BOOL"===e.type||""}))):e.compositeParams.selections=[],e.param.type=t.type,e.param.value="",this.validPram=!1,this.paramErrorMsg=this.$t("debugger.paramErrorMsg.errorType"),"BOOL"===e.param.type&&(e.param.value=!0,this.validPram=!0)},check:function(e){var t=this,i=this.$refs.tree.getNode(e.name),n=i.checked;i.data.watched=n?this.checkboxStatus.checked:this.checkboxStatus.unchecked,this.treeFlag&&i.childNodes&&this.dealCheckPro(i.childNodes,i.indeterminate||n),this.curWatchPointId&&(this.$refs.tree.getCheckedKeys().forEach((function(e){var i=t.$refs.tree.getNode(e);i.data.watched===t.checkboxStatus.noCheckbox&&(i.checked=!1)})),this.$nextTick((function(){i.indeterminate&&(i.checked=!0,i.indeterminate=!1),n&&t.dealParentNode(i);var a=t.$refs.tree.getCheckedKeys(),o=[];if(t.defaultCheckedArr.length!==a.length){t.defaultCheckedArr.length>a.length?o.push(e.name):a.forEach((function(e){-1===t.defaultCheckedArr.indexOf(e)&&o.push(e)}));var s={watch_point_id:t.curWatchPointId?t.curWatchPointId:0,watch_nodes:o,mode:i.indeterminate||n?1:0,graph_name:t.graphFiles.value,rank_id:t.logicCard.value};t.graphFiles.value===t.$t("debugger.all")&&delete s.graph_name,h["a"].updateWatchpoint(s,t.sessionId).then((function(e){t.defaultCheckedArr=a,e&&e.data&&e.data.metadata&&void 0!==e.data.metadata.enable_recheck&&(t.enableRecheck=e.data.metadata.enable_recheck)}),(function(e){t.showErrorMsg(e)}))}})))},dealParentNode:function(e){var t=e.parent;t&&!t.childNodes.filter((function(e){return-1!==e.data.watched})).find((function(e){return!1===e.checked}))&&(t.checked=!0,t.indeterminate=!1,this.dealParentNode(t))},searchCheck:function(e){var t=this,i=this.$refs.searchTree.getNode(e.name),n=i.checked;i.data.watched=n?this.checkboxStatus.checked:this.checkboxStatus.unchecked,i.childNodes&&this.dealCheckPro(i.childNodes,i.indeterminate||n),this.$nextTick((function(){i.indeterminate&&(i.checked=!0,i.indeterminate=!1),n&&t.dealParentNode(i);var a=t.$refs.searchTree.getCheckedKeys(),o=[];if(t.searchCheckedArr.length!==a.length){t.searchCheckedArr.length>a.length?o.push(e.name):a.forEach((function(e){-1===t.searchCheckedArr.indexOf(e)&&o.push(e)}));var s={watch_point_id:t.curWatchPointId?t.curWatchPointId:0,watch_nodes:o,mode:n?1:0,graph_name:t.graphFiles.value,rank_id:t.logicCard.value,search_pattern:{name:t.searchedWord}};t.graphFiles.value===t.$t("debugger.all")&&delete s.graph_name,"all"!==t.nodeTypes.value&&(s.search_pattern.node_category=t.nodeTypes.value),h["a"].updateWatchpoint(s,t.sessionId).then((function(e){t.searchCheckedArr=a,e&&e.data&&e.data.metadata&&void 0!==e.data.metadata.enable_recheck&&(t.enableRecheck=e.data.metadata.enable_recheck)}),(function(e){t.showErrorMsg(e)}))}}))},dealCheckPro:function(e,t){var i=this;e.forEach((function(e){e.indeterminate=!1,-1!==e.data.watched?(e.checked=t,e.data.watched=t?i.checkboxStatus.checked:i.checkboxStatus.unchecked):e.checked=!1,e.childNodes&&i.dealCheckPro(e.childNodes,t)}))},nodeCollapse:function(e,t){t.loaded=!1,t.childNodes=[],this.treeFlag&&this.dealDoubleClick(t.data.name,t.expanded)},filterChange:function(){""===this.searchWord&&"all"===this.nodeTypes.value&&""===this.searchStackContent&&(this.curWatchPointId&&this.queryGraphByWatchpoint(this.curWatchPointId),this.treeFlag=!0,this.$nextTick((function(){setTimeout((function(){var e=document.querySelector(".el-tree-node.is-current.is-focusable");if(e){var t=document.querySelector(".tree-wrap");t.scrollTop=e.offsetTop}}),800)})))},filter:function(){var e=this;if(this.searchWord=this.searchWord.trim(),this.searchStackContent=this.searchStackContent.trim(),this.treeFlag=""===this.searchWord&&"all"===this.nodeTypes.value&&""===this.searchStackContent,this.searchWord||"all"!==this.nodeTypes.value||this.searchStackContent){this.searchedWord=this.searchWord,this.searchedStackContent=this.searchStackContent;var t={name:this.searchWord,watch_point_id:this.curWatchPointId?this.curWatchPointId:0,graph_name:this.graphFiles.value,rank_id:this.logicCard.value,stack_info_key_word:this.searchStackContent};this.graphFiles.value===this.$t("debugger.all")&&delete t.graph_name,"all"!==this.nodeTypes.value&&(t.node_category=this.nodeTypes.value);var i=this.$loading(this.loadingOption);h["a"].search(t,this.sessionId).then((function(t){if(i.close(),t.data&&t.data.nodes){e.searchTreeData=t.data.nodes,e.searchHalfCheckedArr=[],e.searchCheckedArr=[],e.dealSearchResult(e.searchTreeData),e.searchNode.childNodes=[];var n=t.data.nodes.map((function(e){return Object(r["a"])(Object(r["a"])({label:e.name.split("/").pop()},e),{},{showCheckbox:-1!==e.watched})})),a=JSON.parse(JSON.stringify(n));a.forEach((function(e){e.nodes=[]})),e.searchResolve(a),e.searchNode.childNodes.forEach((function(t){t.data.watched===e.checkboxStatus.indeterminate&&(t.indeterminate=!0),t.data.watched===e.checkboxStatus.checked&&(t.checked=!0),t.data.watched===e.checkboxStatus.unchecked&&(t.checked=!1)})),n.forEach((function(t,i){t.nodes&&t.nodes.length&&(t.nodes.forEach((function(e){e.parentName=t.name})),e.dealSearchTreeData(t.nodes))})),e.searchHalfCheckedArr.forEach((function(t){e.$refs.searchTree.getNode(t).indeterminate=!0}))}}),(function(t){i.close(),e.showErrorMsg(t)}))}},dealSearchTreeData:function(e){var t=this;e.forEach((function(e){var i=t.$refs.searchTree.getNode(e.parentName);e.label=e.name.split("/").pop(),e.leaf="name_scope"!==e.type&&"aggregation_scope"!==e.type,e.showCheckbox=-1!==e.watched,t.$refs.searchTree.append(e,i),i.expanded=!0,i.childNodes.forEach((function(e){e.data.watched===t.checkboxStatus.indeterminate&&(e.indeterminate=!0),e.data.watched===t.checkboxStatus.checked&&(e.checked=!0),e.data.watched===t.checkboxStatus.unchecked&&(e.checked=!1)})),e.nodes&&e.nodes.length&&(e.nodes.forEach((function(t){t.parentName=e.name})),t.dealSearchTreeData(e.nodes))}))},dealSearchResult:function(e){var t=this;e.forEach((function(e){e.nodes&&t.dealSearchResult(e.nodes),e.watched===t.checkboxStatus.checked&&t.searchCheckedArr.push(e.name),e.label=e.name.split("/").pop()}))},retrieveAll:function(){var e=this;this.loadingInstance=this.$loading(this.loadingOption);var t={mode:"all"};h["a"].retrieve(t,this.sessionId).then((function(t){if(e.initFail=!1,e.dialogVisible=!1,t.data){if(t.data.metadata&&t.data.metadata.max_graph_node_size&&(e.maxGraphNodeSize=t.data.metadata.max_graph_node_size),t.data.metadata&&t.data.metadata.state===e.state.node_too_large)return e.loadingInstance.close(),e.dialogVisible=!0,void(e.nodeDataIsLarge=!0);if(t.data.graph&&t.data.graph.nodes)e.origialTree=t.data.graph.nodes.map((function(e){return Object(r["a"])({label:e.name.split("/").pop(),leaf:"name_scope"!==e.type&&"aggregation_scope"!==e.type},e)})),e.resolve(e.origialTree),e.dealGraphData(JSON.parse(JSON.stringify(t.data.graph.nodes)));else if(t.data.metadata&&t.data.metadata.state===e.state.waiting)return e.trainId&&(e.noOfflineGraph=!0),e.loadingInstance.close(),void(e.dialogVisible=!0);if(t.data.devices&&t.data.devices.length){if(e.devices=t.data.devices,e.logicCard.value=e.devices[0].rank_id,e.graphFiles.options=JSON.parse(JSON.stringify(e.devices[0].graph_names)),e.graphFiles.options.length>1&&e.graphFiles.options.unshift(e.$t("debugger.all")),e.graphFiles.value=e.graphFiles.options[0],e.trainId&&e.getGraphRuns(),t.data.metadata&&t.data.metadata.state===e.state.waiting&&e.trainId&&!e.graphFiles.value)return e.noOfflineGraphName=!0,void(e.dialogVisible=!0);e.hitWpCondition.graphFile=e.graphFiles.options[0],e.logicCard.options=e.devices.map((function(e){return e.rank_id}))}if(t.data.watch_points&&(e.watchPointArr=t.data.watch_points.map((function(e){return{id:e.id,condition:e.watch_condition.id,params:e.watch_condition.params||[],selected:!1}}))),t.data.metadata){if(t.data.metadata.data_version&&0==t.data.metadata.data_version.state){var i=t.data.metadata.data_version;e.$message({type:"warning",offset:15,center:!0,duration:5e3,message:e.$t("debugger.dataVersionTips",{ms_version:i.ms,mi_version:i.mi})})}t.data.metadata.debugger_version&&(e.debuggerVersion=t.data.metadata.debugger_version),e.metadata=t.data.metadata,e.trainId||(e.metadata.total_step_num=2147483647),t&&t.data&&t.data.metadata&&void 0!==t.data.metadata.enable_recheck&&(e.enableRecheck=t.data.metadata.enable_recheck),e.metadata.backend&&(e.version=e.metadata.backend),t.data.metadata.recommendation_confirmed||e.metadata.state!==e.state.waiting||(e.recommendWatchPointDialog=!0),e.nodeName=e.metadata.node_name,e.currentNodeName=e.nodeName,e.metadata.state!==e.state.pending&&e.metadata.state!==e.state.mismatch||e.loadingInstance.close(),e.pollInit&&(e.pollData(),e.pollInit=!1),e.devices&&e.devices.length&&(e.metadata.ip=e.devices[0].server_ip,e.metadata.device_name=e.devices[0].device_id)}}}),(function(t){e.initFail=!0,e.dialogVisible=!0,e.loadingInstance.close(),e.showErrorMsg(t)}))},loadNode:function(e,t){var i=this;if(0===e.level)e.childNodes=[],this.node||this.resolve||(this.node=e,this.resolve=t),t([]);else if(e.level>=1){this.loadingInstance=this.$loading(this.loadingOption),this.isIntoView=!1;var n=this.$refs.tree.getHalfCheckedKeys(),a={mode:"node",params:{node_type:e.data.type,watch_point_id:this.curWatchPointId?this.curWatchPointId:0,name:e.data.name,graph_name:this.graphFiles.value,rank_id:this.logicCard.value}};this.graphFiles.value===this.$t("debugger.all")&&delete a.params.graph_name,h["a"].retrieve(a,this.sessionId).then((function(a){if(a.data&&a.data.metadata&&i.dealMetadata(a.data.metadata),a.data&&a.data.graph){var o=a.data.graph;if(i.curNodeData=o.nodes.map((function(e){return Object(r["a"])(Object(r["a"])({label:e.name.split("/").pop(),leaf:"name_scope"!==e.type&&"aggregation_scope"!==e.type},e),{},{showCheckbox:-1!==e.watched,disabled:e.disable})})),i.curNodeData.length>i.nodesCountLimit)return i.$message.error(i.$t("graph.tooManyNodes")),i.loadingInstance.close(),void(e.loading=!1);t(i.curNodeData),i.defaultCheckedArr=i.defaultCheckedArr.concat(i.curNodeData.filter((function(e){return e.watched===i.checkboxStatus.checked})).map((function(e){return e.name})));var s=i.curNodeData.filter((function(e){return e.watched===i.checkboxStatus.indeterminate})).map((function(e){return e.name}));e.childNodes.forEach((function(t){-1!==s.indexOf(t.data.name)&&(t.indeterminate=!0,e.indeterminate=!0),t.data.watched===i.checkboxStatus.checked?t.checked=!0:t.data.watched===i.checkboxStatus.unchecked&&(t.checked=!1)})),Object(l["a"])(new Set(n.concat(i.$refs.tree.getHalfCheckedKeys()))).forEach((function(e){i.$refs.tree.getNode(e).indeterminate=!0})),i.selectedNode.name=e.data.name,i.allGraphData[e.data.name]&&!i.allGraphData[e.data.name].isUnfold?i.dealGraphData(JSON.parse(JSON.stringify(o.nodes)),e.data.name):i.selectNode(!0)}else i.selectedNode.name=e.data.name,i.selectNode(!0),t([])}),(function(e){i.showErrorMsg(e),t([])}))}},loadSearchNode:function(e,t){var i=this;if(0===e.level)e.childNodes=[],this.searchNode||this.searchResolve||(this.searchNode=e,this.searchResolve=t);else if(e.level>=1){var n=this.$refs.searchTree.getHalfCheckedKeys();if(e.childNodes&&e.childNodes.length)return e.expanded=!0,e.loaded=!0,void(e.loading=!1);var a={mode:"node",params:{node_type:e.data.type,watch_point_id:this.curWatchPointId?this.curWatchPointId:0,name:e.data.name,graph_name:this.graphFiles.value,rank_id:this.logicCard.value}};this.graphFiles.value===this.$t("debugger.all")&&delete a.params.graph_name,h["a"].retrieve(a,this.sessionId).then((function(a){if(a.data&&a.data.metadata&&i.dealMetadata(a.data.metadata),a.data&&a.data.graph&&a.data.graph.nodes){i.curNodeData=a.data.graph.nodes.map((function(e){return Object(r["a"])(Object(r["a"])({label:e.name.split("/").pop(),leaf:"name_scope"!==e.type&&"aggregation_scope"!==e.type},e),{},{showCheckbox:-1!==e.watched})})),t(i.curNodeData),i.searchCheckedArr=i.searchCheckedArr.concat(i.curNodeData.filter((function(e){return e.watched===i.checkboxStatus.checked})).map((function(e){return e.name})));var o=i.curNodeData.filter((function(e){return e.watched===i.checkboxStatus.indeterminate})).map((function(e){return e.name}));e.childNodes.forEach((function(t){t.data.watched===i.checkboxStatus.checked&&(t.checked=!0),-1!==o.indexOf(t.data.name)&&(t.indeterminate=!0,e.indeterminate=!0)})),Object(l["a"])(new Set(n.concat(i.$refs.searchTree.getHalfCheckedKeys()))).forEach((function(e){i.$refs.searchTree.getNode(e).indeterminate=!0}))}}))}},initRecommendWatchPoints:function(e){var t=this;this.recommendWatchPointDialog=!1;var i={requestBody:{set_recommended:e}};h["a"].setRecommendWatchPoints(i,this.sessionId).then((function(i){i&&i.data&&e&&t.queryWatchPoints(!1)}))},selectWatchPoint:function(e){var t=this;this.curLeafNodeName=null,this.curHalfCheckedKeys=[],this.watchPointArr.forEach((function(i,n){n===e?i.id?(i.selected=!0,t.curWatchPointId=i.id,t.queryGraphByWatchpoint(i.id),""===t.searchWord&&"all"===t.nodeTypes.value&&""===t.searchStackContent||t.filter()):t.loadOriginalTree():i.selected&&(i.selected=!1)}))},queryWatchPoints:function(e){var t=this,i={mode:"watchpoint",graph_name:this.graphFiles.value,rank_id:this.logicCard.value};this.graphFiles.value===this.$t("debugger.all")&&delete i.graph_name,h["a"].retrieve(i,this.sessionId).then((function(i){i.data.watch_points&&(t.watchPointArr=i.data.watch_points.map((function(e){return{id:e.id,condition:e.watch_condition.id,params:e.watch_condition.params||[],selected:!1}})),e&&(t.selectWatchPoint(t.watchPointArr.length-1),t.$nextTick((function(){var e=document.querySelector("#watch-point-list>li:last-child");e&&(e.parentNode.scrollTop=e.offsetTop)}))))}),(function(e){t.showErrorMsg(e)}))},nodeExpandLinkage:function(e,t){var i=this;if(!(e.length>this.nodesCountLimit)){var n=e.map((function(e){return Object(r["a"])(Object(r["a"])({label:e.name.split("/").pop()},e),{},{showCheckbox:-1!==e.watched})})),a=this.$refs.tree.getNode(t);n.forEach((function(e){i.$refs.tree.append(e,t)})),a.childNodes.forEach((function(e){a.checked&&!a.childNodes.find((function(e){return 2!==e.data.watched}))&&-1!==e.data.watched&&(e.checked=!0),e.data.watched===i.checkboxStatus.checked&&(e.checked=!0),e.data.watched===i.checkboxStatus.indeterminate&&(e.indeterminate=!0),"name_scope"!==e.data.type&&"aggregation_scope"!==e.data.type&&(e.isLeaf=!0)})),a.expanded=!0,a.loading=!1,this.$refs.tree.setCurrentKey(t),this.defaultCheckedArr=this.$refs.tree.getCheckedKeys(),this.$nextTick((function(){a.indeterminate&&!a.childNodes.filter((function(e){return-1!==e.data.watched})).find((function(e){return!1===e.checked}))&&(a.indeterminate=!1,a.checked=!0,i.dealParentNode(a)),setTimeout((function(){var e=document.querySelector(".el-tree-node.is-current.is-focusable");if(e){var t=document.querySelector(".tree-wrap");t.scrollTop=e.offsetTop}}),800)}))}},searchWatchpointHits:function(e){var t=this,i=arguments.length>1&&void 0!==arguments[1]&&arguments[1];if("hit"===this.radio1){var n={},a={rank_id:this.logicCard.value};if(this.hitWpCondition.graphFile!==this.$t("debugger.all")&&(a.graph_id=this.hitWpCondition.graphFile),this.hitWpCondition.watchPoint!==this.$t("debugger.all")&&(a.watchpoint_id=this.hitWpCondition.watchPoint),e)if(this.selectedNode.name)if(this.graphFiles.value===this.$t("debugger.all")){var o=this.selectedNode.name.split("/");a.focused_node={node_name:o[1]?this.selectedNode.name.replace("".concat(o[0],"/"),""):o[0],graph_name:o[0]}}else a.focused_node={node_name:this.selectedNode.name,graph_name:this.graphFiles.value};else a.offset=this.pagination.currentPage-1;else a.offset=this.pagination.currentPage-1;a.limit=this.pagination.pageSize,i&&(a.offset=0),n.group_condition=a,h["a"].searchWatchpointHits(n,this.sessionId).then((function(e){e.data.metadata&&t.dealMetadata(e.data.metadata),t.hitsOutdated=e.data.outdated,e.data&&e.data.watch_point_hits?(t.watchPointHits=[],t.pagination.total=e.data.total,t.pagination.currentPage=e.data.offset+1,t.dealWatchpointHits(e.data.watch_point_hits)):a.node_name?t.watchPointHits.length>0?t.watchPointHits.forEach((function(e){e.selected=!1})):t.searchWatchpointHits(!1):(t.pagination.currentPage=1,t.watchPointHits=[],t.pagination.total=0)}),(function(e){t.showErrorMsg(e)}))}else"tree"===this.radio1?this.$nextTick((function(){setTimeout((function(){var e=document.querySelector(".el-tree-node.is-current.is-focusable");if(e){var t=document.querySelector(".tree-wrap");t.scrollTop=e.offsetTop}}),200)})):"stack"===this.radio1&&(this.stacks.list.length||(this.stacks.currentPage=1,this.queryStacks()))},dealWatchpointHits:function(e){var t=this;e&&e.length?(e.forEach((function(e){var i={name:e.node_name,lists:[],selected:!1,id:e.node_name,graph_name:e.graph_name,rank_id:t.logicCard.value};e.tensors&&e.tensors.length&&e.tensors.forEach((function(n){var a="slot: ".concat(n.slot,", ");n.watch_points&&n.watch_points.length&&n.watch_points.forEach((function(n,o){var s="".concat(a).concat(t.$t("debugger.watchPoint")," ").concat(n.id,", "),r=[];n.watch_condition&&(s+=" ".concat(t.transCondition(n.watch_condition.id)),t.formateWatchpointParams(n.watch_condition.params||[]),r=JSON.parse(JSON.stringify(n.watch_condition.params))),i.lists.push({name:s,params:r,id:"".concat(o).concat(e.node_name),tip:n.error_list&&n.error_list.length?n.error_list.map((function(e){return t.$t("debugger.checkTips")[e]})).join("")+t.$t("debugger.checkTips").cannotCheck:""})}))})),t.watchPointHits.push(i)})),this.focusWatchpointHit()):(this.pagination.currentPage=1,this.watchPointHits=[],this.pagination.total=0)},focusWatchpointHit:function(){var e=this;if(this.selectedNode.name){var t=this.selectedNode.name;this.graphFiles.value!==this.$t("debugger.all")&&(t="".concat(this.graphFiles.value,"/").concat(t)),this.expandKeys=[];var i=!1;return this.watchPointHits.forEach((function(n){"".concat(n.graph_name,"/").concat(n.name)===t?(n.selected=!0,i=!0,e.expandKeys.push(n.id)):n.selected=!1})),this.$nextTick((function(){setTimeout((function(){var e=document.querySelector(".hit-item.selected");if(e){var t=document.querySelector(".tree-wrap");t.scrollTop=e.offsetTop}}),200)})),i}},updateTensorValue:function(e){var t=this;this.loadingInstance=this.$loading(this.loadingOption);var i=this.watchPointHits[e],n=i.name,a=this.nodeName;this.nodeName=n,this.isHitIntoView=!1;var o={mode:"node",params:{name:n,single_node:!0,watch_point_id:this.curWatchPointId?this.curWatchPointId:0,graph_name:i.graph_name,rank_id:this.logicCard.value}};this.graphFiles.value===this.$t("debugger.all")&&(delete o.params.graph_name,o.params.name="".concat(i.graph_name,"/").concat(n)),this.watchPointHits.forEach((function(t,i){t.selected=e===i})),this.watchPointHits=JSON.parse(JSON.stringify(this.watchPointHits)),h["a"].retrieve(o,this.sessionId).then((function(e){if(e.data.metadata&&t.dealMetadata(e.data.metadata),t.retrieveTensorHistory({name:t.nodeName},i.graph_name),e.data&&e.data.graph){var a=e.data.graph;t.graphFiles.value!==i.graph_name&&t.graphFiles.value!==t.$t("debugger.all")?(t.graphFiles.value=i.graph_name,t.resetAllData(a,o.params.name)):t.querySingleNode(JSON.parse(JSON.stringify(a)),o.params.name,!0),a.children&&(t.dealTreeData(a.children,n),t.defaultCheckedArr=t.$refs.tree.getCheckedKeys())}}),(function(e){t.showErrorMsg(e),t.nodeName=a}))},queryAllTreeData:function(e,t,i,n){var a=this;n&&(this.loadingInstance=this.$loading(this.loadingOption));var o=e?e.split(":")[0]:"",s={mode:"node",params:{name:o,single_node:!0,watch_point_id:this.curWatchPointId?this.curWatchPointId:0}};this.graphFiles.value===this.$t("debugger.all")&&i&&o?o!==i&&(o="".concat(i,"/").concat(o),s.params.name=o):s.params.graph_name=i,h["a"].retrieve(s,this.sessionId).then((function(e){if(e.data&&e.data.metadata&&a.dealMetadata(e.data.metadata),e.data&&e.data.graph){var t=e.data.graph;t.nodes&&!a.isCurrentGraph?(a.resetAllData(t,o),a.isCurrentGraph=!0):a.querySingleNode(JSON.parse(JSON.stringify(t)),o,!0),t.children&&(a.dealTreeData(t.children,o),a.defaultCheckedArr=a.$refs.tree.getCheckedKeys())}}),(function(e){a.showErrorMsg(e)}))},dealTreeData:function(e,t){var i=this;if(!(e.nodes.length>this.maxSupportNodesNumber)){if(e.nodes){if(e.nodes.length>this.nodesCountLimit&&"name_scope"===this.$refs.tree.getNode(e.scope_name).data.type||this.allGraphData[e.scope_name].maxChainNum>this.maxChainNum)return;var n=e.nodes.map((function(e){return Object(r["a"])(Object(r["a"])({label:e.name.split("/").pop()},e),{},{showCheckbox:-1!==e.watched})}));n.forEach((function(t){var n=i.$refs.tree.getNode(e.scope_name);n.childNodes?-1===n.childNodes.map((function(e){return e.data.name})).indexOf(t.name)&&i.$refs.tree.append(t,n):i.$refs.tree.append(t,n)}));var a=this.$refs.tree.getNode(e.scope_name);a.childNodes.forEach((function(e){e.data.watched===i.checkboxStatus.checked&&(e.checked=!0),e.data.watched===i.checkboxStatus.indeterminate&&(e.indeterminate=!0),"name_scope"!==e.data.type&&"aggregation_scope"!==e.data.type&&(e.isLeaf=!0)})),a.expanded=!0,a.loading=!1}else this.$refs.tree.setCurrentKey(t),this.$nextTick((function(){setTimeout((function(){var e=document.querySelector(".el-tree-node.is-current.is-focusable");if(e){var t=document.querySelector(".tree-wrap");t.scrollTop=e.offsetTop}}),800)}));e.children&&this.dealTreeData(e.children,t)}},resetAllData:function(e,t){var i=this;this.node.childNodes=[],this.origialTree=e.nodes.map((function(e){return Object(r["a"])(Object(r["a"])({label:e.name.split("/").pop(),leaf:"name_scope"!==e.type&&"aggregation_scope"!==e.type},e),{},{showCheckbox:-1!==e.watched})})),this.resolve(this.origialTree),this.node.childNodes.forEach((function(e){e.data.watched===i.checkboxStatus.checked&&(e.checked=!0),e.data.watched===i.checkboxStatus.indeterminate&&(e.indeterminate=!0)})),this.firstFloorNodes=[],this.allGraphData={},f.select("#graph svg").remove(),this.packageDataToObject("",!0,JSON.parse(JSON.stringify(e.nodes))),t?this.querySingleNode(JSON.parse(JSON.stringify(e)),t,!0):this.resetGraph()},stackPageChange:function(e){this.stacks.currentPage=e,this.queryStacks()},queryStacks:function(){var e=this;this.stacks.searchContent=this.stacks.searchContent.trim();var t={offset:this.stacks.currentPage-1,limit:this.stacks.pageSize.size,key_word:this.stacks.searchContent};h["a"].queryStackList(t,this.sessionId).then((function(t){t&&t.data&&t.data.stack_infos?(e.stacks.total=t.data.total,e.stacks.currentPage=t.data.offset+1,e.stacks.list=JSON.parse(JSON.stringify(t.data.stack_infos))):(e.stacks.currentPage=1,e.stacks.total=0,e.stacks.list=[])}),(function(t){e.stacks.currentPage=1,e.stacks.total=0,e.stacks.list=[],e.showErrorMsg(t)}))},loadTensor:function(e){var t=this,i=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n={name:e.name,rank_id:this.logicCard.value,graph_name:e.graph_name,prev:i+""};h["a"].loadTensor(n,this.sessionId).then((function(n){if(n&&n.data&&n.data.node_name){var a=t.fileSizeConversion(e.bytes);t.$message(t.$t("debugger.downloadTip",{fileSize:a.join("")})),t.downloadedTensor.graph_name=e.graph_name,t.downloadedTensor.prev=i,t.downloadedTensor.name===n.data.node_name?t.downloadTensor():t.downloadedTensor.name=n.data.node_name}}),(function(e){t.showErrorMsg(e),t.downloadedTensor={}}))},downloadTensor:function(){var e=p["a"]+"v1/mindinsight/debugger/sessions/".concat(this.sessionId,"/tensor-files/download?")+"name=".concat(this.downloadedTensor.name,"&rank_id=").concat(this.logicCard.value,"&")+"graph_name=".concat(this.downloadedTensor.graph_name,"&prev=").concat(this.downloadedTensor.prev),t=document.createElement("a");t.download="",t.href=e,t.rel="noopener noreferrer",t.target="_blank",t.click(),this.downloadedTensor={}},fileSizeConversion:function(){var e=arguments.length>0&&void 0!==arguments[0]?arguments[0]:0,t=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"bytes",i=["bytes","KB","MB","GB"],n=i.indexOf(t);if(e<0||-1===n)return[e,t];var a=1024;while(e>=a&&n<i.length-1)e/=a,n++;return e=parseFloat(e.toFixed(2)),[e,i[n]]}}},m=g,v=i("2877"),b=Object(v["a"])(m,n,a,!1,null,null,null),w=b.exports,y=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"deb-tensor-wrap"},[n("FlexibleGrid",{attrs:{areas:e.areas,rowSize:e.rowSize,columnSize:e.columnSize,hideAreas:e.hideAreas,showFixed:e.showFixed,gridStyleKey:e.gridStyleKey},on:{resizeGridStyle:e.resizeGridStyle,resizeCallback:e.resizeCallback}},[n("div",{staticClass:"deb-tensor-left",attrs:{slot:"left"},slot:"left"},[n("div",{staticClass:"deb-tensor-left-content"},[n("div",{staticClass:"left-content-title"},[n("span",[e._v(e._s(e.$t("debugger.optimizationOrientation")))])]),n("div",{directives:[{name:"show",rawName:"v-show",value:e.leftDataShow,expression:"leftDataShow"}],staticClass:"left-advice"},e._l(e.tensorList,(function(t){return n("div",{key:t.id,staticClass:"left-content-list"},[n("div",{staticClass:"detection-judgment"},[n("span",[e._v(e._s(e.$t("debugger.watchPoint"))+e._s(t.id))]),n("span",[e._v(e._s(e.$t("symbols.colon")))]),n("span",[e._v(e._s(e.$parent.transCondition(t.condition)))])]),e._l(t.params,(function(t,i){return n("div",{key:i,staticClass:"reason"},[n("div",{staticClass:"tensor-icon icon-secondary"}),n("div",{staticClass:"tensor-content"},[e._v(e._s(t.content))])])})),t.tip?n("div",{staticClass:"hit-tip"},[n("i",{staticClass:"el-icon-warning"}),e._v(e._s(t.tip)+" ")]):e._e(),t.tip?e._e():n("div",{staticClass:"tensor-advice"},[n("span",[e._v(e._s(e.$t("debugger.tuningAdvice")))]),n("div",{staticClass:"advice-list-title"},[n("div",{staticClass:"adviceTitle"},[e._v(e._s(t.tuningAdviceTitle))]),n("div",{staticClass:"advice-list"},e._l(t.tuningAdvice,(function(t,i){return n("div",{key:i,staticClass:"advice-content"},[e._v(e._s(t))])})),0)])])],2)})),0),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.leftDataShow,expression:"!leftDataShow"}],staticClass:"leftNoData"},[e._v(e._s(e.$t("debugger.noWatchPoint")))])])]),n("div",{staticClass:"deb-tensor-rightFixed",attrs:{slot:"rightFixed"},slot:"rightFixed"},[n("div",{staticClass:"deb-con-title"},[n("div",{staticClass:"deb-con-title-left",attrs:{title:e.curRowObj.name}},[e._v(" "+e._s(e.curRowObj.name)+" ")]),n("div",{staticClass:"deb-con-title-right"},[n("div",{staticClass:"close-btn"},[n("img",{attrs:{src:i("4d06")},on:{click:e.closeTensor}})])])])]),n("div",{staticClass:"deb-tensor-rightTop",attrs:{slot:"rightTop"},slot:"rightTop"},[n("div",{directives:[{name:"show",rawName:"v-show",value:e.showTensorValue,expression:"showTensorValue"}],staticClass:"deb-compare-detail"},e._l(e.statisticsArr,(function(t,i){return n("div",{key:i},[0===i?n("label",[e._v(" "+e._s("preStep"===e.gridType?e.$t("debugger.preStatisticsLabel"):e.$t("debugger.curStatisticsLabel"))+" "),n("span",[e._v(e._s("preStep"===e.gridType?e.curRowObj.step-1:e.curRowObj.step))])]):e._e(),1===i?n("label",[e._v(e._s(e.$t("debugger.preStatisticsLabel"))),n("span",[e._v(e._s(e.curRowObj.step-1))])]):e._e(),2===i?n("label",[e._v(e._s(e.$t("debugger.diffStatisticsLabel")))]):e._e(),n("span",[e._v(e._s(e.$t("debugger.max"))+" "+e._s(void 0===t.overall_max?"--":t.overall_max))]),n("span",[e._v(e._s(e.$t("debugger.min"))+" "+e._s(void 0===t.overall_min?"--":t.overall_min))]),n("span",[e._v(e._s(e.$t("debugger.mean"))+" "+e._s(void 0===t.overall_avg?"--":t.overall_avg))]),n("span",[e._v(e._s(e.$t("debugger.nan"))+" "+e._s(void 0===t.overall_nan_count?"--":t.overall_nan_count)+" ")]),n("span",[e._v(e._s(e.$t("debugger.negativeInf"))+" "+e._s(void 0===t.overall_neg_inf_count?"--":t.overall_neg_inf_count)+" ")]),n("span",[e._v(e._s(e.$t("debugger.inf"))+" "+e._s(void 0===t.overall_pos_inf_count?"--":t.overall_pos_inf_count)+" ")]),n("span",[e._v(e._s(e.$t("debugger.zero"))+" "+e._s(void 0===t.overall_zero_count?"--":t.overall_zero_count))]),n("span",[e._v(e._s(e.$t("debugger.negativeNum"))+" "+e._s(void 0===t.overall_neg_zero_count?"--":t.overall_neg_zero_count))]),n("span",[e._v(e._s(e.$t("debugger.positiveNum"))+" "+e._s(void 0===t.overall_pos_zero_count?"--":t.overall_pos_zero_count))]),n("span",[e._v(e._s(e.$t("debugger.true"))+" "+e._s(void 0===t.overall_true_count?"--":t.overall_true_count))]),n("span",[e._v(e._s(e.$t("debugger.false"))+" "+e._s(void 0===t.overall_false_count?"--":t.overall_false_count))])])})),0),n("div",{directives:[{name:"show",rawName:"v-show",value:e.showTensorValue,expression:"showTensorValue"}],staticClass:"deb-con-slide"},[n("div",{staticClass:"deb-con-slide-right"},[n("el-button",{staticClass:"custom-btn",class:{green:"value"===e.gridType},attrs:{size:"mini",disabled:"running"===e.state},on:{click:function(t){return e.tabChange("value")}}},[e._v(e._s(e.$t("debugger.curStep")))]),n("el-button",{staticClass:"custom-btn",class:{green:"preStep"===e.gridType},attrs:{size:"mini",disabled:!e.curRowObj.has_prev_step||"running"===e.state},on:{click:function(t){return e.tabChange("preStep")}}},[e._v(e._s(e.$t("debugger.preStep")))]),n("el-button",{staticClass:"custom-btn",class:{green:"compare"===e.gridType},attrs:{size:"mini",disabled:!e.curRowObj.has_prev_step||"running"===e.state||"oversize"===e.curRowObj.tensor_status},on:{click:function(t){return e.tabChange("compare")}}},[e._v(e._s(e.$t("debugger.compareResult")))]),n("el-button",{staticClass:"custom-btn",attrs:{size:"mini",disabled:"running"===e.state||"compare"===e.gridType||e.curRowObj.oversized||"null"===e.curRowObj.value},on:{click:function(t){return e.$parent.loadTensor(e.curRowObj,"preStep"===e.gridType)}}},[e._v(" "+e._s(e.$t("graph.downloadPic")))])],1),"compare"===e.gridType?n("div",{staticClass:"deb-con-slide-left"},[n("div",{staticClass:"deb-slide-title"},[e._v(e._s(e.$t("debugger.tolerance")))]),n("div",{staticClass:"deb-slide-width"},[n("el-slider",{attrs:{"format-tooltip":e.formatTolenrance},on:{change:function(t){return e.tensorComparisons(e.curRowObj,e.dims,!0)},input:function(t){return e.toleranceInputChange()}},model:{value:e.tolerance,callback:function(t){e.tolerance=t},expression:"tolerance"}})],1),n("div",{staticClass:"deb-slide-input"},[n("el-input",{on:{input:e.toleranceValueChange},nativeOn:{keyup:function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"enter",13,t.key,"Enter")?null:e.tensorComparisons(e.curRowObj,e.dims,!0)}},model:{value:e.toleranceInput,callback:function(t){e.toleranceInput=t},expression:"toleranceInput"}})],1)]):e._e(),n("div",{staticClass:"deb-con-slide-middle"},[e._v(" MIN "),n("div",{staticClass:"grident"},[e._v("0")]),e._v(" MAX ")])]),n("div",{directives:[{name:"show",rawName:"v-show",value:e.showTensorValue,expression:"showTensorValue"}],staticClass:"deb-con-table"},[n("div",{staticClass:"deb-compare-wrap"},["value"===e.gridType||"preStep"===e.gridType?n("debuggerGridTable",{ref:"tensorValue",attrs:{fullData:e.tensorValue,showFilterInput:e.showFilterInput,columnLimitNum:e.columnLimitNum,gridType:"value"},on:{martixFilterChange:function(t){return e.tensorFilterChange(t)}}}):n("debuggerGridTable",{ref:"tensorValue",attrs:{fullData:e.tensorValue,showFilterInput:e.showFilterInput,columnLimitNum:e.columnLimitNum,gridType:"compare"},on:{martixFilterChange:function(t){return e.tensorFilterChange(t)}}})],1)])]),n("div",{staticClass:"deb-tensor-rightBottom",attrs:{slot:"rightBottom"},slot:"rightBottom"},[n("div",{staticClass:"deb-graph-container"},[n("div",{staticClass:"graph-title"},[e._v(" "+e._s(e.$t("debugger.tensorDiagram"))+" "),n("span",{staticClass:"tip"},[n("el-tooltip",{attrs:{placement:"bottom",effect:"light","popper-class":"legend-tip"}},[n("i",{staticClass:"el-icon-info"}),n("div",{attrs:{slot:"content"},slot:"content"},[n("div",[e._v(e._s(e.$t("debugger.selectDetail")))]),n("div",{staticClass:"legend"},[n("div",{staticClass:"item"},[e._v(" "+e._s(e.$t("debugger.tensorTip"))+" "),n("img",{attrs:{src:i("ea825"),alt:""}})]),n("div",{staticClass:"item"},[e._v(" "+e._s(e.$t("debugger.operator"))+" "),n("img",{attrs:{src:i("0946"),alt:""}})])])])])],1)]),n("div",{staticClass:"pre-step-graph"},[n("el-button",{staticClass:"custom-btn",attrs:{size:"mini",disabled:0===e.currentTensorGraphIndex},on:{click:e.backToPreGraph}},[e._v(e._s(e.$t("debugger.back")))]),n("el-button",{staticClass:"custom-btn",attrs:{size:"mini",disabled:e.preTensorGraphs.length-1===e.currentTensorGraphIndex},on:{click:e.goToForwardGraph}},[e._v(e._s(e.$t("debugger.forward")))])],1),e.graphShow?n("div",{staticClass:"deb-graph",attrs:{id:"tensor-graph"}}):e.graphShow?e._e():n("div",{staticClass:"nodata"},[e._v(" "+e._s(e.$t("public.noData"))+" ")]),n("div",{staticClass:"deb-tensor-info"},[n("div",{staticClass:"tensor"},[n("el-tabs",{model:{value:e.activeName,callback:function(t){e.activeName=t},expression:"activeName"}},[n("el-tab-pane",{attrs:{label:e.$t("debugger.tensorMsg"),name:"tensorMsg"}},[n("div",{staticClass:"tensor-detail"},[n("span",[e._v(e._s(e.$t("graph.name")+e.$t("symbols.colon"))+" "+e._s(e.statistics.name))]),n("span",[e._v(e._s(e.$t("debugger.max"))+" "+e._s(void 0===e.statistics.overall_max?"--":e.statistics.overall_max))]),n("span",[e._v(e._s(e.$t("debugger.min"))+" "+e._s(void 0===e.statistics.overall_min?"--":e.statistics.overall_min))]),n("span",[e._v(e._s(e.$t("debugger.mean"))+" "+e._s(void 0===e.statistics.overall_avg?"--":e.statistics.overall_avg)+" ")]),n("span",[e._v(e._s(e.$t("debugger.nan"))+" "+e._s(void 0===e.statistics.overall_nan_count?"--":e.statistics.overall_nan_count)+" ")]),n("span",[e._v(e._s(e.$t("debugger.negativeInf"))+" "+e._s(void 0===e.statistics.overall_neg_inf_count?"--":e.statistics.overall_neg_inf_count)+" ")]),n("span",[e._v(e._s(e.$t("debugger.inf"))+" "+e._s(void 0===e.statistics.overall_pos_inf_count?"--":e.statistics.overall_pos_inf_count)+" ")]),n("span",[e._v(e._s(e.$t("debugger.zero"))+" "+e._s(void 0===e.statistics.overall_zero_count?"--":e.statistics.overall_zero_count))]),n("span",[e._v(e._s(e.$t("debugger.negativeNum"))+" "+e._s(void 0===e.statistics.overall_neg_zero_count?"--":e.statistics.overall_neg_zero_count))]),n("span",[e._v(e._s(e.$t("debugger.positiveNum"))+" "+e._s(void 0===e.statistics.overall_pos_zero_count?"--":e.statistics.overall_pos_zero_count))]),n("span",[e._v(e._s(e.$t("debugger.true"))+" "+e._s(void 0===e.statistics.overall_true_count?"--":e.statistics.overall_true_count))]),n("span",[e._v(e._s(e.$t("debugger.false"))+" "+e._s(void 0===e.statistics.overall_false_count?"--":e.statistics.overall_false_count))])])]),n("el-tab-pane",{attrs:{label:e.$t("debugger.stackInfo"),name:"stack"}},[n("div",{staticClass:"table-content"},[n("ul",{staticClass:"stack-content"},[e._l(e.nodeStackContent,(function(t,i){return n("li",{directives:[{name:"show",rawName:"v-show",value:e.nodeStackContent.length,expression:"nodeStackContent.length"}],key:i,staticClass:"stack-item"},[e._v(" "+e._s(t.file_path?t.file_path+":"+t.line_no:"")+" "),t.file_path?n("br"):e._e(),e._v(" "+e._s(t.code_line)+" ")])})),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.nodeStackContent.length,expression:"!nodeStackContent.length"}],staticClass:"noData-text"},[e._v(" "+e._s(e.$t("public.noData"))+" ")])],2)])])],1)],1),n("div",{staticClass:"watch-point"},[n("div",{staticClass:"watchPoint-title"},[e._v(e._s(e.$t("debugger.curHitNode")))]),n("div",{directives:[{name:"show",rawName:"v-show",value:e.rightDataShow,expression:"rightDataShow"}],staticClass:"point-list"},e._l(e.watchPoints,(function(t,i){return n("div",{key:i},[n("div",{staticClass:"watch-judgment"},[n("span",[e._v(e._s(e.$t("debugger.watchPoint"))+e._s(t.id))]),n("span",[e._v(e._s(e.$t("symbols.colon")))]),n("span",[e._v(e._s(e.getFormateWatchPoint(t)))])])])})),0),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.rightDataShow,expression:"!rightDataShow"}],staticClass:"leftNoData"},[e._v(e._s(e.$t("debugger.noWatchPoint")))])])])])])])],1)},C=[],_=(i("a630"),function(){var e=this,t=e.$createElement,i=e._self._c||t;return i("div",{staticClass:"cl-slickgrid-container"},[i("div",{staticClass:"data-show-container"},[i("div",{directives:[{name:"show",rawName:"v-show",value:e.incorrectData,expression:"incorrectData"}],staticClass:"error-msg-container"},[e._v(" "+e._s(e.$t("components.gridIncorrectDataError"))+" ")]),i("div",{directives:[{name:"show",rawName:"v-show",value:!e.incorrectData&&e.requestError,expression:"!incorrectData && requestError"}],staticClass:"error-msg-container"},[e._v(" "+e._s(e.errorMsg)+" ")]),i("div",{directives:[{name:"show",rawName:"v-show",value:!e.fullData.length&&e.updated&&!e.incorrectData&&!e.requestError,expression:"!fullData.length && updated && !incorrectData && !requestError"}],staticClass:"error-msg-container"},[e._v(" "+e._s(e.$t("components.gridTableNoData"))+" ")]),i("div",{directives:[{name:"show",rawName:"v-show",value:!!e.fullData.length&&!e.incorrectData&&e.displayMode===e.displayModes[0].value,expression:"!!fullData.length && !incorrectData && displayMode === displayModes[0].value"}],ref:"heatmap",staticClass:"heatmap"}),i("div",{directives:[{name:"show",rawName:"v-show",value:!!e.fullData.length&&!e.incorrectData&&e.displayMode===e.displayModes[1].value,expression:"!!fullData.length && !incorrectData && displayMode === displayModes[1].value"}],ref:"tablemap",staticClass:"grid-item",attrs:{id:e.itemId}})]),e.showOperators&&(e.fullData.length||e.requestError)?i("div",{staticClass:"operators-container"},[i("div",{staticClass:"operators"},[e.showFilterInput?i("div",{staticClass:"filter-container",on:{keyup:function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"enter",13,t.key,"Enter")?null:e.filterChange.apply(null,arguments)}}},[i("div",{staticClass:"filter-input-title"},[e._v(e._s(e.$t("components.dimsFilterInputTitle"))+" "),i("span",{staticClass:"el-icon-info",attrs:{title:e.$t("components.dimsFilterInputTip")}})]),e._l(e.filterArr,(function(t,n){return i("div",{key:n,staticClass:"filter-item"},[i("el-input",{staticClass:"filter-input long-input",class:t.showError?"error-border":"",model:{value:t.model,callback:function(i){e.$set(t,"model",i)},expression:"item.model"}}),n===e.filterArr.length-1?i("span",{staticClass:"input-behind"},[e._v(e._s(e.$t("symbols.slashes")))]):i("span",{staticClass:"input-behind"},[e._v(e._s(e.$t("symbols.point")))])],1)})),e.filterArr.length?i("el-button",{staticClass:"filter-check",attrs:{size:"mini"},on:{click:e.filterChange}},[i("i",{staticClass:"el-icon-check"})]):e._e(),e.filterCorrect?e._e():i("span",{staticClass:"filter-incorrect-text"},[e._v(e._s(e.$t("components.inCorrectInput")))])],2):e._e(),i("span",[e._v(e._s(e.$t("tensors.dimension"))+" "+e._s(e.shape))])]),i("div",{staticClass:"operators"},[e._v(" "+e._s(e.$t("tensors.mode"))+" "),i("el-select",{staticClass:"mode-selector",style:{width:e.modeWidth+"px"},on:{change:e.handleDisplayModeChange},model:{value:e.displayMode,callback:function(t){e.displayMode=t},expression:"displayMode"}},e._l(e.displayModes,(function(e){return i("el-option",{key:e.value,attrs:{label:e.label,value:e.value}})})),1),e._v(" "+e._s(e.$t("components.category"))+" "),i("el-select",{staticClass:"category-selector",style:{width:e.categoryWidth+"px"},on:{change:e.accuracyChange},model:{value:e.category,callback:function(t){e.category=t},expression:"category"}},e._l(e.categoryArr,(function(e){return i("el-option",{key:e.label,attrs:{label:e.label,value:e.value}})})),1),e._v(" "+e._s(e.$t("components.gridAccuracy"))+" "),i("span",{staticClass:"el-icon-info",attrs:{title:e.$t("components.accuracyTips")}}),i("el-select",{staticClass:"accuracy-selector",on:{change:e.accuracyChange},model:{value:e.accuracy,callback:function(t){e.accuracy=t},expression:"accuracy"}},e._l(e.accuracyArr,(function(e){return i("el-option",{key:e.label,attrs:{label:e.label,value:e.value}})})),1)],1)]):e._e()])}),x=[],k=(i("cb29"),i("9129"),i("326d"),i("f411"),i("dc1b"),i("f06f"),i("ed3a"),i("7d72"),i("70a2"),i("3a3f"),i("2dd7")),S=i("9fac"),T="table",$="chart",D=[40,0,10,40],N=103,R=82,I=103,P={props:{fullData:{type:Array,default:function(){return[]}},showOperators:{type:Boolean,default:!0},showFilterInput:{type:Boolean,default:!0},fullScreen:{type:Boolean,default:!1},gridType:{type:String,default:"value"},columnLimitNum:{type:Number,default:-1}},data:function(){return{itemId:"",gridObj:null,columnsData:[],columnsLength:0,filterArr:[],formateData:[],formateArr:[],statistics:{},accuracy:10,incorrectData:!1,updated:!1,scrollTop:!1,filterCorrect:!0,requestError:!1,errorMsg:"",viewResizeFlag:!1,accuracyArr:new Array(11).fill(null).map((function(e,t){return{label:t,value:t}})),optionObj:{enableColumnReorder:!1,enableCellNavigation:!0,frozenColumn:0,frozenRow:0},gridTypeKeys:{value:"value",compare:"compare"},axisStartIndex:{rowStartIndex:0,colStartIndex:0},shape:"",categoryArr:[{label:this.$t("components.value"),value:"value"},{label:this.$t("components.scientificCounting"),value:"science"}],category:"value",gridTableThemeObj:k["a"].tensorThemes[this.$store.state.themeIndex],displayMode:$,displayModes:[{label:this.$t("tensors.chartMode"),value:$},{label:this.$t("tensors.tableMode"),value:T}],chartInstance:null,filterStr:null,categoryWidth:0,modeWidth:0}},watch:{showFilterInput:function(){this.resizeView()}},mounted:function(){this.init()},methods:{initCommonInfo:function(e,t,i,n){this.shape=t,this.updated=!0,this.requestError=!1,this.fullData&&this.fullData.length&&(e?(this.viewResizeFlag=!0,this.initializeFilterArr(t,n),this.scrollTop=!0):!this.filterArr.length&&t&&n&&this.initializeFilterArr(t,n),(e||void 0===this.statistics.overall_max)&&(this.statistics=i),this.handleDisplayModeChange())},handleDisplayModeChange:function(){var e=this;this.$nextTick((function(){switch(e.displayMode){case T:e.initHeatmapChart(),e.chartInstance&&e.chartInstance.resize(),e.updateGridData(),e.chartInstance=null;break;case $:e.chartInstance&&e.chartInstance.resize(),e.renderHeatmapChart(),e.chartInstance=null;break}}))},init:function(){this.itemId="".concat((new Date).getTime())+"".concat(this.$store.state.componentsCount),this.$store.commit("componentsNum"),"en-us"===this.$store.state.language?(this.categoryWidth=N,this.modeWidth=R):(this.categoryWidth=I,this.modeWidth=I)},processData:function(e){var t=this.judgeDataType,i=this.accuracy,n=this.category;return t(e)?0:"science"===n?e.toExponential(i):e.toFixed(i)},renderHeatmapChart:function(){var e=this,t=this.fullData;if(t&&t.length&&this.initHeatmapChart()){var i=[],n=[],a=[],o=this.axisStartIndex,s=o.rowStartIndex,r=o.colStartIndex;Array.isArray(t)?Array.isArray(t[0])?("compare"!==this.gridType||Array.isArray(t[0][0])||a.push(s),t.forEach((function(t,o){"compare"!==e.gridType||Array.isArray(t[0])?(a.push(s+o),t.forEach((function(t,a){o||n.push(r+a),Array.isArray(t)?i.push([a,o,e.processData(t[0]),e.processData(t[1]),e.processData(t[2])]):i.push([a,o,t,e.processData(t)])}))):(n.push(r+o),i.push([o,0,e.processData(t[0]),e.processData(t[1]),e.processData(t[2])]))}))):(a.push(s),t.forEach((function(t,a){n.push(r+a),i.push([a,0,t,e.processData(t)])}))):(n.push(r),a.push(s),i.push([0,0,t,this.processData(t)]));var l=this.statistics,c=l.overall_min,h=l.overall_max,d=Math.abs(c),u=Math.abs(h),p=0;Number.isNaN(d)||Number.isNaN(u)?!Number.isNaN(d)&&Number.isNaN(u)?p=d:Number.isNaN(d)&&!Number.isNaN(u)&&(p=u):p=d<u?u:d,this.chartInstance.setOption({visualMap:{min:-p,max:p,inRange:{color:["rgb(".concat(this.gridTableThemeObj.negativeColor,")"),this.gridTableThemeObj.middleColor,"rgb(".concat(this.gridTableThemeObj.positiveColor,")")]}},xAxis:{data:n},yAxis:{data:a},series:{data:i}})}},initHeatmapChart:function(){var e=this;return!this.chartInstance&&(this.displayMode===$&&this.$refs.heatmap?this.chartInstance=S["a"].init(this.$refs.heatmap,S["b"]):this.displayMode===T&&this.$refs.tablemap&&(this.chartInstance=S["a"].init(this.$refs.tablemap,S["b"])),this.chartInstance.setOption({tooltip:{position:"top"},grid:{top:D[0],right:D[1],bottom:D[2],left:D[3]},dataZoom:[{type:"inside",xAxisIndex:0,yAxisIndex:0}],xAxis:{type:"category",data:[],position:"top",splitArea:{show:!0}},yAxis:{type:"category",data:[],splitArea:{show:!0},inverse:!0},visualMap:{show:!1},series:[{name:"tensorHeatmap",type:"heatmap",data:[],label:{show:!1},emphasis:{itemStyle:{shadowBlur:10,shadowColor:"rgba(0, 0, 0, 0.5)"}},tooltip:{formatter:function(t){var i=t.data.length,n=t.data[i-1];return 5===i?"\n                    x: ".concat(t.data[0]," &nbsp&nbsp&nbsp&nbsp&nbspy: ").concat(t.data[1],"<br>\n                    ").concat(e.$t("debugger.curStatisticsLabel")," ").concat(t.data[3],"<br>\n                    ").concat(e.$t("debugger.preStatisticsLabel")," ").concat(t.data[2],"<br>\n                    ").concat(e.$t("scalar.charTipHeadValue")," ").concat(e.$t("symbols.colon")," ").concat(n,"\n                  "):"\n                    x: ".concat(t.data[0],"<br>\n                    y: ").concat(t.data[1],"<br>\n                    ").concat(e.$t("scalar.charTipHeadValue")," ").concat(e.$t("symbols.colon")," ").concat(n,"\n                  ")},position:[0,0]}}]}),!0)},initializeFilterArr:function(e,t){if(this.filterCorrect=!0,t){var i=t.slice(1,t.length-1).split(","),n=[],a=[],o=[],s=i.length;i.forEach((function(t,i){if(n.push({model:t,max:e[i]-1,showError:!1}),t.includes(":")){if(s<=2){var r=t.split(":"),l=Object(u["a"])(r,1),c=l[0];if(c){var h=+c<0?e[i]+ +c:+c;o.push(h)}else o.push(0)}}else if(s--,s<2){var d=+t,p=d<0?e[i]+d:d;a.push(p)}})),this.filterArr=n,2===o.length?this.axisStartIndex={rowStartIndex:o[0],colStartIndex:o[1]}:1===o.length?this.axisStartIndex={rowStartIndex:a.length?a[0]:0,colStartIndex:o[0]}:a.length?this.axisStartIndex={rowStartIndex:a[0],colStartIndex:a.length>1?a[1]:0}:this.axisStartIndex={rowStartIndex:0,colStartIndex:0}}else this.filterArr=[]},formatColumnsData:function(){var e=this;this.columnsData=[{id:-1,name:" ",field:-1,width:120,headerCssClass:"headerStyle"}];var t=this.formateData[0];t?t.forEach((function(t,i){var n=i+e.axisStartIndex.colStartIndex;e.columnsData.push({id:n,name:n,field:n,width:120,headerCssClass:"headerStyle",formatter:e.gridType===e.gridTypeKeys.compare?e.formatCompareColor:e.formatValueColor})})):this.columnsData=[]},formatValueColor:function(e,t,i,n,a){return t&&i&&!this.judgeDataType(i)?i<0?'<span class="table-item-span" style="background:rgba('.concat(this.gridTableThemeObj.negativeColor,", ").concat(i/this.statistics.overall_min,')">').concat(i,"</span>"):'<span class="table-item-span" style="background:rgba('.concat(this.gridTableThemeObj.positiveColor,", ").concat(i/this.statistics.overall_max,')">').concat(i,"</span>"):i},judgeDataType:function(e){return isNaN(e)||"Infinity"===e||"-Infinity"===e||"boolean"===typeof e||"null"===e},formatCompareColor:function(e,t,i,n,a){if(i instanceof Array&&i.length>=3){var o=i[2];return t&&o&&!this.judgeDataType(o)?o<0?'<span class="table-item-span" title="'.concat(i[0],"→").concat(i[1],'" style="background:rgba(227, 125, 41, ').concat(o/this.statistics.overall_min,')">').concat(o,"</span>"):'<span class="table-item-span" title="'.concat(i[0],"→").concat(i[1],'" style="background:rgba(0, 165, 167, ').concat(o/this.statistics.overall_max,')">').concat(o,"</span>"):'<span class="table-item-span" title="'.concat(i[0],"→").concat(i[1],'">').concat(o,"</span>")}return i},calcForArrDims:function(e){var t=0,i=e;while(i instanceof Array&&t<=3)t++,i=i[0];return t},formatGridArray:function(){var e=this,t=this.calcForArrDims(this.fullData);t>3?(this.formateData=[[[]]],this.columnsData=[]):this.formateData=0===t?[[this.fullData]]:1===t?[this.fullData]:this.fullData,this.gridType===this.gridTypeKeys.compare&&2===t&&(this.formateData=[this.formateData]);var i=[];this.gridType===this.gridTypeKeys.compare?this.formateData.forEach((function(t,n){var a={"-1":n+e.axisStartIndex.rowStartIndex};t.forEach((function(t,i){var n=i+e.axisStartIndex.colStartIndex,o=[];t.forEach((function(t){e.judgeDataType(t)?o.push(t):"science"===e.category?o.push(t.toExponential(e.accuracy)):o.push(t.toFixed(e.accuracy))})),a[n]=o})),i.push(a)})):this.formateData.forEach((function(t,n){var a={"-1":n+e.axisStartIndex.rowStartIndex};t.forEach((function(t,i){var n=i+e.axisStartIndex.colStartIndex;e.judgeDataType(t)?a[n]=t:"science"===e.category?a[n]=t.toExponential(e.accuracy):a[n]=t.toFixed(e.accuracy)})),i.push(a)})),this.formateArr=i},updateGrid:function(){var e=this;this.$nextTick((function(){e.gridObj||(e.gridObj=new window.Slick.Grid("#".concat(e.itemId),e.formateArr,e.columnsData,e.optionObj),e.columnsLength=e.columnsData.length),e.gridObj.setData(e.formateArr,e.scrollTop),e.scrollTop=!1;var t=e.columnsData.length;(e.columnsLength!==t||e.viewResizeFlag)&&(e.gridObj.setColumns(e.columnsData),e.columnsLength=t,e.viewResizeFlag=!1),e.gridObj.render()}))},accuracyChange:function(e){this.formatGridArray(),this.displayMode===$?this.renderHeatmapChart():this.requestError||this.incorrectData||this.updateGrid()},filterChange:function(){var e=this,t=!0,i=!1,n=2,a=[],o=[];if(this.filterArr.forEach((function(s,r){var l=s.model.trim();if(isNaN(l))if(-1!==l.indexOf(":")){a.push(r);var c=e.checkCombinatorialInput(s);c?(s.showError=!1,n?n--:i=!0):(s.showError=!0,t=!1)}else s.showError=!0,t=!1;else l<-(s.max+1)||l>s.max||""===l||l%1?(s.showError=!0,t=!1):(s.showError=!1,l=Number(l));o.push(l)})),a.length){var s=a.pop(),r=this.filterArr[s];if(this.columnLimitNum>0&&r&&!r.showError&&r.max>=this.columnLimitNum){var l=this.checkFilterLimitOver(r);l&&(r.showError=!0,t=!1)}}this.filterCorrect=t,i&&t?this.incorrectData=!0:(this.incorrectData=!1,t&&(this.viewResizeFlag=!0,this.$emit("martixFilterChange",o)))},checkFilterLimitOver:function(e){var t=!1,i=e.model.trim(),n=i.split(":"),a=n[0]?n[0]:0,o=n[1]?n[1]:e.max+1;return a=a<0?e.max+Number(a)+1:Number(a),o=o<0?e.max+Number(o)+1:Number(o),o-a>this.columnLimitNum&&(t=!0),t},checkCombinatorialInput:function(e){var t=e.model.trim(),i=t.split(":"),n=i[0],a=i[1],o=2;if(n&&(isNaN(n)||n<-(e.max+1)||n>e.max))return!1;if(a&&(isNaN(a)||a<=-(e.max+1)||a>e.max+1||!Number(a)))return!1;if(i.length>o)return!1;if(n||a){if(n&&a){var s=n<0?e.max+Number(n)+1:Number(n),r=a<0?e.max+Number(a)+1:Number(a);return!(r<=s)}return!0}return!0},updateGridData:function(){this.formatGridArray(),this.formatColumnsData(),this.incorrectData||this.updateGrid()},resizeView:function(){var e=this;this.$nextTick((function(){switch(e.displayMode){case $:e.chartInstance&&e.chartInstance.resize();break;case T:e.gridObj&&(e.incorrectData||e.requestError?e.viewResizeFlag=!0:e.$nextTick((function(){e.gridObj.resizeCanvas(),e.gridObj.render()})));break}}))},toggleFullScreen:function(){this.$emit("toggleFullScreen")},showRequestErrorMessage:function(e,t,i,n){this.shape=t,this.errorMsg=e,(!this.filterArr.length&&t&&i||n)&&this.initializeFilterArr(t,i),this.requestError=!0}},destroyed:function(){}},A=P,E=(i("44b8"),Object(v["a"])(A,_,x,!1,null,null,null)),z=E.exports,M=i("0165"),H=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{ref:"grid",staticClass:"mi-flex-grid",style:e.gridStyle,attrs:{id:"mi-flex-grid"},on:{mousemove:e.handleMouseMove,mouseleave:e.handleMouseLeave,mouseup:e.handleMouseUp}},[e._l(e.areaList,(function(t,a){return n("div",{key:t.name,class:[e.gapAreas["columnGap"].includes(t.name)||e.gapAreas["rowGap"].includes(t.name)?"grid-item-gap":"grid-item"],style:{gridArea:t.name}},[e._t(t.name),e.gapAreas["columnGap"].includes(t.name)?n("div",{staticClass:"grid-columnGap",on:{mousedown:e.handleMouseDown}}):e._e(),e.gapAreas["rowGap"].includes(t.name)?n("div",{staticClass:"grid-rowGap",on:{mousedown:e.handleMouseDown}}):e._e(),a>0&&e.areaList[a-1].hide?n("div",{class:["hide-button","is-"+e.areaList[a-1].hide]},[n("img",{attrs:{src:i("f57b")("./"+e.theme+"/collapse-"+(e.areaList[a-1].isHide?"right":"left")+".svg")},on:{click:function(t){return e.isDisplayView(e.areaList[a-1].name)}}})]):e._e(),a===e.areaList.length-2&&e.areaList[a+1].hide?n("div",{class:["hide-button","is-"+e.areaList[a+1].hide]},[n("img",{attrs:{src:i("f57b")("./"+e.theme+"/collapse-"+(e.areaList[a+1].isHide?"right":"left")+".svg")},on:{click:function(t){return e.isDisplayView(e.areaList[a+1].name)}}})]):e._e()],2)})),n("div",{directives:[{name:"show",rawName:"v-show",value:e.showPreview,expression:"showPreview"}],staticClass:"mi-flex-grid-preview",style:e.previewStyle},e._l(e.areaList,(function(t){return n("div",{key:t.name,staticClass:"grid-preview-item",style:{gridArea:t.name}},[e.gapAreas["columnGap"].includes(t.name)?n("div",{staticClass:"grid-columnGap"}):e._e(),e.gapAreas["rowGap"].includes(t.name)?n("div",{staticClass:"grid-rowGap"}):e._e()])})),0)],2)},F=[],O=i("53ca"),W=i("bee2"),L=i("d4ec"),j=i("ade3"),B=(i("cca6"),i("07ac"),Object(W["a"])((function e(){Object(L["a"])(this,e),Object(j["a"])(this,"gridTemplateAreas",void 0),Object(j["a"])(this,"gridTemplateRows",void 0),Object(j["a"])(this,"gridTemplateColumns",void 0),this.gridTemplateAreas="",this.gridTemplateRows="",this.gridTemplateColumns=""}))),G=Object(W["a"])((function e(){Object(L["a"])(this,e),Object(j["a"])(this,"count",0),Object(j["a"])(this,"px",0),Object(j["a"])(this,"fr",0)})),V={props:{areas:Array,rowSize:Array,columnSize:Array,hideAreas:Object,showFixed:Boolean,gridStyleKey:String},data:function(){return{theme:this.$store.state.themeIndex,previewStyle:new B,gridStyle:new B,areaList:[],areaMap:{},rowStructure:null,rowFR:0,rows:[],rowGaps:[],rowGap:"16px",columnGap:"16px",differIndex:2,resizeDelayTime:200,columnStructure:null,columnFR:0,columns:[],columnGaps:[],showPreview:!1,width:"",viewResizeTimer:null,callBackTimer:null,gapAreas:{columnGap:[],rowGap:[]},saveGridStyleObj:[{key:"debugger",value:null},{key:"debugger-tensor",value:null},{key:"debugger-tensor-inner",value:null},{key:"offline-debugger",value:null},{key:"offline-debugger-tensor",value:null},{key:"offline-debugger-tensor-inner",value:null},{key:"step-trace",value:null},{key:"profiling-gpu",value:null},{key:"model-traceback",value:null},{key:"graph",value:null}]}},watch:{},created:function(){this.calGapAreas(),this.parseGridInfo()},mounted:function(){this.calFRValue(),this.calTriggerPosition(),this.updatePreviewStyle(),window.addEventListener("resize",this.resizeGridStyle,!1)},destroyed:function(){window.removeEventListener("resize",this.resizeGridStyle),this.viewResizeTimer&&(clearTimeout(this.viewResizeTimer),this.viewResizeTimer=null),this.callBackTimer&&(clearTimeout(this.callBackTimer),this.callBackTimer=null)},methods:{changeAreasIsHide:function(){var e=this,t=this.columns,i=this.rows,n=this.differIndex,a=this.areaList;t.forEach((function(e){["left","right"].includes(e.hide)&&(e.isHide=!Boolean(e.currentWidth),a.find((function(t){return t.hide===e.hide})).isHide=e.isHide)}));var o=1;if(i.length!==o){var s=i.findIndex((function(t){return Object.keys(e.hideAreas).includes(t.name)}));i[s]["currentHeight"]?(i[s+n]["hide"]="bottom",i[s]["isHide"]=!1):(i[s+n]["hide"]="",i[s]["hide"]="top-center",i[s]["isHide"]=!0),i[s+n]["currentHeight"]?(i[s]["isHide"]||(i[s]["hide"]="top"),i[s+n]["isHide"]=!1):(i[s]["hide"]="",i[s+n]["hide"]="bottom-center",i[s+n]["isHide"]=!0),a.forEach((function(e){i[s].name===e.name&&(e.hide=i[s].hide,e.isHide=i[s].isHide),i[s+n].name===e.name&&(e.hide=i[s+n].hide,e.isHide=i[s+n].isHide)}))}},resizeGridStyle:function(){var e=this;this.viewResizeTimer=setTimeout((function(){e.getSaveGridStyle(!0),e.calFRValue(),e.calTriggerPosition(),e.updatePreviewStyle(!1)}),this.resizeDelayTime)},isDisplayView:function(e){var t=!(arguments.length>1&&void 0!==arguments[1])||arguments[1],i=this.columns,n=this.rows,a=this.differIndex,o=this.gridStyleKey,s=this.areaList;i.forEach((function(n,s){if(n.name===e&&"left"===n.hide){if(n.isHide){var r=+localStorage.getItem("".concat(o).concat(n.name,"Width"));i[s+a].currentWidth-=r,i[s].currentWidth=r}else t&&localStorage.setItem("".concat(o).concat(n.name,"Width"),n.currentWidth),i[s+a].currentWidth+=n.currentWidth,i[s].currentWidth=0;n.isHide=!n.isHide}if(n.name===e&&"right"===n.hide){if(n.isHide){var l=+localStorage.getItem("".concat(o).concat(n.name,"Width"));i[s-a].currentWidth-=l,i[s].currentWidth=l}else t&&localStorage.setItem("".concat(o).concat(n.name,"Width"),n.currentWidth),i[s-a].currentWidth+=n.currentWidth,i[s].currentWidth=0;n.isHide=!n.isHide}})),n.forEach((function(i,s){if(i.name===e&&/top/.test(i.hide)){if(i.isHide){var r=+localStorage.getItem("".concat(o).concat(i.name,"Height"));n[s+a].currentHeight-=r,n[s].currentHeight=r}else t&&localStorage.setItem("".concat(o).concat(i.name,"Height"),i.currentHeight),n[s+a].currentHeight+=i.currentHeight,n[s].currentHeight=0;i.isHide=!i.isHide}if(i.name===e&&/bottom/.test(i.hide)){if(i.isHide){var l=+localStorage.getItem("".concat(o).concat(i.name,"Height"));n[s-a].currentHeight-=l,n[s].currentHeight=l}else t&&localStorage.setItem("".concat(o).concat(i.name,"Height"),i.currentHeight),n[s-a].currentHeight+=i.currentHeight,n[s].currentHeight=0;i.isHide=!i.isHide}})),s.filter((function(t){t.isHide=t.name===e?!t.isHide:t.isHide})),this.updatePreviewStyle(),t&&this.updateGridStyle(!1,!0)},calTriggerPosition:function(){for(var e=this.rows,t=this.rowGaps,i=this.columns,n=this.columnGaps,a=this.differIndex,o=1,s=0,r=0,l=0;l<e.length-1;l++)s+=e[l].currentHeight,l%a===o&&(t[r]={start:s-e[l].currentHeight,end:s},r++);for(var c=0,h=0,d=0;d<i.length-1;d++)c+=i[d].currentWidth,d%a===o&&(n[h]={start:c-i[d].currentWidth,end:c},h++)},calFRValue:function(){var e=this,t=this.rowStructure,i=this.columnStructure,n=this.rows,a=this.columns,o=this.gridStyleKey,s=this.$refs.grid.getBoundingClientRect(),r=s.width,l=s.height,c=2;this.width=r,this.columnFR=(r-i.px)/i.fr,this.rowFR=(l-t.px)/t.fr,n.forEach((function(t){var i=t.currentHeight,n=parseInt(i);i.includes("fr")?t.currentHeight=n*e.rowFR:t.currentHeight=n})),a.forEach((function(t){var i=t.currentWidth,n=parseInt(i);i.includes("fr")?t.currentWidth=n*e.columnFR:t.currentWidth=n,t.hasOwnProperty("maxWidth")&&["debugger-tensor","offline-debugger-tensor","debugger","offline-debugger"].includes(o)&&(t.maxWidth=r/c)}))},calValueToFR:function(e,t){if(/fr/.test(e))return e;for(var i=e.split(" "),n=new G,a=0;a<i.length;a++)["0px",this.rowGap,this.columnGap].includes(i[a])||(i[a]=i[a].replace(/px/,"fr")),this.handleGridSize(i[a],n);return"column"===t&&(this.columnStructure=n),"row"===t&&(this.rowStructure=n),i.join(" ")},updatePreviewStyle:function(){var e=this,t=!(arguments.length>0&&void 0!==arguments[0])||arguments[0],i=this.previewStyle,n=this.rows,a=this.columns;if(this.changeAreasIsHide(),i.gridTemplateRows=n.map((function(e){return e.currentHeight+"px"})).join(" "),i.gridTemplateColumns=a.map((function(e){return e.currentWidth+"px"})).join(" "),t){var o={};this.areaList.forEach((function(t){e.gapAreas.columnGap.includes(t.name)||e.gapAreas.rowGap.includes(t.name)||(o[t.name]={isHide:t.isHide})})),this.$emit("resizeGridStyle",o)}},updateGridStyle:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]&&arguments[0],i=arguments.length>1&&void 0!==arguments[1]&&arguments[1],n=this.gridStyle,a=this.previewStyle,o=this.columns,s=this.rows,r=this.gridStyleKey,l=this.saveGridStyleObj;if(t||i||(o.forEach((function(t){["left","right"].includes(t.hide)&&t.currentWidth<parseInt(t.minWidth)&&!t.isHide&&(localStorage.setItem("".concat(r).concat(t.name,"Width"),parseInt(t.minWidth)),e.isDisplayView(t.name,t.isHide))})),s.forEach((function(t){["top","bottom"].includes(t.hide)&&t.currentHeight<parseInt(t.minHeight)&&!t.isHide&&(localStorage.setItem("".concat(r).concat(t.name,"Height"),parseInt(t.minHeight)),e.isDisplayView(t.name,t.isHide))}))),t&&(n.gridTemplateAreas=a.gridTemplateAreas),n.gridTemplateRows=a.gridTemplateRows,n.gridTemplateColumns=a.gridTemplateColumns,localStorage.saveGridStyle){var c=JSON.parse(localStorage.saveGridStyle),h=c.find((function(e){return e.key===r}));void 0!==h&&(h.value=n,localStorage.setItem("saveGridStyle",JSON.stringify(c)))}else{var d=l.find((function(e){return e.key===r}));void 0!==d&&(d.value=n,localStorage.setItem("saveGridStyle",JSON.stringify(l)))}["debugger-tensor","offline-debugger-tensor"].includes(r)&&!t&&(this.callBackTimer=setTimeout((function(){e.$emit("resizeCallback")}),this.resizeDelayTime)),this.calTriggerPosition()},getSaveGridStyle:function(){var e=arguments.length>0&&void 0!==arguments[0]&&arguments[0],t=this.gridStyle,i=this.gridStyleKey;if(localStorage.saveGridStyle){var n=JSON.parse(localStorage.saveGridStyle).find((function(e){return e.key===i}));if(void 0!==n&&n.value){var a=n.value,o=a.gridTemplateAreas,s=a.gridTemplateColumns,r=a.gridTemplateRows;t.gridTemplateAreas=o,t.gridTemplateColumns=this.calValueToFR(s,"column"),t.gridTemplateRows=this.calValueToFR(r,"row")}else this.initDefaultGridStyle(e)}else this.initDefaultGridStyle(e);var l=t.gridTemplateColumns.split(" "),c=t.gridTemplateRows.split(" ");this.columns.forEach((function(e,t){/px/.test(l[t])?e.currentWidth=l[t].replace(/px/,""):e.currentWidth=l[t]})),this.rows.forEach((function(e,t){/px/.test(c[t])?e.currentHeight=c[t].replace(/px/,""):e.currentHeight=c[t]}))},initDefaultGridStyle:function(e){var t=[];this.areas.forEach((function(e){t.push('"'.concat(e.join(" "),'"'))})),this.gridStyle.gridTemplateAreas=t.join(" "),e?(this.gridStyle.gridTemplateColumns=this.calValueToFR(this.gridStyle.gridTemplateColumns,"column"),this.gridStyle.gridTemplateRows=this.calValueToFR(this.gridStyle.gridTemplateRows,"row")):(this.gridStyle.gridTemplateColumns=this.calValueToFR(this.columns.map((function(e){return e.defaultWidth})).join(" "),"column"),this.gridStyle.gridTemplateRows=this.calValueToFR(this.rows.map((function(e){return e.defaultHeight})).join(" "),"row"))},calGapAreas:function(){var e=this.areas,t=this.differIndex,i=this.gapAreas,n=this.rowSize,a=this.columnSize,o=this.rowGap,s=this.columnGap,r=this.showFixed,l=1,c=1,h=1,d=1,u=1,p=[],f=1,g=1;e.forEach((function(e){e.forEach((function(n,a){a>=l&&a!==d&&(d=a,h+=t,c++),a>=l&&a===d&&(e.splice(h,0,"columnGap".concat(c)),i["columnGap"].includes("columnGap".concat(c))||i["columnGap"].push("columnGap".concat(c)))})),h=1,d=1,c=1}));for(var m=0;m<e.length;m++)if(m%t===l){for(var v=0;v<e[m].length;v++)e[m-l][v]===e[m][v]?p.push(e[m][v]):(p.push("rowGap".concat(u)),i["rowGap"].includes("rowGap".concat(u))||i["rowGap"].push("rowGap".concat(u)));e.splice(m,0,p),u++,p=[]}n.forEach((function(e,i){i>=l&&(i===l&&r?n.splice(f,0,"0px"):n.splice(f,0,o),f+=t)})),a.forEach((function(e,i){i>=l&&(a.splice(g,0,s),g+=t)}))},parseGridInfo:function(){var e=this,t=this.areas,i=this.rowSize,n=this.columnSize,a=this.areaMap,o=this.previewStyle,s=this.hideAreas,r=this.generateGridDescription(i,"row");this.rows=r.info,this.rowStructure=r.structure;var l=this.generateGridDescription(n,"column");this.columns=l.info,this.columnStructure=l.structure;var c=[];t.forEach((function(e,t){var i=[];e.forEach((function(e,n){i.push(e),a.hasOwnProperty(e)||(a[e]={name:e,rowIndex:[t],columnIndex:[n],hide:s[e],isHide:!1});var o=a[e];o.rowIndex.includes(t)||o.rowIndex.push(t),o.columnIndex.includes(n)||o.columnIndex.push(n)})),c.push('"'.concat(i.join(" "),'"'))})),this.areaList=Object.values(a);for(var h=[],d=[],u=0;u<this.areaList.length;u++)for(var p=0;p<this.areaList[u]["columnIndex"].length;p++)if(this.areaList[u]["columnIndex"].includes(u)){h.push(this.areaList[u]);break}var f=this.areaList.indexOf(h[h.length-1]);if(d=this.areaList.slice(f,this.areaList.length),this.columns.forEach((function(t,i){Object.assign(e.columns[i],h[i])})),this.rows.forEach((function(t,i){i!==e.rows.length&&Object.assign(e.rows[i],d[i])})),o.gridTemplateAreas=c.join(" "),o.gridTemplateRows=r.template,o.gridTemplateColumns=l.template,localStorage.saveGridStyle){var g=JSON.parse(localStorage.saveGridStyle).find((function(t){return t.key===e.gridStyleKey}));void 0!==g&&g.value?this.getSaveGridStyle():this.updateGridStyle(!0)}else this.updateGridStyle(!0)},generateGridDescription:function(e,t){var i=this,n=new G,a=[],o=[];return"row"===t?e.forEach((function(e){if("string"===typeof e)a.push(i.handleGridSize(e,n)),o.push({defaultHeight:e,currentHeight:e,minHeight:0});else if("object"===Object(O["a"])(e)&&e.hasOwnProperty("defaultHeight")){var t=e.defaultHeight;a.push(i.handleGridSize(t,n)),o.push({defaultHeight:t,currentHeight:t,minHeight:e.minHeight,maxHeight:e.maxHeight})}})):e.forEach((function(e){if("string"===typeof e)a.push(i.handleGridSize(e,n)),o.push({defaultWidth:e,currentWidth:e,minWidth:0});else if(e.hasOwnProperty("defaultWidth")){var t=e.defaultWidth;a.push(i.handleGridSize(t,n)),o.push({defaultWidth:t,currentWidth:t,minWidth:e.minWidth,maxWidth:e.maxWidth})}})),{info:o,structure:n,template:a.join(" ")}},handleGridSize:function(e,t){var i=parseInt(e);if(!Number.isNaN(i)){if(e.includes("px"))t.count++,t.px+=i;else{if(!e.includes("fr"))return;t.count++,t.fr+=i}return e}},handleMouseDown:function(e){this.showPreview=!0;for(var t,i,n=this.rowGaps,a=this.columnGaps,o=this.differIndex,s=e.offsetX,r=e.offsetY,l=0;l<n.length;l++){var c=n[l],h=c.start,d=c.end;if((!this.showFixed||0!==l)&&(h<=r+h&&r+h<=d)){t=o*l;break}}for(var u=0;u<a.length;u++){var p=a[u],f=p.start,g=p.end;if(f<=s+f&&s+f<=g){i=o*u;break}}"number"===typeof t&&(this.dragRow=t,r+=n[this.dragRow/o]["start"]),"number"===typeof i&&(this.dragColumn=i,s+=a[this.dragColumn/o]["start"]),this.mouseEvent={offsetX:s,offsetY:r}},handleMouseMove:function(e){var t=this.differIndex,i=this.columns,n=this.areaList,a=this.rows,o=this.width,s=this.columnGap,r="number"===typeof this.dragColumn,l="number"===typeof this.dragRow;if(r||l){var c=e.pageX,h=e.pageY,d=this.$refs.grid.getBoundingClientRect(),u=d.top,p=d.left,f=c-p,g=h-u;if(r){var m=this.dragColumn,v=this.dragColumn+t,b=f-this.mouseEvent.offsetX;"left"===i[m].hide&&(i[m].currentWidth+=b,parseInt(i[m].currentWidth)>=parseInt(i[m]["maxWidth"])?(n[m]["isHide"]=!1,i[m]["isHide"]=!1,i[m].currentWidth=parseInt(i[m]["maxWidth"]),i[v].currentWidth=o-parseInt(s)-parseInt(i[m]["maxWidth"])):i[v].currentWidth-=b),"right"===i[v].hide&&(i[v].currentWidth-=b,parseInt(i[v].currentWidth)>=parseInt(i[v]["maxWidth"])?(n[v]["isHide"]=!1,i[v]["isHide"]=!1,i[v].currentWidth=parseInt(i[v]["maxWidth"]),i[m].currentWidth=o-parseInt(s)-parseInt(i[v]["maxWidth"])):i[m].currentWidth+=b)}if(l){var w=this.dragRow,y=this.dragRow+t,C=g-this.mouseEvent.offsetY;a[w].currentHeight+=C,a[y].currentHeight-=C}this.mouseEvent={offsetX:f,offsetY:g},this.updatePreviewStyle()}},handleMouseLeave:function(){this.showPreview&&this.acceptChange()},handleMouseUp:function(){this.showPreview&&this.acceptChange()},acceptChange:function(){this.updateGridStyle(),this.showPreview=!1,this.dragRow=null,this.dragColumn=null,this.mouseEvent=null}}},q=V,Y=(i("3015"),Object(v["a"])(q,H,F,!1,null,"d53e5320",null)),U=Y.exports,K={select:c["select"],selectAll:c["selectAll"],zoom:c["zoom"]},X={mixins:[],components:{debuggerGridTable:z,FlexibleGrid:U},props:{row:{type:Object,default:function(){}},formateWatchpointParams:Function},data:function(){return{leftShow:!1,tensorList:[],tuningRule:"",tolerance:0,toleranceInput:0,showFilterInput:!0,gridType:"compare",dims:null,statisticsArr:[],tensorValue:[],loadingOption:{lock:!0,text:this.$t("public.dataLoading"),spinner:"el-icon-loading",background:"rgba(0, 0, 0, 0.3)"},curRowObj:this.row,tensorGraphData:{},tensorGraphviz:null,selectedNode:{},statistics:{},leftDataShow:!0,rightDataShow:!0,tuningAdvice:[],tuningAdviceTitle:"",watchPoints:[],callbackFun:null,graphShow:!0,statisticsKeys:["name","overall_avg","overall_count","overall_max","overall_min","overall_nan_count","overall_neg_inf_count","overall_neg_zero_count","overall_pos_inf_count","overall_pos_zero_count","overall_zero_count","overall_true_count","overall_false_count"],loadingInstance:{},initOver:!1,columnLimitNum:1e3,preTensorGraphs:[JSON.parse(JSON.stringify(this.row))],activeName:"tensorMsg",showTensorValue:!0,nodeStackContent:[],currentTensorGraphIndex:0,areas:[["left","rightFixed"],["left","rightTop"],["left","rightBottom"]],rowSize:["40px",{defaultHeight:"500px",minHeight:"227px",maxHeight:"1000px"},{defaultHeight:"1fr",minHeight:"210px",maxHeight:"1000px"}],columnSize:[{defaultWidth:"400px",minWidth:"243px",maxWidth:"700px"},"1fr"],hideAreas:{left:"left",rightTop:"top",rightBottom:"bottom"},showFixed:!0,gridStyleKey:this.$route.query.sessionId?"offline-debugger-tensor":"debugger-tensor"}},computed:{state:function(){return this.$parent.metadata.state}},mounted:function(){var e=this;this.$nextTick((function(){e.callbackFun=e.debounce(e.resizeCallback,200),window.addEventListener("resize",e.callbackFun),e.init()}))},methods:{resizeGridStyle:function(e){this.showTensorValue||e.rightTop.isHide||this.tensorValue.length||(this.showTensorValue=!e.rightTop.isHide,this.foldTensorValue()),this.showTensorValue=!e.rightTop.isHide},init:function(){this.loadingInstance=this.$loading(this.loadingOption),this.initOver=!1,this.showTensorValue||(this.initOver=!0),"value"===this.curRowObj.type?(this.gridType="value",this.showTensorValue&&this.viewValueDetail(this.curRowObj)):(this.gridType="compare",this.showTensorValue&&this.tensorComparisons(this.curRowObj)),this.getTensorGraphData(!0),this.getTensorHitsData()},foldTensorValue:function(){this.showTensorValue&&("compare"===this.gridType?this.tensorComparisons(this.curRowObj,this.dims,!0):this.viewValueDetail(this.curRowObj,this.dims,!0))},backToPreGraph:function(){this.currentTensorGraphIndex--,this.curRowObj=JSON.parse(JSON.stringify(this.preTensorGraphs[this.currentTensorGraphIndex])),this.resetTensor()},goToForwardGraph:function(){this.currentTensorGraphIndex++,this.curRowObj=JSON.parse(JSON.stringify(this.preTensorGraphs[this.currentTensorGraphIndex])),this.resetTensor()},getTensorGraphData:function(e){var t=this,i={tensor_name:this.curRowObj.name,graph_name:this.curRowObj.graph_name,rank_id:this.curRowObj.rank_id};h["a"].getTensorGraphData(i,this.curRowObj.sessionId).then((function(i){if(i&&i.data&&i.data.graph&&i.data.graph.nodes&&i.data.graph.nodes.length){t.graphShow=!0;var n=JSON.parse(JSON.stringify(i.data.graph.nodes));if(t.tensorGraphData={},n.forEach((function(e){t.tensorGraphData[e.name]=Object(r["a"])(Object(r["a"])({},e),{},{type:"node"}),e.slots&&e.slots.length&&e.slots.forEach((function(i){var n=Object(r["a"])(Object(r["a"])({},i),{},{name:"".concat(e.name,":").concat(i.slot),graph_name:e.graph_name,type:"slot"});t.tensorGraphData[n.name]=n}))})),e){t.selectedNode.name=t.curRowObj.name;var a=t.packageData();t.initGraph(a)}else t.selectedNode.name&&t.setNodeData()}else t.graphShow=!1,t.rightDataShow=!1,t.statisticsKeys.forEach((function(e){t.statistics[e]="--"}))}),(function(e){t.graphShow=!1,t.statisticsKeys.forEach((function(e){t.statistics[e]="--"})),t.dealLoading()}))},updateGraphData:function(e,t){e===this.curRowObj.graph_name&&t===this.curRowObj.name&&this.getTensorGraphData(!1)},getTensorHitsData:function(){var e=this,t={tensor_name:this.curRowObj.name,graph_name:this.curRowObj.graph_name,rank_id:this.curRowObj.rank_id};h["a"].tensorHitsData(t,this.curRowObj.sessionId).then((function(t){if(t&&t.data&&t.data.watch_points&&t.data.watch_points.length){e.leftDataShow=!0,e.tensorList=t.data.watch_points.map((function(t){return{id:t.id,condition:t.watch_condition.id,params:t.watch_condition.params||[],selected:!1,tip:t.error_list&&t.error_list.length?t.error_list.map((function(t){return e.$t("debugger.checkTips")[t]})).join("")+e.$t("debugger.checkTips").cannotCheck:""}}));var i=e.$t("debugger.tensorTuningAdvice");e.tensorList.forEach((function(t){var n=i[t.condition];n?(t.tuningAdviceTitle=n[1],t.tuningAdvice=n[2]):t.tuningAdviceTitle=e.$t("debugger.noAdvice"),e.formateWatchpointParams(t.params)}))}else e.leftDataShow=!1}),(function(t){e.leftDataShow=!1,e.$parent.showErrorMsg(t)}))},getFormateWatchPoint:function(e){var t="";return e.params&&e.params.length&&(this.formateWatchpointParams(e.params),t=e.params.map((function(e){return e.content})).join("; ")),"".concat(this.$parent.transCondition(e.condition)).concat(t?" (".concat(t,")"):"")},packageData:function(){var e=this,t="",i="",n=[];Object.keys(this.tensorGraphData).forEach((function(i){var a=e.tensorGraphData[i];"node"===a.type&&(t+="<".concat(a.name,'>[id="').concat(a.name,'";label="').concat(a.name.split("/").pop(),'";shape="ellipse";class="operator"];')),a.slots&&a.slots.length&&(t+=e.packageSubGraph(a)),a.input&&Object.keys(a.input).forEach((function(e){var t=a.input[e].slot_mapping;t&&t.length&&t.forEach((function(t){t&&t.length&&(n.push({source:"".concat(e,":").concat(t[0]),target:"outputOf".concat(e,"_slots"),count:1}),n.push({source:"outputOf".concat(e,"_slots"),target:"".concat(a.name),count:1}))}))}))})),this.$parent.uniqueEdges(n),n.forEach((function(e){i+="<".concat(e.source,">-><").concat(e.target,'>[id="').concat(e.source,"->").concat(e.target,'";')+'label="'.concat(e.count>1?e.count+"tensor":"",'"]')}));var a='node[style="filled";fontsize="10px"];edge[fontsize="6px";];';return"digraph {compound=true;rankdir=TB;".concat(a).concat(t).concat(i,"}")},packageSubGraph:function(e){var t=this,i=e.slots,n=e.name,a="inputOf".concat(e.name,"_slots"),o="outputOf".concat(e.name,"_slots"),s="",r="",l="",c="cluster_".concat(n,"_slots");r+="{rank=min;<".concat(a,'>[shape="circle";')+'id="'.concat(a,'";width=0.02;fixedsize=true;')+'label=""]};',l+="<".concat(n,">-><").concat(a,'>[id="').concat(n,"->").concat(a,'";label="').concat(i.length,'tensor"]');var h=Object.keys(e.output||{});return h.length&&(r+="{rank=max;<".concat(o,'>[shape="circle";')+'id="'.concat(o,'";width=0.02;fixedsize=true;')+'label=""]};'),i.forEach((function(e){var i="".concat(n,":").concat(e.slot);r+="<".concat(i,'>[id="').concat(i,'";label="slot:').concat(e.slot).concat(t.getSlotWatchPointsAbbr(e),'";')+'shape="polygon";class="slot'.concat(i===t.curRowObj.name?" current selected":"",'";fillcolor="#c5e0b3"];'),l+="<".concat(a,">-><").concat(i,'>[id="').concat(a,"->").concat(i,'"];')})),s="subgraph <".concat(c,'>{style="filled";id="').concat(n,'_slots";')+'label="";fillcolor="#ffffff";'.concat(r,"};").concat(l),s},getSlotWatchPointsAbbr:function(e){var t="";if(e&&e.watch_points&&e.watch_points.length){var i=[];e.watch_points.forEach((function(e){e.watch_condition&&e.watch_condition.abbr&&i.push(e.watch_condition.abbr)})),i=Array.from(new Set(i)),t="[".concat(i.join(),"]")}return t},initGraph:function(e){try{this.tensorGraphviz=K.select("#tensor-graph").graphviz({useWorker:!1}).dot(e).attributer(this.$parent.attributer).render(this.startApp)}catch(i){var t=document.querySelector("#tensor-graph svg");t&&t.remove(),this.initGraph(e)}},startApp:function(){var e=this;setTimeout((function(){e.tensorGraphviz&&(e.tensorGraphviz._data=null,e.tensorGraphviz._dictionary=null,e.tensorGraphviz=null)}),100);var t=K.select("#tensor-graph");t.selectAll("title").remove(),this.initZooming(),this.fitGraph();var i=t.selectAll(".node");i.on("click",(function(t,i,n){var a=M["c"];a.stopPropagation(),a.preventDefault(),e.selectedNode.name=n[i].id,e.setNodeData()}));var n=t.selectAll(".node.slot");n.on("dblclick",(function(t,n,a){var o=M["c"];o.stopPropagation(),o.preventDefault();var s=a[n];if(s.id!==e.curRowObj.name){var r=e.tensorGraphData[s.id];e.curRowObj.name=r.name,e.curRowObj.full_name=r.full_name,e.curRowObj.graph_name=r.graph_name,e.curRowObj.has_prev_step=r.has_prev_step,e.curRowObj.type="value",e.curRowObj.shape=JSON.stringify(r.shape||[]),i.on("click",null),i.on("dblclick",null),e.resetTensor(),e.currentTensorGraphIndex++,e.preTensorGraphs.length-1>=e.currentTensorGraphIndex&&e.preTensorGraphs.splice(e.currentTensorGraphIndex),e.preTensorGraphs.push(JSON.parse(JSON.stringify(e.curRowObj)))}})),this.selectedNode.name&&this.setNodeData(),this.$nextTick((function(){e.dealLoading()}))},fitGraph:function(){var e=document.getElementById("tensor-graph"),t=e.querySelector("#graph0"),i=e.getBoundingClientRect(),n=t.getBoundingClientRect(),a=this.$parent.getTransformData(t),o=t.querySelector('g[id="'.concat(this.curRowObj.name,'"]')),s=o.getBoundingClientRect(),r=o.getBBox(),l=r.width/s.width,c=20;if(n.height<i.height/2){var h=(i.height-2*c)/n.height;t.setAttribute("transform","translate(".concat(a.translate[0],",").concat(a.translate[1],") scale(").concat(h*a.scale[0],")")),this.$nextTick((function(){s=o.getBoundingClientRect(),n=t.getBoundingClientRect();var e={x:s.x+s.width/2},r={x:i.x+i.width/2},d=(r.x-e.x)*l,u=(i.top+c-n.top)*l;d=parseFloat(d.toFixed(2)),u=parseFloat(u.toFixed(2)),h=parseFloat((h*a.scale[0]).toFixed(2)),t.setAttribute("transform","translate(".concat(a.translate[0]+d,",").concat(a.translate[1]+u,") scale(").concat(h,")"))}))}},initZooming:function(){var e=this,t=document.querySelector("#tensor-graph svg"),i=t.getBoundingClientRect(),n=document.querySelector("#tensor-graph #graph0"),a=n.getBBox(),o=n.getBoundingClientRect(),s={},r=Math.min(i.width/2/o.width,i.height/2/o.height),l=4,c=50,h={start:{x:0,y:0},end:{x:0,y:0}},d=K.zoom().on("start",(function(){var e=M["c"].sourceEvent;e&&(h.start.x=e.x,h.start.y=e.y)})).on("zoom",(function(){var t=M["c"].sourceEvent;if(t){var o=e.$parent.getTransformData(n);Object.keys(s).length||(s={x:o.translate[0],y:o.translate[1],k:o.scale[0]});var d="",u={},p=o.scale[0],f=n.getBoundingClientRect(),g=a.width/f.width;if("mousemove"===t.type){h.end.x=t.x,h.end.y=t.y;var m=h.end.x-h.start.x,v=h.end.y-h.start.y,b=Math.max(l/g/p,c);f.left+b+m>=i.left+i.width&&(m=Math.min(m,0)),f.left+f.width-b+m<=i.left&&(m=Math.max(m,0)),f.top+b+v>=i.top+i.height&&(v=Math.min(v,0)),f.top+f.height-b+v<=i.top&&(v=Math.max(v,0)),u={x:m*g*p,y:v*g*p},h.start.x=h.end.x,h.start.y=h.end.y}else if("wheel"===t.type){var w=-t.deltaY,y=1.2;p=w>0?o.scale[0]*y:o.scale[0]/y,p=Math.max(e.$parent.scaleRange[0],p,r),p=Math.min(e.$parent.scaleRange[1],p),u={x:(f.x+l/g-t.x)*g*(p-o.scale[0]),y:(f.bottom-l/g-t.y)*g*(p-o.scale[0])}}s={x:o.translate[0]+u.x,y:o.translate[1]+u.y,k:p},d="translate(".concat(s.x,",").concat(s.y,") scale(").concat(s.k,")"),n.setAttribute("transform",d),t.stopPropagation(),t.preventDefault()}})),u=K.select("#tensor-graph svg");u.on(".zoom",null),u.call(d),u.on("dblclick.zoom",null),u.on("wheel.zoom",null);var p=K.select("#tensor-graph #graph0");p.on(".zoom",null),p.call(d)},setNodeData:function(){var e=this;window.getSelection().removeAllRanges();var t=document.querySelector('#tensor-graph g[id="'.concat(this.selectedNode.name,'"]'));K.selectAll("#tensor-graph .node").classed("selected",!1),t&&t.classList.add("selected"),K.selectAll("#tensor-graph .edge").classed("selected",!1),this.selectedNode=JSON.parse(JSON.stringify(this.tensorGraphData[this.selectedNode.name])),"slot"===this.selectedNode.type?(this.selectedNode.statistics&&Object.keys(this.selectedNode.statistics).length?this.statistics=JSON.parse(JSON.stringify(this.selectedNode.statistics)):this.statisticsKeys.forEach((function(t){e.statistics[t]="--"})),this.statistics.name=JSON.parse(JSON.stringify(this.selectedNode.name)),this.selectedNode.watch_points&&this.selectedNode.watch_points.length?(this.watchPoints=this.selectedNode.watch_points.map((function(e){return{id:e.id,condition:e.watch_condition.id,params:e.watch_condition.params||[],selected:!1}})),this.rightDataShow=!0):this.rightDataShow=!1,this.nodeStackContent=JSON.parse(JSON.stringify(this.tensorGraphData[this.selectedNode.name.split(":")[0]].stack_info||[]))):(this.statisticsKeys.forEach((function(t){e.statistics[t]="--"})),this.rightDataShow=!1,this.highLightEdges(),this.nodeStackContent=JSON.parse(JSON.stringify(this.selectedNode.stack_info||[]))),this.$forceUpdate()},highLightEdges:function(){var e=this,t=[],i=this.selectedNode.input||{},n=Object.keys(i);n.length&&n.forEach((function(n){var a=i[n].slot_mapping;a&&a.length&&a.forEach((function(i){i&&i.length&&(t.push("".concat(n,":").concat(i[0],"->outputOf").concat(n,"_slots")),t.push("outputOf".concat(n,"_slots->").concat(e.selectedNode.name)))}))}));var a=this.selectedNode.slots||[];a.length&&(t.push("".concat(this.selectedNode.name,"->inputOf").concat(this.selectedNode.name,"_slots")),a.forEach((function(i){t.push("inputOf".concat(e.selectedNode.name,"_slots->").concat(e.selectedNode.name,":").concat(i.slot))}))),t.forEach((function(e){K.select('#tensor-graph g[id="'.concat(e,'"]')).classed("selected",!0)}))},resetTensor:function(){var e=document.querySelector("#tensor-graph svg");e&&e.remove();var t=this.$refs.tensorValue;t&&t.initCommonInfo(!0,[],{},"[0,0,:,:]"),this.tensorGraphData={},this.init()},closeTensor:function(){this.$emit("close",this.selectedNode.name,this.curRowObj.graph_name)},resizeCallback:function(){this.initZooming(),this.$refs.tensorValue&&this.$refs.tensorValue.resizeView()},debounce:function(e,t){var i=null;return function(){i&&clearTimeout(i),i=setTimeout(e,t)}},toleranceInputChange:function(){this.toleranceInput=this.tolerance},toleranceValueChange:function(e){e=e.replace(/[^0-9]+/g,""),0===Number(e)&&(this.toleranceInput=0,this.tolerance=0),Number(e)<0&&(this.tolerance=0,this.toleranceInput=0),Number(e)>0&&(Number(e)>100?(this.tolerance=100,this.toleranceInput=100):(this.tolerance=Number(e),this.toleranceInput=Number(e)))},formatTolenrance:function(e){return"".concat(e,"%")},tabChange:function(e){this.gridType=e,"compare"===this.gridType?this.tensorComparisons(this.curRowObj,this.dims,!0):this.viewValueDetail(this.curRowObj,this.dims,!0)},tensorComparisons:function(e,t){var i=this,n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],a=t||JSON.stringify(JSON.parse(e.shape).map((function(t,n){return n===JSON.parse(e.shape).length-1?t>i.columnLimitNum?"0:".concat(i.columnLimitNum):":":n===JSON.parse(e.shape).length-2?":":0}))).replace(/"/g,""),o={name:e.name,detail:"data",shape:a,tolerance:this.tolerance/100,graph_name:e.graph_name,rank_id:e.rank_id};n&&(this.loadingInstance=this.$loading(this.loadingOption)),h["a"].tensorComparisons(o,e.sessionId).then((function(t){if(t&&t.data&&t.data.tensor_value){"[]"===e.shape?i.showFilterInput=!1:i.showFilterInput=!0;var n=t.data.tensor_value,o=n.statistics||{};if(i.statisticsArr=[n.curr_step_statistics||{},n.prev_step_statistics||{},n.statistics||{}],"Too large to show."===n.diff)return i.tensorValue=[],i.$nextTick((function(){i.$refs.tensorValue.showRequestErrorMessage(i.$t("debugger.largeDataTip"),JSON.parse(e.shape),a,!0)})),void i.dealLoading();i.tensorValue=n.diff,i.tensorValue&&i.tensorValue instanceof Array&&!(i.tensorValue[0]instanceof Array)&&(i.tensorValue=[i.tensorValue]),i.$nextTick((function(){i.$refs.tensorValue.initCommonInfo(i.tensorValue,JSON.parse(e.shape),o,a)}))}else if("uncached"===t.data.tensor_status)return i.tensorValue=[],i.$nextTick((function(){i.$refs.tensorValue.showRequestErrorMessage(i.$t("debugger.largeDataLoading"),JSON.parse(e.shape),a,!0)})),void i.dealLoading();i.$nextTick((function(){i.dealLoading()}))}),(function(e){i.$parent.showErrorMsg(e),i.dealLoading()}))},tensorFilterChange:function(e){this.dims="[".concat(e.toString(),"]"),"compare"===this.gridType?this.tensorComparisons(this.curRowObj,this.dims,!0):this.viewValueDetail(this.curRowObj,this.dims,!0)},viewValueDetail:function(e,t){var i=this,n=arguments.length>2&&void 0!==arguments[2]&&arguments[2],a=t||JSON.stringify(JSON.parse(e.shape).map((function(t,n){return n===JSON.parse(e.shape).length-1?t>i.columnLimitNum?"0:".concat(i.columnLimitNum):":":n===JSON.parse(e.shape).length-2?":":0}))).replace(/"/g,""),o={name:e.name,detail:"data",shape:a,graph_name:e.graph_name,prev:"preStep"===this.gridType,rank_id:e.rank_id};n&&(this.loadingInstance=this.$loading(this.loadingOption)),h["a"].tensors(o,e.sessionId).then((function(t){if("[]"===e.shape?i.showFilterInput=!1:i.showFilterInput=!0,t.data.tensor_value){var n=t.data.tensor_value.value;i.curRowObj.value="".concat(n);var o=t.data.tensor_value.statistics||{},s=t.data.tensor_value.tensor_status;if(i.statisticsArr=[o],"Too large to show."===n)return i.tensorValue=[],i.$nextTick((function(){i.$refs.tensorValue.showRequestErrorMessage(i.$t("debugger.largeDataTip"),JSON.parse(e.shape),a,!0)})),void i.dealLoading();if("uncached"===s||"oversize"===s){var r=null;return"uncached"===s?r=i.$t("debugger.largeDataLoading"):"oversize"===s?(r=i.$t("debugger.overSize"),i.showFilterInput=!1):r=i.$t("debugger.largeDataTip"),i.tensorValue=[],i.$nextTick((function(){i.$refs.tensorValue.initCommonInfo(r,JSON.parse(e.shape),o,a)})),void i.dealLoading()}null===n&&(n="null"),i.tensorValue=n instanceof Array?n:[n],i.$nextTick((function(){i.$refs.tensorValue.initCommonInfo(i.tensorValue,JSON.parse(e.shape),o,a)})),i.curRowObj.bytes=t.data.tensor_value.bytes||0;var l=i.$parent.fileSizeConversion(i.curRowObj.bytes),c=Object(u["a"])(l,2),h=c[0],d=c[1];i.curRowObj.oversized=d===i.$parent.maxFileSize[1]&&h>i.$parent.maxFileSize[0]}i.$nextTick((function(){i.dealLoading()}))}),(function(e){i.$parent.showErrorMsg(e),i.dealLoading()}))},dealLoading:function(){this.initOver?this.loadingInstance.close():this.initOver=!0}},destroyed:function(){window.removeEventListener("resize",this.callbackFun)}},Q=X,J=(i("bf29"),Object(v["a"])(Q,y,C,!1,null,null,null)),Z=J.exports,ee=function(){var e=this,t=e.$createElement,i=e._self._c||t;return i("div",{staticClass:"el-tree",class:{"el-tree--highlight-current":e.highlightCurrent,"is-dragging":!!e.dragState.draggingNode,"is-drop-not-allow":!e.dragState.allowDrop,"is-drop-inner":"inner"===e.dragState.dropType},attrs:{role:"tree"}},[e._l(e.root.childNodes,(function(t){return i("el-tree-node",{key:e.getNodeKey(t),attrs:{node:t,props:e.props,"render-after-expand":e.renderAfterExpand,"show-checkbox":e.showCheckbox,disabled:e.disabled,"render-content":e.renderContent},on:{"node-expand":e.handleNodeExpand}})})),e.isEmpty?i("div",{staticClass:"el-tree__empty-block"},[i("span",{staticClass:"el-tree__empty-text"},[e._v(e._s(e.emptyText))])]):e._e(),i("div",{directives:[{name:"show",rawName:"v-show",value:e.dragState.showDropIndicator,expression:"dragState.showDropIndicator"}],ref:"dropIndicator",staticClass:"el-tree__drop-indicator"})],2)},te=[],ie=i("5c96"),ne=function(){var e=this,t=this,i=t.$createElement,n=t._self._c||i;return n("div",{directives:[{name:"show",rawName:"v-show",value:t.node.visible,expression:"node.visible"}],ref:"node",staticClass:"el-tree-node",class:{"is-expanded":t.expanded,"is-current":t.node.isCurrent,"is-hidden":!t.node.visible,"is-focusable":!t.node.disabled,"is-checked":!t.node.disabled&&t.node.checked},attrs:{role:"treeitem",tabindex:"-1","aria-expanded":t.expanded,"aria-disabled":t.node.disabled,"aria-checked":t.node.checked,draggable:t.tree.draggable},on:{click:function(e){return e.stopPropagation(),t.handleClick.apply(null,arguments)},contextmenu:function(t){return e.handleContextMenu(t)},dragstart:function(e){return e.stopPropagation(),t.handleDragStart.apply(null,arguments)},dragover:function(e){return e.stopPropagation(),t.handleDragOver.apply(null,arguments)},dragend:function(e){return e.stopPropagation(),t.handleDragEnd.apply(null,arguments)},drop:function(e){return e.stopPropagation(),t.handleDrop.apply(null,arguments)}}},[n("div",{staticClass:"el-tree-node__content",style:{"padding-left":(t.node.level-1)*t.tree.indent+"px"}},[n("span",{class:[{"is-leaf":t.node.isLeaf,expanded:!t.node.isLeaf&&t.expanded},"el-tree-node__expand-icon",t.tree.iconClass?t.tree.iconClass:"el-icon-caret-right"],on:{click:function(e){return e.stopPropagation(),t.handleExpandIconClick.apply(null,arguments)}}}),t.showCheckbox&&t.node.data.showCheckbox?n("el-checkbox",{attrs:{indeterminate:t.node.indeterminate,disabled:t.disabled||!!t.node.disabled},on:{change:t.handleCheckChange},nativeOn:{click:function(e){e.stopPropagation()}},model:{value:t.node.checked,callback:function(e){t.$set(t.node,"checked",e)},expression:"node.checked"}}):t._e(),t.node.loading?n("span",{staticClass:"el-tree-node__loading-icon el-icon-loading"}):t._e(),n("node-content",{attrs:{node:t.node}})],1),n("el-collapse-transition",[!t.renderAfterExpand||t.childNodeRendered?n("div",{directives:[{name:"show",rawName:"v-show",value:t.expanded,expression:"expanded"}],staticClass:"el-tree-node__children",attrs:{role:"group","aria-expanded":t.expanded}},t._l(t.node.childNodes,(function(e){return n("el-tree-node",{key:t.getNodeKey(e),attrs:{"render-content":t.renderContent,"render-after-expand":t.renderAfterExpand,"show-checkbox":t.showCheckbox,disabled:t.disabled,node:e},on:{"node-expand":t.handleChildNodeExpand}})})),1):t._e()])],1)},ae=[],oe=function(){var e=this,t=this,i=t.$createElement,n=t._self._c||i;return n("div",{directives:[{name:"show",rawName:"v-show",value:t.node.visible,expression:"node.visible"}],ref:"node",staticClass:"el-tree-node",class:{"is-expanded":t.expanded,"is-current":t.node.isCurrent,"is-hidden":!t.node.visible,"is-focusable":!t.node.disabled,"is-checked":!t.node.disabled&&t.node.checked},attrs:{role:"treeitem",tabindex:"-1","aria-expanded":t.expanded,"aria-disabled":t.node.disabled,"aria-checked":t.node.checked,draggable:t.tree.draggable},on:{click:function(e){return e.stopPropagation(),t.handleClick.apply(null,arguments)},contextmenu:function(t){return e.handleContextMenu(t)},dragstart:function(e){return e.stopPropagation(),t.handleDragStart.apply(null,arguments)},dragover:function(e){return e.stopPropagation(),t.handleDragOver.apply(null,arguments)},dragend:function(e){return e.stopPropagation(),t.handleDragEnd.apply(null,arguments)},drop:function(e){return e.stopPropagation(),t.handleDrop.apply(null,arguments)}}},[n("div",{staticClass:"el-tree-node__content",style:{"padding-left":(t.node.level-1)*t.tree.indent+"px"}},[n("span",{class:[{"is-leaf":t.node.isLeaf,expanded:!t.node.isLeaf&&t.expanded},"el-tree-node__expand-icon",t.tree.iconClass?t.tree.iconClass:"el-icon-caret-right"],on:{click:function(e){return e.stopPropagation(),t.handleExpandIconClick.apply(null,arguments)}}}),t.showCheckbox?n("el-checkbox",{attrs:{indeterminate:t.node.indeterminate,disabled:!!t.node.disabled},on:{change:t.handleCheckChange},nativeOn:{click:function(e){e.stopPropagation()}},model:{value:t.node.checked,callback:function(e){t.$set(t.node,"checked",e)},expression:"node.checked"}}):t._e(),t.node.loading?n("span",{staticClass:"el-tree-node__loading-icon el-icon-loading"}):t._e(),n("node-content",{attrs:{node:t.node}})],1),n("el-collapse-transition",[!t.renderAfterExpand||t.childNodeRendered?n("div",{directives:[{name:"show",rawName:"v-show",value:t.expanded,expression:"expanded"}],staticClass:"el-tree-node__children",attrs:{role:"group","aria-expanded":t.expanded}},t._l(t.node.childNodes,(function(e){return n("el-tree-node",{key:t.getNodeKey(e),attrs:{"render-content":t.renderContent,"render-after-expand":t.renderAfterExpand,"show-checkbox":t.showCheckbox,node:e},on:{"node-expand":t.handleChildNodeExpand}})})),1):t._e()])],1)},se=[],re=(i("466d"),i("2b0e")),le=re["default"].prototype.$isServer,ce=(le||Number(document.documentMode),function(e){return(e||"").replace(/^[\s\uFEFF]+|[\s\uFEFF]+$/g,"")});(function(){!le&&document.addEventListener})(),function(){!le&&document.removeEventListener}();function he(e,t){if(!e||!t)return!1;if(-1!==t.indexOf(" "))throw new Error("className should not contain space.");return e.classList?e.classList.contains(t):(" "+e.className+" ").indexOf(" "+t+" ")>-1}function de(e,t){if(e){for(var i=e.className,n=(t||"").split(" "),a=0,o=n.length;a<o;a++){var s=n[a];s&&(e.classList?e.classList.add(s):he(e,s)||(i+=" "+s))}e.classList||(e.className=i)}}function ue(e,t){if(e&&t){for(var i=t.split(" "),n=" "+e.className+" ",a=0,o=i.length;a<o;a++){var s=i[a];s&&(e.classList?e.classList.remove(s):he(e,s)&&(n=n.replace(" "+s+" "," ")))}e.classList||(e.className=ce(n))}}var pe=function(){function e(){Object(L["a"])(this,e)}return Object(W["a"])(e,[{key:"beforeEnter",value:function(e){de(e,"collapse-transition"),e.dataset||(e.dataset={}),e.dataset.oldPaddingTop=e.style.paddingTop,e.dataset.oldPaddingBottom=e.style.paddingBottom,e.style.height="0",e.style.paddingTop=0,e.style.paddingBottom=0}},{key:"enter",value:function(e){e.dataset.oldOverflow=e.style.overflow,0!==e.scrollHeight?(e.style.height=e.scrollHeight+"px",e.style.paddingTop=e.dataset.oldPaddingTop,e.style.paddingBottom=e.dataset.oldPaddingBottom):(e.style.height="",e.style.paddingTop=e.dataset.oldPaddingTop,e.style.paddingBottom=e.dataset.oldPaddingBottom),e.style.overflow="hidden"}},{key:"afterEnter",value:function(e){ue(e,"collapse-transition"),e.style.height="",e.style.overflow=e.dataset.oldOverflow}},{key:"beforeLeave",value:function(e){e.dataset||(e.dataset={}),e.dataset.oldPaddingTop=e.style.paddingTop,e.dataset.oldPaddingBottom=e.style.paddingBottom,e.dataset.oldOverflow=e.style.overflow,e.style.height=e.scrollHeight+"px",e.style.overflow="hidden"}},{key:"leave",value:function(e){0!==e.scrollHeight&&(de(e,"collapse-transition"),e.style.height=0,e.style.paddingTop=0,e.style.paddingBottom=0)}},{key:"afterLeave",value:function(e){ue(e,"collapse-transition"),e.style.height="",e.style.overflow=e.dataset.oldOverflow,e.style.paddingTop=e.dataset.oldPaddingTop,e.style.paddingBottom=e.dataset.oldPaddingBottom}}])}(),fe={name:"ElCollapseTransition",functional:!0,render:function(e,t){var i=t.children,n={on:new pe};return e("transition",n,i)}},ge=function(){var e=this,t=e.$createElement,i=e._self._c||t;return i("label",{staticClass:"el-checkbox",class:[e.border&&e.checkboxSize?"el-checkbox--"+e.checkboxSize:"",{"is-disabled":e.isDisabled},{"is-bordered":e.border},{"is-checked":e.isChecked}],attrs:{id:e.id}},[i("span",{staticClass:"el-checkbox__input",class:{"is-disabled":e.isDisabled,"is-checked":e.isChecked,"is-indeterminate":e.indeterminate,"is-focus":e.focus},attrs:{tabindex:!!e.indeterminate&&0,role:!!e.indeterminate&&"checkbox","aria-checked":!!e.indeterminate&&"mixed"}},[i("span",{staticClass:"el-checkbox__inner"}),e.trueLabel||e.falseLabel?i("input",{directives:[{name:"model",rawName:"v-model",value:e.model,expression:"model"}],staticClass:"el-checkbox__original",attrs:{type:"checkbox","aria-hidden":e.indeterminate?"true":"false",name:e.name,disabled:e.isDisabled,"true-value":e.trueLabel,"false-value":e.falseLabel},domProps:{checked:Array.isArray(e.model)?e._i(e.model,null)>-1:e._q(e.model,e.trueLabel)},on:{change:[function(t){var i=e.model,n=t.target,a=n.checked?e.trueLabel:e.falseLabel;if(Array.isArray(i)){var o=null,s=e._i(i,o);n.checked?s<0&&(e.model=i.concat([o])):s>-1&&(e.model=i.slice(0,s).concat(i.slice(s+1)))}else e.model=a},e.handleChange],focus:function(t){e.focus=!0},blur:function(t){e.focus=!1}}}):i("input",{directives:[{name:"model",rawName:"v-model",value:e.model,expression:"model"}],staticClass:"el-checkbox__original",attrs:{type:"checkbox","aria-hidden":e.indeterminate?"true":"false",disabled:e.isDisabled,name:e.name},domProps:{value:e.label,checked:Array.isArray(e.model)?e._i(e.model,e.label)>-1:e.model},on:{change:[function(t){var i=e.model,n=t.target,a=!!n.checked;if(Array.isArray(i)){var o=e.label,s=e._i(i,o);n.checked?s<0&&(e.model=i.concat([o])):s>-1&&(e.model=i.slice(0,s).concat(i.slice(s+1)))}else e.model=a},e.handleChange],focus:function(t){e.focus=!0},blur:function(t){e.focus=!1}}})]),e.$slots.default||e.label?i("span",{staticClass:"el-checkbox__label"},[e._t("default"),e.$slots.default?e._e():[e._v(e._s(e.label))]],2):e._e()])},me=[];function ve(e,t,i){this.$children.forEach((function(n){var a=n.$options.componentName;a===e?n.$emit.apply(n,[t].concat(i)):ve.apply(n,[e,t].concat([i]))}))}var be={methods:{dispatch:function(e,t,i){var n=this.$parent||this.$root,a=n.$options.componentName;while(n&&(!a||a!==e))n=n.$parent,n&&(a=n.$options.componentName);n&&n.$emit.apply(n,[t].concat(i))},broadcast:function(e,t,i){ve.call(this,e,t,i)}}},we={name:"ElCheckbox",mixins:[be],inject:{elForm:{default:""},elFormItem:{default:""}},componentName:"ElCheckbox",data:function(){return{selfModel:!1,focus:!1,isLimitExceeded:!1}},computed:{model:{get:function(){return this.isGroup?this.store:void 0!==this.value?this.value:this.selfModel},set:function(e){this.isGroup?(this.isLimitExceeded=!1,void 0!==this._checkboxGroup.min&&e.length<this._checkboxGroup.min&&(this.isLimitExceeded=!0),void 0!==this._checkboxGroup.max&&e.length>this._checkboxGroup.max&&(this.isLimitExceeded=!0),!1===this.isLimitExceeded&&this.dispatch("ElCheckboxGroup","input",[e])):(this.$emit("input",e),this.selfModel=e)}},isChecked:function(){return"[object Boolean]"==={}.toString.call(this.model)?this.model:Array.isArray(this.model)?this.model.indexOf(this.label)>-1:null!==this.model&&void 0!==this.model?this.model===this.trueLabel:void 0},isGroup:function(){var e=this.$parent;while(e){if("ElCheckboxGroup"===e.$options.componentName)return this._checkboxGroup=e,!0;e=e.$parent}return!1},store:function(){return this._checkboxGroup?this._checkboxGroup.value:this.value},isLimitDisabled:function(){var e=this._checkboxGroup,t=e.max,i=e.min;return!(!t&&!i)&&this.model.length>=t&&!this.isChecked||this.model.length<=i&&this.isChecked},isDisabled:function(){return this.isGroup?this._checkboxGroup.disabled||this.disabled||(this.elForm||{}).disabled||this.isLimitDisabled:this.disabled||(this.elForm||{}).disabled},_elFormItemSize:function(){return(this.elFormItem||{}).elFormItemSize},checkboxSize:function(){var e=this.size||this._elFormItemSize||(this.$ELEMENT||{}).size;return this.isGroup&&this._checkboxGroup.checkboxGroupSize||e}},props:{value:{},label:{},indeterminate:Boolean,disabled:Boolean,checked:Boolean,name:String,trueLabel:[String,Number],falseLabel:[String,Number],id:String,controls:String,border:Boolean,size:String},methods:{addToStore:function(){Array.isArray(this.model)&&-1===this.model.indexOf(this.label)?this.model.push(this.label):this.model=this.trueLabel||!0},handleChange:function(e){var t,i=this;this.isLimitExceeded||(t=e.target.checked?void 0===this.trueLabel||this.trueLabel:void 0!==this.falseLabel&&this.falseLabel,this.$emit("change",t,e),this.$nextTick((function(){i.isGroup&&i.dispatch("ElCheckboxGroup","change",[i._checkboxGroup.value])})))}},created:function(){this.checked&&this.addToStore()},mounted:function(){this.indeterminate&&this.$el.setAttribute("aria-controls",this.controls)},watch:{value:function(e){this.dispatch("ElFormItem","el.form.change",e)}}},ye=we,Ce=Object(v["a"])(ye,ge,me,!1,null,null,null),_e=Ce.exports;_e.install=function(e){e.component(_e.name,_e)};var xe=_e;const ke="$treeNodeId",Se=function(e,t){return e?t[e]:t[ke]};var Te={name:"ElTreeNode",componentName:"ElTreeNode",mixins:[be],props:{node:{default:function(){return{}}},props:{},renderContent:Function,renderAfterExpand:{type:Boolean,default:!0},showCheckbox:{type:Boolean,default:!1}},components:{ElCollapseTransition:fe,ElCheckbox:xe,NodeContent:{props:{node:{required:!0}},render:function(e){var t=this.$parent,i=t.tree,n=this.node,a=n.data,o=n.store;return t.renderContent?t.renderContent.call(t._renderProxy,e,{_self:i.$vnode.context,node:n,data:a,store:o}):i.$scopedSlots.default?i.$scopedSlots.default({node:n,data:a}):e("span",{class:"el-tree-node__label"},[n.label])}}},data:function(){return{tree:null,expanded:!1,childNodeRendered:!1,oldChecked:null,oldIndeterminate:null}},watch:{"node.indeterminate":function(e){this.handleSelectChange(this.node.checked,e)},"node.checked":function(e){this.handleSelectChange(e,this.node.indeterminate)},"node.expanded":function(e){var t=this;this.$nextTick((function(){return t.expanded=e})),e&&(this.childNodeRendered=!0)}},methods:{getNodeKey:function(e){return Se(this.tree.nodeKey,e.data)},handleSelectChange:function(e,t){this.oldChecked!==e&&this.oldIndeterminate!==t&&this.tree.$emit("check-change",this.node.data,e,t),this.oldChecked=e,this.indeterminate=t},handleClick:function(){var e=this.tree.store;e.setCurrentNode(this.node),this.tree.$emit("current-change",e.currentNode?e.currentNode.data:null,e.currentNode),this.tree.currentNode=this,this.tree.expandOnClickNode&&this.handleExpandIconClick(),this.tree.checkOnClickNode&&!this.node.disabled&&this.handleCheckChange(null,{target:{checked:!this.node.checked}}),this.tree.$emit("node-click",this.node.data,this.node,this)},handleContextMenu:function(e){this.tree._events["node-contextmenu"]&&this.tree._events["node-contextmenu"].length>0&&(e.stopPropagation(),e.preventDefault()),this.tree.$emit("node-contextmenu",e,this.node.data,this.node,this)},handleExpandIconClick:function(){this.node.isLeaf||(this.expanded?(this.tree.$emit("node-collapse",this.node.data,this.node,this),this.node.collapse()):(this.node.expand(),this.$emit("node-expand",this.node.data,this.node,this)))},handleCheckChange:function(e,t){var i=this;this.node.setChecked(t.target.checked,!this.tree.checkStrictly),this.$nextTick((function(){var e=i.tree.store;i.tree.$emit("check",i.node.data,{checkedNodes:e.getCheckedNodes(),checkedKeys:e.getCheckedKeys(),halfCheckedNodes:e.getHalfCheckedNodes(),halfCheckedKeys:e.getHalfCheckedKeys()})}))},handleChildNodeExpand:function(e,t,i){this.broadcast("ElTreeNode","tree-node-expand",t),this.tree.$emit("node-expand",e,t,i)},handleDragStart:function(e){this.tree.draggable&&this.tree.$emit("tree-node-drag-start",e,this)},handleDragOver:function(e){this.tree.draggable&&(this.tree.$emit("tree-node-drag-over",e,this),e.preventDefault())},handleDrop:function(e){e.preventDefault()},handleDragEnd:function(e){this.tree.draggable&&this.tree.$emit("tree-node-drag-end",e,this)}},created:function(){var e=this,t=this.$parent;t.isTree?this.tree=t:this.tree=t.tree;var i=this.tree;i||console.warn("Can not find node's tree.");var n=i.props||{},a=n["children"]||"children";this.$watch("node.data.".concat(a),(function(){e.node.updateChildren()})),this.node.expanded&&(this.expanded=!0,this.childNodeRendered=!0),this.tree.accordion&&this.$on("tree-node-expand",(function(t){e.node!==t&&e.node.collapse()}))}},$e=Te,De=Object(v["a"])($e,oe,se,!1,null,null,null),Ne=De.exports,Re={extends:Ne,name:"ElTreeNode",componentName:"ElTreeNode",props:{disabled:{type:Boolean,default:!1}}},Ie=Re,Pe=Object(v["a"])(Ie,ne,ae,!1,null,null,null),Ae=Pe.exports,Ee={extends:ie["Tree"],components:{ElTreeNode:Ae},props:{disabled:{type:Boolean,default:!1}}},ze=Ee,Me=Object(v["a"])(ze,ee,te,!1,null,null,null),He=Me.exports,Fe={select:c["select"],selectAll:c["selectAll"],zoom:c["zoom"],dispatch:c["dispatch"]},Oe={mixins:[d["a"],w],data:function(){return{leftShow:!1,showLegend:!0,searchWord:"",searchedWord:"",searchStackContent:"",searchedStackContent:"",tableData:[],currentRow:null,tipsFlag:!1,props:{label:"label",children:"children",isLeaf:"leaf"},defaultProps:{children:"nodes",label:"label",isLeaf:"leaf"},curNodeData:[],lazy:!0,radio1:"tree",nodeTypes:{options:["all","weight","activation","gradient"],label:{},value:"all"},watchPointArr:[],allWatchPointFlag:!0,enableRecheck:!1,dynamicTreeData:[],watchPointHits:[],defaultCheckedArr:[],origialTree:[],conditionCollections:[],createWatchPointArr:[],createWPDialogVisible:!1,validPram:!1,curWatchPointId:null,step:"",metadata:{},searchTreeData:[],treeFlag:!0,curCheckedName:"",isSelected:!1,searchCheckedArr:[],curHalfCheckedKeys:[],tensorValue:[],loadingOption:{lock:!0,text:this.$t("public.dataLoading"),spinner:"el-icon-loading",background:"rgba(0, 0, 0, 0.3)"},outputLength:0,inputLength:0,curLeafNodeName:null,graphFiles:{options:[],value:"",graphs:{}},logicCard:{options:[],value:""},devices:[],selectedNode:{},contextmenu:{point:{},id:"contextMenu",dom:null},code:"",version:"Ascend",nodeName:"",pollInit:!0,collapseTable:!1,dialogVisible:!1,initFail:!1,isIntoView:!0,tolerance:0,tensorCompareFlag:!1,gridType:"compare",curRowObj:{},dims:null,node:null,resolve:null,toleranceInput:0,showFilterInput:!0,currentNodeName:"",statisticsArr:[],tabs:{activeName:"tensor"},searchNode:null,searchResolve:null,isCurrentGraph:!0,expandKeys:[],isHitIntoView:!0,trainId:this.$route.query.dir,recommendWatchPointDialog:!1,hitsOutdated:!1,conflictFlag:!1,debuggerVersion:{},checkboxStatus:{unchecked:0,indeterminate:1,checked:2,noCheckbox:-1},percentParams:["zero_percentage_ge","range_percentage_lt","range_percentage_gt"],oldState:"",treeDisabled:!1,pagination:{currentPage:1,pageSize:10,pageCount:5,total:0},state:{running:"running",pending:"pending",mismatch:"mismatch",sending:"sending",waiting:"waiting",node_too_large:"node_too_large"},loadingInstance:null,paramErrorMsg:"",sessionId:this.$route.query.sessionId,isShowInp:!1,newStep:"",nodeStackContent:[],collapseConditions:!1,stacks:{list:[],total:0,currentPage:1,pageSize:{options:[10,20,50],size:10},searchContent:""},noOfflineGraph:!1,hitWpCondition:{graphFile:"",watchPoint:""},hitWatchPointArr:[],showGraphCount:!0,allGraphIdArr:[],graphIdArr:[],visibleGraphIdArr:[],graphCountWidth:120,showCount:5,graphNameObj:{value:this.$t("debugger.all"),options:[]},hasDataObj:{value:"all",options:[{label:this.$t("debugger.noDistinguish"),value:"all"},{label:this.$t("debugger.yes"),value:!0},{label:this.$t("debugger.no"),value:!1}]},noOfflineGraphName:!1,areas:[["left","rightTop"],["left","rightBottom"]],rowSize:[{defaultHeight:"550px",minHeight:"219px",maxHeight:"1000px"},{defaultHeight:"1fr",minHeight:"140px",maxHeight:"1000px"}],columnSize:[{defaultWidth:"400px",minWidth:"243px",maxWidth:"700px"},"1fr"],hideAreas:{left:"left",rightTop:"top",rightBottom:"bottom"},showFixed:!1,gridStyleKey:this.$route.query.sessionId?"offline-debugger":"debugger",isGraphRunsInit:!1,timer:null,themeIndex:this.$store.state.themeIndex,nodeDataIsLarge:!1}},components:{debuggerTensor:Z,tree:He,FlexibleGrid:U},mounted:function(){document.title="".concat(this.$t("debugger.debugger"),"-MindInsight"),this.nodeTypes.label=this.$t("debugger.nodeType"),this.pageKey="debugger",this.initDebugger(),window.addEventListener("resize",this.initSvgSize,!1)},watch:{"metadata.state":{handler:function(e,t){e===this.state.node_too_large?this.dialogVisible=!0:this.dialogVisible=!1,e===this.state.mismatch?this.conflictFlag=!0:this.conflictFlag=!1,e===this.state.pending?t?location.reload():this.dialogVisible=!0:this.dialogVisible=!1,e===this.state.running||e===this.state.sending?this.treeDisabled=!0:this.treeDisabled=!1,e===this.state.sending&&t&&(this.oldState=t),e===this.state.waiting&&(this.oldState===this.state.pending||t===this.state.pending?this.retrieveAll():this.oldState!==this.state.running&&t!==this.state.running||(this.pagination.currentPage=1,this.watchPointHits=[],this.pagination.total=0,this.searchWatchpointHits(!0)))},deep:!0},"metadata.step":{handler:function(){this.allGraphIdArr.length&&this.scrollGraphCount()}},watchPointArr:{handler:function(e){var t=this,i=e.map((function(e){return{label:t.getWatchPointContent(e),value:e.id}}));i.length>1&&i.unshift({label:this.$t("debugger.all"),value:this.$t("debugger.all")}),this.hitWpCondition.watchPoint=i.length>0?i[0].value:"",this.hitWatchPointArr=i}}},methods:{handleScroll:function(){var e=document.querySelector("#graph-count-container").scrollLeft;this.updateVisibleData(e)},updateVisibleData:function(e){if(this.graphIdArr.length){var t=20,i=Math.ceil((document.querySelector("#graph-count-container").clientWidth-t)/this.graphCountWidth),n=Math.floor(e/this.graphCountWidth),a=n+2*i;this.graphIdArr.length-n<i&&(n=n-i>0?n-i:0),this.visibleGraphIdArr=this.graphIdArr.slice(n,a),this.$refs.content.style.transform="translateX(".concat(n*this.graphCountWidth,"px)")}},resizeGridStyle:function(e){this.allGraphIdArr.length&&this.scrollGraphCount()},initDebugger:function(){this.trainId?(document.title="".concat(this.trainId,"-").concat(this.$t("debugger.debugger"),"-MindInsight"),this.retrieveAll(),this.queryStacks()):this.getSession()},getGraphRuns:function(){var e=this;h["a"].getGraphRuns(this.sessionId,this.logicCard.value).then((function(t){var i;if(e.isGraphRunsInit=!0,null!==(i=t.data.graph_runs)&&void 0!==i&&i.length){e.allGraphIdArr=t.data.graph_runs,e.graphIdArr=JSON.parse(JSON.stringify(e.allGraphIdArr));var n=Object(l["a"])(new Set(e.allGraphIdArr.map((function(e){return e.graph_name}))));n.length>1?e.graphNameObj.options=[e.$t("debugger.all")].concat(Object(l["a"])(n)):e.graphNameObj.options=n,e.scrollGraphCount()}else e.$message(e.$t("debugger.noExexutionGraph"))}),(function(){e.isGraphRunsInit=!0}))},graphExecutionSelect:function(){var e=this;this.graphIdArr=this.allGraphIdArr.filter((function(t){return e.graphNameObj.value===e.$t("debugger.all")||e.graphNameObj.value===t.graph_name})).filter((function(t){return"all"===e.hasDataObj.value||e.hasDataObj.value===t.has_data})),this.scrollGraphCount()},scrollGraphCount:function(){var e=this;this.$nextTick((function(){var t=e.graphIdArr.findIndex((function(t){return t.count===e.metadata.step})),i=20,n=(document.querySelector("#graph-count-container").clientWidth-i)/e.graphCountWidth,a=t<n?0:e.graphCountWidth*t;document.querySelector("#graph-count-container").scrollLeft=a,e.updateVisibleData(a)}))},stackOperator:function(e){e&&e.file_path&&(e.line_no||0===e.line_no)&&(this.searchStackContent="".concat(e.file_path,":").concat(e.line_no),this.radio1="tree",this.filter())},showTensor:function(e,t){this.curRowObj=JSON.parse(JSON.stringify(e)),this.curRowObj.type=t,this.curRowObj.curFileName=this.graphFiles.value,this.curRowObj.step=this.metadata.step,this.curRowObj.rank_id=this.logicCard.value,this.curRowObj.sessionId=this.sessionId,this.tensorCompareFlag=!0},closeTensor:function(e,t){this.tensorCompareFlag=!1,e&&t&&this.queryAllTreeData(e,!0,t,!0)},logicCardChange:function(){var e=this;this.graphFiles.options=JSON.parse(JSON.stringify(this.devices.find((function(t){return t.rank_id===e.logicCard.value})).graph_names)),this.graphFiles.options.length>1&&this.graphFiles.options.unshift(this.$t("debugger.all")),this.graphFiles.value!==this.graphFiles.options[0]&&(this.isCurrentGraph=!1),this.graphFiles.value=this.graphFiles.options[0];var t=this.devices.find((function(t){return t.rank_id===e.logicCard.value}));this.metadata.ip=t.server_ip,this.metadata.device_name=t.device_id,"all"!==this.nodeTypes.value||this.searchWord||this.searchStackContent?this.filter():this.queryGraphByFile()},queryGraphByFile:function(){var e=this,t={mode:"node",params:{watch_point_id:this.curWatchPointId?this.curWatchPointId:0,graph_name:this.graphFiles.value,rank_id:this.logicCard.value}};this.graphFiles.value===this.$t("debugger.all")&&delete t.params.graph_name,h["a"].retrieve(t,this.sessionId).then((function(t){if(t.data&&t.data.metadata&&e.dealMetadata(t.data.metadata),t.data&&t.data.graph){var i=t.data.graph;e.origialTree=t.data.graph.nodes.map((function(e){return Object(r["a"])(Object(r["a"])({label:e.name.split("/").pop(),leaf:"name_scope"!==e.type&&"aggregation_scope"!==e.type},e),{},{showCheckbox:-1!==e.watched})})),e.node.childNodes=[],e.resolve(e.origialTree),e.defaultCheckedArr=e.origialTree.filter((function(t){return t.watched===e.checkboxStatus.checked})).map((function(e){return e.name})),e.node.childNodes.forEach((function(t){t.data.watched===e.checkboxStatus.indeterminate&&(t.indeterminate=!0),t.data.watched===e.checkboxStatus.unchecked&&(t.checked=!1),t.data.watched===e.checkboxStatus.checked&&(t.checked=!0)})),e.firstFloorNodes=[],e.allGraphData={},Fe.select("#graph svg").remove(),e.selectedNode.name="",e.dealGraphData(JSON.parse(JSON.stringify(i.nodes))),e.tableData=[]}}),(function(t){e.showErrorMsg(t),e.resolve([])})),this.filter()},selectAllFiles:function(e){var t=this;if((e||this.$refs.tree.getCheckedKeys().length||this.$refs.tree.getHalfCheckedKeys().length)&&(!e||this.node.childNodes.find((function(e){return!1===e.checked})))){var i=[];if(this.node.childNodes.forEach((function(n){(e!==n.checked||!e&&n.indeterminate)&&i.push(n.data.name),n.indeterminate=!1,n.checked=e,n.data.watched=e?t.checkboxStatus.checked:t.checkboxStatus.unchecked,n.childNodes&&t.dealCheckPro(n.childNodes,e)})),e&&(i=this.$refs.tree.getCheckedKeys()),this.curWatchPointId){var n={watch_point_id:this.curWatchPointId?this.curWatchPointId:0,watch_nodes:i,mode:e?1:0,graph_name:this.graphFiles.value,rank_id:this.logicCard.value};this.graphFiles.value===this.$t("debugger.all")&&delete n.graph_name,h["a"].updateWatchpoint(n,this.sessionId).then((function(e){t.defaultCheckedArr=t.$refs.tree.getCheckedKeys(),e&&e.data&&e.data.metadata&&void 0!==e.data.metadata.enable_recheck&&(t.enableRecheck=e.data.metadata.enable_recheck)}),(function(e){t.showErrorMsg(e)}))}}},nodeTypesChange:function(){"all"!==this.nodeTypes.value||this.searchWord||this.searchStackContent?this.filter():(this.treeFlag=!0,this.node.level=0,this.queryGraphByFile())},queryGraphByWatchpoint:function(e){var t=this,i={mode:"watchpoint",params:{watch_point_id:e,graph_name:this.graphFiles.value,rank_id:this.logicCard.value}};this.graphFiles.value===this.$t("debugger.all")&&delete i.params.graph_name,h["a"].retrieve(i,this.sessionId).then((function(i){if(i.data&&i.data.graph){var n=i.data.graph;t.origialTree=n.nodes.map((function(e){return Object(r["a"])(Object(r["a"])({label:e.name.split("/").pop()},e),{},{showCheckbox:-1!==e.watched,disabled:e.disable})})),t.node.childNodes=[],t.curWatchPointId=e,t.resolve(t.origialTree),t.$refs.tree.getCheckedKeys().forEach((function(e){t.$refs.tree.setChecked(e,!1)})),t.defaultCheckedArr=t.origialTree.filter((function(e){return e.watched===t.checkboxStatus.checked})).map((function(e){return e.name}));var a=t.origialTree.filter((function(e){return e.watched===t.checkboxStatus.indeterminate})).map((function(e){return e.name}));t.node.childNodes.forEach((function(e){-1!==a.indexOf(e.data.name)&&(e.indeterminate=!0),e.data.watched===t.checkboxStatus.checked?e.checked=!0:e.data.watched===t.checkboxStatus.unchecked&&(e.checked=!1)})),t.tableData=[],t.firstFloorNodes=[],t.allGraphData={},Fe.select("#graph svg").remove(),t.selectedNode.name="",t.dealGraphData(JSON.parse(JSON.stringify(n.nodes)))}}),(function(e){t.showErrorMsg(e)}))},getWatchPointContent:function(e){var t=this,i="";return e.params.length&&(i=e.params.map((function(e){var i=t.transCondition(e.name),n=t.percentParams.includes(e.name)?"%":"";return"".concat(i," ").concat(e.value+n)})).join(", "),i="(".concat(i,")")),"".concat(this.$t("debugger.watchPoint")," ").concat(e.id,": ").concat(this.transCondition(e.condition)," ").concat(i)},formateWatchpointParams:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:[];t.forEach((function(t){var i=e.percentParams.includes(t.name)?"%":"",n="".concat(e.transCondition(t.name),": ").concat(e.$t("debugger.setValue")," ").concat(t.value+i);n+=t.actual_value||0===t.actual_value?", ".concat(e.$t("debugger.actualValue")," ").concat(t.actual_value+i):"",t.content=n}))},startApp:function(){var e=this;this.initSvgSize(!0),this.graph.dom=document.querySelector("#graph #graph0");var t=this.graph.dom.getBBox();this.graph.size={width:t.width,height:t.height};var i="";i=this.graph.dom.getAttribute("transform")?this.graph.dom.getAttribute("transform").split(/[(,)]/):["translate","0","0"," scale","1"],this.graph.transform={k:parseFloat(i[4]),x:parseFloat(i[1]),y:parseFloat(i[2])},this.graph.minScale=Math.min(this.svg.size.width/2/this.graph.size.width,this.svg.size.height/2/this.graph.size.height),this.initZooming(),this.initContextMenu(),this.selectedNode.name&&this.selectNode(!0,!0),this.$nextTick((function(){e.loadingInstance.close()}))},initContextMenu:function(){var e=this;this.contextmenu.dom=document.querySelector("#contextMenu");var t=document.querySelector("#graph svg"),i=["Parameter","Const","Depend","make_tuple","tuple_getitem","ControlDepend"],n=Fe.dispatch("start","contextmenu").on("start",(function(t){e.contextmenu.dom.style.display="none",e.contextmenu.point={x:t.x,y:t.y}})).on("contextmenu",(function(i){if(e.metadata.state===e.state.waiting){var n=t.getBoundingClientRect();e.contextmenu.dom.style.left="".concat(e.contextmenu.point.x-n.x,"px"),e.contextmenu.dom.style.top="".concat(e.contextmenu.point.y-n.y,"px"),e.contextmenu.dom.style.display="block",e.selectedNode.name=i.name,e.selectNode(!1,!0)}})),a=Fe.selectAll("g.node, g.cluster");a.on("contextmenu",(function(t,a,o){event.preventDefault();var s=e.allGraphData[o[a].id.replace("_unfold","")];s&&("GPU"!==e.version||i.includes(s.type)||s.type.endsWith("_scope")||s.type.endsWith("Summary")||setTimeout((function(){n.call("contextmenu",e,s)}),10))}),!0),document.oncontextmenu=function(t){n.apply("start",e,[t])},document.onmousedown=function(){e.contextmenu.dom.style.display="none"},this.contextmenu.dom.onmousedown=function(){e.continueTo()}},continueTo:function(){var e=this;this.pagination.currentPage=1,this.watchPointHits=[],this.pagination.total=0;var t={mode:"continue",level:"node",name:this.selectedNode.name.replace("_unfold",""),graph_name:this.graphFiles.value,rank_id:this.logicCard.value};this.graphFiles.value===this.$t("debugger.all")&&delete t.graph_name,h["a"].control(t,this.sessionId).then((function(t){t&&t.data&&t.data.metadata&&void 0!==t.data.metadata.enable_recheck&&(e.enableRecheck=t.data.metadata.enable_recheck)}),(function(t){e.showErrorMsg(t)}))},dealDoubleClick:function(e){var t=this,i=arguments.length>1&&void 0!==arguments[1]&&arguments[1];e=e.replace("_unfold",""),e&&this.allGraphData[e]&&(this.loadingInstance=this.$loading(this.loadingOption),this.$nextTick((function(){setTimeout((function(){t.allGraphData[e].isUnfold?(t.selectedNode.name=e,t.deleteNamespace(e)):i?t.loadingInstance.close():t.queryGraphData(e)}),200)})))},deleteNamespace:function(e){this.selectedNode.more?(this.allGraphData[e].isUnfold=!0,this.selectedNode.name="".concat(e,"_unfold"),this.layoutNamescope(e,!0)):(this.packageDataToObject(e,!1),this.layoutController(e))},queryGraphData:function(e){var t=this,i=this.allGraphData[e]?this.allGraphData[e].type:"name_scope",n=e?"node":"all",a={mode:n};e&&(a.params={watch_point_id:this.curWatchPointId?this.curWatchPointId:0,name:e,node_type:i,single_node:!1,graph_name:this.graphFiles.value,rank_id:this.logicCard.value},this.graphFiles.value===this.$t("debugger.all")&&delete a.params.graph_name),h["a"].retrieve(a,this.sessionId).then((function(i){if(i&&i.data&&i.data.graph){var n=i.data.graph,a=JSON.parse(JSON.stringify(n.nodes));t.nodeExpandLinkage(a,e),t.dealGraphData(a,e)}}),(function(e){t.showErrorMsg(e)})).catch((function(i){i&&i.includes("larger than maximum 65535 allowed")?t.$message.error(t.$t("graph.dataTooLarge")):t.$bus.$emit("showWarmText",!0),e&&t.allGraphData[e]&&(t.allGraphData[e].isUnfold=!1,t.allGraphData[e].children=[],t.allGraphData[e].size=[],t.allGraphData[e].html="")}))},dealGraphData:function(e,t){var i=3500,n=t?this.nodesCountLimit:i,a=!!this.allGraphData[t]&&this.allGraphData[t].independent_layout;if(!a&&e.length>n)this.$message.error(this.$t("graph.tooManyNodes")),this.packageDataToObject(t,!1),this.loadingInstance.close();else if(e&&e.length)if(this.packageDataToObject(t,!0,e),t){if("aggregation_scope"===this.allGraphData[t].type&&(this.dealAggregationNodes(t),this.allGraphData[t].maxChainNum>this.maxChainNum))return this.$message.error(this.$t("graph.tooManyChain")),this.allGraphData[t].isUnfold=!0,this.selectedNode.name=t,this.deleteNamespace(t),void this.loadingInstance.close();this.allGraphData[t].isUnfold=!0,this.selectedNode.name="".concat(t,"_unfold"),this.layoutNamescope(t,!0)}else this.initGraph()},selectNode:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]&&arguments[0],i=arguments.length>1?arguments[1]:void 0;window.getSelection().removeAllRanges(),Fe.selectAll(".node polygon, .node ellipse, .node rect, .node path").classed("selected",!1);var n=this.selectedNode.name.split("^"),a={},o=n[0].replace("_unfold","");if(this.selectedNode.name=o,a.data=this.allGraphData[o],a.data&&(o=this.allGraphData[o].isUnfold?"".concat(o,"_unfold"):o,a.dom=document.querySelector('#graph g[id="'.concat(o,'"]')),a.dom)){var s=n.length>1;(t||s)&&a.dom&&this.selectNodePosition(o,s),Fe.select('#graph g[id="'.concat(o,'"]')).select("polygon, rect, ellipse, path").classed("selected",!0),this.highlightProxyNodes(o.replace("_unfold","")),this.highLightEdges(a.data),this.$refs.tree.setCurrentKey(o.replace("_unfold","")),this.isIntoView&&this.$nextTick((function(){setTimeout((function(){var e=document.querySelector(".el-tree-node.is-current.is-focusable");if(e){var t=document.querySelector(".tree-wrap");t.scrollTop=e.offsetTop}}),800)})),this.isIntoView=!0;var r=this.allGraphData[n[0].replace("_unfold","")].type,l=["name_scope","aggregation_scope"];!i||this.selectedNode.name.includes("more...")||l.includes(r)||(this.graph.timer&&clearTimeout(this.graph.timer),this.graph.timer=setTimeout((function(){var t=n[0].replace("_unfold","");e.graphFiles.value===e.$t("debugger.all")?e.retrieveTensorHistory({name:t.replace("".concat(t.split("/")[0],"/"),"")},t.split("/")[0]):e.retrieveTensorHistory({name:t},e.graphFiles.value)}),500)),l.includes(r)?this.tableData=[]:this.graphFiles.value===this.$t("debugger.all")?this.nodeName=this.selectedNode.name.replace("".concat(this.selectedNode.name.split("/")[0],"/"),""):this.nodeName=this.selectedNode.name}"detail"===this.tabs.activeName?this.setSelectedNodeData(a.data):"stack"===this.tabs.activeName&&(this.nodeStackContent=JSON.parse(JSON.stringify(a.data.stack_info||[]))),this.watchPointHits.length&&"hit"===this.radio1&&this.isHitIntoView&&(this.focusWatchpointHit()||this.searchWatchpointHits(!0)),this.isHitIntoView=!0,this.loadingInstance.close()},setSelectedNodeData:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]?arguments[0]:{},i=[];this.selectedNode.inputNum=0,this.selectedNode.outputNum=0,t.output&&Object.keys(t.output).forEach((function(t){var n=e.graphFiles.value;e.graphFiles.value===e.$t("debugger.all")&&(n=t.split("/")[0],t=t.replace("".concat(n,"/"),""));var a={name:t,IOType:"output",graph_name:n,rank_id:e.logicCard.value};i.push(a),e.selectedNode.outputNum++})),t.input&&Object.keys(t.input).forEach((function(t){var n=e.graphFiles.value;e.graphFiles.value===e.$t("debugger.all")&&(n=t.split("/")[0],t=t.replace("".concat(n,"/"),""));var a={name:t,IOType:"input",graph_name:n,rank_id:e.logicCard.value};i.push(a),e.selectedNode.inputNum++})),this.selectedNode.IOInfo=i},selectNodePosition:function(e,t){var i=this,n=document.querySelector('#graph0 g[id="'.concat(e,'"]')),a=n.getBoundingClientRect(),o={};o.rect=this.graph.dom.getBoundingClientRect(),o.initWidth=o.rect.width/this.graph.transform.k,o.initHeight=o.rect.height/this.graph.transform.k;var s={x:a.left+a.width/2-(this.svg.size.left+this.svg.size.width/2),y:a.top+a.height/2-(this.svg.size.top+this.svg.size.height/2)};this.graph.transform.x-=s.x*(this.graph.size.width/o.initWidth),this.graph.transform.y-=s.y*(this.graph.size.height/o.initHeight),this.graph.dom.setAttribute("transform","translate(".concat(this.graph.transform.x,",")+"".concat(this.graph.transform.y,") scale(").concat(this.graph.transform.k,")"));var r=Math.min(2*Math.abs(s.x),2*Math.abs(s.y),t?800:0);this.graph.dom.style.transition="".concat(r/1e3,"s"),this.graph.dom.style["transition-timing-function"]="linear",setTimeout((function(){i.graph.dom.style.transition=""}),r)},resetGraph:function(){var e=this,t={};this.firstFloorNodes.forEach((function(i){var n=e.allGraphData[i];n.isUnfold=!1,n.children=[],n.size=[],n.html="",t[n.name]=n})),this.allGraphData=JSON.parse(JSON.stringify(t)),Fe.select("#graph svg").remove(),this.selectedNode.name="",this.initGraph()},querySingleNode:function(e,t,i){this.selectedNode.name=t,this.selectedNode.more=!1;var n=this.findStartUnfoldNode(e.children);if(n&&n.nodes&&n.nodes.length>this.maxSupportNodesNumber&&(this.$message.error(this.$t("graph.tooManyNodes")),this.loadingInstance.close()),this.allGraphData[t])if(Fe.select('g[id="'.concat(t,'"], g[id="').concat(t,'_unfold"]')).size())this.selectNode(!0,i),this.loadingInstance.close();else{var a=t.substring(0,t.lastIndexOf("/"));if(this.allGraphData[a]&&this.allGraphData[a].isUnfold){var o=this.allGraphData[a];if(o&&o.childIdsList)for(var s=0;s<o.childIdsList.length;s++)if(o.childIdsList[s].includes(t)){o.index=s;break}this.selectedNode.name=t,this.layoutNamescope(a,!0)}}else n&&this.dealAutoUnfoldNamescopesData(n)},dealAutoUnfoldNamescopesData:function(e){if(!e.scope_name)return this.dealAutoUnfoldNamescopesData(e.children);if(this.allGraphData[e.scope_name]&&this.allGraphData[e.scope_name].isUnfold)return this.dealAutoUnfoldNamescopesData(e.children);if("name_scope"===this.allGraphData[e.scope_name].type&&e.nodes.length>this.nodesCountLimit)this.selectedNode.name=e.scope_name,this.querySingleNode(e,e.scope_name,!0),this.selectNode(!0,!0),this.loadingInstance.close(),this.$message.error(this.$t("graph.tooManyNodes"));else{var t=JSON.parse(JSON.stringify(e.nodes));if(this.packageDataToObject(e.scope_name,!0,t),this.allGraphData[e.scope_name]&&"aggregation_scope"===this.allGraphData[e.scope_name].type){this.dealAggregationNodes(e.scope_name);var i=this.allGraphData[e.scope_name];if(i)for(var n=0;n<i.childIdsList.length;n++)if(i.childIdsList[n].includes(this.selectedNode.name)){i.index=n;break}if(this.allGraphData[e.scope_name].maxChainNum>this.maxChainNum)return this.selectedNode.name=e.scope_name,this.allGraphData[e.scope_name].isUnfold=!1,this.deleteNamespace(e.scope_name),this.loadingInstance.close(),void this.$message.error(this.$t("graph.tooManyChain"))}e.children.scope_name?this.dealAutoUnfoldNamescopesData(e.children):this.layoutNamescope(e.scope_name,!0)}},showErrorMsg:function(e){if(this.loadingInstance&&this.loadingInstance.close(),"ECONNABORTED"===e.code&&/^timeout/.test(e.message))this.$message.error(this.$t("public.timeout"));else if(e&&e.response&&e.response.data&&e.response.data.error_code)if(this.$t("error")["".concat(e.response.data.error_code)])this.$message.error(this.$t("error")["".concat(e.response.data.error_code)]);else{if("5054B101"===e.response.data.error_code)return;this.$message.error(e.response.data.error_msg)}},tabsChange:function(){var e=this.allGraphData[this.selectedNode.name];e&&("detail"===this.tabs.activeName?this.setSelectedNodeData(e):"stack"===this.tabs.activeName&&(this.nodeStackContent=JSON.parse(JSON.stringify(e.stack_info||[]))))},toSummeryList:function(){if(this.dialogVisible=!1,this.nodeDataIsLarge)return this.nodeDataIsLarge=!1,void this.deleteSession();this.$router.push({path:"/summary-manage"})},initSvgSize:function(){var e=this,t=arguments.length>0&&void 0!==arguments[0]&&arguments[0],i=function(){var t=document.querySelector("#graph svg").getBoundingClientRect();e.svg.size={width:t.width,height:t.height,left:t.left,top:t.top},e.resizeTimer=null};t?i():(this.resizeTimer&&clearTimeout(this.resizeTimer),this.resizeTimer=setTimeout(i,this.resizeDelay)),this.graphIdArr.length&&(this.timer&&clearTimeout(this.timer),this.timer=setTimeout((function(){e.scrollGraphCount()}),this.resizeDelay))},nodeIsDynamicShape:function(e){var t=e.is_dynamic_shape_node;return t}},destroyed:function(){window.removeEventListener("resize",this.initSvgSize)}},We=Oe,Le=(i("ea2e"),Object(v["a"])(We,o,s,!1,null,null,null));t["default"]=Le.exports},"5e85":function(e,t,i){e.exports=i.p+"static/img/icon9.8d29ce95.svg"},"60da":function(e,t,i){"use strict";var n=i("83ab"),a=i("d039"),o=i("df75"),s=i("7418"),r=i("d1e7"),l=i("7b0b"),c=i("44ad"),h=Object.assign,d=Object.defineProperty;e.exports=!h||a((function(){if(n&&1!==h({b:1},h(d({},"a",{enumerable:!0,get:function(){d(this,"b",{value:3,enumerable:!1})}}),{b:2})).b)return!0;var e={},t={},i=Symbol(),a="abcdefghijklmnopqrst";return e[i]=7,a.split("").forEach((function(e){t[e]=e})),7!=h({},e)[i]||o(h({},t)).join("")!=a}))?function(e,t){var i=l(e),a=arguments.length,h=1,d=s.f,u=r.f;while(a>h){var p,f=c(arguments[h++]),g=d?o(f).concat(d(f)):o(f),m=g.length,v=0;while(m>v)p=g[v++],n&&!u.call(f,p)||(i[p]=f[p])}return i}:h},6220:function(e,t,i){},6657:function(e,t,i){e.exports=i.p+"static/img/graph-stepimg1.0a7ae664.svg"},6780:function(e,t,i){},7012:function(e,t){e.exports="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAFkAAAAOCAYAAABEgLDgAAAAAXNSR0IArs4c6QAAAKtJREFUWAnt1FEKwyAQhOFYerBex1v0Fl7HnKyR4g8VxMwmj51AmJfZDX4Qt82PLFBKqe19yQO9mKID/9xvwJ9+/r3lO+dcFQ8jK0q984PMlIRtZLiEnCAztcQ2MkxCLpCZnmIbGR4hBWS2DNgpMMgCpy7wxX7ofTevCvi6CMgF/vrhungGvuHqucCAS93ISNzLKS4rjYzEtVzistLISMRSwmWlkZHQMoTLygPMOjzZRkfJYgAAAABJRU5ErkJggg=="},"70a2":function(e,t){(function(e){function t(t){var i,n,a,o,s,r=this,l={groupItemMetadataProvider:null,inlineFilters:!1},c="id",h=[],d=[],u={},p=null,f=null,g=null,m=!1,v=!0,b={},w={},y=[],C=[],_={getter:null,formatter:null,comparer:function(e,t){return e.value===t.value?0:e.value>t.value?1:-1},predefinedValues:[],aggregators:[],aggregateEmpty:!1,aggregateCollapsed:!1,aggregateChildGroups:!1,collapsed:!1,displayTotalsRow:!0,lazyTotalsCalculation:!1},x=[],k=[],S=[],T=":|:",$=0,D=0,N=0,R=new Slick.Event,I=new Slick.Event,P=new Slick.Event,A=new Slick.Event,E=new Slick.Event,z=new Slick.Event,M=new Slick.Event,H=new Slick.Event;function F(){m=!0}function O(){m=!1,qe()}function W(e){b=e}function L(e){a=e}function j(e){var t;e=e||0;for(var i=e,n=h.length;i<n;i++){if(t=h[i][c],void 0===t)throw new Error("Each data element must implement a unique 'id' property");u[t]=i}}function B(){for(var e,t=0,i=h.length;t<i;t++)if(e=h[t][c],void 0===e||u[e]!==t)throw new Error("Each data element must implement a unique 'id' property")}function G(){return h}function V(){return c}function q(e,t){void 0!==t&&(c=t),h=y=e,u={},j(),B(),qe(),R.notify({idProperty:t},null,r)}function Y(e){E.notify(U(),null,r),void 0!=e.pageSize&&($=e.pageSize,D=$?Math.min(D,Math.max(0,Math.ceil(N/$)-1)):0),void 0!=e.pageNum&&(D=Math.min(e.pageNum,Math.max(0,Math.ceil(N/$)-1))),z.notify(U(),null,r),qe()}function U(){var e=$?Math.max(1,Math.ceil(N/$)):1;return{pageSize:$,pageNum:D,totalRows:N,totalPages:e,dataView:r}}function K(e,t){v=t,n=e,i=null,!1===t&&h.reverse(),h.sort(e),!1===t&&h.reverse(),u={},j(),qe()}function X(e,t){v=t,i=e,n=null;var a=Object.prototype.toString;Object.prototype.toString="function"==typeof e?e:function(){return this[e]},!1===t&&h.reverse(),h.sort(),Object.prototype.toString=a,!1===t&&h.reverse(),u={},j(),qe()}function Q(){n?K(n,v):i&&X(i,v)}function J(){return y}function Z(){return f}function ee(e){f=e,t.inlineFilters&&(o=Fe(),s=Oe()),qe()}function te(){return x}function ie(i){t.groupItemMetadataProvider||(t.groupItemMetadataProvider=new Slick.Data.GroupItemMetadataProvider),k=[],S=[],i=i||[],x=i instanceof Array?i:[i];for(var n=0;n<x.length;n++){var a=x[n]=e.extend(!0,{},_,x[n]);a.getterIsAFn="function"===typeof a.getter,a.compiledAccumulators=[];var o=a.aggregators.length;while(o--)a.compiledAccumulators[o]=He(a.aggregators[o]);S[n]={}}qe()}function ne(e,t,i){ie(null!=e?{getter:e,formatter:t,comparer:i}:[])}function ae(e,t){if(!x.length)throw new Error("At least one grouping must be specified before calling setAggregators().");x[0].aggregators=e,x[0].aggregateCollapsed=t,ie(x)}function oe(e){return h[e]}function se(e){return u[e]}function re(){if(!p){p={};for(var e=0,t=d.length;e<t;e++)p[d[e][c]]=e}}function le(e){return re(),p[e[c]]}function ce(e){return re(),p[e]}function he(e){return h[u[e]]}function de(e){var t=[];re();for(var i=0,n=e.length;i<n;i++){var a=p[e[i][c]];null!=a&&(t[t.length]=a)}return t}function ue(e){var t=[];re();for(var i=0,n=e.length;i<n;i++){var a=p[e[i]];null!=a&&(t[t.length]=a)}return t}function pe(e){for(var t=[],i=0,n=e.length;i<n;i++)e[i]<d.length&&(t[t.length]=d[e[i]][c]);return t}function fe(e,t){if(void 0===u[e])throw new Error("Invalid id");if(e!==t[c]){var i=t[c];if(null==i)throw new Error("Cannot update item to associate with a null id");if(void 0!==u[i])throw new Error("Cannot update item to associate with a non-unique id");u[i]=u[e],delete u[e],g&&g[e]&&delete g[e],e=i}h[u[e]]=t,g||(g={}),g[e]=!0,qe()}function ge(e,t){h.splice(e,0,t),j(e),qe()}function me(e){h.push(e),j(h.length-1),qe()}function ve(e){var t=u[e];if(void 0===t)throw new Error("Invalid id");delete u[e],h.splice(t,1),j(t),qe()}function be(e){if(!n)throw new Error("sortedAddItem() requires a sort comparer, use sort()");ge(ye(e),e)}function we(e,t){if(void 0===u[e]||e!==t[c])throw new Error("Invalid or non-matching id "+u[e]);if(!n)throw new Error("sortedUpdateItem() requires a sort comparer, use sort()");var i=he(e);0!==n(i,t)?(ve(e),be(t)):fe(e,t)}function ye(e){var t=0,i=h.length;while(t<i){var a=t+i>>>1;-1===n(h[a],e)?t=a+1:i=a}return t}function Ce(){return d.length}function _e(e){var t=d[e];if(t&&t.__group&&t.totals&&!t.totals.initialized){var i=x[t.level];i.displayTotalsRow||(Pe(t.totals),t.title=i.formatter?i.formatter(t):t.value)}else t&&t.__groupTotals&&!t.initialized&&Pe(t);return t}function xe(e){var i=d[e];return void 0===i?null:i.__group?t.groupItemMetadataProvider.getGroupRowMetadata(i):i.__groupTotals?t.groupItemMetadataProvider.getTotalsRowMetadata(i):null}function ke(e,t){if(null==e)for(var i=0;i<x.length;i++)S[i]={},x[i].collapsed=t,!0===t?H.notify({level:i,groupingKey:null}):M.notify({level:i,groupingKey:null});else S[e]={},x[e].collapsed=t,!0===t?H.notify({level:e,groupingKey:null}):M.notify({level:e,groupingKey:null});qe()}function Se(e){ke(e,!0)}function Te(e){ke(e,!1)}function $e(e,t,i){S[e][t]=x[e].collapsed^i,qe()}function De(e){var t,i,n=Array.prototype.slice.call(arguments),a=n[0];1===n.length&&-1!==a.indexOf(T)?(t=a,i=a.split(T).length-1):(t=n.join(T),i=n.length-1),$e(i,t,!0),H.notify({level:i,groupingKey:t})}function Ne(e){var t,i,n=Array.prototype.slice.call(arguments),a=n[0];1===n.length&&-1!==a.indexOf(T)?(i=a.split(T).length-1,t=a):(i=n.length-1,t=n.join(T)),$e(i,t,!1),M.notify({level:i,groupingKey:t})}function Re(){return k}function Ie(e,t){for(var i,n,a,o=[],s={},r=t?t.level+1:0,l=x[r],c=0,h=l.predefinedValues.length;c<h;c++)n=l.predefinedValues[c],i=s[n],i||(i=new Slick.Group,i.value=n,i.level=r,i.groupingKey=(t?t.groupingKey+T:"")+n,o[o.length]=i,s[n]=i);for(c=0,h=e.length;c<h;c++)a=e[c],n=l.getterIsAFn?l.getter(a):a[l.getter],i=s[n],i||(i=new Slick.Group,i.value=n,i.level=r,i.groupingKey=(t?t.groupingKey+T:"")+n,o[o.length]=i,s[n]=i),i.rows[i.count++]=a;if(r<x.length-1)for(c=0;c<o.length;c++)i=o[c],i.groups=Ie(i.rows,i);return o.length&&Ee(o),o.sort(x[r].comparer),o}function Pe(e){var t,i=e.group,n=x[i.level],a=i.level==x.length,o=n.aggregators.length;if(!a&&n.aggregateChildGroups){var s=i.groups.length;while(s--)i.groups[s].totals.initialized||Pe(i.groups[s].totals)}while(o--)t=n.aggregators[o],t.init(),!a&&n.aggregateChildGroups?n.compiledAccumulators[o].call(t,i.groups):n.compiledAccumulators[o].call(t,i.rows),t.storeResult(e);e.initialized=!0}function Ae(e){var t=x[e.level],i=new Slick.GroupTotals;i.group=e,e.totals=i,t.lazyTotalsCalculation||Pe(i)}function Ee(e,t){t=t||0;var i,n=x[t],a=n.collapsed,o=S[t],s=e.length;while(s--)i=e[s],i.collapsed&&!n.aggregateCollapsed||(i.groups&&Ee(i.groups,t+1),n.aggregators.length&&(n.aggregateEmpty||i.rows.length||i.groups&&i.groups.length)&&Ae(i),i.collapsed=a^o[i.groupingKey],i.title=n.formatter?n.formatter(i):i.value)}function ze(e,t){t=t||0;for(var i,n,a=x[t],o=[],s=0,r=0,l=e.length;r<l;r++){if(n=e[r],o[s++]=n,!n.collapsed){i=n.groups?ze(n.groups,t+1):n.rows;for(var c=0,h=i.length;c<h;c++)o[s++]=i[c]}n.totals&&a.displayTotalsRow&&(!n.collapsed||a.aggregateCollapsed)&&(o[s++]=n.totals)}return o}function Me(e){var t=e.toString(),i=t.indexOf("function")>=0,n=i?/^function[^(]*\(([^)]*)\)\s*{([\s\S]*)}$/:/^[^(]*\(([^)]*)\)\s*{([\s\S]*)}$/,a=e.toString().match(n);return{params:a[1].split(","),body:a[2]}}function He(e){if(e.accumulate){var t=Me(e.accumulate),i=new Function("_items","for (var "+t.params[0]+", _i=0, _il=_items.length; _i<_il; _i++) {"+t.params[0]+" = _items[_i]; "+t.body+"}"),n="compiledAccumulatorLoop";return i.displayName=n,i.name=We(i,n),i}return function(){}}function Fe(){var e=Me(f),t="{ continue _coreloop; }$1",i="{ _retval[_idx++] = $item$; continue _coreloop; }$1",n=e.body.replace(/return false\s*([;}]|\}|$)/gi,t).replace(/return!1([;}]|\}|$)/gi,t).replace(/return true\s*([;}]|\}|$)/gi,i).replace(/return!0([;}]|\}|$)/gi,i).replace(/return ([^;}]+?)\s*([;}]|$)/gi,"{ if ($1) { _retval[_idx++] = $item$; }; continue _coreloop; }$2"),a=["var _retval = [], _idx = 0; ","var $item$, $args$ = _args; ","_coreloop: ","for (var _i = 0, _il = _items.length; _i < _il; _i++) { ","$item$ = _items[_i]; ","$filter$; ","} ","return _retval; "].join("");a=a.replace(/\$filter\$/gi,n),a=a.replace(/\$item\$/gi,e.params[0]),a=a.replace(/\$args\$/gi,e.params[1]);var o=new Function("_items,_args",a),s="compiledFilter";return o.displayName=s,o.name=We(o,s),o}function Oe(){var e=Me(f),t="{ continue _coreloop; }$1",i="{ _cache[_i] = true;_retval[_idx++] = $item$; continue _coreloop; }$1",n=e.body.replace(/return false\s*([;}]|\}|$)/gi,t).replace(/return!1([;}]|\}|$)/gi,t).replace(/return true\s*([;}]|\}|$)/gi,i).replace(/return!0([;}]|\}|$)/gi,i).replace(/return ([^;}]+?)\s*([;}]|$)/gi,"{ if ((_cache[_i] = $1)) { _retval[_idx++] = $item$; }; continue _coreloop; }$2"),a=["var _retval = [], _idx = 0; ","var $item$, $args$ = _args; ","_coreloop: ","for (var _i = 0, _il = _items.length; _i < _il; _i++) { ","$item$ = _items[_i]; ","if (_cache[_i]) { ","_retval[_idx++] = $item$; ","continue _coreloop; ","} ","$filter$; ","} ","return _retval; "].join("");a=a.replace(/\$filter\$/gi,n),a=a.replace(/\$item\$/gi,e.params[0]),a=a.replace(/\$args\$/gi,e.params[1]);var o=new Function("_items,_args,_cache",a),s="compiledFilterWithCaching";return o.displayName=s,o.name=We(o,s),o}function We(e,t){try{Object.defineProperty(e,"name",{writable:!0,value:t})}catch(i){e.name=t}}function Le(e,t){for(var i=[],n=0,a=0,o=e.length;a<o;a++)f(e[a],t)&&(i[n++]=e[a]);return i}function je(e,t,i){for(var n,a=[],o=0,s=0,r=e.length;s<r;s++)n=e[s],i[s]?a[o++]=n:f(n,t)&&(a[o++]=n,i[s]=!0);return a}function Be(e){if(f){var i=t.inlineFilters?o:Le,n=t.inlineFilters?s:je;b.isFilterNarrowing?y=i(y,a):b.isFilterExpanding?y=n(e,a,C):b.isFilterUnchanged||(y=i(e,a))}else y=$?e:e.concat();var r;return $?(y.length<=D*$&&(D=0===y.length?0:Math.floor((y.length-1)/$)),r=y.slice($*D,$*D+$)):r=y,{totalRows:y.length,rows:r}}function Ge(e,t){var i,n,a,o=[],s=0,r=Math.max(t.length,e.length);b&&b.ignoreDiffsBefore&&(s=Math.max(0,Math.min(t.length,b.ignoreDiffsBefore))),b&&b.ignoreDiffsAfter&&(r=Math.min(t.length,Math.max(0,b.ignoreDiffsAfter)));for(var l=s,h=e.length;l<r;l++)l>=h?o[o.length]=l:(i=t[l],n=e[l],(!i||x.length&&(a=i.__nonDataRow||n.__nonDataRow)&&i.__group!==n.__group||i.__group&&!i.equals(n)||a&&(i.__groupTotals||n.__groupTotals)||i[c]!=n[c]||g&&g[i[c]])&&(o[o.length]=l));return o}function Ve(e){p=null,b.isFilterNarrowing==w.isFilterNarrowing&&b.isFilterExpanding==w.isFilterExpanding||(C=[]);var t=Be(e);N=t.totalRows;var i=t.rows;k=[],x.length&&(k=Ie(i),k.length&&(i=ze(k)));var n=Ge(d,i);return d=i,n}function qe(){if(!m){var t=e.extend(!0,{},U()),i=d.length,n=N,a=Ve(h,f);$&&N<D*$&&(D=Math.max(0,Math.ceil(N/$)-1),a=Ve(h,f)),g=null,w=b,b={},n!==N&&(E.notify(t,null,r),z.notify(U(),null,r)),i!==d.length&&I.notify({previous:i,current:d.length,dataView:r,callingOnRowsChanged:a.length>0},null,r),a.length>0&&P.notify({rows:a,dataView:r,calledOnRowCountChanged:i!==d.length},null,r),(i!==d.length||a.length>0)&&A.notify({rowsDiff:a,previousRowCount:i,currentRowCount:d.length,rowCountChanged:i!==d.length,rowsChanged:a.length>0,dataView:r},null,r)}}function Ye(t,i,n){var a,o=this,s=o.mapRowsToIds(t.getSelectedRows()),r=new Slick.Event;function l(e){s.join(",")!=e.join(",")&&(s=e,r.notify({grid:t,ids:s,dataView:o},new Slick.EventData,o))}function c(){if(s.length>0){a=!0;var e=o.mapIdsToRows(s);i||l(o.mapRowsToIds(e)),t.setSelectedRows(e),a=!1}}return t.onSelectedRowsChanged.subscribe((function(i,r){if(!a){var c=o.mapRowsToIds(t.getSelectedRows());if(n&&t.getOptions().multiSelect){var h=e.grep(s,(function(e){return void 0===o.getRowById(e)}));l(h.concat(c))}else l(c)}})),this.onRowsOrCountChanged.subscribe(c),r}function Ue(e,t){var i,n;function a(e){for(var t in i={},e){var n=d[t][c];i[n]=e[t]}}function o(){if(i){n=!0,re();var a={};for(var o in i){var s=p[o];void 0!=s&&(a[s]=i[o])}e.setCellCssStyles(t,a),n=!1}}a(e.getCellCssStyles(t)),e.onCellCssStylesChanged.subscribe((function(i,s){n||t==s.key&&(s.hash?a(s.hash):(e.onCellCssStylesChanged.unsubscribe(),r.onRowsOrCountChanged.unsubscribe(o)))})),this.onRowsOrCountChanged.subscribe(o)}t=e.extend(!0,{},l,t),e.extend(this,{beginUpdate:F,endUpdate:O,setPagingOptions:Y,getPagingInfo:U,getIdPropertyName:V,getItems:G,setItems:q,setFilter:ee,getFilter:Z,getFilteredItems:J,sort:K,fastSort:X,reSort:Q,setGrouping:ie,getGrouping:te,groupBy:ne,setAggregators:ae,collapseAllGroups:Se,expandAllGroups:Te,collapseGroup:De,expandGroup:Ne,getGroups:Re,getIdxById:se,getRowByItem:le,getRowById:ce,getItemById:he,getItemByIdx:oe,mapItemsToRows:de,mapRowsToIds:pe,mapIdsToRows:ue,setRefreshHints:W,setFilterArgs:L,refresh:qe,updateItem:fe,insertItem:ge,addItem:me,deleteItem:ve,sortedAddItem:be,sortedUpdateItem:we,syncGridSelection:Ye,syncGridCellCssStyles:Ue,getLength:Ce,getItem:_e,getItemMetadata:xe,onSetItemsCalled:R,onRowCountChanged:I,onRowsChanged:P,onRowsOrCountChanged:A,onBeforePagingInfoChanged:E,onPagingInfoChanged:z,onGroupExpanded:M,onGroupCollapsed:H})}function i(e){this.field_=e,this.init=function(){this.count_=0,this.nonNullCount_=0,this.sum_=0},this.accumulate=function(e){var t=e[this.field_];this.count_++,null==t||""===t||isNaN(t)||(this.nonNullCount_++,this.sum_+=parseFloat(t))},this.storeResult=function(e){e.avg||(e.avg={}),0!==this.nonNullCount_&&(e.avg[this.field_]=this.sum_/this.nonNullCount_)}}function n(e){this.field_=e,this.init=function(){this.min_=null},this.accumulate=function(e){var t=e[this.field_];null==t||""===t||isNaN(t)||(null==this.min_||t<this.min_)&&(this.min_=t)},this.storeResult=function(e){e.min||(e.min={}),e.min[this.field_]=this.min_}}function a(e){this.field_=e,this.init=function(){this.max_=null},this.accumulate=function(e){var t=e[this.field_];null==t||""===t||isNaN(t)||(null==this.max_||t>this.max_)&&(this.max_=t)},this.storeResult=function(e){e.max||(e.max={}),e.max[this.field_]=this.max_}}function o(e){this.field_=e,this.init=function(){this.sum_=null},this.accumulate=function(e){var t=e[this.field_];null==t||""===t||isNaN(t)||(this.sum_+=parseFloat(t))},this.storeResult=function(e){e.sum||(e.sum={}),e.sum[this.field_]=this.sum_}}function s(e){this.field_=e,this.init=function(){},this.storeResult=function(e){e.count||(e.count={}),e.count[this.field_]=e.group.rows.length}}e.extend(!0,window,{Slick:{Data:{DataView:t,Aggregators:{Avg:i,Min:n,Max:a,Sum:o,Count:s}}}})})(jQuery)},"73d7":function(e,t,i){e.exports=i.p+"static/img/graph-stepimg3.e758bd1a.svg"},"77fb":function(e,t,i){e.exports=i.p+"static/img/name-scope.5935c318.svg"},"7d72":function(e,t){(function(e){function t(){var e=!1,t=!1;this.stopPropagation=function(){e=!0},this.isPropagationStopped=function(){return e},this.stopImmediatePropagation=function(){t=!0},this.isImmediatePropagationStopped=function(){return t}}function i(){var e=[];this.subscribe=function(t){e.push(t)},this.unsubscribe=function(t){for(var i=e.length-1;i>=0;i--)e[i]===t&&e.splice(i,1)},this.notify=function(i,n,a){var o;n=n||new t,a=a||this;for(var s=0;s<e.length&&!n.isPropagationStopped()&&!n.isImmediatePropagationStopped();s++)o=e[s].call(a,n,i);return o}}function n(){var e=[];this.subscribe=function(t,i){return e.push({event:t,handler:i}),t.subscribe(i),this},this.unsubscribe=function(t,i){var n=e.length;while(n--)if(e[n].event===t&&e[n].handler===i)return e.splice(n,1),void t.unsubscribe(i);return this},this.unsubscribeAll=function(){var t=e.length;while(t--)e[t].event.unsubscribe(e[t].handler);return e=[],this}}function a(e,t,i,n){void 0===i&&void 0===n&&(i=e,n=t),this.fromRow=Math.min(e,i),this.fromCell=Math.min(t,n),this.toRow=Math.max(e,i),this.toCell=Math.max(t,n),this.isSingleRow=function(){return this.fromRow==this.toRow},this.isSingleCell=function(){return this.fromRow==this.toRow&&this.fromCell==this.toCell},this.contains=function(e,t){return e>=this.fromRow&&e<=this.toRow&&t>=this.fromCell&&t<=this.toCell},this.toString=function(){return this.isSingleCell()?"("+this.fromRow+":"+this.fromCell+")":"("+this.fromRow+":"+this.fromCell+" - "+this.toRow+":"+this.toCell+")"}}function o(){this.__nonDataRow=!0}function s(){this.__group=!0,this.level=0,this.count=0,this.value=null,this.title=null,this.collapsed=!1,this.selectChecked=!1,this.totals=null,this.rows=[],this.groups=null,this.groupingKey=null}function r(){this.__groupTotals=!0,this.group=null,this.initialized=!1}function l(){var e=null;this.isActive=function(t){return t?e===t:null!==e},this.activate=function(t){if(t!==e){if(null!==e)throw new Error("SlickGrid.EditorLock.activate: an editController is still active, can't activate another editController");if(!t.commitCurrentEdit)throw new Error("SlickGrid.EditorLock.activate: editController must implement .commitCurrentEdit()");if(!t.cancelCurrentEdit)throw new Error("SlickGrid.EditorLock.activate: editController must implement .cancelCurrentEdit()");e=t}},this.deactivate=function(t){if(e!==t)throw new Error("SlickGrid.EditorLock.deactivate: specified editController is not the currently active one");e=null},this.commitCurrentEdit=function(){return!e||e.commitCurrentEdit()},this.cancelCurrentEdit=function(){return!e||e.cancelCurrentEdit()}}function c(t){var i={};function n(){a(t)}function a(e){e.forEach((function(e){i[e.id]=e,e.columns&&a(e.columns)}))}function o(e,t){return e.filter((function(e){var i=t.call(e);return i&&e.columns&&(e.columns=o(e.columns,t)),i&&(!e.columns||e.columns.length)}))}function s(e,t){e.sort((function(e,i){var n=r(t.getColumnIndex(e.id)),a=r(t.getColumnIndex(i.id));return n-a})).forEach((function(e){e.columns&&s(e.columns,t)}))}function r(e){return"undefined"===typeof e?-1:e}function l(e){if(!e.length)return e.columns?1+l(e.columns):1;for(var t in e)return l(e[t])}function c(e,t,i){var n=[];if(i=i||0,t==i)return e.length&&e.forEach((function(e){e.columns&&(e.extractColumns=function(){return h(e)})})),e;for(var a in e)e[a].columns&&(n=n.concat(c(e[a].columns,t,i+1)));return n}function h(e){var t=[];if(e.hasOwnProperty("length"))for(var i=0;i<e.length;i++)t=t.concat(h(e[i]));else{if(!e.hasOwnProperty("columns"))return e;t=t.concat(h(e.columns))}return t}function d(){return e.extend(!0,[],t)}n(),this.hasDepth=function(){for(var e in t)if(t[e].hasOwnProperty("columns"))return!0;return!1},this.getTreeColumns=function(){return t},this.extractColumns=function(){return this.hasDepth()?h(t):t},this.getDepth=function(){return l(t)},this.getColumnsInDepth=function(e){return c(t,e)},this.getColumnsInGroup=function(e){return h(e)},this.visibleColumns=function(){return o(d(),(function(){return this.visible}))},this.filter=function(e){return o(d(),e)},this.reOrder=function(e){return s(t,e)},this.getById=function(e){return i[e]},this.getInIds=function(e){return e.map((function(e){return i[e]}))}}e.extend(!0,window,{Slick:{Event:i,EventData:t,EventHandler:n,Range:a,NonDataRow:o,Group:s,GroupTotals:r,EditorLock:l,GlobalEditorLock:new l,TreeColumns:c,keyCode:{SPACE:8,BACKSPACE:8,DELETE:46,DOWN:40,END:35,ENTER:13,ESCAPE:27,HOME:36,INSERT:45,LEFT:37,PAGE_DOWN:34,PAGE_UP:33,RIGHT:39,TAB:9,UP:38,A:65},preClickClassName:"slick-edit-preclick",GridAutosizeColsMode:{None:"NOA",LegacyOff:"LOF",LegacyForceFit:"LFF",IgnoreViewport:"IGV",FitColsToViewport:"FCV",FitViewportToCols:"FVC"},ColAutosizeMode:{Locked:"LCK",Guide:"GUI",Content:"CON",ContentIntelligent:"CTI"},RowSelectionMode:{FirstRow:"FS1",FirstNRows:"FSN",AllRows:"ALL",LastRow:"LS1"},ValueFilterMode:{None:"NONE",DeDuplicate:"DEDP",GetGreatestAndSub:"GR8T",GetLongestTextAndSub:"LNSB",GetLongestText:"LNSC"},WidthEvalMode:{CanvasTextSize:"CANV",HTML:"HTML"}}}),s.prototype=new o,s.prototype.equals=function(e){return this.value===e.value&&this.count===e.count&&this.collapsed===e.collapsed&&this.title===e.title},r.prototype=new o})(jQuery)},"7db0":function(e,t,i){"use strict";var n=i("23e7"),a=i("b727").find,o=i("44d2"),s=i("ae40"),r="find",l=!0,c=s(r);r in[]&&Array(1)[r]((function(){l=!1})),n({target:"Array",proto:!0,forced:l||!c},{find:function(e){return a(this,e,arguments.length>1?arguments[1]:void 0)}}),o(r)},"81d5":function(e,t,i){"use strict";var n=i("7b0b"),a=i("23cb"),o=i("50c4");e.exports=function(e){var t=n(this),i=o(t.length),s=arguments.length,r=a(s>1?arguments[1]:void 0,i),l=s>2?arguments[2]:void 0,c=void 0===l?i:a(l,i);while(c>r)t[r++]=e;return t}},"81e5":function(e,t,i){e.exports=i.p+"static/img/graph-stepimg1.96306728.svg"},"841c":function(e,t,i){"use strict";var n=i("d784"),a=i("825a"),o=i("1d80"),s=i("129f"),r=i("14c3");n("search",1,(function(e,t,i){return[function(t){var i=o(this),n=void 0==t?void 0:t[e];return void 0!==n?n.call(t,i):new RegExp(t)[e](String(i))},function(e){var n=i(t,e,this);if(n.done)return n.value;var o=a(e),l=String(this),c=o.lastIndex;s(c,0)||(o.lastIndex=0);var h=r(o,l);return s(o.lastIndex,c)||(o.lastIndex=c),null===h?-1:h.index}]}))},"89d2":function(e,t,i){e.exports=i.p+"static/img/link.b8d48feb.svg"},"8a79":function(e,t,i){"use strict";var n=i("23e7"),a=i("06cf").f,o=i("50c4"),s=i("5a34"),r=i("1d80"),l=i("ab13"),c=i("c430"),h="".endsWith,d=Math.min,u=l("endsWith"),p=!c&&!u&&!!function(){var e=a(String.prototype,"endsWith");return e&&!e.writable}();n({target:"String",proto:!0,forced:!p&&!u},{endsWith:function(e){var t=String(r(this));s(e);var i=arguments.length>1?arguments[1]:void 0,n=o(t.length),a=void 0===i?n:d(o(i),n),l=String(e);return h?h.call(t,l,a):t.slice(a-l.length,a)===l}})},"8c7d":function(e,t,i){e.exports=i.p+"static/img/dynamic-shape-node.2b7eae99.svg"},"8cb0":function(e,t,i){e.exports=i.p+"static/img/creat-dataset.17b9b72d.svg"},"8d38":function(e,t,i){"use strict";var n,a,o=i("2909"),s=i("3835"),r=i("b85c"),l=(i("99af"),i("4de4"),i("a630"),i("caad"),i("e260"),i("a15b"),i("d81d"),i("fb6a"),i("a434"),i("b0c0"),i("a9e3"),i("b64b"),i("d3b7"),i("ac1f"),i("6062"),i("2532"),i("3ca3"),i("5319"),i("2ca0"),i("498a"),i("159b"),i("ddb0"),i("2dd7")),c=i("5698"),h=i("0165"),d=(i("6d21"),{select:c["select"],selectAll:c["selectAll"],zoom:c["zoom"]}),u={data:function(){return{graph:{dom:Object,size:Object,transform:Object,minScale:Number,defaultPadding:4},svg:{size:Object},graphviz:null,graphvizTemp:null,scaleRange:[.001,10],clickScope:{},frameSpace:25,curColorIndex:0,themeIndex:this.$store.state.themeIndex,totalMemory:33554432,allGraphData:{},firstFloorNodes:[],nodesCountLimit:1500,maxChainNum:70,graphContainer:null,resizeTimer:null,resizeDelay:500,pageKey:"",errorCount:0,maxErrorCount:3}},mounted:function(){this.$nextTick(this.init)},methods:{init:function(){d.select("#graphTemp").size()||d.select("body").append("div").attr("id","graphTemp"),d.select("#subgraphTemp").size()||d.select("body").append("div").attr("id","subgraphTemp"),this.graphContainer=document.querySelector("#graph"),this.graphContainer&&(this.graphContainer.addEventListener("click",this.controlEvent.bind(null,"click")),this.graphContainer.addEventListener("dblclick",this.controlEvent.bind(null,"dblclick"))),this.graphviz=d.select("#graph").graphviz({useWorker:!1,totalMemory:this.totalMemory}).attributer(this.attributer),this.graphvizTemp=d.select("#graphTemp").graphviz({useWorker:!1,totalMemory:this.totalMemory}).attributer((function(e){"polygon"===e.tag&&"transparent"!==e.attributes.stroke&&(e.attributes.stroke="rgb(120, 120, 120)")}))},controlEvent:function(e,t){if(t){for(var i,n=Array.from(t.path||t.composedPath&&t.composedPath()||[]),a=0;a<n.length;a++){var o=n[a].classList;if(o&&o.contains("node")){i=n[a];break}}i&&("click"===e?this.clickEvent(i):this.dblclickEvent(i))}},clickEvent:function(e){var t=this,i=e.id,n=e.classList.value;setTimeout((function(){t.clickScope={id:i,class:n}}),10),setTimeout((function(){t.clickScope={}}),1e3),this.selectedNode.name=i,"graph"===this.pageKey?this.selectNode(!1):"debugger"===this.pageKey&&(this.selectNode(!1,!0),this.contextmenu.dom.style.display="none")},dblclickEvent:function(e){var t=e.id,i=e.classList.value,n=t;this.selectedNode.more=-1!==n.indexOf("more...")&&-1===document.querySelector('#graph g[id="'.concat(n,'"]')).attributes.class.value.indexOf("plain");var a=(i.includes("aggregation")||i.includes("cluster")||this.selectedNode.more)&&(!this.clickScope.id||this.clickScope.id&&t===this.clickScope.id);if(this.selectedNode.more){var o=n.includes("right")?1:-1,s=document.querySelector('#graph g[id="'.concat(n,'"]')).parentNode.id;n=s.replace("_unfold",""),this.allGraphData[n].index+=o,this.allGraphData[n].index=Math.max(0,Math.min(this.allGraphData[n].index,this.allGraphData[n].childIdsList.length-1)),this.selectedNode.name=n}a?this.dealDoubleClick(n):this.clickScope.id&&(this.selectedNode.name=this.clickScope.id,"graph"===this.pageKey&&this.selectNode(!1)),this.nodeCollapseLinkage(this.selectedNode.name)},nodeCollapseLinkage:function(e){var t=this.$refs.tree.getNode(e.replace("_unfold",""));t&&(t.expanded=!1,t.loaded=!1,t.childNodes=[])},initGraph:function(){var e=this,t=this.packageGraphData(),i=function(){try{e.graphviz.dot(t).render((function(){e.errorCount=0,e.afterInitGraph()}))}catch(a){var n=document.querySelector("#graph svg");if(n&&n.remove(),e.errorCount++,!(e.errorCount<=e.maxErrorCount))return e.$message.error(e.$t("graph.openFail")),e.loadingInstance&&e.loadingInstance.close(),void(e.loading&&(e.loading.show=!1));i()}};i()},afterInitGraph:function(){var e=this,t=document.querySelector("#graph svg");t&&t.removeAttribute("viewBox"),setTimeout((function(){var t=["_data","_dictionary"];if(e.graphviz){var i,n=Object(r["a"])(t);try{for(n.s();!(i=n.n()).done;){var a=i.value;e.graphviz[a]=null}}catch(c){n.e(c)}finally{n.f()}}if(e.graphvizTemp){var o,s=Object(r["a"])(t);try{for(s.s();!(o=s.n()).done;){var l=o.value;e.graphvizTemp[l]=null}}catch(c){s.e(c)}finally{s.f()}}}),100),this.fitGraph("graph"),this.transplantChildrenDom();var i=document.querySelector("#subgraphTemp svg");i&&i.remove();var n=Array.from(d.select("#graph").selectAll("g.node, g.edge").nodes());n.forEach((function(e){if(e.hasAttribute("transform")||e.setAttribute("transform","translate(0,0)"),Array.prototype.includes.call(e.classList,"plain")){var t=e.querySelector("title");t.textContent=t.textContent.split("^")[0]}})),d.selectAll("g.edge>title").remove(),d.select("#graph g#graph0 title").remove(),this.startApp()},initZooming:function(){var e=this,t=100,i={start:{x:0,y:0},end:{x:0,y:0}},n=d.zoom().on("start",(function(){var t=h["c"].sourceEvent;i.start.x=t.x,i.start.y=t.y,"debugger"===e.pageKey&&(e.contextmenu.dom.style.display="none")})).on("zoom",(function(){var n=h["c"].sourceEvent;n.stopPropagation(),n.preventDefault();var a=e.getTransformData(e.graph.dom),o="",s={},r=a.scale[0];if("mousemove"===n.type)i.end.x=n.x,i.end.y=n.y,s={x:i.end.x-i.start.x,y:i.end.y-i.start.y},i.start.x=i.end.x,i.start.y=i.end.y;else if("wheel"===n.type){var l=-n.deltaY,c=1.2;r=l>0?a.scale[0]*c:a.scale[0]/c,r=Math.max(e.scaleRange[0],r,e.graph.minScale),r=Math.min(e.scaleRange[1],r),s={x:(e.svg.size.left-e.graph.defaultPadding-n.x+e.graph.transform.x)/a.scale[0]*(r-a.scale[0]),y:(e.svg.size.top+e.graph.transform.y-e.graph.defaultPadding-n.y)/a.scale[0]*(r-a.scale[0])}}e.graph.transform={x:a.translate[0]+s.x,y:a.translate[1]+s.y,k:r},e.graph.transform.x=Math.max(t-e.graph.size.width*r,e.graph.transform.x),e.graph.transform.x=Math.min(e.svg.size.width-t,e.graph.transform.x),e.graph.transform.y=Math.max(e.graph.transform.y,t),e.graph.transform.y=Math.min(e.graph.size.height*r+e.svg.size.height-t,e.graph.transform.y),o="translate(".concat(e.graph.transform.x,",").concat(e.graph.transform.y,") scale(").concat(e.graph.transform.k,")"),e.graph.dom.setAttribute("transform",o),"graph"===e.pageKey&&e.setInsideBoxData()})),a=d.select("#graph svg");a.on(".zoom",null),a.call(n),a.on("dblclick.zoom",null)},attributer:function(e,t,i){var n="ellipse"===e.tag||"circle"===e.tag||"polygon"===e.tag&&"transparent"!==e.attributes.stroke;if("svg"===e.tag){var a="100%",o="100%";e.attributes.width=a,e.attributes.height=o}else n&&(e.attributes.stroke="rgb(120, 120, 120)")},fitGraph:function(e){var t=document.getElementById(e),i=t.offsetWidth,n=t.offsetHeight,a=document.querySelector("#".concat(e," #graph0")),o=a.getBBox(),r=this.getTransformData(a),l=Object(s["a"])(r.translate,2),c=l[0],h=l[1],d=Object(s["a"])(r.scale,1),u=d[0];if(this.selectedNode.more)u=this.graph.transform.k;else if(o.width<=i&&o.height<=n){var p=.8;u=Math.min(i*p/o.width,n*p/o.height),c=(i-o.width*u)/2+u*this.graph.defaultPadding,h=(n+o.height*u)/2-u*this.graph.defaultPadding}else o.width>i&&o.height>n?c=-(o.width*u-i)/2:o.width<=i?c=(i-o.width*u)/2:o.height<=n&&(h=n/2+o.height/2);a.setAttribute("transform","translate(".concat(c,",").concat(h,") scale(").concat(u,")"))},layoutController:function(e){if("graph"!==this.pageKey||this.loading.show||(this.loading.info=this.$t("graph.searchLoading"),this.loading.show=!0),e.includes("/")){var t=e.split("/").slice(0,-1).join("/");this.layoutNamescope(t,!0)}else{var i=document.querySelector("#graph svg");i&&i.remove(),this.initGraph()}},layoutNamescope:function(e){var t=this,i=this.packageNamescope(e),n=function(){try{t.graphvizTemp.dot(i).render((function(){t.errorCount=0;var i=document.querySelector("#graphTemp svg");i&&i.removeAttribute("viewBox"),t.dealNamescopeTempGraph(e)}))}catch(o){var a=document.querySelector("#graphTemp svg");if(a&&a.remove(),t.errorCount++,!(t.errorCount<=t.maxErrorCount))return t.$message.error(t.$t("graph.openFail")),t.loadingInstance&&t.loadingInstance.close(),void(t.loading&&(t.loading.show=!1));n()}};n()},dealAggregationNodes:function(e){var t=this,i=10,n=[],a=1;this.allGraphData[e].children.forEach((function(i){var s=t.getAssociatedNode(i,"".concat(e,"/"));s.length&&n.push(s);var r=t.dealChainingData(i,"".concat(e,"/"));a=Math.max.apply(Math,Object(o["a"])(r).concat([a]))}));for(var s=[],r=[],l=0;l<n.length;l++)n[l].length>i?s.push(n[l]):r.length+n[l].length<=i?r=r.concat(n[l]):r.length&&(s.push(r),r=[].concat(n[l])),l===n.length-1&&r.length&&s.push(r);this.allGraphData[e].childIdsList=s,this.allGraphData[e].index=0,this.allGraphData[e].maxChainNum=a},dealChainingData:function(e,t){var i=this,n=this.allGraphData[e];if(!n.chained){n.chained=!0;var a=[];Object.keys(n.input).forEach((function(e){e.includes(t)&&(a=a.concat(i.dealChainingData(e,t)))})),a.length&&(n.chainNum=Object(o["a"])(new Set(a.map((function(e){return e+1})))))}return n.chainNum},getAssociatedNode:function(e,t){var i=this,n=this.allGraphData[e],a=[];return n.grouped||(n.grouped=!0,a.push(n.name),Object.keys(n.input).forEach((function(e){if(e.startsWith(t)&&!i.allGraphData[e].grouped){var n=i.getAssociatedNode(e,t);a=a.concat(n)}})),Object.keys(n.output).forEach((function(e){if(e.startsWith(t)&&!i.allGraphData[e].grouped){var n=i.getAssociatedNode(e,t);a=a.concat(n)}}))),a},packageGraphData:function(){var e=this.getChildNodesByName(),t='node[style="filled";fontsize="10px"];edge[fontsize="5px";];';return"digraph {".concat(t).concat(this.packageNodes(e)).concat(this.packageEdges(e),"}")},packageNodes:function(e,t){var i=this,n="";return e.forEach((function(e){var t=e.name.split("/").pop();if("aggregation_scope"===e.type)n+="<".concat(e.name,'>[id="').concat(e.name,'";')+'label="'.concat(t,'";class="aggregation";')+"".concat(e.isUnfold?'shape="polygon";width='.concat(e.size[0],";")+"height=".concat(e.size[1],";fixedsize=true;"):'shape="octagon";',"];");else if("name_scope"===e.type){var a=l["a"].graphColorArr[i.themeIndex][i.curColorIndex];i.curColorIndex=i.curColorIndex%4,i.curColorIndex++,n+="<".concat(e.name,'>[id="').concat(e.name,'";fillcolor="').concat(a,'";')+'shape="polygon";label="'.concat(t,'";class="cluster";')+"".concat(e.isUnfold?"width=".concat(e.size[0],";height=").concat(e.size[1],";fixedsize=true;"):"","];")}else if("Const"===e.type)n+="<".concat(e.name,'>[id="').concat(e.name,'";label="').concat(t,'\n\n\n";')+'shape="circle";width="0.14";height="0.14";fixedsize=true;];';else{var o=e.is_dynamic_shape_node;n+=o?"<".concat(e.name,'>[id="').concat(e.name,'";shape="ellipse";class="dynamicShape"')+'label="'.concat(t,'"];'):"<".concat(e.name,'>[id="').concat(e.name,'";shape="ellipse";')+'label="'.concat(t,'"];')}if(!i.allGraphData[e.name].isUnfold){for(var s=Object.keys(e.proxy_input||{}),r=e.name,c="",h=!1,d=0;d<Math.min(5,s.length);d++){c=s[d],h=!(!i.allGraphData[s[d]]||"Const"!==i.allGraphData[s[d]].type);var u=h?'shape="circle";width="0.14";height="0.14";fixedsize=true;'+'label="'.concat(c.split("/").pop(),'\n\n\n";'):'shape="Mrecord";label="'.concat(c.split("/").pop(),'";');n+="<".concat(c,"^").concat(r,'>[id="').concat(c,"^").concat(r,'";')+"".concat(u,'class="plain"];')}s.length>5&&(n+="<analogNodesInput^".concat(r,'>[id="analogNodesInput^')+"".concat(r,'";label="').concat(s.length-5,' more...";shape="Mrecord";')+'class="plain";];'),s=Object.keys(e.proxy_output||{}),c=e.name;for(var p=0;p<Math.min(5,s.length);p++){r=s[p],h=!(!i.allGraphData[s[p]]||"Const"!==i.allGraphData[s[p]].type);var f=h?'shape="circle";width="0.14";height="0.14";fixedsize=true;'+'label="'.concat(r.split("/").pop(),'\n\n\n";'):'shape="Mrecord";label="'.concat(r.split("/").pop(),'";');n+="<".concat(r,"^").concat(c,'>[id="').concat(r,"^").concat(c,'";')+"".concat(f,'class="plain";];')}s.length>5&&(n+="<analogNodesOutput^".concat(c,'>[id="analogNodesOutput^')+"".concat(c,'";shape="Mrecord";label="').concat(s.length-5,' more...";')+'class="plain";];')}})),n},packageEdges:function(e,t){var i=this,n="",a=[],o="analogNodesInputOf".concat(t),s="analogNodesOutputOf".concat(t),r=!1,l=!1,c=!!t&&this.allGraphData[t].independent_layout;return e.forEach((function(e){if(!c){var n=e.input||{},h=Object.keys(n);h.forEach((function(s){if(n[s]&&!n[s].independent_layout){var l=i.findChildNamescope(s,t)||(s?o:""),c=e.name;if(e.independent_layout){var h=e.name.split("/");h.splice(h.length-2,2),c="".concat(h.join("/"),"_unfold")}if(l&&c&&!c.includes(l.replace("_unfold","")+"/")&&!l.includes(c.replace("_unfold","")+"/"))if(!t||t&&l.startsWith("".concat(t,"/"))){var d={source:l,target:c,shape:n[s].shape,edge_type:n[s].edge_type,data_type:n[s].data_type,count:1};a.push(d)}else if(c.replace("_unfold","")!==t){var u={source:o,target:c,shape:n[s].shape,edge_type:n[s].edge_type,data_type:n[s].data_type,count:1};a.push(u),r=!0}}})),e.independent_layout||Object.keys(e.output||{}).forEach((function(n){if(!e.output[n].independent_layout){var o=e.name,r=i.findChildNamescope(n,t)||s;if(o&&r&&t&&!r.startsWith("".concat(t,"/"))&&o!==t){var c={source:o,target:s,shape:e.output[n].shape,edge_type:e.output[n].edge_type,data_type:e.output[n].data_type,count:1};a.push(c),l=!0}}}))}if(!i.allGraphData[e.name].isUnfold){for(var d=Object.keys(e.proxy_input||{}),u=0;u<Math.min(5,d.length);u++){var p=e.name,f=d[u],g={source:"".concat(f,"^").concat(p),target:p,shape:e.proxy_input[d[u]].shape,edge_type:e.proxy_input[d[u]].edge_type,data_type:e.proxy_input[d[u]].data_type,count:1};a.push(g)}if(d.length>5){var m={source:"analogNodesInput^".concat(e.name),target:e.name,shape:[],edge_type:"",data_type:"",count:1};a.push(m)}d=Object.keys(e.proxy_output||{});for(var v=0;v<Math.min(5,d.length);v++){var b=e.name,w=d[v],y={source:b,target:"".concat(w,"^").concat(b),shape:e.proxy_output[d[v]].shape,edge_type:e.proxy_output[d[v]].edge_type,data_type:e.proxy_output[d[v]].data_type,count:1};a.push(y)}if(d.length>5){var C={source:e.name,target:"analogNodesOutput^".concat(e.name),shape:[],edge_type:"",data_type:"",count:1};a.push(C)}}})),t&&!this.allGraphData[t].independent_layout&&(r&&(n+="{rank=min;<".concat(o,'>[shape="circle";')+'id="'.concat(o,'";fixedsize=true;width=0.02;label="";')+'class="edge-point"]};'),l&&(n+="{rank=max;<".concat(s,'>[shape="circle";')+'id="'.concat(s,'";width=0.02;fixedsize=true;')+'label="";class="edge-point"]};')),this.uniqueEdges(a),a.forEach((function(e){var t="control"===e.edge_type?"_control":"";n+="<".concat(e.source,">-><").concat(e.target,'>[id="').concat(e.source,"->")+"".concat(e.target).concat(t,'";label="').concat(i.getEdgeLabel(e),'";')+"".concat("control"===e.edge_type?"style=dashed":"","];")})),n},getEdgeLabel:function(e){if("control"===e.edge_type)return"";var t="";if(e.count&&1!==e.count)t="".concat(e.count,"tensors");else if(e.shape&&e.shape.length)if(e.shape.length>1)t="tuple(".concat(e.shape.length," items)");else{var i=e.shape[0];if(i&&i.length){var n=i.some((function(e){return"number"!==typeof e}));t=n?"tuple(".concat(i.length," items)"):"".concat(e.data_type," ").concat(i.join("×"))}else e.data_type&&(t="".concat(e.data_type))}else e.data_type&&(t="".concat(e.data_type));return t},getChildNodesByName:function(e){var t=this,i=[];if(e){var n=this.allGraphData[e],a="aggregation_scope"===n.type?n.childIdsList[n.index]:n.children;if(i=a.map((function(e){return t.allGraphData[e]})),"aggregation_scope"===n.type){var o=n.childIdsList;if(o&&o.length>1){if(n.index>0){for(var s=0,r=0;r<n.index;r++)s+=o[r].length;var l={name:"".concat(e,"/left/").concat(s," more..."),attr:{},input:{},output:{},proxy_input:{},proxy_output:{},type:""};this.allGraphData[l.name]=l,i.unshift(l)}if(n.index<o.length-1){for(var c=0,h=n.index+1;h<o.length;h++)c+=o[h].length;var d={name:"".concat(e,"/right/").concat(c," more..."),attr:{},input:{},output:{},proxy_input:{},proxy_output:{},type:""};this.allGraphData[d.name]=d,i.push(d)}}}}else i=this.firstFloorNodes.map((function(e){return t.allGraphData[e]}));return i},findChildNamescope:function(e,t){if(t){if(e.startsWith(t)){var i=t.split("/").length;return e.split("/").slice(0,i+1).join("/")}return null}return e.split("/")[0]},uniqueEdges:function(e){for(var t=0;t<e.length-1;t++)for(var i=t+1;i<e.length;i++){var n=e[t].source===e[i].source&&e[t].target===e[i].target&&e[t].edge_type===e[i].edge_type;n&&(e[t].count+=e[i].count,e.splice(i--,1))}},dealNamescopeTempGraph:function(e){if(e&&this.allGraphData[e]){var t=this.allGraphData[e].type,i="aggregation_scope"===t?"node cluster aggregation":"node cluster",n="#graphTemp #graph0 ",a="aggregation_scope"===t?l["a"].graphTheme[this.themeIndex].aggregationOpenColor:"#ffe4d6",o=(e.split("/").length-1)%4;"name_scope"===t&&(a=l["a"].graphColorArr[this.themeIndex][o]);var s=d.select(n).node(),r=s.getBBox(),c=d.select(n).insert("g").attr("id","".concat(e,"_unfold")).attr("class",i).attr("style","fill:".concat(a,";"));if(c.append("title").text(e),c.node().appendChild(d.select("#graphTemp #graph0>text").attr("y",r.y-10).node()),Array.prototype.forEach.call(document.querySelector(n).querySelectorAll("g"),(function(e){if(e.id!==c.node().id){if(Array.prototype.includes.call(e.classList,"plain")){var t=e.querySelector("title");t.textContent=t.textContent.split("^")[0]}e.setAttribute("transform","translate(0,0)"),c.node().appendChild(e)}})),c.insert("rect","title").attr("style","fill:".concat(a,";")).attr("stroke","rgb(167, 167, 167)").attr("x",c.node().getBBox().x-this.frameSpace).attr("y",c.node().getBBox().y-this.frameSpace).attr("width",c.node().getBBox().width+2*this.frameSpace).attr("height",c.node().getBBox().height+2*this.frameSpace),r=d.select("".concat(n,'g[id="').concat(e,'_unfold"]')).node().getBBox(),this.allGraphData[e].isUnfold=!0,this.allGraphData[e].size=[r.width/72,r.height/72],d.select("#subgraphTemp svg").size()){var h=document.querySelector("#subgraphTemp #graph0 g"),u=h.id.replace("_unfold",""),p=document.querySelector('#graphTemp g[id="'.concat(u,'"]')),f=p.getBBox(),g=h.getBBox(),m="translate(".concat(f.x-g.x,",").concat(f.y-g.y,")");h.setAttribute("transform",m),p.parentNode.appendChild(h),document.querySelector("#subgraphTemp svg").remove(),p.remove()}for(var v=document.querySelector("#graphTemp #graph0").children,b=0;b<v.length;b++)v[b].id!=="".concat(e,"_unfold")&&v[b--].remove();this.generateIOBus(e),document.querySelector("#subgraphTemp").appendChild(document.querySelector("#graphTemp svg")),this.transplantChildrenDom(e),this.layoutController(e)}},transplantChildrenDom:function(e){var t=this,i=[],n="#subgraphTemp ";e?i=this.allGraphData[e].children:(n="#graph ",i=this.firstFloorNodes),i.forEach((function(e){var i=t.allGraphData[e],a=("name_scope"===i.type||"aggregation_scope"===i.type)&&i.isUnfold;if(a){document.querySelector("#graphTemp").innerHTML=i.html;var o=document.querySelector("".concat(n,'g[id="').concat(e,'"]')),s=document.querySelector('#graphTemp #graph0 g[id="'.concat(e,'_unfold"]'));if(o&&s){var r=o.getBBox(),l=s.getBBox(),c="translate(".concat(r.x-l.x,",").concat(r.y-l.y,")");s.setAttribute("transform",c),o.parentNode.appendChild(s),o.remove()}var h=document.querySelector("#graphTemp svg");h&&h.remove()}})),e&&(this.allGraphData[e].html=document.querySelector("#subgraphTemp svg").outerHTML)},generateIOBus:function(e){d.select('#graphTemp g[id="analogNodesInputOf'.concat(e,'"]')).size()&&this.generateEdge({source:"".concat(e,"_unfold"),target:"analogNodesInputOf".concat(e)},e,"input"),d.select('#graphTemp g[id="analogNodesOutputOf'.concat(e,'"]')).size()&&this.generateEdge({source:"analogNodesOutputOf".concat(e),target:"".concat(e,"_unfold")},e,"output")},packageNamescope:function(e){var t=this.getChildNodesByName(e),i=this.packageNodes(t,e),n=this.packageEdges(t,e),a='node[style="filled";fontsize="10px";];edge[fontsize="5px";];',o="digraph {".concat(a,'label="').concat(e.split("/").pop(),'";')+"".concat(i).concat(n,"}");return o},generateEdge:function(e,t,i){var n=this.getEdgePoints(e,i),a=this.getEdgeLabel(e),o=d.select("#graphTemp g#graph0".concat(t?' g[id="'.concat(t,'_unfold"]'):"")).append("g").attr("id","".concat(e.source.replace("_unfold",""),"->").concat(e.target.replace("_unfold",""))).attr("class","edge");o.append("title").text(a);var s=o.append("marker");s.attr("id","".concat(t+i,"marker")).attr("refX",6).attr("refY",3).attr("markerWidth",8).attr("markerHeight",6).attr("orient","auto"),s.append("path").attr("d","M1,1 L1,5 L6,3 z").attr("fill","rgb(120, 120, 120)").attr("stroke","rgb(120, 120, 120)"),o.append("path").attr("stroke","rgb(167, 167, 167)").attr("stroke-width",1).attr("stroke-dasharray","".concat("control"===e.edge_type?"5,2":"0")).attr("marker-end","url(#".concat(t+i,"marker)")).attr("d","M".concat(n[0].x,",").concat(n[0].y,"L").concat(n[1].x,",").concat(n[1].y)),o.append("text").attr("text-anchor","middle").attr("font-family","Times,serif").attr("font-size","5px").attr("fill","#000000").attr("x",(n[0].x+n[1].x)/2).attr("y",(n[0].y+n[1].y)/2).text(a)},getEdgePoints:function(e,t){var i=d.select('#graphTemp g[id="'.concat(e.source,'"]')).node().getBBox(),n=d.select('#graphTemp g[id="'.concat(e.target,'"]')).node().getBBox();return i.points=this.getBoxPoints(i),n.points=this.getBoxPoints(n),"input"===t?[i.points.top,n.points.top]:[i.points.bottom,n.points.bottom]},getBoxPoints:function(e){var t={top:{x:e.x+e.width/2,y:e.y},bottom:{x:e.x+e.width/2,y:e.y+e.height}};return t},getTransformData:function(e){if(!e)return{};var t=e.getAttribute("transform"),i={};if(t){var n=t.trim().split(" ");n.forEach((function(e){if(e=e.trim(),e){var t=e.indexOf("("),n=e.indexOf(")"),a=e.substring(t+1,n).split(",").map((function(e){return parseFloat(e)||0}));i[e.substring(0,t)]=a}}))}return i},highlightProxyNodes:function(e){var t=d.selectAll("#graph g.node").nodes().filter((function(t){var i=t.id.split("^")[0].replace("_unfold","");return i===e})),i=["polygon","ellipse","path","rect"];t.forEach((function(e){e.childNodes&&e.childNodes.forEach((function(e){i.includes(e.tagName)&&e.setAttribute("class","selected")}))}))},highLightEdges:function(e){var t=this,i={},n=e.input||{},a=e.output||{},o=this.findExsitNode(e.name);if(o&&!e.independent_layout&&(Object.keys(n).forEach((function(e){var a=t.findExsitNode(e);a&&(i["".concat(a,"->").concat(o)]={source:a,target:o,edge_type:n[e].edge_type||""})})),Object.keys(a).forEach((function(e){var n=t.findExsitNode(e);n&&(i["".concat(o,"->").concat(n)]={source:o,target:n,edge_type:a[e].edge_type||""})}))),!e.isUnfold){for(var r=Object.keys(e.proxy_input||{}),l=0;l<Math.min(5,r.length);l++){var c="".concat(r[l],"^").concat(e.name);i["".concat(c,"->").concat(e.name)]={source:c,target:e.name,edge_type:e.proxy_input[r[l]].edge_type||""}}if(r.length>5){var h="analogNodesInput^".concat(e.name);i["".concat(h,"->").concat(e.name)]={source:h,target:e.name,edge_type:""}}r=Object.keys(e.proxy_output||{});for(var u=0;u<Math.min(5,r.length);u++){var p="".concat(r[u],"^").concat(e.name);i["".concat(e.name,"->").concat(p)]={source:e.name,target:p,edge_type:e.proxy_output[r[u]].edge_type||""}}if(r.length>5){var f="analogNodesOutput^".concat(e.name);i["".concat(e.name,"->").concat(f)]={source:e.name,target:f,edge_type:""}}}var g={};Object.keys(i).forEach((function(e){if(e.includes("^"))g[e]=i[e];else{for(var t="control"===i[e].edge_type?"_control":"",n=e.split("->"),a=Object(s["a"])(n,2),o=a[0],r=a[1],l=[],c=o.split("/"),h=r.split("/"),d=Math.min(c.length,h.length),u=-1,p=0;p<d;p++)c.slice(0,p+1).join("/")===h.slice(0,p+1).join("/")&&(u=p);for(var f=u+2;f<c.length;f++){var m=c.slice(0,f+1).join("/"),v=c.slice(0,f).join("/");l.push("".concat(m,"->analogNodesOutputOf").concat(v).concat(t)),l.push("analogNodesOutputOf".concat(v,"->").concat(v))}l.push("".concat(c.slice(0,u+2).join("/"),"->")+"".concat(h.slice(0,u+2).join("/")).concat(t));for(var b=u+2;b<h.length;b++){var w=h.slice(0,b).join("/"),y=h.slice(0,b+1).join("/");l.push("".concat(w,"->analogNodesInputOf").concat(w)),l.push("analogNodesInputOf".concat(w,"->").concat(y).concat(t))}for(var C=0;C<l.length;C++){var _=l[C].split("->"),x=Object(s["a"])(_,2),k=x[0],S=x[1];k.startsWith(S+"/")||S.startsWith(k+"/")||(g["".concat(k,"->").concat(S)]={source:k,target:S,edge_type:i[e].edge_type})}}})),d.selectAll("#graph g.edge").classed("highlighted",!1),Object.keys(g).forEach((function(e){d.select('#graph g[id="'.concat(e,'"]')).classed("highlighted",!0)}))},findExsitNode:function(e){for(var t="",i=e.split("/"),n=i.length;n>0;n--){var a=i.slice(0,n).join("/");if(this.allGraphData[a]){t=a;break}}return t&&this.allGraphData[t]&&this.allGraphData[t].independent_layout&&(t=t.split("/").slice(0,-1).join("/")),t},packageDataToObject:function(e,t,i){var n=this;if(e)if(t)this.allGraphData[e].isUnfold=!0,i.forEach((function(t){t.isUnfold=!1,t.children=[],t.size=[],t.html="",t.grouped=!1,t.chained=!1,t.chainNum=[1],n.allGraphData[t.name]=t,n.allGraphData[e].children.push(t.name)}));else{var a=Object.keys(this.allGraphData).filter((function(t){return t.startsWith("".concat(e,"/"))}));a.forEach((function(e){delete n.allGraphData[e]})),this.allGraphData[e].isUnfold=!1,this.allGraphData[e].children=[],this.allGraphData[e].size=[],this.allGraphData[e].html=""}else this.allGraphData={},this.firstFloorNodes=[],i.forEach((function(e){e.isUnfold=!1,e.children=[],e.size=[],e.html="",n.allGraphData[e.name]=e,n.firstFloorNodes.push(e.name)}))},findStartUnfoldNode:function(e){var t=this;return e&&e.scope_name?this.allGraphData[e.scope_name]&&this.allGraphData[e.scope_name].isUnfold?e.nodes.some((function(e){return e.name===t.selectedNode.name}))?e:this.findStartUnfoldNode(e.children):e:null}},destroyed:function(){this.graphContainer&&(this.graphContainer.removeEventListener("click",this.controlEvent),this.graphContainer.removeEventListener("dblclick",this.controlEvent))}},p=u,f=(i("0210"),i("2877")),g=Object(f["a"])(p,n,a,!1,null,null,null);t["a"]=g.exports},9129:function(e,t,i){var n=i("23e7");n({target:"Number",stat:!0},{isNaN:function(e){return e!=e}})},9175:function(e,t,i){e.exports=i.p+"static/img/map-dataset.ea36ef19.svg"},"91b0":function(e,t,i){e.exports=i.p+"static/img/icon3.3459fabb.svg"},9452:function(e,t,i){e.exports=i.p+"static/img/graph-step3.c6c32329.svg"},"96c0":function(e,t,i){e.exports=i.p+"static/img/icon-down.b27a06c5.svg"},"97a7":function(e,t,i){e.exports=i.p+"static/img/scalar-guid.111b3272.svg"},"97cd":function(e,t,i){e.exports=i.p+"static/img/operator-node.1a9f3db8.svg"},"98c0":function(e,t,i){e.exports=i.p+"static/img/tensor-guid.c9811239.svg"},"98f4":function(e,t,i){e.exports=i.p+"static/img/shuffle-dataset.3fa06fa2.svg"},"9b3c":function(e,t,i){e.exports=i.p+"static/img/name-scope.5935c318.svg"},"9e69":function(e,t,i){e.exports=i.p+"static/img/name-scope.8f41c852.svg"},"9f1b":function(e,t,i){e.exports=i.p+"static/img/creat-dataset.17b9b72d.svg"},a094:function(e,t,i){e.exports=i.p+"static/img/graph-step2.538b5a8b.svg"},a294:function(e,t,i){e.exports=i.p+"static/img/repeat-dataset.f82c8a5f.svg"},aa39:function(e,t,i){e.exports=i.p+"static/img/graph-stepimg3.bc1dc1d7.svg"},ac8c:function(e,t,i){e.exports=i.p+"static/img/pipeline-open.f56705a4.svg"},b205:function(e,t,i){e.exports=i.p+"static/img/icon4.bf5a7395.svg"},b680:function(e,t,i){"use strict";var n=i("23e7"),a=i("a691"),o=i("408a"),s=i("1148"),r=i("d039"),l=1..toFixed,c=Math.floor,h=function(e,t,i){return 0===t?i:t%2===1?h(e,t-1,i*e):h(e*e,t/2,i)},d=function(e){var t=0,i=e;while(i>=4096)t+=12,i/=4096;while(i>=2)t+=1,i/=2;return t},u=l&&("0.000"!==8e-5.toFixed(3)||"1"!==.9.toFixed(0)||"1.25"!==1.255.toFixed(2)||"1000000000000000128"!==(0xde0b6b3a7640080).toFixed(0))||!r((function(){l.call({})}));n({target:"Number",proto:!0,forced:u},{toFixed:function(e){var t,i,n,r,l=o(this),u=a(e),p=[0,0,0,0,0,0],f="",g="0",m=function(e,t){var i=-1,n=t;while(++i<6)n+=e*p[i],p[i]=n%1e7,n=c(n/1e7)},v=function(e){var t=6,i=0;while(--t>=0)i+=p[t],p[t]=c(i/e),i=i%e*1e7},b=function(){var e=6,t="";while(--e>=0)if(""!==t||0===e||0!==p[e]){var i=String(p[e]);t=""===t?i:t+s.call("0",7-i.length)+i}return t};if(u<0||u>20)throw RangeError("Incorrect fraction digits");if(l!=l)return"NaN";if(l<=-1e21||l>=1e21)return String(l);if(l<0&&(f="-",l=-l),l>1e-21)if(t=d(l*h(2,69,1))-69,i=t<0?l*h(2,-t,1):l/h(2,t,1),i*=4503599627370496,t=52-t,t>0){m(0,i),n=u;while(n>=7)m(1e7,0),n-=7;m(h(10,n,1),0),n=t-1;while(n>=23)v(1<<23),n-=23;v(1<<n),m(1,1),v(2),g=b()}else m(0,i),m(1<<-t,0),g=b()+s.call("0",u);return u>0?(r=g.length,g=f+(r<=u?"0."+s.call("0",u-r)+g:g.slice(0,r-u)+"."+g.slice(r-u))):g=f+g,g}})},b85c:function(e,t,i){"use strict";i.d(t,"a",(function(){return a}));i("e01a"),i("d28b"),i("e260"),i("d3b7"),i("3ca3"),i("ddb0");var n=i("06c5");function a(e,t){var i="undefined"!=typeof Symbol&&e[Symbol.iterator]||e["@@iterator"];if(!i){if(Array.isArray(e)||(i=Object(n["a"])(e))||t&&e&&"number"==typeof e.length){i&&(e=i);var a=0,o=function(){};return{s:o,n:function(){return a>=e.length?{done:!0}:{done:!1,value:e[a++]}},e:function(e){throw e},f:o}}throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}var s,r=!0,l=!1;return{s:function(){i=i.call(e)},n:function(){var e=i.next();return r=e.done,e},e:function(e){l=!0,s=e},f:function(){try{r||null==i["return"]||i["return"]()}finally{if(l)throw s}}}}},ba85:function(e,t,i){e.exports=i.p+"static/img/icon7.a39591b9.svg"},baf5:function(e,t,i){e.exports=i.p+"static/img/explain-tp.1098a435.svg"},bb51:function(e,t,i){},bbf2:function(e,t,i){e.exports=i.p+"static/img/icon10.fef9c1ec.svg"},bc0b:function(e,t,i){var n={"./0/polymetric.svg":"4ab4","./1/polymetric.svg":"29c4"};function a(e){var t=o(e);return i(t)}function o(e){if(!i.o(n,e)){var t=new Error("Cannot find module '"+e+"'");throw t.code="MODULE_NOT_FOUND",t}return n[e]}a.keys=function(){return Object.keys(n)},a.resolve=o,e.exports=a,a.id="bc0b"},bee2:function(e,t,i){"use strict";i.d(t,"a",(function(){return o}));var n=i("a38e");function a(e,t){for(var i=0;i<t.length;i++){var a=t[i];a.enumerable=a.enumerable||!1,a.configurable=!0,"value"in a&&(a.writable=!0),Object.defineProperty(e,Object(n["a"])(a.key),a)}}function o(e,t,i){return t&&a(e.prototype,t),i&&a(e,i),Object.defineProperty(e,"prototype",{writable:!1}),e}},bf29:function(e,t,i){"use strict";i("6220")},bf6c:function(e,t,i){e.exports=i.p+"static/img/batch-img.01b05453.svg"},c1c9:function(e,t,i){},c356:function(e,t,i){e.exports=i.p+"static/img/graph-step1-en.9967e755.svg"},c37b:function(e,t){e.exports="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAFgAAAAPCAYAAABgHgh7AAAAAXNSR0IArs4c6QAAAR1JREFUWAnt1zEOgjAUBmDboC5sXMPJg7B4hwLHMG4muhIaFidmvYST7p7DsUnxYVJsG0pb52di6Ov7fcOXQnCxwE+QQNu2m6CgFaJWjaVDQAhxr+v62jTN1hGZ3EbgSRbnZi6lfMRAI7DTcrYRDI3As47ephcagb2GQQEnNIGHNoMRS2sML4pCYO/n0vf9Gb4ry8lV3iilezB8JvDQPkEq1ZNZll2gFtgzXXQjz3o40Tkc0GNCCOEQXus/SNNUDDX2DBcGJ9i+03U2ff2C4sAY64i+i2u3ALyavaFr3OkT6S9sWZYdHE459JOJEG7FCxiwVVWNExB4pPhr4YRV0xBYScRdvbBqHAIribBrMGzYOEyNApzzHbxFRP8x+wD6ZuTZmwHa4gAAAABJRU5ErkJggg=="},c740:function(e,t,i){"use strict";var n=i("23e7"),a=i("b727").findIndex,o=i("44d2"),s=i("ae40"),r="findIndex",l=!0,c=s(r);r in[]&&Array(1)[r]((function(){l=!1})),n({target:"Array",proto:!0,forced:l||!c},{findIndex:function(e){return a(this,e,arguments.length>1?arguments[1]:void 0)}}),o(r)},c77d:function(e,t,i){e.exports=i.p+"static/img/map-dataset.ea36ef19.svg"},c8d2:function(e,t,i){var n=i("d039"),a=i("5899"),o="​᠎";e.exports=function(e){return n((function(){return!!a[e]()||o[e]()!=o||a[e].name!==e}))}},c8e3:function(e,t,i){e.exports=i.p+"static/img/guideIcon.b94f3b97.svg"},cb29:function(e,t,i){var n=i("23e7"),a=i("81d5"),o=i("44d2");n({target:"Array",proto:!0},{fill:a}),o("fill")},cb78:function(e,t,i){e.exports=i.p+"static/img/dynamic-shape-node.2b7eae99.svg"},cca6:function(e,t,i){var n=i("23e7"),a=i("60da");n({target:"Object",stat:!0,forced:Object.assign!==a},{assign:a})},cd21:function(e,t,i){e.exports=i.p+"static/img/virtual-node.3cfb7240.svg"},ce5d:function(e,t,i){e.exports=i.p+"static/img/guideIconHover.11506553.svg"},d18b:function(e,t,i){e.exports=i.p+"static/img/operator-node.e4e15cb8.svg"},d4ec:function(e,t,i){"use strict";function n(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}i.d(t,"a",(function(){return n}))},d674:function(e,t,i){e.exports=i.p+"static/img/creat-dataset.b46fdd0b.svg"},d73b:function(e,t,i){var n={"./0/virtual-node.svg":"1cac","./1/virtual-node.svg":"cd21"};function a(e){var t=o(e);return i(t)}function o(e){if(!i.o(n,e)){var t=new Error("Cannot find module '"+e+"'");throw t.code="MODULE_NOT_FOUND",t}return n[e]}a.keys=function(){return Object.keys(n)},a.resolve=o,e.exports=a,a.id="d73b"},daff:function(e,t,i){e.exports=i.p+"static/img/image-guid.33f41c3e.svg"},dbb4:function(e,t,i){var n=i("23e7"),a=i("83ab"),o=i("56ef"),s=i("fc6a"),r=i("06cf"),l=i("8418");n({target:"Object",stat:!0,sham:!a},{getOwnPropertyDescriptors:function(e){var t,i,n=s(e),a=r.f,c=o(n),h={},d=0;while(c.length>d)i=a(n,t=c[d++]),void 0!==i&&l(h,t,i);return h}})},dbf9:function(e,t,i){e.exports=i.p+"static/img/shuffle-dataset.3fa06fa2.svg"},dc1b:function(e,t,i){var n,a;
/*!
 * jQuery JavaScript Library v3.1.0
 * https://jquery.com/
 *
 * Includes Sizzle.js
 * https://sizzlejs.com/
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2016-07-07T21:44Z
 */(function(t,i){"use strict";"object"===typeof e.exports?e.exports=t.document?i(t,!0):function(e){if(!e.document)throw new Error("jQuery requires a window with a document");return i(e)}:i(t)})("undefined"!==typeof window?window:this,(function(i,o){"use strict";var s=[],r=i.document,l=Object.getPrototypeOf,c=s.slice,h=s.concat,d=s.push,u=s.indexOf,p={},f=p.toString,g=p.hasOwnProperty,m=g.toString,v=m.call(Object),b={};function w(e,t){t=t||r;var i=t.createElement("script");i.text=e,t.head.appendChild(i).parentNode.removeChild(i)}var y="3.1.0",C=function(e,t){return new C.fn.init(e,t)},_=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,x=/^-ms-/,k=/-([a-z])/g,S=function(e,t){return t.toUpperCase()};function T(e){var t=!!e&&"length"in e&&e.length,i=C.type(e);return"function"!==i&&!C.isWindow(e)&&("array"===i||0===t||"number"===typeof t&&t>0&&t-1 in e)}C.fn=C.prototype={jquery:y,constructor:C,length:0,toArray:function(){return c.call(this)},get:function(e){return null!=e?e<0?this[e+this.length]:this[e]:c.call(this)},pushStack:function(e){var t=C.merge(this.constructor(),e);return t.prevObject=this,t},each:function(e){return C.each(this,e)},map:function(e){return this.pushStack(C.map(this,(function(t,i){return e.call(t,i,t)})))},slice:function(){return this.pushStack(c.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},eq:function(e){var t=this.length,i=+e+(e<0?t:0);return this.pushStack(i>=0&&i<t?[this[i]]:[])},end:function(){return this.prevObject||this.constructor()},push:d,sort:s.sort,splice:s.splice},C.extend=C.fn.extend=function(){var e,t,i,n,a,o,s=arguments[0]||{},r=1,l=arguments.length,c=!1;for("boolean"===typeof s&&(c=s,s=arguments[r]||{},r++),"object"===typeof s||C.isFunction(s)||(s={}),r===l&&(s=this,r--);r<l;r++)if(null!=(e=arguments[r]))for(t in e)i=s[t],n=e[t],s!==n&&(c&&n&&(C.isPlainObject(n)||(a=C.isArray(n)))?(a?(a=!1,o=i&&C.isArray(i)?i:[]):o=i&&C.isPlainObject(i)?i:{},s[t]=C.extend(c,o,n)):void 0!==n&&(s[t]=n));return s},C.extend({expando:"jQuery"+(y+Math.random()).replace(/\D/g,""),isReady:!0,error:function(e){throw new Error(e)},noop:function(){},isFunction:function(e){return"function"===C.type(e)},isArray:Array.isArray,isWindow:function(e){return null!=e&&e===e.window},isNumeric:function(e){var t=C.type(e);return("number"===t||"string"===t)&&!isNaN(e-parseFloat(e))},isPlainObject:function(e){var t,i;return!(!e||"[object Object]"!==f.call(e))&&(t=l(e),!t||(i=g.call(t,"constructor")&&t.constructor,"function"===typeof i&&m.call(i)===v))},isEmptyObject:function(e){var t;for(t in e)return!1;return!0},type:function(e){return null==e?e+"":"object"===typeof e||"function"===typeof e?p[f.call(e)]||"object":typeof e},globalEval:function(e){w(e)},camelCase:function(e){return e.replace(x,"ms-").replace(k,S)},nodeName:function(e,t){return e.nodeName&&e.nodeName.toLowerCase()===t.toLowerCase()},each:function(e,t){var i,n=0;if(T(e)){for(i=e.length;n<i;n++)if(!1===t.call(e[n],n,e[n]))break}else for(n in e)if(!1===t.call(e[n],n,e[n]))break;return e},trim:function(e){return null==e?"":(e+"").replace(_,"")},makeArray:function(e,t){var i=t||[];return null!=e&&(T(Object(e))?C.merge(i,"string"===typeof e?[e]:e):d.call(i,e)),i},inArray:function(e,t,i){return null==t?-1:u.call(t,e,i)},merge:function(e,t){for(var i=+t.length,n=0,a=e.length;n<i;n++)e[a++]=t[n];return e.length=a,e},grep:function(e,t,i){for(var n,a=[],o=0,s=e.length,r=!i;o<s;o++)n=!t(e[o],o),n!==r&&a.push(e[o]);return a},map:function(e,t,i){var n,a,o=0,s=[];if(T(e))for(n=e.length;o<n;o++)a=t(e[o],o,i),null!=a&&s.push(a);else for(o in e)a=t(e[o],o,i),null!=a&&s.push(a);return h.apply([],s)},guid:1,proxy:function(e,t){var i,n,a;if("string"===typeof t&&(i=e[t],t=e,e=i),C.isFunction(e))return n=c.call(arguments,2),a=function(){return e.apply(t||this,n.concat(c.call(arguments)))},a.guid=e.guid=e.guid||C.guid++,a},now:Date.now,support:b}),"function"===typeof Symbol&&(C.fn[Symbol.iterator]=s[Symbol.iterator]),C.each("Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),(function(e,t){p["[object "+t+"]"]=t.toLowerCase()}));var $=
/*!
 * Sizzle CSS Selector Engine v2.3.0
 * https://sizzlejs.com/
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2016-01-04
 */
function(e){var t,i,n,a,o,s,r,l,c,h,d,u,p,f,g,m,v,b,w,y="sizzle"+1*new Date,C=e.document,_=0,x=0,k=se(),S=se(),T=se(),$=function(e,t){return e===t&&(d=!0),0},D={}.hasOwnProperty,N=[],R=N.pop,I=N.push,P=N.push,A=N.slice,E=function(e,t){for(var i=0,n=e.length;i<n;i++)if(e[i]===t)return i;return-1},z="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",M="[\\x20\\t\\r\\n\\f]",H="(?:\\\\.|[\\w-]|[^\0-\\xa0])+",F="\\["+M+"*("+H+")(?:"+M+"*([*^$|!~]?=)"+M+"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|("+H+"))|)"+M+"*\\]",O=":("+H+")(?:\\((('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|((?:\\\\.|[^\\\\()[\\]]|"+F+")*)|.*)\\)|)",W=new RegExp(M+"+","g"),L=new RegExp("^"+M+"+|((?:^|[^\\\\])(?:\\\\.)*)"+M+"+$","g"),j=new RegExp("^"+M+"*,"+M+"*"),B=new RegExp("^"+M+"*([>+~]|"+M+")"+M+"*"),G=new RegExp("="+M+"*([^\\]'\"]*?)"+M+"*\\]","g"),V=new RegExp(O),q=new RegExp("^"+H+"$"),Y={ID:new RegExp("^#("+H+")"),CLASS:new RegExp("^\\.("+H+")"),TAG:new RegExp("^("+H+"|[*])"),ATTR:new RegExp("^"+F),PSEUDO:new RegExp("^"+O),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+M+"*(even|odd|(([+-]|)(\\d*)n|)"+M+"*(?:([+-]|)"+M+"*(\\d+)|))"+M+"*\\)|)","i"),bool:new RegExp("^(?:"+z+")$","i"),needsContext:new RegExp("^"+M+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+M+"*((?:-\\d)?\\d*)"+M+"*\\)|)(?=[^-]|$)","i")},U=/^(?:input|select|textarea|button)$/i,K=/^h\d$/i,X=/^[^{]+\{\s*\[native \w/,Q=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,J=/[+~]/,Z=new RegExp("\\\\([\\da-f]{1,6}"+M+"?|("+M+")|.)","ig"),ee=function(e,t,i){var n="0x"+t-65536;return n!==n||i?t:n<0?String.fromCharCode(n+65536):String.fromCharCode(n>>10|55296,1023&n|56320)},te=/([\0-\x1f\x7f]|^-?\d)|^-$|[^\x80-\uFFFF\w-]/g,ie=function(e,t){return t?"\0"===e?"�":e.slice(0,-1)+"\\"+e.charCodeAt(e.length-1).toString(16)+" ":"\\"+e},ne=function(){u()},ae=be((function(e){return!0===e.disabled}),{dir:"parentNode",next:"legend"});try{P.apply(N=A.call(C.childNodes),C.childNodes),N[C.childNodes.length].nodeType}catch(Se){P={apply:N.length?function(e,t){I.apply(e,A.call(t))}:function(e,t){var i=e.length,n=0;while(e[i++]=t[n++]);e.length=i-1}}}function oe(e,t,n,a){var o,r,c,h,d,f,v,b=t&&t.ownerDocument,_=t?t.nodeType:9;if(n=n||[],"string"!==typeof e||!e||1!==_&&9!==_&&11!==_)return n;if(!a&&((t?t.ownerDocument||t:C)!==p&&u(t),t=t||p,g)){if(11!==_&&(d=Q.exec(e)))if(o=d[1]){if(9===_){if(!(c=t.getElementById(o)))return n;if(c.id===o)return n.push(c),n}else if(b&&(c=b.getElementById(o))&&w(t,c)&&c.id===o)return n.push(c),n}else{if(d[2])return P.apply(n,t.getElementsByTagName(e)),n;if((o=d[3])&&i.getElementsByClassName&&t.getElementsByClassName)return P.apply(n,t.getElementsByClassName(o)),n}if(i.qsa&&!T[e+" "]&&(!m||!m.test(e))){if(1!==_)b=t,v=e;else if("object"!==t.nodeName.toLowerCase()){(h=t.getAttribute("id"))?h=h.replace(te,ie):t.setAttribute("id",h=y),f=s(e),r=f.length;while(r--)f[r]="#"+h+" "+ve(f[r]);v=f.join(","),b=J.test(e)&&ge(t.parentNode)||t}if(v)try{return P.apply(n,b.querySelectorAll(v)),n}catch(x){}finally{h===y&&t.removeAttribute("id")}}}return l(e.replace(L,"$1"),t,n,a)}function se(){var e=[];function t(i,a){return e.push(i+" ")>n.cacheLength&&delete t[e.shift()],t[i+" "]=a}return t}function re(e){return e[y]=!0,e}function le(e){var t=p.createElement("fieldset");try{return!!e(t)}catch(Se){return!1}finally{t.parentNode&&t.parentNode.removeChild(t),t=null}}function ce(e,t){var i=e.split("|"),a=i.length;while(a--)n.attrHandle[i[a]]=t}function he(e,t){var i=t&&e,n=i&&1===e.nodeType&&1===t.nodeType&&e.sourceIndex-t.sourceIndex;if(n)return n;if(i)while(i=i.nextSibling)if(i===t)return-1;return e?1:-1}function de(e){return function(t){var i=t.nodeName.toLowerCase();return"input"===i&&t.type===e}}function ue(e){return function(t){var i=t.nodeName.toLowerCase();return("input"===i||"button"===i)&&t.type===e}}function pe(e){return function(t){return"label"in t&&t.disabled===e||"form"in t&&t.disabled===e||"form"in t&&!1===t.disabled&&(t.isDisabled===e||t.isDisabled!==!e&&("label"in t||!ae(t))!==e)}}function fe(e){return re((function(t){return t=+t,re((function(i,n){var a,o=e([],i.length,t),s=o.length;while(s--)i[a=o[s]]&&(i[a]=!(n[a]=i[a]))}))}))}function ge(e){return e&&"undefined"!==typeof e.getElementsByTagName&&e}for(t in i=oe.support={},o=oe.isXML=function(e){var t=e&&(e.ownerDocument||e).documentElement;return!!t&&"HTML"!==t.nodeName},u=oe.setDocument=function(e){var t,a,s=e?e.ownerDocument||e:C;return s!==p&&9===s.nodeType&&s.documentElement?(p=s,f=p.documentElement,g=!o(p),C!==p&&(a=p.defaultView)&&a.top!==a&&(a.addEventListener?a.addEventListener("unload",ne,!1):a.attachEvent&&a.attachEvent("onunload",ne)),i.attributes=le((function(e){return e.className="i",!e.getAttribute("className")})),i.getElementsByTagName=le((function(e){return e.appendChild(p.createComment("")),!e.getElementsByTagName("*").length})),i.getElementsByClassName=X.test(p.getElementsByClassName),i.getById=le((function(e){return f.appendChild(e).id=y,!p.getElementsByName||!p.getElementsByName(y).length})),i.getById?(n.find["ID"]=function(e,t){if("undefined"!==typeof t.getElementById&&g){var i=t.getElementById(e);return i?[i]:[]}},n.filter["ID"]=function(e){var t=e.replace(Z,ee);return function(e){return e.getAttribute("id")===t}}):(delete n.find["ID"],n.filter["ID"]=function(e){var t=e.replace(Z,ee);return function(e){var i="undefined"!==typeof e.getAttributeNode&&e.getAttributeNode("id");return i&&i.value===t}}),n.find["TAG"]=i.getElementsByTagName?function(e,t){return"undefined"!==typeof t.getElementsByTagName?t.getElementsByTagName(e):i.qsa?t.querySelectorAll(e):void 0}:function(e,t){var i,n=[],a=0,o=t.getElementsByTagName(e);if("*"===e){while(i=o[a++])1===i.nodeType&&n.push(i);return n}return o},n.find["CLASS"]=i.getElementsByClassName&&function(e,t){if("undefined"!==typeof t.getElementsByClassName&&g)return t.getElementsByClassName(e)},v=[],m=[],(i.qsa=X.test(p.querySelectorAll))&&(le((function(e){f.appendChild(e).innerHTML="<a id='"+y+"'></a><select id='"+y+"-\r\\' msallowcapture=''><option selected=''></option></select>",e.querySelectorAll("[msallowcapture^='']").length&&m.push("[*^$]="+M+"*(?:''|\"\")"),e.querySelectorAll("[selected]").length||m.push("\\["+M+"*(?:value|"+z+")"),e.querySelectorAll("[id~="+y+"-]").length||m.push("~="),e.querySelectorAll(":checked").length||m.push(":checked"),e.querySelectorAll("a#"+y+"+*").length||m.push(".#.+[+~]")})),le((function(e){e.innerHTML="<a href='' disabled='disabled'></a><select disabled='disabled'><option/></select>";var t=p.createElement("input");t.setAttribute("type","hidden"),e.appendChild(t).setAttribute("name","D"),e.querySelectorAll("[name=d]").length&&m.push("name"+M+"*[*^$|!~]?="),2!==e.querySelectorAll(":enabled").length&&m.push(":enabled",":disabled"),f.appendChild(e).disabled=!0,2!==e.querySelectorAll(":disabled").length&&m.push(":enabled",":disabled"),e.querySelectorAll("*,:x"),m.push(",.*:")}))),(i.matchesSelector=X.test(b=f.matches||f.webkitMatchesSelector||f.mozMatchesSelector||f.oMatchesSelector||f.msMatchesSelector))&&le((function(e){i.disconnectedMatch=b.call(e,"*"),b.call(e,"[s!='']:x"),v.push("!=",O)})),m=m.length&&new RegExp(m.join("|")),v=v.length&&new RegExp(v.join("|")),t=X.test(f.compareDocumentPosition),w=t||X.test(f.contains)?function(e,t){var i=9===e.nodeType?e.documentElement:e,n=t&&t.parentNode;return e===n||!(!n||1!==n.nodeType||!(i.contains?i.contains(n):e.compareDocumentPosition&&16&e.compareDocumentPosition(n)))}:function(e,t){if(t)while(t=t.parentNode)if(t===e)return!0;return!1},$=t?function(e,t){if(e===t)return d=!0,0;var n=!e.compareDocumentPosition-!t.compareDocumentPosition;return n||(n=(e.ownerDocument||e)===(t.ownerDocument||t)?e.compareDocumentPosition(t):1,1&n||!i.sortDetached&&t.compareDocumentPosition(e)===n?e===p||e.ownerDocument===C&&w(C,e)?-1:t===p||t.ownerDocument===C&&w(C,t)?1:h?E(h,e)-E(h,t):0:4&n?-1:1)}:function(e,t){if(e===t)return d=!0,0;var i,n=0,a=e.parentNode,o=t.parentNode,s=[e],r=[t];if(!a||!o)return e===p?-1:t===p?1:a?-1:o?1:h?E(h,e)-E(h,t):0;if(a===o)return he(e,t);i=e;while(i=i.parentNode)s.unshift(i);i=t;while(i=i.parentNode)r.unshift(i);while(s[n]===r[n])n++;return n?he(s[n],r[n]):s[n]===C?-1:r[n]===C?1:0},p):p},oe.matches=function(e,t){return oe(e,null,null,t)},oe.matchesSelector=function(e,t){if((e.ownerDocument||e)!==p&&u(e),t=t.replace(G,"='$1']"),i.matchesSelector&&g&&!T[t+" "]&&(!v||!v.test(t))&&(!m||!m.test(t)))try{var n=b.call(e,t);if(n||i.disconnectedMatch||e.document&&11!==e.document.nodeType)return n}catch(Se){}return oe(t,p,null,[e]).length>0},oe.contains=function(e,t){return(e.ownerDocument||e)!==p&&u(e),w(e,t)},oe.attr=function(e,t){(e.ownerDocument||e)!==p&&u(e);var a=n.attrHandle[t.toLowerCase()],o=a&&D.call(n.attrHandle,t.toLowerCase())?a(e,t,!g):void 0;return void 0!==o?o:i.attributes||!g?e.getAttribute(t):(o=e.getAttributeNode(t))&&o.specified?o.value:null},oe.escape=function(e){return(e+"").replace(te,ie)},oe.error=function(e){throw new Error("Syntax error, unrecognized expression: "+e)},oe.uniqueSort=function(e){var t,n=[],a=0,o=0;if(d=!i.detectDuplicates,h=!i.sortStable&&e.slice(0),e.sort($),d){while(t=e[o++])t===e[o]&&(a=n.push(o));while(a--)e.splice(n[a],1)}return h=null,e},a=oe.getText=function(e){var t,i="",n=0,o=e.nodeType;if(o){if(1===o||9===o||11===o){if("string"===typeof e.textContent)return e.textContent;for(e=e.firstChild;e;e=e.nextSibling)i+=a(e)}else if(3===o||4===o)return e.nodeValue}else while(t=e[n++])i+=a(t);return i},n=oe.selectors={cacheLength:50,createPseudo:re,match:Y,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(e){return e[1]=e[1].replace(Z,ee),e[3]=(e[3]||e[4]||e[5]||"").replace(Z,ee),"~="===e[2]&&(e[3]=" "+e[3]+" "),e.slice(0,4)},CHILD:function(e){return e[1]=e[1].toLowerCase(),"nth"===e[1].slice(0,3)?(e[3]||oe.error(e[0]),e[4]=+(e[4]?e[5]+(e[6]||1):2*("even"===e[3]||"odd"===e[3])),e[5]=+(e[7]+e[8]||"odd"===e[3])):e[3]&&oe.error(e[0]),e},PSEUDO:function(e){var t,i=!e[6]&&e[2];return Y["CHILD"].test(e[0])?null:(e[3]?e[2]=e[4]||e[5]||"":i&&V.test(i)&&(t=s(i,!0))&&(t=i.indexOf(")",i.length-t)-i.length)&&(e[0]=e[0].slice(0,t),e[2]=i.slice(0,t)),e.slice(0,3))}},filter:{TAG:function(e){var t=e.replace(Z,ee).toLowerCase();return"*"===e?function(){return!0}:function(e){return e.nodeName&&e.nodeName.toLowerCase()===t}},CLASS:function(e){var t=k[e+" "];return t||(t=new RegExp("(^|"+M+")"+e+"("+M+"|$)"))&&k(e,(function(e){return t.test("string"===typeof e.className&&e.className||"undefined"!==typeof e.getAttribute&&e.getAttribute("class")||"")}))},ATTR:function(e,t,i){return function(n){var a=oe.attr(n,e);return null==a?"!="===t:!t||(a+="","="===t?a===i:"!="===t?a!==i:"^="===t?i&&0===a.indexOf(i):"*="===t?i&&a.indexOf(i)>-1:"$="===t?i&&a.slice(-i.length)===i:"~="===t?(" "+a.replace(W," ")+" ").indexOf(i)>-1:"|="===t&&(a===i||a.slice(0,i.length+1)===i+"-"))}},CHILD:function(e,t,i,n,a){var o="nth"!==e.slice(0,3),s="last"!==e.slice(-4),r="of-type"===t;return 1===n&&0===a?function(e){return!!e.parentNode}:function(t,i,l){var c,h,d,u,p,f,g=o!==s?"nextSibling":"previousSibling",m=t.parentNode,v=r&&t.nodeName.toLowerCase(),b=!l&&!r,w=!1;if(m){if(o){while(g){u=t;while(u=u[g])if(r?u.nodeName.toLowerCase()===v:1===u.nodeType)return!1;f=g="only"===e&&!f&&"nextSibling"}return!0}if(f=[s?m.firstChild:m.lastChild],s&&b){u=m,d=u[y]||(u[y]={}),h=d[u.uniqueID]||(d[u.uniqueID]={}),c=h[e]||[],p=c[0]===_&&c[1],w=p&&c[2],u=p&&m.childNodes[p];while(u=++p&&u&&u[g]||(w=p=0)||f.pop())if(1===u.nodeType&&++w&&u===t){h[e]=[_,p,w];break}}else if(b&&(u=t,d=u[y]||(u[y]={}),h=d[u.uniqueID]||(d[u.uniqueID]={}),c=h[e]||[],p=c[0]===_&&c[1],w=p),!1===w)while(u=++p&&u&&u[g]||(w=p=0)||f.pop())if((r?u.nodeName.toLowerCase()===v:1===u.nodeType)&&++w&&(b&&(d=u[y]||(u[y]={}),h=d[u.uniqueID]||(d[u.uniqueID]={}),h[e]=[_,w]),u===t))break;return w-=a,w===n||w%n===0&&w/n>=0}}},PSEUDO:function(e,t){var i,a=n.pseudos[e]||n.setFilters[e.toLowerCase()]||oe.error("unsupported pseudo: "+e);return a[y]?a(t):a.length>1?(i=[e,e,"",t],n.setFilters.hasOwnProperty(e.toLowerCase())?re((function(e,i){var n,o=a(e,t),s=o.length;while(s--)n=E(e,o[s]),e[n]=!(i[n]=o[s])})):function(e){return a(e,0,i)}):a}},pseudos:{not:re((function(e){var t=[],i=[],n=r(e.replace(L,"$1"));return n[y]?re((function(e,t,i,a){var o,s=n(e,null,a,[]),r=e.length;while(r--)(o=s[r])&&(e[r]=!(t[r]=o))})):function(e,a,o){return t[0]=e,n(t,null,o,i),t[0]=null,!i.pop()}})),has:re((function(e){return function(t){return oe(e,t).length>0}})),contains:re((function(e){return e=e.replace(Z,ee),function(t){return(t.textContent||t.innerText||a(t)).indexOf(e)>-1}})),lang:re((function(e){return q.test(e||"")||oe.error("unsupported lang: "+e),e=e.replace(Z,ee).toLowerCase(),function(t){var i;do{if(i=g?t.lang:t.getAttribute("xml:lang")||t.getAttribute("lang"))return i=i.toLowerCase(),i===e||0===i.indexOf(e+"-")}while((t=t.parentNode)&&1===t.nodeType);return!1}})),target:function(t){var i=e.location&&e.location.hash;return i&&i.slice(1)===t.id},root:function(e){return e===f},focus:function(e){return e===p.activeElement&&(!p.hasFocus||p.hasFocus())&&!!(e.type||e.href||~e.tabIndex)},enabled:pe(!1),disabled:pe(!0),checked:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&!!e.checked||"option"===t&&!!e.selected},selected:function(e){return e.parentNode&&e.parentNode.selectedIndex,!0===e.selected},empty:function(e){for(e=e.firstChild;e;e=e.nextSibling)if(e.nodeType<6)return!1;return!0},parent:function(e){return!n.pseudos["empty"](e)},header:function(e){return K.test(e.nodeName)},input:function(e){return U.test(e.nodeName)},button:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&"button"===e.type||"button"===t},text:function(e){var t;return"input"===e.nodeName.toLowerCase()&&"text"===e.type&&(null==(t=e.getAttribute("type"))||"text"===t.toLowerCase())},first:fe((function(){return[0]})),last:fe((function(e,t){return[t-1]})),eq:fe((function(e,t,i){return[i<0?i+t:i]})),even:fe((function(e,t){for(var i=0;i<t;i+=2)e.push(i);return e})),odd:fe((function(e,t){for(var i=1;i<t;i+=2)e.push(i);return e})),lt:fe((function(e,t,i){for(var n=i<0?i+t:i;--n>=0;)e.push(n);return e})),gt:fe((function(e,t,i){for(var n=i<0?i+t:i;++n<t;)e.push(n);return e}))}},n.pseudos["nth"]=n.pseudos["eq"],{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})n.pseudos[t]=de(t);for(t in{submit:!0,reset:!0})n.pseudos[t]=ue(t);function me(){}function ve(e){for(var t=0,i=e.length,n="";t<i;t++)n+=e[t].value;return n}function be(e,t,i){var n=t.dir,a=t.next,o=a||n,s=i&&"parentNode"===o,r=x++;return t.first?function(t,i,a){while(t=t[n])if(1===t.nodeType||s)return e(t,i,a)}:function(t,i,l){var c,h,d,u=[_,r];if(l){while(t=t[n])if((1===t.nodeType||s)&&e(t,i,l))return!0}else while(t=t[n])if(1===t.nodeType||s)if(d=t[y]||(t[y]={}),h=d[t.uniqueID]||(d[t.uniqueID]={}),a&&a===t.nodeName.toLowerCase())t=t[n]||t;else{if((c=h[o])&&c[0]===_&&c[1]===r)return u[2]=c[2];if(h[o]=u,u[2]=e(t,i,l))return!0}}}function we(e){return e.length>1?function(t,i,n){var a=e.length;while(a--)if(!e[a](t,i,n))return!1;return!0}:e[0]}function ye(e,t,i){for(var n=0,a=t.length;n<a;n++)oe(e,t[n],i);return i}function Ce(e,t,i,n,a){for(var o,s=[],r=0,l=e.length,c=null!=t;r<l;r++)(o=e[r])&&(i&&!i(o,n,a)||(s.push(o),c&&t.push(r)));return s}function _e(e,t,i,n,a,o){return n&&!n[y]&&(n=_e(n)),a&&!a[y]&&(a=_e(a,o)),re((function(o,s,r,l){var c,h,d,u=[],p=[],f=s.length,g=o||ye(t||"*",r.nodeType?[r]:r,[]),m=!e||!o&&t?g:Ce(g,u,e,r,l),v=i?a||(o?e:f||n)?[]:s:m;if(i&&i(m,v,r,l),n){c=Ce(v,p),n(c,[],r,l),h=c.length;while(h--)(d=c[h])&&(v[p[h]]=!(m[p[h]]=d))}if(o){if(a||e){if(a){c=[],h=v.length;while(h--)(d=v[h])&&c.push(m[h]=d);a(null,v=[],c,l)}h=v.length;while(h--)(d=v[h])&&(c=a?E(o,d):u[h])>-1&&(o[c]=!(s[c]=d))}}else v=Ce(v===s?v.splice(f,v.length):v),a?a(null,s,v,l):P.apply(s,v)}))}function xe(e){for(var t,i,a,o=e.length,s=n.relative[e[0].type],r=s||n.relative[" "],l=s?1:0,h=be((function(e){return e===t}),r,!0),d=be((function(e){return E(t,e)>-1}),r,!0),u=[function(e,i,n){var a=!s&&(n||i!==c)||((t=i).nodeType?h(e,i,n):d(e,i,n));return t=null,a}];l<o;l++)if(i=n.relative[e[l].type])u=[be(we(u),i)];else{if(i=n.filter[e[l].type].apply(null,e[l].matches),i[y]){for(a=++l;a<o;a++)if(n.relative[e[a].type])break;return _e(l>1&&we(u),l>1&&ve(e.slice(0,l-1).concat({value:" "===e[l-2].type?"*":""})).replace(L,"$1"),i,l<a&&xe(e.slice(l,a)),a<o&&xe(e=e.slice(a)),a<o&&ve(e))}u.push(i)}return we(u)}function ke(e,t){var i=t.length>0,a=e.length>0,o=function(o,s,r,l,h){var d,f,m,v=0,b="0",w=o&&[],y=[],C=c,x=o||a&&n.find["TAG"]("*",h),k=_+=null==C?1:Math.random()||.1,S=x.length;for(h&&(c=s===p||s||h);b!==S&&null!=(d=x[b]);b++){if(a&&d){f=0,s||d.ownerDocument===p||(u(d),r=!g);while(m=e[f++])if(m(d,s||p,r)){l.push(d);break}h&&(_=k)}i&&((d=!m&&d)&&v--,o&&w.push(d))}if(v+=b,i&&b!==v){f=0;while(m=t[f++])m(w,y,s,r);if(o){if(v>0)while(b--)w[b]||y[b]||(y[b]=R.call(l));y=Ce(y)}P.apply(l,y),h&&!o&&y.length>0&&v+t.length>1&&oe.uniqueSort(l)}return h&&(_=k,c=C),w};return i?re(o):o}return me.prototype=n.filters=n.pseudos,n.setFilters=new me,s=oe.tokenize=function(e,t){var i,a,o,s,r,l,c,h=S[e+" "];if(h)return t?0:h.slice(0);r=e,l=[],c=n.preFilter;while(r){for(s in i&&!(a=j.exec(r))||(a&&(r=r.slice(a[0].length)||r),l.push(o=[])),i=!1,(a=B.exec(r))&&(i=a.shift(),o.push({value:i,type:a[0].replace(L," ")}),r=r.slice(i.length)),n.filter)!(a=Y[s].exec(r))||c[s]&&!(a=c[s](a))||(i=a.shift(),o.push({value:i,type:s,matches:a}),r=r.slice(i.length));if(!i)break}return t?r.length:r?oe.error(e):S(e,l).slice(0)},r=oe.compile=function(e,t){var i,n=[],a=[],o=T[e+" "];if(!o){t||(t=s(e)),i=t.length;while(i--)o=xe(t[i]),o[y]?n.push(o):a.push(o);o=T(e,ke(a,n)),o.selector=e}return o},l=oe.select=function(e,t,a,o){var l,c,h,d,u,p="function"===typeof e&&e,f=!o&&s(e=p.selector||e);if(a=a||[],1===f.length){if(c=f[0]=f[0].slice(0),c.length>2&&"ID"===(h=c[0]).type&&i.getById&&9===t.nodeType&&g&&n.relative[c[1].type]){if(t=(n.find["ID"](h.matches[0].replace(Z,ee),t)||[])[0],!t)return a;p&&(t=t.parentNode),e=e.slice(c.shift().value.length)}l=Y["needsContext"].test(e)?0:c.length;while(l--){if(h=c[l],n.relative[d=h.type])break;if((u=n.find[d])&&(o=u(h.matches[0].replace(Z,ee),J.test(c[0].type)&&ge(t.parentNode)||t))){if(c.splice(l,1),e=o.length&&ve(c),!e)return P.apply(a,o),a;break}}}return(p||r(e,f))(o,t,!g,a,!t||J.test(e)&&ge(t.parentNode)||t),a},i.sortStable=y.split("").sort($).join("")===y,i.detectDuplicates=!!d,u(),i.sortDetached=le((function(e){return 1&e.compareDocumentPosition(p.createElement("fieldset"))})),le((function(e){return e.innerHTML="<a href='#'></a>","#"===e.firstChild.getAttribute("href")}))||ce("type|href|height|width",(function(e,t,i){if(!i)return e.getAttribute(t,"type"===t.toLowerCase()?1:2)})),i.attributes&&le((function(e){return e.innerHTML="<input/>",e.firstChild.setAttribute("value",""),""===e.firstChild.getAttribute("value")}))||ce("value",(function(e,t,i){if(!i&&"input"===e.nodeName.toLowerCase())return e.defaultValue})),le((function(e){return null==e.getAttribute("disabled")}))||ce(z,(function(e,t,i){var n;if(!i)return!0===e[t]?t.toLowerCase():(n=e.getAttributeNode(t))&&n.specified?n.value:null})),oe}(i);C.find=$,C.expr=$.selectors,C.expr[":"]=C.expr.pseudos,C.uniqueSort=C.unique=$.uniqueSort,C.text=$.getText,C.isXMLDoc=$.isXML,C.contains=$.contains,C.escapeSelector=$.escape;var D=function(e,t,i){var n=[],a=void 0!==i;while((e=e[t])&&9!==e.nodeType)if(1===e.nodeType){if(a&&C(e).is(i))break;n.push(e)}return n},N=function(e,t){for(var i=[];e;e=e.nextSibling)1===e.nodeType&&e!==t&&i.push(e);return i},R=C.expr.match.needsContext,I=/^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i,P=/^.[^:#\[\.,]*$/;function A(e,t,i){if(C.isFunction(t))return C.grep(e,(function(e,n){return!!t.call(e,n,e)!==i}));if(t.nodeType)return C.grep(e,(function(e){return e===t!==i}));if("string"===typeof t){if(P.test(t))return C.filter(t,e,i);t=C.filter(t,e)}return C.grep(e,(function(e){return u.call(t,e)>-1!==i&&1===e.nodeType}))}C.filter=function(e,t,i){var n=t[0];return i&&(e=":not("+e+")"),1===t.length&&1===n.nodeType?C.find.matchesSelector(n,e)?[n]:[]:C.find.matches(e,C.grep(t,(function(e){return 1===e.nodeType})))},C.fn.extend({find:function(e){var t,i,n=this.length,a=this;if("string"!==typeof e)return this.pushStack(C(e).filter((function(){for(t=0;t<n;t++)if(C.contains(a[t],this))return!0})));for(i=this.pushStack([]),t=0;t<n;t++)C.find(e,a[t],i);return n>1?C.uniqueSort(i):i},filter:function(e){return this.pushStack(A(this,e||[],!1))},not:function(e){return this.pushStack(A(this,e||[],!0))},is:function(e){return!!A(this,"string"===typeof e&&R.test(e)?C(e):e||[],!1).length}});var E,z=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/,M=C.fn.init=function(e,t,i){var n,a;if(!e)return this;if(i=i||E,"string"===typeof e){if(n="<"===e[0]&&">"===e[e.length-1]&&e.length>=3?[null,e,null]:z.exec(e),!n||!n[1]&&t)return!t||t.jquery?(t||i).find(e):this.constructor(t).find(e);if(n[1]){if(t=t instanceof C?t[0]:t,C.merge(this,C.parseHTML(n[1],t&&t.nodeType?t.ownerDocument||t:r,!0)),I.test(n[1])&&C.isPlainObject(t))for(n in t)C.isFunction(this[n])?this[n](t[n]):this.attr(n,t[n]);return this}return a=r.getElementById(n[2]),a&&(this[0]=a,this.length=1),this}return e.nodeType?(this[0]=e,this.length=1,this):C.isFunction(e)?void 0!==i.ready?i.ready(e):e(C):C.makeArray(e,this)};M.prototype=C.fn,E=C(r);var H=/^(?:parents|prev(?:Until|All))/,F={children:!0,contents:!0,next:!0,prev:!0};function O(e,t){while((e=e[t])&&1!==e.nodeType);return e}C.fn.extend({has:function(e){var t=C(e,this),i=t.length;return this.filter((function(){for(var e=0;e<i;e++)if(C.contains(this,t[e]))return!0}))},closest:function(e,t){var i,n=0,a=this.length,o=[],s="string"!==typeof e&&C(e);if(!R.test(e))for(;n<a;n++)for(i=this[n];i&&i!==t;i=i.parentNode)if(i.nodeType<11&&(s?s.index(i)>-1:1===i.nodeType&&C.find.matchesSelector(i,e))){o.push(i);break}return this.pushStack(o.length>1?C.uniqueSort(o):o)},index:function(e){return e?"string"===typeof e?u.call(C(e),this[0]):u.call(this,e.jquery?e[0]:e):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(e,t){return this.pushStack(C.uniqueSort(C.merge(this.get(),C(e,t))))},addBack:function(e){return this.add(null==e?this.prevObject:this.prevObject.filter(e))}}),C.each({parent:function(e){var t=e.parentNode;return t&&11!==t.nodeType?t:null},parents:function(e){return D(e,"parentNode")},parentsUntil:function(e,t,i){return D(e,"parentNode",i)},next:function(e){return O(e,"nextSibling")},prev:function(e){return O(e,"previousSibling")},nextAll:function(e){return D(e,"nextSibling")},prevAll:function(e){return D(e,"previousSibling")},nextUntil:function(e,t,i){return D(e,"nextSibling",i)},prevUntil:function(e,t,i){return D(e,"previousSibling",i)},siblings:function(e){return N((e.parentNode||{}).firstChild,e)},children:function(e){return N(e.firstChild)},contents:function(e){return e.contentDocument||C.merge([],e.childNodes)}},(function(e,t){C.fn[e]=function(i,n){var a=C.map(this,t,i);return"Until"!==e.slice(-5)&&(n=i),n&&"string"===typeof n&&(a=C.filter(n,a)),this.length>1&&(F[e]||C.uniqueSort(a),H.test(e)&&a.reverse()),this.pushStack(a)}}));var W=/\S+/g;function L(e){var t={};return C.each(e.match(W)||[],(function(e,i){t[i]=!0})),t}function j(e){return e}function B(e){throw e}function G(e,t,i){var n;try{e&&C.isFunction(n=e.promise)?n.call(e).done(t).fail(i):e&&C.isFunction(n=e.then)?n.call(e,t,i):t.call(void 0,e)}catch(e){i.call(void 0,e)}}C.Callbacks=function(e){e="string"===typeof e?L(e):C.extend({},e);var t,i,n,a,o=[],s=[],r=-1,l=function(){for(a=e.once,n=t=!0;s.length;r=-1){i=s.shift();while(++r<o.length)!1===o[r].apply(i[0],i[1])&&e.stopOnFalse&&(r=o.length,i=!1)}e.memory||(i=!1),t=!1,a&&(o=i?[]:"")},c={add:function(){return o&&(i&&!t&&(r=o.length-1,s.push(i)),function t(i){C.each(i,(function(i,n){C.isFunction(n)?e.unique&&c.has(n)||o.push(n):n&&n.length&&"string"!==C.type(n)&&t(n)}))}(arguments),i&&!t&&l()),this},remove:function(){return C.each(arguments,(function(e,t){var i;while((i=C.inArray(t,o,i))>-1)o.splice(i,1),i<=r&&r--})),this},has:function(e){return e?C.inArray(e,o)>-1:o.length>0},empty:function(){return o&&(o=[]),this},disable:function(){return a=s=[],o=i="",this},disabled:function(){return!o},lock:function(){return a=s=[],i||t||(o=i=""),this},locked:function(){return!!a},fireWith:function(e,i){return a||(i=i||[],i=[e,i.slice?i.slice():i],s.push(i),t||l()),this},fire:function(){return c.fireWith(this,arguments),this},fired:function(){return!!n}};return c},C.extend({Deferred:function(e){var t=[["notify","progress",C.Callbacks("memory"),C.Callbacks("memory"),2],["resolve","done",C.Callbacks("once memory"),C.Callbacks("once memory"),0,"resolved"],["reject","fail",C.Callbacks("once memory"),C.Callbacks("once memory"),1,"rejected"]],n="pending",a={state:function(){return n},always:function(){return o.done(arguments).fail(arguments),this},catch:function(e){return a.then(null,e)},pipe:function(){var e=arguments;return C.Deferred((function(i){C.each(t,(function(t,n){var a=C.isFunction(e[n[4]])&&e[n[4]];o[n[1]]((function(){var e=a&&a.apply(this,arguments);e&&C.isFunction(e.promise)?e.promise().progress(i.notify).done(i.resolve).fail(i.reject):i[n[0]+"With"](this,a?[e]:arguments)}))})),e=null})).promise()},then:function(e,n,a){var o=0;function s(e,t,n,a){return function(){var r=this,l=arguments,c=function(){var i,c;if(!(e<o)){if(i=n.apply(r,l),i===t.promise())throw new TypeError("Thenable self-resolution");c=i&&("object"===typeof i||"function"===typeof i)&&i.then,C.isFunction(c)?a?c.call(i,s(o,t,j,a),s(o,t,B,a)):(o++,c.call(i,s(o,t,j,a),s(o,t,B,a),s(o,t,j,t.notifyWith))):(n!==j&&(r=void 0,l=[i]),(a||t.resolveWith)(r,l))}},h=a?c:function(){try{c()}catch(i){C.Deferred.exceptionHook&&C.Deferred.exceptionHook(i,h.stackTrace),e+1>=o&&(n!==B&&(r=void 0,l=[i]),t.rejectWith(r,l))}};e?h():(C.Deferred.getStackHook&&(h.stackTrace=C.Deferred.getStackHook()),i.setTimeout(h))}}return C.Deferred((function(i){t[0][3].add(s(0,i,C.isFunction(a)?a:j,i.notifyWith)),t[1][3].add(s(0,i,C.isFunction(e)?e:j)),t[2][3].add(s(0,i,C.isFunction(n)?n:B))})).promise()},promise:function(e){return null!=e?C.extend(e,a):a}},o={};return C.each(t,(function(e,i){var s=i[2],r=i[5];a[i[1]]=s.add,r&&s.add((function(){n=r}),t[3-e][2].disable,t[0][2].lock),s.add(i[3].fire),o[i[0]]=function(){return o[i[0]+"With"](this===o?void 0:this,arguments),this},o[i[0]+"With"]=s.fireWith})),a.promise(o),e&&e.call(o,o),o},when:function(e){var t=arguments.length,i=t,n=Array(i),a=c.call(arguments),o=C.Deferred(),s=function(e){return function(i){n[e]=this,a[e]=arguments.length>1?c.call(arguments):i,--t||o.resolveWith(n,a)}};if(t<=1&&(G(e,o.done(s(i)).resolve,o.reject),"pending"===o.state()||C.isFunction(a[i]&&a[i].then)))return o.then();while(i--)G(a[i],s(i),o.reject);return o.promise()}});var V=/^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;C.Deferred.exceptionHook=function(e,t){i.console&&i.console.warn&&e&&V.test(e.name)&&i.console.warn("jQuery.Deferred exception: "+e.message,e.stack,t)},C.readyException=function(e){i.setTimeout((function(){throw e}))};var q=C.Deferred();function Y(){r.removeEventListener("DOMContentLoaded",Y),i.removeEventListener("load",Y),C.ready()}C.fn.ready=function(e){return q.then(e).catch((function(e){C.readyException(e)})),this},C.extend({isReady:!1,readyWait:1,holdReady:function(e){e?C.readyWait++:C.ready(!0)},ready:function(e){(!0===e?--C.readyWait:C.isReady)||(C.isReady=!0,!0!==e&&--C.readyWait>0||q.resolveWith(r,[C]))}}),C.ready.then=q.then,"complete"===r.readyState||"loading"!==r.readyState&&!r.documentElement.doScroll?i.setTimeout(C.ready):(r.addEventListener("DOMContentLoaded",Y),i.addEventListener("load",Y));var U=function(e,t,i,n,a,o,s){var r=0,l=e.length,c=null==i;if("object"===C.type(i))for(r in a=!0,i)U(e,t,r,i[r],!0,o,s);else if(void 0!==n&&(a=!0,C.isFunction(n)||(s=!0),c&&(s?(t.call(e,n),t=null):(c=t,t=function(e,t,i){return c.call(C(e),i)})),t))for(;r<l;r++)t(e[r],i,s?n:n.call(e[r],r,t(e[r],i)));return a?e:c?t.call(e):l?t(e[0],i):o},K=function(e){return 1===e.nodeType||9===e.nodeType||!+e.nodeType};function X(){this.expando=C.expando+X.uid++}X.uid=1,X.prototype={cache:function(e){var t=e[this.expando];return t||(t={},K(e)&&(e.nodeType?e[this.expando]=t:Object.defineProperty(e,this.expando,{value:t,configurable:!0}))),t},set:function(e,t,i){var n,a=this.cache(e);if("string"===typeof t)a[C.camelCase(t)]=i;else for(n in t)a[C.camelCase(n)]=t[n];return a},get:function(e,t){return void 0===t?this.cache(e):e[this.expando]&&e[this.expando][C.camelCase(t)]},access:function(e,t,i){return void 0===t||t&&"string"===typeof t&&void 0===i?this.get(e,t):(this.set(e,t,i),void 0!==i?i:t)},remove:function(e,t){var i,n=e[this.expando];if(void 0!==n){if(void 0!==t){C.isArray(t)?t=t.map(C.camelCase):(t=C.camelCase(t),t=t in n?[t]:t.match(W)||[]),i=t.length;while(i--)delete n[t[i]]}(void 0===t||C.isEmptyObject(n))&&(e.nodeType?e[this.expando]=void 0:delete e[this.expando])}},hasData:function(e){var t=e[this.expando];return void 0!==t&&!C.isEmptyObject(t)}};var Q=new X,J=new X,Z=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,ee=/[A-Z]/g;function te(e,t,i){var n;if(void 0===i&&1===e.nodeType)if(n="data-"+t.replace(ee,"-$&").toLowerCase(),i=e.getAttribute(n),"string"===typeof i){try{i="true"===i||"false"!==i&&("null"===i?null:+i+""===i?+i:Z.test(i)?JSON.parse(i):i)}catch(a){}J.set(e,t,i)}else i=void 0;return i}C.extend({hasData:function(e){return J.hasData(e)||Q.hasData(e)},data:function(e,t,i){return J.access(e,t,i)},removeData:function(e,t){J.remove(e,t)},_data:function(e,t,i){return Q.access(e,t,i)},_removeData:function(e,t){Q.remove(e,t)}}),C.fn.extend({data:function(e,t){var i,n,a,o=this[0],s=o&&o.attributes;if(void 0===e){if(this.length&&(a=J.get(o),1===o.nodeType&&!Q.get(o,"hasDataAttrs"))){i=s.length;while(i--)s[i]&&(n=s[i].name,0===n.indexOf("data-")&&(n=C.camelCase(n.slice(5)),te(o,n,a[n])));Q.set(o,"hasDataAttrs",!0)}return a}return"object"===typeof e?this.each((function(){J.set(this,e)})):U(this,(function(t){var i;if(o&&void 0===t)return i=J.get(o,e),void 0!==i?i:(i=te(o,e),void 0!==i?i:void 0);this.each((function(){J.set(this,e,t)}))}),null,t,arguments.length>1,null,!0)},removeData:function(e){return this.each((function(){J.remove(this,e)}))}}),C.extend({queue:function(e,t,i){var n;if(e)return t=(t||"fx")+"queue",n=Q.get(e,t),i&&(!n||C.isArray(i)?n=Q.access(e,t,C.makeArray(i)):n.push(i)),n||[]},dequeue:function(e,t){t=t||"fx";var i=C.queue(e,t),n=i.length,a=i.shift(),o=C._queueHooks(e,t),s=function(){C.dequeue(e,t)};"inprogress"===a&&(a=i.shift(),n--),a&&("fx"===t&&i.unshift("inprogress"),delete o.stop,a.call(e,s,o)),!n&&o&&o.empty.fire()},_queueHooks:function(e,t){var i=t+"queueHooks";return Q.get(e,i)||Q.access(e,i,{empty:C.Callbacks("once memory").add((function(){Q.remove(e,[t+"queue",i])}))})}}),C.fn.extend({queue:function(e,t){var i=2;return"string"!==typeof e&&(t=e,e="fx",i--),arguments.length<i?C.queue(this[0],e):void 0===t?this:this.each((function(){var i=C.queue(this,e,t);C._queueHooks(this,e),"fx"===e&&"inprogress"!==i[0]&&C.dequeue(this,e)}))},dequeue:function(e){return this.each((function(){C.dequeue(this,e)}))},clearQueue:function(e){return this.queue(e||"fx",[])},promise:function(e,t){var i,n=1,a=C.Deferred(),o=this,s=this.length,r=function(){--n||a.resolveWith(o,[o])};"string"!==typeof e&&(t=e,e=void 0),e=e||"fx";while(s--)i=Q.get(o[s],e+"queueHooks"),i&&i.empty&&(n++,i.empty.add(r));return r(),a.promise(t)}});var ie=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,ne=new RegExp("^(?:([+-])=|)("+ie+")([a-z%]*)$","i"),ae=["Top","Right","Bottom","Left"],oe=function(e,t){return e=t||e,"none"===e.style.display||""===e.style.display&&C.contains(e.ownerDocument,e)&&"none"===C.css(e,"display")},se=function(e,t,i,n){var a,o,s={};for(o in t)s[o]=e.style[o],e.style[o]=t[o];for(o in a=i.apply(e,n||[]),t)e.style[o]=s[o];return a};function re(e,t,i,n){var a,o=1,s=20,r=n?function(){return n.cur()}:function(){return C.css(e,t,"")},l=r(),c=i&&i[3]||(C.cssNumber[t]?"":"px"),h=(C.cssNumber[t]||"px"!==c&&+l)&&ne.exec(C.css(e,t));if(h&&h[3]!==c){c=c||h[3],i=i||[],h=+l||1;do{o=o||".5",h/=o,C.style(e,t,h+c)}while(o!==(o=r()/l)&&1!==o&&--s)}return i&&(h=+h||+l||0,a=i[1]?h+(i[1]+1)*i[2]:+i[2],n&&(n.unit=c,n.start=h,n.end=a)),a}var le={};function ce(e){var t,i=e.ownerDocument,n=e.nodeName,a=le[n];return a||(t=i.body.appendChild(i.createElement(n)),a=C.css(t,"display"),t.parentNode.removeChild(t),"none"===a&&(a="block"),le[n]=a,a)}function he(e,t){for(var i,n,a=[],o=0,s=e.length;o<s;o++)n=e[o],n.style&&(i=n.style.display,t?("none"===i&&(a[o]=Q.get(n,"display")||null,a[o]||(n.style.display="")),""===n.style.display&&oe(n)&&(a[o]=ce(n))):"none"!==i&&(a[o]="none",Q.set(n,"display",i)));for(o=0;o<s;o++)null!=a[o]&&(e[o].style.display=a[o]);return e}C.fn.extend({show:function(){return he(this,!0)},hide:function(){return he(this)},toggle:function(e){return"boolean"===typeof e?e?this.show():this.hide():this.each((function(){oe(this)?C(this).show():C(this).hide()}))}});var de=/^(?:checkbox|radio)$/i,ue=/<([a-z][^\/\0>\x20\t\r\n\f]+)/i,pe=/^$|\/(?:java|ecma)script/i,fe={option:[1,"<select multiple='multiple'>","</select>"],thead:[1,"<table>","</table>"],col:[2,"<table><colgroup>","</colgroup></table>"],tr:[2,"<table><tbody>","</tbody></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:[0,"",""]};function ge(e,t){var i="undefined"!==typeof e.getElementsByTagName?e.getElementsByTagName(t||"*"):"undefined"!==typeof e.querySelectorAll?e.querySelectorAll(t||"*"):[];return void 0===t||t&&C.nodeName(e,t)?C.merge([e],i):i}function me(e,t){for(var i=0,n=e.length;i<n;i++)Q.set(e[i],"globalEval",!t||Q.get(t[i],"globalEval"))}fe.optgroup=fe.option,fe.tbody=fe.tfoot=fe.colgroup=fe.caption=fe.thead,fe.th=fe.td;var ve=/<|&#?\w+;/;function be(e,t,i,n,a){for(var o,s,r,l,c,h,d=t.createDocumentFragment(),u=[],p=0,f=e.length;p<f;p++)if(o=e[p],o||0===o)if("object"===C.type(o))C.merge(u,o.nodeType?[o]:o);else if(ve.test(o)){s=s||d.appendChild(t.createElement("div")),r=(ue.exec(o)||["",""])[1].toLowerCase(),l=fe[r]||fe._default,s.innerHTML=l[1]+C.htmlPrefilter(o)+l[2],h=l[0];while(h--)s=s.lastChild;C.merge(u,s.childNodes),s=d.firstChild,s.textContent=""}else u.push(t.createTextNode(o));d.textContent="",p=0;while(o=u[p++])if(n&&C.inArray(o,n)>-1)a&&a.push(o);else if(c=C.contains(o.ownerDocument,o),s=ge(d.appendChild(o),"script"),c&&me(s),i){h=0;while(o=s[h++])pe.test(o.type||"")&&i.push(o)}return d}(function(){var e=r.createDocumentFragment(),t=e.appendChild(r.createElement("div")),i=r.createElement("input");i.setAttribute("type","radio"),i.setAttribute("checked","checked"),i.setAttribute("name","t"),t.appendChild(i),b.checkClone=t.cloneNode(!0).cloneNode(!0).lastChild.checked,t.innerHTML="<textarea>x</textarea>",b.noCloneChecked=!!t.cloneNode(!0).lastChild.defaultValue})();var we=r.documentElement,ye=/^key/,Ce=/^(?:mouse|pointer|contextmenu|drag|drop)|click/,_e=/^([^.]*)(?:\.(.+)|)/;function xe(){return!0}function ke(){return!1}function Se(){try{return r.activeElement}catch(e){}}function Te(e,t,i,n,a,o){var s,r;if("object"===typeof t){for(r in"string"!==typeof i&&(n=n||i,i=void 0),t)Te(e,r,i,n,t[r],o);return e}if(null==n&&null==a?(a=i,n=i=void 0):null==a&&("string"===typeof i?(a=n,n=void 0):(a=n,n=i,i=void 0)),!1===a)a=ke;else if(!a)return e;return 1===o&&(s=a,a=function(e){return C().off(e),s.apply(this,arguments)},a.guid=s.guid||(s.guid=C.guid++)),e.each((function(){C.event.add(this,t,a,n,i)}))}C.event={global:{},add:function(e,t,i,n,a){var o,s,r,l,c,h,d,u,p,f,g,m=Q.get(e);if(m){i.handler&&(o=i,i=o.handler,a=o.selector),a&&C.find.matchesSelector(we,a),i.guid||(i.guid=C.guid++),(l=m.events)||(l=m.events={}),(s=m.handle)||(s=m.handle=function(t){return"undefined"!==typeof C&&C.event.triggered!==t.type?C.event.dispatch.apply(e,arguments):void 0}),t=(t||"").match(W)||[""],c=t.length;while(c--)r=_e.exec(t[c])||[],p=g=r[1],f=(r[2]||"").split(".").sort(),p&&(d=C.event.special[p]||{},p=(a?d.delegateType:d.bindType)||p,d=C.event.special[p]||{},h=C.extend({type:p,origType:g,data:n,handler:i,guid:i.guid,selector:a,needsContext:a&&C.expr.match.needsContext.test(a),namespace:f.join(".")},o),(u=l[p])||(u=l[p]=[],u.delegateCount=0,d.setup&&!1!==d.setup.call(e,n,f,s)||e.addEventListener&&e.addEventListener(p,s)),d.add&&(d.add.call(e,h),h.handler.guid||(h.handler.guid=i.guid)),a?u.splice(u.delegateCount++,0,h):u.push(h),C.event.global[p]=!0)}},remove:function(e,t,i,n,a){var o,s,r,l,c,h,d,u,p,f,g,m=Q.hasData(e)&&Q.get(e);if(m&&(l=m.events)){t=(t||"").match(W)||[""],c=t.length;while(c--)if(r=_e.exec(t[c])||[],p=g=r[1],f=(r[2]||"").split(".").sort(),p){d=C.event.special[p]||{},p=(n?d.delegateType:d.bindType)||p,u=l[p]||[],r=r[2]&&new RegExp("(^|\\.)"+f.join("\\.(?:.*\\.|)")+"(\\.|$)"),s=o=u.length;while(o--)h=u[o],!a&&g!==h.origType||i&&i.guid!==h.guid||r&&!r.test(h.namespace)||n&&n!==h.selector&&("**"!==n||!h.selector)||(u.splice(o,1),h.selector&&u.delegateCount--,d.remove&&d.remove.call(e,h));s&&!u.length&&(d.teardown&&!1!==d.teardown.call(e,f,m.handle)||C.removeEvent(e,p,m.handle),delete l[p])}else for(p in l)C.event.remove(e,p+t[c],i,n,!0);C.isEmptyObject(l)&&Q.remove(e,"handle events")}},dispatch:function(e){var t,i,n,a,o,s,r=C.event.fix(e),l=new Array(arguments.length),c=(Q.get(this,"events")||{})[r.type]||[],h=C.event.special[r.type]||{};for(l[0]=r,t=1;t<arguments.length;t++)l[t]=arguments[t];if(r.delegateTarget=this,!h.preDispatch||!1!==h.preDispatch.call(this,r)){s=C.event.handlers.call(this,r,c),t=0;while((a=s[t++])&&!r.isPropagationStopped()){r.currentTarget=a.elem,i=0;while((o=a.handlers[i++])&&!r.isImmediatePropagationStopped())r.rnamespace&&!r.rnamespace.test(o.namespace)||(r.handleObj=o,r.data=o.data,n=((C.event.special[o.origType]||{}).handle||o.handler).apply(a.elem,l),void 0!==n&&!1===(r.result=n)&&(r.preventDefault(),r.stopPropagation()))}return h.postDispatch&&h.postDispatch.call(this,r),r.result}},handlers:function(e,t){var i,n,a,o,s=[],r=t.delegateCount,l=e.target;if(r&&l.nodeType&&("click"!==e.type||isNaN(e.button)||e.button<1))for(;l!==this;l=l.parentNode||this)if(1===l.nodeType&&(!0!==l.disabled||"click"!==e.type)){for(n=[],i=0;i<r;i++)o=t[i],a=o.selector+" ",void 0===n[a]&&(n[a]=o.needsContext?C(a,this).index(l)>-1:C.find(a,this,null,[l]).length),n[a]&&n.push(o);n.length&&s.push({elem:l,handlers:n})}return r<t.length&&s.push({elem:this,handlers:t.slice(r)}),s},addProp:function(e,t){Object.defineProperty(C.Event.prototype,e,{enumerable:!0,configurable:!0,get:C.isFunction(t)?function(){if(this.originalEvent)return t(this.originalEvent)}:function(){if(this.originalEvent)return this.originalEvent[e]},set:function(t){Object.defineProperty(this,e,{enumerable:!0,configurable:!0,writable:!0,value:t})}})},fix:function(e){return e[C.expando]?e:new C.Event(e)},special:{load:{noBubble:!0},focus:{trigger:function(){if(this!==Se()&&this.focus)return this.focus(),!1},delegateType:"focusin"},blur:{trigger:function(){if(this===Se()&&this.blur)return this.blur(),!1},delegateType:"focusout"},click:{trigger:function(){if("checkbox"===this.type&&this.click&&C.nodeName(this,"input"))return this.click(),!1},_default:function(e){return C.nodeName(e.target,"a")}},beforeunload:{postDispatch:function(e){void 0!==e.result&&e.originalEvent&&(e.originalEvent.returnValue=e.result)}}}},C.removeEvent=function(e,t,i){e.removeEventListener&&e.removeEventListener(t,i)},C.Event=function(e,t){if(!(this instanceof C.Event))return new C.Event(e,t);e&&e.type?(this.originalEvent=e,this.type=e.type,this.isDefaultPrevented=e.defaultPrevented||void 0===e.defaultPrevented&&!1===e.returnValue?xe:ke,this.target=e.target&&3===e.target.nodeType?e.target.parentNode:e.target,this.currentTarget=e.currentTarget,this.relatedTarget=e.relatedTarget):this.type=e,t&&C.extend(this,t),this.timeStamp=e&&e.timeStamp||C.now(),this[C.expando]=!0},C.Event.prototype={constructor:C.Event,isDefaultPrevented:ke,isPropagationStopped:ke,isImmediatePropagationStopped:ke,isSimulated:!1,preventDefault:function(){var e=this.originalEvent;this.isDefaultPrevented=xe,e&&!this.isSimulated&&e.preventDefault()},stopPropagation:function(){var e=this.originalEvent;this.isPropagationStopped=xe,e&&!this.isSimulated&&e.stopPropagation()},stopImmediatePropagation:function(){var e=this.originalEvent;this.isImmediatePropagationStopped=xe,e&&!this.isSimulated&&e.stopImmediatePropagation(),this.stopPropagation()}},C.each({altKey:!0,bubbles:!0,cancelable:!0,changedTouches:!0,ctrlKey:!0,detail:!0,eventPhase:!0,metaKey:!0,pageX:!0,pageY:!0,shiftKey:!0,view:!0,char:!0,charCode:!0,key:!0,keyCode:!0,button:!0,buttons:!0,clientX:!0,clientY:!0,offsetX:!0,offsetY:!0,pointerId:!0,pointerType:!0,screenX:!0,screenY:!0,targetTouches:!0,toElement:!0,touches:!0,which:function(e){var t=e.button;return null==e.which&&ye.test(e.type)?null!=e.charCode?e.charCode:e.keyCode:!e.which&&void 0!==t&&Ce.test(e.type)?1&t?1:2&t?3:4&t?2:0:e.which}},C.event.addProp),C.each({mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},(function(e,t){C.event.special[e]={delegateType:t,bindType:t,handle:function(e){var i,n=this,a=e.relatedTarget,o=e.handleObj;return a&&(a===n||C.contains(n,a))||(e.type=o.origType,i=o.handler.apply(this,arguments),e.type=t),i}}})),C.fn.extend({on:function(e,t,i,n){return Te(this,e,t,i,n)},one:function(e,t,i,n){return Te(this,e,t,i,n,1)},off:function(e,t,i){var n,a;if(e&&e.preventDefault&&e.handleObj)return n=e.handleObj,C(e.delegateTarget).off(n.namespace?n.origType+"."+n.namespace:n.origType,n.selector,n.handler),this;if("object"===typeof e){for(a in e)this.off(a,t,e[a]);return this}return!1!==t&&"function"!==typeof t||(i=t,t=void 0),!1===i&&(i=ke),this.each((function(){C.event.remove(this,e,i,t)}))}});var $e=/<(?!area|br|col|embed|hr|img|input|link|meta|param)(([a-z][^\/\0>\x20\t\r\n\f]*)[^>]*)\/>/gi,De=/<script|<style|<link/i,Ne=/checked\s*(?:[^=]|=\s*.checked.)/i,Re=/^true\/(.*)/,Ie=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;function Pe(e,t){return C.nodeName(e,"table")&&C.nodeName(11!==t.nodeType?t:t.firstChild,"tr")&&e.getElementsByTagName("tbody")[0]||e}function Ae(e){return e.type=(null!==e.getAttribute("type"))+"/"+e.type,e}function Ee(e){var t=Re.exec(e.type);return t?e.type=t[1]:e.removeAttribute("type"),e}function ze(e,t){var i,n,a,o,s,r,l,c;if(1===t.nodeType){if(Q.hasData(e)&&(o=Q.access(e),s=Q.set(t,o),c=o.events,c))for(a in delete s.handle,s.events={},c)for(i=0,n=c[a].length;i<n;i++)C.event.add(t,a,c[a][i]);J.hasData(e)&&(r=J.access(e),l=C.extend({},r),J.set(t,l))}}function Me(e,t){var i=t.nodeName.toLowerCase();"input"===i&&de.test(e.type)?t.checked=e.checked:"input"!==i&&"textarea"!==i||(t.defaultValue=e.defaultValue)}function He(e,t,i,n){t=h.apply([],t);var a,o,s,r,l,c,d=0,u=e.length,p=u-1,f=t[0],g=C.isFunction(f);if(g||u>1&&"string"===typeof f&&!b.checkClone&&Ne.test(f))return e.each((function(a){var o=e.eq(a);g&&(t[0]=f.call(this,a,o.html())),He(o,t,i,n)}));if(u&&(a=be(t,e[0].ownerDocument,!1,e,n),o=a.firstChild,1===a.childNodes.length&&(a=o),o||n)){for(s=C.map(ge(a,"script"),Ae),r=s.length;d<u;d++)l=a,d!==p&&(l=C.clone(l,!0,!0),r&&C.merge(s,ge(l,"script"))),i.call(e[d],l,d);if(r)for(c=s[s.length-1].ownerDocument,C.map(s,Ee),d=0;d<r;d++)l=s[d],pe.test(l.type||"")&&!Q.access(l,"globalEval")&&C.contains(c,l)&&(l.src?C._evalUrl&&C._evalUrl(l.src):w(l.textContent.replace(Ie,""),c))}return e}function Fe(e,t,i){for(var n,a=t?C.filter(t,e):e,o=0;null!=(n=a[o]);o++)i||1!==n.nodeType||C.cleanData(ge(n)),n.parentNode&&(i&&C.contains(n.ownerDocument,n)&&me(ge(n,"script")),n.parentNode.removeChild(n));return e}C.extend({htmlPrefilter:function(e){return e.replace($e,"<$1></$2>")},clone:function(e,t,i){var n,a,o,s,r=e.cloneNode(!0),l=C.contains(e.ownerDocument,e);if(!b.noCloneChecked&&(1===e.nodeType||11===e.nodeType)&&!C.isXMLDoc(e))for(s=ge(r),o=ge(e),n=0,a=o.length;n<a;n++)Me(o[n],s[n]);if(t)if(i)for(o=o||ge(e),s=s||ge(r),n=0,a=o.length;n<a;n++)ze(o[n],s[n]);else ze(e,r);return s=ge(r,"script"),s.length>0&&me(s,!l&&ge(e,"script")),r},cleanData:function(e){for(var t,i,n,a=C.event.special,o=0;void 0!==(i=e[o]);o++)if(K(i)){if(t=i[Q.expando]){if(t.events)for(n in t.events)a[n]?C.event.remove(i,n):C.removeEvent(i,n,t.handle);i[Q.expando]=void 0}i[J.expando]&&(i[J.expando]=void 0)}}}),C.fn.extend({detach:function(e){return Fe(this,e,!0)},remove:function(e){return Fe(this,e)},text:function(e){return U(this,(function(e){return void 0===e?C.text(this):this.empty().each((function(){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||(this.textContent=e)}))}),null,e,arguments.length)},append:function(){return He(this,arguments,(function(e){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var t=Pe(this,e);t.appendChild(e)}}))},prepend:function(){return He(this,arguments,(function(e){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var t=Pe(this,e);t.insertBefore(e,t.firstChild)}}))},before:function(){return He(this,arguments,(function(e){this.parentNode&&this.parentNode.insertBefore(e,this)}))},after:function(){return He(this,arguments,(function(e){this.parentNode&&this.parentNode.insertBefore(e,this.nextSibling)}))},empty:function(){for(var e,t=0;null!=(e=this[t]);t++)1===e.nodeType&&(C.cleanData(ge(e,!1)),e.textContent="");return this},clone:function(e,t){return e=null!=e&&e,t=null==t?e:t,this.map((function(){return C.clone(this,e,t)}))},html:function(e){return U(this,(function(e){var t=this[0]||{},i=0,n=this.length;if(void 0===e&&1===t.nodeType)return t.innerHTML;if("string"===typeof e&&!De.test(e)&&!fe[(ue.exec(e)||["",""])[1].toLowerCase()]){e=C.htmlPrefilter(e);try{for(;i<n;i++)t=this[i]||{},1===t.nodeType&&(C.cleanData(ge(t,!1)),t.innerHTML=e);t=0}catch(a){}}t&&this.empty().append(e)}),null,e,arguments.length)},replaceWith:function(){var e=[];return He(this,arguments,(function(t){var i=this.parentNode;C.inArray(this,e)<0&&(C.cleanData(ge(this)),i&&i.replaceChild(t,this))}),e)}}),C.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},(function(e,t){C.fn[e]=function(e){for(var i,n=[],a=C(e),o=a.length-1,s=0;s<=o;s++)i=s===o?this:this.clone(!0),C(a[s])[t](i),d.apply(n,i.get());return this.pushStack(n)}}));var Oe=/^margin/,We=new RegExp("^("+ie+")(?!px)[a-z%]+$","i"),Le=function(e){var t=e.ownerDocument.defaultView;return t&&t.opener||(t=i),t.getComputedStyle(e)};function je(e,t,i){var n,a,o,s,r=e.style;return i=i||Le(e),i&&(s=i.getPropertyValue(t)||i[t],""!==s||C.contains(e.ownerDocument,e)||(s=C.style(e,t)),!b.pixelMarginRight()&&We.test(s)&&Oe.test(t)&&(n=r.width,a=r.minWidth,o=r.maxWidth,r.minWidth=r.maxWidth=r.width=s,s=i.width,r.width=n,r.minWidth=a,r.maxWidth=o)),void 0!==s?s+"":s}function Be(e,t){return{get:function(){if(!e())return(this.get=t).apply(this,arguments);delete this.get}}}(function(){function e(){if(l){l.style.cssText="box-sizing:border-box;position:relative;display:block;margin:auto;border:1px;padding:1px;top:1%;width:50%",l.innerHTML="",we.appendChild(s);var e=i.getComputedStyle(l);t="1%"!==e.top,o="2px"===e.marginLeft,n="4px"===e.width,l.style.marginRight="50%",a="4px"===e.marginRight,we.removeChild(s),l=null}}var t,n,a,o,s=r.createElement("div"),l=r.createElement("div");l.style&&(l.style.backgroundClip="content-box",l.cloneNode(!0).style.backgroundClip="",b.clearCloneStyle="content-box"===l.style.backgroundClip,s.style.cssText="border:0;width:8px;height:0;top:0;left:-9999px;padding:0;margin-top:1px;position:absolute",s.appendChild(l),C.extend(b,{pixelPosition:function(){return e(),t},boxSizingReliable:function(){return e(),n},pixelMarginRight:function(){return e(),a},reliableMarginLeft:function(){return e(),o}}))})();var Ge=/^(none|table(?!-c[ea]).+)/,Ve={position:"absolute",visibility:"hidden",display:"block"},qe={letterSpacing:"0",fontWeight:"400"},Ye=["Webkit","Moz","ms"],Ue=r.createElement("div").style;function Ke(e){if(e in Ue)return e;var t=e[0].toUpperCase()+e.slice(1),i=Ye.length;while(i--)if(e=Ye[i]+t,e in Ue)return e}function Xe(e,t,i){var n=ne.exec(t);return n?Math.max(0,n[2]-(i||0))+(n[3]||"px"):t}function Qe(e,t,i,n,a){for(var o=i===(n?"border":"content")?4:"width"===t?1:0,s=0;o<4;o+=2)"margin"===i&&(s+=C.css(e,i+ae[o],!0,a)),n?("content"===i&&(s-=C.css(e,"padding"+ae[o],!0,a)),"margin"!==i&&(s-=C.css(e,"border"+ae[o]+"Width",!0,a))):(s+=C.css(e,"padding"+ae[o],!0,a),"padding"!==i&&(s+=C.css(e,"border"+ae[o]+"Width",!0,a)));return s}function Je(e,t,i){var n,a=!0,o=Le(e),s="border-box"===C.css(e,"boxSizing",!1,o);if(e.getClientRects().length&&(n=e.getBoundingClientRect()[t]),n<=0||null==n){if(n=je(e,t,o),(n<0||null==n)&&(n=e.style[t]),We.test(n))return n;a=s&&(b.boxSizingReliable()||n===e.style[t]),n=parseFloat(n)||0}return n+Qe(e,t,i||(s?"border":"content"),a,o)+"px"}function Ze(e,t,i,n,a){return new Ze.prototype.init(e,t,i,n,a)}C.extend({cssHooks:{opacity:{get:function(e,t){if(t){var i=je(e,"opacity");return""===i?"1":i}}}},cssNumber:{animationIterationCount:!0,columnCount:!0,fillOpacity:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{float:"cssFloat"},style:function(e,t,i,n){if(e&&3!==e.nodeType&&8!==e.nodeType&&e.style){var a,o,s,r=C.camelCase(t),l=e.style;if(t=C.cssProps[r]||(C.cssProps[r]=Ke(r)||r),s=C.cssHooks[t]||C.cssHooks[r],void 0===i)return s&&"get"in s&&void 0!==(a=s.get(e,!1,n))?a:l[t];o=typeof i,"string"===o&&(a=ne.exec(i))&&a[1]&&(i=re(e,t,a),o="number"),null!=i&&i===i&&("number"===o&&(i+=a&&a[3]||(C.cssNumber[r]?"":"px")),b.clearCloneStyle||""!==i||0!==t.indexOf("background")||(l[t]="inherit"),s&&"set"in s&&void 0===(i=s.set(e,i,n))||(l[t]=i))}},css:function(e,t,i,n){var a,o,s,r=C.camelCase(t);return t=C.cssProps[r]||(C.cssProps[r]=Ke(r)||r),s=C.cssHooks[t]||C.cssHooks[r],s&&"get"in s&&(a=s.get(e,!0,i)),void 0===a&&(a=je(e,t,n)),"normal"===a&&t in qe&&(a=qe[t]),""===i||i?(o=parseFloat(a),!0===i||isFinite(o)?o||0:a):a}}),C.each(["height","width"],(function(e,t){C.cssHooks[t]={get:function(e,i,n){if(i)return!Ge.test(C.css(e,"display"))||e.getClientRects().length&&e.getBoundingClientRect().width?Je(e,t,n):se(e,Ve,(function(){return Je(e,t,n)}))},set:function(e,i,n){var a,o=n&&Le(e),s=n&&Qe(e,t,n,"border-box"===C.css(e,"boxSizing",!1,o),o);return s&&(a=ne.exec(i))&&"px"!==(a[3]||"px")&&(e.style[t]=i,i=C.css(e,t)),Xe(e,i,s)}}})),C.cssHooks.marginLeft=Be(b.reliableMarginLeft,(function(e,t){if(t)return(parseFloat(je(e,"marginLeft"))||e.getBoundingClientRect().left-se(e,{marginLeft:0},(function(){return e.getBoundingClientRect().left})))+"px"})),C.each({margin:"",padding:"",border:"Width"},(function(e,t){C.cssHooks[e+t]={expand:function(i){for(var n=0,a={},o="string"===typeof i?i.split(" "):[i];n<4;n++)a[e+ae[n]+t]=o[n]||o[n-2]||o[0];return a}},Oe.test(e)||(C.cssHooks[e+t].set=Xe)})),C.fn.extend({css:function(e,t){return U(this,(function(e,t,i){var n,a,o={},s=0;if(C.isArray(t)){for(n=Le(e),a=t.length;s<a;s++)o[t[s]]=C.css(e,t[s],!1,n);return o}return void 0!==i?C.style(e,t,i):C.css(e,t)}),e,t,arguments.length>1)}}),C.Tween=Ze,Ze.prototype={constructor:Ze,init:function(e,t,i,n,a,o){this.elem=e,this.prop=i,this.easing=a||C.easing._default,this.options=t,this.start=this.now=this.cur(),this.end=n,this.unit=o||(C.cssNumber[i]?"":"px")},cur:function(){var e=Ze.propHooks[this.prop];return e&&e.get?e.get(this):Ze.propHooks._default.get(this)},run:function(e){var t,i=Ze.propHooks[this.prop];return this.options.duration?this.pos=t=C.easing[this.easing](e,this.options.duration*e,0,1,this.options.duration):this.pos=t=e,this.now=(this.end-this.start)*t+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),i&&i.set?i.set(this):Ze.propHooks._default.set(this),this}},Ze.prototype.init.prototype=Ze.prototype,Ze.propHooks={_default:{get:function(e){var t;return 1!==e.elem.nodeType||null!=e.elem[e.prop]&&null==e.elem.style[e.prop]?e.elem[e.prop]:(t=C.css(e.elem,e.prop,""),t&&"auto"!==t?t:0)},set:function(e){C.fx.step[e.prop]?C.fx.step[e.prop](e):1!==e.elem.nodeType||null==e.elem.style[C.cssProps[e.prop]]&&!C.cssHooks[e.prop]?e.elem[e.prop]=e.now:C.style(e.elem,e.prop,e.now+e.unit)}}},Ze.propHooks.scrollTop=Ze.propHooks.scrollLeft={set:function(e){e.elem.nodeType&&e.elem.parentNode&&(e.elem[e.prop]=e.now)}},C.easing={linear:function(e){return e},swing:function(e){return.5-Math.cos(e*Math.PI)/2},_default:"swing"},C.fx=Ze.prototype.init,C.fx.step={};var et,tt,it=/^(?:toggle|show|hide)$/,nt=/queueHooks$/;function at(){tt&&(i.requestAnimationFrame(at),C.fx.tick())}function ot(){return i.setTimeout((function(){et=void 0})),et=C.now()}function st(e,t){var i,n=0,a={height:e};for(t=t?1:0;n<4;n+=2-t)i=ae[n],a["margin"+i]=a["padding"+i]=e;return t&&(a.opacity=a.width=e),a}function rt(e,t,i){for(var n,a=(ht.tweeners[t]||[]).concat(ht.tweeners["*"]),o=0,s=a.length;o<s;o++)if(n=a[o].call(i,t,e))return n}function lt(e,t,i){var n,a,o,s,r,l,c,h,d="width"in t||"height"in t,u=this,p={},f=e.style,g=e.nodeType&&oe(e),m=Q.get(e,"fxshow");for(n in i.queue||(s=C._queueHooks(e,"fx"),null==s.unqueued&&(s.unqueued=0,r=s.empty.fire,s.empty.fire=function(){s.unqueued||r()}),s.unqueued++,u.always((function(){u.always((function(){s.unqueued--,C.queue(e,"fx").length||s.empty.fire()}))}))),t)if(a=t[n],it.test(a)){if(delete t[n],o=o||"toggle"===a,a===(g?"hide":"show")){if("show"!==a||!m||void 0===m[n])continue;g=!0}p[n]=m&&m[n]||C.style(e,n)}if(l=!C.isEmptyObject(t),l||!C.isEmptyObject(p))for(n in d&&1===e.nodeType&&(i.overflow=[f.overflow,f.overflowX,f.overflowY],c=m&&m.display,null==c&&(c=Q.get(e,"display")),h=C.css(e,"display"),"none"===h&&(c?h=c:(he([e],!0),c=e.style.display||c,h=C.css(e,"display"),he([e]))),("inline"===h||"inline-block"===h&&null!=c)&&"none"===C.css(e,"float")&&(l||(u.done((function(){f.display=c})),null==c&&(h=f.display,c="none"===h?"":h)),f.display="inline-block")),i.overflow&&(f.overflow="hidden",u.always((function(){f.overflow=i.overflow[0],f.overflowX=i.overflow[1],f.overflowY=i.overflow[2]}))),l=!1,p)l||(m?"hidden"in m&&(g=m.hidden):m=Q.access(e,"fxshow",{display:c}),o&&(m.hidden=!g),g&&he([e],!0),u.done((function(){for(n in g||he([e]),Q.remove(e,"fxshow"),p)C.style(e,n,p[n])}))),l=rt(g?m[n]:0,n,u),n in m||(m[n]=l.start,g&&(l.end=l.start,l.start=0))}function ct(e,t){var i,n,a,o,s;for(i in e)if(n=C.camelCase(i),a=t[n],o=e[i],C.isArray(o)&&(a=o[1],o=e[i]=o[0]),i!==n&&(e[n]=o,delete e[i]),s=C.cssHooks[n],s&&"expand"in s)for(i in o=s.expand(o),delete e[n],o)i in e||(e[i]=o[i],t[i]=a);else t[n]=a}function ht(e,t,i){var n,a,o=0,s=ht.prefilters.length,r=C.Deferred().always((function(){delete l.elem})),l=function(){if(a)return!1;for(var t=et||ot(),i=Math.max(0,c.startTime+c.duration-t),n=i/c.duration||0,o=1-n,s=0,l=c.tweens.length;s<l;s++)c.tweens[s].run(o);return r.notifyWith(e,[c,o,i]),o<1&&l?i:(r.resolveWith(e,[c]),!1)},c=r.promise({elem:e,props:C.extend({},t),opts:C.extend(!0,{specialEasing:{},easing:C.easing._default},i),originalProperties:t,originalOptions:i,startTime:et||ot(),duration:i.duration,tweens:[],createTween:function(t,i){var n=C.Tween(e,c.opts,t,i,c.opts.specialEasing[t]||c.opts.easing);return c.tweens.push(n),n},stop:function(t){var i=0,n=t?c.tweens.length:0;if(a)return this;for(a=!0;i<n;i++)c.tweens[i].run(1);return t?(r.notifyWith(e,[c,1,0]),r.resolveWith(e,[c,t])):r.rejectWith(e,[c,t]),this}}),h=c.props;for(ct(h,c.opts.specialEasing);o<s;o++)if(n=ht.prefilters[o].call(c,e,h,c.opts),n)return C.isFunction(n.stop)&&(C._queueHooks(c.elem,c.opts.queue).stop=C.proxy(n.stop,n)),n;return C.map(h,rt,c),C.isFunction(c.opts.start)&&c.opts.start.call(e,c),C.fx.timer(C.extend(l,{elem:e,anim:c,queue:c.opts.queue})),c.progress(c.opts.progress).done(c.opts.done,c.opts.complete).fail(c.opts.fail).always(c.opts.always)}C.Animation=C.extend(ht,{tweeners:{"*":[function(e,t){var i=this.createTween(e,t);return re(i.elem,e,ne.exec(t),i),i}]},tweener:function(e,t){C.isFunction(e)?(t=e,e=["*"]):e=e.match(W);for(var i,n=0,a=e.length;n<a;n++)i=e[n],ht.tweeners[i]=ht.tweeners[i]||[],ht.tweeners[i].unshift(t)},prefilters:[lt],prefilter:function(e,t){t?ht.prefilters.unshift(e):ht.prefilters.push(e)}}),C.speed=function(e,t,i){var n=e&&"object"===typeof e?C.extend({},e):{complete:i||!i&&t||C.isFunction(e)&&e,duration:e,easing:i&&t||t&&!C.isFunction(t)&&t};return C.fx.off||r.hidden?n.duration=0:n.duration="number"===typeof n.duration?n.duration:n.duration in C.fx.speeds?C.fx.speeds[n.duration]:C.fx.speeds._default,null!=n.queue&&!0!==n.queue||(n.queue="fx"),n.old=n.complete,n.complete=function(){C.isFunction(n.old)&&n.old.call(this),n.queue&&C.dequeue(this,n.queue)},n},C.fn.extend({fadeTo:function(e,t,i,n){return this.filter(oe).css("opacity",0).show().end().animate({opacity:t},e,i,n)},animate:function(e,t,i,n){var a=C.isEmptyObject(e),o=C.speed(t,i,n),s=function(){var t=ht(this,C.extend({},e),o);(a||Q.get(this,"finish"))&&t.stop(!0)};return s.finish=s,a||!1===o.queue?this.each(s):this.queue(o.queue,s)},stop:function(e,t,i){var n=function(e){var t=e.stop;delete e.stop,t(i)};return"string"!==typeof e&&(i=t,t=e,e=void 0),t&&!1!==e&&this.queue(e||"fx",[]),this.each((function(){var t=!0,a=null!=e&&e+"queueHooks",o=C.timers,s=Q.get(this);if(a)s[a]&&s[a].stop&&n(s[a]);else for(a in s)s[a]&&s[a].stop&&nt.test(a)&&n(s[a]);for(a=o.length;a--;)o[a].elem!==this||null!=e&&o[a].queue!==e||(o[a].anim.stop(i),t=!1,o.splice(a,1));!t&&i||C.dequeue(this,e)}))},finish:function(e){return!1!==e&&(e=e||"fx"),this.each((function(){var t,i=Q.get(this),n=i[e+"queue"],a=i[e+"queueHooks"],o=C.timers,s=n?n.length:0;for(i.finish=!0,C.queue(this,e,[]),a&&a.stop&&a.stop.call(this,!0),t=o.length;t--;)o[t].elem===this&&o[t].queue===e&&(o[t].anim.stop(!0),o.splice(t,1));for(t=0;t<s;t++)n[t]&&n[t].finish&&n[t].finish.call(this);delete i.finish}))}}),C.each(["toggle","show","hide"],(function(e,t){var i=C.fn[t];C.fn[t]=function(e,n,a){return null==e||"boolean"===typeof e?i.apply(this,arguments):this.animate(st(t,!0),e,n,a)}})),C.each({slideDown:st("show"),slideUp:st("hide"),slideToggle:st("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},(function(e,t){C.fn[e]=function(e,i,n){return this.animate(t,e,i,n)}})),C.timers=[],C.fx.tick=function(){var e,t=0,i=C.timers;for(et=C.now();t<i.length;t++)e=i[t],e()||i[t]!==e||i.splice(t--,1);i.length||C.fx.stop(),et=void 0},C.fx.timer=function(e){C.timers.push(e),e()?C.fx.start():C.timers.pop()},C.fx.interval=13,C.fx.start=function(){tt||(tt=i.requestAnimationFrame?i.requestAnimationFrame(at):i.setInterval(C.fx.tick,C.fx.interval))},C.fx.stop=function(){i.cancelAnimationFrame?i.cancelAnimationFrame(tt):i.clearInterval(tt),tt=null},C.fx.speeds={slow:600,fast:200,_default:400},C.fn.delay=function(e,t){return e=C.fx&&C.fx.speeds[e]||e,t=t||"fx",this.queue(t,(function(t,n){var a=i.setTimeout(t,e);n.stop=function(){i.clearTimeout(a)}}))},function(){var e=r.createElement("input"),t=r.createElement("select"),i=t.appendChild(r.createElement("option"));e.type="checkbox",b.checkOn=""!==e.value,b.optSelected=i.selected,e=r.createElement("input"),e.value="t",e.type="radio",b.radioValue="t"===e.value}();var dt,ut=C.expr.attrHandle;C.fn.extend({attr:function(e,t){return U(this,C.attr,e,t,arguments.length>1)},removeAttr:function(e){return this.each((function(){C.removeAttr(this,e)}))}}),C.extend({attr:function(e,t,i){var n,a,o=e.nodeType;if(3!==o&&8!==o&&2!==o)return"undefined"===typeof e.getAttribute?C.prop(e,t,i):(1===o&&C.isXMLDoc(e)||(a=C.attrHooks[t.toLowerCase()]||(C.expr.match.bool.test(t)?dt:void 0)),void 0!==i?null===i?void C.removeAttr(e,t):a&&"set"in a&&void 0!==(n=a.set(e,i,t))?n:(e.setAttribute(t,i+""),i):a&&"get"in a&&null!==(n=a.get(e,t))?n:(n=C.find.attr(e,t),null==n?void 0:n))},attrHooks:{type:{set:function(e,t){if(!b.radioValue&&"radio"===t&&C.nodeName(e,"input")){var i=e.value;return e.setAttribute("type",t),i&&(e.value=i),t}}}},removeAttr:function(e,t){var i,n=0,a=t&&t.match(W);if(a&&1===e.nodeType)while(i=a[n++])e.removeAttribute(i)}}),dt={set:function(e,t,i){return!1===t?C.removeAttr(e,i):e.setAttribute(i,i),i}},C.each(C.expr.match.bool.source.match(/\w+/g),(function(e,t){var i=ut[t]||C.find.attr;ut[t]=function(e,t,n){var a,o,s=t.toLowerCase();return n||(o=ut[s],ut[s]=a,a=null!=i(e,t,n)?s:null,ut[s]=o),a}}));var pt=/^(?:input|select|textarea|button)$/i,ft=/^(?:a|area)$/i;C.fn.extend({prop:function(e,t){return U(this,C.prop,e,t,arguments.length>1)},removeProp:function(e){return this.each((function(){delete this[C.propFix[e]||e]}))}}),C.extend({prop:function(e,t,i){var n,a,o=e.nodeType;if(3!==o&&8!==o&&2!==o)return 1===o&&C.isXMLDoc(e)||(t=C.propFix[t]||t,a=C.propHooks[t]),void 0!==i?a&&"set"in a&&void 0!==(n=a.set(e,i,t))?n:e[t]=i:a&&"get"in a&&null!==(n=a.get(e,t))?n:e[t]},propHooks:{tabIndex:{get:function(e){var t=C.find.attr(e,"tabindex");return t?parseInt(t,10):pt.test(e.nodeName)||ft.test(e.nodeName)&&e.href?0:-1}}},propFix:{for:"htmlFor",class:"className"}}),b.optSelected||(C.propHooks.selected={get:function(e){var t=e.parentNode;return t&&t.parentNode&&t.parentNode.selectedIndex,null},set:function(e){var t=e.parentNode;t&&(t.selectedIndex,t.parentNode&&t.parentNode.selectedIndex)}}),C.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],(function(){C.propFix[this.toLowerCase()]=this}));var gt=/[\t\r\n\f]/g;function mt(e){return e.getAttribute&&e.getAttribute("class")||""}C.fn.extend({addClass:function(e){var t,i,n,a,o,s,r,l=0;if(C.isFunction(e))return this.each((function(t){C(this).addClass(e.call(this,t,mt(this)))}));if("string"===typeof e&&e){t=e.match(W)||[];while(i=this[l++])if(a=mt(i),n=1===i.nodeType&&(" "+a+" ").replace(gt," "),n){s=0;while(o=t[s++])n.indexOf(" "+o+" ")<0&&(n+=o+" ");r=C.trim(n),a!==r&&i.setAttribute("class",r)}}return this},removeClass:function(e){var t,i,n,a,o,s,r,l=0;if(C.isFunction(e))return this.each((function(t){C(this).removeClass(e.call(this,t,mt(this)))}));if(!arguments.length)return this.attr("class","");if("string"===typeof e&&e){t=e.match(W)||[];while(i=this[l++])if(a=mt(i),n=1===i.nodeType&&(" "+a+" ").replace(gt," "),n){s=0;while(o=t[s++])while(n.indexOf(" "+o+" ")>-1)n=n.replace(" "+o+" "," ");r=C.trim(n),a!==r&&i.setAttribute("class",r)}}return this},toggleClass:function(e,t){var i=typeof e;return"boolean"===typeof t&&"string"===i?t?this.addClass(e):this.removeClass(e):C.isFunction(e)?this.each((function(i){C(this).toggleClass(e.call(this,i,mt(this),t),t)})):this.each((function(){var t,n,a,o;if("string"===i){n=0,a=C(this),o=e.match(W)||[];while(t=o[n++])a.hasClass(t)?a.removeClass(t):a.addClass(t)}else void 0!==e&&"boolean"!==i||(t=mt(this),t&&Q.set(this,"__className__",t),this.setAttribute&&this.setAttribute("class",t||!1===e?"":Q.get(this,"__className__")||""))}))},hasClass:function(e){var t,i,n=0;t=" "+e+" ";while(i=this[n++])if(1===i.nodeType&&(" "+mt(i)+" ").replace(gt," ").indexOf(t)>-1)return!0;return!1}});var vt=/\r/g,bt=/[\x20\t\r\n\f]+/g;C.fn.extend({val:function(e){var t,i,n,a=this[0];return arguments.length?(n=C.isFunction(e),this.each((function(i){var a;1===this.nodeType&&(a=n?e.call(this,i,C(this).val()):e,null==a?a="":"number"===typeof a?a+="":C.isArray(a)&&(a=C.map(a,(function(e){return null==e?"":e+""}))),t=C.valHooks[this.type]||C.valHooks[this.nodeName.toLowerCase()],t&&"set"in t&&void 0!==t.set(this,a,"value")||(this.value=a))}))):a?(t=C.valHooks[a.type]||C.valHooks[a.nodeName.toLowerCase()],t&&"get"in t&&void 0!==(i=t.get(a,"value"))?i:(i=a.value,"string"===typeof i?i.replace(vt,""):null==i?"":i)):void 0}}),C.extend({valHooks:{option:{get:function(e){var t=C.find.attr(e,"value");return null!=t?t:C.trim(C.text(e)).replace(bt," ")}},select:{get:function(e){for(var t,i,n=e.options,a=e.selectedIndex,o="select-one"===e.type,s=o?null:[],r=o?a+1:n.length,l=a<0?r:o?a:0;l<r;l++)if(i=n[l],(i.selected||l===a)&&!i.disabled&&(!i.parentNode.disabled||!C.nodeName(i.parentNode,"optgroup"))){if(t=C(i).val(),o)return t;s.push(t)}return s},set:function(e,t){var i,n,a=e.options,o=C.makeArray(t),s=a.length;while(s--)n=a[s],(n.selected=C.inArray(C.valHooks.option.get(n),o)>-1)&&(i=!0);return i||(e.selectedIndex=-1),o}}}}),C.each(["radio","checkbox"],(function(){C.valHooks[this]={set:function(e,t){if(C.isArray(t))return e.checked=C.inArray(C(e).val(),t)>-1}},b.checkOn||(C.valHooks[this].get=function(e){return null===e.getAttribute("value")?"on":e.value})}));var wt=/^(?:focusinfocus|focusoutblur)$/;C.extend(C.event,{trigger:function(e,t,n,a){var o,s,l,c,h,d,u,p=[n||r],f=g.call(e,"type")?e.type:e,m=g.call(e,"namespace")?e.namespace.split("."):[];if(s=l=n=n||r,3!==n.nodeType&&8!==n.nodeType&&!wt.test(f+C.event.triggered)&&(f.indexOf(".")>-1&&(m=f.split("."),f=m.shift(),m.sort()),h=f.indexOf(":")<0&&"on"+f,e=e[C.expando]?e:new C.Event(f,"object"===typeof e&&e),e.isTrigger=a?2:3,e.namespace=m.join("."),e.rnamespace=e.namespace?new RegExp("(^|\\.)"+m.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,e.result=void 0,e.target||(e.target=n),t=null==t?[e]:C.makeArray(t,[e]),u=C.event.special[f]||{},a||!u.trigger||!1!==u.trigger.apply(n,t))){if(!a&&!u.noBubble&&!C.isWindow(n)){for(c=u.delegateType||f,wt.test(c+f)||(s=s.parentNode);s;s=s.parentNode)p.push(s),l=s;l===(n.ownerDocument||r)&&p.push(l.defaultView||l.parentWindow||i)}o=0;while((s=p[o++])&&!e.isPropagationStopped())e.type=o>1?c:u.bindType||f,d=(Q.get(s,"events")||{})[e.type]&&Q.get(s,"handle"),d&&d.apply(s,t),d=h&&s[h],d&&d.apply&&K(s)&&(e.result=d.apply(s,t),!1===e.result&&e.preventDefault());return e.type=f,a||e.isDefaultPrevented()||u._default&&!1!==u._default.apply(p.pop(),t)||!K(n)||h&&C.isFunction(n[f])&&!C.isWindow(n)&&(l=n[h],l&&(n[h]=null),C.event.triggered=f,n[f](),C.event.triggered=void 0,l&&(n[h]=l)),e.result}},simulate:function(e,t,i){var n=C.extend(new C.Event,i,{type:e,isSimulated:!0});C.event.trigger(n,null,t)}}),C.fn.extend({trigger:function(e,t){return this.each((function(){C.event.trigger(e,t,this)}))},triggerHandler:function(e,t){var i=this[0];if(i)return C.event.trigger(e,t,i,!0)}}),C.each("blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu".split(" "),(function(e,t){C.fn[t]=function(e,i){return arguments.length>0?this.on(t,null,e,i):this.trigger(t)}})),C.fn.extend({hover:function(e,t){return this.mouseenter(e).mouseleave(t||e)}}),b.focusin="onfocusin"in i,b.focusin||C.each({focus:"focusin",blur:"focusout"},(function(e,t){var i=function(e){C.event.simulate(t,e.target,C.event.fix(e))};C.event.special[t]={setup:function(){var n=this.ownerDocument||this,a=Q.access(n,t);a||n.addEventListener(e,i,!0),Q.access(n,t,(a||0)+1)},teardown:function(){var n=this.ownerDocument||this,a=Q.access(n,t)-1;a?Q.access(n,t,a):(n.removeEventListener(e,i,!0),Q.remove(n,t))}}}));var yt=i.location,Ct=C.now(),_t=/\?/;C.parseXML=function(e){var t;if(!e||"string"!==typeof e)return null;try{t=(new i.DOMParser).parseFromString(e,"text/xml")}catch(n){t=void 0}return t&&!t.getElementsByTagName("parsererror").length||C.error("Invalid XML: "+e),t};var xt=/\[\]$/,kt=/\r?\n/g,St=/^(?:submit|button|image|reset|file)$/i,Tt=/^(?:input|select|textarea|keygen)/i;function $t(e,t,i,n){var a;if(C.isArray(t))C.each(t,(function(t,a){i||xt.test(e)?n(e,a):$t(e+"["+("object"===typeof a&&null!=a?t:"")+"]",a,i,n)}));else if(i||"object"!==C.type(t))n(e,t);else for(a in t)$t(e+"["+a+"]",t[a],i,n)}C.param=function(e,t){var i,n=[],a=function(e,t){var i=C.isFunction(t)?t():t;n[n.length]=encodeURIComponent(e)+"="+encodeURIComponent(null==i?"":i)};if(C.isArray(e)||e.jquery&&!C.isPlainObject(e))C.each(e,(function(){a(this.name,this.value)}));else for(i in e)$t(i,e[i],t,a);return n.join("&")},C.fn.extend({serialize:function(){return C.param(this.serializeArray())},serializeArray:function(){return this.map((function(){var e=C.prop(this,"elements");return e?C.makeArray(e):this})).filter((function(){var e=this.type;return this.name&&!C(this).is(":disabled")&&Tt.test(this.nodeName)&&!St.test(e)&&(this.checked||!de.test(e))})).map((function(e,t){var i=C(this).val();return null==i?null:C.isArray(i)?C.map(i,(function(e){return{name:t.name,value:e.replace(kt,"\r\n")}})):{name:t.name,value:i.replace(kt,"\r\n")}})).get()}});var Dt=/%20/g,Nt=/#.*$/,Rt=/([?&])_=[^&]*/,It=/^(.*?):[ \t]*([^\r\n]*)$/gm,Pt=/^(?:about|app|app-storage|.+-extension|file|res|widget):$/,At=/^(?:GET|HEAD)$/,Et=/^\/\//,zt={},Mt={},Ht="*/".concat("*"),Ft=r.createElement("a");function Ot(e){return function(t,i){"string"!==typeof t&&(i=t,t="*");var n,a=0,o=t.toLowerCase().match(W)||[];if(C.isFunction(i))while(n=o[a++])"+"===n[0]?(n=n.slice(1)||"*",(e[n]=e[n]||[]).unshift(i)):(e[n]=e[n]||[]).push(i)}}function Wt(e,t,i,n){var a={},o=e===Mt;function s(r){var l;return a[r]=!0,C.each(e[r]||[],(function(e,r){var c=r(t,i,n);return"string"!==typeof c||o||a[c]?o?!(l=c):void 0:(t.dataTypes.unshift(c),s(c),!1)})),l}return s(t.dataTypes[0])||!a["*"]&&s("*")}function Lt(e,t){var i,n,a=C.ajaxSettings.flatOptions||{};for(i in t)void 0!==t[i]&&((a[i]?e:n||(n={}))[i]=t[i]);return n&&C.extend(!0,e,n),e}function jt(e,t,i){var n,a,o,s,r=e.contents,l=e.dataTypes;while("*"===l[0])l.shift(),void 0===n&&(n=e.mimeType||t.getResponseHeader("Content-Type"));if(n)for(a in r)if(r[a]&&r[a].test(n)){l.unshift(a);break}if(l[0]in i)o=l[0];else{for(a in i){if(!l[0]||e.converters[a+" "+l[0]]){o=a;break}s||(s=a)}o=o||s}if(o)return o!==l[0]&&l.unshift(o),i[o]}function Bt(e,t,i,n){var a,o,s,r,l,c={},h=e.dataTypes.slice();if(h[1])for(s in e.converters)c[s.toLowerCase()]=e.converters[s];o=h.shift();while(o)if(e.responseFields[o]&&(i[e.responseFields[o]]=t),!l&&n&&e.dataFilter&&(t=e.dataFilter(t,e.dataType)),l=o,o=h.shift(),o)if("*"===o)o=l;else if("*"!==l&&l!==o){if(s=c[l+" "+o]||c["* "+o],!s)for(a in c)if(r=a.split(" "),r[1]===o&&(s=c[l+" "+r[0]]||c["* "+r[0]],s)){!0===s?s=c[a]:!0!==c[a]&&(o=r[0],h.unshift(r[1]));break}if(!0!==s)if(s&&e.throws)t=s(t);else try{t=s(t)}catch(d){return{state:"parsererror",error:s?d:"No conversion from "+l+" to "+o}}}return{state:"success",data:t}}Ft.href=yt.href,C.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:yt.href,type:"GET",isLocal:Pt.test(yt.protocol),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":Ht,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/\bxml\b/,html:/\bhtml/,json:/\bjson\b/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":JSON.parse,"text xml":C.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(e,t){return t?Lt(Lt(e,C.ajaxSettings),t):Lt(C.ajaxSettings,e)},ajaxPrefilter:Ot(zt),ajaxTransport:Ot(Mt),ajax:function(e,t){"object"===typeof e&&(t=e,e=void 0),t=t||{};var n,a,o,s,l,c,h,d,u,p,f=C.ajaxSetup({},t),g=f.context||f,m=f.context&&(g.nodeType||g.jquery)?C(g):C.event,v=C.Deferred(),b=C.Callbacks("once memory"),w=f.statusCode||{},y={},_={},x="canceled",k={readyState:0,getResponseHeader:function(e){var t;if(h){if(!s){s={};while(t=It.exec(o))s[t[1].toLowerCase()]=t[2]}t=s[e.toLowerCase()]}return null==t?null:t},getAllResponseHeaders:function(){return h?o:null},setRequestHeader:function(e,t){return null==h&&(e=_[e.toLowerCase()]=_[e.toLowerCase()]||e,y[e]=t),this},overrideMimeType:function(e){return null==h&&(f.mimeType=e),this},statusCode:function(e){var t;if(e)if(h)k.always(e[k.status]);else for(t in e)w[t]=[w[t],e[t]];return this},abort:function(e){var t=e||x;return n&&n.abort(t),S(0,t),this}};if(v.promise(k),f.url=((e||f.url||yt.href)+"").replace(Et,yt.protocol+"//"),f.type=t.method||t.type||f.method||f.type,f.dataTypes=(f.dataType||"*").toLowerCase().match(W)||[""],null==f.crossDomain){c=r.createElement("a");try{c.href=f.url,c.href=c.href,f.crossDomain=Ft.protocol+"//"+Ft.host!==c.protocol+"//"+c.host}catch(T){f.crossDomain=!0}}if(f.data&&f.processData&&"string"!==typeof f.data&&(f.data=C.param(f.data,f.traditional)),Wt(zt,f,t,k),h)return k;for(u in d=C.event&&f.global,d&&0===C.active++&&C.event.trigger("ajaxStart"),f.type=f.type.toUpperCase(),f.hasContent=!At.test(f.type),a=f.url.replace(Nt,""),f.hasContent?f.data&&f.processData&&0===(f.contentType||"").indexOf("application/x-www-form-urlencoded")&&(f.data=f.data.replace(Dt,"+")):(p=f.url.slice(a.length),f.data&&(a+=(_t.test(a)?"&":"?")+f.data,delete f.data),!1===f.cache&&(a=a.replace(Rt,""),p=(_t.test(a)?"&":"?")+"_="+Ct+++p),f.url=a+p),f.ifModified&&(C.lastModified[a]&&k.setRequestHeader("If-Modified-Since",C.lastModified[a]),C.etag[a]&&k.setRequestHeader("If-None-Match",C.etag[a])),(f.data&&f.hasContent&&!1!==f.contentType||t.contentType)&&k.setRequestHeader("Content-Type",f.contentType),k.setRequestHeader("Accept",f.dataTypes[0]&&f.accepts[f.dataTypes[0]]?f.accepts[f.dataTypes[0]]+("*"!==f.dataTypes[0]?", "+Ht+"; q=0.01":""):f.accepts["*"]),f.headers)k.setRequestHeader(u,f.headers[u]);if(f.beforeSend&&(!1===f.beforeSend.call(g,k,f)||h))return k.abort();if(x="abort",b.add(f.complete),k.done(f.success),k.fail(f.error),n=Wt(Mt,f,t,k),n){if(k.readyState=1,d&&m.trigger("ajaxSend",[k,f]),h)return k;f.async&&f.timeout>0&&(l=i.setTimeout((function(){k.abort("timeout")}),f.timeout));try{h=!1,n.send(y,S)}catch(T){if(h)throw T;S(-1,T)}}else S(-1,"No Transport");function S(e,t,s,r){var c,u,p,y,_,x=t;h||(h=!0,l&&i.clearTimeout(l),n=void 0,o=r||"",k.readyState=e>0?4:0,c=e>=200&&e<300||304===e,s&&(y=jt(f,k,s)),y=Bt(f,y,k,c),c?(f.ifModified&&(_=k.getResponseHeader("Last-Modified"),_&&(C.lastModified[a]=_),_=k.getResponseHeader("etag"),_&&(C.etag[a]=_)),204===e||"HEAD"===f.type?x="nocontent":304===e?x="notmodified":(x=y.state,u=y.data,p=y.error,c=!p)):(p=x,!e&&x||(x="error",e<0&&(e=0))),k.status=e,k.statusText=(t||x)+"",c?v.resolveWith(g,[u,x,k]):v.rejectWith(g,[k,x,p]),k.statusCode(w),w=void 0,d&&m.trigger(c?"ajaxSuccess":"ajaxError",[k,f,c?u:p]),b.fireWith(g,[k,x]),d&&(m.trigger("ajaxComplete",[k,f]),--C.active||C.event.trigger("ajaxStop")))}return k},getJSON:function(e,t,i){return C.get(e,t,i,"json")},getScript:function(e,t){return C.get(e,void 0,t,"script")}}),C.each(["get","post"],(function(e,t){C[t]=function(e,i,n,a){return C.isFunction(i)&&(a=a||n,n=i,i=void 0),C.ajax(C.extend({url:e,type:t,dataType:a,data:i,success:n},C.isPlainObject(e)&&e))}})),C._evalUrl=function(e){return C.ajax({url:e,type:"GET",dataType:"script",cache:!0,async:!1,global:!1,throws:!0})},C.fn.extend({wrapAll:function(e){var t;return this[0]&&(C.isFunction(e)&&(e=e.call(this[0])),t=C(e,this[0].ownerDocument).eq(0).clone(!0),this[0].parentNode&&t.insertBefore(this[0]),t.map((function(){var e=this;while(e.firstElementChild)e=e.firstElementChild;return e})).append(this)),this},wrapInner:function(e){return C.isFunction(e)?this.each((function(t){C(this).wrapInner(e.call(this,t))})):this.each((function(){var t=C(this),i=t.contents();i.length?i.wrapAll(e):t.append(e)}))},wrap:function(e){var t=C.isFunction(e);return this.each((function(i){C(this).wrapAll(t?e.call(this,i):e)}))},unwrap:function(e){return this.parent(e).not("body").each((function(){C(this).replaceWith(this.childNodes)})),this}}),C.expr.pseudos.hidden=function(e){return!C.expr.pseudos.visible(e)},C.expr.pseudos.visible=function(e){return!!(e.offsetWidth||e.offsetHeight||e.getClientRects().length)},C.ajaxSettings.xhr=function(){try{return new i.XMLHttpRequest}catch(e){}};var Gt={0:200,1223:204},Vt=C.ajaxSettings.xhr();b.cors=!!Vt&&"withCredentials"in Vt,b.ajax=Vt=!!Vt,C.ajaxTransport((function(e){var t,n;if(b.cors||Vt&&!e.crossDomain)return{send:function(a,o){var s,r=e.xhr();if(r.open(e.type,e.url,e.async,e.username,e.password),e.xhrFields)for(s in e.xhrFields)r[s]=e.xhrFields[s];for(s in e.mimeType&&r.overrideMimeType&&r.overrideMimeType(e.mimeType),e.crossDomain||a["X-Requested-With"]||(a["X-Requested-With"]="XMLHttpRequest"),a)r.setRequestHeader(s,a[s]);t=function(e){return function(){t&&(t=n=r.onload=r.onerror=r.onabort=r.onreadystatechange=null,"abort"===e?r.abort():"error"===e?"number"!==typeof r.status?o(0,"error"):o(r.status,r.statusText):o(Gt[r.status]||r.status,r.statusText,"text"!==(r.responseType||"text")||"string"!==typeof r.responseText?{binary:r.response}:{text:r.responseText},r.getAllResponseHeaders()))}},r.onload=t(),n=r.onerror=t("error"),void 0!==r.onabort?r.onabort=n:r.onreadystatechange=function(){4===r.readyState&&i.setTimeout((function(){t&&n()}))},t=t("abort");try{r.send(e.hasContent&&e.data||null)}catch(l){if(t)throw l}},abort:function(){t&&t()}}})),C.ajaxPrefilter((function(e){e.crossDomain&&(e.contents.script=!1)})),C.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/\b(?:java|ecma)script\b/},converters:{"text script":function(e){return C.globalEval(e),e}}}),C.ajaxPrefilter("script",(function(e){void 0===e.cache&&(e.cache=!1),e.crossDomain&&(e.type="GET")})),C.ajaxTransport("script",(function(e){var t,i;if(e.crossDomain)return{send:function(n,a){t=C("<script>").prop({charset:e.scriptCharset,src:e.url}).on("load error",i=function(e){t.remove(),i=null,e&&a("error"===e.type?404:200,e.type)}),r.head.appendChild(t[0])},abort:function(){i&&i()}}}));var qt=[],Yt=/(=)\?(?=&|$)|\?\?/;function Ut(e){return C.isWindow(e)?e:9===e.nodeType&&e.defaultView}C.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var e=qt.pop()||C.expando+"_"+Ct++;return this[e]=!0,e}}),C.ajaxPrefilter("json jsonp",(function(e,t,n){var a,o,s,r=!1!==e.jsonp&&(Yt.test(e.url)?"url":"string"===typeof e.data&&0===(e.contentType||"").indexOf("application/x-www-form-urlencoded")&&Yt.test(e.data)&&"data");if(r||"jsonp"===e.dataTypes[0])return a=e.jsonpCallback=C.isFunction(e.jsonpCallback)?e.jsonpCallback():e.jsonpCallback,r?e[r]=e[r].replace(Yt,"$1"+a):!1!==e.jsonp&&(e.url+=(_t.test(e.url)?"&":"?")+e.jsonp+"="+a),e.converters["script json"]=function(){return s||C.error(a+" was not called"),s[0]},e.dataTypes[0]="json",o=i[a],i[a]=function(){s=arguments},n.always((function(){void 0===o?C(i).removeProp(a):i[a]=o,e[a]&&(e.jsonpCallback=t.jsonpCallback,qt.push(a)),s&&C.isFunction(o)&&o(s[0]),s=o=void 0})),"script"})),b.createHTMLDocument=function(){var e=r.implementation.createHTMLDocument("").body;return e.innerHTML="<form></form><form></form>",2===e.childNodes.length}(),C.parseHTML=function(e,t,i){return"string"!==typeof e?[]:("boolean"===typeof t&&(i=t,t=!1),t||(b.createHTMLDocument?(t=r.implementation.createHTMLDocument(""),n=t.createElement("base"),n.href=r.location.href,t.head.appendChild(n)):t=r),a=I.exec(e),o=!i&&[],a?[t.createElement(a[1])]:(a=be([e],t,o),o&&o.length&&C(o).remove(),C.merge([],a.childNodes)));var n,a,o},C.fn.load=function(e,t,i){var n,a,o,s=this,r=e.indexOf(" ");return r>-1&&(n=C.trim(e.slice(r)),e=e.slice(0,r)),C.isFunction(t)?(i=t,t=void 0):t&&"object"===typeof t&&(a="POST"),s.length>0&&C.ajax({url:e,type:a||"GET",dataType:"html",data:t}).done((function(e){o=arguments,s.html(n?C("<div>").append(C.parseHTML(e)).find(n):e)})).always(i&&function(e,t){s.each((function(){i.apply(this,o||[e.responseText,t,e])}))}),this},C.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],(function(e,t){C.fn[t]=function(e){return this.on(t,e)}})),C.expr.pseudos.animated=function(e){return C.grep(C.timers,(function(t){return e===t.elem})).length},C.offset={setOffset:function(e,t,i){var n,a,o,s,r,l,c,h=C.css(e,"position"),d=C(e),u={};"static"===h&&(e.style.position="relative"),r=d.offset(),o=C.css(e,"top"),l=C.css(e,"left"),c=("absolute"===h||"fixed"===h)&&(o+l).indexOf("auto")>-1,c?(n=d.position(),s=n.top,a=n.left):(s=parseFloat(o)||0,a=parseFloat(l)||0),C.isFunction(t)&&(t=t.call(e,i,C.extend({},r))),null!=t.top&&(u.top=t.top-r.top+s),null!=t.left&&(u.left=t.left-r.left+a),"using"in t?t.using.call(e,u):d.css(u)}},C.fn.extend({offset:function(e){if(arguments.length)return void 0===e?this:this.each((function(t){C.offset.setOffset(this,e,t)}));var t,i,n,a,o=this[0];return o?o.getClientRects().length?(n=o.getBoundingClientRect(),n.width||n.height?(a=o.ownerDocument,i=Ut(a),t=a.documentElement,{top:n.top+i.pageYOffset-t.clientTop,left:n.left+i.pageXOffset-t.clientLeft}):n):{top:0,left:0}:void 0},position:function(){if(this[0]){var e,t,i=this[0],n={top:0,left:0};return"fixed"===C.css(i,"position")?t=i.getBoundingClientRect():(e=this.offsetParent(),t=this.offset(),C.nodeName(e[0],"html")||(n=e.offset()),n={top:n.top+C.css(e[0],"borderTopWidth",!0),left:n.left+C.css(e[0],"borderLeftWidth",!0)}),{top:t.top-n.top-C.css(i,"marginTop",!0),left:t.left-n.left-C.css(i,"marginLeft",!0)}}},offsetParent:function(){return this.map((function(){var e=this.offsetParent;while(e&&"static"===C.css(e,"position"))e=e.offsetParent;return e||we}))}}),C.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},(function(e,t){var i="pageYOffset"===t;C.fn[e]=function(n){return U(this,(function(e,n,a){var o=Ut(e);if(void 0===a)return o?o[t]:e[n];o?o.scrollTo(i?o.pageXOffset:a,i?a:o.pageYOffset):e[n]=a}),e,n,arguments.length)}})),C.each(["top","left"],(function(e,t){C.cssHooks[t]=Be(b.pixelPosition,(function(e,i){if(i)return i=je(e,t),We.test(i)?C(e).position()[t]+"px":i}))})),C.each({Height:"height",Width:"width"},(function(e,t){C.each({padding:"inner"+e,content:t,"":"outer"+e},(function(i,n){C.fn[n]=function(a,o){var s=arguments.length&&(i||"boolean"!==typeof a),r=i||(!0===a||!0===o?"margin":"border");return U(this,(function(t,i,a){var o;return C.isWindow(t)?0===n.indexOf("outer")?t["inner"+e]:t.document.documentElement["client"+e]:9===t.nodeType?(o=t.documentElement,Math.max(t.body["scroll"+e],o["scroll"+e],t.body["offset"+e],o["offset"+e],o["client"+e])):void 0===a?C.css(t,i,r):C.style(t,i,a,r)}),t,s?a:void 0,s)}}))})),C.fn.extend({bind:function(e,t,i){return this.on(e,null,t,i)},unbind:function(e,t){return this.off(e,null,t)},delegate:function(e,t,i,n){return this.on(t,e,i,n)},undelegate:function(e,t,i){return 1===arguments.length?this.off(e,"**"):this.off(t,e||"**",i)}}),C.parseJSON=JSON.parse,n=[],a=function(){return C}.apply(t,n),void 0===a||(e.exports=a);var Kt=i.jQuery,Xt=i.$;return C.noConflict=function(e){return i.$===C&&(i.$=Xt),e&&i.jQuery===C&&(i.jQuery=Kt),C},o||(i.jQuery=i.$=C),C}))},ddb6:function(e,t,i){e.exports=i.p+"static/img/graph-step1-en.f1f6a124.svg"},e057:function(e,t,i){var n={"./0/operator-node.svg":"d18b","./1/operator-node.svg":"97cd"};function a(e){var t=o(e);return i(t)}function o(e){if(!i.o(n,e)){var t=new Error("Cannot find module '"+e+"'");throw t.code="MODULE_NOT_FOUND",t}return n[e]}a.keys=function(){return Object.keys(n)},a.resolve=o,e.exports=a,a.id="e057"},e08c:function(e,t){e.exports="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABAAAAAQCAYAAAAf8/9hAAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAAyZpVFh0WE1MOmNvbS5hZG9iZS54bXAAAAAAADw/eHBhY2tldCBiZWdpbj0i77u/IiBpZD0iVzVNME1wQ2VoaUh6cmVTek5UY3prYzlkIj8+IDx4OnhtcG1ldGEgeG1sbnM6eD0iYWRvYmU6bnM6bWV0YS8iIHg6eG1wdGs9IkFkb2JlIFhNUCBDb3JlIDUuNi1jMTM4IDc5LjE1OTgyNCwgMjAxNi8wOS8xNC0wMTowOTowMSAgICAgICAgIj4gPHJkZjpSREYgeG1sbnM6cmRmPSJodHRwOi8vd3d3LnczLm9yZy8xOTk5LzAyLzIyLXJkZi1zeW50YXgtbnMjIj4gPHJkZjpEZXNjcmlwdGlvbiByZGY6YWJvdXQ9IiIgeG1sbnM6eG1wPSJodHRwOi8vbnMuYWRvYmUuY29tL3hhcC8xLjAvIiB4bWxuczp4bXBNTT0iaHR0cDovL25zLmFkb2JlLmNvbS94YXAvMS4wL21tLyIgeG1sbnM6c3RSZWY9Imh0dHA6Ly9ucy5hZG9iZS5jb20veGFwLzEuMC9zVHlwZS9SZXNvdXJjZVJlZiMiIHhtcDpDcmVhdG9yVG9vbD0iQWRvYmUgUGhvdG9zaG9wIENDIDIwMTcgKFdpbmRvd3MpIiB4bXBNTTpJbnN0YW5jZUlEPSJ4bXAuaWlkOkM2RTk2MTdDQUQyMzExRTk5NTMxREEwM0Y1RTkyRTgzIiB4bXBNTTpEb2N1bWVudElEPSJ4bXAuZGlkOkM2RTk2MTdEQUQyMzExRTk5NTMxREEwM0Y1RTkyRTgzIj4gPHhtcE1NOkRlcml2ZWRGcm9tIHN0UmVmOmluc3RhbmNlSUQ9InhtcC5paWQ6QzZFOTYxN0FBRDIzMTFFOTk1MzFEQTAzRjVFOTJFODMiIHN0UmVmOmRvY3VtZW50SUQ9InhtcC5kaWQ6QzZFOTYxN0JBRDIzMTFFOTk1MzFEQTAzRjVFOTJFODMiLz4gPC9yZGY6RGVzY3JpcHRpb24+IDwvcmRmOlJERj4gPC94OnhtcG1ldGE+IDw/eHBhY2tldCBlbmQ9InIiPz4nS1hsAAAAxElEQVR42mL8//8/AyWAceANmDlzJkUGMOGRSwTiN0CcTo4BBUA8F4iFgXg6EBeRYkAdEPcD8R0gNgfia0DcC8S12AxgQQ4PIO4G4mIgvgzEbkD8AojtgXgXEDcBMTcQV2BzATMQz4BqPgXEDlDNIPAWiJ2A+BgQlwPxFKhlKAaA/JsGxIeA2AWI36G59CMQuwPxPiDOhoYLigGmQLwRiD2B+DOO8PoCxD5AvB6IddDDQJvIaP8OxEHEpoOhkhcoNQAgwACvNDJTYJfE+QAAAABJRU5ErkJggg=="},e439:function(e,t,i){var n=i("23e7"),a=i("d039"),o=i("fc6a"),s=i("06cf").f,r=i("83ab"),l=a((function(){s(1)})),c=!r||l;n({target:"Object",stat:!0,forced:c,sham:!r},{getOwnPropertyDescriptor:function(e,t){return s(o(e),t)}})},e5a4:function(e,t,i){e.exports=i.p+"static/img/repeat-dataset.129b2a7f.svg"},e6da:function(e,t,i){e.exports=i.p+"static/img/explain-reality.75b3aae4.svg"},e7e6:function(e,t,i){var n={"./0/constant-node.svg":"54ae","./1/constant-node.svg":"ec63"};function a(e){var t=o(e);return i(t)}function o(e){if(!i.o(n,e)){var t=new Error("Cannot find module '"+e+"'");throw t.code="MODULE_NOT_FOUND",t}return n[e]}a.keys=function(){return Object.keys(n)},a.resolve=o,e.exports=a,a.id="e7e6"},ea2e:function(e,t,i){"use strict";i("c1c9")},ea825:function(e,t,i){e.exports=i.p+"static/img/deb-slot.14270dce.png"},eb76:function(e,t,i){e.exports=i.p+"static/img/graph-stepimg2.82c61922.svg"},eb90:function(e,t,i){e.exports=i.p+"static/img/graph-step2-en.ff55d634.svg"},eba6:function(e,t,i){e.exports=i.p+"static/img/batch-img.01b05453.svg"},ec63:function(e,t,i){e.exports=i.p+"static/img/constant-node.6511f557.svg"},ecab:function(e,t,i){e.exports=i.p+"static/img/guideIcon.d4000211.svg"},ed3a:function(e,t){(function(e){e.fn.drag=function(t,i,n){var a="string"==typeof t?t:"",o=e.isFunction(t)?t:e.isFunction(i)?i:null;return 0!==a.indexOf("drag")&&(a="drag"+a),n=(t==o?i:n)||{},o?this.on(a,n,o):this.trigger(a)};var t=e.event,i=t.special,n=i.drag={defaults:{which:1,distance:0,not:":input",handle:null,relative:!1,drop:!0,click:!1},datakey:"dragdata",noBubble:!0,add:function(t){var i=e.data(this,n.datakey),a=t.data||{};i.related+=1,e.each(n.defaults,(function(e,t){void 0!==a[e]&&(i[e]=a[e])}))},remove:function(){e.data(this,n.datakey).related-=1},setup:function(){if(!e.data(this,n.datakey)){var i=e.extend({related:0},n.defaults);e.data(this,n.datakey,i),t.add(this,"touchstart mousedown",n.init,i),this.attachEvent&&this.attachEvent("ondragstart",n.dontstart)}},teardown:function(){var i=e.data(this,n.datakey)||{};i.related||(e.removeData(this,n.datakey),t.remove(this,"touchstart mousedown",n.init),n.textselect(!0),this.detachEvent&&this.detachEvent("ondragstart",n.dontstart))},init:function(a){if(!n.touched){var o,s=a.data;if(!(0!=a.which&&s.which>0&&a.which!=s.which)){var r=e(a.target).attr("class")||"";if(!e(a.target).is(s.not)&&r&&-1!==r.toString().indexOf("slick")&&(!s.handle||e(a.target).closest(s.handle,a.currentTarget).length)&&(n.touched="touchstart"==a.type?this:null,s.propagates=1,s.mousedown=this,s.interactions=[n.interaction(this,s)],s.target=a.target,s.pageX=a.pageX,s.pageY=a.pageY,s.dragging=null,o=n.hijack(a,"draginit",s),s.propagates))return o=n.flatten(o),o&&o.length&&(s.interactions=[],e.each(o,(function(){s.interactions.push(n.interaction(this,s))}))),s.propagates=s.interactions.length,!1!==s.drop&&i.drop&&i.drop.handler(a,s),n.textselect(!1),n.touched?t.add(n.touched,"touchmove touchend",n.handler,s):t.add(document,"mousemove mouseup",n.handler,s),!(!n.touched||s.live)&&void 0}}},interaction:function(t,i){var a=t&&t.ownerDocument&&e(t)[i.relative?"position":"offset"]()||{top:0,left:0};return{drag:t,callback:new n.callback,droppable:[],offset:a}},handler:function(a){var o=a.data;switch(a.type){case!o.dragging&&"touchmove":a.preventDefault();case!o.dragging&&"mousemove":if(Math.pow(a.pageX-o.pageX,2)+Math.pow(a.pageY-o.pageY,2)<Math.pow(o.distance,2))break;a.target=o.target,n.hijack(a,"dragstart",o),o.propagates&&(o.dragging=!0);case"touchmove":a.preventDefault();case"mousemove":if(o.dragging){if(n.hijack(a,"drag",o),o.propagates){!1!==o.drop&&i.drop&&i.drop.handler(a,o);break}a.type="mouseup"}case"touchend":case"mouseup":default:n.touched?t.remove(n.touched,"touchmove touchend",n.handler):t.remove(document,"mousemove mouseup",n.handler),o.dragging&&(!1!==o.drop&&i.drop&&i.drop.handler(a,o),n.hijack(a,"dragend",o)),n.textselect(!0),!1===o.click&&o.dragging&&e.data(o.mousedown,"suppress.click",(new Date).getTime()+5),o.dragging=n.touched=!1;break}},hijack:function(i,a,o,s,r){if(o){var l,c,h,d={event:i.originalEvent,type:i.type},u=a.indexOf("drop")?"drag":"drop",p=s||0,f=isNaN(s)?o.interactions.length:s;i.type=a;var g=function(){};i.originalEvent=new jQuery.Event(d.event,{preventDefault:g,stopPropagation:g,stopImmediatePropagation:g}),o.results=[];do{if(c=o.interactions[p]){if("dragend"!==a&&c.cancelled)continue;h=n.properties(i,o,c),c.results=[],e(r||c[u]||o.droppable).each((function(s,r){if(h.target=r,i.isPropagationStopped=function(){return!1},l=r?t.dispatch.call(r,i,h):null,!1===l?("drag"==u&&(c.cancelled=!0,o.propagates-=1),"drop"==a&&(c[u][s]=null)):"dropinit"==a&&c.droppable.push(n.element(l)||r),"dragstart"==a&&(c.proxy=e(n.element(l)||c.drag)[0]),c.results.push(l),delete i.result,"dropinit"!==a)return l})),o.results[p]=n.flatten(c.results),"dropinit"==a&&(c.droppable=n.flatten(c.droppable)),"dragstart"!=a||c.cancelled||h.update()}}while(++p<f);return i.type=d.type,i.originalEvent=d.event,n.flatten(o.results)}},properties:function(e,t,i){var a=i.callback;return a.drag=i.drag,a.proxy=i.proxy||i.drag,a.startX=t.pageX,a.startY=t.pageY,a.deltaX=e.pageX-t.pageX,a.deltaY=e.pageY-t.pageY,a.originalX=i.offset.left,a.originalY=i.offset.top,a.offsetX=a.originalX+a.deltaX,a.offsetY=a.originalY+a.deltaY,a.drop=n.flatten((i.drop||[]).slice()),a.available=n.flatten((i.droppable||[]).slice()),a},element:function(e){if(e&&(e.jquery||1==e.nodeType))return e},flatten:function(t){return e.map(t,(function(t){return t&&t.jquery?e.makeArray(t):t&&t.length?n.flatten(t):t}))},textselect:function(t){e(document)[t?"off":"on"]("selectstart",n.dontstart).css("MozUserSelect",t?"":"none"),document.unselectable=t?"off":"on"},dontstart:function(){return!1},callback:function(){}};n.callback.prototype={update:function(){i.drop&&this.available.length&&e.each(this.available,(function(e){i.drop.locate(this,e)}))}};var a=t.dispatch;t.dispatch=function(t){if(!(e.data(this,"suppress."+t.type)-(new Date).getTime()>0))return a.apply(this,arguments);e.removeData(this,"suppress."+t.type)},i.draginit=i.dragstart=i.dragend=n})(jQuery)},ef08d:function(e,t,i){e.exports=i.p+"static/img/icon8.008745b8.svg"},f06f:function(module,exports){
/*! jQuery UI - v1.9.2 - 2012-11-23
* http://jqueryui.com
* Includes: jquery.ui.core.js, jquery.ui.widget.js, jquery.ui.mouse.js, jquery.ui.draggable.js, jquery.ui.droppable.js, jquery.ui.resizable.js, jquery.ui.selectable.js, jquery.ui.sortable.js, jquery.ui.effect.js, jquery.ui.accordion.js, jquery.ui.autocomplete.js, jquery.ui.button.js, jquery.ui.datepicker.js, jquery.ui.dialog.js, jquery.ui.effect-blind.js, jquery.ui.effect-bounce.js, jquery.ui.effect-clip.js, jquery.ui.effect-drop.js, jquery.ui.effect-explode.js, jquery.ui.effect-fade.js, jquery.ui.effect-fold.js, jquery.ui.effect-highlight.js, jquery.ui.effect-pulsate.js, jquery.ui.effect-scale.js, jquery.ui.effect-shake.js, jquery.ui.effect-slide.js, jquery.ui.effect-transfer.js, jquery.ui.menu.js, jquery.ui.position.js, jquery.ui.progressbar.js, jquery.ui.slider.js, jquery.ui.spinner.js, jquery.ui.tabs.js, jquery.ui.tooltip.js
* Copyright 2012 jQuery Foundation and other contributors; Licensed MIT */
(function(e,t){var i=0,n=/^ui-id-\d+$/;function a(t,i){var n,a,s,r=t.nodeName.toLowerCase();return"area"===r?(n=t.parentNode,a=n.name,!(!t.href||!a||"map"!==n.nodeName.toLowerCase())&&(s=e("img[usemap=#"+a+"]")[0],!!s&&o(s))):(/input|select|textarea|button|object/.test(r)?!t.disabled:"a"===r&&t.href||i)&&o(t)}function o(t){return e.expr.filters.visible(t)&&!e(t).parents().andSelf().filter((function(){return"hidden"===e.css(this,"visibility")})).length}e.ui=e.ui||{},e.ui.version||(e.extend(e.ui,{version:"1.9.2",keyCode:{BACKSPACE:8,COMMA:188,DELETE:46,DOWN:40,END:35,ENTER:13,ESCAPE:27,HOME:36,LEFT:37,NUMPAD_ADD:107,NUMPAD_DECIMAL:110,NUMPAD_DIVIDE:111,NUMPAD_ENTER:108,NUMPAD_MULTIPLY:106,NUMPAD_SUBTRACT:109,PAGE_DOWN:34,PAGE_UP:33,PERIOD:190,RIGHT:39,SPACE:32,TAB:9,UP:38}}),e.fn.extend({_focus:e.fn.focus,focus:function(t,i){return"number"===typeof t?this.each((function(){var n=this;setTimeout((function(){e(n).focus(),i&&i.call(n)}),t)})):this._focus.apply(this,arguments)},scrollParent:function(){var t;return t=e.ui.ie&&/(static|relative)/.test(this.css("position"))||/absolute/.test(this.css("position"))?this.parents().filter((function(){return/(relative|absolute|fixed)/.test(e.css(this,"position"))&&/(auto|scroll)/.test(e.css(this,"overflow")+e.css(this,"overflow-y")+e.css(this,"overflow-x"))})).eq(0):this.parents().filter((function(){return/(auto|scroll)/.test(e.css(this,"overflow")+e.css(this,"overflow-y")+e.css(this,"overflow-x"))})).eq(0),/fixed/.test(this.css("position"))||!t.length?e(document):t},zIndex:function(i){if(i!==t)return this.css("zIndex",i);if(this.length){var n,a,o=e(this[0]);while(o.length&&o[0]!==document){if(n=o.css("position"),("absolute"===n||"relative"===n||"fixed"===n)&&(a=parseInt(o.css("zIndex"),10),!isNaN(a)&&0!==a))return a;o=o.parent()}}return 0},uniqueId:function(){return this.each((function(){this.id||(this.id="ui-id-"+ ++i)}))},removeUniqueId:function(){return this.each((function(){n.test(this.id)&&e(this).removeAttr("id")}))}}),e.extend(e.expr[":"],{data:e.expr.createPseudo?e.expr.createPseudo((function(t){return function(i){return!!e.data(i,t)}})):function(t,i,n){return!!e.data(t,n[3])},focusable:function(t){return a(t,!isNaN(e.attr(t,"tabindex")))},tabbable:function(t){var i=e.attr(t,"tabindex"),n=isNaN(i);return(n||i>=0)&&a(t,!n)}}),e((function(){var t=document.body,i=t.appendChild(i=document.createElement("div"));i.offsetHeight,e.extend(i.style,{minHeight:"100px",height:"auto",padding:0,borderWidth:0}),e.support.minHeight=100===i.offsetHeight,e.support.selectstart="onselectstart"in i,t.removeChild(i).style.display="none"})),e("<a>").outerWidth(1).jquery||e.each(["Width","Height"],(function(i,n){var a="Width"===n?["Left","Right"]:["Top","Bottom"],o=n.toLowerCase(),s={innerWidth:e.fn.innerWidth,innerHeight:e.fn.innerHeight,outerWidth:e.fn.outerWidth,outerHeight:e.fn.outerHeight};function r(t,i,n,o){return e.each(a,(function(){i-=parseFloat(e.css(t,"padding"+this))||0,n&&(i-=parseFloat(e.css(t,"border"+this+"Width"))||0),o&&(i-=parseFloat(e.css(t,"margin"+this))||0)})),i}e.fn["inner"+n]=function(i){return i===t?s["inner"+n].call(this):this.each((function(){e(this).css(o,r(this,i)+"px")}))},e.fn["outer"+n]=function(t,i){return"number"!==typeof t?s["outer"+n].call(this,t):this.each((function(){e(this).css(o,r(this,t,!0,i)+"px")}))}})),e("<a>").data("a-b","a").removeData("a-b").data("a-b")&&(e.fn.removeData=function(t){return function(i){return arguments.length?t.call(this,e.camelCase(i)):t.call(this)}}(e.fn.removeData)),function(){var t=/msie ([\w.]+)/.exec(navigator.userAgent.toLowerCase())||[];e.ui.ie=!!t.length,e.ui.ie6=6===parseFloat(t[1],10)}(),e.fn.extend({disableSelection:function(){return this.bind((e.support.selectstart?"selectstart":"mousedown")+".ui-disableSelection",(function(e){e.preventDefault()}))},enableSelection:function(){return this.unbind(".ui-disableSelection")}}),e.extend(e.ui,{plugin:{add:function(t,i,n){var a,o=e.ui[t].prototype;for(a in n)o.plugins[a]=o.plugins[a]||[],o.plugins[a].push([i,n[a]])},call:function(e,t,i){var n,a=e.plugins[t];if(a&&e.element[0].parentNode&&11!==e.element[0].parentNode.nodeType)for(n=0;n<a.length;n++)e.options[a[n][0]]&&a[n][1].apply(e.element,i)}},contains:e.contains,hasScroll:function(t,i){if("hidden"===e(t).css("overflow"))return!1;var n=i&&"left"===i?"scrollLeft":"scrollTop",a=!1;return t[n]>0||(t[n]=1,a=t[n]>0,t[n]=0,a)},isOverAxis:function(e,t,i){return e>t&&e<t+i},isOver:function(t,i,n,a,o,s){return e.ui.isOverAxis(t,n,o)&&e.ui.isOverAxis(i,a,s)}}))})(jQuery),function(e,t){var i=0,n=Array.prototype.slice,a=e.cleanData;e.cleanData=function(t){for(var i,n=0;null!=(i=t[n]);n++)try{e(i).triggerHandler("remove")}catch(o){}a(t)},e.widget=function(t,i,n){var a,o,s,r,l=t.split(".")[0];t=t.split(".")[1],a=l+"-"+t,n||(n=i,i=e.Widget),e.expr[":"][a.toLowerCase()]=function(t){return!!e.data(t,a)},e[l]=e[l]||{},o=e[l][t],s=e[l][t]=function(e,t){if(!this._createWidget)return new s(e,t);arguments.length&&this._createWidget(e,t)},e.extend(s,o,{version:n.version,_proto:e.extend({},n),_childConstructors:[]}),r=new i,r.options=e.widget.extend({},r.options),e.each(n,(function(t,a){e.isFunction(a)&&(n[t]=function(){var e=function(){return i.prototype[t].apply(this,arguments)},n=function(e){return i.prototype[t].apply(this,e)};return function(){var t,i=this._super,o=this._superApply;return this._super=e,this._superApply=n,t=a.apply(this,arguments),this._super=i,this._superApply=o,t}}())})),s.prototype=e.widget.extend(r,{widgetEventPrefix:o?r.widgetEventPrefix:t},n,{constructor:s,namespace:l,widgetName:t,widgetBaseClass:a,widgetFullName:a}),o?(e.each(o._childConstructors,(function(t,i){var n=i.prototype;e.widget(n.namespace+"."+n.widgetName,s,i._proto)})),delete o._childConstructors):i._childConstructors.push(s),e.widget.bridge(t,s)},e.widget.extend=function(i){for(var a,o,s=n.call(arguments,1),r=0,l=s.length;r<l;r++)for(a in s[r])o=s[r][a],s[r].hasOwnProperty(a)&&o!==t&&(e.isPlainObject(o)?i[a]=e.isPlainObject(i[a])?e.widget.extend({},i[a],o):e.widget.extend({},o):i[a]=o);return i},e.widget.bridge=function(i,a){var o=a.prototype.widgetFullName||i;e.fn[i]=function(s){var r="string"===typeof s,l=n.call(arguments,1),c=this;return s=!r&&l.length?e.widget.extend.apply(null,[s].concat(l)):s,r?this.each((function(){var n,a=e.data(this,o);return a?e.isFunction(a[s])&&"_"!==s.charAt(0)?(n=a[s].apply(a,l),n!==a&&n!==t?(c=n&&n.jquery?c.pushStack(n.get()):n,!1):void 0):e.error("no such method '"+s+"' for "+i+" widget instance"):e.error("cannot call methods on "+i+" prior to initialization; attempted to call method '"+s+"'")})):this.each((function(){var t=e.data(this,o);t?t.option(s||{})._init():e.data(this,o,new a(s,this))})),c}},e.Widget=function(){},e.Widget._childConstructors=[],e.Widget.prototype={widgetName:"widget",widgetEventPrefix:"",defaultElement:"<div>",options:{disabled:!1,create:null},_createWidget:function(t,n){n=e(n||this.defaultElement||this)[0],this.element=e(n),this.uuid=i++,this.eventNamespace="."+this.widgetName+this.uuid,this.options=e.widget.extend({},this.options,this._getCreateOptions(),t),this.bindings=e(),this.hoverable=e(),this.focusable=e(),n!==this&&(e.data(n,this.widgetName,this),e.data(n,this.widgetFullName,this),this._on(!0,this.element,{remove:function(e){e.target===n&&this.destroy()}}),this.document=e(n.style?n.ownerDocument:n.document||n),this.window=e(this.document[0].defaultView||this.document[0].parentWindow)),this._create(),this._trigger("create",null,this._getCreateEventData()),this._init()},_getCreateOptions:e.noop,_getCreateEventData:e.noop,_create:e.noop,_init:e.noop,destroy:function(){this._destroy(),this.element.unbind(this.eventNamespace).removeData(this.widgetName).removeData(this.widgetFullName).removeData(e.camelCase(this.widgetFullName)),this.widget().unbind(this.eventNamespace).removeAttr("aria-disabled").removeClass(this.widgetFullName+"-disabled ui-state-disabled"),this.bindings.unbind(this.eventNamespace),this.hoverable.removeClass("ui-state-hover"),this.focusable.removeClass("ui-state-focus")},_destroy:e.noop,widget:function(){return this.element},option:function(i,n){var a,o,s,r=i;if(0===arguments.length)return e.widget.extend({},this.options);if("string"===typeof i)if(r={},a=i.split("."),i=a.shift(),a.length){for(o=r[i]=e.widget.extend({},this.options[i]),s=0;s<a.length-1;s++)o[a[s]]=o[a[s]]||{},o=o[a[s]];if(i=a.pop(),n===t)return o[i]===t?null:o[i];o[i]=n}else{if(n===t)return this.options[i]===t?null:this.options[i];r[i]=n}return this._setOptions(r),this},_setOptions:function(e){var t;for(t in e)this._setOption(t,e[t]);return this},_setOption:function(e,t){return this.options[e]=t,"disabled"===e&&(this.widget().toggleClass(this.widgetFullName+"-disabled ui-state-disabled",!!t).attr("aria-disabled",t),this.hoverable.removeClass("ui-state-hover"),this.focusable.removeClass("ui-state-focus")),this},enable:function(){return this._setOption("disabled",!1)},disable:function(){return this._setOption("disabled",!0)},_on:function(t,i,n){var a,o=this;"boolean"!==typeof t&&(n=i,i=t,t=!1),n?(i=a=e(i),this.bindings=this.bindings.add(i)):(n=i,i=this.element,a=this.widget()),e.each(n,(function(n,s){function r(){if(t||!0!==o.options.disabled&&!e(this).hasClass("ui-state-disabled"))return("string"===typeof s?o[s]:s).apply(o,arguments)}"string"!==typeof s&&(r.guid=s.guid=s.guid||r.guid||e.guid++);var l=n.match(/^(\w+)\s*(.*)$/),c=l[1]+o.eventNamespace,h=l[2];h?a.delegate(h,c,r):i.bind(c,r)}))},_off:function(e,t){t=(t||"").split(" ").join(this.eventNamespace+" ")+this.eventNamespace,e.unbind(t).undelegate(t)},_delay:function(e,t){function i(){return("string"===typeof e?n[e]:e).apply(n,arguments)}var n=this;return setTimeout(i,t||0)},_hoverable:function(t){this.hoverable=this.hoverable.add(t),this._on(t,{mouseenter:function(t){e(t.currentTarget).addClass("ui-state-hover")},mouseleave:function(t){e(t.currentTarget).removeClass("ui-state-hover")}})},_focusable:function(t){this.focusable=this.focusable.add(t),this._on(t,{focusin:function(t){e(t.currentTarget).addClass("ui-state-focus")},focusout:function(t){e(t.currentTarget).removeClass("ui-state-focus")}})},_trigger:function(t,i,n){var a,o,s=this.options[t];if(n=n||{},i=e.Event(i),i.type=(t===this.widgetEventPrefix?t:this.widgetEventPrefix+t).toLowerCase(),i.target=this.element[0],o=i.originalEvent,o)for(a in o)a in i||(i[a]=o[a]);return this.element.trigger(i,n),!(e.isFunction(s)&&!1===s.apply(this.element[0],[i].concat(n))||i.isDefaultPrevented())}},e.each({show:"fadeIn",hide:"fadeOut"},(function(t,i){e.Widget.prototype["_"+t]=function(n,a,o){"string"===typeof a&&(a={effect:a});var s,r=a?!0===a||"number"===typeof a?i:a.effect||i:t;a=a||{},"number"===typeof a&&(a={duration:a}),s=!e.isEmptyObject(a),a.complete=o,a.delay&&n.delay(a.delay),s&&e.effects&&(e.effects.effect[r]||!1!==e.uiBackCompat&&e.effects[r])?n[t](a):r!==t&&n[r]?n[r](a.duration,a.easing,o):n.queue((function(i){e(this)[t](),o&&o.call(n[0]),i()}))}})),!1!==e.uiBackCompat&&(e.Widget.prototype._getCreateOptions=function(){return e.metadata&&e.metadata.get(this.element[0])[this.widgetName]})}(jQuery),function(e,t){var i=!1;e(document).mouseup((function(e){i=!1})),e.widget("ui.mouse",{version:"1.9.2",options:{cancel:"input,textarea,button,select,option",distance:1,delay:0},_mouseInit:function(){var t=this;this.element.bind("mousedown."+this.widgetName,(function(e){return t._mouseDown(e)})).bind("click."+this.widgetName,(function(i){if(!0===e.data(i.target,t.widgetName+".preventClickEvent"))return e.removeData(i.target,t.widgetName+".preventClickEvent"),i.stopImmediatePropagation(),!1})),this.started=!1},_mouseDestroy:function(){this.element.unbind("."+this.widgetName),this._mouseMoveDelegate&&e(document).unbind("mousemove."+this.widgetName,this._mouseMoveDelegate).unbind("mouseup."+this.widgetName,this._mouseUpDelegate)},_mouseDown:function(t){if(!i){this._mouseStarted&&this._mouseUp(t),this._mouseDownEvent=t;var n=this,a=1===t.which,o=!("string"!==typeof this.options.cancel||!t.target.nodeName)&&e(t.target).closest(this.options.cancel).length;return!(a&&!o&&this._mouseCapture(t))||(this.mouseDelayMet=!this.options.delay,this.mouseDelayMet||(this._mouseDelayTimer=setTimeout((function(){n.mouseDelayMet=!0}),this.options.delay)),this._mouseDistanceMet(t)&&this._mouseDelayMet(t)&&(this._mouseStarted=!1!==this._mouseStart(t),!this._mouseStarted)?(t.preventDefault(),!0):(!0===e.data(t.target,this.widgetName+".preventClickEvent")&&e.removeData(t.target,this.widgetName+".preventClickEvent"),this._mouseMoveDelegate=function(e){return n._mouseMove(e)},this._mouseUpDelegate=function(e){return n._mouseUp(e)},e(document).bind("mousemove."+this.widgetName,this._mouseMoveDelegate).bind("mouseup."+this.widgetName,this._mouseUpDelegate),t.preventDefault(),i=!0,!0))}},_mouseMove:function(t){return!e.ui.ie||document.documentMode>=9||t.button?this._mouseStarted?(this._mouseDrag(t),t.preventDefault()):(this._mouseDistanceMet(t)&&this._mouseDelayMet(t)&&(this._mouseStarted=!1!==this._mouseStart(this._mouseDownEvent,t),this._mouseStarted?this._mouseDrag(t):this._mouseUp(t)),!this._mouseStarted):this._mouseUp(t)},_mouseUp:function(t){return e(document).unbind("mousemove."+this.widgetName,this._mouseMoveDelegate).unbind("mouseup."+this.widgetName,this._mouseUpDelegate),this._mouseStarted&&(this._mouseStarted=!1,t.target===this._mouseDownEvent.target&&e.data(t.target,this.widgetName+".preventClickEvent",!0),this._mouseStop(t)),!1},_mouseDistanceMet:function(e){return Math.max(Math.abs(this._mouseDownEvent.pageX-e.pageX),Math.abs(this._mouseDownEvent.pageY-e.pageY))>=this.options.distance},_mouseDelayMet:function(e){return this.mouseDelayMet},_mouseStart:function(e){},_mouseDrag:function(e){},_mouseStop:function(e){},_mouseCapture:function(e){return!0}})}(jQuery),function(e,t){e.widget("ui.draggable",e.ui.mouse,{version:"1.9.2",widgetEventPrefix:"drag",options:{addClasses:!0,appendTo:"parent",axis:!1,connectToSortable:!1,containment:!1,cursor:"auto",cursorAt:!1,grid:!1,handle:!1,helper:"original",iframeFix:!1,opacity:!1,refreshPositions:!1,revert:!1,revertDuration:500,scope:"default",scroll:!0,scrollSensitivity:20,scrollSpeed:20,snap:!1,snapMode:"both",snapTolerance:20,stack:!1,zIndex:!1},_create:function(){"original"!=this.options.helper||/^(?:r|a|f)/.test(this.element.css("position"))||(this.element[0].style.position="relative"),this.options.addClasses&&this.element.addClass("ui-draggable"),this.options.disabled&&this.element.addClass("ui-draggable-disabled"),this._mouseInit()},_destroy:function(){this.element.removeClass("ui-draggable ui-draggable-dragging ui-draggable-disabled"),this._mouseDestroy()},_mouseCapture:function(t){var i=this.options;return!(this.helper||i.disabled||e(t.target).is(".ui-resizable-handle"))&&(this.handle=this._getHandle(t),!!this.handle&&(e(!0===i.iframeFix?"iframe":i.iframeFix).each((function(){e('<div class="ui-draggable-iframeFix" style="background: #fff;"></div>').css({width:this.offsetWidth+"px",height:this.offsetHeight+"px",position:"absolute",opacity:"0.001",zIndex:1e3}).css(e(this).offset()).appendTo("body")})),!0))},_mouseStart:function(t){var i=this.options;return this.helper=this._createHelper(t),this.helper.addClass("ui-draggable-dragging"),this._cacheHelperProportions(),e.ui.ddmanager&&(e.ui.ddmanager.current=this),this._cacheMargins(),this.cssPosition=this.helper.css("position"),this.scrollParent=this.helper.scrollParent(),this.offset=this.positionAbs=this.element.offset(),this.offset={top:this.offset.top-this.margins.top,left:this.offset.left-this.margins.left},e.extend(this.offset,{click:{left:t.pageX-this.offset.left,top:t.pageY-this.offset.top},parent:this._getParentOffset(),relative:this._getRelativeOffset()}),this.originalPosition=this.position=this._generatePosition(t),this.originalPageX=t.pageX,this.originalPageY=t.pageY,i.cursorAt&&this._adjustOffsetFromHelper(i.cursorAt),i.containment&&this._setContainment(),!1===this._trigger("start",t)?(this._clear(),!1):(this._cacheHelperProportions(),e.ui.ddmanager&&!i.dropBehaviour&&e.ui.ddmanager.prepareOffsets(this,t),this._mouseDrag(t,!0),e.ui.ddmanager&&e.ui.ddmanager.dragStart(this,t),!0)},_mouseDrag:function(t,i){if(this.position=this._generatePosition(t),this.positionAbs=this._convertPositionTo("absolute"),!i){var n=this._uiHash();if(!1===this._trigger("drag",t,n))return this._mouseUp({}),!1;this.position=n.position}return this.options.axis&&"y"==this.options.axis||(this.helper[0].style.left=this.position.left+"px"),this.options.axis&&"x"==this.options.axis||(this.helper[0].style.top=this.position.top+"px"),e.ui.ddmanager&&e.ui.ddmanager.drag(this,t),!1},_mouseStop:function(t){var i=!1;e.ui.ddmanager&&!this.options.dropBehaviour&&(i=e.ui.ddmanager.drop(this,t)),this.dropped&&(i=this.dropped,this.dropped=!1);var n=this.element[0],a=!1;while(n&&(n=n.parentNode))n==document&&(a=!0);if(!a&&"original"===this.options.helper)return!1;if("invalid"==this.options.revert&&!i||"valid"==this.options.revert&&i||!0===this.options.revert||e.isFunction(this.options.revert)&&this.options.revert.call(this.element,i)){var o=this;e(this.helper).animate(this.originalPosition,parseInt(this.options.revertDuration,10),(function(){!1!==o._trigger("stop",t)&&o._clear()}))}else!1!==this._trigger("stop",t)&&this._clear();return!1},_mouseUp:function(t){return e("div.ui-draggable-iframeFix").each((function(){this.parentNode.removeChild(this)})),e.ui.ddmanager&&e.ui.ddmanager.dragStop(this,t),e.ui.mouse.prototype._mouseUp.call(this,t)},cancel:function(){return this.helper.is(".ui-draggable-dragging")?this._mouseUp({}):this._clear(),this},_getHandle:function(t){var i=!this.options.handle||!e(this.options.handle,this.element).length;return e(this.options.handle,this.element).find("*").andSelf().each((function(){this==t.target&&(i=!0)})),i},_createHelper:function(t){var i=this.options,n=e.isFunction(i.helper)?e(i.helper.apply(this.element[0],[t])):"clone"==i.helper?this.element.clone().removeAttr("id"):this.element;return n.parents("body").length||n.appendTo("parent"==i.appendTo?this.element[0].parentNode:i.appendTo),n[0]==this.element[0]||/(fixed|absolute)/.test(n.css("position"))||n.css("position","absolute"),n},_adjustOffsetFromHelper:function(t){"string"==typeof t&&(t=t.split(" ")),e.isArray(t)&&(t={left:+t[0],top:+t[1]||0}),"left"in t&&(this.offset.click.left=t.left+this.margins.left),"right"in t&&(this.offset.click.left=this.helperProportions.width-t.right+this.margins.left),"top"in t&&(this.offset.click.top=t.top+this.margins.top),"bottom"in t&&(this.offset.click.top=this.helperProportions.height-t.bottom+this.margins.top)},_getParentOffset:function(){this.offsetParent=this.helper.offsetParent();var t=this.offsetParent.offset();return"absolute"==this.cssPosition&&this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])&&(t.left+=this.scrollParent.scrollLeft(),t.top+=this.scrollParent.scrollTop()),(this.offsetParent[0]==document.body||this.offsetParent[0].tagName&&"html"==this.offsetParent[0].tagName.toLowerCase()&&e.ui.ie)&&(t={top:0,left:0}),{top:t.top+(parseInt(this.offsetParent.css("borderTopWidth"),10)||0),left:t.left+(parseInt(this.offsetParent.css("borderLeftWidth"),10)||0)}},_getRelativeOffset:function(){if("relative"==this.cssPosition){var e=this.element.position();return{top:e.top-(parseInt(this.helper.css("top"),10)||0)+this.scrollParent.scrollTop(),left:e.left-(parseInt(this.helper.css("left"),10)||0)+this.scrollParent.scrollLeft()}}return{top:0,left:0}},_cacheMargins:function(){this.margins={left:parseInt(this.element.css("marginLeft"),10)||0,top:parseInt(this.element.css("marginTop"),10)||0,right:parseInt(this.element.css("marginRight"),10)||0,bottom:parseInt(this.element.css("marginBottom"),10)||0}},_cacheHelperProportions:function(){this.helperProportions={width:this.helper.outerWidth(),height:this.helper.outerHeight()}},_setContainment:function(){var t=this.options;if("parent"==t.containment&&(t.containment=this.helper[0].parentNode),"document"!=t.containment&&"window"!=t.containment||(this.containment=["document"==t.containment?0:e(window).scrollLeft()-this.offset.relative.left-this.offset.parent.left,"document"==t.containment?0:e(window).scrollTop()-this.offset.relative.top-this.offset.parent.top,("document"==t.containment?0:e(window).scrollLeft())+e("document"==t.containment?document:window).width()-this.helperProportions.width-this.margins.left,("document"==t.containment?0:e(window).scrollTop())+(e("document"==t.containment?document:window).height()||document.body.parentNode.scrollHeight)-this.helperProportions.height-this.margins.top]),/^(document|window|parent)$/.test(t.containment)||t.containment.constructor==Array)t.containment.constructor==Array&&(this.containment=t.containment);else{var i=e(t.containment),n=i[0];if(!n)return;i.offset();var a="hidden"!=e(n).css("overflow");this.containment=[(parseInt(e(n).css("borderLeftWidth"),10)||0)+(parseInt(e(n).css("paddingLeft"),10)||0),(parseInt(e(n).css("borderTopWidth"),10)||0)+(parseInt(e(n).css("paddingTop"),10)||0),(a?Math.max(n.scrollWidth,n.offsetWidth):n.offsetWidth)-(parseInt(e(n).css("borderLeftWidth"),10)||0)-(parseInt(e(n).css("paddingRight"),10)||0)-this.helperProportions.width-this.margins.left-this.margins.right,(a?Math.max(n.scrollHeight,n.offsetHeight):n.offsetHeight)-(parseInt(e(n).css("borderTopWidth"),10)||0)-(parseInt(e(n).css("paddingBottom"),10)||0)-this.helperProportions.height-this.margins.top-this.margins.bottom],this.relative_container=i}},_convertPositionTo:function(t,i){i||(i=this.position);var n="absolute"==t?1:-1,a=(this.options,"absolute"!=this.cssPosition||this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent),o=/(html|body)/i.test(a[0].tagName);return{top:i.top+this.offset.relative.top*n+this.offset.parent.top*n-("fixed"==this.cssPosition?-this.scrollParent.scrollTop():o?0:a.scrollTop())*n,left:i.left+this.offset.relative.left*n+this.offset.parent.left*n-("fixed"==this.cssPosition?-this.scrollParent.scrollLeft():o?0:a.scrollLeft())*n}},_generatePosition:function(t){var i=this.options,n="absolute"!=this.cssPosition||this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent,a=/(html|body)/i.test(n[0].tagName),o=t.pageX,s=t.pageY;if(this.originalPosition){var r;if(this.containment){if(this.relative_container){var l=this.relative_container.offset();r=[this.containment[0]+l.left,this.containment[1]+l.top,this.containment[2]+l.left,this.containment[3]+l.top]}else r=this.containment;t.pageX-this.offset.click.left<r[0]&&(o=r[0]+this.offset.click.left),t.pageY-this.offset.click.top<r[1]&&(s=r[1]+this.offset.click.top),t.pageX-this.offset.click.left>r[2]&&(o=r[2]+this.offset.click.left),t.pageY-this.offset.click.top>r[3]&&(s=r[3]+this.offset.click.top)}if(i.grid){var c=i.grid[1]?this.originalPageY+Math.round((s-this.originalPageY)/i.grid[1])*i.grid[1]:this.originalPageY;s=r&&(c-this.offset.click.top<r[1]||c-this.offset.click.top>r[3])?c-this.offset.click.top<r[1]?c+i.grid[1]:c-i.grid[1]:c;var h=i.grid[0]?this.originalPageX+Math.round((o-this.originalPageX)/i.grid[0])*i.grid[0]:this.originalPageX;o=r&&(h-this.offset.click.left<r[0]||h-this.offset.click.left>r[2])?h-this.offset.click.left<r[0]?h+i.grid[0]:h-i.grid[0]:h}}return{top:s-this.offset.click.top-this.offset.relative.top-this.offset.parent.top+("fixed"==this.cssPosition?-this.scrollParent.scrollTop():a?0:n.scrollTop()),left:o-this.offset.click.left-this.offset.relative.left-this.offset.parent.left+("fixed"==this.cssPosition?-this.scrollParent.scrollLeft():a?0:n.scrollLeft())}},_clear:function(){this.helper.removeClass("ui-draggable-dragging"),this.helper[0]==this.element[0]||this.cancelHelperRemoval||this.helper.remove(),this.helper=null,this.cancelHelperRemoval=!1},_trigger:function(t,i,n){return n=n||this._uiHash(),e.ui.plugin.call(this,t,[i,n]),"drag"==t&&(this.positionAbs=this._convertPositionTo("absolute")),e.Widget.prototype._trigger.call(this,t,i,n)},plugins:{},_uiHash:function(e){return{helper:this.helper,position:this.position,originalPosition:this.originalPosition,offset:this.positionAbs}}}),e.ui.plugin.add("draggable","connectToSortable",{start:function(t,i){var n=e(this).data("draggable"),a=n.options,o=e.extend({},i,{item:n.element});n.sortables=[],e(a.connectToSortable).each((function(){var i=e.data(this,"sortable");i&&!i.options.disabled&&(n.sortables.push({instance:i,shouldRevert:i.options.revert}),i.refreshPositions(),i._trigger("activate",t,o))}))},stop:function(t,i){var n=e(this).data("draggable"),a=e.extend({},i,{item:n.element});e.each(n.sortables,(function(){this.instance.isOver?(this.instance.isOver=0,n.cancelHelperRemoval=!0,this.instance.cancelHelperRemoval=!1,this.shouldRevert&&(this.instance.options.revert=!0),this.instance._mouseStop(t),this.instance.options.helper=this.instance.options._helper,"original"==n.options.helper&&this.instance.currentItem.css({top:"auto",left:"auto"})):(this.instance.cancelHelperRemoval=!1,this.instance._trigger("deactivate",t,a))}))},drag:function(t,i){var n=e(this).data("draggable"),a=this;e.each(n.sortables,(function(o){var s=!1,r=this;this.instance.positionAbs=n.positionAbs,this.instance.helperProportions=n.helperProportions,this.instance.offset.click=n.offset.click,this.instance._intersectsWith(this.instance.containerCache)&&(s=!0,e.each(n.sortables,(function(){return this.instance.positionAbs=n.positionAbs,this.instance.helperProportions=n.helperProportions,this.instance.offset.click=n.offset.click,this!=r&&this.instance._intersectsWith(this.instance.containerCache)&&e.ui.contains(r.instance.element[0],this.instance.element[0])&&(s=!1),s}))),s?(this.instance.isOver||(this.instance.isOver=1,this.instance.currentItem=e(a).clone().removeAttr("id").appendTo(this.instance.element).data("sortable-item",!0),this.instance.options._helper=this.instance.options.helper,this.instance.options.helper=function(){return i.helper[0]},t.target=this.instance.currentItem[0],this.instance._mouseCapture(t,!0),this.instance._mouseStart(t,!0,!0),this.instance.offset.click.top=n.offset.click.top,this.instance.offset.click.left=n.offset.click.left,this.instance.offset.parent.left-=n.offset.parent.left-this.instance.offset.parent.left,this.instance.offset.parent.top-=n.offset.parent.top-this.instance.offset.parent.top,n._trigger("toSortable",t),n.dropped=this.instance.element,n.currentItem=n.element,this.instance.fromOutside=n),this.instance.currentItem&&this.instance._mouseDrag(t)):this.instance.isOver&&(this.instance.isOver=0,this.instance.cancelHelperRemoval=!0,this.instance.options.revert=!1,this.instance._trigger("out",t,this.instance._uiHash(this.instance)),this.instance._mouseStop(t,!0),this.instance.options.helper=this.instance.options._helper,this.instance.currentItem.remove(),this.instance.placeholder&&this.instance.placeholder.remove(),n._trigger("fromSortable",t),n.dropped=!1)}))}}),e.ui.plugin.add("draggable","cursor",{start:function(t,i){var n=e("body"),a=e(this).data("draggable").options;n.css("cursor")&&(a._cursor=n.css("cursor")),n.css("cursor",a.cursor)},stop:function(t,i){var n=e(this).data("draggable").options;n._cursor&&e("body").css("cursor",n._cursor)}}),e.ui.plugin.add("draggable","opacity",{start:function(t,i){var n=e(i.helper),a=e(this).data("draggable").options;n.css("opacity")&&(a._opacity=n.css("opacity")),n.css("opacity",a.opacity)},stop:function(t,i){var n=e(this).data("draggable").options;n._opacity&&e(i.helper).css("opacity",n._opacity)}}),e.ui.plugin.add("draggable","scroll",{start:function(t,i){var n=e(this).data("draggable");n.scrollParent[0]!=document&&"HTML"!=n.scrollParent[0].tagName&&(n.overflowOffset=n.scrollParent.offset())},drag:function(t,i){var n=e(this).data("draggable"),a=n.options,o=!1;n.scrollParent[0]!=document&&"HTML"!=n.scrollParent[0].tagName?(a.axis&&"x"==a.axis||(n.overflowOffset.top+n.scrollParent[0].offsetHeight-t.pageY<a.scrollSensitivity?n.scrollParent[0].scrollTop=o=n.scrollParent[0].scrollTop+a.scrollSpeed:t.pageY-n.overflowOffset.top<a.scrollSensitivity&&(n.scrollParent[0].scrollTop=o=n.scrollParent[0].scrollTop-a.scrollSpeed)),a.axis&&"y"==a.axis||(n.overflowOffset.left+n.scrollParent[0].offsetWidth-t.pageX<a.scrollSensitivity?n.scrollParent[0].scrollLeft=o=n.scrollParent[0].scrollLeft+a.scrollSpeed:t.pageX-n.overflowOffset.left<a.scrollSensitivity&&(n.scrollParent[0].scrollLeft=o=n.scrollParent[0].scrollLeft-a.scrollSpeed))):(a.axis&&"x"==a.axis||(t.pageY-e(document).scrollTop()<a.scrollSensitivity?o=e(document).scrollTop(e(document).scrollTop()-a.scrollSpeed):e(window).height()-(t.pageY-e(document).scrollTop())<a.scrollSensitivity&&(o=e(document).scrollTop(e(document).scrollTop()+a.scrollSpeed))),a.axis&&"y"==a.axis||(t.pageX-e(document).scrollLeft()<a.scrollSensitivity?o=e(document).scrollLeft(e(document).scrollLeft()-a.scrollSpeed):e(window).width()-(t.pageX-e(document).scrollLeft())<a.scrollSensitivity&&(o=e(document).scrollLeft(e(document).scrollLeft()+a.scrollSpeed)))),!1!==o&&e.ui.ddmanager&&!a.dropBehaviour&&e.ui.ddmanager.prepareOffsets(n,t)}}),e.ui.plugin.add("draggable","snap",{start:function(t,i){var n=e(this).data("draggable"),a=n.options;n.snapElements=[],e(a.snap.constructor!=String?a.snap.items||":data(draggable)":a.snap).each((function(){var t=e(this),i=t.offset();this!=n.element[0]&&n.snapElements.push({item:this,width:t.outerWidth(),height:t.outerHeight(),top:i.top,left:i.left})}))},drag:function(t,i){for(var n=e(this).data("draggable"),a=n.options,o=a.snapTolerance,s=i.offset.left,r=s+n.helperProportions.width,l=i.offset.top,c=l+n.helperProportions.height,h=n.snapElements.length-1;h>=0;h--){var d=n.snapElements[h].left,u=d+n.snapElements[h].width,p=n.snapElements[h].top,f=p+n.snapElements[h].height;if(d-o<s&&s<u+o&&p-o<l&&l<f+o||d-o<s&&s<u+o&&p-o<c&&c<f+o||d-o<r&&r<u+o&&p-o<l&&l<f+o||d-o<r&&r<u+o&&p-o<c&&c<f+o){if("inner"!=a.snapMode){var g=Math.abs(p-c)<=o,m=Math.abs(f-l)<=o,v=Math.abs(d-r)<=o,b=Math.abs(u-s)<=o;g&&(i.position.top=n._convertPositionTo("relative",{top:p-n.helperProportions.height,left:0}).top-n.margins.top),m&&(i.position.top=n._convertPositionTo("relative",{top:f,left:0}).top-n.margins.top),v&&(i.position.left=n._convertPositionTo("relative",{top:0,left:d-n.helperProportions.width}).left-n.margins.left),b&&(i.position.left=n._convertPositionTo("relative",{top:0,left:u}).left-n.margins.left)}var w=g||m||v||b;if("outer"!=a.snapMode){g=Math.abs(p-l)<=o,m=Math.abs(f-c)<=o,v=Math.abs(d-s)<=o,b=Math.abs(u-r)<=o;g&&(i.position.top=n._convertPositionTo("relative",{top:p,left:0}).top-n.margins.top),m&&(i.position.top=n._convertPositionTo("relative",{top:f-n.helperProportions.height,left:0}).top-n.margins.top),v&&(i.position.left=n._convertPositionTo("relative",{top:0,left:d}).left-n.margins.left),b&&(i.position.left=n._convertPositionTo("relative",{top:0,left:u-n.helperProportions.width}).left-n.margins.left)}!n.snapElements[h].snapping&&(g||m||v||b||w)&&n.options.snap.snap&&n.options.snap.snap.call(n.element,t,e.extend(n._uiHash(),{snapItem:n.snapElements[h].item})),n.snapElements[h].snapping=g||m||v||b||w}else n.snapElements[h].snapping&&n.options.snap.release&&n.options.snap.release.call(n.element,t,e.extend(n._uiHash(),{snapItem:n.snapElements[h].item})),n.snapElements[h].snapping=!1}}}),e.ui.plugin.add("draggable","stack",{start:function(t,i){var n=e(this).data("draggable").options,a=e.makeArray(e(n.stack)).sort((function(t,i){return(parseInt(e(t).css("zIndex"),10)||0)-(parseInt(e(i).css("zIndex"),10)||0)}));if(a.length){var o=parseInt(a[0].style.zIndex)||0;e(a).each((function(e){this.style.zIndex=o+e})),this[0].style.zIndex=o+a.length}}}),e.ui.plugin.add("draggable","zIndex",{start:function(t,i){var n=e(i.helper),a=e(this).data("draggable").options;n.css("zIndex")&&(a._zIndex=n.css("zIndex")),n.css("zIndex",a.zIndex)},stop:function(t,i){var n=e(this).data("draggable").options;n._zIndex&&e(i.helper).css("zIndex",n._zIndex)}})}(jQuery),function(e,t){e.widget("ui.droppable",{version:"1.9.2",widgetEventPrefix:"drop",options:{accept:"*",activeClass:!1,addClasses:!0,greedy:!1,hoverClass:!1,scope:"default",tolerance:"intersect"},_create:function(){var t=this.options,i=t.accept;this.isover=0,this.isout=1,this.accept=e.isFunction(i)?i:function(e){return e.is(i)},this.proportions={width:this.element[0].offsetWidth,height:this.element[0].offsetHeight},e.ui.ddmanager.droppables[t.scope]=e.ui.ddmanager.droppables[t.scope]||[],e.ui.ddmanager.droppables[t.scope].push(this),t.addClasses&&this.element.addClass("ui-droppable")},_destroy:function(){for(var t=e.ui.ddmanager.droppables[this.options.scope],i=0;i<t.length;i++)t[i]==this&&t.splice(i,1);this.element.removeClass("ui-droppable ui-droppable-disabled")},_setOption:function(t,i){"accept"==t&&(this.accept=e.isFunction(i)?i:function(e){return e.is(i)}),e.Widget.prototype._setOption.apply(this,arguments)},_activate:function(t){var i=e.ui.ddmanager.current;this.options.activeClass&&this.element.addClass(this.options.activeClass),i&&this._trigger("activate",t,this.ui(i))},_deactivate:function(t){var i=e.ui.ddmanager.current;this.options.activeClass&&this.element.removeClass(this.options.activeClass),i&&this._trigger("deactivate",t,this.ui(i))},_over:function(t){var i=e.ui.ddmanager.current;i&&(i.currentItem||i.element)[0]!=this.element[0]&&this.accept.call(this.element[0],i.currentItem||i.element)&&(this.options.hoverClass&&this.element.addClass(this.options.hoverClass),this._trigger("over",t,this.ui(i)))},_out:function(t){var i=e.ui.ddmanager.current;i&&(i.currentItem||i.element)[0]!=this.element[0]&&this.accept.call(this.element[0],i.currentItem||i.element)&&(this.options.hoverClass&&this.element.removeClass(this.options.hoverClass),this._trigger("out",t,this.ui(i)))},_drop:function(t,i){var n=i||e.ui.ddmanager.current;if(!n||(n.currentItem||n.element)[0]==this.element[0])return!1;var a=!1;return this.element.find(":data(droppable)").not(".ui-draggable-dragging").each((function(){var t=e.data(this,"droppable");if(t.options.greedy&&!t.options.disabled&&t.options.scope==n.options.scope&&t.accept.call(t.element[0],n.currentItem||n.element)&&e.ui.intersect(n,e.extend(t,{offset:t.element.offset()}),t.options.tolerance))return a=!0,!1})),!a&&(!!this.accept.call(this.element[0],n.currentItem||n.element)&&(this.options.activeClass&&this.element.removeClass(this.options.activeClass),this.options.hoverClass&&this.element.removeClass(this.options.hoverClass),this._trigger("drop",t,this.ui(n)),this.element))},ui:function(e){return{draggable:e.currentItem||e.element,helper:e.helper,position:e.position,offset:e.positionAbs}}}),e.ui.intersect=function(t,i,n){if(!i.offset)return!1;var a=(t.positionAbs||t.position.absolute).left,o=a+t.helperProportions.width,s=(t.positionAbs||t.position.absolute).top,r=s+t.helperProportions.height,l=i.offset.left,c=l+i.proportions.width,h=i.offset.top,d=h+i.proportions.height;switch(n){case"fit":return l<=a&&o<=c&&h<=s&&r<=d;case"intersect":return l<a+t.helperProportions.width/2&&o-t.helperProportions.width/2<c&&h<s+t.helperProportions.height/2&&r-t.helperProportions.height/2<d;case"pointer":var u=(t.positionAbs||t.position.absolute).left+(t.clickOffset||t.offset.click).left,p=(t.positionAbs||t.position.absolute).top+(t.clickOffset||t.offset.click).top,f=e.ui.isOver(p,u,h,l,i.proportions.height,i.proportions.width);return f;case"touch":return(s>=h&&s<=d||r>=h&&r<=d||s<h&&r>d)&&(a>=l&&a<=c||o>=l&&o<=c||a<l&&o>c);default:return!1}},e.ui.ddmanager={current:null,droppables:{default:[]},prepareOffsets:function(t,i){var n=e.ui.ddmanager.droppables[t.options.scope]||[],a=i?i.type:null,o=(t.currentItem||t.element).find(":data(droppable)").andSelf();e:for(var s=0;s<n.length;s++)if(!(n[s].options.disabled||t&&!n[s].accept.call(n[s].element[0],t.currentItem||t.element))){for(var r=0;r<o.length;r++)if(o[r]==n[s].element[0]){n[s].proportions.height=0;continue e}n[s].visible="none"!=n[s].element.css("display"),n[s].visible&&("mousedown"==a&&n[s]._activate.call(n[s],i),n[s].offset=n[s].element.offset(),n[s].proportions={width:n[s].element[0].offsetWidth,height:n[s].element[0].offsetHeight})}},drop:function(t,i){var n=!1;return e.each(e.ui.ddmanager.droppables[t.options.scope]||[],(function(){this.options&&(!this.options.disabled&&this.visible&&e.ui.intersect(t,this,this.options.tolerance)&&(n=this._drop.call(this,i)||n),!this.options.disabled&&this.visible&&this.accept.call(this.element[0],t.currentItem||t.element)&&(this.isout=1,this.isover=0,this._deactivate.call(this,i)))})),n},dragStart:function(t,i){t.element.parentsUntil("body").bind("scroll.droppable",(function(){t.options.refreshPositions||e.ui.ddmanager.prepareOffsets(t,i)}))},drag:function(t,i){t.options.refreshPositions&&e.ui.ddmanager.prepareOffsets(t,i),e.each(e.ui.ddmanager.droppables[t.options.scope]||[],(function(){if(!this.options.disabled&&!this.greedyChild&&this.visible){var n=e.ui.intersect(t,this,this.options.tolerance),a=n||1!=this.isover?n&&0==this.isover?"isover":null:"isout";if(a){var o;if(this.options.greedy){var s=this.options.scope,r=this.element.parents(":data(droppable)").filter((function(){return e.data(this,"droppable").options.scope===s}));r.length&&(o=e.data(r[0],"droppable"),o.greedyChild="isover"==a?1:0)}o&&"isover"==a&&(o["isover"]=0,o["isout"]=1,o._out.call(o,i)),this[a]=1,this["isout"==a?"isover":"isout"]=0,this["isover"==a?"_over":"_out"].call(this,i),o&&"isout"==a&&(o["isout"]=0,o["isover"]=1,o._over.call(o,i))}}}))},dragStop:function(t,i){t.element.parentsUntil("body").unbind("scroll.droppable"),t.options.refreshPositions||e.ui.ddmanager.prepareOffsets(t,i)}}}(jQuery),function(e,t){e.widget("ui.resizable",e.ui.mouse,{version:"1.9.2",widgetEventPrefix:"resize",options:{alsoResize:!1,animate:!1,animateDuration:"slow",animateEasing:"swing",aspectRatio:!1,autoHide:!1,containment:!1,ghost:!1,grid:!1,handles:"e,s,se",helper:!1,maxHeight:null,maxWidth:null,minHeight:10,minWidth:10,zIndex:1e3},_create:function(){var t=this,i=this.options;if(this.element.addClass("ui-resizable"),e.extend(this,{_aspectRatio:!!i.aspectRatio,aspectRatio:i.aspectRatio,originalElement:this.element,_proportionallyResizeElements:[],_helper:i.helper||i.ghost||i.animate?i.helper||"ui-resizable-helper":null}),this.element[0].nodeName.match(/canvas|textarea|input|select|button|img/i)&&(this.element.wrap(e('<div class="ui-wrapper" style="overflow: hidden;"></div>').css({position:this.element.css("position"),width:this.element.outerWidth(),height:this.element.outerHeight(),top:this.element.css("top"),left:this.element.css("left")})),this.element=this.element.parent().data("resizable",this.element.data("resizable")),this.elementIsWrapper=!0,this.element.css({marginLeft:this.originalElement.css("marginLeft"),marginTop:this.originalElement.css("marginTop"),marginRight:this.originalElement.css("marginRight"),marginBottom:this.originalElement.css("marginBottom")}),this.originalElement.css({marginLeft:0,marginTop:0,marginRight:0,marginBottom:0}),this.originalResizeStyle=this.originalElement.css("resize"),this.originalElement.css("resize","none"),this._proportionallyResizeElements.push(this.originalElement.css({position:"static",zoom:1,display:"block"})),this.originalElement.css({margin:this.originalElement.css("margin")}),this._proportionallyResize()),this.handles=i.handles||(e(".ui-resizable-handle",this.element).length?{n:".ui-resizable-n",e:".ui-resizable-e",s:".ui-resizable-s",w:".ui-resizable-w",se:".ui-resizable-se",sw:".ui-resizable-sw",ne:".ui-resizable-ne",nw:".ui-resizable-nw"}:"e,s,se"),this.handles.constructor==String){"all"==this.handles&&(this.handles="n,e,s,w,se,sw,ne,nw");var n=this.handles.split(",");this.handles={};for(var a=0;a<n.length;a++){var o=e.trim(n[a]),s="ui-resizable-"+o,r=e('<div class="ui-resizable-handle '+s+'"></div>');r.css({zIndex:i.zIndex}),"se"==o&&r.addClass("ui-icon ui-icon-gripsmall-diagonal-se"),this.handles[o]=".ui-resizable-"+o,this.element.append(r)}}this._renderAxis=function(t){for(var i in t=t||this.element,this.handles){if(this.handles[i].constructor==String&&(this.handles[i]=e(this.handles[i],this.element).show()),this.elementIsWrapper&&this.originalElement[0].nodeName.match(/textarea|input|select|button/i)){var n=e(this.handles[i],this.element),a=0;a=/sw|ne|nw|se|n|s/.test(i)?n.outerHeight():n.outerWidth();var o=["padding",/ne|nw|n/.test(i)?"Top":/se|sw|s/.test(i)?"Bottom":/^e$/.test(i)?"Right":"Left"].join("");t.css(o,a),this._proportionallyResize()}e(this.handles[i]).length}},this._renderAxis(this.element),this._handles=e(".ui-resizable-handle",this.element).disableSelection(),this._handles.mouseover((function(){if(!t.resizing){if(this.className)var e=this.className.match(/ui-resizable-(se|sw|ne|nw|n|e|s|w)/i);t.axis=e&&e[1]?e[1]:"se"}})),i.autoHide&&(this._handles.hide(),e(this.element).addClass("ui-resizable-autohide").mouseenter((function(){i.disabled||(e(this).removeClass("ui-resizable-autohide"),t._handles.show())})).mouseleave((function(){i.disabled||t.resizing||(e(this).addClass("ui-resizable-autohide"),t._handles.hide())}))),this._mouseInit()},_destroy:function(){this._mouseDestroy();var t=function(t){e(t).removeClass("ui-resizable ui-resizable-disabled ui-resizable-resizing").removeData("resizable").removeData("ui-resizable").unbind(".resizable").find(".ui-resizable-handle").remove()};if(this.elementIsWrapper){t(this.element);var i=this.element;this.originalElement.css({position:i.css("position"),width:i.outerWidth(),height:i.outerHeight(),top:i.css("top"),left:i.css("left")}).insertAfter(i),i.remove()}return this.originalElement.css("resize",this.originalResizeStyle),t(this.originalElement),this},_mouseCapture:function(t){var i=!1;for(var n in this.handles)e(this.handles[n])[0]==t.target&&(i=!0);return!this.options.disabled&&i},_mouseStart:function(t){var n=this.options,a=this.element.position(),o=this.element;this.resizing=!0,this.documentScroll={top:e(document).scrollTop(),left:e(document).scrollLeft()},(o.is(".ui-draggable")||/absolute/.test(o.css("position")))&&o.css({position:"absolute",top:a.top,left:a.left}),this._renderProxy();var s=i(this.helper.css("left")),r=i(this.helper.css("top"));n.containment&&(s+=e(n.containment).scrollLeft()||0,r+=e(n.containment).scrollTop()||0),this.offset=this.helper.offset(),this.position={left:s,top:r},this.size=this._helper?{width:o.outerWidth(),height:o.outerHeight()}:{width:o.width(),height:o.height()},this.originalSize=this._helper?{width:o.outerWidth(),height:o.outerHeight()}:{width:o.width(),height:o.height()},this.originalPosition={left:s,top:r},this.sizeDiff={width:o.outerWidth()-o.width(),height:o.outerHeight()-o.height()},this.originalMousePosition={left:t.pageX,top:t.pageY},this.aspectRatio="number"==typeof n.aspectRatio?n.aspectRatio:this.originalSize.width/this.originalSize.height||1;var l=e(".ui-resizable-"+this.axis).css("cursor");return e("body").css("cursor","auto"==l?this.axis+"-resize":l),o.addClass("ui-resizable-resizing"),this._propagate("start",t),!0},_mouseDrag:function(e){var t=this.helper,i=(this.options,this.originalMousePosition),n=this.axis,a=e.pageX-i.left||0,o=e.pageY-i.top||0,s=this._change[n];if(!s)return!1;var r=s.apply(this,[e,a,o]);return this._updateVirtualBoundaries(e.shiftKey),(this._aspectRatio||e.shiftKey)&&(r=this._updateRatio(r,e)),r=this._respectSize(r,e),this._propagate("resize",e),t.css({top:this.position.top+"px",left:this.position.left+"px",width:this.size.width+"px",height:this.size.height+"px"}),!this._helper&&this._proportionallyResizeElements.length&&this._proportionallyResize(),this._updateCache(r),this._trigger("resize",e,this.ui()),!1},_mouseStop:function(t){this.resizing=!1;var i=this.options,n=this;if(this._helper){var a=this._proportionallyResizeElements,o=a.length&&/textarea/i.test(a[0].nodeName),s=o&&e.ui.hasScroll(a[0],"left")?0:n.sizeDiff.height,r=o?0:n.sizeDiff.width,l={width:n.helper.width()-r,height:n.helper.height()-s},c=parseInt(n.element.css("left"),10)+(n.position.left-n.originalPosition.left)||null,h=parseInt(n.element.css("top"),10)+(n.position.top-n.originalPosition.top)||null;i.animate||this.element.css(e.extend(l,{top:h,left:c})),n.helper.height(n.size.height),n.helper.width(n.size.width),this._helper&&!i.animate&&this._proportionallyResize()}return e("body").css("cursor","auto"),this.element.removeClass("ui-resizable-resizing"),this._propagate("stop",t),this._helper&&this.helper.remove(),!1},_updateVirtualBoundaries:function(e){var t,i,a,o,s,r=this.options;s={minWidth:n(r.minWidth)?r.minWidth:0,maxWidth:n(r.maxWidth)?r.maxWidth:1/0,minHeight:n(r.minHeight)?r.minHeight:0,maxHeight:n(r.maxHeight)?r.maxHeight:1/0},(this._aspectRatio||e)&&(t=s.minHeight*this.aspectRatio,a=s.minWidth/this.aspectRatio,i=s.maxHeight*this.aspectRatio,o=s.maxWidth/this.aspectRatio,t>s.minWidth&&(s.minWidth=t),a>s.minHeight&&(s.minHeight=a),i<s.maxWidth&&(s.maxWidth=i),o<s.maxHeight&&(s.maxHeight=o)),this._vBoundaries=s},_updateCache:function(e){this.options;this.offset=this.helper.offset(),n(e.left)&&(this.position.left=e.left),n(e.top)&&(this.position.top=e.top),n(e.height)&&(this.size.height=e.height),n(e.width)&&(this.size.width=e.width)},_updateRatio:function(e,t){this.options;var i=this.position,a=this.size,o=this.axis;return n(e.height)?e.width=e.height*this.aspectRatio:n(e.width)&&(e.height=e.width/this.aspectRatio),"sw"==o&&(e.left=i.left+(a.width-e.width),e.top=null),"nw"==o&&(e.top=i.top+(a.height-e.height),e.left=i.left+(a.width-e.width)),e},_respectSize:function(e,t){this.helper;var i=this._vBoundaries,a=(this._aspectRatio||t.shiftKey,this.axis),o=n(e.width)&&i.maxWidth&&i.maxWidth<e.width,s=n(e.height)&&i.maxHeight&&i.maxHeight<e.height,r=n(e.width)&&i.minWidth&&i.minWidth>e.width,l=n(e.height)&&i.minHeight&&i.minHeight>e.height;r&&(e.width=i.minWidth),l&&(e.height=i.minHeight),o&&(e.width=i.maxWidth),s&&(e.height=i.maxHeight);var c=this.originalPosition.left+this.originalSize.width,h=this.position.top+this.size.height,d=/sw|nw|w/.test(a),u=/nw|ne|n/.test(a);r&&d&&(e.left=c-i.minWidth),o&&d&&(e.left=c-i.maxWidth),l&&u&&(e.top=h-i.minHeight),s&&u&&(e.top=h-i.maxHeight);var p=!e.width&&!e.height;return p&&!e.left&&e.top?e.top=null:p&&!e.top&&e.left&&(e.left=null),e},_proportionallyResize:function(){this.options;if(this._proportionallyResizeElements.length)for(var t=this.helper||this.element,i=0;i<this._proportionallyResizeElements.length;i++){var n=this._proportionallyResizeElements[i];if(!this.borderDif){var a=[n.css("borderTopWidth"),n.css("borderRightWidth"),n.css("borderBottomWidth"),n.css("borderLeftWidth")],o=[n.css("paddingTop"),n.css("paddingRight"),n.css("paddingBottom"),n.css("paddingLeft")];this.borderDif=e.map(a,(function(e,t){var i=parseInt(e,10)||0,n=parseInt(o[t],10)||0;return i+n}))}n.css({height:t.height()-this.borderDif[0]-this.borderDif[2]||0,width:t.width()-this.borderDif[1]-this.borderDif[3]||0})}},_renderProxy:function(){var t=this.element,i=this.options;if(this.elementOffset=t.offset(),this._helper){this.helper=this.helper||e('<div style="overflow:hidden;"></div>');var n=e.ui.ie6?1:0,a=e.ui.ie6?2:-1;this.helper.addClass(this._helper).css({width:this.element.outerWidth()+a,height:this.element.outerHeight()+a,position:"absolute",left:this.elementOffset.left-n+"px",top:this.elementOffset.top-n+"px",zIndex:++i.zIndex}),this.helper.appendTo("body").disableSelection()}else this.helper=this.element},_change:{e:function(e,t,i){return{width:this.originalSize.width+t}},w:function(e,t,i){this.options;var n=this.originalSize,a=this.originalPosition;return{left:a.left+t,width:n.width-t}},n:function(e,t,i){this.options;var n=this.originalSize,a=this.originalPosition;return{top:a.top+i,height:n.height-i}},s:function(e,t,i){return{height:this.originalSize.height+i}},se:function(t,i,n){return e.extend(this._change.s.apply(this,arguments),this._change.e.apply(this,[t,i,n]))},sw:function(t,i,n){return e.extend(this._change.s.apply(this,arguments),this._change.w.apply(this,[t,i,n]))},ne:function(t,i,n){return e.extend(this._change.n.apply(this,arguments),this._change.e.apply(this,[t,i,n]))},nw:function(t,i,n){return e.extend(this._change.n.apply(this,arguments),this._change.w.apply(this,[t,i,n]))}},_propagate:function(t,i){e.ui.plugin.call(this,t,[i,this.ui()]),"resize"!=t&&this._trigger(t,i,this.ui())},plugins:{},ui:function(){return{originalElement:this.originalElement,element:this.element,helper:this.helper,position:this.position,size:this.size,originalSize:this.originalSize,originalPosition:this.originalPosition}}}),e.ui.plugin.add("resizable","alsoResize",{start:function(t,i){var n=e(this).data("resizable"),a=n.options,o=function(t){e(t).each((function(){var t=e(this);t.data("resizable-alsoresize",{width:parseInt(t.width(),10),height:parseInt(t.height(),10),left:parseInt(t.css("left"),10),top:parseInt(t.css("top"),10)})}))};"object"!=typeof a.alsoResize||a.alsoResize.parentNode?o(a.alsoResize):a.alsoResize.length?(a.alsoResize=a.alsoResize[0],o(a.alsoResize)):e.each(a.alsoResize,(function(e){o(e)}))},resize:function(t,i){var n=e(this).data("resizable"),a=n.options,o=n.originalSize,s=n.originalPosition,r={height:n.size.height-o.height||0,width:n.size.width-o.width||0,top:n.position.top-s.top||0,left:n.position.left-s.left||0},l=function(t,n){e(t).each((function(){var t=e(this),a=e(this).data("resizable-alsoresize"),o={},s=n&&n.length?n:t.parents(i.originalElement[0]).length?["width","height"]:["width","height","top","left"];e.each(s,(function(e,t){var i=(a[t]||0)+(r[t]||0);i&&i>=0&&(o[t]=i||null)})),t.css(o)}))};"object"!=typeof a.alsoResize||a.alsoResize.nodeType?l(a.alsoResize):e.each(a.alsoResize,(function(e,t){l(e,t)}))},stop:function(t,i){e(this).removeData("resizable-alsoresize")}}),e.ui.plugin.add("resizable","animate",{stop:function(t,i){var n=e(this).data("resizable"),a=n.options,o=n._proportionallyResizeElements,s=o.length&&/textarea/i.test(o[0].nodeName),r=s&&e.ui.hasScroll(o[0],"left")?0:n.sizeDiff.height,l=s?0:n.sizeDiff.width,c={width:n.size.width-l,height:n.size.height-r},h=parseInt(n.element.css("left"),10)+(n.position.left-n.originalPosition.left)||null,d=parseInt(n.element.css("top"),10)+(n.position.top-n.originalPosition.top)||null;n.element.animate(e.extend(c,d&&h?{top:d,left:h}:{}),{duration:a.animateDuration,easing:a.animateEasing,step:function(){var i={width:parseInt(n.element.css("width"),10),height:parseInt(n.element.css("height"),10),top:parseInt(n.element.css("top"),10),left:parseInt(n.element.css("left"),10)};o&&o.length&&e(o[0]).css({width:i.width,height:i.height}),n._updateCache(i),n._propagate("resize",t)}})}}),e.ui.plugin.add("resizable","containment",{start:function(t,n){var a=e(this).data("resizable"),o=a.options,s=a.element,r=o.containment,l=r instanceof e?r.get(0):/parent/.test(r)?s.parent().get(0):r;if(l)if(a.containerElement=e(l),/document/.test(r)||r==document)a.containerOffset={left:0,top:0},a.containerPosition={left:0,top:0},a.parentData={element:e(document),left:0,top:0,width:e(document).width(),height:e(document).height()||document.body.parentNode.scrollHeight};else{var c=e(l),h=[];e(["Top","Right","Left","Bottom"]).each((function(e,t){h[e]=i(c.css("padding"+t))})),a.containerOffset=c.offset(),a.containerPosition=c.position(),a.containerSize={height:c.innerHeight()-h[3],width:c.innerWidth()-h[1]};var d=a.containerOffset,u=a.containerSize.height,p=a.containerSize.width,f=e.ui.hasScroll(l,"left")?l.scrollWidth:p,g=e.ui.hasScroll(l)?l.scrollHeight:u;a.parentData={element:l,left:d.left,top:d.top,width:f,height:g}}},resize:function(t,i){var n=e(this).data("resizable"),a=n.options,o=(n.containerSize,n.containerOffset),s=(n.size,n.position),r=n._aspectRatio||t.shiftKey,l={top:0,left:0},c=n.containerElement;c[0]!=document&&/static/.test(c.css("position"))&&(l=o),s.left<(n._helper?o.left:0)&&(n.size.width=n.size.width+(n._helper?n.position.left-o.left:n.position.left-l.left),r&&(n.size.height=n.size.width/n.aspectRatio),n.position.left=a.helper?o.left:0),s.top<(n._helper?o.top:0)&&(n.size.height=n.size.height+(n._helper?n.position.top-o.top:n.position.top),r&&(n.size.width=n.size.height*n.aspectRatio),n.position.top=n._helper?o.top:0),n.offset.left=n.parentData.left+n.position.left,n.offset.top=n.parentData.top+n.position.top;var h=Math.abs((n._helper,n.offset.left-l.left+n.sizeDiff.width)),d=Math.abs((n._helper?n.offset.top-l.top:n.offset.top-o.top)+n.sizeDiff.height),u=n.containerElement.get(0)==n.element.parent().get(0),p=/relative|absolute/.test(n.containerElement.css("position"));u&&p&&(h-=n.parentData.left),h+n.size.width>=n.parentData.width&&(n.size.width=n.parentData.width-h,r&&(n.size.height=n.size.width/n.aspectRatio)),d+n.size.height>=n.parentData.height&&(n.size.height=n.parentData.height-d,r&&(n.size.width=n.size.height*n.aspectRatio))},stop:function(t,i){var n=e(this).data("resizable"),a=n.options,o=(n.position,n.containerOffset),s=n.containerPosition,r=n.containerElement,l=e(n.helper),c=l.offset(),h=l.outerWidth()-n.sizeDiff.width,d=l.outerHeight()-n.sizeDiff.height;n._helper&&!a.animate&&/relative/.test(r.css("position"))&&e(this).css({left:c.left-s.left-o.left,width:h,height:d}),n._helper&&!a.animate&&/static/.test(r.css("position"))&&e(this).css({left:c.left-s.left-o.left,width:h,height:d})}}),e.ui.plugin.add("resizable","ghost",{start:function(t,i){var n=e(this).data("resizable"),a=n.options,o=n.size;n.ghost=n.originalElement.clone(),n.ghost.css({opacity:.25,display:"block",position:"relative",height:o.height,width:o.width,margin:0,left:0,top:0}).addClass("ui-resizable-ghost").addClass("string"==typeof a.ghost?a.ghost:""),n.ghost.appendTo(n.helper)},resize:function(t,i){var n=e(this).data("resizable");n.options;n.ghost&&n.ghost.css({position:"relative",height:n.size.height,width:n.size.width})},stop:function(t,i){var n=e(this).data("resizable");n.options;n.ghost&&n.helper&&n.helper.get(0).removeChild(n.ghost.get(0))}}),e.ui.plugin.add("resizable","grid",{resize:function(t,i){var n=e(this).data("resizable"),a=n.options,o=n.size,s=n.originalSize,r=n.originalPosition,l=n.axis;a._aspectRatio||t.shiftKey;a.grid="number"==typeof a.grid?[a.grid,a.grid]:a.grid;var c=Math.round((o.width-s.width)/(a.grid[0]||1))*(a.grid[0]||1),h=Math.round((o.height-s.height)/(a.grid[1]||1))*(a.grid[1]||1);/^(se|s|e)$/.test(l)?(n.size.width=s.width+c,n.size.height=s.height+h):/^(ne)$/.test(l)?(n.size.width=s.width+c,n.size.height=s.height+h,n.position.top=r.top-h):/^(sw)$/.test(l)?(n.size.width=s.width+c,n.size.height=s.height+h,n.position.left=r.left-c):(n.size.width=s.width+c,n.size.height=s.height+h,n.position.top=r.top-h,n.position.left=r.left-c)}});var i=function(e){return parseInt(e,10)||0},n=function(e){return!isNaN(parseInt(e,10))}}(jQuery),function(e,t){e.widget("ui.selectable",e.ui.mouse,{version:"1.9.2",options:{appendTo:"body",autoRefresh:!0,distance:0,filter:"*",tolerance:"touch"},_create:function(){var t,i=this;this.element.addClass("ui-selectable"),this.dragged=!1,this.refresh=function(){t=e(i.options.filter,i.element[0]),t.addClass("ui-selectee"),t.each((function(){var t=e(this),i=t.offset();e.data(this,"selectable-item",{element:this,$element:t,left:i.left,top:i.top,right:i.left+t.outerWidth(),bottom:i.top+t.outerHeight(),startselected:!1,selected:t.hasClass("ui-selected"),selecting:t.hasClass("ui-selecting"),unselecting:t.hasClass("ui-unselecting")})}))},this.refresh(),this.selectees=t.addClass("ui-selectee"),this._mouseInit(),this.helper=e("<div class='ui-selectable-helper'></div>")},_destroy:function(){this.selectees.removeClass("ui-selectee").removeData("selectable-item"),this.element.removeClass("ui-selectable ui-selectable-disabled"),this._mouseDestroy()},_mouseStart:function(t){var i=this;if(this.opos=[t.pageX,t.pageY],!this.options.disabled){var n=this.options;this.selectees=e(n.filter,this.element[0]),this._trigger("start",t),e(n.appendTo).append(this.helper),this.helper.css({left:t.clientX,top:t.clientY,width:0,height:0}),n.autoRefresh&&this.refresh(),this.selectees.filter(".ui-selected").each((function(){var n=e.data(this,"selectable-item");n.startselected=!0,t.metaKey||t.ctrlKey||(n.$element.removeClass("ui-selected"),n.selected=!1,n.$element.addClass("ui-unselecting"),n.unselecting=!0,i._trigger("unselecting",t,{unselecting:n.element}))})),e(t.target).parents().andSelf().each((function(){var n=e.data(this,"selectable-item");if(n){var a=!t.metaKey&&!t.ctrlKey||!n.$element.hasClass("ui-selected");return n.$element.removeClass(a?"ui-unselecting":"ui-selected").addClass(a?"ui-selecting":"ui-unselecting"),n.unselecting=!a,n.selecting=a,n.selected=a,a?i._trigger("selecting",t,{selecting:n.element}):i._trigger("unselecting",t,{unselecting:n.element}),!1}}))}},_mouseDrag:function(t){var i=this;if(this.dragged=!0,!this.options.disabled){var n=this.options,a=this.opos[0],o=this.opos[1],s=t.pageX,r=t.pageY;if(a>s){var l=s;s=a,a=l}if(o>r){l=r;r=o,o=l}return this.helper.css({left:a,top:o,width:s-a,height:r-o}),this.selectees.each((function(){var l=e.data(this,"selectable-item");if(l&&l.element!=i.element[0]){var c=!1;"touch"==n.tolerance?c=!(l.left>s||l.right<a||l.top>r||l.bottom<o):"fit"==n.tolerance&&(c=l.left>a&&l.right<s&&l.top>o&&l.bottom<r),c?(l.selected&&(l.$element.removeClass("ui-selected"),l.selected=!1),l.unselecting&&(l.$element.removeClass("ui-unselecting"),l.unselecting=!1),l.selecting||(l.$element.addClass("ui-selecting"),l.selecting=!0,i._trigger("selecting",t,{selecting:l.element}))):(l.selecting&&((t.metaKey||t.ctrlKey)&&l.startselected?(l.$element.removeClass("ui-selecting"),l.selecting=!1,l.$element.addClass("ui-selected"),l.selected=!0):(l.$element.removeClass("ui-selecting"),l.selecting=!1,l.startselected&&(l.$element.addClass("ui-unselecting"),l.unselecting=!0),i._trigger("unselecting",t,{unselecting:l.element}))),l.selected&&(t.metaKey||t.ctrlKey||l.startselected||(l.$element.removeClass("ui-selected"),l.selected=!1,l.$element.addClass("ui-unselecting"),l.unselecting=!0,i._trigger("unselecting",t,{unselecting:l.element}))))}})),!1}},_mouseStop:function(t){var i=this;this.dragged=!1;this.options;return e(".ui-unselecting",this.element[0]).each((function(){var n=e.data(this,"selectable-item");n.$element.removeClass("ui-unselecting"),n.unselecting=!1,n.startselected=!1,i._trigger("unselected",t,{unselected:n.element})})),e(".ui-selecting",this.element[0]).each((function(){var n=e.data(this,"selectable-item");n.$element.removeClass("ui-selecting").addClass("ui-selected"),n.selecting=!1,n.selected=!0,n.startselected=!0,i._trigger("selected",t,{selected:n.element})})),this._trigger("stop",t),this.helper.remove(),!1}})}(jQuery),function(e,t){e.widget("ui.sortable",e.ui.mouse,{version:"1.9.2",widgetEventPrefix:"sort",ready:!1,options:{appendTo:"parent",axis:!1,connectWith:!1,containment:!1,cursor:"auto",cursorAt:!1,dropOnEmpty:!0,forcePlaceholderSize:!1,forceHelperSize:!1,grid:!1,handle:!1,helper:"original",items:"> *",opacity:!1,placeholder:!1,revert:!1,scroll:!0,scrollSensitivity:20,scrollSpeed:20,scope:"default",tolerance:"intersect",zIndex:1e3},_create:function(){var e=this.options;this.containerCache={},this.element.addClass("ui-sortable"),this.refresh(),this.floating=!!this.items.length&&("x"===e.axis||/left|right/.test(this.items[0].item.css("float"))||/inline|table-cell/.test(this.items[0].item.css("display"))),this.offset=this.element.offset(),this._mouseInit(),this.ready=!0},_destroy:function(){this.element.removeClass("ui-sortable ui-sortable-disabled"),this._mouseDestroy();for(var e=this.items.length-1;e>=0;e--)this.items[e].item.removeData(this.widgetName+"-item");return this},_setOption:function(t,i){"disabled"===t?(this.options[t]=i,this.widget().toggleClass("ui-sortable-disabled",!!i)):e.Widget.prototype._setOption.apply(this,arguments)},_mouseCapture:function(t,i){var n=this;if(this.reverting)return!1;if(this.options.disabled||"static"==this.options.type)return!1;this._refreshItems(t);var a=null;e(t.target).parents().each((function(){if(e.data(this,n.widgetName+"-item")==n)return a=e(this),!1}));if(e.data(t.target,n.widgetName+"-item")==n&&(a=e(t.target)),!a)return!1;if(this.options.handle&&!i){var o=!1;if(e(this.options.handle,a).find("*").andSelf().each((function(){this==t.target&&(o=!0)})),!o)return!1}return this.currentItem=a,this._removeCurrentsFromItems(),!0},_mouseStart:function(t,i,n){var a=this.options;if(this.currentContainer=this,this.refreshPositions(),this.helper=this._createHelper(t),this._cacheHelperProportions(),this._cacheMargins(),this.scrollParent=this.helper.scrollParent(),this.offset=this.currentItem.offset(),this.offset={top:this.offset.top-this.margins.top,left:this.offset.left-this.margins.left},e.extend(this.offset,{click:{left:t.pageX-this.offset.left,top:t.pageY-this.offset.top},parent:this._getParentOffset(),relative:this._getRelativeOffset()}),this.helper.css("position","absolute"),this.cssPosition=this.helper.css("position"),this.originalPosition=this._generatePosition(t),this.originalPageX=t.pageX,this.originalPageY=t.pageY,a.cursorAt&&this._adjustOffsetFromHelper(a.cursorAt),this.domPosition={prev:this.currentItem.prev()[0],parent:this.currentItem.parent()[0]},this.helper[0]!=this.currentItem[0]&&this.currentItem.hide(),this._createPlaceholder(),a.containment&&this._setContainment(),a.cursor&&(e("body").css("cursor")&&(this._storedCursor=e("body").css("cursor")),e("body").css("cursor",a.cursor)),a.opacity&&(this.helper.css("opacity")&&(this._storedOpacity=this.helper.css("opacity")),this.helper.css("opacity",a.opacity)),a.zIndex&&(this.helper.css("zIndex")&&(this._storedZIndex=this.helper.css("zIndex")),this.helper.css("zIndex",a.zIndex)),this.scrollParent[0]!=document&&"HTML"!=this.scrollParent[0].tagName&&(this.overflowOffset=this.scrollParent.offset()),this._trigger("start",t,this._uiHash()),this._preserveHelperProportions||this._cacheHelperProportions(),!n)for(var o=this.containers.length-1;o>=0;o--)this.containers[o]._trigger("activate",t,this._uiHash(this));return e.ui.ddmanager&&(e.ui.ddmanager.current=this),e.ui.ddmanager&&!a.dropBehaviour&&e.ui.ddmanager.prepareOffsets(this,t),this.dragging=!0,this.helper.addClass("ui-sortable-helper"),this._mouseDrag(t),!0},_mouseDrag:function(t){if(this.position=this._generatePosition(t),this.positionAbs=this._convertPositionTo("absolute"),this.lastPositionAbs||(this.lastPositionAbs=this.positionAbs),this.options.scroll){var i=this.options,n=!1;this.scrollParent[0]!=document&&"HTML"!=this.scrollParent[0].tagName?(this.overflowOffset.top+this.scrollParent[0].offsetHeight-t.pageY<i.scrollSensitivity?this.scrollParent[0].scrollTop=n=this.scrollParent[0].scrollTop+i.scrollSpeed:t.pageY-this.overflowOffset.top<i.scrollSensitivity&&(this.scrollParent[0].scrollTop=n=this.scrollParent[0].scrollTop-i.scrollSpeed),this.overflowOffset.left+this.scrollParent[0].offsetWidth-t.pageX<i.scrollSensitivity?this.scrollParent[0].scrollLeft=n=this.scrollParent[0].scrollLeft+i.scrollSpeed:t.pageX-this.overflowOffset.left<i.scrollSensitivity&&(this.scrollParent[0].scrollLeft=n=this.scrollParent[0].scrollLeft-i.scrollSpeed)):(t.pageY-e(document).scrollTop()<i.scrollSensitivity?n=e(document).scrollTop(e(document).scrollTop()-i.scrollSpeed):e(window).height()-(t.pageY-e(document).scrollTop())<i.scrollSensitivity&&(n=e(document).scrollTop(e(document).scrollTop()+i.scrollSpeed)),t.pageX-e(document).scrollLeft()<i.scrollSensitivity?n=e(document).scrollLeft(e(document).scrollLeft()-i.scrollSpeed):e(window).width()-(t.pageX-e(document).scrollLeft())<i.scrollSensitivity&&(n=e(document).scrollLeft(e(document).scrollLeft()+i.scrollSpeed))),!1!==n&&e.ui.ddmanager&&!i.dropBehaviour&&e.ui.ddmanager.prepareOffsets(this,t)}this.positionAbs=this._convertPositionTo("absolute"),this.options.axis&&"y"==this.options.axis||(this.helper[0].style.left=this.position.left+"px"),this.options.axis&&"x"==this.options.axis||(this.helper[0].style.top=this.position.top+"px");for(var a=this.items.length-1;a>=0;a--){var o=this.items[a],s=o.item[0],r=this._intersectsWithPointer(o);if(r&&(o.instance===this.currentContainer&&s!=this.currentItem[0]&&this.placeholder[1==r?"next":"prev"]()[0]!=s&&!e.contains(this.placeholder[0],s)&&("semi-dynamic"!=this.options.type||!e.contains(this.element[0],s)))){if(this.direction=1==r?"down":"up","pointer"!=this.options.tolerance&&!this._intersectsWithSides(o))break;this._rearrange(t,o),this._trigger("change",t,this._uiHash());break}}return this._contactContainers(t),e.ui.ddmanager&&e.ui.ddmanager.drag(this,t),this._trigger("sort",t,this._uiHash()),this.lastPositionAbs=this.positionAbs,!1},_mouseStop:function(t,i){if(t){if(e.ui.ddmanager&&!this.options.dropBehaviour&&e.ui.ddmanager.drop(this,t),this.options.revert){var n=this,a=this.placeholder.offset();this.reverting=!0,e(this.helper).animate({left:a.left-this.offset.parent.left-this.margins.left+(this.offsetParent[0]==document.body?0:this.offsetParent[0].scrollLeft),top:a.top-this.offset.parent.top-this.margins.top+(this.offsetParent[0]==document.body?0:this.offsetParent[0].scrollTop)},parseInt(this.options.revert,10)||500,(function(){n._clear(t)}))}else this._clear(t,i);return!1}},cancel:function(){if(this.dragging){this._mouseUp({target:null}),"original"==this.options.helper?this.currentItem.css(this._storedCSS).removeClass("ui-sortable-helper"):this.currentItem.show();for(var t=this.containers.length-1;t>=0;t--)this.containers[t]._trigger("deactivate",null,this._uiHash(this)),this.containers[t].containerCache.over&&(this.containers[t]._trigger("out",null,this._uiHash(this)),this.containers[t].containerCache.over=0)}return this.placeholder&&(this.placeholder[0].parentNode&&this.placeholder[0].parentNode.removeChild(this.placeholder[0]),"original"!=this.options.helper&&this.helper&&this.helper[0].parentNode&&this.helper.remove(),e.extend(this,{helper:null,dragging:!1,reverting:!1,_noFinalSort:null}),this.domPosition.prev?e(this.domPosition.prev).after(this.currentItem):e(this.domPosition.parent).prepend(this.currentItem)),this},serialize:function(t){var i=this._getItemsAsjQuery(t&&t.connected),n=[];return t=t||{},e(i).each((function(){var i=(e(t.item||this).attr(t.attribute||"id")||"").match(t.expression||/(.+)[-=_](.+)/);i&&n.push((t.key||i[1]+"[]")+"="+(t.key&&t.expression?i[1]:i[2]))})),!n.length&&t.key&&n.push(t.key+"="),n.join("&")},toArray:function(t){var i=this._getItemsAsjQuery(t&&t.connected),n=[];return t=t||{},i.each((function(){n.push(e(t.item||this).attr(t.attribute||"id")||"")})),n},_intersectsWith:function(e){var t=this.positionAbs.left,i=t+this.helperProportions.width,n=this.positionAbs.top,a=n+this.helperProportions.height,o=e.left,s=o+e.width,r=e.top,l=r+e.height,c=this.offset.click.top,h=this.offset.click.left,d=n+c>r&&n+c<l&&t+h>o&&t+h<s;return"pointer"==this.options.tolerance||this.options.forcePointerForContainers||"pointer"!=this.options.tolerance&&this.helperProportions[this.floating?"width":"height"]>e[this.floating?"width":"height"]?d:o<t+this.helperProportions.width/2&&i-this.helperProportions.width/2<s&&r<n+this.helperProportions.height/2&&a-this.helperProportions.height/2<l},_intersectsWithPointer:function(t){var i="x"===this.options.axis||e.ui.isOverAxis(this.positionAbs.top+this.offset.click.top,t.top,t.height),n="y"===this.options.axis||e.ui.isOverAxis(this.positionAbs.left+this.offset.click.left,t.left,t.width),a=i&&n,o=this._getDragVerticalDirection(),s=this._getDragHorizontalDirection();return!!a&&(this.floating?s&&"right"==s||"down"==o?2:1:o&&("down"==o?2:1))},_intersectsWithSides:function(t){var i=e.ui.isOverAxis(this.positionAbs.top+this.offset.click.top,t.top+t.height/2,t.height),n=e.ui.isOverAxis(this.positionAbs.left+this.offset.click.left,t.left+t.width/2,t.width),a=this._getDragVerticalDirection(),o=this._getDragHorizontalDirection();return this.floating&&o?"right"==o&&n||"left"==o&&!n:a&&("down"==a&&i||"up"==a&&!i)},_getDragVerticalDirection:function(){var e=this.positionAbs.top-this.lastPositionAbs.top;return 0!=e&&(e>0?"down":"up")},_getDragHorizontalDirection:function(){var e=this.positionAbs.left-this.lastPositionAbs.left;return 0!=e&&(e>0?"right":"left")},refresh:function(e){return this._refreshItems(e),this.refreshPositions(),this},_connectWith:function(){var e=this.options;return e.connectWith.constructor==String?[e.connectWith]:e.connectWith},_getItemsAsjQuery:function(t){var i=[],n=[],a=this._connectWith();if(a&&t)for(var o=a.length-1;o>=0;o--)for(var s=e(a[o]),r=s.length-1;r>=0;r--){var l=e.data(s[r],this.widgetName);l&&l!=this&&!l.options.disabled&&n.push([e.isFunction(l.options.items)?l.options.items.call(l.element):e(l.options.items,l.element).not(".ui-sortable-helper").not(".ui-sortable-placeholder"),l])}n.push([e.isFunction(this.options.items)?this.options.items.call(this.element,null,{options:this.options,item:this.currentItem}):e(this.options.items,this.element).not(".ui-sortable-helper").not(".ui-sortable-placeholder"),this]);for(o=n.length-1;o>=0;o--)n[o][0].each((function(){i.push(this)}));return e(i)},_removeCurrentsFromItems:function(){var t=this.currentItem.find(":data("+this.widgetName+"-item)");this.items=e.grep(this.items,(function(e){for(var i=0;i<t.length;i++)if(t[i]==e.item[0])return!1;return!0}))},_refreshItems:function(t){this.items=[],this.containers=[this];var i=this.items,n=[[e.isFunction(this.options.items)?this.options.items.call(this.element[0],t,{item:this.currentItem}):e(this.options.items,this.element),this]],a=this._connectWith();if(a&&this.ready)for(var o=a.length-1;o>=0;o--)for(var s=e(a[o]),r=s.length-1;r>=0;r--){var l=e.data(s[r],this.widgetName);l&&l!=this&&!l.options.disabled&&(n.push([e.isFunction(l.options.items)?l.options.items.call(l.element[0],t,{item:this.currentItem}):e(l.options.items,l.element),l]),this.containers.push(l))}for(o=n.length-1;o>=0;o--)for(var c=n[o][1],h=n[o][0],d=(r=0,h.length);r<d;r++){var u=e(h[r]);u.data(this.widgetName+"-item",c),i.push({item:u,instance:c,width:0,height:0,left:0,top:0})}},refreshPositions:function(t){this.offsetParent&&this.helper&&(this.offset.parent=this._getParentOffset());for(var i=this.items.length-1;i>=0;i--){var n=this.items[i];if(n.instance==this.currentContainer||!this.currentContainer||n.item[0]==this.currentItem[0]){var a=this.options.toleranceElement?e(this.options.toleranceElement,n.item):n.item;t||(n.width=a.outerWidth(),n.height=a.outerHeight());var o=a.offset();n.left=o.left,n.top=o.top}}if(this.options.custom&&this.options.custom.refreshContainers)this.options.custom.refreshContainers.call(this);else for(i=this.containers.length-1;i>=0;i--){o=this.containers[i].element.offset();this.containers[i].containerCache.left=o.left,this.containers[i].containerCache.top=o.top,this.containers[i].containerCache.width=this.containers[i].element.outerWidth(),this.containers[i].containerCache.height=this.containers[i].element.outerHeight()}return this},_createPlaceholder:function(t){t=t||this;var i=t.options;if(!i.placeholder||i.placeholder.constructor==String){var n=i.placeholder;i.placeholder={element:function(){var i=e(document.createElement(t.currentItem[0].nodeName)).addClass(n||t.currentItem[0].className+" ui-sortable-placeholder").removeClass("ui-sortable-helper")[0];return n||(i.style.visibility="hidden"),i},update:function(e,a){n&&!i.forcePlaceholderSize||(a.height()||a.height(t.currentItem.innerHeight()-parseInt(t.currentItem.css("paddingTop")||0,10)-parseInt(t.currentItem.css("paddingBottom")||0,10)),a.width()||a.width(t.currentItem.innerWidth()-parseInt(t.currentItem.css("paddingLeft")||0,10)-parseInt(t.currentItem.css("paddingRight")||0,10)))}}}t.placeholder=e(i.placeholder.element.call(t.element,t.currentItem)),t.currentItem.after(t.placeholder),i.placeholder.update(t,t.placeholder)},_contactContainers:function(t){for(var i=null,n=null,a=this.containers.length-1;a>=0;a--)if(!e.contains(this.currentItem[0],this.containers[a].element[0]))if(this._intersectsWith(this.containers[a].containerCache)){if(i&&e.contains(this.containers[a].element[0],i.element[0]))continue;i=this.containers[a],n=a}else this.containers[a].containerCache.over&&(this.containers[a]._trigger("out",t,this._uiHash(this)),this.containers[a].containerCache.over=0);if(i)if(1===this.containers.length)this.containers[n]._trigger("over",t,this._uiHash(this)),this.containers[n].containerCache.over=1;else{for(var o=1e4,s=null,r=this.containers[n].floating?"left":"top",l=this.containers[n].floating?"width":"height",c=this.positionAbs[r]+this.offset.click[r],h=this.items.length-1;h>=0;h--)if(e.contains(this.containers[n].element[0],this.items[h].item[0])&&this.items[h].item[0]!=this.currentItem[0]){var d=this.items[h].item.offset()[r],u=!1;Math.abs(d-c)>Math.abs(d+this.items[h][l]-c)&&(u=!0,d+=this.items[h][l]),Math.abs(d-c)<o&&(o=Math.abs(d-c),s=this.items[h],this.direction=u?"up":"down")}if(!s&&!this.options.dropOnEmpty)return;this.currentContainer=this.containers[n],s?this._rearrange(t,s,null,!0):this._rearrange(t,null,this.containers[n].element,!0),this._trigger("change",t,this._uiHash()),this.containers[n]._trigger("change",t,this._uiHash(this)),this.options.placeholder.update(this.currentContainer,this.placeholder),this.containers[n]._trigger("over",t,this._uiHash(this)),this.containers[n].containerCache.over=1}},_createHelper:function(t){var i=this.options,n=e.isFunction(i.helper)?e(i.helper.apply(this.element[0],[t,this.currentItem])):"clone"==i.helper?this.currentItem.clone():this.currentItem;return n.parents("body").length||e("parent"!=i.appendTo?i.appendTo:this.currentItem[0].parentNode)[0].appendChild(n[0]),n[0]==this.currentItem[0]&&(this._storedCSS={width:this.currentItem[0].style.width,height:this.currentItem[0].style.height,position:this.currentItem.css("position"),top:this.currentItem.css("top"),left:this.currentItem.css("left")}),(""==n[0].style.width||i.forceHelperSize)&&n.width(this.currentItem.width()),(""==n[0].style.height||i.forceHelperSize)&&n.height(this.currentItem.height()),n},_adjustOffsetFromHelper:function(t){"string"==typeof t&&(t=t.split(" ")),e.isArray(t)&&(t={left:+t[0],top:+t[1]||0}),"left"in t&&(this.offset.click.left=t.left+this.margins.left),"right"in t&&(this.offset.click.left=this.helperProportions.width-t.right+this.margins.left),"top"in t&&(this.offset.click.top=t.top+this.margins.top),"bottom"in t&&(this.offset.click.top=this.helperProportions.height-t.bottom+this.margins.top)},_getParentOffset:function(){this.offsetParent=this.helper.offsetParent();var t=this.offsetParent.offset();return"absolute"==this.cssPosition&&this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])&&(t.left+=this.scrollParent.scrollLeft(),t.top+=this.scrollParent.scrollTop()),(this.offsetParent[0]==document.body||this.offsetParent[0].tagName&&"html"==this.offsetParent[0].tagName.toLowerCase()&&e.ui.ie)&&(t={top:0,left:0}),{top:t.top+(parseInt(this.offsetParent.css("borderTopWidth"),10)||0),left:t.left+(parseInt(this.offsetParent.css("borderLeftWidth"),10)||0)}},_getRelativeOffset:function(){if("relative"==this.cssPosition){var e=this.currentItem.position();return{top:e.top-(parseInt(this.helper.css("top"),10)||0)+this.scrollParent.scrollTop(),left:e.left-(parseInt(this.helper.css("left"),10)||0)+this.scrollParent.scrollLeft()}}return{top:0,left:0}},_cacheMargins:function(){this.margins={left:parseInt(this.currentItem.css("marginLeft"),10)||0,top:parseInt(this.currentItem.css("marginTop"),10)||0}},_cacheHelperProportions:function(){this.helperProportions={width:this.helper.outerWidth(),height:this.helper.outerHeight()}},_setContainment:function(){var t=this.options;if("parent"==t.containment&&(t.containment=this.helper[0].parentNode),"document"!=t.containment&&"window"!=t.containment||(this.containment=[0-this.offset.relative.left-this.offset.parent.left,0-this.offset.relative.top-this.offset.parent.top,e("document"==t.containment?document:window).width()-this.helperProportions.width-this.margins.left,(e("document"==t.containment?document:window).height()||document.body.parentNode.scrollHeight)-this.helperProportions.height-this.margins.top]),!/^(document|window|parent)$/.test(t.containment)){var i=e(t.containment)[0],n=e(t.containment).offset(),a="hidden"!=e(i).css("overflow");this.containment=[n.left+(parseInt(e(i).css("borderLeftWidth"),10)||0)+(parseInt(e(i).css("paddingLeft"),10)||0)-this.margins.left,n.top+(parseInt(e(i).css("borderTopWidth"),10)||0)+(parseInt(e(i).css("paddingTop"),10)||0)-this.margins.top,n.left+(a?Math.max(i.scrollWidth,i.offsetWidth):i.offsetWidth)-(parseInt(e(i).css("borderLeftWidth"),10)||0)-(parseInt(e(i).css("paddingRight"),10)||0)-this.helperProportions.width-this.margins.left,n.top+(a?Math.max(i.scrollHeight,i.offsetHeight):i.offsetHeight)-(parseInt(e(i).css("borderTopWidth"),10)||0)-(parseInt(e(i).css("paddingBottom"),10)||0)-this.helperProportions.height-this.margins.top]}},_convertPositionTo:function(t,i){i||(i=this.position);var n="absolute"==t?1:-1,a=(this.options,"absolute"!=this.cssPosition||this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent),o=/(html|body)/i.test(a[0].tagName);return{top:i.top+this.offset.relative.top*n+this.offset.parent.top*n-("fixed"==this.cssPosition?-this.scrollParent.scrollTop():o?0:a.scrollTop())*n,left:i.left+this.offset.relative.left*n+this.offset.parent.left*n-("fixed"==this.cssPosition?-this.scrollParent.scrollLeft():o?0:a.scrollLeft())*n}},_generatePosition:function(t){var i=this.options,n="absolute"!=this.cssPosition||this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent,a=/(html|body)/i.test(n[0].tagName);"relative"!=this.cssPosition||this.scrollParent[0]!=document&&this.scrollParent[0]!=this.offsetParent[0]||(this.offset.relative=this._getRelativeOffset());var o=t.pageX,s=t.pageY;if(this.originalPosition&&(this.containment&&(t.pageX-this.offset.click.left<this.containment[0]&&(o=this.containment[0]+this.offset.click.left),t.pageY-this.offset.click.top<this.containment[1]&&(s=this.containment[1]+this.offset.click.top),t.pageX-this.offset.click.left>this.containment[2]&&(o=this.containment[2]+this.offset.click.left),t.pageY-this.offset.click.top>this.containment[3]&&(s=this.containment[3]+this.offset.click.top)),i.grid)){var r=this.originalPageY+Math.round((s-this.originalPageY)/i.grid[1])*i.grid[1];s=this.containment&&(r-this.offset.click.top<this.containment[1]||r-this.offset.click.top>this.containment[3])?r-this.offset.click.top<this.containment[1]?r+i.grid[1]:r-i.grid[1]:r;var l=this.originalPageX+Math.round((o-this.originalPageX)/i.grid[0])*i.grid[0];o=this.containment&&(l-this.offset.click.left<this.containment[0]||l-this.offset.click.left>this.containment[2])?l-this.offset.click.left<this.containment[0]?l+i.grid[0]:l-i.grid[0]:l}return{top:s-this.offset.click.top-this.offset.relative.top-this.offset.parent.top+("fixed"==this.cssPosition?-this.scrollParent.scrollTop():a?0:n.scrollTop()),left:o-this.offset.click.left-this.offset.relative.left-this.offset.parent.left+("fixed"==this.cssPosition?-this.scrollParent.scrollLeft():a?0:n.scrollLeft())}},_rearrange:function(e,t,i,n){i?i[0].appendChild(this.placeholder[0]):t.item[0].parentNode.insertBefore(this.placeholder[0],"down"==this.direction?t.item[0]:t.item[0].nextSibling),this.counter=this.counter?++this.counter:1;var a=this.counter;this._delay((function(){a==this.counter&&this.refreshPositions(!n)}))},_clear:function(t,i){this.reverting=!1;var n=[];if(!this._noFinalSort&&this.currentItem.parent().length&&this.placeholder.before(this.currentItem),this._noFinalSort=null,this.helper[0]==this.currentItem[0]){for(var a in this._storedCSS)"auto"!=this._storedCSS[a]&&"static"!=this._storedCSS[a]||(this._storedCSS[a]="");this.currentItem.css(this._storedCSS).removeClass("ui-sortable-helper")}else this.currentItem.show();this.fromOutside&&!i&&n.push((function(e){this._trigger("receive",e,this._uiHash(this.fromOutside))})),!this.fromOutside&&this.domPosition.prev==this.currentItem.prev().not(".ui-sortable-helper")[0]&&this.domPosition.parent==this.currentItem.parent()[0]||i||n.push((function(e){this._trigger("update",e,this._uiHash())})),this!==this.currentContainer&&(i||(n.push((function(e){this._trigger("remove",e,this._uiHash())})),n.push(function(e){return function(t){e._trigger("receive",t,this._uiHash(this))}}.call(this,this.currentContainer)),n.push(function(e){return function(t){e._trigger("update",t,this._uiHash(this))}}.call(this,this.currentContainer))));for(a=this.containers.length-1;a>=0;a--)i||n.push(function(e){return function(t){e._trigger("deactivate",t,this._uiHash(this))}}.call(this,this.containers[a])),this.containers[a].containerCache.over&&(n.push(function(e){return function(t){e._trigger("out",t,this._uiHash(this))}}.call(this,this.containers[a])),this.containers[a].containerCache.over=0);if(this._storedCursor&&e("body").css("cursor",this._storedCursor),this._storedOpacity&&this.helper.css("opacity",this._storedOpacity),this._storedZIndex&&this.helper.css("zIndex","auto"==this._storedZIndex?"":this._storedZIndex),this.dragging=!1,this.cancelHelperRemoval){if(!i){this._trigger("beforeStop",t,this._uiHash());for(a=0;a<n.length;a++)n[a].call(this,t);this._trigger("stop",t,this._uiHash())}return this.fromOutside=!1,!1}if(i||this._trigger("beforeStop",t,this._uiHash()),this.placeholder[0].parentNode.removeChild(this.placeholder[0]),this.helper[0]!=this.currentItem[0]&&this.helper.remove(),this.helper=null,!i){for(a=0;a<n.length;a++)n[a].call(this,t);this._trigger("stop",t,this._uiHash())}return this.fromOutside=!1,!0},_trigger:function(){!1===e.Widget.prototype._trigger.apply(this,arguments)&&this.cancel()},_uiHash:function(t){var i=t||this;return{helper:i.helper,placeholder:i.placeholder||e([]),position:i.position,originalPosition:i.originalPosition,offset:i.positionAbs,item:i.currentItem,sender:t?t.element:null}}})}(jQuery),jQuery.effects||function(e,t){var i=!1!==e.uiBackCompat,n="ui-effects-";e.effects={effect:{}},
/*!
 * jQuery Color Animations v2.0.0
 * http://jquery.com/
 *
 * Copyright 2012 jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 *
 * Date: Mon Aug 13 13:41:02 2012 -0500
 */
function(t,i){var n,a="backgroundColor borderBottomColor borderLeftColor borderRightColor borderTopColor color columnRuleColor outlineColor textDecorationColor textEmphasisColor".split(" "),o=/^([\-+])=\s*(\d+\.?\d*)/,s=[{re:/rgba?\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*(?:,\s*(\d+(?:\.\d+)?)\s*)?\)/,parse:function(e){return[e[1],e[2],e[3],e[4]]}},{re:/rgba?\(\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*(?:,\s*(\d+(?:\.\d+)?)\s*)?\)/,parse:function(e){return[2.55*e[1],2.55*e[2],2.55*e[3],e[4]]}},{re:/#([a-f0-9]{2})([a-f0-9]{2})([a-f0-9]{2})/,parse:function(e){return[parseInt(e[1],16),parseInt(e[2],16),parseInt(e[3],16)]}},{re:/#([a-f0-9])([a-f0-9])([a-f0-9])/,parse:function(e){return[parseInt(e[1]+e[1],16),parseInt(e[2]+e[2],16),parseInt(e[3]+e[3],16)]}},{re:/hsla?\(\s*(\d+(?:\.\d+)?)\s*,\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*(?:,\s*(\d+(?:\.\d+)?)\s*)?\)/,space:"hsla",parse:function(e){return[e[1],e[2]/100,e[3]/100,e[4]]}}],r=t.Color=function(e,i,n,a){return new t.Color.fn.parse(e,i,n,a)},l={rgba:{props:{red:{idx:0,type:"byte"},green:{idx:1,type:"byte"},blue:{idx:2,type:"byte"}}},hsla:{props:{hue:{idx:0,type:"degrees"},saturation:{idx:1,type:"percent"},lightness:{idx:2,type:"percent"}}}},c={byte:{floor:!0,max:255},percent:{max:1},degrees:{mod:360,floor:!0}},h=r.support={},d=t("<p>")[0],u=t.each;function p(e,t,i){var n=c[t.type]||{};return null==e?i||!t.def?null:t.def:(e=n.floor?~~e:parseFloat(e),isNaN(e)?t.def:n.mod?(e+n.mod)%n.mod:0>e?0:n.max<e?n.max:e)}function f(e){var i=r(),a=i._rgba=[];return e=e.toLowerCase(),u(s,(function(t,n){var o,s=n.re.exec(e),r=s&&n.parse(s),c=n.space||"rgba";if(r)return o=i[c](r),i[l[c].cache]=o[l[c].cache],a=i._rgba=o._rgba,!1})),a.length?("0,0,0,0"===a.join()&&t.extend(a,n.transparent),i):n[e]}function g(e,t,i){return i=(i+1)%1,6*i<1?e+(t-e)*i*6:2*i<1?t:3*i<2?e+(t-e)*(2/3-i)*6:e}d.style.cssText="background-color:rgba(1,1,1,.5)",h.rgba=d.style.backgroundColor.indexOf("rgba")>-1,u(l,(function(e,t){t.cache="_"+e,t.props.alpha={idx:3,type:"percent",def:1}})),r.fn=t.extend(r.prototype,{parse:function(a,o,s,c){if(a===i)return this._rgba=[null,null,null,null],this;(a.jquery||a.nodeType)&&(a=t(a).css(o),o=i);var h=this,d=t.type(a),g=this._rgba=[];return o!==i&&(a=[a,o,s,c],d="array"),"string"===d?this.parse(f(a)||n._default):"array"===d?(u(l.rgba.props,(function(e,t){g[t.idx]=p(a[t.idx],t)})),this):"object"===d?(u(l,a instanceof r?function(e,t){a[t.cache]&&(h[t.cache]=a[t.cache].slice())}:function(t,i){var n=i.cache;u(i.props,(function(e,t){if(!h[n]&&i.to){if("alpha"===e||null==a[e])return;h[n]=i.to(h._rgba)}h[n][t.idx]=p(a[e],t,!0)})),h[n]&&e.inArray(null,h[n].slice(0,3))<0&&(h[n][3]=1,i.from&&(h._rgba=i.from(h[n])))}),this):void 0},is:function(e){var t=r(e),i=!0,n=this;return u(l,(function(e,a){var o,s=t[a.cache];return s&&(o=n[a.cache]||a.to&&a.to(n._rgba)||[],u(a.props,(function(e,t){if(null!=s[t.idx])return i=s[t.idx]===o[t.idx],i}))),i})),i},_space:function(){var e=[],t=this;return u(l,(function(i,n){t[n.cache]&&e.push(i)})),e.pop()},transition:function(e,t){var i=r(e),n=i._space(),a=l[n],o=0===this.alpha()?r("transparent"):this,s=o[a.cache]||a.to(o._rgba),h=s.slice();return i=i[a.cache],u(a.props,(function(e,n){var a=n.idx,o=s[a],r=i[a],l=c[n.type]||{};null!==r&&(null===o?h[a]=r:(l.mod&&(r-o>l.mod/2?o+=l.mod:o-r>l.mod/2&&(o-=l.mod)),h[a]=p((r-o)*t+o,n)))})),this[n](h)},blend:function(e){if(1===this._rgba[3])return this;var i=this._rgba.slice(),n=i.pop(),a=r(e)._rgba;return r(t.map(i,(function(e,t){return(1-n)*a[t]+n*e})))},toRgbaString:function(){var e="rgba(",i=t.map(this._rgba,(function(e,t){return null==e?t>2?1:0:e}));return 1===i[3]&&(i.pop(),e="rgb("),e+i.join()+")"},toHslaString:function(){var e="hsla(",i=t.map(this.hsla(),(function(e,t){return null==e&&(e=t>2?1:0),t&&t<3&&(e=Math.round(100*e)+"%"),e}));return 1===i[3]&&(i.pop(),e="hsl("),e+i.join()+")"},toHexString:function(e){var i=this._rgba.slice(),n=i.pop();return e&&i.push(~~(255*n)),"#"+t.map(i,(function(e){return e=(e||0).toString(16),1===e.length?"0"+e:e})).join("")},toString:function(){return 0===this._rgba[3]?"transparent":this.toRgbaString()}}),r.fn.parse.prototype=r.fn,l.hsla.to=function(e){if(null==e[0]||null==e[1]||null==e[2])return[null,null,null,e[3]];var t,i,n=e[0]/255,a=e[1]/255,o=e[2]/255,s=e[3],r=Math.max(n,a,o),l=Math.min(n,a,o),c=r-l,h=r+l,d=.5*h;return t=l===r?0:n===r?60*(a-o)/c+360:a===r?60*(o-n)/c+120:60*(n-a)/c+240,i=0===d||1===d?d:d<=.5?c/h:c/(2-h),[Math.round(t)%360,i,d,null==s?1:s]},l.hsla.from=function(e){if(null==e[0]||null==e[1]||null==e[2])return[null,null,null,e[3]];var t=e[0]/360,i=e[1],n=e[2],a=e[3],o=n<=.5?n*(1+i):n+i-n*i,s=2*n-o;return[Math.round(255*g(s,o,t+1/3)),Math.round(255*g(s,o,t)),Math.round(255*g(s,o,t-1/3)),a]},u(l,(function(e,n){var a=n.props,s=n.cache,l=n.to,c=n.from;r.fn[e]=function(e){if(l&&!this[s]&&(this[s]=l(this._rgba)),e===i)return this[s].slice();var n,o=t.type(e),h="array"===o||"object"===o?e:arguments,d=this[s].slice();return u(a,(function(e,t){var i=h["object"===o?e:t.idx];null==i&&(i=d[t.idx]),d[t.idx]=p(i,t)})),c?(n=r(c(d)),n[s]=d,n):r(d)},u(a,(function(i,n){r.fn[i]||(r.fn[i]=function(a){var s,r=t.type(a),l="alpha"===i?this._hsla?"hsla":"rgba":e,c=this[l](),h=c[n.idx];return"undefined"===r?h:("function"===r&&(a=a.call(this,h),r=t.type(a)),null==a&&n.empty?this:("string"===r&&(s=o.exec(a),s&&(a=h+parseFloat(s[2])*("+"===s[1]?1:-1))),c[n.idx]=a,this[l](c)))})}))})),u(a,(function(e,i){t.cssHooks[i]={set:function(e,n){var a,o,s="";if("string"!==t.type(n)||(a=f(n))){if(n=r(a||n),!h.rgba&&1!==n._rgba[3]){o="backgroundColor"===i?e.parentNode:e;while((""===s||"transparent"===s)&&o&&o.style)try{s=t.css(o,"backgroundColor"),o=o.parentNode}catch(l){}n=n.blend(s&&"transparent"!==s?s:"_default")}n=n.toRgbaString()}try{e.style[i]=n}catch(c){}}},t.fx.step[i]=function(e){e.colorInit||(e.start=r(e.elem,i),e.end=r(e.end),e.colorInit=!0),t.cssHooks[i].set(e.elem,e.start.transition(e.end,e.pos))}})),t.cssHooks.borderColor={expand:function(e){var t={};return u(["Top","Right","Bottom","Left"],(function(i,n){t["border"+n+"Color"]=e})),t}},n=t.Color.names={aqua:"#00ffff",black:"#000000",blue:"#0000ff",fuchsia:"#ff00ff",gray:"#808080",green:"#008000",lime:"#00ff00",maroon:"#800000",navy:"#000080",olive:"#808000",purple:"#800080",red:"#ff0000",silver:"#c0c0c0",teal:"#008080",white:"#ffffff",yellow:"#ffff00",transparent:[null,null,null,0],_default:"#ffffff"}}(jQuery),function(){var i=["add","remove","toggle"],n={border:1,borderBottom:1,borderColor:1,borderLeft:1,borderRight:1,borderTop:1,borderWidth:1,margin:1,padding:1};function a(){var t,i,n=this.ownerDocument.defaultView?this.ownerDocument.defaultView.getComputedStyle(this,null):this.currentStyle,a={};if(n&&n.length&&n[0]&&n[n[0]]){i=n.length;while(i--)t=n[i],"string"===typeof n[t]&&(a[e.camelCase(t)]=n[t])}else for(t in n)"string"===typeof n[t]&&(a[t]=n[t]);return a}function o(t,i){var a,o,s={};for(a in i)o=i[a],t[a]!==o&&(n[a]||!e.fx.step[a]&&isNaN(parseFloat(o))||(s[a]=o));return s}e.each(["borderLeftStyle","borderRightStyle","borderBottomStyle","borderTopStyle"],(function(t,i){e.fx.step[i]=function(e){("none"!==e.end&&!e.setAttr||1===e.pos&&!e.setAttr)&&(jQuery.style(e.elem,i,e.end),e.setAttr=!0)}})),e.effects.animateClass=function(t,n,s,r){var l=e.speed(n,s,r);return this.queue((function(){var n,s=e(this),r=s.attr("class")||"",c=l.children?s.find("*").andSelf():s;c=c.map((function(){var t=e(this);return{el:t,start:a.call(this)}})),n=function(){e.each(i,(function(e,i){t[i]&&s[i+"Class"](t[i])}))},n(),c=c.map((function(){return this.end=a.call(this.el[0]),this.diff=o(this.start,this.end),this})),s.attr("class",r),c=c.map((function(){var t=this,i=e.Deferred(),n=jQuery.extend({},l,{queue:!1,complete:function(){i.resolve(t)}});return this.el.animate(this.diff,n),i.promise()})),e.when.apply(e,c.get()).done((function(){n(),e.each(arguments,(function(){var t=this.el;e.each(this.diff,(function(e){t.css(e,"")}))})),l.complete.call(s[0])}))}))},e.fn.extend({_addClass:e.fn.addClass,addClass:function(t,i,n,a){return i?e.effects.animateClass.call(this,{add:t},i,n,a):this._addClass(t)},_removeClass:e.fn.removeClass,removeClass:function(t,i,n,a){return i?e.effects.animateClass.call(this,{remove:t},i,n,a):this._removeClass(t)},_toggleClass:e.fn.toggleClass,toggleClass:function(i,n,a,o,s){return"boolean"===typeof n||n===t?a?e.effects.animateClass.call(this,n?{add:i}:{remove:i},a,o,s):this._toggleClass(i,n):e.effects.animateClass.call(this,{toggle:i},n,a,o)},switchClass:function(t,i,n,a,o){return e.effects.animateClass.call(this,{add:i,remove:t},n,a,o)}})}(),function(){function a(t,i,n,a){return e.isPlainObject(t)&&(i=t,t=t.effect),t={effect:t},null==i&&(i={}),e.isFunction(i)&&(a=i,n=null,i={}),("number"===typeof i||e.fx.speeds[i])&&(a=n,n=i,i={}),e.isFunction(n)&&(a=n,n=null),i&&e.extend(t,i),n=n||i.duration,t.duration=e.fx.off?0:"number"===typeof n?n:n in e.fx.speeds?e.fx.speeds[n]:e.fx.speeds._default,t.complete=a||i.complete,t}function o(t){return!(t&&"number"!==typeof t&&!e.fx.speeds[t])||"string"===typeof t&&!e.effects.effect[t]&&(!i||!e.effects[t])}e.extend(e.effects,{version:"1.9.2",save:function(e,t){for(var i=0;i<t.length;i++)null!==t[i]&&e.data(n+t[i],e[0].style[t[i]])},restore:function(e,i){var a,o;for(o=0;o<i.length;o++)null!==i[o]&&(a=e.data(n+i[o]),a===t&&(a=""),e.css(i[o],a))},setMode:function(e,t){return"toggle"===t&&(t=e.is(":hidden")?"show":"hide"),t},getBaseline:function(e,t){var i,n;switch(e[0]){case"top":i=0;break;case"middle":i=.5;break;case"bottom":i=1;break;default:i=e[0]/t.height}switch(e[1]){case"left":n=0;break;case"center":n=.5;break;case"right":n=1;break;default:n=e[1]/t.width}return{x:n,y:i}},createWrapper:function(t){if(t.parent().is(".ui-effects-wrapper"))return t.parent();var i={width:t.outerWidth(!0),height:t.outerHeight(!0),float:t.css("float")},n=e("<div></div>").addClass("ui-effects-wrapper").css({fontSize:"100%",background:"transparent",border:"none",margin:0,padding:0}),a={width:t.width(),height:t.height()},o=document.activeElement;try{o.id}catch(s){o=document.body}return t.wrap(n),(t[0]===o||e.contains(t[0],o))&&e(o).focus(),n=t.parent(),"static"===t.css("position")?(n.css({position:"relative"}),t.css({position:"relative"})):(e.extend(i,{position:t.css("position"),zIndex:t.css("z-index")}),e.each(["top","left","bottom","right"],(function(e,n){i[n]=t.css(n),isNaN(parseInt(i[n],10))&&(i[n]="auto")})),t.css({position:"relative",top:0,left:0,right:"auto",bottom:"auto"})),t.css(a),n.css(i).show()},removeWrapper:function(t){var i=document.activeElement;return t.parent().is(".ui-effects-wrapper")&&(t.parent().replaceWith(t),(t[0]===i||e.contains(t[0],i))&&e(i).focus()),t},setTransition:function(t,i,n,a){return a=a||{},e.each(i,(function(e,i){var o=t.cssUnit(i);o[0]>0&&(a[i]=o[0]*n+o[1])})),a}}),e.fn.extend({effect:function(){var t=a.apply(this,arguments),n=t.mode,o=t.queue,s=e.effects.effect[t.effect],r=!s&&i&&e.effects[t.effect];if(e.fx.off||!s&&!r)return n?this[n](t.duration,t.complete):this.each((function(){t.complete&&t.complete.call(this)}));function l(i){var n=e(this),a=t.complete,o=t.mode;function r(){e.isFunction(a)&&a.call(n[0]),e.isFunction(i)&&i()}(n.is(":hidden")?"hide"===o:"show"===o)?r():s.call(n[0],t,r)}return s?!1===o?this.each(l):this.queue(o||"fx",l):r.call(this,{options:t,duration:t.duration,callback:t.complete,mode:t.mode})},_show:e.fn.show,show:function(e){if(o(e))return this._show.apply(this,arguments);var t=a.apply(this,arguments);return t.mode="show",this.effect.call(this,t)},_hide:e.fn.hide,hide:function(e){if(o(e))return this._hide.apply(this,arguments);var t=a.apply(this,arguments);return t.mode="hide",this.effect.call(this,t)},__toggle:e.fn.toggle,toggle:function(t){if(o(t)||"boolean"===typeof t||e.isFunction(t))return this.__toggle.apply(this,arguments);var i=a.apply(this,arguments);return i.mode="toggle",this.effect.call(this,i)},cssUnit:function(t){var i=this.css(t),n=[];return e.each(["em","px","%","pt"],(function(e,t){i.indexOf(t)>0&&(n=[parseFloat(i),t])})),n}})}(),function(){var t={};e.each(["Quad","Cubic","Quart","Quint","Expo"],(function(e,i){t[i]=function(t){return Math.pow(t,e+2)}})),e.extend(t,{Sine:function(e){return 1-Math.cos(e*Math.PI/2)},Circ:function(e){return 1-Math.sqrt(1-e*e)},Elastic:function(e){return 0===e||1===e?e:-Math.pow(2,8*(e-1))*Math.sin((80*(e-1)-7.5)*Math.PI/15)},Back:function(e){return e*e*(3*e-2)},Bounce:function(e){var t,i=4;while(e<((t=Math.pow(2,--i))-1)/11);return 1/Math.pow(4,3-i)-7.5625*Math.pow((3*t-2)/22-e,2)}}),e.each(t,(function(t,i){e.easing["easeIn"+t]=i,e.easing["easeOut"+t]=function(e){return 1-i(1-e)},e.easing["easeInOut"+t]=function(e){return e<.5?i(2*e)/2:1-i(-2*e+2)/2}}))}()}(jQuery),function(e,t){var i=0,n={},a={};n.height=n.paddingTop=n.paddingBottom=n.borderTopWidth=n.borderBottomWidth="hide",a.height=a.paddingTop=a.paddingBottom=a.borderTopWidth=a.borderBottomWidth="show",e.widget("ui.accordion",{version:"1.9.2",options:{active:0,animate:{},collapsible:!1,event:"click",header:"> li > :first-child,> :not(li):even",heightStyle:"auto",icons:{activeHeader:"ui-icon-triangle-1-s",header:"ui-icon-triangle-1-e"},activate:null,beforeActivate:null},_create:function(){var t=this.accordionId="ui-accordion-"+(this.element.attr("id")||++i),n=this.options;this.prevShow=this.prevHide=e(),this.element.addClass("ui-accordion ui-widget ui-helper-reset"),this.headers=this.element.find(n.header).addClass("ui-accordion-header ui-helper-reset ui-state-default ui-corner-all"),this._hoverable(this.headers),this._focusable(this.headers),this.headers.next().addClass("ui-accordion-content ui-helper-reset ui-widget-content ui-corner-bottom").hide(),n.collapsible||!1!==n.active&&null!=n.active||(n.active=0),n.active<0&&(n.active+=this.headers.length),this.active=this._findActive(n.active).addClass("ui-accordion-header-active ui-state-active").toggleClass("ui-corner-all ui-corner-top"),this.active.next().addClass("ui-accordion-content-active").show(),this._createIcons(),this.refresh(),this.element.attr("role","tablist"),this.headers.attr("role","tab").each((function(i){var n=e(this),a=n.attr("id"),o=n.next(),s=o.attr("id");a||(a=t+"-header-"+i,n.attr("id",a)),s||(s=t+"-panel-"+i,o.attr("id",s)),n.attr("aria-controls",s),o.attr("aria-labelledby",a)})).next().attr("role","tabpanel"),this.headers.not(this.active).attr({"aria-selected":"false",tabIndex:-1}).next().attr({"aria-expanded":"false","aria-hidden":"true"}).hide(),this.active.length?this.active.attr({"aria-selected":"true",tabIndex:0}).next().attr({"aria-expanded":"true","aria-hidden":"false"}):this.headers.eq(0).attr("tabIndex",0),this._on(this.headers,{keydown:"_keydown"}),this._on(this.headers.next(),{keydown:"_panelKeyDown"}),this._setupEvents(n.event)},_getCreateEventData:function(){return{header:this.active,content:this.active.length?this.active.next():e()}},_createIcons:function(){var t=this.options.icons;t&&(e("<span>").addClass("ui-accordion-header-icon ui-icon "+t.header).prependTo(this.headers),this.active.children(".ui-accordion-header-icon").removeClass(t.header).addClass(t.activeHeader),this.headers.addClass("ui-accordion-icons"))},_destroyIcons:function(){this.headers.removeClass("ui-accordion-icons").children(".ui-accordion-header-icon").remove()},_destroy:function(){var e;this.element.removeClass("ui-accordion ui-widget ui-helper-reset").removeAttr("role"),this.headers.removeClass("ui-accordion-header ui-accordion-header-active ui-helper-reset ui-state-default ui-corner-all ui-state-active ui-state-disabled ui-corner-top").removeAttr("role").removeAttr("aria-selected").removeAttr("aria-controls").removeAttr("tabIndex").each((function(){/^ui-accordion/.test(this.id)&&this.removeAttribute("id")})),this._destroyIcons(),e=this.headers.next().css("display","").removeAttr("role").removeAttr("aria-expanded").removeAttr("aria-hidden").removeAttr("aria-labelledby").removeClass("ui-helper-reset ui-widget-content ui-corner-bottom ui-accordion-content ui-accordion-content-active ui-state-disabled").each((function(){/^ui-accordion/.test(this.id)&&this.removeAttribute("id")})),"content"!==this.options.heightStyle&&e.css("height","")},_setOption:function(e,t){"active"!==e?("event"===e&&(this.options.event&&this._off(this.headers,this.options.event),this._setupEvents(t)),this._super(e,t),"collapsible"!==e||t||!1!==this.options.active||this._activate(0),"icons"===e&&(this._destroyIcons(),t&&this._createIcons()),"disabled"===e&&this.headers.add(this.headers.next()).toggleClass("ui-state-disabled",!!t)):this._activate(t)},_keydown:function(t){if(!t.altKey&&!t.ctrlKey){var i=e.ui.keyCode,n=this.headers.length,a=this.headers.index(t.target),o=!1;switch(t.keyCode){case i.RIGHT:case i.DOWN:o=this.headers[(a+1)%n];break;case i.LEFT:case i.UP:o=this.headers[(a-1+n)%n];break;case i.SPACE:case i.ENTER:this._eventHandler(t);break;case i.HOME:o=this.headers[0];break;case i.END:o=this.headers[n-1];break}o&&(e(t.target).attr("tabIndex",-1),e(o).attr("tabIndex",0),o.focus(),t.preventDefault())}},_panelKeyDown:function(t){t.keyCode===e.ui.keyCode.UP&&t.ctrlKey&&e(t.currentTarget).prev().focus()},refresh:function(){var t,i,n=this.options.heightStyle,a=this.element.parent();"fill"===n?(e.support.minHeight||(i=a.css("overflow"),a.css("overflow","hidden")),t=a.height(),this.element.siblings(":visible").each((function(){var i=e(this),n=i.css("position");"absolute"!==n&&"fixed"!==n&&(t-=i.outerHeight(!0))})),i&&a.css("overflow",i),this.headers.each((function(){t-=e(this).outerHeight(!0)})),this.headers.next().each((function(){e(this).height(Math.max(0,t-e(this).innerHeight()+e(this).height()))})).css("overflow","auto")):"auto"===n&&(t=0,this.headers.next().each((function(){t=Math.max(t,e(this).css("height","").height())})).height(t))},_activate:function(t){var i=this._findActive(t)[0];i!==this.active[0]&&(i=i||this.active[0],this._eventHandler({target:i,currentTarget:i,preventDefault:e.noop}))},_findActive:function(t){return"number"===typeof t?this.headers.eq(t):e()},_setupEvents:function(t){var i={};t&&(e.each(t.split(" "),(function(e,t){i[t]="_eventHandler"})),this._on(this.headers,i))},_eventHandler:function(t){var i=this.options,n=this.active,a=e(t.currentTarget),o=a[0]===n[0],s=o&&i.collapsible,r=s?e():a.next(),l=n.next(),c={oldHeader:n,oldPanel:l,newHeader:s?e():a,newPanel:r};t.preventDefault(),o&&!i.collapsible||!1===this._trigger("beforeActivate",t,c)||(i.active=!s&&this.headers.index(a),this.active=o?e():a,this._toggle(c),n.removeClass("ui-accordion-header-active ui-state-active"),i.icons&&n.children(".ui-accordion-header-icon").removeClass(i.icons.activeHeader).addClass(i.icons.header),o||(a.removeClass("ui-corner-all").addClass("ui-accordion-header-active ui-state-active ui-corner-top"),i.icons&&a.children(".ui-accordion-header-icon").removeClass(i.icons.header).addClass(i.icons.activeHeader),a.next().addClass("ui-accordion-content-active")))},_toggle:function(t){var i=t.newPanel,n=this.prevShow.length?this.prevShow:t.oldPanel;this.prevShow.add(this.prevHide).stop(!0,!0),this.prevShow=i,this.prevHide=n,this.options.animate?this._animate(i,n,t):(n.hide(),i.show(),this._toggleComplete(t)),n.attr({"aria-expanded":"false","aria-hidden":"true"}),n.prev().attr("aria-selected","false"),i.length&&n.length?n.prev().attr("tabIndex",-1):i.length&&this.headers.filter((function(){return 0===e(this).attr("tabIndex")})).attr("tabIndex",-1),i.attr({"aria-expanded":"true","aria-hidden":"false"}).prev().attr({"aria-selected":"true",tabIndex:0})},_animate:function(e,t,i){var o,s,r,l=this,c=0,h=e.length&&(!t.length||e.index()<t.index()),d=this.options.animate||{},u=h&&d.down||d,p=function(){l._toggleComplete(i)};return"number"===typeof u&&(r=u),"string"===typeof u&&(s=u),s=s||u.easing||d.easing,r=r||u.duration||d.duration,t.length?e.length?(o=e.show().outerHeight(),t.animate(n,{duration:r,easing:s,step:function(e,t){t.now=Math.round(e)}}),void e.hide().animate(a,{duration:r,easing:s,complete:p,step:function(e,i){i.now=Math.round(e),"height"!==i.prop?c+=i.now:"content"!==l.options.heightStyle&&(i.now=Math.round(o-t.outerHeight()-c),c=0)}})):t.animate(n,r,s,p):e.animate(a,r,s,p)},_toggleComplete:function(e){var t=e.oldPanel;t.removeClass("ui-accordion-content-active").prev().removeClass("ui-corner-top").addClass("ui-corner-all"),t.length&&(t.parent()[0].className=t.parent()[0].className),this._trigger("activate",null,e)}}),!1!==e.uiBackCompat&&(function(e,t){e.extend(t.options,{navigation:!1,navigationFilter:function(){return this.href.toLowerCase()===location.href.toLowerCase()}});var i=t._create;t._create=function(){if(this.options.navigation){var t=this,n=this.element.find(this.options.header),a=n.next(),o=n.add(a).find("a").filter(this.options.navigationFilter)[0];o&&n.add(a).each((function(i){if(e.contains(this,o))return t.options.active=Math.floor(i/2),!1}))}i.call(this)}}(jQuery,jQuery.ui.accordion.prototype),function(e,t){e.extend(t.options,{heightStyle:null,autoHeight:!0,clearStyle:!1,fillSpace:!1});var i=t._create,n=t._setOption;e.extend(t,{_create:function(){this.options.heightStyle=this.options.heightStyle||this._mergeHeightStyle(),i.call(this)},_setOption:function(e){"autoHeight"!==e&&"clearStyle"!==e&&"fillSpace"!==e||(this.options.heightStyle=this._mergeHeightStyle()),n.apply(this,arguments)},_mergeHeightStyle:function(){var e=this.options;return e.fillSpace?"fill":e.clearStyle?"content":e.autoHeight?"auto":void 0}})}(jQuery,jQuery.ui.accordion.prototype),function(e,t){e.extend(t.options.icons,{activeHeader:null,headerSelected:"ui-icon-triangle-1-s"});var i=t._createIcons;t._createIcons=function(){this.options.icons&&(this.options.icons.activeHeader=this.options.icons.activeHeader||this.options.icons.headerSelected),i.call(this)}}(jQuery,jQuery.ui.accordion.prototype),function(e,t){t.activate=t._activate;var i=t._findActive;t._findActive=function(e){return-1===e&&(e=!1),e&&"number"!==typeof e&&(e=this.headers.index(this.headers.filter(e)),-1===e&&(e=!1)),i.call(this,e)}}(jQuery,jQuery.ui.accordion.prototype),jQuery.ui.accordion.prototype.resize=jQuery.ui.accordion.prototype.refresh,function(e,t){e.extend(t.options,{change:null,changestart:null});var i=t._trigger;t._trigger=function(e,t,n){var a=i.apply(this,arguments);return!!a&&("beforeActivate"===e?a=i.call(this,"changestart",t,{oldHeader:n.oldHeader,oldContent:n.oldPanel,newHeader:n.newHeader,newContent:n.newPanel}):"activate"===e&&(a=i.call(this,"change",t,{oldHeader:n.oldHeader,oldContent:n.oldPanel,newHeader:n.newHeader,newContent:n.newPanel})),a)}}(jQuery,jQuery.ui.accordion.prototype),function(e,t){e.extend(t.options,{animate:null,animated:"slide"});var i=t._create;t._create=function(){var e=this.options;null===e.animate&&(e.animated?"slide"===e.animated?e.animate=300:"bounceslide"===e.animated?e.animate={duration:200,down:{easing:"easeOutBounce",duration:1e3}}:e.animate=e.animated:e.animate=!1),i.call(this)}}(jQuery,jQuery.ui.accordion.prototype))}(jQuery),function(e,t){var i=0;e.widget("ui.autocomplete",{version:"1.9.2",defaultElement:"<input>",options:{appendTo:"body",autoFocus:!1,delay:300,minLength:1,position:{my:"left top",at:"left bottom",collision:"none"},source:null,change:null,close:null,focus:null,open:null,response:null,search:null,select:null},pending:0,_create:function(){var t,i,n;this.isMultiLine=this._isMultiLine(),this.valueMethod=this.element[this.element.is("input,textarea")?"val":"text"],this.isNewMenu=!0,this.element.addClass("ui-autocomplete-input").attr("autocomplete","off"),this._on(this.element,{keydown:function(a){if(this.element.prop("readOnly"))return t=!0,n=!0,void(i=!0);t=!1,n=!1,i=!1;var o=e.ui.keyCode;switch(a.keyCode){case o.PAGE_UP:t=!0,this._move("previousPage",a);break;case o.PAGE_DOWN:t=!0,this._move("nextPage",a);break;case o.UP:t=!0,this._keyEvent("previous",a);break;case o.DOWN:t=!0,this._keyEvent("next",a);break;case o.ENTER:case o.NUMPAD_ENTER:this.menu.active&&(t=!0,a.preventDefault(),this.menu.select(a));break;case o.TAB:this.menu.active&&this.menu.select(a);break;case o.ESCAPE:this.menu.element.is(":visible")&&(this._value(this.term),this.close(a),a.preventDefault());break;default:i=!0,this._searchTimeout(a);break}},keypress:function(n){if(t)return t=!1,void n.preventDefault();if(!i){var a=e.ui.keyCode;switch(n.keyCode){case a.PAGE_UP:this._move("previousPage",n);break;case a.PAGE_DOWN:this._move("nextPage",n);break;case a.UP:this._keyEvent("previous",n);break;case a.DOWN:this._keyEvent("next",n);break}}},input:function(e){if(n)return n=!1,void e.preventDefault();this._searchTimeout(e)},focus:function(){this.selectedItem=null,this.previous=this._value()},blur:function(e){this.cancelBlur?delete this.cancelBlur:(clearTimeout(this.searching),this.close(e),this._change(e))}}),this._initSource(),this.menu=e("<ul>").addClass("ui-autocomplete").appendTo(this.document.find(this.options.appendTo||"body")[0]).menu({input:e(),role:null}).zIndex(this.element.zIndex()+1).hide().data("menu"),this._on(this.menu.element,{mousedown:function(t){t.preventDefault(),this.cancelBlur=!0,this._delay((function(){delete this.cancelBlur}));var i=this.menu.element[0];e(t.target).closest(".ui-menu-item").length||this._delay((function(){var t=this;this.document.one("mousedown",(function(n){n.target===t.element[0]||n.target===i||e.contains(i,n.target)||t.close()}))}))},menufocus:function(t,i){if(this.isNewMenu&&(this.isNewMenu=!1,t.originalEvent&&/^mouse/.test(t.originalEvent.type)))return this.menu.blur(),void this.document.one("mousemove",(function(){e(t.target).trigger(t.originalEvent)}));var n=i.item.data("ui-autocomplete-item")||i.item.data("item.autocomplete");!1!==this._trigger("focus",t,{item:n})?t.originalEvent&&/^key/.test(t.originalEvent.type)&&this._value(n.value):this.liveRegion.text(n.value)},menuselect:function(e,t){var i=t.item.data("ui-autocomplete-item")||t.item.data("item.autocomplete"),n=this.previous;this.element[0]!==this.document[0].activeElement&&(this.element.focus(),this.previous=n,this._delay((function(){this.previous=n,this.selectedItem=i}))),!1!==this._trigger("select",e,{item:i})&&this._value(i.value),this.term=this._value(),this.close(e),this.selectedItem=i}}),this.liveRegion=e("<span>",{role:"status","aria-live":"polite"}).addClass("ui-helper-hidden-accessible").insertAfter(this.element),e.fn.bgiframe&&this.menu.element.bgiframe(),this._on(this.window,{beforeunload:function(){this.element.removeAttr("autocomplete")}})},_destroy:function(){clearTimeout(this.searching),this.element.removeClass("ui-autocomplete-input").removeAttr("autocomplete"),this.menu.element.remove(),this.liveRegion.remove()},_setOption:function(e,t){this._super(e,t),"source"===e&&this._initSource(),"appendTo"===e&&this.menu.element.appendTo(this.document.find(t||"body")[0]),"disabled"===e&&t&&this.xhr&&this.xhr.abort()},_isMultiLine:function(){return!!this.element.is("textarea")||!this.element.is("input")&&this.element.prop("isContentEditable")},_initSource:function(){var t,i,n=this;e.isArray(this.options.source)?(t=this.options.source,this.source=function(i,n){n(e.ui.autocomplete.filter(t,i.term))}):"string"===typeof this.options.source?(i=this.options.source,this.source=function(t,a){n.xhr&&n.xhr.abort(),n.xhr=e.ajax({url:i,data:t,dataType:"json",success:function(e){a(e)},error:function(){a([])}})}):this.source=this.options.source},_searchTimeout:function(e){clearTimeout(this.searching),this.searching=this._delay((function(){this.term!==this._value()&&(this.selectedItem=null,this.search(null,e))}),this.options.delay)},search:function(e,t){return e=null!=e?e:this._value(),this.term=this._value(),e.length<this.options.minLength?this.close(t):!1!==this._trigger("search",t)?this._search(e):void 0},_search:function(e){this.pending++,this.element.addClass("ui-autocomplete-loading"),this.cancelSearch=!1,this.source({term:e},this._response())},_response:function(){var e=this,t=++i;return function(n){t===i&&e.__response(n),e.pending--,e.pending||e.element.removeClass("ui-autocomplete-loading")}},__response:function(e){e&&(e=this._normalize(e)),this._trigger("response",null,{content:e}),!this.options.disabled&&e&&e.length&&!this.cancelSearch?(this._suggest(e),this._trigger("open")):this._close()},close:function(e){this.cancelSearch=!0,this._close(e)},_close:function(e){this.menu.element.is(":visible")&&(this.menu.element.hide(),this.menu.blur(),this.isNewMenu=!0,this._trigger("close",e))},_change:function(e){this.previous!==this._value()&&this._trigger("change",e,{item:this.selectedItem})},_normalize:function(t){return t.length&&t[0].label&&t[0].value?t:e.map(t,(function(t){return"string"===typeof t?{label:t,value:t}:e.extend({label:t.label||t.value,value:t.value||t.label},t)}))},_suggest:function(t){var i=this.menu.element.empty().zIndex(this.element.zIndex()+1);this._renderMenu(i,t),this.menu.refresh(),i.show(),this._resizeMenu(),i.position(e.extend({of:this.element},this.options.position)),this.options.autoFocus&&this.menu.next()},_resizeMenu:function(){var e=this.menu.element;e.outerWidth(Math.max(e.width("").outerWidth()+1,this.element.outerWidth()))},_renderMenu:function(t,i){var n=this;e.each(i,(function(e,i){n._renderItemData(t,i)}))},_renderItemData:function(e,t){return this._renderItem(e,t).data("ui-autocomplete-item",t)},_renderItem:function(t,i){return e("<li>").append(e("<a>").text(i.label)).appendTo(t)},_move:function(e,t){if(this.menu.element.is(":visible"))return this.menu.isFirstItem()&&/^previous/.test(e)||this.menu.isLastItem()&&/^next/.test(e)?(this._value(this.term),void this.menu.blur()):void this.menu[e](t);this.search(null,t)},widget:function(){return this.menu.element},_value:function(){return this.valueMethod.apply(this.element,arguments)},_keyEvent:function(e,t){this.isMultiLine&&!this.menu.element.is(":visible")||(this._move(e,t),t.preventDefault())}}),e.extend(e.ui.autocomplete,{escapeRegex:function(e){return e.replace(/[\-\[\]{}()*+?.,\\\^$|#\s]/g,"\\$&")},filter:function(t,i){var n=new RegExp(e.ui.autocomplete.escapeRegex(i),"i");return e.grep(t,(function(e){return n.test(e.label||e.value||e)}))}}),e.widget("ui.autocomplete",e.ui.autocomplete,{options:{messages:{noResults:"No search results.",results:function(e){return e+(e>1?" results are":" result is")+" available, use up and down arrow keys to navigate."}}},__response:function(e){var t;this._superApply(arguments),this.options.disabled||this.cancelSearch||(t=e&&e.length?this.options.messages.results(e.length):this.options.messages.noResults,this.liveRegion.text(t))}})}(jQuery),function(e,t){var i,n,a,o,s="ui-button ui-widget ui-state-default ui-corner-all",r="ui-state-hover ui-state-active ",l="ui-button-icons-only ui-button-icon-only ui-button-text-icons ui-button-text-icon-primary ui-button-text-icon-secondary ui-button-text-only",c=function(){var t=e(this).find(":ui-button");setTimeout((function(){t.button("refresh")}),1)},h=function(t){var i=t.name,n=t.form,a=e([]);return i&&(a=n?e(n).find("[name='"+i+"']"):e("[name='"+i+"']",t.ownerDocument).filter((function(){return!this.form}))),a};e.widget("ui.button",{version:"1.9.2",defaultElement:"<button>",options:{disabled:null,text:!0,label:null,icons:{primary:null,secondary:null}},_create:function(){this.element.closest("form").unbind("reset"+this.eventNamespace).bind("reset"+this.eventNamespace,c),"boolean"!==typeof this.options.disabled?this.options.disabled=!!this.element.prop("disabled"):this.element.prop("disabled",this.options.disabled),this._determineButtonType(),this.hasTitle=!!this.buttonElement.attr("title");var t=this,r=this.options,l="checkbox"===this.type||"radio"===this.type,d=l?"":"ui-state-active",u="ui-state-focus";null===r.label&&(r.label="input"===this.type?this.buttonElement.val():this.buttonElement.html()),this._hoverable(this.buttonElement),this.buttonElement.addClass(s).attr("role","button").bind("mouseenter"+this.eventNamespace,(function(){r.disabled||this===i&&e(this).addClass("ui-state-active")})).bind("mouseleave"+this.eventNamespace,(function(){r.disabled||e(this).removeClass(d)})).bind("click"+this.eventNamespace,(function(e){r.disabled&&(e.preventDefault(),e.stopImmediatePropagation())})),this.element.bind("focus"+this.eventNamespace,(function(){t.buttonElement.addClass(u)})).bind("blur"+this.eventNamespace,(function(){t.buttonElement.removeClass(u)})),l&&(this.element.bind("change"+this.eventNamespace,(function(){o||t.refresh()})),this.buttonElement.bind("mousedown"+this.eventNamespace,(function(e){r.disabled||(o=!1,n=e.pageX,a=e.pageY)})).bind("mouseup"+this.eventNamespace,(function(e){r.disabled||n===e.pageX&&a===e.pageY||(o=!0)}))),"checkbox"===this.type?this.buttonElement.bind("click"+this.eventNamespace,(function(){if(r.disabled||o)return!1;e(this).toggleClass("ui-state-active"),t.buttonElement.attr("aria-pressed",t.element[0].checked)})):"radio"===this.type?this.buttonElement.bind("click"+this.eventNamespace,(function(){if(r.disabled||o)return!1;e(this).addClass("ui-state-active"),t.buttonElement.attr("aria-pressed","true");var i=t.element[0];h(i).not(i).map((function(){return e(this).button("widget")[0]})).removeClass("ui-state-active").attr("aria-pressed","false")})):(this.buttonElement.bind("mousedown"+this.eventNamespace,(function(){if(r.disabled)return!1;e(this).addClass("ui-state-active"),i=this,t.document.one("mouseup",(function(){i=null}))})).bind("mouseup"+this.eventNamespace,(function(){if(r.disabled)return!1;e(this).removeClass("ui-state-active")})).bind("keydown"+this.eventNamespace,(function(t){if(r.disabled)return!1;t.keyCode!==e.ui.keyCode.SPACE&&t.keyCode!==e.ui.keyCode.ENTER||e(this).addClass("ui-state-active")})).bind("keyup"+this.eventNamespace,(function(){e(this).removeClass("ui-state-active")})),this.buttonElement.is("a")&&this.buttonElement.keyup((function(t){t.keyCode===e.ui.keyCode.SPACE&&e(this).click()}))),this._setOption("disabled",r.disabled),this._resetButton()},_determineButtonType:function(){var e,t,i;this.element.is("[type=checkbox]")?this.type="checkbox":this.element.is("[type=radio]")?this.type="radio":this.element.is("input")?this.type="input":this.type="button","checkbox"===this.type||"radio"===this.type?(e=this.element.parents().last(),t="label[for='"+this.element.attr("id")+"']",this.buttonElement=e.find(t),this.buttonElement.length||(e=e.length?e.siblings():this.element.siblings(),this.buttonElement=e.filter(t),this.buttonElement.length||(this.buttonElement=e.find(t))),this.element.addClass("ui-helper-hidden-accessible"),i=this.element.is(":checked"),i&&this.buttonElement.addClass("ui-state-active"),this.buttonElement.prop("aria-pressed",i)):this.buttonElement=this.element},widget:function(){return this.buttonElement},_destroy:function(){this.element.removeClass("ui-helper-hidden-accessible"),this.buttonElement.removeClass(s+" "+r+" "+l).removeAttr("role").removeAttr("aria-pressed").html(this.buttonElement.find(".ui-button-text").html()),this.hasTitle||this.buttonElement.removeAttr("title")},_setOption:function(e,t){this._super(e,t),"disabled"!==e?this._resetButton():t?this.element.prop("disabled",!0):this.element.prop("disabled",!1)},refresh:function(){var t=this.element.is("input, button")?this.element.is(":disabled"):this.element.hasClass("ui-button-disabled");t!==this.options.disabled&&this._setOption("disabled",t),"radio"===this.type?h(this.element[0]).each((function(){e(this).is(":checked")?e(this).button("widget").addClass("ui-state-active").attr("aria-pressed","true"):e(this).button("widget").removeClass("ui-state-active").attr("aria-pressed","false")})):"checkbox"===this.type&&(this.element.is(":checked")?this.buttonElement.addClass("ui-state-active").attr("aria-pressed","true"):this.buttonElement.removeClass("ui-state-active").attr("aria-pressed","false"))},_resetButton:function(){if("input"!==this.type){var t=this.buttonElement.removeClass(l),i=e("<span></span>",this.document[0]).addClass("ui-button-text").html(this.options.label).appendTo(t.empty()).text(),n=this.options.icons,a=n.primary&&n.secondary,o=[];n.primary||n.secondary?(this.options.text&&o.push("ui-button-text-icon"+(a?"s":n.primary?"-primary":"-secondary")),n.primary&&t.prepend("<span class='ui-button-icon-primary ui-icon "+n.primary+"'></span>"),n.secondary&&t.append("<span class='ui-button-icon-secondary ui-icon "+n.secondary+"'></span>"),this.options.text||(o.push(a?"ui-button-icons-only":"ui-button-icon-only"),this.hasTitle||t.attr("title",e.trim(i)))):o.push("ui-button-text-only"),t.addClass(o.join(" "))}else this.options.label&&this.element.val(this.options.label)}}),e.widget("ui.buttonset",{version:"1.9.2",options:{items:"button, input[type=button], input[type=submit], input[type=reset], input[type=checkbox], input[type=radio], a, :data(button)"},_create:function(){this.element.addClass("ui-buttonset")},_init:function(){this.refresh()},_setOption:function(e,t){"disabled"===e&&this.buttons.button("option",e,t),this._super(e,t)},refresh:function(){var t="rtl"===this.element.css("direction");this.buttons=this.element.find(this.options.items).filter(":ui-button").button("refresh").end().not(":ui-button").button().end().map((function(){return e(this).button("widget")[0]})).removeClass("ui-corner-all ui-corner-left ui-corner-right").filter(":first").addClass(t?"ui-corner-right":"ui-corner-left").end().filter(":last").addClass(t?"ui-corner-left":"ui-corner-right").end().end()},_destroy:function(){this.element.removeClass("ui-buttonset"),this.buttons.map((function(){return e(this).button("widget")[0]})).removeClass("ui-corner-left ui-corner-right").end().button("destroy")}})}(jQuery),function($,undefined){$.extend($.ui,{datepicker:{version:"1.9.2"}});var PROP_NAME="datepicker",dpuuid=(new Date).getTime(),instActive;function Datepicker(){this.debug=!1,this._curInst=null,this._keyEvent=!1,this._disabledInputs=[],this._datepickerShowing=!1,this._inDialog=!1,this._mainDivId="ui-datepicker-div",this._inlineClass="ui-datepicker-inline",this._appendClass="ui-datepicker-append",this._triggerClass="ui-datepicker-trigger",this._dialogClass="ui-datepicker-dialog",this._disableClass="ui-datepicker-disabled",this._unselectableClass="ui-datepicker-unselectable",this._currentClass="ui-datepicker-current-day",this._dayOverClass="ui-datepicker-days-cell-over",this.regional=[],this.regional[""]={closeText:"Done",prevText:"Prev",nextText:"Next",currentText:"Today",monthNames:["January","February","March","April","May","June","July","August","September","October","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayNamesShort:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],dayNamesMin:["Su","Mo","Tu","We","Th","Fr","Sa"],weekHeader:"Wk",dateFormat:"mm/dd/yy",firstDay:0,isRTL:!1,showMonthAfterYear:!1,yearSuffix:""},this._defaults={showOn:"focus",showAnim:"fadeIn",showOptions:{},defaultDate:null,appendText:"",buttonText:"...",buttonImage:"",buttonImageOnly:!1,hideIfNoPrevNext:!1,navigationAsDateFormat:!1,gotoCurrent:!1,changeMonth:!1,changeYear:!1,yearRange:"c-10:c+10",showOtherMonths:!1,selectOtherMonths:!1,showWeek:!1,calculateWeek:this.iso8601Week,shortYearCutoff:"+10",minDate:null,maxDate:null,duration:"fast",beforeShowDay:null,beforeShow:null,onSelect:null,onChangeMonthYear:null,onClose:null,numberOfMonths:1,showCurrentAtPos:0,stepMonths:1,stepBigMonths:12,altField:"",altFormat:"",constrainInput:!0,showButtonPanel:!1,autoSize:!1,disabled:!1},$.extend(this._defaults,this.regional[""]),this.dpDiv=bindHover($('<div id="'+this._mainDivId+'" class="ui-datepicker ui-widget ui-widget-content ui-helper-clearfix ui-corner-all"></div>'))}function bindHover(e){var t="button, .ui-datepicker-prev, .ui-datepicker-next, .ui-datepicker-calendar td a";return e.delegate(t,"mouseout",(function(){$(this).removeClass("ui-state-hover"),-1!=this.className.indexOf("ui-datepicker-prev")&&$(this).removeClass("ui-datepicker-prev-hover"),-1!=this.className.indexOf("ui-datepicker-next")&&$(this).removeClass("ui-datepicker-next-hover")})).delegate(t,"mouseover",(function(){$.datepicker._isDisabledDatepicker(instActive.inline?e.parent()[0]:instActive.input[0])||($(this).parents(".ui-datepicker-calendar").find("a").removeClass("ui-state-hover"),$(this).addClass("ui-state-hover"),-1!=this.className.indexOf("ui-datepicker-prev")&&$(this).addClass("ui-datepicker-prev-hover"),-1!=this.className.indexOf("ui-datepicker-next")&&$(this).addClass("ui-datepicker-next-hover"))}))}function extendRemove(e,t){for(var i in $.extend(e,t),t)null!=t[i]&&t[i]!=undefined||(e[i]=t[i]);return e}$.extend(Datepicker.prototype,{markerClassName:"hasDatepicker",maxRows:4,log:function(){this.debug&&console.log.apply("",arguments)},_widgetDatepicker:function(){return this.dpDiv},setDefaults:function(e){return extendRemove(this._defaults,e||{}),this},_attachDatepicker:function(target,settings){var inlineSettings=null;for(var attrName in this._defaults){var attrValue=target.getAttribute("date:"+attrName);if(attrValue){inlineSettings=inlineSettings||{};try{inlineSettings[attrName]=eval(attrValue)}catch(err){inlineSettings[attrName]=attrValue}}}var nodeName=target.nodeName.toLowerCase(),inline="div"==nodeName||"span"==nodeName;target.id||(this.uuid+=1,target.id="dp"+this.uuid);var inst=this._newInst($(target),inline);inst.settings=$.extend({},settings||{},inlineSettings||{}),"input"==nodeName?this._connectDatepicker(target,inst):inline&&this._inlineDatepicker(target,inst)},_newInst:function(e,t){var i=e[0].id.replace(/([^A-Za-z0-9_-])/g,"\\\\$1");return{id:i,input:e,selectedDay:0,selectedMonth:0,selectedYear:0,drawMonth:0,drawYear:0,inline:t,dpDiv:t?bindHover($('<div class="'+this._inlineClass+' ui-datepicker ui-widget ui-widget-content ui-helper-clearfix ui-corner-all"></div>')):this.dpDiv}},_connectDatepicker:function(e,t){var i=$(e);t.append=$([]),t.trigger=$([]),i.hasClass(this.markerClassName)||(this._attachments(i,t),i.addClass(this.markerClassName).keydown(this._doKeyDown).keypress(this._doKeyPress).keyup(this._doKeyUp).bind("setData.datepicker",(function(e,i,n){t.settings[i]=n})).bind("getData.datepicker",(function(e,i){return this._get(t,i)})),this._autoSize(t),$.data(e,PROP_NAME,t),t.settings.disabled&&this._disableDatepicker(e))},_attachments:function(e,t){var i=this._get(t,"appendText"),n=this._get(t,"isRTL");t.append&&t.append.remove(),i&&(t.append=$('<span class="'+this._appendClass+'">'+i+"</span>"),e[n?"before":"after"](t.append)),e.unbind("focus",this._showDatepicker),t.trigger&&t.trigger.remove();var a=this._get(t,"showOn");if("focus"!=a&&"both"!=a||e.focus(this._showDatepicker),"button"==a||"both"==a){var o=this._get(t,"buttonText"),s=this._get(t,"buttonImage");t.trigger=$(this._get(t,"buttonImageOnly")?$("<img/>").addClass(this._triggerClass).attr({src:s,alt:o,title:o}):$('<button type="button"></button>').addClass(this._triggerClass).html(""==s?o:$("<img/>").attr({src:s,alt:o,title:o}))),e[n?"before":"after"](t.trigger),t.trigger.click((function(){return $.datepicker._datepickerShowing&&$.datepicker._lastInput==e[0]?$.datepicker._hideDatepicker():$.datepicker._datepickerShowing&&$.datepicker._lastInput!=e[0]?($.datepicker._hideDatepicker(),$.datepicker._showDatepicker(e[0])):$.datepicker._showDatepicker(e[0]),!1}))}},_autoSize:function(e){if(this._get(e,"autoSize")&&!e.inline){var t=new Date(2009,11,20),i=this._get(e,"dateFormat");if(i.match(/[DM]/)){var n=function(e){for(var t=0,i=0,n=0;n<e.length;n++)e[n].length>t&&(t=e[n].length,i=n);return i};t.setMonth(n(this._get(e,i.match(/MM/)?"monthNames":"monthNamesShort"))),t.setDate(n(this._get(e,i.match(/DD/)?"dayNames":"dayNamesShort"))+20-t.getDay())}e.input.attr("size",this._formatDate(e,t).length)}},_inlineDatepicker:function(e,t){var i=$(e);i.hasClass(this.markerClassName)||(i.addClass(this.markerClassName).append(t.dpDiv).bind("setData.datepicker",(function(e,i,n){t.settings[i]=n})).bind("getData.datepicker",(function(e,i){return this._get(t,i)})),$.data(e,PROP_NAME,t),this._setDate(t,this._getDefaultDate(t),!0),this._updateDatepicker(t),this._updateAlternate(t),t.settings.disabled&&this._disableDatepicker(e),t.dpDiv.css("display","block"))},_dialogDatepicker:function(e,t,i,n,a){var o=this._dialogInst;if(!o){this.uuid+=1;var s="dp"+this.uuid;this._dialogInput=$('<input type="text" id="'+s+'" style="position: absolute; top: -100px; width: 0px;"/>'),this._dialogInput.keydown(this._doKeyDown),$("body").append(this._dialogInput),o=this._dialogInst=this._newInst(this._dialogInput,!1),o.settings={},$.data(this._dialogInput[0],PROP_NAME,o)}if(extendRemove(o.settings,n||{}),t=t&&t.constructor==Date?this._formatDate(o,t):t,this._dialogInput.val(t),this._pos=a?a.length?a:[a.pageX,a.pageY]:null,!this._pos){var r=document.documentElement.clientWidth,l=document.documentElement.clientHeight,c=document.documentElement.scrollLeft||document.body.scrollLeft,h=document.documentElement.scrollTop||document.body.scrollTop;this._pos=[r/2-100+c,l/2-150+h]}return this._dialogInput.css("left",this._pos[0]+20+"px").css("top",this._pos[1]+"px"),o.settings.onSelect=i,this._inDialog=!0,this.dpDiv.addClass(this._dialogClass),this._showDatepicker(this._dialogInput[0]),$.blockUI&&$.blockUI(this.dpDiv),$.data(this._dialogInput[0],PROP_NAME,o),this},_destroyDatepicker:function(e){var t=$(e),i=$.data(e,PROP_NAME);if(t.hasClass(this.markerClassName)){var n=e.nodeName.toLowerCase();$.removeData(e,PROP_NAME),"input"==n?(i.append.remove(),i.trigger.remove(),t.removeClass(this.markerClassName).unbind("focus",this._showDatepicker).unbind("keydown",this._doKeyDown).unbind("keypress",this._doKeyPress).unbind("keyup",this._doKeyUp)):"div"!=n&&"span"!=n||t.removeClass(this.markerClassName).empty()}},_enableDatepicker:function(e){var t=$(e),i=$.data(e,PROP_NAME);if(t.hasClass(this.markerClassName)){var n=e.nodeName.toLowerCase();if("input"==n)e.disabled=!1,i.trigger.filter("button").each((function(){this.disabled=!1})).end().filter("img").css({opacity:"1.0",cursor:""});else if("div"==n||"span"==n){var a=t.children("."+this._inlineClass);a.children().removeClass("ui-state-disabled"),a.find("select.ui-datepicker-month, select.ui-datepicker-year").prop("disabled",!1)}this._disabledInputs=$.map(this._disabledInputs,(function(t){return t==e?null:t}))}},_disableDatepicker:function(e){var t=$(e),i=$.data(e,PROP_NAME);if(t.hasClass(this.markerClassName)){var n=e.nodeName.toLowerCase();if("input"==n)e.disabled=!0,i.trigger.filter("button").each((function(){this.disabled=!0})).end().filter("img").css({opacity:"0.5",cursor:"default"});else if("div"==n||"span"==n){var a=t.children("."+this._inlineClass);a.children().addClass("ui-state-disabled"),a.find("select.ui-datepicker-month, select.ui-datepicker-year").prop("disabled",!0)}this._disabledInputs=$.map(this._disabledInputs,(function(t){return t==e?null:t})),this._disabledInputs[this._disabledInputs.length]=e}},_isDisabledDatepicker:function(e){if(!e)return!1;for(var t=0;t<this._disabledInputs.length;t++)if(this._disabledInputs[t]==e)return!0;return!1},_getInst:function(e){try{return $.data(e,PROP_NAME)}catch(t){throw"Missing instance data for this datepicker"}},_optionDatepicker:function(e,t,i){var n=this._getInst(e);if(2==arguments.length&&"string"==typeof t)return"defaults"==t?$.extend({},$.datepicker._defaults):n?"all"==t?$.extend({},n.settings):this._get(n,t):null;var a=t||{};if("string"==typeof t&&(a={},a[t]=i),n){this._curInst==n&&this._hideDatepicker();var o=this._getDateDatepicker(e,!0),s=this._getMinMaxDate(n,"min"),r=this._getMinMaxDate(n,"max");extendRemove(n.settings,a),null!==s&&a["dateFormat"]!==undefined&&a["minDate"]===undefined&&(n.settings.minDate=this._formatDate(n,s)),null!==r&&a["dateFormat"]!==undefined&&a["maxDate"]===undefined&&(n.settings.maxDate=this._formatDate(n,r)),this._attachments($(e),n),this._autoSize(n),this._setDate(n,o),this._updateAlternate(n),this._updateDatepicker(n)}},_changeDatepicker:function(e,t,i){this._optionDatepicker(e,t,i)},_refreshDatepicker:function(e){var t=this._getInst(e);t&&this._updateDatepicker(t)},_setDateDatepicker:function(e,t){var i=this._getInst(e);i&&(this._setDate(i,t),this._updateDatepicker(i),this._updateAlternate(i))},_getDateDatepicker:function(e,t){var i=this._getInst(e);return i&&!i.inline&&this._setDateFromField(i,t),i?this._getDate(i):null},_doKeyDown:function(e){var t=$.datepicker._getInst(e.target),i=!0,n=t.dpDiv.is(".ui-datepicker-rtl");if(t._keyEvent=!0,$.datepicker._datepickerShowing)switch(e.keyCode){case 9:$.datepicker._hideDatepicker(),i=!1;break;case 13:var a=$("td."+$.datepicker._dayOverClass+":not(."+$.datepicker._currentClass+")",t.dpDiv);a[0]&&$.datepicker._selectDay(e.target,t.selectedMonth,t.selectedYear,a[0]);var o=$.datepicker._get(t,"onSelect");if(o){var s=$.datepicker._formatDate(t);o.apply(t.input?t.input[0]:null,[s,t])}else $.datepicker._hideDatepicker();return!1;case 27:$.datepicker._hideDatepicker();break;case 33:$.datepicker._adjustDate(e.target,e.ctrlKey?-$.datepicker._get(t,"stepBigMonths"):-$.datepicker._get(t,"stepMonths"),"M");break;case 34:$.datepicker._adjustDate(e.target,e.ctrlKey?+$.datepicker._get(t,"stepBigMonths"):+$.datepicker._get(t,"stepMonths"),"M");break;case 35:(e.ctrlKey||e.metaKey)&&$.datepicker._clearDate(e.target),i=e.ctrlKey||e.metaKey;break;case 36:(e.ctrlKey||e.metaKey)&&$.datepicker._gotoToday(e.target),i=e.ctrlKey||e.metaKey;break;case 37:(e.ctrlKey||e.metaKey)&&$.datepicker._adjustDate(e.target,n?1:-1,"D"),i=e.ctrlKey||e.metaKey,e.originalEvent.altKey&&$.datepicker._adjustDate(e.target,e.ctrlKey?-$.datepicker._get(t,"stepBigMonths"):-$.datepicker._get(t,"stepMonths"),"M");break;case 38:(e.ctrlKey||e.metaKey)&&$.datepicker._adjustDate(e.target,-7,"D"),i=e.ctrlKey||e.metaKey;break;case 39:(e.ctrlKey||e.metaKey)&&$.datepicker._adjustDate(e.target,n?-1:1,"D"),i=e.ctrlKey||e.metaKey,e.originalEvent.altKey&&$.datepicker._adjustDate(e.target,e.ctrlKey?+$.datepicker._get(t,"stepBigMonths"):+$.datepicker._get(t,"stepMonths"),"M");break;case 40:(e.ctrlKey||e.metaKey)&&$.datepicker._adjustDate(e.target,7,"D"),i=e.ctrlKey||e.metaKey;break;default:i=!1}else 36==e.keyCode&&e.ctrlKey?$.datepicker._showDatepicker(this):i=!1;i&&(e.preventDefault(),e.stopPropagation())},_doKeyPress:function(e){var t=$.datepicker._getInst(e.target);if($.datepicker._get(t,"constrainInput")){var i=$.datepicker._possibleChars($.datepicker._get(t,"dateFormat")),n=String.fromCharCode(e.charCode==undefined?e.keyCode:e.charCode);return e.ctrlKey||e.metaKey||n<" "||!i||i.indexOf(n)>-1}},_doKeyUp:function(e){var t=$.datepicker._getInst(e.target);if(t.input.val()!=t.lastVal)try{var i=$.datepicker.parseDate($.datepicker._get(t,"dateFormat"),t.input?t.input.val():null,$.datepicker._getFormatConfig(t));i&&($.datepicker._setDateFromField(t),$.datepicker._updateAlternate(t),$.datepicker._updateDatepicker(t))}catch(n){$.datepicker.log(n)}return!0},_showDatepicker:function(e){if(e=e.target||e,"input"!=e.nodeName.toLowerCase()&&(e=$("input",e.parentNode)[0]),!$.datepicker._isDisabledDatepicker(e)&&$.datepicker._lastInput!=e){var t=$.datepicker._getInst(e);$.datepicker._curInst&&$.datepicker._curInst!=t&&($.datepicker._curInst.dpDiv.stop(!0,!0),t&&$.datepicker._datepickerShowing&&$.datepicker._hideDatepicker($.datepicker._curInst.input[0]));var i=$.datepicker._get(t,"beforeShow"),n=i?i.apply(e,[e,t]):{};if(!1!==n){extendRemove(t.settings,n),t.lastVal=null,$.datepicker._lastInput=e,$.datepicker._setDateFromField(t),$.datepicker._inDialog&&(e.value=""),$.datepicker._pos||($.datepicker._pos=$.datepicker._findPos(e),$.datepicker._pos[1]+=e.offsetHeight);var a=!1;$(e).parents().each((function(){return a|="fixed"==$(this).css("position"),!a}));var o={left:$.datepicker._pos[0],top:$.datepicker._pos[1]};if($.datepicker._pos=null,t.dpDiv.empty(),t.dpDiv.css({position:"absolute",display:"block",top:"-1000px"}),$.datepicker._updateDatepicker(t),o=$.datepicker._checkOffset(t,o,a),t.dpDiv.css({position:$.datepicker._inDialog&&$.blockUI?"static":a?"fixed":"absolute",display:"none",left:o.left+"px",top:o.top+"px"}),!t.inline){var s=$.datepicker._get(t,"showAnim"),r=$.datepicker._get(t,"duration"),l=function(){var e=t.dpDiv.find("iframe.ui-datepicker-cover");if(e.length){var i=$.datepicker._getBorders(t.dpDiv);e.css({left:-i[0],top:-i[1],width:t.dpDiv.outerWidth(),height:t.dpDiv.outerHeight()})}};t.dpDiv.zIndex($(e).zIndex()+1),$.datepicker._datepickerShowing=!0,$.effects&&($.effects.effect[s]||$.effects[s])?t.dpDiv.show(s,$.datepicker._get(t,"showOptions"),r,l):t.dpDiv[s||"show"](s?r:null,l),s&&r||l(),t.input.is(":visible")&&!t.input.is(":disabled")&&t.input.focus(),$.datepicker._curInst=t}}}},_updateDatepicker:function(e){this.maxRows=4;var t=$.datepicker._getBorders(e.dpDiv);instActive=e,e.dpDiv.empty().append(this._generateHTML(e)),this._attachHandlers(e);var i=e.dpDiv.find("iframe.ui-datepicker-cover");i.length&&i.css({left:-t[0],top:-t[1],width:e.dpDiv.outerWidth(),height:e.dpDiv.outerHeight()}),e.dpDiv.find("."+this._dayOverClass+" a").mouseover();var n=this._getNumberOfMonths(e),a=n[1],o=17;if(e.dpDiv.removeClass("ui-datepicker-multi-2 ui-datepicker-multi-3 ui-datepicker-multi-4").width(""),a>1&&e.dpDiv.addClass("ui-datepicker-multi-"+a).css("width",o*a+"em"),e.dpDiv[(1!=n[0]||1!=n[1]?"add":"remove")+"Class"]("ui-datepicker-multi"),e.dpDiv[(this._get(e,"isRTL")?"add":"remove")+"Class"]("ui-datepicker-rtl"),e==$.datepicker._curInst&&$.datepicker._datepickerShowing&&e.input&&e.input.is(":visible")&&!e.input.is(":disabled")&&e.input[0]!=document.activeElement&&e.input.focus(),e.yearshtml){var s=e.yearshtml;setTimeout((function(){s===e.yearshtml&&e.yearshtml&&e.dpDiv.find("select.ui-datepicker-year:first").replaceWith(e.yearshtml),s=e.yearshtml=null}),0)}},_getBorders:function(e){var t=function(e){return{thin:1,medium:2,thick:3}[e]||e};return[parseFloat(t(e.css("border-left-width"))),parseFloat(t(e.css("border-top-width")))]},_checkOffset:function(e,t,i){var n=e.dpDiv.outerWidth(),a=e.dpDiv.outerHeight(),o=e.input?e.input.outerWidth():0,s=e.input?e.input.outerHeight():0,r=document.documentElement.clientWidth+(i?0:$(document).scrollLeft()),l=document.documentElement.clientHeight+(i?0:$(document).scrollTop());return t.left-=this._get(e,"isRTL")?n-o:0,t.left-=i&&t.left==e.input.offset().left?$(document).scrollLeft():0,t.top-=i&&t.top==e.input.offset().top+s?$(document).scrollTop():0,t.left-=Math.min(t.left,t.left+n>r&&r>n?Math.abs(t.left+n-r):0),t.top-=Math.min(t.top,t.top+a>l&&l>a?Math.abs(a+s):0),t},_findPos:function(e){var t=this._getInst(e),i=this._get(t,"isRTL");while(e&&("hidden"==e.type||1!=e.nodeType||$.expr.filters.hidden(e)))e=e[i?"previousSibling":"nextSibling"];var n=$(e).offset();return[n.left,n.top]},_hideDatepicker:function(e){var t=this._curInst;if(t&&(!e||t==$.data(e,PROP_NAME))&&this._datepickerShowing){var i=this._get(t,"showAnim"),n=this._get(t,"duration"),a=function(){$.datepicker._tidyDialog(t)};$.effects&&($.effects.effect[i]||$.effects[i])?t.dpDiv.hide(i,$.datepicker._get(t,"showOptions"),n,a):t.dpDiv["slideDown"==i?"slideUp":"fadeIn"==i?"fadeOut":"hide"](i?n:null,a),i||a(),this._datepickerShowing=!1;var o=this._get(t,"onClose");o&&o.apply(t.input?t.input[0]:null,[t.input?t.input.val():"",t]),this._lastInput=null,this._inDialog&&(this._dialogInput.css({position:"absolute",left:"0",top:"-100px"}),$.blockUI&&($.unblockUI(),$("body").append(this.dpDiv))),this._inDialog=!1}},_tidyDialog:function(e){e.dpDiv.removeClass(this._dialogClass).unbind(".ui-datepicker-calendar")},_checkExternalClick:function(e){if($.datepicker._curInst){var t=$(e.target),i=$.datepicker._getInst(t[0]);(t[0].id==$.datepicker._mainDivId||0!=t.parents("#"+$.datepicker._mainDivId).length||t.hasClass($.datepicker.markerClassName)||t.closest("."+$.datepicker._triggerClass).length||!$.datepicker._datepickerShowing||$.datepicker._inDialog&&$.blockUI)&&(!t.hasClass($.datepicker.markerClassName)||$.datepicker._curInst==i)||$.datepicker._hideDatepicker()}},_adjustDate:function(e,t,i){var n=$(e),a=this._getInst(n[0]);this._isDisabledDatepicker(n[0])||(this._adjustInstDate(a,t+("M"==i?this._get(a,"showCurrentAtPos"):0),i),this._updateDatepicker(a))},_gotoToday:function(e){var t=$(e),i=this._getInst(t[0]);if(this._get(i,"gotoCurrent")&&i.currentDay)i.selectedDay=i.currentDay,i.drawMonth=i.selectedMonth=i.currentMonth,i.drawYear=i.selectedYear=i.currentYear;else{var n=new Date;i.selectedDay=n.getDate(),i.drawMonth=i.selectedMonth=n.getMonth(),i.drawYear=i.selectedYear=n.getFullYear()}this._notifyChange(i),this._adjustDate(t)},_selectMonthYear:function(e,t,i){var n=$(e),a=this._getInst(n[0]);a["selected"+("M"==i?"Month":"Year")]=a["draw"+("M"==i?"Month":"Year")]=parseInt(t.options[t.selectedIndex].value,10),this._notifyChange(a),this._adjustDate(n)},_selectDay:function(e,t,i,n){var a=$(e);if(!$(n).hasClass(this._unselectableClass)&&!this._isDisabledDatepicker(a[0])){var o=this._getInst(a[0]);o.selectedDay=o.currentDay=$("a",n).html(),o.selectedMonth=o.currentMonth=t,o.selectedYear=o.currentYear=i,this._selectDate(e,this._formatDate(o,o.currentDay,o.currentMonth,o.currentYear))}},_clearDate:function(e){var t=$(e);this._getInst(t[0]);this._selectDate(t,"")},_selectDate:function(e,t){var i=$(e),n=this._getInst(i[0]);t=null!=t?t:this._formatDate(n),n.input&&n.input.val(t),this._updateAlternate(n);var a=this._get(n,"onSelect");a?a.apply(n.input?n.input[0]:null,[t,n]):n.input&&n.input.trigger("change"),n.inline?this._updateDatepicker(n):(this._hideDatepicker(),this._lastInput=n.input[0],"object"!=typeof n.input[0]&&n.input.focus(),this._lastInput=null)},_updateAlternate:function(e){var t=this._get(e,"altField");if(t){var i=this._get(e,"altFormat")||this._get(e,"dateFormat"),n=this._getDate(e),a=this.formatDate(i,n,this._getFormatConfig(e));$(t).each((function(){$(this).val(a)}))}},noWeekends:function(e){var t=e.getDay();return[t>0&&t<6,""]},iso8601Week:function(e){var t=new Date(e.getTime());t.setDate(t.getDate()+4-(t.getDay()||7));var i=t.getTime();return t.setMonth(0),t.setDate(1),Math.floor(Math.round((i-t)/864e5)/7)+1},parseDate:function(e,t,i){if(null==e||null==t)throw"Invalid arguments";if(t="object"==typeof t?t.toString():t+"",""==t)return null;var n=(i?i.shortYearCutoff:null)||this._defaults.shortYearCutoff;n="string"!=typeof n?n:(new Date).getFullYear()%100+parseInt(n,10);for(var a=(i?i.dayNamesShort:null)||this._defaults.dayNamesShort,o=(i?i.dayNames:null)||this._defaults.dayNames,s=(i?i.monthNamesShort:null)||this._defaults.monthNamesShort,r=(i?i.monthNames:null)||this._defaults.monthNames,l=-1,c=-1,h=-1,d=-1,u=!1,p=function(t){var i=b+1<e.length&&e.charAt(b+1)==t;return i&&b++,i},f=function(e){var i=p(e),n="@"==e?14:"!"==e?20:"y"==e&&i?4:"o"==e?3:2,a=new RegExp("^\\d{1,"+n+"}"),o=t.substring(v).match(a);if(!o)throw"Missing number at position "+v;return v+=o[0].length,parseInt(o[0],10)},g=function(e,i,n){var a=$.map(p(e)?n:i,(function(e,t){return[[t,e]]})).sort((function(e,t){return-(e[1].length-t[1].length)})),o=-1;if($.each(a,(function(e,i){var n=i[1];if(t.substr(v,n.length).toLowerCase()==n.toLowerCase())return o=i[0],v+=n.length,!1})),-1!=o)return o+1;throw"Unknown name at position "+v},m=function(){if(t.charAt(v)!=e.charAt(b))throw"Unexpected literal at position "+v;v++},v=0,b=0;b<e.length;b++)if(u)"'"!=e.charAt(b)||p("'")?m():u=!1;else switch(e.charAt(b)){case"d":h=f("d");break;case"D":g("D",a,o);break;case"o":d=f("o");break;case"m":c=f("m");break;case"M":c=g("M",s,r);break;case"y":l=f("y");break;case"@":var w=new Date(f("@"));l=w.getFullYear(),c=w.getMonth()+1,h=w.getDate();break;case"!":w=new Date((f("!")-this._ticksTo1970)/1e4);l=w.getFullYear(),c=w.getMonth()+1,h=w.getDate();break;case"'":p("'")?m():u=!0;break;default:m()}if(v<t.length){var y=t.substr(v);if(!/^\s+/.test(y))throw"Extra/unparsed characters found in date: "+y}if(-1==l?l=(new Date).getFullYear():l<100&&(l+=(new Date).getFullYear()-(new Date).getFullYear()%100+(l<=n?0:-100)),d>-1){c=1,h=d;do{var C=this._getDaysInMonth(l,c-1);if(h<=C)break;c++,h-=C}while(1)}w=this._daylightSavingAdjust(new Date(l,c-1,h));if(w.getFullYear()!=l||w.getMonth()+1!=c||w.getDate()!=h)throw"Invalid date";return w},ATOM:"yy-mm-dd",COOKIE:"D, dd M yy",ISO_8601:"yy-mm-dd",RFC_822:"D, d M y",RFC_850:"DD, dd-M-y",RFC_1036:"D, d M y",RFC_1123:"D, d M yy",RFC_2822:"D, d M yy",RSS:"D, d M y",TICKS:"!",TIMESTAMP:"@",W3C:"yy-mm-dd",_ticksTo1970:24*(718685+Math.floor(492.5)-Math.floor(19.7)+Math.floor(4.925))*60*60*1e7,formatDate:function(e,t,i){if(!t)return"";var n=(i?i.dayNamesShort:null)||this._defaults.dayNamesShort,a=(i?i.dayNames:null)||this._defaults.dayNames,o=(i?i.monthNamesShort:null)||this._defaults.monthNamesShort,s=(i?i.monthNames:null)||this._defaults.monthNames,r=function(t){var i=u+1<e.length&&e.charAt(u+1)==t;return i&&u++,i},l=function(e,t,i){var n=""+t;if(r(e))while(n.length<i)n="0"+n;return n},c=function(e,t,i,n){return r(e)?n[t]:i[t]},h="",d=!1;if(t)for(var u=0;u<e.length;u++)if(d)"'"!=e.charAt(u)||r("'")?h+=e.charAt(u):d=!1;else switch(e.charAt(u)){case"d":h+=l("d",t.getDate(),2);break;case"D":h+=c("D",t.getDay(),n,a);break;case"o":h+=l("o",Math.round((new Date(t.getFullYear(),t.getMonth(),t.getDate()).getTime()-new Date(t.getFullYear(),0,0).getTime())/864e5),3);break;case"m":h+=l("m",t.getMonth()+1,2);break;case"M":h+=c("M",t.getMonth(),o,s);break;case"y":h+=r("y")?t.getFullYear():(t.getYear()%100<10?"0":"")+t.getYear()%100;break;case"@":h+=t.getTime();break;case"!":h+=1e4*t.getTime()+this._ticksTo1970;break;case"'":r("'")?h+="'":d=!0;break;default:h+=e.charAt(u)}return h},_possibleChars:function(e){for(var t="",i=!1,n=function(t){var i=a+1<e.length&&e.charAt(a+1)==t;return i&&a++,i},a=0;a<e.length;a++)if(i)"'"!=e.charAt(a)||n("'")?t+=e.charAt(a):i=!1;else switch(e.charAt(a)){case"d":case"m":case"y":case"@":t+="0123456789";break;case"D":case"M":return null;case"'":n("'")?t+="'":i=!0;break;default:t+=e.charAt(a)}return t},_get:function(e,t){return e.settings[t]!==undefined?e.settings[t]:this._defaults[t]},_setDateFromField:function(e,t){if(e.input.val()!=e.lastVal){var i,n,a=this._get(e,"dateFormat"),o=e.lastVal=e.input?e.input.val():null;i=n=this._getDefaultDate(e);var s=this._getFormatConfig(e);try{i=this.parseDate(a,o,s)||n}catch(event){this.log(event),o=t?"":o}e.selectedDay=i.getDate(),e.drawMonth=e.selectedMonth=i.getMonth(),e.drawYear=e.selectedYear=i.getFullYear(),e.currentDay=o?i.getDate():0,e.currentMonth=o?i.getMonth():0,e.currentYear=o?i.getFullYear():0,this._adjustInstDate(e)}},_getDefaultDate:function(e){return this._restrictMinMax(e,this._determineDate(e,this._get(e,"defaultDate"),new Date))},_determineDate:function(e,t,i){var n=function(e){var t=new Date;return t.setDate(t.getDate()+e),t},a=function(t){try{return $.datepicker.parseDate($.datepicker._get(e,"dateFormat"),t,$.datepicker._getFormatConfig(e))}catch(l){}var i=(t.toLowerCase().match(/^c/)?$.datepicker._getDate(e):null)||new Date,n=i.getFullYear(),a=i.getMonth(),o=i.getDate(),s=/([+-]?[0-9]+)\s*(d|D|w|W|m|M|y|Y)?/g,r=s.exec(t);while(r){switch(r[2]||"d"){case"d":case"D":o+=parseInt(r[1],10);break;case"w":case"W":o+=7*parseInt(r[1],10);break;case"m":case"M":a+=parseInt(r[1],10),o=Math.min(o,$.datepicker._getDaysInMonth(n,a));break;case"y":case"Y":n+=parseInt(r[1],10),o=Math.min(o,$.datepicker._getDaysInMonth(n,a));break}r=s.exec(t)}return new Date(n,a,o)},o=null==t||""===t?i:"string"==typeof t?a(t):"number"==typeof t?isNaN(t)?i:n(t):new Date(t.getTime());return o=o&&"Invalid Date"==o.toString()?i:o,o&&(o.setHours(0),o.setMinutes(0),o.setSeconds(0),o.setMilliseconds(0)),this._daylightSavingAdjust(o)},_daylightSavingAdjust:function(e){return e?(e.setHours(e.getHours()>12?e.getHours()+2:0),e):null},_setDate:function(e,t,i){var n=!t,a=e.selectedMonth,o=e.selectedYear,s=this._restrictMinMax(e,this._determineDate(e,t,new Date));e.selectedDay=e.currentDay=s.getDate(),e.drawMonth=e.selectedMonth=e.currentMonth=s.getMonth(),e.drawYear=e.selectedYear=e.currentYear=s.getFullYear(),a==e.selectedMonth&&o==e.selectedYear||i||this._notifyChange(e),this._adjustInstDate(e),e.input&&e.input.val(n?"":this._formatDate(e))},_getDate:function(e){var t=!e.currentYear||e.input&&""==e.input.val()?null:this._daylightSavingAdjust(new Date(e.currentYear,e.currentMonth,e.currentDay));return t},_attachHandlers:function(e){var t=this._get(e,"stepMonths"),i="#"+e.id.replace(/\\\\/g,"\\");e.dpDiv.find("[data-handler]").map((function(){var e={prev:function(){window["DP_jQuery_"+dpuuid].datepicker._adjustDate(i,-t,"M")},next:function(){window["DP_jQuery_"+dpuuid].datepicker._adjustDate(i,+t,"M")},hide:function(){window["DP_jQuery_"+dpuuid].datepicker._hideDatepicker()},today:function(){window["DP_jQuery_"+dpuuid].datepicker._gotoToday(i)},selectDay:function(){return window["DP_jQuery_"+dpuuid].datepicker._selectDay(i,+this.getAttribute("data-month"),+this.getAttribute("data-year"),this),!1},selectMonth:function(){return window["DP_jQuery_"+dpuuid].datepicker._selectMonthYear(i,this,"M"),!1},selectYear:function(){return window["DP_jQuery_"+dpuuid].datepicker._selectMonthYear(i,this,"Y"),!1}};$(this).bind(this.getAttribute("data-event"),e[this.getAttribute("data-handler")])}))},_generateHTML:function(e){var t=new Date;t=this._daylightSavingAdjust(new Date(t.getFullYear(),t.getMonth(),t.getDate()));var i=this._get(e,"isRTL"),n=this._get(e,"showButtonPanel"),a=this._get(e,"hideIfNoPrevNext"),o=this._get(e,"navigationAsDateFormat"),s=this._getNumberOfMonths(e),r=this._get(e,"showCurrentAtPos"),l=this._get(e,"stepMonths"),c=1!=s[0]||1!=s[1],h=this._daylightSavingAdjust(e.currentDay?new Date(e.currentYear,e.currentMonth,e.currentDay):new Date(9999,9,9)),d=this._getMinMaxDate(e,"min"),u=this._getMinMaxDate(e,"max"),p=e.drawMonth-r,f=e.drawYear;if(p<0&&(p+=12,f--),u){var g=this._daylightSavingAdjust(new Date(u.getFullYear(),u.getMonth()-s[0]*s[1]+1,u.getDate()));g=d&&g<d?d:g;while(this._daylightSavingAdjust(new Date(f,p,1))>g)p--,p<0&&(p=11,f--)}e.drawMonth=p,e.drawYear=f;var m=this._get(e,"prevText");m=o?this.formatDate(m,this._daylightSavingAdjust(new Date(f,p-l,1)),this._getFormatConfig(e)):m;var v=this._canAdjustMonth(e,-1,f,p)?'<a class="ui-datepicker-prev ui-corner-all" data-handler="prev" data-event="click" title="'+m+'"><span class="ui-icon ui-icon-circle-triangle-'+(i?"e":"w")+'">'+m+"</span></a>":a?"":'<a class="ui-datepicker-prev ui-corner-all ui-state-disabled" title="'+m+'"><span class="ui-icon ui-icon-circle-triangle-'+(i?"e":"w")+'">'+m+"</span></a>",b=this._get(e,"nextText");b=o?this.formatDate(b,this._daylightSavingAdjust(new Date(f,p+l,1)),this._getFormatConfig(e)):b;var w=this._canAdjustMonth(e,1,f,p)?'<a class="ui-datepicker-next ui-corner-all" data-handler="next" data-event="click" title="'+b+'"><span class="ui-icon ui-icon-circle-triangle-'+(i?"w":"e")+'">'+b+"</span></a>":a?"":'<a class="ui-datepicker-next ui-corner-all ui-state-disabled" title="'+b+'"><span class="ui-icon ui-icon-circle-triangle-'+(i?"w":"e")+'">'+b+"</span></a>",y=this._get(e,"currentText"),C=this._get(e,"gotoCurrent")&&e.currentDay?h:t;y=o?this.formatDate(y,C,this._getFormatConfig(e)):y;var _=e.inline?"":'<button type="button" class="ui-datepicker-close ui-state-default ui-priority-primary ui-corner-all" data-handler="hide" data-event="click">'+this._get(e,"closeText")+"</button>",x=n?'<div class="ui-datepicker-buttonpane ui-widget-content">'+(i?_:"")+(this._isInRange(e,C)?'<button type="button" class="ui-datepicker-current ui-state-default ui-priority-secondary ui-corner-all" data-handler="today" data-event="click">'+y+"</button>":"")+(i?"":_)+"</div>":"",k=parseInt(this._get(e,"firstDay"),10);k=isNaN(k)?0:k;for(var S=this._get(e,"showWeek"),T=this._get(e,"dayNames"),D=(this._get(e,"dayNamesShort"),this._get(e,"dayNamesMin")),N=this._get(e,"monthNames"),R=this._get(e,"monthNamesShort"),I=this._get(e,"beforeShowDay"),P=this._get(e,"showOtherMonths"),A=this._get(e,"selectOtherMonths"),E=(this._get(e,"calculateWeek")||this.iso8601Week,this._getDefaultDate(e)),z="",M=0;M<s[0];M++){var H="";this.maxRows=4;for(var F=0;F<s[1];F++){var O=this._daylightSavingAdjust(new Date(f,p,e.selectedDay)),W=" ui-corner-all",L="";if(c){if(L+='<div class="ui-datepicker-group',s[1]>1)switch(F){case 0:L+=" ui-datepicker-group-first",W=" ui-corner-"+(i?"right":"left");break;case s[1]-1:L+=" ui-datepicker-group-last",W=" ui-corner-"+(i?"left":"right");break;default:L+=" ui-datepicker-group-middle",W="";break}L+='">'}L+='<div class="ui-datepicker-header ui-widget-header ui-helper-clearfix'+W+'">'+(/all|left/.test(W)&&0==M?i?w:v:"")+(/all|right/.test(W)&&0==M?i?v:w:"")+this._generateMonthYearHeader(e,p,f,d,u,M>0||F>0,N,R)+'</div><table class="ui-datepicker-calendar"><thead><tr>';for(var j=S?'<th class="ui-datepicker-week-col">'+this._get(e,"weekHeader")+"</th>":"",B=0;B<7;B++){var G=(B+k)%7;j+="<th"+((B+k+6)%7>=5?' class="ui-datepicker-week-end"':"")+'><span title="'+T[G]+'">'+D[G]+"</span></th>"}L+=j+"</tr></thead><tbody>";var V=this._getDaysInMonth(f,p);f==e.selectedYear&&p==e.selectedMonth&&(e.selectedDay=Math.min(e.selectedDay,V));var q=(this._getFirstDayOfMonth(f,p)-k+7)%7,Y=Math.ceil((q+V)/7),U=c&&this.maxRows>Y?this.maxRows:Y;this.maxRows=U;for(var K=this._daylightSavingAdjust(new Date(f,p,1-q)),X=0;X<U;X++){L+="<tr>";var Q=S?'<td class="ui-datepicker-week-col">'+this._get(e,"calculateWeek")(K)+"</td>":"";for(B=0;B<7;B++){var J=I?I.apply(e.input?e.input[0]:null,[K]):[!0,""],Z=K.getMonth()!=p,ee=Z&&!A||!J[0]||d&&K<d||u&&K>u;Q+='<td class="'+((B+k+6)%7>=5?" ui-datepicker-week-end":"")+(Z?" ui-datepicker-other-month":"")+(K.getTime()==O.getTime()&&p==e.selectedMonth&&e._keyEvent||E.getTime()==K.getTime()&&E.getTime()==O.getTime()?" "+this._dayOverClass:"")+(ee?" "+this._unselectableClass+" ui-state-disabled":"")+(Z&&!P?"":" "+J[1]+(K.getTime()==h.getTime()?" "+this._currentClass:"")+(K.getTime()==t.getTime()?" ui-datepicker-today":""))+'"'+(Z&&!P||!J[2]?"":' title="'+J[2]+'"')+(ee?"":' data-handler="selectDay" data-event="click" data-month="'+K.getMonth()+'" data-year="'+K.getFullYear()+'"')+">"+(Z&&!P?"&#xa0;":ee?'<span class="ui-state-default">'+K.getDate()+"</span>":'<a class="ui-state-default'+(K.getTime()==t.getTime()?" ui-state-highlight":"")+(K.getTime()==h.getTime()?" ui-state-active":"")+(Z?" ui-priority-secondary":"")+'" href="#">'+K.getDate()+"</a>")+"</td>",K.setDate(K.getDate()+1),K=this._daylightSavingAdjust(K)}L+=Q+"</tr>"}p++,p>11&&(p=0,f++),L+="</tbody></table>"+(c?"</div>"+(s[0]>0&&F==s[1]-1?'<div class="ui-datepicker-row-break"></div>':""):""),H+=L}z+=H}return z+=x+($.ui.ie6&&!e.inline?'<iframe src="javascript:false;" class="ui-datepicker-cover" frameborder="0"></iframe>':""),e._keyEvent=!1,z},_generateMonthYearHeader:function(e,t,i,n,a,o,s,r){var l=this._get(e,"changeMonth"),c=this._get(e,"changeYear"),h=this._get(e,"showMonthAfterYear"),d='<div class="ui-datepicker-title">',u="";if(o||!l)u+='<span class="ui-datepicker-month">'+s[t]+"</span>";else{var p=n&&n.getFullYear()==i,f=a&&a.getFullYear()==i;u+='<select class="ui-datepicker-month" data-handler="selectMonth" data-event="change">';for(var g=0;g<12;g++)(!p||g>=n.getMonth())&&(!f||g<=a.getMonth())&&(u+='<option value="'+g+'"'+(g==t?' selected="selected"':"")+">"+r[g]+"</option>");u+="</select>"}if(h||(d+=u+(!o&&l&&c?"":"&#xa0;")),!e.yearshtml)if(e.yearshtml="",o||!c)d+='<span class="ui-datepicker-year">'+i+"</span>";else{var m=this._get(e,"yearRange").split(":"),v=(new Date).getFullYear(),b=function(e){var t=e.match(/c[+-].*/)?i+parseInt(e.substring(1),10):e.match(/[+-].*/)?v+parseInt(e,10):parseInt(e,10);return isNaN(t)?v:t},w=b(m[0]),y=Math.max(w,b(m[1]||""));for(w=n?Math.max(w,n.getFullYear()):w,y=a?Math.min(y,a.getFullYear()):y,e.yearshtml+='<select class="ui-datepicker-year" data-handler="selectYear" data-event="change">';w<=y;w++)e.yearshtml+='<option value="'+w+'"'+(w==i?' selected="selected"':"")+">"+w+"</option>";e.yearshtml+="</select>",d+=e.yearshtml,e.yearshtml=null}return d+=this._get(e,"yearSuffix"),h&&(d+=(!o&&l&&c?"":"&#xa0;")+u),d+="</div>",d},_adjustInstDate:function(e,t,i){var n=e.drawYear+("Y"==i?t:0),a=e.drawMonth+("M"==i?t:0),o=Math.min(e.selectedDay,this._getDaysInMonth(n,a))+("D"==i?t:0),s=this._restrictMinMax(e,this._daylightSavingAdjust(new Date(n,a,o)));e.selectedDay=s.getDate(),e.drawMonth=e.selectedMonth=s.getMonth(),e.drawYear=e.selectedYear=s.getFullYear(),"M"!=i&&"Y"!=i||this._notifyChange(e)},_restrictMinMax:function(e,t){var i=this._getMinMaxDate(e,"min"),n=this._getMinMaxDate(e,"max"),a=i&&t<i?i:t;return a=n&&a>n?n:a,a},_notifyChange:function(e){var t=this._get(e,"onChangeMonthYear");t&&t.apply(e.input?e.input[0]:null,[e.selectedYear,e.selectedMonth+1,e])},_getNumberOfMonths:function(e){var t=this._get(e,"numberOfMonths");return null==t?[1,1]:"number"==typeof t?[1,t]:t},_getMinMaxDate:function(e,t){return this._determineDate(e,this._get(e,t+"Date"),null)},_getDaysInMonth:function(e,t){return 32-this._daylightSavingAdjust(new Date(e,t,32)).getDate()},_getFirstDayOfMonth:function(e,t){return new Date(e,t,1).getDay()},_canAdjustMonth:function(e,t,i,n){var a=this._getNumberOfMonths(e),o=this._daylightSavingAdjust(new Date(i,n+(t<0?t:a[0]*a[1]),1));return t<0&&o.setDate(this._getDaysInMonth(o.getFullYear(),o.getMonth())),this._isInRange(e,o)},_isInRange:function(e,t){var i=this._getMinMaxDate(e,"min"),n=this._getMinMaxDate(e,"max");return(!i||t.getTime()>=i.getTime())&&(!n||t.getTime()<=n.getTime())},_getFormatConfig:function(e){var t=this._get(e,"shortYearCutoff");return t="string"!=typeof t?t:(new Date).getFullYear()%100+parseInt(t,10),{shortYearCutoff:t,dayNamesShort:this._get(e,"dayNamesShort"),dayNames:this._get(e,"dayNames"),monthNamesShort:this._get(e,"monthNamesShort"),monthNames:this._get(e,"monthNames")}},_formatDate:function(e,t,i,n){t||(e.currentDay=e.selectedDay,e.currentMonth=e.selectedMonth,e.currentYear=e.selectedYear);var a=t?"object"==typeof t?t:this._daylightSavingAdjust(new Date(n,i,t)):this._daylightSavingAdjust(new Date(e.currentYear,e.currentMonth,e.currentDay));return this.formatDate(this._get(e,"dateFormat"),a,this._getFormatConfig(e))}}),$.fn.datepicker=function(e){if(!this.length)return this;$.datepicker.initialized||($(document).mousedown($.datepicker._checkExternalClick).find(document.body).append($.datepicker.dpDiv),$.datepicker.initialized=!0);var t=Array.prototype.slice.call(arguments,1);return"string"!=typeof e||"isDisabled"!=e&&"getDate"!=e&&"widget"!=e?"option"==e&&2==arguments.length&&"string"==typeof arguments[1]?$.datepicker["_"+e+"Datepicker"].apply($.datepicker,[this[0]].concat(t)):this.each((function(){"string"==typeof e?$.datepicker["_"+e+"Datepicker"].apply($.datepicker,[this].concat(t)):$.datepicker._attachDatepicker(this,e)})):$.datepicker["_"+e+"Datepicker"].apply($.datepicker,[this[0]].concat(t))},$.datepicker=new Datepicker,$.datepicker.initialized=!1,$.datepicker.uuid=(new Date).getTime(),$.datepicker.version="1.9.2",window["DP_jQuery_"+dpuuid]=$}(jQuery),function(e,t){var i="ui-dialog ui-widget ui-widget-content ui-corner-all ",n={buttons:!0,height:!0,maxHeight:!0,maxWidth:!0,minHeight:!0,minWidth:!0,width:!0},a={maxHeight:!0,maxWidth:!0,minHeight:!0,minWidth:!0};e.widget("ui.dialog",{version:"1.9.2",options:{autoOpen:!0,buttons:{},closeOnEscape:!0,closeText:"close",dialogClass:"",draggable:!0,hide:null,height:"auto",maxHeight:!1,maxWidth:!1,minHeight:150,minWidth:150,modal:!1,position:{my:"center",at:"center",of:window,collision:"fit",using:function(t){var i=e(this).css(t).offset().top;i<0&&e(this).css("top",t.top-i)}},resizable:!0,show:null,stack:!0,title:"",width:300,zIndex:1e3},_create:function(){this.originalTitle=this.element.attr("title"),"string"!==typeof this.originalTitle&&(this.originalTitle=""),this.oldPosition={parent:this.element.parent(),index:this.element.parent().children().index(this.element)},this.options.title=this.options.title||this.originalTitle;var t,n,a,o,s,r=this,l=this.options,c=l.title||"&#160;";t=(this.uiDialog=e("<div>")).addClass(i+l.dialogClass).css({display:"none",outline:0,zIndex:l.zIndex}).attr("tabIndex",-1).keydown((function(t){l.closeOnEscape&&!t.isDefaultPrevented()&&t.keyCode&&t.keyCode===e.ui.keyCode.ESCAPE&&(r.close(t),t.preventDefault())})).mousedown((function(e){r.moveToTop(!1,e)})).appendTo("body"),this.element.show().removeAttr("title").addClass("ui-dialog-content ui-widget-content").appendTo(t),n=(this.uiDialogTitlebar=e("<div>")).addClass("ui-dialog-titlebar  ui-widget-header  ui-corner-all  ui-helper-clearfix").bind("mousedown",(function(){t.focus()})).prependTo(t),a=e("<a href='#'></a>").addClass("ui-dialog-titlebar-close  ui-corner-all").attr("role","button").click((function(e){e.preventDefault(),r.close(e)})).appendTo(n),(this.uiDialogTitlebarCloseText=e("<span>")).addClass("ui-icon ui-icon-closethick").text(l.closeText).appendTo(a),o=e("<span>").uniqueId().addClass("ui-dialog-title").html(c).prependTo(n),s=(this.uiDialogButtonPane=e("<div>")).addClass("ui-dialog-buttonpane ui-widget-content ui-helper-clearfix"),(this.uiButtonSet=e("<div>")).addClass("ui-dialog-buttonset").appendTo(s),t.attr({role:"dialog","aria-labelledby":o.attr("id")}),n.find("*").add(n).disableSelection(),this._hoverable(a),this._focusable(a),l.draggable&&e.fn.draggable&&this._makeDraggable(),l.resizable&&e.fn.resizable&&this._makeResizable(),this._createButtons(l.buttons),this._isOpen=!1,e.fn.bgiframe&&t.bgiframe(),this._on(t,{keydown:function(i){if(l.modal&&i.keyCode===e.ui.keyCode.TAB){var n=e(":tabbable",t),a=n.filter(":first"),o=n.filter(":last");return i.target!==o[0]||i.shiftKey?i.target===a[0]&&i.shiftKey?(o.focus(1),!1):void 0:(a.focus(1),!1)}}})},_init:function(){this.options.autoOpen&&this.open()},_destroy:function(){var e,t=this.oldPosition;this.overlay&&this.overlay.destroy(),this.uiDialog.hide(),this.element.removeClass("ui-dialog-content ui-widget-content").hide().appendTo("body"),this.uiDialog.remove(),this.originalTitle&&this.element.attr("title",this.originalTitle),e=t.parent.children().eq(t.index),e.length&&e[0]!==this.element[0]?e.before(this.element):t.parent.append(this.element)},widget:function(){return this.uiDialog},close:function(t){var i,n,a=this;if(this._isOpen&&!1!==this._trigger("beforeClose",t))return this._isOpen=!1,this.overlay&&this.overlay.destroy(),this.options.hide?this._hide(this.uiDialog,this.options.hide,(function(){a._trigger("close",t)})):(this.uiDialog.hide(),this._trigger("close",t)),e.ui.dialog.overlay.resize(),this.options.modal&&(i=0,e(".ui-dialog").each((function(){this!==a.uiDialog[0]&&(n=e(this).css("z-index"),isNaN(n)||(i=Math.max(i,n)))})),e.ui.dialog.maxZ=i),this},isOpen:function(){return this._isOpen},moveToTop:function(t,i){var n,a=this.options;return a.modal&&!t||!a.stack&&!a.modal?this._trigger("focus",i):(a.zIndex>e.ui.dialog.maxZ&&(e.ui.dialog.maxZ=a.zIndex),this.overlay&&(e.ui.dialog.maxZ+=1,e.ui.dialog.overlay.maxZ=e.ui.dialog.maxZ,this.overlay.$el.css("z-index",e.ui.dialog.overlay.maxZ)),n={scrollTop:this.element.scrollTop(),scrollLeft:this.element.scrollLeft()},e.ui.dialog.maxZ+=1,this.uiDialog.css("z-index",e.ui.dialog.maxZ),this.element.attr(n),this._trigger("focus",i),this)},open:function(){if(!this._isOpen){var t,i=this.options,n=this.uiDialog;return this._size(),this._position(i.position),n.show(i.show),this.overlay=i.modal?new e.ui.dialog.overlay(this):null,this.moveToTop(!0),t=this.element.find(":tabbable"),t.length||(t=this.uiDialogButtonPane.find(":tabbable"),t.length||(t=n)),t.eq(0).focus(),this._isOpen=!0,this._trigger("open"),this}},_createButtons:function(t){var i=this,n=!1;this.uiDialogButtonPane.remove(),this.uiButtonSet.empty(),"object"===typeof t&&null!==t&&e.each(t,(function(){return!(n=!0)})),n?(e.each(t,(function(t,n){var a,o;n=e.isFunction(n)?{click:n,text:t}:n,n=e.extend({type:"button"},n),o=n.click,n.click=function(){o.apply(i.element[0],arguments)},a=e("<button></button>",n).appendTo(i.uiButtonSet),e.fn.button&&a.button()})),this.uiDialog.addClass("ui-dialog-buttons"),this.uiDialogButtonPane.appendTo(this.uiDialog)):this.uiDialog.removeClass("ui-dialog-buttons")},_makeDraggable:function(){var t=this,i=this.options;function n(e){return{position:e.position,offset:e.offset}}this.uiDialog.draggable({cancel:".ui-dialog-content, .ui-dialog-titlebar-close",handle:".ui-dialog-titlebar",containment:"document",start:function(i,a){e(this).addClass("ui-dialog-dragging"),t._trigger("dragStart",i,n(a))},drag:function(e,i){t._trigger("drag",e,n(i))},stop:function(a,o){i.position=[o.position.left-t.document.scrollLeft(),o.position.top-t.document.scrollTop()],e(this).removeClass("ui-dialog-dragging"),t._trigger("dragStop",a,n(o)),e.ui.dialog.overlay.resize()}})},_makeResizable:function(i){i=i===t?this.options.resizable:i;var n=this,a=this.options,o=this.uiDialog.css("position"),s="string"===typeof i?i:"n,e,s,w,se,sw,ne,nw";function r(e){return{originalPosition:e.originalPosition,originalSize:e.originalSize,position:e.position,size:e.size}}this.uiDialog.resizable({cancel:".ui-dialog-content",containment:"document",alsoResize:this.element,maxWidth:a.maxWidth,maxHeight:a.maxHeight,minWidth:a.minWidth,minHeight:this._minHeight(),handles:s,start:function(t,i){e(this).addClass("ui-dialog-resizing"),n._trigger("resizeStart",t,r(i))},resize:function(e,t){n._trigger("resize",e,r(t))},stop:function(t,i){e(this).removeClass("ui-dialog-resizing"),a.height=e(this).height(),a.width=e(this).width(),n._trigger("resizeStop",t,r(i)),e.ui.dialog.overlay.resize()}}).css("position",o).find(".ui-resizable-se").addClass("ui-icon ui-icon-grip-diagonal-se")},_minHeight:function(){var e=this.options;return"auto"===e.height?e.minHeight:Math.min(e.minHeight,e.height)},_position:function(t){var i,n=[],a=[0,0];t?(("string"===typeof t||"object"===typeof t&&"0"in t)&&(n=t.split?t.split(" "):[t[0],t[1]],1===n.length&&(n[1]=n[0]),e.each(["left","top"],(function(e,t){+n[e]===n[e]&&(a[e]=n[e],n[e]=t)})),t={my:n[0]+(a[0]<0?a[0]:"+"+a[0])+" "+n[1]+(a[1]<0?a[1]:"+"+a[1]),at:n.join(" ")}),t=e.extend({},e.ui.dialog.prototype.options.position,t)):t=e.ui.dialog.prototype.options.position,i=this.uiDialog.is(":visible"),i||this.uiDialog.show(),this.uiDialog.position(t),i||this.uiDialog.hide()},_setOptions:function(t){var i=this,o={},s=!1;e.each(t,(function(e,t){i._setOption(e,t),e in n&&(s=!0),e in a&&(o[e]=t)})),s&&this._size(),this.uiDialog.is(":data(resizable)")&&this.uiDialog.resizable("option",o)},_setOption:function(t,n){var a,o,s=this.uiDialog;switch(t){case"buttons":this._createButtons(n);break;case"closeText":this.uiDialogTitlebarCloseText.text(""+n);break;case"dialogClass":s.removeClass(this.options.dialogClass).addClass(i+n);break;case"disabled":n?s.addClass("ui-dialog-disabled"):s.removeClass("ui-dialog-disabled");break;case"draggable":a=s.is(":data(draggable)"),a&&!n&&s.draggable("destroy"),!a&&n&&this._makeDraggable();break;case"position":this._position(n);break;case"resizable":o=s.is(":data(resizable)"),o&&!n&&s.resizable("destroy"),o&&"string"===typeof n&&s.resizable("option","handles",n),o||!1===n||this._makeResizable(n);break;case"title":e(".ui-dialog-title",this.uiDialogTitlebar).html(""+(n||"&#160;"));break}this._super(t,n)},_size:function(){var t,i,n,a=this.options,o=this.uiDialog.is(":visible");this.element.show().css({width:"auto",minHeight:0,height:0}),a.minWidth>a.width&&(a.width=a.minWidth),t=this.uiDialog.css({height:"auto",width:a.width}).outerHeight(),i=Math.max(0,a.minHeight-t),"auto"===a.height?e.support.minHeight?this.element.css({minHeight:i,height:"auto"}):(this.uiDialog.show(),n=this.element.css("height","auto").height(),o||this.uiDialog.hide(),this.element.height(Math.max(n,i))):this.element.height(Math.max(a.height-t,0)),this.uiDialog.is(":data(resizable)")&&this.uiDialog.resizable("option","minHeight",this._minHeight())}}),e.extend(e.ui.dialog,{uuid:0,maxZ:0,getTitleId:function(e){var t=e.attr("id");return t||(this.uuid+=1,t=this.uuid),"ui-dialog-title-"+t},overlay:function(t){this.$el=e.ui.dialog.overlay.create(t)}}),e.extend(e.ui.dialog.overlay,{instances:[],oldInstances:[],maxZ:0,events:e.map("focus,mousedown,mouseup,keydown,keypress,click".split(","),(function(e){return e+".dialog-overlay"})).join(" "),create:function(t){0===this.instances.length&&(setTimeout((function(){e.ui.dialog.overlay.instances.length&&e(document).bind(e.ui.dialog.overlay.events,(function(t){if(e(t.target).zIndex()<e.ui.dialog.overlay.maxZ)return!1}))}),1),e(window).bind("resize.dialog-overlay",e.ui.dialog.overlay.resize));var i=this.oldInstances.pop()||e("<div>").addClass("ui-widget-overlay");return e(document).bind("keydown.dialog-overlay",(function(n){var a=e.ui.dialog.overlay.instances;0!==a.length&&a[a.length-1]===i&&t.options.closeOnEscape&&!n.isDefaultPrevented()&&n.keyCode&&n.keyCode===e.ui.keyCode.ESCAPE&&(t.close(n),n.preventDefault())})),i.appendTo(document.body).css({width:this.width(),height:this.height()}),e.fn.bgiframe&&i.bgiframe(),this.instances.push(i),i},destroy:function(t){var i=e.inArray(t,this.instances),n=0;-1!==i&&this.oldInstances.push(this.instances.splice(i,1)[0]),0===this.instances.length&&e([document,window]).unbind(".dialog-overlay"),t.height(0).width(0).remove(),e.each(this.instances,(function(){n=Math.max(n,this.css("z-index"))})),this.maxZ=n},height:function(){var t,i;return e.ui.ie?(t=Math.max(document.documentElement.scrollHeight,document.body.scrollHeight),i=Math.max(document.documentElement.offsetHeight,document.body.offsetHeight),t<i?e(window).height()+"px":t+"px"):e(document).height()+"px"},width:function(){var t,i;return e.ui.ie?(t=Math.max(document.documentElement.scrollWidth,document.body.scrollWidth),i=Math.max(document.documentElement.offsetWidth,document.body.offsetWidth),t<i?e(window).width()+"px":t+"px"):e(document).width()+"px"},resize:function(){var t=e([]);e.each(e.ui.dialog.overlay.instances,(function(){t=t.add(this)})),t.css({width:0,height:0}).css({width:e.ui.dialog.overlay.width(),height:e.ui.dialog.overlay.height()})}}),e.extend(e.ui.dialog.overlay.prototype,{destroy:function(){e.ui.dialog.overlay.destroy(this.$el)}})}(jQuery),function(e,t){var i=/up|down|vertical/,n=/up|left|vertical|horizontal/;e.effects.effect.blind=function(t,a){var o,s,r,l=e(this),c=["position","top","bottom","left","right","height","width"],h=e.effects.setMode(l,t.mode||"hide"),d=t.direction||"up",u=i.test(d),p=u?"height":"width",f=u?"top":"left",g=n.test(d),m={},v="show"===h;l.parent().is(".ui-effects-wrapper")?e.effects.save(l.parent(),c):e.effects.save(l,c),l.show(),o=e.effects.createWrapper(l).css({overflow:"hidden"}),s=o[p](),r=parseFloat(o.css(f))||0,m[p]=v?s:0,g||(l.css(u?"bottom":"right",0).css(u?"top":"left","auto").css({position:"absolute"}),m[f]=v?r:s+r),v&&(o.css(p,0),g||o.css(f,r+s)),o.animate(m,{duration:t.duration,easing:t.easing,queue:!1,complete:function(){"hide"===h&&l.hide(),e.effects.restore(l,c),e.effects.removeWrapper(l),a()}})}}(jQuery),function(e,t){e.effects.effect.bounce=function(t,i){var n,a,o,s=e(this),r=["position","top","bottom","left","right","height","width"],l=e.effects.setMode(s,t.mode||"effect"),c="hide"===l,h="show"===l,d=t.direction||"up",u=t.distance,p=t.times||5,f=2*p+(h||c?1:0),g=t.duration/f,m=t.easing,v="up"===d||"down"===d?"top":"left",b="up"===d||"left"===d,w=s.queue(),y=w.length;for((h||c)&&r.push("opacity"),e.effects.save(s,r),s.show(),e.effects.createWrapper(s),u||(u=s["top"===v?"outerHeight":"outerWidth"]()/3),h&&(o={opacity:1},o[v]=0,s.css("opacity",0).css(v,b?2*-u:2*u).animate(o,g,m)),c&&(u/=Math.pow(2,p-1)),o={},o[v]=0,n=0;n<p;n++)a={},a[v]=(b?"-=":"+=")+u,s.animate(a,g,m).animate(o,g,m),u=c?2*u:u/2;c&&(a={opacity:0},a[v]=(b?"-=":"+=")+u,s.animate(a,g,m)),s.queue((function(){c&&s.hide(),e.effects.restore(s,r),e.effects.removeWrapper(s),i()})),y>1&&w.splice.apply(w,[1,0].concat(w.splice(y,f+1))),s.dequeue()}}(jQuery),function(e,t){e.effects.effect.clip=function(t,i){var n,a,o,s=e(this),r=["position","top","bottom","left","right","height","width"],l=e.effects.setMode(s,t.mode||"hide"),c="show"===l,h=t.direction||"vertical",d="vertical"===h,u=d?"height":"width",p=d?"top":"left",f={};e.effects.save(s,r),s.show(),n=e.effects.createWrapper(s).css({overflow:"hidden"}),a="IMG"===s[0].tagName?n:s,o=a[u](),c&&(a.css(u,0),a.css(p,o/2)),f[u]=c?o:0,f[p]=c?0:o/2,a.animate(f,{queue:!1,duration:t.duration,easing:t.easing,complete:function(){c||s.hide(),e.effects.restore(s,r),e.effects.removeWrapper(s),i()}})}}(jQuery),function(e,t){e.effects.effect.drop=function(t,i){var n,a=e(this),o=["position","top","bottom","left","right","opacity","height","width"],s=e.effects.setMode(a,t.mode||"hide"),r="show"===s,l=t.direction||"left",c="up"===l||"down"===l?"top":"left",h="up"===l||"left"===l?"pos":"neg",d={opacity:r?1:0};e.effects.save(a,o),a.show(),e.effects.createWrapper(a),n=t.distance||a["top"===c?"outerHeight":"outerWidth"](!0)/2,r&&a.css("opacity",0).css(c,"pos"===h?-n:n),d[c]=(r?"pos"===h?"+=":"-=":"pos"===h?"-=":"+=")+n,a.animate(d,{queue:!1,duration:t.duration,easing:t.easing,complete:function(){"hide"===s&&a.hide(),e.effects.restore(a,o),e.effects.removeWrapper(a),i()}})}}(jQuery),function(e,t){e.effects.effect.explode=function(t,i){var n,a,o,s,r,l,c=t.pieces?Math.round(Math.sqrt(t.pieces)):3,h=c,d=e(this),u=e.effects.setMode(d,t.mode||"hide"),p="show"===u,f=d.show().css("visibility","hidden").offset(),g=Math.ceil(d.outerWidth()/h),m=Math.ceil(d.outerHeight()/c),v=[];function b(){v.push(this),v.length===c*h&&w()}for(n=0;n<c;n++)for(s=f.top+n*m,l=n-(c-1)/2,a=0;a<h;a++)o=f.left+a*g,r=a-(h-1)/2,d.clone().appendTo("body").wrap("<div></div>").css({position:"absolute",visibility:"visible",left:-a*g,top:-n*m}).parent().addClass("ui-effects-explode").css({position:"absolute",overflow:"hidden",width:g,height:m,left:o+(p?r*g:0),top:s+(p?l*m:0),opacity:p?0:1}).animate({left:o+(p?0:r*g),top:s+(p?0:l*m),opacity:p?1:0},t.duration||500,t.easing,b);function w(){d.css({visibility:"visible"}),e(v).remove(),p||d.hide(),i()}}}(jQuery),function(e,t){e.effects.effect.fade=function(t,i){var n=e(this),a=e.effects.setMode(n,t.mode||"toggle");n.animate({opacity:a},{queue:!1,duration:t.duration,easing:t.easing,complete:i})}}(jQuery),function(e,t){e.effects.effect.fold=function(t,i){var n,a,o=e(this),s=["position","top","bottom","left","right","height","width"],r=e.effects.setMode(o,t.mode||"hide"),l="show"===r,c="hide"===r,h=t.size||15,d=/([0-9]+)%/.exec(h),u=!!t.horizFirst,p=l!==u,f=p?["width","height"]:["height","width"],g=t.duration/2,m={},v={};e.effects.save(o,s),o.show(),n=e.effects.createWrapper(o).css({overflow:"hidden"}),a=p?[n.width(),n.height()]:[n.height(),n.width()],d&&(h=parseInt(d[1],10)/100*a[c?0:1]),l&&n.css(u?{height:0,width:h}:{height:h,width:0}),m[f[0]]=l?a[0]:h,v[f[1]]=l?a[1]:0,n.animate(m,g,t.easing).animate(v,g,t.easing,(function(){c&&o.hide(),e.effects.restore(o,s),e.effects.removeWrapper(o),i()}))}}(jQuery),function(e,t){e.effects.effect.highlight=function(t,i){var n=e(this),a=["backgroundImage","backgroundColor","opacity"],o=e.effects.setMode(n,t.mode||"show"),s={backgroundColor:n.css("backgroundColor")};"hide"===o&&(s.opacity=0),e.effects.save(n,a),n.show().css({backgroundImage:"none",backgroundColor:t.color||"#ffff99"}).animate(s,{queue:!1,duration:t.duration,easing:t.easing,complete:function(){"hide"===o&&n.hide(),e.effects.restore(n,a),i()}})}}(jQuery),function(e,t){e.effects.effect.pulsate=function(t,i){var n,a=e(this),o=e.effects.setMode(a,t.mode||"show"),s="show"===o,r="hide"===o,l=s||"hide"===o,c=2*(t.times||5)+(l?1:0),h=t.duration/c,d=0,u=a.queue(),p=u.length;for(!s&&a.is(":visible")||(a.css("opacity",0).show(),d=1),n=1;n<c;n++)a.animate({opacity:d},h,t.easing),d=1-d;a.animate({opacity:d},h,t.easing),a.queue((function(){r&&a.hide(),i()})),p>1&&u.splice.apply(u,[1,0].concat(u.splice(p,c+1))),a.dequeue()}}(jQuery),function(e,t){e.effects.effect.puff=function(t,i){var n=e(this),a=e.effects.setMode(n,t.mode||"hide"),o="hide"===a,s=parseInt(t.percent,10)||150,r=s/100,l={height:n.height(),width:n.width(),outerHeight:n.outerHeight(),outerWidth:n.outerWidth()};e.extend(t,{effect:"scale",queue:!1,fade:!0,mode:a,complete:i,percent:o?s:100,from:o?l:{height:l.height*r,width:l.width*r,outerHeight:l.outerHeight*r,outerWidth:l.outerWidth*r}}),n.effect(t)},e.effects.effect.scale=function(t,i){var n=e(this),a=e.extend(!0,{},t),o=e.effects.setMode(n,t.mode||"effect"),s=parseInt(t.percent,10)||(0===parseInt(t.percent,10)||"hide"===o?0:100),r=t.direction||"both",l=t.origin,c={height:n.height(),width:n.width(),outerHeight:n.outerHeight(),outerWidth:n.outerWidth()},h={y:"horizontal"!==r?s/100:1,x:"vertical"!==r?s/100:1};a.effect="size",a.queue=!1,a.complete=i,"effect"!==o&&(a.origin=l||["middle","center"],a.restore=!0),a.from=t.from||("show"===o?{height:0,width:0,outerHeight:0,outerWidth:0}:c),a.to={height:c.height*h.y,width:c.width*h.x,outerHeight:c.outerHeight*h.y,outerWidth:c.outerWidth*h.x},a.fade&&("show"===o&&(a.from.opacity=0,a.to.opacity=1),"hide"===o&&(a.from.opacity=1,a.to.opacity=0)),n.effect(a)},e.effects.effect.size=function(t,i){var n,a,o,s=e(this),r=["position","top","bottom","left","right","width","height","overflow","opacity"],l=["position","top","bottom","left","right","overflow","opacity"],c=["width","height","overflow"],h=["fontSize"],d=["borderTopWidth","borderBottomWidth","paddingTop","paddingBottom"],u=["borderLeftWidth","borderRightWidth","paddingLeft","paddingRight"],p=e.effects.setMode(s,t.mode||"effect"),f=t.restore||"effect"!==p,g=t.scale||"both",m=t.origin||["middle","center"],v=s.css("position"),b=f?r:l,w={height:0,width:0,outerHeight:0,outerWidth:0};"show"===p&&s.show(),n={height:s.height(),width:s.width(),outerHeight:s.outerHeight(),outerWidth:s.outerWidth()},"toggle"===t.mode&&"show"===p?(s.from=t.to||w,s.to=t.from||n):(s.from=t.from||("show"===p?w:n),s.to=t.to||("hide"===p?w:n)),o={from:{y:s.from.height/n.height,x:s.from.width/n.width},to:{y:s.to.height/n.height,x:s.to.width/n.width}},"box"!==g&&"both"!==g||(o.from.y!==o.to.y&&(b=b.concat(d),s.from=e.effects.setTransition(s,d,o.from.y,s.from),s.to=e.effects.setTransition(s,d,o.to.y,s.to)),o.from.x!==o.to.x&&(b=b.concat(u),s.from=e.effects.setTransition(s,u,o.from.x,s.from),s.to=e.effects.setTransition(s,u,o.to.x,s.to))),"content"!==g&&"both"!==g||o.from.y!==o.to.y&&(b=b.concat(h).concat(c),s.from=e.effects.setTransition(s,h,o.from.y,s.from),s.to=e.effects.setTransition(s,h,o.to.y,s.to)),e.effects.save(s,b),s.show(),e.effects.createWrapper(s),s.css("overflow","hidden").css(s.from),m&&(a=e.effects.getBaseline(m,n),s.from.top=(n.outerHeight-s.outerHeight())*a.y,s.from.left=(n.outerWidth-s.outerWidth())*a.x,s.to.top=(n.outerHeight-s.to.outerHeight)*a.y,s.to.left=(n.outerWidth-s.to.outerWidth)*a.x),s.css(s.from),"content"!==g&&"both"!==g||(d=d.concat(["marginTop","marginBottom"]).concat(h),u=u.concat(["marginLeft","marginRight"]),c=r.concat(d).concat(u),s.find("*[width]").each((function(){var i=e(this),n={height:i.height(),width:i.width(),outerHeight:i.outerHeight(),outerWidth:i.outerWidth()};f&&e.effects.save(i,c),i.from={height:n.height*o.from.y,width:n.width*o.from.x,outerHeight:n.outerHeight*o.from.y,outerWidth:n.outerWidth*o.from.x},i.to={height:n.height*o.to.y,width:n.width*o.to.x,outerHeight:n.height*o.to.y,outerWidth:n.width*o.to.x},o.from.y!==o.to.y&&(i.from=e.effects.setTransition(i,d,o.from.y,i.from),i.to=e.effects.setTransition(i,d,o.to.y,i.to)),o.from.x!==o.to.x&&(i.from=e.effects.setTransition(i,u,o.from.x,i.from),i.to=e.effects.setTransition(i,u,o.to.x,i.to)),i.css(i.from),i.animate(i.to,t.duration,t.easing,(function(){f&&e.effects.restore(i,c)}))}))),s.animate(s.to,{queue:!1,duration:t.duration,easing:t.easing,complete:function(){0===s.to.opacity&&s.css("opacity",s.from.opacity),"hide"===p&&s.hide(),e.effects.restore(s,b),f||("static"===v?s.css({position:"relative",top:s.to.top,left:s.to.left}):e.each(["top","left"],(function(e,t){s.css(t,(function(t,i){var n=parseInt(i,10),a=e?s.to.left:s.to.top;return"auto"===i?a+"px":n+a+"px"}))}))),e.effects.removeWrapper(s),i()}})}}(jQuery),function(e,t){e.effects.effect.shake=function(t,i){var n,a=e(this),o=["position","top","bottom","left","right","height","width"],s=e.effects.setMode(a,t.mode||"effect"),r=t.direction||"left",l=t.distance||20,c=t.times||3,h=2*c+1,d=Math.round(t.duration/h),u="up"===r||"down"===r?"top":"left",p="up"===r||"left"===r,f={},g={},m={},v=a.queue(),b=v.length;for(e.effects.save(a,o),a.show(),e.effects.createWrapper(a),f[u]=(p?"-=":"+=")+l,g[u]=(p?"+=":"-=")+2*l,m[u]=(p?"-=":"+=")+2*l,a.animate(f,d,t.easing),n=1;n<c;n++)a.animate(g,d,t.easing).animate(m,d,t.easing);a.animate(g,d,t.easing).animate(f,d/2,t.easing).queue((function(){"hide"===s&&a.hide(),e.effects.restore(a,o),e.effects.removeWrapper(a),i()})),b>1&&v.splice.apply(v,[1,0].concat(v.splice(b,h+1))),a.dequeue()}}(jQuery),function(e,t){e.effects.effect.slide=function(t,i){var n,a=e(this),o=["position","top","bottom","left","right","width","height"],s=e.effects.setMode(a,t.mode||"show"),r="show"===s,l=t.direction||"left",c="up"===l||"down"===l?"top":"left",h="up"===l||"left"===l,d={};e.effects.save(a,o),a.show(),n=t.distance||a["top"===c?"outerHeight":"outerWidth"](!0),e.effects.createWrapper(a).css({overflow:"hidden"}),r&&a.css(c,h?isNaN(n)?"-"+n:-n:n),d[c]=(r?h?"+=":"-=":h?"-=":"+=")+n,a.animate(d,{queue:!1,duration:t.duration,easing:t.easing,complete:function(){"hide"===s&&a.hide(),e.effects.restore(a,o),e.effects.removeWrapper(a),i()}})}}(jQuery),function(e,t){e.effects.effect.transfer=function(t,i){var n=e(this),a=e(t.to),o="fixed"===a.css("position"),s=e("body"),r=o?s.scrollTop():0,l=o?s.scrollLeft():0,c=a.offset(),h={top:c.top-r,left:c.left-l,height:a.innerHeight(),width:a.innerWidth()},d=n.offset(),u=e('<div class="ui-effects-transfer"></div>').appendTo(document.body).addClass(t.className).css({top:d.top-r,left:d.left-l,height:n.innerHeight(),width:n.innerWidth(),position:o?"fixed":"absolute"}).animate(h,t.duration,t.easing,(function(){u.remove(),i()}))}}(jQuery),function(e,t){var i=!1;e.widget("ui.menu",{version:"1.9.2",defaultElement:"<ul>",delay:300,options:{icons:{submenu:"ui-icon-carat-1-e"},menus:"ul",position:{my:"left top",at:"right top"},role:"menu",blur:null,focus:null,select:null},_create:function(){this.activeMenu=this.element,this.element.uniqueId().addClass("ui-menu ui-widget ui-widget-content ui-corner-all").toggleClass("ui-menu-icons",!!this.element.find(".ui-icon").length).attr({role:this.options.role,tabIndex:0}).bind("click"+this.eventNamespace,e.proxy((function(e){this.options.disabled&&e.preventDefault()}),this)),this.options.disabled&&this.element.addClass("ui-state-disabled").attr("aria-disabled","true"),this._on({"mousedown .ui-menu-item > a":function(e){e.preventDefault()},"click .ui-state-disabled > a":function(e){e.preventDefault()},"click .ui-menu-item:has(a)":function(t){var n=e(t.target).closest(".ui-menu-item");!i&&n.not(".ui-state-disabled").length&&(i=!0,this.select(t),n.has(".ui-menu").length?this.expand(t):this.element.is(":focus")||(this.element.trigger("focus",[!0]),this.active&&1===this.active.parents(".ui-menu").length&&clearTimeout(this.timer)))},"mouseenter .ui-menu-item":function(t){var i=e(t.currentTarget);i.siblings().children(".ui-state-active").removeClass("ui-state-active"),this.focus(t,i)},mouseleave:"collapseAll","mouseleave .ui-menu":"collapseAll",focus:function(e,t){var i=this.active||this.element.children(".ui-menu-item").eq(0);t||this.focus(e,i)},blur:function(t){this._delay((function(){e.contains(this.element[0],this.document[0].activeElement)||this.collapseAll(t)}))},keydown:"_keydown"}),this.refresh(),this._on(this.document,{click:function(t){e(t.target).closest(".ui-menu").length||this.collapseAll(t),i=!1}})},_destroy:function(){this.element.removeAttr("aria-activedescendant").find(".ui-menu").andSelf().removeClass("ui-menu ui-widget ui-widget-content ui-corner-all ui-menu-icons").removeAttr("role").removeAttr("tabIndex").removeAttr("aria-labelledby").removeAttr("aria-expanded").removeAttr("aria-hidden").removeAttr("aria-disabled").removeUniqueId().show(),this.element.find(".ui-menu-item").removeClass("ui-menu-item").removeAttr("role").removeAttr("aria-disabled").children("a").removeUniqueId().removeClass("ui-corner-all ui-state-hover").removeAttr("tabIndex").removeAttr("role").removeAttr("aria-haspopup").children().each((function(){var t=e(this);t.data("ui-menu-submenu-carat")&&t.remove()})),this.element.find(".ui-menu-divider").removeClass("ui-menu-divider ui-widget-content")},_keydown:function(t){var i,n,a,o,s,r=!0;function l(e){return e.replace(/[\-\[\]{}()*+?.,\\\^$|#\s]/g,"\\$&")}switch(t.keyCode){case e.ui.keyCode.PAGE_UP:this.previousPage(t);break;case e.ui.keyCode.PAGE_DOWN:this.nextPage(t);break;case e.ui.keyCode.HOME:this._move("first","first",t);break;case e.ui.keyCode.END:this._move("last","last",t);break;case e.ui.keyCode.UP:this.previous(t);break;case e.ui.keyCode.DOWN:this.next(t);break;case e.ui.keyCode.LEFT:this.collapse(t);break;case e.ui.keyCode.RIGHT:this.active&&!this.active.is(".ui-state-disabled")&&this.expand(t);break;case e.ui.keyCode.ENTER:case e.ui.keyCode.SPACE:this._activate(t);break;case e.ui.keyCode.ESCAPE:this.collapse(t);break;default:r=!1,n=this.previousFilter||"",a=String.fromCharCode(t.keyCode),o=!1,clearTimeout(this.filterTimer),a===n?o=!0:a=n+a,s=new RegExp("^"+l(a),"i"),i=this.activeMenu.children(".ui-menu-item").filter((function(){return s.test(e(this).children("a").text())})),i=o&&-1!==i.index(this.active.next())?this.active.nextAll(".ui-menu-item"):i,i.length||(a=String.fromCharCode(t.keyCode),s=new RegExp("^"+l(a),"i"),i=this.activeMenu.children(".ui-menu-item").filter((function(){return s.test(e(this).children("a").text())}))),i.length?(this.focus(t,i),i.length>1?(this.previousFilter=a,this.filterTimer=this._delay((function(){delete this.previousFilter}),1e3)):delete this.previousFilter):delete this.previousFilter}r&&t.preventDefault()},_activate:function(e){this.active.is(".ui-state-disabled")||(this.active.children("a[aria-haspopup='true']").length?this.expand(e):this.select(e))},refresh:function(){var t,i=this.options.icons.submenu,n=this.element.find(this.options.menus);n.filter(":not(.ui-menu)").addClass("ui-menu ui-widget ui-widget-content ui-corner-all").hide().attr({role:this.options.role,"aria-hidden":"true","aria-expanded":"false"}).each((function(){var t=e(this),n=t.prev("a"),a=e("<span>").addClass("ui-menu-icon ui-icon "+i).data("ui-menu-submenu-carat",!0);n.attr("aria-haspopup","true").prepend(a),t.attr("aria-labelledby",n.attr("id"))})),t=n.add(this.element),t.children(":not(.ui-menu-item):has(a)").addClass("ui-menu-item").attr("role","presentation").children("a").uniqueId().addClass("ui-corner-all").attr({tabIndex:-1,role:this._itemRole()}),t.children(":not(.ui-menu-item)").each((function(){var t=e(this);/[^\-—–\s]/.test(t.text())||t.addClass("ui-widget-content ui-menu-divider")})),t.children(".ui-state-disabled").attr("aria-disabled","true"),this.active&&!e.contains(this.element[0],this.active[0])&&this.blur()},_itemRole:function(){return{menu:"menuitem",listbox:"option"}[this.options.role]},focus:function(e,t){var i,n;this.blur(e,e&&"focus"===e.type),this._scrollIntoView(t),this.active=t.first(),n=this.active.children("a").addClass("ui-state-focus"),this.options.role&&this.element.attr("aria-activedescendant",n.attr("id")),this.active.parent().closest(".ui-menu-item").children("a:first").addClass("ui-state-active"),e&&"keydown"===e.type?this._close():this.timer=this._delay((function(){this._close()}),this.delay),i=t.children(".ui-menu"),i.length&&/^mouse/.test(e.type)&&this._startOpening(i),this.activeMenu=t.parent(),this._trigger("focus",e,{item:t})},_scrollIntoView:function(t){var i,n,a,o,s,r;this._hasScroll()&&(i=parseFloat(e.css(this.activeMenu[0],"borderTopWidth"))||0,n=parseFloat(e.css(this.activeMenu[0],"paddingTop"))||0,a=t.offset().top-this.activeMenu.offset().top-i-n,o=this.activeMenu.scrollTop(),s=this.activeMenu.height(),r=t.height(),a<0?this.activeMenu.scrollTop(o+a):a+r>s&&this.activeMenu.scrollTop(o+a-s+r))},blur:function(e,t){t||clearTimeout(this.timer),this.active&&(this.active.children("a").removeClass("ui-state-focus"),this.active=null,this._trigger("blur",e,{item:this.active}))},_startOpening:function(e){clearTimeout(this.timer),"true"===e.attr("aria-hidden")&&(this.timer=this._delay((function(){this._close(),this._open(e)}),this.delay))},_open:function(t){var i=e.extend({of:this.active},this.options.position);clearTimeout(this.timer),this.element.find(".ui-menu").not(t.parents(".ui-menu")).hide().attr("aria-hidden","true"),t.show().removeAttr("aria-hidden").attr("aria-expanded","true").position(i)},collapseAll:function(t,i){clearTimeout(this.timer),this.timer=this._delay((function(){var n=i?this.element:e(t&&t.target).closest(this.element.find(".ui-menu"));n.length||(n=this.element),this._close(n),this.blur(t),this.activeMenu=n}),this.delay)},_close:function(e){e||(e=this.active?this.active.parent():this.element),e.find(".ui-menu").hide().attr("aria-hidden","true").attr("aria-expanded","false").end().find("a.ui-state-active").removeClass("ui-state-active")},collapse:function(e){var t=this.active&&this.active.parent().closest(".ui-menu-item",this.element);t&&t.length&&(this._close(),this.focus(e,t))},expand:function(e){var t=this.active&&this.active.children(".ui-menu ").children(".ui-menu-item").first();t&&t.length&&(this._open(t.parent()),this._delay((function(){this.focus(e,t)})))},next:function(e){this._move("next","first",e)},previous:function(e){this._move("prev","last",e)},isFirstItem:function(){return this.active&&!this.active.prevAll(".ui-menu-item").length},isLastItem:function(){return this.active&&!this.active.nextAll(".ui-menu-item").length},_move:function(e,t,i){var n;this.active&&(n="first"===e||"last"===e?this.active["first"===e?"prevAll":"nextAll"](".ui-menu-item").eq(-1):this.active[e+"All"](".ui-menu-item").eq(0)),n&&n.length&&this.active||(n=this.activeMenu.children(".ui-menu-item")[t]()),this.focus(i,n)},nextPage:function(t){var i,n,a;this.active?this.isLastItem()||(this._hasScroll()?(n=this.active.offset().top,a=this.element.height(),this.active.nextAll(".ui-menu-item").each((function(){return i=e(this),i.offset().top-n-a<0})),this.focus(t,i)):this.focus(t,this.activeMenu.children(".ui-menu-item")[this.active?"last":"first"]())):this.next(t)},previousPage:function(t){var i,n,a;this.active?this.isFirstItem()||(this._hasScroll()?(n=this.active.offset().top,a=this.element.height(),this.active.prevAll(".ui-menu-item").each((function(){return i=e(this),i.offset().top-n+a>0})),this.focus(t,i)):this.focus(t,this.activeMenu.children(".ui-menu-item").first())):this.next(t)},_hasScroll:function(){return this.element.outerHeight()<this.element.prop("scrollHeight")},select:function(t){this.active=this.active||e(t.target).closest(".ui-menu-item");var i={item:this.active};this.active.has(".ui-menu").length||this.collapseAll(t,!0),this._trigger("select",t,i)}})}(jQuery),function(e,t){e.ui=e.ui||{};var i,n=Math.max,a=Math.abs,o=Math.round,s=/left|center|right/,r=/top|center|bottom/,l=/[\+\-]\d+%?/,c=/^\w+/,h=/%$/,d=e.fn.position;function u(e,t,i){return[parseInt(e[0],10)*(h.test(e[0])?t/100:1),parseInt(e[1],10)*(h.test(e[1])?i/100:1)]}function p(t,i){return parseInt(e.css(t,i),10)||0}e.position={scrollbarWidth:function(){if(i!==t)return i;var n,a,o=e("<div style='display:block;width:50px;height:50px;overflow:hidden;'><div style='height:100px;width:auto;'></div></div>"),s=o.children()[0];return e("body").append(o),n=s.offsetWidth,o.css("overflow","scroll"),a=s.offsetWidth,n===a&&(a=o[0].clientWidth),o.remove(),i=n-a},getScrollInfo:function(t){var i=t.isWindow?"":t.element.css("overflow-x"),n=t.isWindow?"":t.element.css("overflow-y"),a="scroll"===i||"auto"===i&&t.width<t.element[0].scrollWidth,o="scroll"===n||"auto"===n&&t.height<t.element[0].scrollHeight;return{width:a?e.position.scrollbarWidth():0,height:o?e.position.scrollbarWidth():0}},getWithinInfo:function(t){var i=e(t||window),n=e.isWindow(i[0]);return{element:i,isWindow:n,offset:i.offset()||{left:0,top:0},scrollLeft:i.scrollLeft(),scrollTop:i.scrollTop(),width:n?i.width():i.outerWidth(),height:n?i.height():i.outerHeight()}}},e.fn.position=function(t){if(!t||!t.of)return d.apply(this,arguments);t=e.extend({},t);var i,h,f,g,m,v=e(t.of),b=e.position.getWithinInfo(t.within),w=e.position.getScrollInfo(b),y=v[0],C=(t.collision||"flip").split(" "),_={};return 9===y.nodeType?(h=v.width(),f=v.height(),g={top:0,left:0}):e.isWindow(y)?(h=v.width(),f=v.height(),g={top:v.scrollTop(),left:v.scrollLeft()}):y.preventDefault?(t.at="left top",h=f=0,g={top:y.pageY,left:y.pageX}):(h=v.outerWidth(),f=v.outerHeight(),g=v.offset()),m=e.extend({},g),e.each(["my","at"],(function(){var e,i,n=(t[this]||"").split(" ");1===n.length&&(n=s.test(n[0])?n.concat(["center"]):r.test(n[0])?["center"].concat(n):["center","center"]),n[0]=s.test(n[0])?n[0]:"center",n[1]=r.test(n[1])?n[1]:"center",e=l.exec(n[0]),i=l.exec(n[1]),_[this]=[e?e[0]:0,i?i[0]:0],t[this]=[c.exec(n[0])[0],c.exec(n[1])[0]]})),1===C.length&&(C[1]=C[0]),"right"===t.at[0]?m.left+=h:"center"===t.at[0]&&(m.left+=h/2),"bottom"===t.at[1]?m.top+=f:"center"===t.at[1]&&(m.top+=f/2),i=u(_.at,h,f),m.left+=i[0],m.top+=i[1],this.each((function(){var s,r,l=e(this),c=l.outerWidth(),d=l.outerHeight(),y=p(this,"marginLeft"),x=p(this,"marginTop"),k=c+y+p(this,"marginRight")+w.width,S=d+x+p(this,"marginBottom")+w.height,T=e.extend({},m),$=u(_.my,l.outerWidth(),l.outerHeight());"right"===t.my[0]?T.left-=c:"center"===t.my[0]&&(T.left-=c/2),"bottom"===t.my[1]?T.top-=d:"center"===t.my[1]&&(T.top-=d/2),T.left+=$[0],T.top+=$[1],e.support.offsetFractions||(T.left=o(T.left),T.top=o(T.top)),s={marginLeft:y,marginTop:x},e.each(["left","top"],(function(n,a){e.ui.position[C[n]]&&e.ui.position[C[n]][a](T,{targetWidth:h,targetHeight:f,elemWidth:c,elemHeight:d,collisionPosition:s,collisionWidth:k,collisionHeight:S,offset:[i[0]+$[0],i[1]+$[1]],my:t.my,at:t.at,within:b,elem:l})})),e.fn.bgiframe&&l.bgiframe(),t.using&&(r=function(e){var i=g.left-T.left,o=i+h-c,s=g.top-T.top,r=s+f-d,u={target:{element:v,left:g.left,top:g.top,width:h,height:f},element:{element:l,left:T.left,top:T.top,width:c,height:d},horizontal:o<0?"left":i>0?"right":"center",vertical:r<0?"top":s>0?"bottom":"middle"};h<c&&a(i+o)<h&&(u.horizontal="center"),f<d&&a(s+r)<f&&(u.vertical="middle"),n(a(i),a(o))>n(a(s),a(r))?u.important="horizontal":u.important="vertical",t.using.call(this,e,u)}),l.offset(e.extend(T,{using:r}))}))},e.ui.position={fit:{left:function(e,t){var i,a=t.within,o=a.isWindow?a.scrollLeft:a.offset.left,s=a.width,r=e.left-t.collisionPosition.marginLeft,l=o-r,c=r+t.collisionWidth-s-o;t.collisionWidth>s?l>0&&c<=0?(i=e.left+l+t.collisionWidth-s-o,e.left+=l-i):e.left=c>0&&l<=0?o:l>c?o+s-t.collisionWidth:o:l>0?e.left+=l:c>0?e.left-=c:e.left=n(e.left-r,e.left)},top:function(e,t){var i,a=t.within,o=a.isWindow?a.scrollTop:a.offset.top,s=t.within.height,r=e.top-t.collisionPosition.marginTop,l=o-r,c=r+t.collisionHeight-s-o;t.collisionHeight>s?l>0&&c<=0?(i=e.top+l+t.collisionHeight-s-o,e.top+=l-i):e.top=c>0&&l<=0?o:l>c?o+s-t.collisionHeight:o:l>0?e.top+=l:c>0?e.top-=c:e.top=n(e.top-r,e.top)}},flip:{left:function(e,t){var i,n,o=t.within,s=o.offset.left+o.scrollLeft,r=o.width,l=o.isWindow?o.scrollLeft:o.offset.left,c=e.left-t.collisionPosition.marginLeft,h=c-l,d=c+t.collisionWidth-r-l,u="left"===t.my[0]?-t.elemWidth:"right"===t.my[0]?t.elemWidth:0,p="left"===t.at[0]?t.targetWidth:"right"===t.at[0]?-t.targetWidth:0,f=-2*t.offset[0];h<0?(i=e.left+u+p+f+t.collisionWidth-r-s,(i<0||i<a(h))&&(e.left+=u+p+f)):d>0&&(n=e.left-t.collisionPosition.marginLeft+u+p+f-l,(n>0||a(n)<d)&&(e.left+=u+p+f))},top:function(e,t){var i,n,o=t.within,s=o.offset.top+o.scrollTop,r=o.height,l=o.isWindow?o.scrollTop:o.offset.top,c=e.top-t.collisionPosition.marginTop,h=c-l,d=c+t.collisionHeight-r-l,u="top"===t.my[1],p=u?-t.elemHeight:"bottom"===t.my[1]?t.elemHeight:0,f="top"===t.at[1]?t.targetHeight:"bottom"===t.at[1]?-t.targetHeight:0,g=-2*t.offset[1];h<0?(n=e.top+p+f+g+t.collisionHeight-r-s,e.top+p+f+g>h&&(n<0||n<a(h))&&(e.top+=p+f+g)):d>0&&(i=e.top-t.collisionPosition.marginTop+p+f+g-l,e.top+p+f+g>d&&(i>0||a(i)<d)&&(e.top+=p+f+g))}},flipfit:{left:function(){e.ui.position.flip.left.apply(this,arguments),e.ui.position.fit.left.apply(this,arguments)},top:function(){e.ui.position.flip.top.apply(this,arguments),e.ui.position.fit.top.apply(this,arguments)}}},function(){var t,i,n,a,o,s=document.getElementsByTagName("body")[0],r=document.createElement("div");for(o in t=document.createElement(s?"div":"body"),n={visibility:"hidden",width:0,height:0,border:0,margin:0,background:"none"},s&&e.extend(n,{position:"absolute",left:"-1000px",top:"-1000px"}),n)t.style[o]=n[o];t.appendChild(r),i=s||document.documentElement,i.insertBefore(t,i.firstChild),r.style.cssText="position: absolute; left: 10.7432222px;",a=e(r).offset().left,e.support.offsetFractions=a>10&&a<11,t.innerHTML="",i.removeChild(t)}(),!1!==e.uiBackCompat&&function(e){var i=e.fn.position;e.fn.position=function(n){if(!n||!n.offset)return i.call(this,n);var a=n.offset.split(" "),o=n.at.split(" ");return 1===a.length&&(a[1]=a[0]),/^\d/.test(a[0])&&(a[0]="+"+a[0]),/^\d/.test(a[1])&&(a[1]="+"+a[1]),1===o.length&&(/left|center|right/.test(o[0])?o[1]="center":(o[1]=o[0],o[0]="center")),i.call(this,e.extend(n,{at:o[0]+a[0]+" "+o[1]+a[1],offset:t}))}}(jQuery)}(jQuery),function(e,t){e.widget("ui.progressbar",{version:"1.9.2",options:{value:0,max:100},min:0,_create:function(){this.element.addClass("ui-progressbar ui-widget ui-widget-content ui-corner-all").attr({role:"progressbar","aria-valuemin":this.min,"aria-valuemax":this.options.max,"aria-valuenow":this._value()}),this.valueDiv=e("<div class='ui-progressbar-value ui-widget-header ui-corner-left'></div>").appendTo(this.element),this.oldValue=this._value(),this._refreshValue()},_destroy:function(){this.element.removeClass("ui-progressbar ui-widget ui-widget-content ui-corner-all").removeAttr("role").removeAttr("aria-valuemin").removeAttr("aria-valuemax").removeAttr("aria-valuenow"),this.valueDiv.remove()},value:function(e){return e===t?this._value():(this._setOption("value",e),this)},_setOption:function(e,t){"value"===e&&(this.options.value=t,this._refreshValue(),this._value()===this.options.max&&this._trigger("complete")),this._super(e,t)},_value:function(){var e=this.options.value;return"number"!==typeof e&&(e=0),Math.min(this.options.max,Math.max(this.min,e))},_percentage:function(){return 100*this._value()/this.options.max},_refreshValue:function(){var e=this.value(),t=this._percentage();this.oldValue!==e&&(this.oldValue=e,this._trigger("change")),this.valueDiv.toggle(e>this.min).toggleClass("ui-corner-right",e===this.options.max).width(t.toFixed(0)+"%"),this.element.attr("aria-valuenow",e)}})}(jQuery),function(e,t){var i=5;e.widget("ui.slider",e.ui.mouse,{version:"1.9.2",widgetEventPrefix:"slide",options:{animate:!1,distance:0,max:100,min:0,orientation:"horizontal",range:!1,step:1,value:0,values:null},_create:function(){var t,n,a=this.options,o=this.element.find(".ui-slider-handle").addClass("ui-state-default ui-corner-all"),s="<a class='ui-slider-handle ui-state-default ui-corner-all' href='#'></a>",r=[];for(this._keySliding=!1,this._mouseSliding=!1,this._animateOff=!0,this._handleIndex=null,this._detectOrientation(),this._mouseInit(),this.element.addClass("ui-slider ui-slider-"+this.orientation+" ui-widget ui-widget-content ui-corner-all"+(a.disabled?" ui-slider-disabled ui-disabled":"")),this.range=e([]),a.range&&(!0===a.range&&(a.values||(a.values=[this._valueMin(),this._valueMin()]),a.values.length&&2!==a.values.length&&(a.values=[a.values[0],a.values[0]])),this.range=e("<div></div>").appendTo(this.element).addClass("ui-slider-range ui-widget-header"+("min"===a.range||"max"===a.range?" ui-slider-range-"+a.range:""))),n=a.values&&a.values.length||1,t=o.length;t<n;t++)r.push(s);this.handles=o.add(e(r.join("")).appendTo(this.element)),this.handle=this.handles.eq(0),this.handles.add(this.range).filter("a").click((function(e){e.preventDefault()})).mouseenter((function(){a.disabled||e(this).addClass("ui-state-hover")})).mouseleave((function(){e(this).removeClass("ui-state-hover")})).focus((function(){a.disabled?e(this).blur():(e(".ui-slider .ui-state-focus").removeClass("ui-state-focus"),e(this).addClass("ui-state-focus"))})).blur((function(){e(this).removeClass("ui-state-focus")})),this.handles.each((function(t){e(this).data("ui-slider-handle-index",t)})),this._on(this.handles,{keydown:function(t){var n,a,o,s,r=e(t.target).data("ui-slider-handle-index");switch(t.keyCode){case e.ui.keyCode.HOME:case e.ui.keyCode.END:case e.ui.keyCode.PAGE_UP:case e.ui.keyCode.PAGE_DOWN:case e.ui.keyCode.UP:case e.ui.keyCode.RIGHT:case e.ui.keyCode.DOWN:case e.ui.keyCode.LEFT:if(t.preventDefault(),!this._keySliding&&(this._keySliding=!0,e(t.target).addClass("ui-state-active"),n=this._start(t,r),!1===n))return;break}switch(s=this.options.step,a=o=this.options.values&&this.options.values.length?this.values(r):this.value(),t.keyCode){case e.ui.keyCode.HOME:o=this._valueMin();break;case e.ui.keyCode.END:o=this._valueMax();break;case e.ui.keyCode.PAGE_UP:o=this._trimAlignValue(a+(this._valueMax()-this._valueMin())/i);break;case e.ui.keyCode.PAGE_DOWN:o=this._trimAlignValue(a-(this._valueMax()-this._valueMin())/i);break;case e.ui.keyCode.UP:case e.ui.keyCode.RIGHT:if(a===this._valueMax())return;o=this._trimAlignValue(a+s);break;case e.ui.keyCode.DOWN:case e.ui.keyCode.LEFT:if(a===this._valueMin())return;o=this._trimAlignValue(a-s);break}this._slide(t,r,o)},keyup:function(t){var i=e(t.target).data("ui-slider-handle-index");this._keySliding&&(this._keySliding=!1,this._stop(t,i),this._change(t,i),e(t.target).removeClass("ui-state-active"))}}),this._refreshValue(),this._animateOff=!1},_destroy:function(){this.handles.remove(),this.range.remove(),this.element.removeClass("ui-slider ui-slider-horizontal ui-slider-vertical ui-slider-disabled ui-widget ui-widget-content ui-corner-all"),this._mouseDestroy()},_mouseCapture:function(t){var i,n,a,o,s,r,l,c,h=this,d=this.options;return!d.disabled&&(this.elementSize={width:this.element.outerWidth(),height:this.element.outerHeight()},this.elementOffset=this.element.offset(),i={x:t.pageX,y:t.pageY},n=this._normValueFromMouse(i),a=this._valueMax()-this._valueMin()+1,this.handles.each((function(t){var i=Math.abs(n-h.values(t));a>i&&(a=i,o=e(this),s=t)})),!0===d.range&&this.values(1)===d.min&&(s+=1,o=e(this.handles[s])),r=this._start(t,s),!1!==r&&(this._mouseSliding=!0,this._handleIndex=s,o.addClass("ui-state-active").focus(),l=o.offset(),c=!e(t.target).parents().andSelf().is(".ui-slider-handle"),this._clickOffset=c?{left:0,top:0}:{left:t.pageX-l.left-o.width()/2,top:t.pageY-l.top-o.height()/2-(parseInt(o.css("borderTopWidth"),10)||0)-(parseInt(o.css("borderBottomWidth"),10)||0)+(parseInt(o.css("marginTop"),10)||0)},this.handles.hasClass("ui-state-hover")||this._slide(t,s,n),this._animateOff=!0,!0))},_mouseStart:function(){return!0},_mouseDrag:function(e){var t={x:e.pageX,y:e.pageY},i=this._normValueFromMouse(t);return this._slide(e,this._handleIndex,i),!1},_mouseStop:function(e){return this.handles.removeClass("ui-state-active"),this._mouseSliding=!1,this._stop(e,this._handleIndex),this._change(e,this._handleIndex),this._handleIndex=null,this._clickOffset=null,this._animateOff=!1,!1},_detectOrientation:function(){this.orientation="vertical"===this.options.orientation?"vertical":"horizontal"},_normValueFromMouse:function(e){var t,i,n,a,o;return"horizontal"===this.orientation?(t=this.elementSize.width,i=e.x-this.elementOffset.left-(this._clickOffset?this._clickOffset.left:0)):(t=this.elementSize.height,i=e.y-this.elementOffset.top-(this._clickOffset?this._clickOffset.top:0)),n=i/t,n>1&&(n=1),n<0&&(n=0),"vertical"===this.orientation&&(n=1-n),a=this._valueMax()-this._valueMin(),o=this._valueMin()+n*a,this._trimAlignValue(o)},_start:function(e,t){var i={handle:this.handles[t],value:this.value()};return this.options.values&&this.options.values.length&&(i.value=this.values(t),i.values=this.values()),this._trigger("start",e,i)},_slide:function(e,t,i){var n,a,o;this.options.values&&this.options.values.length?(n=this.values(t?0:1),2===this.options.values.length&&!0===this.options.range&&(0===t&&i>n||1===t&&i<n)&&(i=n),i!==this.values(t)&&(a=this.values(),a[t]=i,o=this._trigger("slide",e,{handle:this.handles[t],value:i,values:a}),n=this.values(t?0:1),!1!==o&&this.values(t,i,!0))):i!==this.value()&&(o=this._trigger("slide",e,{handle:this.handles[t],value:i}),!1!==o&&this.value(i))},_stop:function(e,t){var i={handle:this.handles[t],value:this.value()};this.options.values&&this.options.values.length&&(i.value=this.values(t),i.values=this.values()),this._trigger("stop",e,i)},_change:function(e,t){if(!this._keySliding&&!this._mouseSliding){var i={handle:this.handles[t],value:this.value()};this.options.values&&this.options.values.length&&(i.value=this.values(t),i.values=this.values()),this._trigger("change",e,i)}},value:function(e){return arguments.length?(this.options.value=this._trimAlignValue(e),this._refreshValue(),void this._change(null,0)):this._value()},values:function(t,i){var n,a,o;if(arguments.length>1)return this.options.values[t]=this._trimAlignValue(i),this._refreshValue(),void this._change(null,t);if(!arguments.length)return this._values();if(!e.isArray(arguments[0]))return this.options.values&&this.options.values.length?this._values(t):this.value();for(n=this.options.values,a=arguments[0],o=0;o<n.length;o+=1)n[o]=this._trimAlignValue(a[o]),this._change(null,o);this._refreshValue()},_setOption:function(t,i){var n,a=0;switch(e.isArray(this.options.values)&&(a=this.options.values.length),e.Widget.prototype._setOption.apply(this,arguments),t){case"disabled":i?(this.handles.filter(".ui-state-focus").blur(),this.handles.removeClass("ui-state-hover"),this.handles.prop("disabled",!0),this.element.addClass("ui-disabled")):(this.handles.prop("disabled",!1),this.element.removeClass("ui-disabled"));break;case"orientation":this._detectOrientation(),this.element.removeClass("ui-slider-horizontal ui-slider-vertical").addClass("ui-slider-"+this.orientation),this._refreshValue();break;case"value":this._animateOff=!0,this._refreshValue(),this._change(null,0),this._animateOff=!1;break;case"values":for(this._animateOff=!0,this._refreshValue(),n=0;n<a;n+=1)this._change(null,n);this._animateOff=!1;break;case"min":case"max":this._animateOff=!0,this._refreshValue(),this._animateOff=!1;break}},_value:function(){var e=this.options.value;return e=this._trimAlignValue(e),e},_values:function(e){var t,i,n;if(arguments.length)return t=this.options.values[e],t=this._trimAlignValue(t),t;for(i=this.options.values.slice(),n=0;n<i.length;n+=1)i[n]=this._trimAlignValue(i[n]);return i},_trimAlignValue:function(e){if(e<=this._valueMin())return this._valueMin();if(e>=this._valueMax())return this._valueMax();var t=this.options.step>0?this.options.step:1,i=(e-this._valueMin())%t,n=e-i;return 2*Math.abs(i)>=t&&(n+=i>0?t:-t),parseFloat(n.toFixed(5))},_valueMin:function(){return this.options.min},_valueMax:function(){return this.options.max},_refreshValue:function(){var t,i,n,a,o,s=this.options.range,r=this.options,l=this,c=!this._animateOff&&r.animate,h={};this.options.values&&this.options.values.length?this.handles.each((function(n){i=(l.values(n)-l._valueMin())/(l._valueMax()-l._valueMin())*100,h["horizontal"===l.orientation?"left":"bottom"]=i+"%",e(this).stop(1,1)[c?"animate":"css"](h,r.animate),!0===l.options.range&&("horizontal"===l.orientation?(0===n&&l.range.stop(1,1)[c?"animate":"css"]({left:i+"%"},r.animate),1===n&&l.range[c?"animate":"css"]({width:i-t+"%"},{queue:!1,duration:r.animate})):(0===n&&l.range.stop(1,1)[c?"animate":"css"]({bottom:i+"%"},r.animate),1===n&&l.range[c?"animate":"css"]({height:i-t+"%"},{queue:!1,duration:r.animate}))),t=i})):(n=this.value(),a=this._valueMin(),o=this._valueMax(),i=o!==a?(n-a)/(o-a)*100:0,h["horizontal"===this.orientation?"left":"bottom"]=i+"%",this.handle.stop(1,1)[c?"animate":"css"](h,r.animate),"min"===s&&"horizontal"===this.orientation&&this.range.stop(1,1)[c?"animate":"css"]({width:i+"%"},r.animate),"max"===s&&"horizontal"===this.orientation&&this.range[c?"animate":"css"]({width:100-i+"%"},{queue:!1,duration:r.animate}),"min"===s&&"vertical"===this.orientation&&this.range.stop(1,1)[c?"animate":"css"]({height:i+"%"},r.animate),"max"===s&&"vertical"===this.orientation&&this.range[c?"animate":"css"]({height:100-i+"%"},{queue:!1,duration:r.animate}))}})}(jQuery),function(e){function t(e){return function(){var t=this.element.val();e.apply(this,arguments),this._refresh(),t!==this.element.val()&&this._trigger("change")}}e.widget("ui.spinner",{version:"1.9.2",defaultElement:"<input>",widgetEventPrefix:"spin",options:{culture:null,icons:{down:"ui-icon-triangle-1-s",up:"ui-icon-triangle-1-n"},incremental:!0,max:null,min:null,numberFormat:null,page:10,step:1,change:null,spin:null,start:null,stop:null},_create:function(){this._setOption("max",this.options.max),this._setOption("min",this.options.min),this._setOption("step",this.options.step),this._value(this.element.val(),!0),this._draw(),this._on(this._events),this._refresh(),this._on(this.window,{beforeunload:function(){this.element.removeAttr("autocomplete")}})},_getCreateOptions:function(){var t={},i=this.element;return e.each(["min","max","step"],(function(e,n){var a=i.attr(n);void 0!==a&&a.length&&(t[n]=a)})),t},_events:{keydown:function(e){this._start(e)&&this._keydown(e)&&e.preventDefault()},keyup:"_stop",focus:function(){this.previous=this.element.val()},blur:function(e){this.cancelBlur?delete this.cancelBlur:(this._refresh(),this.previous!==this.element.val()&&this._trigger("change",e))},mousewheel:function(e,t){if(t){if(!this.spinning&&!this._start(e))return!1;this._spin((t>0?1:-1)*this.options.step,e),clearTimeout(this.mousewheelTimer),this.mousewheelTimer=this._delay((function(){this.spinning&&this._stop(e)}),100),e.preventDefault()}},"mousedown .ui-spinner-button":function(t){var i;function n(){var e=this.element[0]===this.document[0].activeElement;e||(this.element.focus(),this.previous=i,this._delay((function(){this.previous=i})))}i=this.element[0]===this.document[0].activeElement?this.previous:this.element.val(),t.preventDefault(),n.call(this),this.cancelBlur=!0,this._delay((function(){delete this.cancelBlur,n.call(this)})),!1!==this._start(t)&&this._repeat(null,e(t.currentTarget).hasClass("ui-spinner-up")?1:-1,t)},"mouseup .ui-spinner-button":"_stop","mouseenter .ui-spinner-button":function(t){if(e(t.currentTarget).hasClass("ui-state-active"))return!1!==this._start(t)&&void this._repeat(null,e(t.currentTarget).hasClass("ui-spinner-up")?1:-1,t)},"mouseleave .ui-spinner-button":"_stop"},_draw:function(){var e=this.uiSpinner=this.element.addClass("ui-spinner-input").attr("autocomplete","off").wrap(this._uiSpinnerHtml()).parent().append(this._buttonHtml());this.element.attr("role","spinbutton"),this.buttons=e.find(".ui-spinner-button").attr("tabIndex",-1).button().removeClass("ui-corner-all"),this.buttons.height()>Math.ceil(.5*e.height())&&e.height()>0&&e.height(e.height()),this.options.disabled&&this.disable()},_keydown:function(t){var i=this.options,n=e.ui.keyCode;switch(t.keyCode){case n.UP:return this._repeat(null,1,t),!0;case n.DOWN:return this._repeat(null,-1,t),!0;case n.PAGE_UP:return this._repeat(null,i.page,t),!0;case n.PAGE_DOWN:return this._repeat(null,-i.page,t),!0}return!1},_uiSpinnerHtml:function(){return"<span class='ui-spinner ui-widget ui-widget-content ui-corner-all'></span>"},_buttonHtml:function(){return"<a class='ui-spinner-button ui-spinner-up ui-corner-tr'><span class='ui-icon "+this.options.icons.up+"'>&#9650;</span></a><a class='ui-spinner-button ui-spinner-down ui-corner-br'><span class='ui-icon "+this.options.icons.down+"'>&#9660;</span></a>"},_start:function(e){return!(!this.spinning&&!1===this._trigger("start",e))&&(this.counter||(this.counter=1),this.spinning=!0,!0)},_repeat:function(e,t,i){e=e||500,clearTimeout(this.timer),this.timer=this._delay((function(){this._repeat(40,t,i)}),e),this._spin(t*this.options.step,i)},_spin:function(e,t){var i=this.value()||0;this.counter||(this.counter=1),i=this._adjustValue(i+e*this._increment(this.counter)),this.spinning&&!1===this._trigger("spin",t,{value:i})||(this._value(i),this.counter++)},_increment:function(t){var i=this.options.incremental;return i?e.isFunction(i)?i(t):Math.floor(t*t*t/5e4-t*t/500+17*t/200+1):1},_precision:function(){var e=this._precisionOf(this.options.step);return null!==this.options.min&&(e=Math.max(e,this._precisionOf(this.options.min))),e},_precisionOf:function(e){var t=e.toString(),i=t.indexOf(".");return-1===i?0:t.length-i-1},_adjustValue:function(e){var t,i,n=this.options;return t=null!==n.min?n.min:0,i=e-t,i=Math.round(i/n.step)*n.step,e=t+i,e=parseFloat(e.toFixed(this._precision())),null!==n.max&&e>n.max?n.max:null!==n.min&&e<n.min?n.min:e},_stop:function(e){this.spinning&&(clearTimeout(this.timer),clearTimeout(this.mousewheelTimer),this.counter=0,this.spinning=!1,this._trigger("stop",e))},_setOption:function(e,t){if("culture"===e||"numberFormat"===e){var i=this._parse(this.element.val());return this.options[e]=t,void this.element.val(this._format(i))}"max"!==e&&"min"!==e&&"step"!==e||"string"===typeof t&&(t=this._parse(t)),this._super(e,t),"disabled"===e&&(t?(this.element.prop("disabled",!0),this.buttons.button("disable")):(this.element.prop("disabled",!1),this.buttons.button("enable")))},_setOptions:t((function(e){this._super(e),this._value(this.element.val())})),_parse:function(e){return"string"===typeof e&&""!==e&&(e=window.Globalize&&this.options.numberFormat?Globalize.parseFloat(e,10,this.options.culture):+e),""===e||isNaN(e)?null:e},_format:function(e){return""===e?"":window.Globalize&&this.options.numberFormat?Globalize.format(e,this.options.numberFormat,this.options.culture):e},_refresh:function(){this.element.attr({"aria-valuemin":this.options.min,"aria-valuemax":this.options.max,"aria-valuenow":this._parse(this.element.val())})},_value:function(e,t){var i;""!==e&&(i=this._parse(e),null!==i&&(t||(i=this._adjustValue(i)),e=this._format(i))),this.element.val(e),this._refresh()},_destroy:function(){this.element.removeClass("ui-spinner-input").prop("disabled",!1).removeAttr("autocomplete").removeAttr("role").removeAttr("aria-valuemin").removeAttr("aria-valuemax").removeAttr("aria-valuenow"),this.uiSpinner.replaceWith(this.element)},stepUp:t((function(e){this._stepUp(e)})),_stepUp:function(e){this._spin((e||1)*this.options.step)},stepDown:t((function(e){this._stepDown(e)})),_stepDown:function(e){this._spin((e||1)*-this.options.step)},pageUp:t((function(e){this._stepUp((e||1)*this.options.page)})),pageDown:t((function(e){this._stepDown((e||1)*this.options.page)})),value:function(e){if(!arguments.length)return this._parse(this.element.val());t(this._value).call(this,e)},widget:function(){return this.uiSpinner}})}(jQuery),function(e,t){var i=0,n=/#.*$/;function a(){return++i}function o(e){return e.hash.length>1&&e.href.replace(n,"")===location.href.replace(n,"").replace(/\s/g,"%20")}e.widget("ui.tabs",{version:"1.9.2",delay:300,options:{active:null,collapsible:!1,event:"click",heightStyle:"content",hide:null,show:null,activate:null,beforeActivate:null,beforeLoad:null,load:null},_create:function(){var t=this,i=this.options,n=i.active,a=location.hash.substring(1);this.running=!1,this.element.addClass("ui-tabs ui-widget ui-widget-content ui-corner-all").toggleClass("ui-tabs-collapsible",i.collapsible).delegate(".ui-tabs-nav > li","mousedown"+this.eventNamespace,(function(t){e(this).is(".ui-state-disabled")&&t.preventDefault()})).delegate(".ui-tabs-anchor","focus"+this.eventNamespace,(function(){e(this).closest("li").is(".ui-state-disabled")&&this.blur()})),this._processTabs(),null===n&&(a&&this.tabs.each((function(t,i){if(e(i).attr("aria-controls")===a)return n=t,!1})),null===n&&(n=this.tabs.index(this.tabs.filter(".ui-tabs-active"))),null!==n&&-1!==n||(n=!!this.tabs.length&&0)),!1!==n&&(n=this.tabs.index(this.tabs.eq(n)),-1===n&&(n=!i.collapsible&&0)),i.active=n,!i.collapsible&&!1===i.active&&this.anchors.length&&(i.active=0),e.isArray(i.disabled)&&(i.disabled=e.unique(i.disabled.concat(e.map(this.tabs.filter(".ui-state-disabled"),(function(e){return t.tabs.index(e)})))).sort()),!1!==this.options.active&&this.anchors.length?this.active=this._findActive(this.options.active):this.active=e(),this._refresh(),this.active.length&&this.load(i.active)},_getCreateEventData:function(){return{tab:this.active,panel:this.active.length?this._getPanelForTab(this.active):e()}},_tabKeydown:function(t){var i=e(this.document[0].activeElement).closest("li"),n=this.tabs.index(i),a=!0;if(!this._handlePageNav(t)){switch(t.keyCode){case e.ui.keyCode.RIGHT:case e.ui.keyCode.DOWN:n++;break;case e.ui.keyCode.UP:case e.ui.keyCode.LEFT:a=!1,n--;break;case e.ui.keyCode.END:n=this.anchors.length-1;break;case e.ui.keyCode.HOME:n=0;break;case e.ui.keyCode.SPACE:return t.preventDefault(),clearTimeout(this.activating),void this._activate(n);case e.ui.keyCode.ENTER:return t.preventDefault(),clearTimeout(this.activating),void this._activate(n!==this.options.active&&n);default:return}t.preventDefault(),clearTimeout(this.activating),n=this._focusNextTab(n,a),t.ctrlKey||(i.attr("aria-selected","false"),this.tabs.eq(n).attr("aria-selected","true"),this.activating=this._delay((function(){this.option("active",n)}),this.delay))}},_panelKeydown:function(t){this._handlePageNav(t)||t.ctrlKey&&t.keyCode===e.ui.keyCode.UP&&(t.preventDefault(),this.active.focus())},_handlePageNav:function(t){return t.altKey&&t.keyCode===e.ui.keyCode.PAGE_UP?(this._activate(this._focusNextTab(this.options.active-1,!1)),!0):t.altKey&&t.keyCode===e.ui.keyCode.PAGE_DOWN?(this._activate(this._focusNextTab(this.options.active+1,!0)),!0):void 0},_findNextTab:function(t,i){var n=this.tabs.length-1;function a(){return t>n&&(t=0),t<0&&(t=n),t}while(-1!==e.inArray(a(),this.options.disabled))t=i?t+1:t-1;return t},_focusNextTab:function(e,t){return e=this._findNextTab(e,t),this.tabs.eq(e).focus(),e},_setOption:function(e,t){"active"!==e?"disabled"!==e?(this._super(e,t),"collapsible"===e&&(this.element.toggleClass("ui-tabs-collapsible",t),t||!1!==this.options.active||this._activate(0)),"event"===e&&this._setupEvents(t),"heightStyle"===e&&this._setupHeightStyle(t)):this._setupDisabled(t):this._activate(t)},_tabId:function(e){return e.attr("aria-controls")||"ui-tabs-"+a()},_sanitizeSelector:function(e){return e?e.replace(/[!"$%&'()*+,.\/:;<=>?@\[\]\^`{|}~]/g,"\\$&"):""},refresh:function(){var t=this.options,i=this.tablist.children(":has(a[href])");t.disabled=e.map(i.filter(".ui-state-disabled"),(function(e){return i.index(e)})),this._processTabs(),!1!==t.active&&this.anchors.length?this.active.length&&!e.contains(this.tablist[0],this.active[0])?this.tabs.length===t.disabled.length?(t.active=!1,this.active=e()):this._activate(this._findNextTab(Math.max(0,t.active-1),!1)):t.active=this.tabs.index(this.active):(t.active=!1,this.active=e()),this._refresh()},_refresh:function(){this._setupDisabled(this.options.disabled),this._setupEvents(this.options.event),this._setupHeightStyle(this.options.heightStyle),this.tabs.not(this.active).attr({"aria-selected":"false",tabIndex:-1}),this.panels.not(this._getPanelForTab(this.active)).hide().attr({"aria-expanded":"false","aria-hidden":"true"}),this.active.length?(this.active.addClass("ui-tabs-active ui-state-active").attr({"aria-selected":"true",tabIndex:0}),this._getPanelForTab(this.active).show().attr({"aria-expanded":"true","aria-hidden":"false"})):this.tabs.eq(0).attr("tabIndex",0)},_processTabs:function(){var t=this;this.tablist=this._getList().addClass("ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all").attr("role","tablist"),this.tabs=this.tablist.find("> li:has(a[href])").addClass("ui-state-default ui-corner-top").attr({role:"tab",tabIndex:-1}),this.anchors=this.tabs.map((function(){return e("a",this)[0]})).addClass("ui-tabs-anchor").attr({role:"presentation",tabIndex:-1}),this.panels=e(),this.anchors.each((function(i,n){var a,s,r,l=e(n).uniqueId().attr("id"),c=e(n).closest("li"),h=c.attr("aria-controls");o(n)?(a=n.hash,s=t.element.find(t._sanitizeSelector(a))):(r=t._tabId(c),a="#"+r,s=t.element.find(a),s.length||(s=t._createPanel(r),s.insertAfter(t.panels[i-1]||t.tablist)),s.attr("aria-live","polite")),s.length&&(t.panels=t.panels.add(s)),h&&c.data("ui-tabs-aria-controls",h),c.attr({"aria-controls":a.substring(1),"aria-labelledby":l}),s.attr("aria-labelledby",l)})),this.panels.addClass("ui-tabs-panel ui-widget-content ui-corner-bottom").attr("role","tabpanel")},_getList:function(){return this.element.find("ol,ul").eq(0)},_createPanel:function(t){return e("<div>").attr("id",t).addClass("ui-tabs-panel ui-widget-content ui-corner-bottom").data("ui-tabs-destroy",!0)},_setupDisabled:function(t){e.isArray(t)&&(t.length?t.length===this.anchors.length&&(t=!0):t=!1);for(var i,n=0;i=this.tabs[n];n++)!0===t||-1!==e.inArray(n,t)?e(i).addClass("ui-state-disabled").attr("aria-disabled","true"):e(i).removeClass("ui-state-disabled").removeAttr("aria-disabled");this.options.disabled=t},_setupEvents:function(t){var i={click:function(e){e.preventDefault()}};t&&e.each(t.split(" "),(function(e,t){i[t]="_eventHandler"})),this._off(this.anchors.add(this.tabs).add(this.panels)),this._on(this.anchors,i),this._on(this.tabs,{keydown:"_tabKeydown"}),this._on(this.panels,{keydown:"_panelKeydown"}),this._focusable(this.tabs),this._hoverable(this.tabs)},_setupHeightStyle:function(t){var i,n,a=this.element.parent();"fill"===t?(e.support.minHeight||(n=a.css("overflow"),a.css("overflow","hidden")),i=a.height(),this.element.siblings(":visible").each((function(){var t=e(this),n=t.css("position");"absolute"!==n&&"fixed"!==n&&(i-=t.outerHeight(!0))})),n&&a.css("overflow",n),this.element.children().not(this.panels).each((function(){i-=e(this).outerHeight(!0)})),this.panels.each((function(){e(this).height(Math.max(0,i-e(this).innerHeight()+e(this).height()))})).css("overflow","auto")):"auto"===t&&(i=0,this.panels.each((function(){i=Math.max(i,e(this).height("").height())})).height(i))},_eventHandler:function(t){var i=this.options,n=this.active,a=e(t.currentTarget),o=a.closest("li"),s=o[0]===n[0],r=s&&i.collapsible,l=r?e():this._getPanelForTab(o),c=n.length?this._getPanelForTab(n):e(),h={oldTab:n,oldPanel:c,newTab:r?e():o,newPanel:l};t.preventDefault(),o.hasClass("ui-state-disabled")||o.hasClass("ui-tabs-loading")||this.running||s&&!i.collapsible||!1===this._trigger("beforeActivate",t,h)||(i.active=!r&&this.tabs.index(o),this.active=s?e():o,this.xhr&&this.xhr.abort(),c.length||l.length||e.error("jQuery UI Tabs: Mismatching fragment identifier."),l.length&&this.load(this.tabs.index(o),t),this._toggle(t,h))},_toggle:function(t,i){var n=this,a=i.newPanel,o=i.oldPanel;function s(){n.running=!1,n._trigger("activate",t,i)}function r(){i.newTab.closest("li").addClass("ui-tabs-active ui-state-active"),a.length&&n.options.show?n._show(a,n.options.show,s):(a.show(),s())}this.running=!0,o.length&&this.options.hide?this._hide(o,this.options.hide,(function(){i.oldTab.closest("li").removeClass("ui-tabs-active ui-state-active"),r()})):(i.oldTab.closest("li").removeClass("ui-tabs-active ui-state-active"),o.hide(),r()),o.attr({"aria-expanded":"false","aria-hidden":"true"}),i.oldTab.attr("aria-selected","false"),a.length&&o.length?i.oldTab.attr("tabIndex",-1):a.length&&this.tabs.filter((function(){return 0===e(this).attr("tabIndex")})).attr("tabIndex",-1),a.attr({"aria-expanded":"true","aria-hidden":"false"}),i.newTab.attr({"aria-selected":"true",tabIndex:0})},_activate:function(t){var i,n=this._findActive(t);n[0]!==this.active[0]&&(n.length||(n=this.active),i=n.find(".ui-tabs-anchor")[0],this._eventHandler({target:i,currentTarget:i,preventDefault:e.noop}))},_findActive:function(t){return!1===t?e():this.tabs.eq(t)},_getIndex:function(e){return"string"===typeof e&&(e=this.anchors.index(this.anchors.filter("[href$='"+e+"']"))),e},_destroy:function(){this.xhr&&this.xhr.abort(),this.element.removeClass("ui-tabs ui-widget ui-widget-content ui-corner-all ui-tabs-collapsible"),this.tablist.removeClass("ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all").removeAttr("role"),this.anchors.removeClass("ui-tabs-anchor").removeAttr("role").removeAttr("tabIndex").removeData("href.tabs").removeData("load.tabs").removeUniqueId(),this.tabs.add(this.panels).each((function(){e.data(this,"ui-tabs-destroy")?e(this).remove():e(this).removeClass("ui-state-default ui-state-active ui-state-disabled ui-corner-top ui-corner-bottom ui-widget-content ui-tabs-active ui-tabs-panel").removeAttr("tabIndex").removeAttr("aria-live").removeAttr("aria-busy").removeAttr("aria-selected").removeAttr("aria-labelledby").removeAttr("aria-hidden").removeAttr("aria-expanded").removeAttr("role")})),this.tabs.each((function(){var t=e(this),i=t.data("ui-tabs-aria-controls");i?t.attr("aria-controls",i):t.removeAttr("aria-controls")})),this.panels.show(),"content"!==this.options.heightStyle&&this.panels.css("height","")},enable:function(i){var n=this.options.disabled;!1!==n&&(i===t?n=!1:(i=this._getIndex(i),n=e.isArray(n)?e.map(n,(function(e){return e!==i?e:null})):e.map(this.tabs,(function(e,t){return t!==i?t:null}))),this._setupDisabled(n))},disable:function(i){var n=this.options.disabled;if(!0!==n){if(i===t)n=!0;else{if(i=this._getIndex(i),-1!==e.inArray(i,n))return;n=e.isArray(n)?e.merge([i],n).sort():[i]}this._setupDisabled(n)}},load:function(t,i){t=this._getIndex(t);var n=this,a=this.tabs.eq(t),s=a.find(".ui-tabs-anchor"),r=this._getPanelForTab(a),l={tab:a,panel:r};o(s[0])||(this.xhr=e.ajax(this._ajaxSettings(s,i,l)),this.xhr&&"canceled"!==this.xhr.statusText&&(a.addClass("ui-tabs-loading"),r.attr("aria-busy","true"),this.xhr.success((function(e){setTimeout((function(){r.html(e),n._trigger("load",i,l)}),1)})).complete((function(e,t){setTimeout((function(){"abort"===t&&n.panels.stop(!1,!0),a.removeClass("ui-tabs-loading"),r.removeAttr("aria-busy"),e===n.xhr&&delete n.xhr}),1)}))))},_ajaxSettings:function(t,i,n){var a=this;return{url:t.attr("href"),beforeSend:function(t,o){return a._trigger("beforeLoad",i,e.extend({jqXHR:t,ajaxSettings:o},n))}}},_getPanelForTab:function(t){var i=e(t).attr("aria-controls");return this.element.find(this._sanitizeSelector("#"+i))}}),!1!==e.uiBackCompat&&(e.ui.tabs.prototype._ui=function(e,t){return{tab:e,panel:t,index:this.anchors.index(e)}},e.widget("ui.tabs",e.ui.tabs,{url:function(e,t){this.anchors.eq(e).attr("href",t)}}),e.widget("ui.tabs",e.ui.tabs,{options:{ajaxOptions:null,cache:!1},_create:function(){this._super();var t=this;this._on({tabsbeforeload:function(i,n){e.data(n.tab[0],"cache.tabs")?i.preventDefault():n.jqXHR.success((function(){t.options.cache&&e.data(n.tab[0],"cache.tabs",!0)}))}})},_ajaxSettings:function(t,i,n){var a=this.options.ajaxOptions;return e.extend({},a,{error:function(e,t){try{a.error(e,t,n.tab.closest("li").index(),n.tab[0])}catch(i){}}},this._superApply(arguments))},_setOption:function(e,t){"cache"===e&&!1===t&&this.anchors.removeData("cache.tabs"),this._super(e,t)},_destroy:function(){this.anchors.removeData("cache.tabs"),this._super()},url:function(e){this.anchors.eq(e).removeData("cache.tabs"),this._superApply(arguments)}}),e.widget("ui.tabs",e.ui.tabs,{abort:function(){this.xhr&&this.xhr.abort()}}),e.widget("ui.tabs",e.ui.tabs,{options:{spinner:"<em>Loading&#8230;</em>"},_create:function(){this._super(),this._on({tabsbeforeload:function(e,t){if(e.target===this.element[0]&&this.options.spinner){var i=t.tab.find("span"),n=i.html();i.html(this.options.spinner),t.jqXHR.complete((function(){i.html(n)}))}}})}}),e.widget("ui.tabs",e.ui.tabs,{options:{enable:null,disable:null},enable:function(t){var i,n=this.options;(t&&!0===n.disabled||e.isArray(n.disabled)&&-1!==e.inArray(t,n.disabled))&&(i=!0),this._superApply(arguments),i&&this._trigger("enable",null,this._ui(this.anchors[t],this.panels[t]))},disable:function(t){var i,n=this.options;(t&&!1===n.disabled||e.isArray(n.disabled)&&-1===e.inArray(t,n.disabled))&&(i=!0),this._superApply(arguments),i&&this._trigger("disable",null,this._ui(this.anchors[t],this.panels[t]))}}),e.widget("ui.tabs",e.ui.tabs,{options:{add:null,remove:null,tabTemplate:"<li><a href='#{href}'><span>#{label}</span></a></li>"},add:function(i,n,a){a===t&&(a=this.anchors.length);var o,s,r=this.options,l=e(r.tabTemplate.replace(/#\{href\}/g,i).replace(/#\{label\}/g,n)),c=i.indexOf("#")?this._tabId(l):i.replace("#","");return l.addClass("ui-state-default ui-corner-top").data("ui-tabs-destroy",!0),l.attr("aria-controls",c),o=a>=this.tabs.length,s=this.element.find("#"+c),s.length||(s=this._createPanel(c),o?a>0?s.insertAfter(this.panels.eq(-1)):s.appendTo(this.element):s.insertBefore(this.panels[a])),s.addClass("ui-tabs-panel ui-widget-content ui-corner-bottom").hide(),o?l.appendTo(this.tablist):l.insertBefore(this.tabs[a]),r.disabled=e.map(r.disabled,(function(e){return e>=a?++e:e})),this.refresh(),1===this.tabs.length&&!1===r.active&&this.option("active",0),this._trigger("add",null,this._ui(this.anchors[a],this.panels[a])),this},remove:function(t){t=this._getIndex(t);var i=this.options,n=this.tabs.eq(t).remove(),a=this._getPanelForTab(n).remove();return n.hasClass("ui-tabs-active")&&this.anchors.length>2&&this._activate(t+(t+1<this.anchors.length?1:-1)),i.disabled=e.map(e.grep(i.disabled,(function(e){return e!==t})),(function(e){return e>=t?--e:e})),this.refresh(),this._trigger("remove",null,this._ui(n.find("a")[0],a[0])),this}}),e.widget("ui.tabs",e.ui.tabs,{length:function(){return this.anchors.length}}),e.widget("ui.tabs",e.ui.tabs,{options:{idPrefix:"ui-tabs-"},_tabId:function(t){var i=t.is("li")?t.find("a[href]"):t;return i=i[0],e(i).closest("li").attr("aria-controls")||i.title&&i.title.replace(/\s/g,"_").replace(/[^\w\u00c0-\uFFFF\-]/g,"")||this.options.idPrefix+a()}}),e.widget("ui.tabs",e.ui.tabs,{options:{panelTemplate:"<div></div>"},_createPanel:function(t){return e(this.options.panelTemplate).attr("id",t).addClass("ui-tabs-panel ui-widget-content ui-corner-bottom").data("ui-tabs-destroy",!0)}}),e.widget("ui.tabs",e.ui.tabs,{_create:function(){var e=this.options;null===e.active&&e.selected!==t&&(e.active=-1!==e.selected&&e.selected),this._super(),e.selected=e.active,!1===e.selected&&(e.selected=-1)},_setOption:function(e,t){if("selected"!==e)return this._super(e,t);var i=this.options;this._super("active",-1!==t&&t),i.selected=i.active,!1===i.selected&&(i.selected=-1)},_eventHandler:function(){this._superApply(arguments),this.options.selected=this.options.active,!1===this.options.selected&&(this.options.selected=-1)}}),e.widget("ui.tabs",e.ui.tabs,{options:{show:null,select:null},_create:function(){this._super(),!1!==this.options.active&&this._trigger("show",null,this._ui(this.active.find(".ui-tabs-anchor")[0],this._getPanelForTab(this.active)[0]))},_trigger:function(e,t,i){var n,a,o=this._superApply(arguments);return!!o&&("beforeActivate"===e?(n=i.newTab.length?i.newTab:i.oldTab,a=i.newPanel.length?i.newPanel:i.oldPanel,o=this._super("select",t,{tab:n.find(".ui-tabs-anchor")[0],panel:a[0],index:n.closest("li").index()})):"activate"===e&&i.newTab.length&&(o=this._super("show",t,{tab:i.newTab.find(".ui-tabs-anchor")[0],panel:i.newPanel[0],index:i.newTab.closest("li").index()})),o)}}),e.widget("ui.tabs",e.ui.tabs,{select:function(e){if(e=this._getIndex(e),-1===e){if(!this.options.collapsible||-1===this.options.selected)return;e=this.options.selected}this.anchors.eq(e).trigger(this.options.event+this.eventNamespace)}}),function(){var t=0;e.widget("ui.tabs",e.ui.tabs,{options:{cookie:null},_create:function(){var e,t=this.options;null==t.active&&t.cookie&&(e=parseInt(this._cookie(),10),-1===e&&(e=!1),t.active=e),this._super()},_cookie:function(i){var n=[this.cookie||(this.cookie=this.options.cookie.name||"ui-tabs-"+ ++t)];return arguments.length&&(n.push(!1===i?-1:i),n.push(this.options.cookie)),e.cookie.apply(null,n)},_refresh:function(){this._super(),this.options.cookie&&this._cookie(this.options.active,this.options.cookie)},_eventHandler:function(){this._superApply(arguments),this.options.cookie&&this._cookie(this.options.active,this.options.cookie)},_destroy:function(){this._super(),this.options.cookie&&this._cookie(null,this.options.cookie)}})}(),e.widget("ui.tabs",e.ui.tabs,{_trigger:function(t,i,n){var a=e.extend({},n);return"load"===t&&(a.panel=a.panel[0],a.tab=a.tab.find(".ui-tabs-anchor")[0]),this._super(t,i,a)}}),e.widget("ui.tabs",e.ui.tabs,{options:{fx:null},_getFx:function(){var t,i,n=this.options.fx;return n&&(e.isArray(n)?(t=n[0],i=n[1]):t=i=n),n?{show:i,hide:t}:null},_toggle:function(e,t){var i=this,n=t.newPanel,a=t.oldPanel,o=this._getFx();if(!o)return this._super(e,t);function s(){i.running=!1,i._trigger("activate",e,t)}function r(){t.newTab.closest("li").addClass("ui-tabs-active ui-state-active"),n.length&&o.show?n.animate(o.show,o.show.duration,(function(){s()})):(n.show(),s())}i.running=!0,a.length&&o.hide?a.animate(o.hide,o.hide.duration,(function(){t.oldTab.closest("li").removeClass("ui-tabs-active ui-state-active"),r()})):(t.oldTab.closest("li").removeClass("ui-tabs-active ui-state-active"),a.hide(),r())}}))}(jQuery),function(e){var t=0;function i(t,i){var n=(t.attr("aria-describedby")||"").split(/\s+/);n.push(i),t.data("ui-tooltip-id",i).attr("aria-describedby",e.trim(n.join(" ")))}function n(t){var i=t.data("ui-tooltip-id"),n=(t.attr("aria-describedby")||"").split(/\s+/),a=e.inArray(i,n);-1!==a&&n.splice(a,1),t.removeData("ui-tooltip-id"),n=e.trim(n.join(" ")),n?t.attr("aria-describedby",n):t.removeAttr("aria-describedby")}e.widget("ui.tooltip",{version:"1.9.2",options:{content:function(){return e(this).attr("title")},hide:!0,items:"[title]:not([disabled])",position:{my:"left top+15",at:"left bottom",collision:"flipfit flip"},show:!0,tooltipClass:null,track:!1,close:null,open:null},_create:function(){this._on({mouseover:"open",focusin:"open"}),this.tooltips={},this.parents={},this.options.disabled&&this._disable()},_setOption:function(t,i){var n=this;if("disabled"===t)return this[i?"_disable":"_enable"](),void(this.options[t]=i);this._super(t,i),"content"===t&&e.each(this.tooltips,(function(e,t){n._updateContent(t)}))},_disable:function(){var t=this;e.each(this.tooltips,(function(i,n){var a=e.Event("blur");a.target=a.currentTarget=n[0],t.close(a,!0)})),this.element.find(this.options.items).andSelf().each((function(){var t=e(this);t.is("[title]")&&t.data("ui-tooltip-title",t.attr("title")).attr("title","")}))},_enable:function(){this.element.find(this.options.items).andSelf().each((function(){var t=e(this);t.data("ui-tooltip-title")&&t.attr("title",t.data("ui-tooltip-title"))}))},open:function(t){var i=this,n=e(t?t.target:this.element).closest(this.options.items);n.length&&!n.data("ui-tooltip-id")&&(n.attr("title")&&n.data("ui-tooltip-title",n.attr("title")),n.data("ui-tooltip-open",!0),t&&"mouseover"===t.type&&n.parents().each((function(){var t,n=e(this);n.data("ui-tooltip-open")&&(t=e.Event("blur"),t.target=t.currentTarget=this,i.close(t,!0)),n.attr("title")&&(n.uniqueId(),i.parents[this.id]={element:this,title:n.attr("title")},n.attr("title",""))})),this._updateContent(n,t))},_updateContent:function(e,t){var i,n=this.options.content,a=this,o=t?t.type:null;if("string"===typeof n)return this._open(t,e,n);i=n.call(e[0],(function(i){e.data("ui-tooltip-open")&&a._delay((function(){t&&(t.type=o),this._open(t,e,i)}))})),i&&this._open(t,e,i)},_open:function(t,n,a){var o,s,r,l=e.extend({},this.options.position);function c(e){l.of=e,o.is(":hidden")||o.position(l)}a&&(o=this._find(n),o.length?o.find(".ui-tooltip-content").html(a):(n.is("[title]")&&(t&&"mouseover"===t.type?n.attr("title",""):n.removeAttr("title")),o=this._tooltip(n),i(n,o.attr("id")),o.find(".ui-tooltip-content").html(a),this.options.track&&t&&/^mouse/.test(t.type)?(this._on(this.document,{mousemove:c}),c(t)):o.position(e.extend({of:n},this.options.position)),o.hide(),this._show(o,this.options.show),this.options.show&&this.options.show.delay&&(r=setInterval((function(){o.is(":visible")&&(c(l.of),clearInterval(r))}),e.fx.interval)),this._trigger("open",t,{tooltip:o}),s={keyup:function(t){if(t.keyCode===e.ui.keyCode.ESCAPE){var i=e.Event(t);i.currentTarget=n[0],this.close(i,!0)}},remove:function(){this._removeTooltip(o)}},t&&"mouseover"!==t.type||(s.mouseleave="close"),t&&"focusin"!==t.type||(s.focusout="close"),this._on(!0,n,s)))},close:function(t){var i=this,a=e(t?t.currentTarget:this.element),o=this._find(a);this.closing||(a.data("ui-tooltip-title")&&a.attr("title",a.data("ui-tooltip-title")),n(a),o.stop(!0),this._hide(o,this.options.hide,(function(){i._removeTooltip(e(this))})),a.removeData("ui-tooltip-open"),this._off(a,"mouseleave focusout keyup"),a[0]!==this.element[0]&&this._off(a,"remove"),this._off(this.document,"mousemove"),t&&"mouseleave"===t.type&&e.each(this.parents,(function(t,n){e(n.element).attr("title",n.title),delete i.parents[t]})),this.closing=!0,this._trigger("close",t,{tooltip:o}),this.closing=!1)},_tooltip:function(i){var n="ui-tooltip-"+t++,a=e("<div>").attr({id:n,role:"tooltip"}).addClass("ui-tooltip ui-widget ui-corner-all ui-widget-content "+(this.options.tooltipClass||""));return e("<div>").addClass("ui-tooltip-content").appendTo(a),a.appendTo(this.document[0].body),e.fn.bgiframe&&a.bgiframe(),this.tooltips[n]=i,a},_find:function(t){var i=t.data("ui-tooltip-id");return i?e("#"+i):e()},_removeTooltip:function(e){e.remove(),delete this.tooltips[e.attr("id")]},_destroy:function(){var t=this;e.each(this.tooltips,(function(i,n){var a=e.Event("blur");a.target=a.currentTarget=n[0],t.close(a,!0),e("#"+i).remove(),n.data("ui-tooltip-title")&&(n.attr("title",n.data("ui-tooltip-title")),n.removeData("ui-tooltip-title"))}))}})}(jQuery)},f411:function(e,t,i){},f57b:function(e,t,i){var n={"./0/batch-img.svg":"bf6c","./0/collapse-left.svg":"1db6","./0/collapse-right.svg":"564b","./0/constant-node.svg":"54ae","./0/creat-dataset.svg":"9f1b","./0/dynamic-shape-node.svg":"0a03","./0/graph-step1-en.svg":"ddb6","./0/graph-step1.svg":"428b","./0/graph-step2-en.svg":"eb90","./0/graph-step2.svg":"04c5","./0/graph-step3-en.svg":"18bc","./0/graph-step3.svg":"26b4","./0/graph-stepimg1.svg":"6657","./0/graph-stepimg2.svg":"eb76","./0/graph-stepimg3.svg":"aa39","./0/guideIcon.svg":"ecab","./0/guideIconHover.svg":"ce5d","./0/map-dataset.svg":"c77d","./0/name-scope.svg":"77fb","./0/operator-node.svg":"d18b","./0/polymetric.svg":"4ab4","./0/repeat-dataset.svg":"e5a4","./0/shuffle-dataset.svg":"dbf9","./0/virtual-node.svg":"1cac","./1/batch-img.svg":"3d78","./1/collapse-left.svg":"f8c9","./1/collapse-right.svg":"3805","./1/constant-node.svg":"ec63","./1/creat-dataset.svg":"d674","./1/dynamic-shape-node.svg":"cb78","./1/graph-step1-en.svg":"c356","./1/graph-step1.svg":"385b","./1/graph-step2-en.svg":"1ec7","./1/graph-step2.svg":"a094","./1/graph-step3-en.svg":"344d","./1/graph-step3.svg":"9452","./1/graph-stepimg1.svg":"81e5","./1/graph-stepimg2.svg":"3581","./1/graph-stepimg3.svg":"73d7","./1/guideIcon.svg":"c8e3","./1/guideIconHover.svg":"241b","./1/map-dataset.svg":"1300","./1/name-scope.svg":"9e69","./1/operator-node.svg":"97cd","./1/polymetric.svg":"29c4","./1/repeat-dataset.svg":"a294","./1/shuffle-dataset.svg":"2b98","./1/virtual-node.svg":"cd21","./batch-img.svg":"eba6","./constant-node.svg":"3325","./creat-dataset.svg":"8cb0","./data-process-guid.svg":"532e","./dynamic-shape-node.svg":"8c7d","./explain-fn.svg":"5801","./explain-forecast.svg":"3f34","./explain-fp.svg":"341a","./explain-reality.svg":"e6da","./explain-tn.svg":"1684","./explain-tp.svg":"baf5","./graph-guid.svg":"56de","./histogram-guid.svg":"4936","./icon-down.svg":"96c0","./icon1.svg":"4425","./icon10.svg":"bbf2","./icon2.svg":"014a","./icon3.svg":"91b0","./icon4.svg":"b205","./icon5.svg":"2eb2","./icon6.svg":"0a6a","./icon7.svg":"ba85","./icon8.svg":"ef08d","./icon9.svg":"5e85","./image-guid.svg":"daff","./map-dataset.svg":"9175","./name-scope.svg":"9b3c","./operator-node.svg":"5185","./polymetric.svg":"3e9a","./queue.svg":"1b54","./repeat-dataset.svg":"2fcd","./scalar-guid.svg":"97a7","./shuffle-dataset.svg":"98f4","./svg/link.svg":"89d2","./svg/pipeline-close.svg":"2de0","./svg/pipeline-open.svg":"ac8c","./tensor-guid.svg":"98c0","./virtual-node.svg":"2cd9"};function a(e){var t=o(e);return i(t)}function o(e){if(!i.o(n,e)){var t=new Error("Cannot find module '"+e+"'");throw t.code="MODULE_NOT_FOUND",t}return n[e]}a.keys=function(){return Object.keys(n)},a.resolve=o,e.exports=a,a.id="f57b"},f848:function(e,t,i){},f8c9:function(e,t,i){e.exports=i.p+"static/img/collapse-left.523794c7.svg"}}]);