(window["webpackJsonp"]=window["webpackJsonp"]||[]).push([["chunk-25b42778"],{"06c5":function(e,t,i){"use strict";i.d(t,"a",(function(){return o}));i("a630"),i("fb6a"),i("b0c0"),i("d3b7"),i("ac1f"),i("00b4"),i("25f0"),i("3ca3");var n=i("6b75");function o(e,t){if(e){if("string"==typeof e)return Object(n["a"])(e,t);var i={}.toString.call(e).slice(8,-1);return"Object"===i&&e.constructor&&(i=e.constructor.name),"Map"===i||"Set"===i?Array.from(e):"Arguments"===i||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(i)?Object(n["a"])(e,t):void 0}}},1148:function(e,t,i){"use strict";var n=i("a691"),o=i("1d80");e.exports="".repeat||function(e){var t=String(o(this)),i="",s=n(e);if(s<0||s==1/0)throw RangeError("Wrong number of repetitions");for(;s>0;(s>>>=1)&&(t+=t))1&s&&(i+=t);return i}},2909:function(e,t,i){"use strict";i.d(t,"a",(function(){return l}));var n=i("6b75");function o(e){if(Array.isArray(e))return Object(n["a"])(e)}i("e01a"),i("d28b"),i("a630"),i("e260"),i("d3b7"),i("3ca3"),i("ddb0");function s(e){if("undefined"!=typeof Symbol&&null!=e[Symbol.iterator]||null!=e["@@iterator"])return Array.from(e)}var a=i("06c5");function r(){throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.")}function l(e){return o(e)||s(e)||Object(a["a"])(e)||r()}},"326d":function(e,t,i){},"32c7":function(e,t,i){"use strict";i("5181")},"3a3f":function(module,exports){
/**
 * @license
 * (c) 2009-2016 Michael Leibman
 * michael{dot}leibman{at}gmail{dot}com
 * http://github.com/mleibman/slickgrid
 *
 * Distributed under MIT license.
 * All rights reserved.
 *
 * SlickGrid v2.4
 *
 * NOTES:
 *     Cell/row DOM manipulations are done directly bypassing jQuery's DOM manipulation methods.
 *     This increases the speed dramatically, but can only be done safely because there are no event handlers
 *     or data associated with any cell/row DOM nodes.  Cell editors must make sure they implement .destroy()
 *     and do proper cleanup.
 */
if("undefined"===typeof jQuery)throw new Error("SlickGrid requires jquery module to be loaded");if(!jQuery.fn.drag)throw new Error("SlickGrid requires jquery.event.drag module to be loaded");if("undefined"===typeof Slick)throw new Error("slick.core.js not loaded");(function($){"use strict";var scrollbarDimensions,maxSupportedCssHeight;function SlickGrid(container,data,columns,options){var defaults={alwaysShowVerticalScroll:!1,alwaysAllowHorizontalScroll:!1,explicitInitialization:!1,rowHeight:25,defaultColumnWidth:80,enableAddRow:!1,leaveSpaceForNewRows:!1,editable:!1,autoEdit:!0,suppressActiveCellChangeOnEdit:!1,enableCellNavigation:!0,enableColumnReorder:!0,asyncEditorLoading:!1,asyncEditorLoadDelay:100,forceFitColumns:!1,enableAsyncPostRender:!1,asyncPostRenderDelay:50,enableAsyncPostRenderCleanup:!1,asyncPostRenderCleanupDelay:40,autoHeight:!1,editorLock:Slick.GlobalEditorLock,showColumnHeader:!0,showHeaderRow:!1,headerRowHeight:25,createFooterRow:!1,showFooterRow:!1,footerRowHeight:25,createPreHeaderPanel:!1,showPreHeaderPanel:!1,preHeaderPanelHeight:25,showTopPanel:!1,topPanelHeight:25,formatterFactory:null,editorFactory:null,cellFlashingCssClass:"flashing",selectedCellCssClass:"selected",multiSelect:!0,enableTextSelectionOnCells:!1,dataItemColumnValueExtractor:null,frozenBottom:!1,frozenColumn:-1,frozenRow:-1,fullWidthRows:!1,multiColumnSort:!1,numberedMultiColumnSort:!1,tristateMultiColumnSort:!1,sortColNumberInSeparateSpan:!1,defaultFormatter:defaultFormatter,forceSyncScrolling:!1,addNewRowCssClass:"new-row",preserveCopiedSelectionOnPaste:!1,showCellSelection:!0,viewportClass:null,minRowBuffer:3,emulatePagingWhenScrolling:!0,editorCellNavOnLRKeys:!1,doPaging:!0,autosizeColsMode:Slick.GridAutosizeColsMode.LegacyOff,autosizeColPaddingPx:4,autosizeTextAvgToMWidthRatio:.75,viewportSwitchToScrollModeWidthPercent:void 0,viewportMinWidthPx:void 0,viewportMaxWidthPx:void 0},columnDefaults={name:"",resizable:!0,sortable:!1,minWidth:30,maxWidth:void 0,rerenderOnResize:!1,headerCssClass:null,defaultSortAsc:!0,focusable:!0,selectable:!0},columnAutosizeDefaults={ignoreHeaderText:!1,colValueArray:void 0,allowAddlPercent:void 0,formatterOverride:void 0,autosizeMode:Slick.ColAutosizeMode.ContentIntelligent,rowSelectionModeOnInit:void 0,rowSelectionMode:Slick.RowSelectionMode.FirstNRows,rowSelectionCount:100,valueFilterMode:Slick.ValueFilterMode.None,widthEvalMode:Slick.WidthEvalMode.CanvasTextSize,sizeToRemaining:void 0,widthPx:void 0,colDataTypeOf:void 0},th,h,ph,n,cj,page=0,offset=0,vScrollDir=1,initialized=!1,$container,uid="slickgrid_"+Math.round(1e6*Math.random()),self=this,$focusSink,$focusSink2,$groupHeaders=$(),$headerScroller,$headers,$headerRow,$headerRowScroller,$headerRowSpacerL,$headerRowSpacerR,$footerRow,$footerRowScroller,$footerRowSpacerL,$footerRowSpacerR,$preHeaderPanel,$preHeaderPanelScroller,$preHeaderPanelSpacer,$preHeaderPanelR,$preHeaderPanelScrollerR,$preHeaderPanelSpacerR,$topPanelScroller,$topPanel,$viewport,$canvas,$style,$boundAncestors,treeColumns,stylesheet,columnCssRulesL,columnCssRulesR,viewportH,viewportW,canvasWidth,canvasWidthL,canvasWidthR,headersWidth,headersWidthL,headersWidthR,viewportHasHScroll,viewportHasVScroll,headerColumnWidthDiff=0,headerColumnHeightDiff=0,cellWidthDiff=0,cellHeightDiff=0,jQueryNewWidthBehaviour=!1,absoluteColumnMinWidth,hasFrozenRows=!1,frozenRowsHeight=0,actualFrozenRow=-1,paneTopH=0,paneBottomH=0,viewportTopH=0,viewportBottomH=0,topPanelH=0,headerRowH=0,footerRowH=0,tabbingDirection=1,$activeCanvasNode,$activeViewportNode,activePosX,activeRow,activeCell,activeCellNode=null,currentEditor=null,serializedEditorValue,editController,rowsCache={},renderedRows=0,numVisibleRows=0,prevScrollTop=0,scrollTop=0,lastRenderedScrollTop=0,lastRenderedScrollLeft=0,prevScrollLeft=0,scrollLeft=0,selectionModel,selectedRows=[],plugins=[],cellCssClasses={},columnsById={},sortColumns=[],columnPosLeft=[],columnPosRight=[],pagingActive=!1,pagingIsLastPage=!1,scrollThrottle=ActionThrottle(render,50),h_editorLoader=null,h_render=null,h_postrender=null,h_postrenderCleanup=null,postProcessedRows={},postProcessToRow=null,postProcessFromRow=null,postProcessedCleanupQueue=[],postProcessgroupId=0,counter_rows_rendered=0,counter_rows_removed=0,rowNodeFromLastMouseWheelEvent,zombieRowNodeFromLastMouseWheelEvent,zombieRowCacheFromLastMouseWheelEvent,zombieRowPostProcessedFromLastMouseWheelEvent,$paneHeaderL,$paneHeaderR,$paneTopL,$paneTopR,$paneBottomL,$paneBottomR,$headerScrollerL,$headerScrollerR,$headerL,$headerR,$groupHeadersL,$groupHeadersR,$headerRowScrollerL,$headerRowScrollerR,$footerRowScrollerL,$footerRowScrollerR,$headerRowL,$headerRowR,$footerRowL,$footerRowR,$topPanelScrollerL,$topPanelScrollerR,$topPanelL,$topPanelR,$viewportTopL,$viewportTopR,$viewportBottomL,$viewportBottomR,$canvasTopL,$canvasTopR,$canvasBottomL,$canvasBottomR,$viewportScrollContainerX,$viewportScrollContainerY,$headerScrollContainer,$headerRowScrollContainer,$footerRowScrollContainer,cssShow={position:"absolute",visibility:"hidden",display:"block"},$hiddenParents,oldProps=[],columnResizeDragging=!1;function init(){if($container=container instanceof jQuery?container:$(container),$container.length<1)throw new Error("SlickGrid requires a valid container, "+container+" does not exist in the DOM.");if(cacheCssForHiddenInit(),maxSupportedCssHeight=maxSupportedCssHeight||getMaxSupportedCssHeight(),options=$.extend({},defaults,options),validateAndEnforceOptions(),columnDefaults.width=options.defaultColumnWidth,treeColumns=new Slick.TreeColumns(columns),columns=treeColumns.extractColumns(),updateColumnProps(),options.enableColumnReorder&&!$.fn.sortable)throw new Error("SlickGrid's 'enableColumnReorder = true' option requires jquery-ui.sortable module to be loaded");if(editController={commitCurrentEdit:commitCurrentEdit,cancelCurrentEdit:cancelCurrentEdit},$container.empty().css("overflow","hidden").css("outline",0).addClass(uid).addClass("ui-widget"),/relative|absolute|fixed/.test($container.css("position"))||$container.css("position","relative"),$focusSink=$("<div tabIndex='0' hideFocus style='position:fixed;width:0;height:0;top:0;left:0;outline:0;'></div>").appendTo($container),$paneHeaderL=$("<div class='slick-pane slick-pane-header slick-pane-left' tabIndex='0' />").appendTo($container),$paneHeaderR=$("<div class='slick-pane slick-pane-header slick-pane-right' tabIndex='0' />").appendTo($container),$paneTopL=$("<div class='slick-pane slick-pane-top slick-pane-left' tabIndex='0' />").appendTo($container),$paneTopR=$("<div class='slick-pane slick-pane-top slick-pane-right' tabIndex='0' />").appendTo($container),$paneBottomL=$("<div class='slick-pane slick-pane-bottom slick-pane-left' tabIndex='0' />").appendTo($container),$paneBottomR=$("<div class='slick-pane slick-pane-bottom slick-pane-right' tabIndex='0' />").appendTo($container),options.createPreHeaderPanel&&($preHeaderPanelScroller=$("<div class='slick-preheader-panel ui-state-default' style='overflow:hidden;position:relative;' />").appendTo($paneHeaderL),$preHeaderPanel=$("<div />").appendTo($preHeaderPanelScroller),$preHeaderPanelSpacer=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($preHeaderPanelScroller),$preHeaderPanelScrollerR=$("<div class='slick-preheader-panel ui-state-default' style='overflow:hidden;position:relative;' />").appendTo($paneHeaderR),$preHeaderPanelR=$("<div />").appendTo($preHeaderPanelScrollerR),$preHeaderPanelSpacerR=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($preHeaderPanelScrollerR),options.showPreHeaderPanel||($preHeaderPanelScroller.hide(),$preHeaderPanelScrollerR.hide())),$headerScrollerL=$("<div class='slick-header ui-state-default slick-header-left' />").appendTo($paneHeaderL),$headerScrollerR=$("<div class='slick-header ui-state-default slick-header-right' />").appendTo($paneHeaderR),$headerScroller=$().add($headerScrollerL).add($headerScrollerR),treeColumns.hasDepth()){$groupHeadersL=[],$groupHeadersR=[];for(var e=0;e<treeColumns.getDepth()-1;e++)$groupHeadersL[e]=$("<div class='slick-group-header-columns slick-group-header-columns-left' style='left:-1000px' />").appendTo($headerScrollerL),$groupHeadersR[e]=$("<div class='slick-group-header-columns slick-group-header-columns-right' style='left:-1000px' />").appendTo($headerScrollerR);$groupHeaders=$().add($groupHeadersL).add($groupHeadersR)}$headerL=$("<div class='slick-header-columns slick-header-columns-left' style='left:-1000px' />").appendTo($headerScrollerL),$headerR=$("<div class='slick-header-columns slick-header-columns-right' style='left:-1000px' />").appendTo($headerScrollerR),$headers=$().add($headerL).add($headerR),$headerRowScrollerL=$("<div class='slick-headerrow ui-state-default' />").appendTo($paneTopL),$headerRowScrollerR=$("<div class='slick-headerrow ui-state-default' />").appendTo($paneTopR),$headerRowScroller=$().add($headerRowScrollerL).add($headerRowScrollerR),$headerRowSpacerL=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($headerRowScrollerL),$headerRowSpacerR=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").appendTo($headerRowScrollerR),$headerRowL=$("<div class='slick-headerrow-columns slick-headerrow-columns-left' />").appendTo($headerRowScrollerL),$headerRowR=$("<div class='slick-headerrow-columns slick-headerrow-columns-right' />").appendTo($headerRowScrollerR),$headerRow=$().add($headerRowL).add($headerRowR),$topPanelScrollerL=$("<div class='slick-top-panel-scroller ui-state-default' />").appendTo($paneTopL),$topPanelScrollerR=$("<div class='slick-top-panel-scroller ui-state-default' />").appendTo($paneTopR),$topPanelScroller=$().add($topPanelScrollerL).add($topPanelScrollerR),$topPanelL=$("<div class='slick-top-panel' style='width:10000px' />").appendTo($topPanelScrollerL),$topPanelR=$("<div class='slick-top-panel' style='width:10000px' />").appendTo($topPanelScrollerR),$topPanel=$().add($topPanelL).add($topPanelR),options.showColumnHeader||$headerScroller.hide(),options.showTopPanel||$topPanelScroller.hide(),options.showHeaderRow||$headerRowScroller.hide(),$viewportTopL=$("<div class='slick-viewport slick-viewport-top slick-viewport-left' tabIndex='0' hideFocus />").appendTo($paneTopL),$viewportTopR=$("<div class='slick-viewport slick-viewport-top slick-viewport-right' tabIndex='0' hideFocus />").appendTo($paneTopR),$viewportBottomL=$("<div class='slick-viewport slick-viewport-bottom slick-viewport-left' tabIndex='0' hideFocus />").appendTo($paneBottomL),$viewportBottomR=$("<div class='slick-viewport slick-viewport-bottom slick-viewport-right' tabIndex='0' hideFocus />").appendTo($paneBottomR),$viewport=$().add($viewportTopL).add($viewportTopR).add($viewportBottomL).add($viewportBottomR),$activeViewportNode=$viewportTopL,$canvasTopL=$("<div class='grid-canvas grid-canvas-top grid-canvas-left' tabIndex='0' hideFocus />").appendTo($viewportTopL),$canvasTopR=$("<div class='grid-canvas grid-canvas-top grid-canvas-right' tabIndex='0' hideFocus />").appendTo($viewportTopR),$canvasBottomL=$("<div class='grid-canvas grid-canvas-bottom grid-canvas-left' tabIndex='0' hideFocus />").appendTo($viewportBottomL),$canvasBottomR=$("<div class='grid-canvas grid-canvas-bottom grid-canvas-right' tabIndex='0' hideFocus />").appendTo($viewportBottomR),options.viewportClass&&$viewport.toggleClass(options.viewportClass,!0),$canvas=$().add($canvasTopL).add($canvasTopR).add($canvasBottomL).add($canvasBottomR),scrollbarDimensions=scrollbarDimensions||measureScrollbar(),$activeCanvasNode=$canvasTopL,$preHeaderPanelSpacer&&$preHeaderPanelSpacer.css("width",getCanvasWidth()+scrollbarDimensions.width+"px"),$headers.width(getHeadersWidth()),$headerRowSpacerL.css("width",getCanvasWidth()+scrollbarDimensions.width+"px"),$headerRowSpacerR.css("width",getCanvasWidth()+scrollbarDimensions.width+"px"),options.createFooterRow&&($footerRowScrollerR=$("<div class='slick-footerrow ui-state-default' />").appendTo($paneTopR),$footerRowScrollerL=$("<div class='slick-footerrow ui-state-default' />").appendTo($paneTopL),$footerRowScroller=$().add($footerRowScrollerL).add($footerRowScrollerR),$footerRowSpacerL=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").css("width",getCanvasWidth()+scrollbarDimensions.width+"px").appendTo($footerRowScrollerL),$footerRowSpacerR=$("<div style='display:block;height:1px;position:absolute;top:0;left:0;'></div>").css("width",getCanvasWidth()+scrollbarDimensions.width+"px").appendTo($footerRowScrollerR),$footerRowL=$("<div class='slick-footerrow-columns slick-footerrow-columns-left' />").appendTo($footerRowScrollerL),$footerRowR=$("<div class='slick-footerrow-columns slick-footerrow-columns-right' />").appendTo($footerRowScrollerR),$footerRow=$().add($footerRowL).add($footerRowR),options.showFooterRow||$footerRowScroller.hide()),$focusSink2=$focusSink.clone().appendTo($container),options.explicitInitialization||finishInitialization()}function finishInitialization(){initialized||(initialized=!0,getViewportWidth(),getViewportHeight(),measureCellPaddingAndBorder(),disableSelection($headers),options.enableTextSelectionOnCells||$viewport.on("selectstart.ui",(function(e){return $(e.target).is("input,textarea")})),setFrozenOptions(),setPaneVisibility(),setScroller(),setOverflow(),updateColumnCaches(),createColumnHeaders(),createColumnGroupHeaders(),createColumnFooter(),setupColumnSort(),createCssRules(),resizeCanvas(),bindAncestorScrollEvents(),$container.on("resize.slickgrid",resizeCanvas),$viewport.on("scroll",handleScroll),jQuery.fn.mousewheel&&$viewport.on("mousewheel",handleMouseWheel),$headerScroller.on("contextmenu",handleHeaderContextMenu).on("click",handleHeaderClick).on("mouseenter",".slick-header-column",handleHeaderMouseEnter).on("mouseleave",".slick-header-column",handleHeaderMouseLeave),$headerRowScroller.on("scroll",handleHeaderRowScroll),options.createFooterRow&&($footerRow.on("contextmenu",handleFooterContextMenu).on("click",handleFooterClick),$footerRowScroller.on("scroll",handleFooterRowScroll)),options.createPreHeaderPanel&&$preHeaderPanelScroller.on("scroll",handlePreHeaderPanelScroll),$focusSink.add($focusSink2).on("keydown",handleKeyDown),$canvas.on("keydown",handleKeyDown).on("click",handleClick).on("dblclick",handleDblClick).on("contextmenu",handleContextMenu).on("draginit",handleDragInit).on("dragstart",{distance:3},handleDragStart).on("drag",handleDrag).on("dragend",handleDragEnd).on("mouseenter",".slick-cell",handleMouseEnter).on("mouseleave",".slick-cell",handleMouseLeave),restoreCssFromHiddenInit())}function cacheCssForHiddenInit(){$hiddenParents=$container.parents().addBack().not(":visible"),$hiddenParents.each((function(){var e={};for(var t in cssShow)e[t]=this.style[t],this.style[t]=cssShow[t];oldProps.push(e)}))}function restoreCssFromHiddenInit(){$hiddenParents.each((function(e){var t=oldProps[e];for(var i in cssShow)this.style[i]=t[i]}))}function hasFrozenColumns(){return options.frozenColumn>-1}function registerPlugin(e){plugins.unshift(e),e.init(self)}function unregisterPlugin(e){for(var t=plugins.length;t>=0;t--)if(plugins[t]===e){plugins[t].destroy&&plugins[t].destroy(),plugins.splice(t,1);break}}function getPluginByName(e){for(var t=plugins.length-1;t>=0;t--)if(plugins[t].pluginName===e)return plugins[t]}function setSelectionModel(e){selectionModel&&(selectionModel.onSelectedRangesChanged.unsubscribe(handleSelectedRangesChanged),selectionModel.destroy&&selectionModel.destroy()),selectionModel=e,selectionModel&&(selectionModel.init(self),selectionModel.onSelectedRangesChanged.subscribe(handleSelectedRangesChanged))}function getSelectionModel(){return selectionModel}function getCanvasNode(e,t){e||(e=0),t||(t=0);var i="number"===typeof e?e:getColumnIndex(e);return hasFrozenRows&&t>=actualFrozenRow+(options.frozenBottom?0:1)?hasFrozenColumns()&&i>options.frozenColumn?$canvasBottomR[0]:$canvasBottomL[0]:hasFrozenColumns()&&i>options.frozenColumn?$canvasTopR[0]:$canvasTopL[0]}function getActiveCanvasNode(e){return setActiveCanvasNode(e),$activeCanvasNode[0]}function getCanvases(){return $canvas}function setActiveCanvasNode(e){e&&($activeCanvasNode=$(e.target).closest(".grid-canvas"))}function getViewportNode(){return $viewport[0]}function getActiveViewportNode(e){return setActiveViewPortNode(e),$activeViewportNode[0]}function setActiveViewportNode(e){e&&($activeViewportNode=$(e.target).closest(".slick-viewport"))}function measureScrollbar(){var e=$('<div class="'+$viewport.className+'" style="position:absolute; top:-10000px; left:-10000px; overflow:auto; width:100px; height:100px;"></div>').appendTo("body"),t=$('<div style="width:200px; height:200px; overflow:auto;"></div>').appendTo(e),i={width:e[0].offsetWidth-e[0].clientWidth,height:e[0].offsetHeight-e[0].clientHeight};return t.remove(),e.remove(),i}function getHeadersWidth(){headersWidth=headersWidthL=headersWidthR=0;for(var e=!options.autoHeight,t=0,i=columns.length;t<i;t++){var n=columns[t].width;options.frozenColumn>-1&&t>options.frozenColumn?headersWidthR+=n:headersWidthL+=n}return e&&(options.frozenColumn>-1&&t>options.frozenColumn?headersWidthR+=scrollbarDimensions.width:headersWidthL+=scrollbarDimensions.width),hasFrozenColumns()?(headersWidthL+=1e3,headersWidthR=Math.max(headersWidthR,viewportW)+headersWidthL,headersWidthR+=scrollbarDimensions.width):(headersWidthL+=scrollbarDimensions.width,headersWidthL=Math.max(headersWidthL,viewportW)+1e3),headersWidth=headersWidthL+headersWidthR,Math.max(headersWidth,viewportW)+1e3}function getHeadersWidthL(){return headersWidthL=0,columns.forEach((function(e,t){options.frozenColumn>-1&&t>options.frozenColumn||(headersWidthL+=e.width)})),hasFrozenColumns()?headersWidthL+=1e3:(headersWidthL+=scrollbarDimensions.width,headersWidthL=Math.max(headersWidthL,viewportW)+1e3),headersWidthL}function getHeadersWidthR(){return headersWidthR=0,columns.forEach((function(e,t){options.frozenColumn>-1&&t>options.frozenColumn&&(headersWidthR+=e.width)})),hasFrozenColumns()&&(headersWidthR=Math.max(headersWidthR,viewportW)+getHeadersWidthL(),headersWidthR+=scrollbarDimensions.width),headersWidthR}function getCanvasWidth(){var e=viewportHasVScroll?viewportW-scrollbarDimensions.width:viewportW,t=columns.length;canvasWidthL=canvasWidthR=0;while(t--)hasFrozenColumns()&&t>options.frozenColumn?canvasWidthR+=columns[t].width:canvasWidthL+=columns[t].width;var i=canvasWidthL+canvasWidthR;return options.fullWidthRows?Math.max(i,e):i}function updateCanvasWidth(e){var t,i=canvasWidth,n=canvasWidthL,o=canvasWidthR;canvasWidth=getCanvasWidth(),t=canvasWidth!==i||canvasWidthL!==n||canvasWidthR!==o,(t||hasFrozenColumns()||hasFrozenRows)&&($canvasTopL.width(canvasWidthL),getHeadersWidth(),$headerL.width(headersWidthL),$headerR.width(headersWidthR),hasFrozenColumns()?($canvasTopR.width(canvasWidthR),$paneHeaderL.width(canvasWidthL),$paneHeaderR.css("left",canvasWidthL),$paneHeaderR.css("width",viewportW-canvasWidthL),$paneTopL.width(canvasWidthL),$paneTopR.css("left",canvasWidthL),$paneTopR.css("width",viewportW-canvasWidthL),$headerRowScrollerL.width(canvasWidthL),$headerRowScrollerR.width(viewportW-canvasWidthL),$headerRowL.width(canvasWidthL),$headerRowR.width(canvasWidthR),options.createFooterRow&&($footerRowScrollerL.width(canvasWidthL),$footerRowScrollerR.width(viewportW-canvasWidthL),$footerRowL.width(canvasWidthL),$footerRowR.width(canvasWidthR)),options.createPreHeaderPanel&&$preHeaderPanel.width(canvasWidth),$viewportTopL.width(canvasWidthL),$viewportTopR.width(viewportW-canvasWidthL),hasFrozenRows&&($paneBottomL.width(canvasWidthL),$paneBottomR.css("left",canvasWidthL),$viewportBottomL.width(canvasWidthL),$viewportBottomR.width(viewportW-canvasWidthL),$canvasBottomL.width(canvasWidthL),$canvasBottomR.width(canvasWidthR))):($paneHeaderL.width("100%"),$paneTopL.width("100%"),$headerRowScrollerL.width("100%"),$headerRowL.width(canvasWidth),options.createFooterRow&&($footerRowScrollerL.width("100%"),$footerRowL.width(canvasWidth)),options.createPreHeaderPanel&&($preHeaderPanel.width("100%"),$preHeaderPanel.width(canvasWidth)),$viewportTopL.width("100%"),hasFrozenRows&&($viewportBottomL.width("100%"),$canvasBottomL.width(canvasWidthL))),viewportHasHScroll=canvasWidth>viewportW-scrollbarDimensions.width),$headerRowSpacerL.width(canvasWidth+(viewportHasVScroll?scrollbarDimensions.width:0)),$headerRowSpacerR.width(canvasWidth+(viewportHasVScroll?scrollbarDimensions.width:0)),options.createFooterRow&&($footerRowSpacerL.width(canvasWidth+(viewportHasVScroll?scrollbarDimensions.width:0)),$footerRowSpacerR.width(canvasWidth+(viewportHasVScroll?scrollbarDimensions.width:0))),(t||e)&&applyColumnWidths()}function disableSelection(e){e&&e.jquery&&e.attr("unselectable","on").css("MozUserSelect","none").on("selectstart.ui",(function(){return!1}))}function getMaxSupportedCssHeight(){var e=1e6,t=navigator.userAgent.toLowerCase().match(/firefox/)?6e6:1e9,i=$("<div style='display:none' />").appendTo(document.body);while(1){var n=2*e;if(i.css("height",n),n>t||i.height()!==n)break;e=n}return i.remove(),e}function getUID(){return uid}function getHeaderColumnWidthDiff(){return headerColumnWidthDiff}function getScrollbarDimensions(){return scrollbarDimensions}function bindAncestorScrollEvents(){var e=hasFrozenRows&&!options.frozenBottom?$canvasBottomL[0]:$canvasTopL[0];while((e=e.parentNode)!=document.body&&null!=e)if(e==$viewportTopL[0]||e.scrollWidth!=e.clientWidth||e.scrollHeight!=e.clientHeight){var t=$(e);$boundAncestors=$boundAncestors?$boundAncestors.add(t):t,t.on("scroll."+uid,handleActiveCellPositionChange)}}function unbindAncestorScrollEvents(){$boundAncestors&&($boundAncestors.off("scroll."+uid),$boundAncestors=null)}function updateColumnHeader(e,t,i){if(initialized){var n=getColumnIndex(e);if(null!=n){var o=columns[n],s=$headers.children().eq(n);s&&(void 0!==t&&(columns[n].name=t),void 0!==i&&(columns[n].toolTip=i),trigger(self.onBeforeHeaderCellDestroy,{node:s[0],column:o,grid:self}),s.attr("title",i||"").children().eq(0).html(t),trigger(self.onHeaderCellRendered,{node:s[0],column:o,grid:self}))}}}function getHeader(e){if(!e)return hasFrozenColumns()?$headers:$headerL;var t=getColumnIndex(e.id);return hasFrozenColumns()?t<=options.frozenColumn?$headerL:$headerR:$headerL}function getHeaderColumn(e){var t="number"===typeof e?e:getColumnIndex(e),i=hasFrozenColumns()?t<=options.frozenColumn?$headerL:$headerR:$headerL,n=hasFrozenColumns()?t<=options.frozenColumn?t:t-options.frozenColumn-1:t,o=i.children().eq(n);return o&&o[0]}function getHeaderRow(){return hasFrozenColumns()?$headerRow:$headerRow[0]}function getFooterRow(){return hasFrozenColumns()?$footerRow:$footerRow[0]}function getPreHeaderPanel(){return $preHeaderPanel[0]}function getPreHeaderPanelRight(){return $preHeaderPanelR[0]}function getHeaderRowColumn(e){var t,i="number"===typeof e?e:getColumnIndex(e);hasFrozenColumns()?i<=options.frozenColumn?t=$headerRowL:(t=$headerRowR,i-=options.frozenColumn+1):t=$headerRowL;var n=t.children().eq(i);return n&&n[0]}function getFooterRowColumn(e){var t,i="number"===typeof e?e:getColumnIndex(e);hasFrozenColumns()?i<=options.frozenColumn?t=$footerRowL:(t=$footerRowR,i-=options.frozenColumn+1):t=$footerRowL;var n=t&&t.children().eq(i);return n&&n[0]}function createColumnFooter(){if(options.createFooterRow){$footerRow.find(".slick-footerrow-column").each((function(){var e=$(this).data("column");e&&trigger(self.onBeforeFooterRowCellDestroy,{node:this,column:e,grid:self})})),$footerRowL.empty(),$footerRowR.empty();for(var e=0;e<columns.length;e++){var t=columns[e],i=$("<div class='ui-state-default slick-footerrow-column l"+e+" r"+e+"'></div>").data("column",t).addClass(hasFrozenColumns()&&e<=options.frozenColumn?"frozen":"").appendTo(hasFrozenColumns()&&e>options.frozenColumn?$footerRowR:$footerRowL);trigger(self.onFooterRowCellRendered,{node:i[0],column:t,grid:self})}}}function createColumnGroupHeaders(){var e=0,t=!1;if(treeColumns.hasDepth()){for(var i=0;i<$groupHeadersL.length;i++){$groupHeadersL[i].empty(),$groupHeadersR[i].empty();var n=treeColumns.getColumnsInDepth(i);for(var o in n){var s=n[o];e+=s.extractColumns().length,hasFrozenColumns()&&0===i&&e-1===options.frozenColumn&&(t=!0),$("<div class='ui-state-default slick-group-header-column' />").html("<span class='slick-column-name'>"+s.name+"</span>").attr("id",""+uid+s.id).attr("title",s.toolTip||"").data("column",s).addClass(s.headerCssClass||"").addClass(hasFrozenColumns()&&e-1>options.frozenColumn?"frozen":"").appendTo(hasFrozenColumns()&&e-1>options.frozenColumn?$groupHeadersR[i]:$groupHeadersL[i])}if(hasFrozenColumns()&&0===i&&!t){$groupHeadersL[i].empty(),$groupHeadersR[i].empty(),alert("All columns of group should to be grouped!");break}}applyColumnGroupHeaderWidths()}}function createColumnHeaders(){function e(){$(this).addClass("ui-state-hover")}function t(){$(this).removeClass("ui-state-hover")}$headers.find(".slick-header-column").each((function(){var e=$(this).data("column");e&&trigger(self.onBeforeHeaderCellDestroy,{node:this,column:e,grid:self})})),$headerL.empty(),$headerR.empty(),getHeadersWidth(),$headerL.width(headersWidthL),$headerR.width(headersWidthR),$headerRow.find(".slick-headerrow-column").each((function(){var e=$(this).data("column");e&&trigger(self.onBeforeHeaderRowCellDestroy,{node:this,column:e,grid:self})})),$headerRowL.empty(),$headerRowR.empty(),options.createFooterRow&&($footerRowL.find(".slick-footerrow-column").each((function(){var e=$(this).data("column");e&&trigger(self.onBeforeFooterRowCellDestroy,{node:this,column:e,grid:self})})),$footerRowL.empty(),hasFrozenColumns()&&($footerRowR.find(".slick-footerrow-column").each((function(){var e=$(this).data("column");e&&trigger(self.onBeforeFooterRowCellDestroy,{node:this,column:e,grid:self})})),$footerRowR.empty()));for(var i=0;i<columns.length;i++){var n=columns[i],o=hasFrozenColumns()?i<=options.frozenColumn?$headerL:$headerR:$headerL,s=hasFrozenColumns()?i<=options.frozenColumn?$headerRowL:$headerRowR:$headerRowL,a=$("<div class='ui-state-default slick-header-column' />").html("<span class='slick-column-name'>"+n.name+"</span>").width(n.width-headerColumnWidthDiff).attr("id",""+uid+n.id).attr("title",n.toolTip||"").data("column",n).addClass(n.headerCssClass||"").addClass(hasFrozenColumns()&&i<=options.frozenColumn?"frozen":"").appendTo(o);if((options.enableColumnReorder||n.sortable)&&a.on("mouseenter",e).on("mouseleave",t),n.hasOwnProperty("headerCellAttrs")&&n.headerCellAttrs instanceof Object)for(var r in n.headerCellAttrs)n.headerCellAttrs.hasOwnProperty(r)&&a.attr(r,n.headerCellAttrs[r]);if(n.sortable&&(a.addClass("slick-header-sortable"),a.append("<span class='slick-sort-indicator"+(options.numberedMultiColumnSort&&!options.sortColNumberInSeparateSpan?" slick-sort-indicator-numbered":"")+"' />"),options.numberedMultiColumnSort&&options.sortColNumberInSeparateSpan&&a.append("<span class='slick-sort-indicator-numbered' />")),trigger(self.onHeaderCellRendered,{node:a[0],column:n,grid:self}),options.showHeaderRow){var l=$("<div class='ui-state-default slick-headerrow-column l"+i+" r"+i+"'></div>").data("column",n).addClass(hasFrozenColumns()&&i<=options.frozenColumn?"frozen":"").appendTo(s);trigger(self.onHeaderRowCellRendered,{node:l[0],column:n,grid:self})}if(options.createFooterRow&&options.showFooterRow){var c=$("<div class='ui-state-default slick-footerrow-column l"+i+" r"+i+"'></div>").data("column",n).appendTo($footerRow);trigger(self.onFooterRowCellRendered,{node:c[0],column:n,grid:self})}}setSortColumns(sortColumns),setupColumnResize(),options.enableColumnReorder&&("function"==typeof options.enableColumnReorder?options.enableColumnReorder(self,$headers,headerColumnWidthDiff,setColumns,setupColumnResize,columns,getColumnIndex,uid,trigger):setupColumnReorder())}function setupColumnSort(){$headers.click((function(e){if(!columnResizeDragging&&(e.metaKey=e.metaKey||e.ctrlKey,!$(e.target).hasClass("slick-resizable-handle"))){var t=$(e.target).closest(".slick-header-column");if(t.length){var i=t.data("column");if(i.sortable){if(!getEditorLock().commitCurrentEdit())return;for(var n=null,o=0;o<sortColumns.length;o++)if(sortColumns[o].columnId==i.id){n=sortColumns[o],n.sortAsc=!n.sortAsc;break}var s=!!n;options.tristateMultiColumnSort?(n||(n={columnId:i.id,sortAsc:i.defaultSortAsc}),s&&n.sortAsc&&(sortColumns.splice(o,1),n=null),options.multiColumnSort||(sortColumns=[]),!n||s&&options.multiColumnSort||sortColumns.push(n)):e.metaKey&&options.multiColumnSort?n&&sortColumns.splice(o,1):((e.shiftKey||e.metaKey)&&options.multiColumnSort||(sortColumns=[]),n?0===sortColumns.length&&sortColumns.push(n):(n={columnId:i.id,sortAsc:i.defaultSortAsc},sortColumns.push(n))),setSortColumns(sortColumns),options.multiColumnSort?trigger(self.onSort,{multiColumnSort:!0,sortCols:$.map(sortColumns,(function(e){return{columnId:columns[getColumnIndex(e.columnId)].id,sortCol:columns[getColumnIndex(e.columnId)],sortAsc:e.sortAsc}}))},e):trigger(self.onSort,{multiColumnSort:!1,columnId:sortColumns.length>0?i.id:null,sortCol:sortColumns.length>0?i:null,sortAsc:!(sortColumns.length>0)||sortColumns[0].sortAsc},e)}}}}))}function currentPositionInHeader(e){var t=0;return $headers.find(".slick-header-column").each((function(i){if(this.id==e)return t=i,!1})),t}function limitPositionInGroup(e){var t,i=0,n=0;return treeColumns.getColumnsInDepth($groupHeadersL.length-1).some((function(o){return i=n,n+=o.columns.length,o.columns.some((function(i){return i.id===e&&(t=o),t})),t})),n--,{start:i,end:n,group:t}}function remove(e,t){var i=e.lastIndexOf(t);i>-1&&(e.splice(i,1),remove(e,t))}function columnPositionValidInGroup(e){var t=currentPositionInHeader(e[0].id),i=limitPositionInGroup(e.data("column").id),n=i.start<=t&&t<=i.end;return{limit:i,valid:n,message:n?"":'Column "'.concat(e.text(),'" can be reordered only within the "',i.group.name,'" group!')}}function setupColumnReorder(){$headers.filter(":ui-sortable").sortable("destroy");var e,t=null;function i(){$viewportScrollContainerX[0].scrollLeft=$viewportScrollContainerX[0].scrollLeft+10}function n(){$viewportScrollContainerX[0].scrollLeft=$viewportScrollContainerX[0].scrollLeft-10}$headers.sortable({containment:"parent",distance:3,axis:"x",cursor:"default",tolerance:"intersection",helper:"clone",placeholder:"slick-sortable-placeholder ui-state-default slick-header-column",start:function(t,i){i.placeholder.width(i.helper.outerWidth()-headerColumnWidthDiff),e=!hasFrozenColumns()||i.placeholder.offset().left+i.placeholder.width()>$viewportScrollContainerX.offset().left,$(i.helper).addClass("slick-header-column-active")},beforeStop:function(e,t){$(t.helper).removeClass("slick-header-column-active")},sort:function(o,s){e&&o.originalEvent.pageX>$container[0].clientWidth?t||(t=setInterval(i,100)):e&&o.originalEvent.pageX<$viewportScrollContainerX.offset().left?t||(t=setInterval(n,100)):(clearInterval(t),t=null)},stop:function(e,i){var n=!1;clearInterval(t),t=null;var o=null;if(treeColumns.hasDepth()){var s=columnPositionValidInGroup(i.item);o=s.limit,n=!s.valid,n&&alert(s.message)}if(!n&&getEditorLock().commitCurrentEdit()){var a=$headerL.sortable("toArray");a=a.concat($headerR.sortable("toArray"));for(var r=[],l=0;l<a.length;l++)r.push(columns[getColumnIndex(a[l].replace(uid,""))]);setColumns(r),trigger(self.onColumnsReordered,{impactedColumns:getImpactedColumns(o)}),e.stopPropagation(),setupColumnResize()}else $(this).sortable("cancel")}})}function getImpactedColumns(e){var t=[];if(e)for(var i=e.start;i<=e.end;i++)t.push(columns[i]);else t=columns;return t}function setupColumnResize(){var e,t,i,n,o,s,a,r,l;o=$headers.children(),o.find(".slick-resizable-handle").remove(),o.each((function(e,t){e>=columns.length||columns[e].resizable&&(void 0===r&&(r=e),l=e)})),void 0!==r&&o.each((function(c,h){c>=columns.length||c<r||options.forceFitColumns&&c>=l||($(h),$("<div class='slick-resizable-handle' />").appendTo(h).on("dragstart",(function(t,r){if(!getEditorLock().commitCurrentEdit())return!1;n=t.pageX,$(this).parent().addClass("slick-header-column-active");var l=null,h=null;if(o.each((function(e,t){e>=columns.length||(columns[e].previousWidth=$(t).outerWidth())})),options.forceFitColumns)for(l=0,h=0,e=c+1;e<columns.length;e++)i=columns[e],i.resizable&&(null!==h&&(i.maxWidth?h+=i.maxWidth-i.previousWidth:h=null),l+=i.previousWidth-Math.max(i.minWidth||0,absoluteColumnMinWidth));var u=0,d=0;for(e=0;e<=c;e++)i=columns[e],i.resizable&&(null!==d&&(i.maxWidth?d+=i.maxWidth-i.previousWidth:d=null),u+=i.previousWidth-Math.max(i.minWidth||0,absoluteColumnMinWidth));null===l&&(l=1e5),null===u&&(u=1e5),null===h&&(h=1e5),null===d&&(d=1e5),a=n+Math.min(l,d),s=n-Math.min(u,h)})).on("drag",(function(o,r){columnResizeDragging=!0;var l,h,u=Math.min(a,Math.max(s,o.pageX))-n,d=0;if(u<0){for(h=u,e=c;e>=0;e--)i=columns[e],i.resizable&&(l=Math.max(i.minWidth||0,absoluteColumnMinWidth),h&&i.previousWidth+h<l?(h+=i.previousWidth-l,i.width=l):(i.width=i.previousWidth+h,h=0));for(t=0;t<=c;t++)i=columns[t],hasFrozenColumns()&&t>options.frozenColumn?i.width:d+=i.width;if(options.forceFitColumns)for(h=-u,e=c+1;e<columns.length;e++)i=columns[e],i.resizable&&(h&&i.maxWidth&&i.maxWidth-i.previousWidth<h?(h-=i.maxWidth-i.previousWidth,i.width=i.maxWidth):(i.width=i.previousWidth+h,h=0),hasFrozenColumns()&&e>options.frozenColumn?i.width:d+=i.width);else for(e=c+1;e<columns.length;e++)i=columns[e],hasFrozenColumns()&&e>options.frozenColumn?i.width:d+=i.width;if(options.forceFitColumns)for(h=-u,e=c+1;e<columns.length;e++)i=columns[e],i.resizable&&(h&&i.maxWidth&&i.maxWidth-i.previousWidth<h?(h-=i.maxWidth-i.previousWidth,i.width=i.maxWidth):(i.width=i.previousWidth+h,h=0))}else{for(h=u,d=0,0,e=c;e>=0;e--)i=columns[e],i.resizable&&(h&&i.maxWidth&&i.maxWidth-i.previousWidth<h?(h-=i.maxWidth-i.previousWidth,i.width=i.maxWidth):(i.width=i.previousWidth+h,h=0));for(t=0;t<=c;t++)i=columns[t],hasFrozenColumns()&&t>options.frozenColumn?i.width:d+=i.width;if(options.forceFitColumns)for(h=-u,e=c+1;e<columns.length;e++)i=columns[e],i.resizable&&(l=Math.max(i.minWidth||0,absoluteColumnMinWidth),h&&i.previousWidth+h<l?(h+=i.previousWidth-l,i.width=l):(i.width=i.previousWidth+h,h=0),hasFrozenColumns()&&e>options.frozenColumn?i.width:d+=i.width);else for(e=c+1;e<columns.length;e++)i=columns[e],hasFrozenColumns()&&e>options.frozenColumn?i.width:d+=i.width}hasFrozenColumns()&&d!=canvasWidthL&&($headerL.width(d+1e3),$paneHeaderR.css("left",d)),applyColumnHeaderWidths(),applyColumnGroupHeaderWidths(),options.syncColumnCellResize&&applyColumnWidths()})).on("dragend",(function(t,n){var s;for($(this).parent().removeClass("slick-header-column-active"),e=0;e<columns.length;e++)i=columns[e],s=$(o[e]).outerWidth(),i.previousWidth!==s&&i.rerenderOnResize&&invalidateAllRows();updateCanvasWidth(!0),render(),trigger(self.onColumnsResized,{triggeredByColumn:$(this).parent().attr("id").replace(uid,"")}),setTimeout((function(){columnResizeDragging=!1}),300)})))}))}function getVBoxDelta(e){var t=["borderTopWidth","borderBottomWidth","paddingTop","paddingBottom"],i=0;return e&&"function"===typeof e.css&&$.each(t,(function(t,n){i+=parseFloat(e.css(n))||0})),i}function setFrozenOptions(){if(options.frozenColumn=options.frozenColumn>=0&&options.frozenColumn<columns.length?parseInt(options.frozenColumn):-1,options.frozenRow>-1){hasFrozenRows=!0,frozenRowsHeight=options.frozenRow*options.rowHeight;var e=getDataLength();actualFrozenRow=options.frozenBottom?e-options.frozenRow:options.frozenRow}else hasFrozenRows=!1}function setPaneVisibility(){hasFrozenColumns()?($paneHeaderR.show(),$paneTopR.show(),hasFrozenRows?($paneBottomL.show(),$paneBottomR.show()):($paneBottomR.hide(),$paneBottomL.hide())):($paneHeaderR.hide(),$paneTopR.hide(),$paneBottomR.hide(),hasFrozenRows?$paneBottomL.show():($paneBottomR.hide(),$paneBottomL.hide()))}function setOverflow(){$viewportTopL.css({"overflow-x":hasFrozenColumns()?hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"hidden":"scroll":hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"hidden":"auto","overflow-y":options.alwaysShowVerticalScroll?"scroll":hasFrozenColumns()?"hidden":hasFrozenRows?"scroll":"auto"}),$viewportTopR.css({"overflow-x":hasFrozenColumns()?hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"hidden":"scroll":hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"hidden":"auto","overflow-y":options.alwaysShowVerticalScroll?"scroll":(hasFrozenColumns(),hasFrozenRows?"scroll":"auto")}),$viewportBottomL.css({"overflow-x":hasFrozenColumns()?hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"scroll":"auto":(hasFrozenRows&&options.alwaysAllowHorizontalScroll,"auto"),"overflow-y":options.alwaysShowVerticalScroll?"scroll":hasFrozenColumns()?"hidden":hasFrozenRows?"scroll":"auto"}),$viewportBottomR.css({"overflow-x":hasFrozenColumns()?hasFrozenRows&&!options.alwaysAllowHorizontalScroll?"scroll":"auto":(hasFrozenRows&&options.alwaysAllowHorizontalScroll,"auto"),"overflow-y":options.alwaysShowVerticalScroll?"scroll":(hasFrozenColumns(),"auto")}),options.viewportClass&&($viewportTopL.toggleClass(options.viewportClass,!0),$viewportTopR.toggleClass(options.viewportClass,!0),$viewportBottomL.toggleClass(options.viewportClass,!0),$viewportBottomR.toggleClass(options.viewportClass,!0))}function setScroller(){hasFrozenColumns()?($headerScrollContainer=$headerScrollerR,$headerRowScrollContainer=$headerRowScrollerR,$footerRowScrollContainer=$footerRowScrollerR,hasFrozenRows?options.frozenBottom?($viewportScrollContainerX=$viewportBottomR,$viewportScrollContainerY=$viewportTopR):$viewportScrollContainerX=$viewportScrollContainerY=$viewportBottomR:$viewportScrollContainerX=$viewportScrollContainerY=$viewportTopR):($headerScrollContainer=$headerScrollerL,$headerRowScrollContainer=$headerRowScrollerL,$footerRowScrollContainer=$footerRowScrollerL,hasFrozenRows?options.frozenBottom?($viewportScrollContainerX=$viewportBottomL,$viewportScrollContainerY=$viewportTopL):$viewportScrollContainerX=$viewportScrollContainerY=$viewportBottomL:$viewportScrollContainerX=$viewportScrollContainerY=$viewportTopL)}function measureCellPaddingAndBorder(){var e,t=["borderLeftWidth","borderRightWidth","paddingLeft","paddingRight"],i=["borderTopWidth","borderBottomWidth","paddingTop","paddingBottom"],n=$.fn.jquery.split(".");jQueryNewWidthBehaviour=1==n[0]&&n[1]>=8||n[0]>=2,e=$("<div class='ui-state-default slick-header-column' style='visibility:hidden'>-</div>").appendTo($headers),headerColumnWidthDiff=headerColumnHeightDiff=0,"border-box"!=e.css("box-sizing")&&"border-box"!=e.css("-moz-box-sizing")&&"border-box"!=e.css("-webkit-box-sizing")&&($.each(t,(function(t,i){headerColumnWidthDiff+=parseFloat(e.css(i))||0})),$.each(i,(function(t,i){headerColumnHeightDiff+=parseFloat(e.css(i))||0}))),e.remove();var o=$("<div class='slick-row' />").appendTo($canvas);e=$("<div class='slick-cell' id='' style='visibility:hidden'>-</div>").appendTo(o),cellWidthDiff=cellHeightDiff=0,"border-box"!=e.css("box-sizing")&&"border-box"!=e.css("-moz-box-sizing")&&"border-box"!=e.css("-webkit-box-sizing")&&($.each(t,(function(t,i){cellWidthDiff+=parseFloat(e.css(i))||0})),$.each(i,(function(t,i){cellHeightDiff+=parseFloat(e.css(i))||0}))),o.remove(),absoluteColumnMinWidth=Math.max(headerColumnWidthDiff,cellWidthDiff)}function createCssRules(){$style=$("<style type='text/css' rel='stylesheet' />").appendTo($("head"));for(var e=options.rowHeight-cellHeightDiff,t=["."+uid+" .slick-group-header-column { left: 1000px; }","."+uid+" .slick-header-column { left: 1000px; }","."+uid+" .slick-top-panel { height:"+options.topPanelHeight+"px; }","."+uid+" .slick-preheader-panel { height:"+options.preHeaderPanelHeight+"px; }","."+uid+" .slick-headerrow-columns { height:"+options.headerRowHeight+"px; }","."+uid+" .slick-footerrow-columns { height:"+options.footerRowHeight+"px; }","."+uid+" .slick-cell { height:"+e+"px; }","."+uid+" .slick-row { height:"+options.rowHeight+"px; }"],i=0;i<columns.length;i++)t.push("."+uid+" .l"+i+" { }"),t.push("."+uid+" .r"+i+" { }");$style[0].styleSheet?$style[0].styleSheet.cssText=t.join(" "):$style[0].appendChild(document.createTextNode(t.join(" ")))}function getColumnCssRules(e){var t;if(!stylesheet){var i=document.styleSheets;for(t=0;t<i.length;t++)if((i[t].ownerNode||i[t].owningElement)==$style[0]){stylesheet=i[t];break}if(!stylesheet)throw new Error("Cannot find stylesheet.");columnCssRulesL=[],columnCssRulesR=[];var n,o,s=stylesheet.cssRules||stylesheet.rules;for(t=0;t<s.length;t++){var a=s[t].selectorText;(n=/\.l\d+/.exec(a))?(o=parseInt(n[0].substr(2,n[0].length-2),10),columnCssRulesL[o]=s[t]):(n=/\.r\d+/.exec(a))&&(o=parseInt(n[0].substr(2,n[0].length-2),10),columnCssRulesR[o]=s[t])}}return{left:columnCssRulesL[e],right:columnCssRulesR[e]}}function removeCssRules(){$style.remove(),stylesheet=null}function destroy(){getEditorLock().cancelCurrentEdit(),trigger(self.onBeforeDestroy,{});var e=plugins.length;while(e--)unregisterPlugin(plugins[e]);options.enableColumnReorder&&$headers.filter(":ui-sortable").sortable("destroy"),unbindAncestorScrollEvents(),$container.off(".slickgrid"),removeCssRules(),$canvas.off("draginit dragstart dragend drag"),$container.empty().removeClass(uid)}var canvas=null,canvas_context=null;function autosizeColumn(e,t){var n=e;if("number"===typeof e)n=columns[e];else if("string"===typeof e)for(i=0;i<columns.length;i++)columns[i].Id===e&&(n=columns[i]);var o=$(getCanvasNode(0,0));getColAutosizeWidth(n,o,t)}function autosizeColumns(e,t){if(e=e||options.autosizeColsMode,e!==Slick.GridAutosizeColsMode.LegacyForceFit&&e!==Slick.GridAutosizeColsMode.LegacyOff){if(e!==Slick.GridAutosizeColsMode.None){canvas=document.createElement("canvas"),canvas&&canvas.getContext&&(canvas_context=canvas.getContext("2d"));var i,n,o,s,a=$(getCanvasNode(0,0)),r=viewportHasVScroll?viewportW-scrollbarDimensions.width:viewportW,l=0,c=0,h=0,u=0,d=0;for(i=0;i<columns.length;i++)n=columns[i],getColAutosizeWidth(n,a,t),d+=n.autoSize.autosizeMode===Slick.ColAutosizeMode.Locked?n.width:0,u+=n.autoSize.autosizeMode===Slick.ColAutosizeMode.Locked?n.width:n.minWidth,l+=n.autoSize.widthPx,c+=n.autoSize.sizeToRemaining?0:n.autoSize.widthPx,h+=n.autoSize.sizeToRemaining&&n.minWidth||0;var p=l-c;if(e===Slick.GridAutosizeColsMode.FitViewportToCols){var f=l+scrollbarDimensions.width;e=Slick.GridAutosizeColsMode.IgnoreViewport,options.viewportMaxWidthPx&&f>options.viewportMaxWidthPx?(f=options.viewportMaxWidthPx,e=Slick.GridAutosizeColsMode.FitColsToViewport):options.viewportMinWidthPx&&f<options.viewportMinWidthPx&&(f=options.viewportMinWidthPx,e=Slick.GridAutosizeColsMode.FitColsToViewport),$container.width(f)}if(e===Slick.GridAutosizeColsMode.FitColsToViewport)if(p>0&&c<r-h)for(i=0;i<columns.length;i++){n=columns[i];var g=r-c;o=n.autoSize.sizeToRemaining?g*n.autoSize.widthPx/p:n.autoSize.widthPx,n.rerenderOnResize&&n.width!=o&&(s=!0),n.width=o}else if(options.viewportSwitchToScrollModeWidthPercent&&c+h>r*options.viewportSwitchToScrollModeWidthPercent/100||u>r)e=Slick.GridAutosizeColsMode.IgnoreViewport;else{var m=c-d,v=r-d-h;for(i=0;i<columns.length;i++)n=columns[i],o=n.width,n.autoSize.autosizeMode!==Slick.ColAutosizeMode.Locked&&(n.autoSize.sizeToRemaining?o=n.minWidth:(o=v/m*n.autoSize.widthPx,o<n.minWidth&&(o=n.minWidth),m-=n.autoSize.widthPx,v-=o)),n.rerenderOnResize&&n.width!=o&&(s=!0),n.width=o}if(e===Slick.GridAutosizeColsMode.IgnoreViewport)for(i=0;i<columns.length;i++)o=columns[i].autoSize.widthPx,columns[i].rerenderOnResize&&columns[i].width!=o&&(s=!0),columns[i].width=o;reRenderColumns(s)}}else legacyAutosizeColumns()}function LogColWidths(){var e="Col Widths:";for(i=0;i<columns.length;i++)e+=" "+columns[i].width;console.log(e)}function getColAutosizeWidth(e,t,i){var n=e.autoSize;if(n.widthPx=e.width,n.autosizeMode!==Slick.ColAutosizeMode.Locked&&n.autosizeMode!==Slick.ColAutosizeMode.Guide){var o=getDataLength();if(n.autosizeMode===Slick.ColAutosizeMode.ContentIntelligent){var s,a=n.colDataTypeOf;if(o>0){var r=getDataItem(0);r&&(s=r[e.field],a=typeof s,"object"===a&&(s instanceof Date&&(a="date"),"undefined"!==typeof moment&&s instanceof moment&&(a="moment")))}"boolean"===a&&(n.colValueArray=[!0,!1]),"number"===a&&(n.valueFilterMode=Slick.ValueFilterMode.GetGreatestAndSub,n.rowSelectionMode=Slick.RowSelectionMode.AllRows),"string"===a&&(n.valueFilterMode=Slick.ValueFilterMode.GetLongestText,n.rowSelectionMode=Slick.RowSelectionMode.AllRows,n.allowAddlPercent=5),"date"===a&&(n.colValueArray=[new Date(2009,8,30,12,20,20)]),"moment"===a&&"undefined"!==typeof moment&&(n.colValueArray=[moment([2009,8,30,12,20,20])])}var l=getColContentSize(e,t,i),c=n.allowAddlPercent?1+n.allowAddlPercent/100:1;l=l*c+options.autosizeColPaddingPx,e.minWidth&&l<e.minWidth&&(l=e.minWidth),e.maxWidth&&l>e.maxWidth&&(l=e.maxWidth),n.widthPx=l}}function getColContentSize(e,t,i){var n,o=e.autoSize,s=1,a=0,r=0;if(o.ignoreHeaderText||(r=getColHeaderWidth(e)),o.colValueArray)return a=getColWidth(e,t,o.colValueArray),Math.max(r,a);var l=getData();l.getItems&&(l=l.getItems());var c=(i?o.rowSelectionModeOnInit:void 0)||o.rowSelectionMode;if(c===Slick.RowSelectionMode.FirstRow&&(l=l.slice(0,1)),c===Slick.RowSelectionMode.LastRow&&(l=l.slice(l.length-1,l.length)),c===Slick.RowSelectionMode.FirstNRows&&(l=l.slice(0,o.rowSelectionCount)),o.valueFilterMode===Slick.ValueFilterMode.DeDuplicate){var h={};for(u=0,n=l.length;u<n;u++)h[l[u][e.field]]=!0;if(Object.keys)l=Object.keys(h);else for(var u in l=[],h)l.push(u)}if(o.valueFilterMode===Slick.ValueFilterMode.GetGreatestAndSub){var d,p=0;for(u=0,n=l.length;u<n;u++)g=l[u][e.field],Math.abs(g)>p&&(d=g,p=Math.abs(g));d=""+d,d=Array(d.length+1).join("9"),d=+d,l=[d]}if(o.valueFilterMode===Slick.ValueFilterMode.GetLongestTextAndSub){var f=0;for(u=0,n=l.length;u<n;u++)g=l[u][e.field],(g||"").length>f&&(f=g.length);g=Array(f+1).join("m"),s=options.autosizeTextAvgToMWidthRatio,l=[g]}if(o.valueFilterMode===Slick.ValueFilterMode.GetLongestText){f=0;var g,m=0;for(u=0,n=l.length;u<n;u++)g=l[u][e.field],(g||"").length>f&&(f=g.length,m=u);g=l[m][e.field],l=[g]}return a=getColWidth(e,t,l)*s,Math.max(r,a)}function getColWidth(e,t,i){var n=getColumnIndex(e.id),o=$('<div class="slick-row ui-widget-content"></div>'),s=$('<div class="slick-cell"></div>');s.css({position:"absolute",visibility:"hidden","text-overflow":"initial","white-space":"nowrap"}),o.append(s),t.append(o);var a,r,l,c,h,u=0;return canvas_context&&e.autoSize.widthEvalMode===Slick.WidthEvalMode.CanvasTextSize?(canvas_context.font=s.css("font-size")+" "+s.css("font-family"),$(i).each((function(t,i){h=Array.isArray(i)?i[e.field]:i,r=""+h,a=r?canvas_context.measureText(r).width:0,a>u&&(u=a,l=r)})),s.html(l),a=s.outerWidth(),o.remove(),a):($(i).each((function(t,i){h=Array.isArray(i)?i[e.field]:i,c=e.formatterOverride?e.formatterOverride(t,n,h,e,i):e.formatter?e.formatter(t,n,h,e,i):""+h,applyFormatResultToCellNode(c,s[0]),a=s.outerWidth(),a>u&&(u=a)})),o.remove(),u)}function getColHeaderWidth(e){var t=0,i=getUID()+e.id,n=document.getElementById(i),o=i+"_";if(n){var s=n.cloneNode(!0);s.id=o,s.style.cssText="position: absolute; visibility: hidden;right: auto;text-overflow: initial;white-space: nowrap;",n.parentNode.insertBefore(s,n),t=s.offsetWidth,s.parentNode.removeChild(s)}else{var a=getHeader(e);n=$("<div class='ui-state-default slick-header-column' />").html("<span class='slick-column-name'>"+e.name+"</span>").attr("id",o).css({position:"absolute",visibility:"hidden",right:"auto","text-overflow:":"initial","white-space":"nowrap"}).addClass(e.headerCssClass||"").appendTo(a),t=n[0].offsetWidth,a[0].removeChild(n[0])}return t}function legacyAutosizeColumns(){var e,t,i,n=[],o=0,s=0,a=viewportHasVScroll?viewportW-scrollbarDimensions.width:viewportW;for(e=0;e<columns.length;e++)t=columns[e],n.push(t.width),s+=t.width,t.resizable&&(o+=t.width-Math.max(t.minWidth,absoluteColumnMinWidth));i=s;while(s>a&&o){var r=(s-a)/o;for(e=0;e<columns.length&&s>a;e++){t=columns[e];var l=n[e];if(!(!t.resizable||l<=t.minWidth||l<=absoluteColumnMinWidth)){var c=Math.max(t.minWidth,absoluteColumnMinWidth),h=Math.floor(r*(l-c))||1;h=Math.min(h,l-c),s-=h,o-=h,n[e]-=h}}if(i<=s)break;i=s}i=s;while(s<a){var u=a/s;for(e=0;e<columns.length&&s<a;e++){t=columns[e];var d,p=n[e];d=!t.resizable||t.maxWidth<=p?0:Math.min(Math.floor(u*p)-p,t.maxWidth-p||1e6)||1,s+=d,n[e]+=s<=a?d:0}if(i>=s)break;i=s}var f=!1;for(e=0;e<columns.length;e++)columns[e].rerenderOnResize&&columns[e].width!=n[e]&&(f=!0),columns[e].width=n[e];reRenderColumns(f)}function reRenderColumns(e){applyColumnHeaderWidths(),applyColumnGroupHeaderWidths(),updateCanvasWidth(!0),trigger(self.onAutosizeColumns,{columns:columns}),e&&(invalidateAllRows(),render())}function trigger(e,t,i){return i=i||new Slick.EventData,t=t||{},t.grid=self,e.notify(t,i,self)}function getEditorLock(){return options.editorLock}function getEditController(){return editController}function getColumnIndex(e){return columnsById[e]}function applyColumnGroupHeaderWidths(){if(treeColumns.hasDepth())for(var e=$groupHeadersL.length-1;e>=0;e--){treeColumns.getColumnsInDepth(e);$().add($groupHeadersL[e]).add($groupHeadersR[e]).each((function(e){var t=$(this),i=0;t.width(0===e?getHeadersWidthL():getHeadersWidthR()),t.children().each((function(){var e=$(this),n=$(this).data("column");n.width=0,n.columns.forEach((function(){var e=t.next().children(":eq("+i+++")");n.width+=e.outerWidth()})),e.width(n.width-headerColumnWidthDiff)}))}))}}function applyColumnHeaderWidths(){if(initialized){for(var e,t=0,i=$headers.children(),n=columns.length;t<n;t++)e=$(i[t]),jQueryNewWidthBehaviour?e.outerWidth()!==columns[t].width&&e.outerWidth(columns[t].width):e.width()!==columns[t].width-headerColumnWidthDiff&&e.width(columns[t].width-headerColumnWidthDiff);updateColumnCaches()}}function applyColumnWidths(){for(var e,t,i=0,n=0;n<columns.length;n++)e=columns[n].width,t=getColumnCssRules(n),t.left.style.left=i+"px",t.right.style.right=(-1!=options.frozenColumn&&n>options.frozenColumn?canvasWidthR:canvasWidthL)-i-e+"px",options.frozenColumn==n?i=0:i+=columns[n].width}function setSortColumn(e,t){setSortColumns([{columnId:e,sortAsc:t}])}function setSortColumns(e){sortColumns=e;var t=options.numberedMultiColumnSort&&sortColumns.length>1,i=$headers.children();i.removeClass("slick-header-column-sorted").find(".slick-sort-indicator").removeClass("slick-sort-indicator-asc slick-sort-indicator-desc"),i.find(".slick-sort-indicator-numbered").text(""),$.each(sortColumns,(function(e,n){null==n.sortAsc&&(n.sortAsc=!0);var o=getColumnIndex(n.columnId);null!=o&&(i.eq(o).addClass("slick-header-column-sorted").find(".slick-sort-indicator").addClass(n.sortAsc?"slick-sort-indicator-asc":"slick-sort-indicator-desc"),t&&i.eq(o).find(".slick-sort-indicator-numbered").text(e+1))}))}function getSortColumns(){return sortColumns}function handleSelectedRangesChanged(e,t){var i=selectedRows.slice(0);selectedRows=[];for(var n={},o=0;o<t.length;o++)for(var s=t[o].fromRow;s<=t[o].toRow;s++){n[s]||(selectedRows.push(s),n[s]={});for(var a=t[o].fromCell;a<=t[o].toCell;a++)canCellBeSelected(s,a)&&(n[s][columns[a].id]=options.selectedCellCssClass)}setCellCssStyles(options.selectedCellCssClass,n),simpleArrayEquals(i,selectedRows)&&trigger(self.onSelectedRowsChanged,{rows:getSelectedRows(),previousSelectedRows:i},e)}function simpleArrayEquals(e,t){return Array.isArray(e)&&Array.isArray(t)&&t.sort().toString()!==e.sort().toString()}function getColumns(){return columns}function updateColumnCaches(){columnPosLeft=[],columnPosRight=[];for(var e=0,t=0,i=columns.length;t<i;t++)columnPosLeft[t]=e,columnPosRight[t]=e+columns[t].width,options.frozenColumn==t?e=0:e+=columns[t].width}function updateColumnProps(){columnsById={};for(var e=0;e<columns.length;e++){columns[e].width&&(columns[e].widthRequest=columns[e].width);var t=columns[e]=$.extend({},columnDefaults,columns[e]);t.autoSize=$.extend({},columnAutosizeDefaults,t.autoSize),columnsById[t.id]=e,t.minWidth&&t.width<t.minWidth&&(t.width=t.minWidth),t.maxWidth&&t.width>t.maxWidth&&(t.width=t.maxWidth),t.resizable}}function setColumns(e){var t=new Slick.TreeColumns(e);t.hasDepth()?(treeColumns=t,columns=treeColumns.extractColumns()):columns=e,updateColumnProps(),updateColumnCaches(),initialized&&(setPaneVisibility(),setOverflow(),invalidateAllRows(),createColumnHeaders(),createColumnGroupHeaders(),createColumnFooter(),removeCssRules(),createCssRules(),resizeCanvas(),updateCanvasWidth(),applyColumnHeaderWidths(),applyColumnWidths(),handleScroll())}function getOptions(){return options}function setOptions(e,t){getEditorLock().commitCurrentEdit()&&(makeActiveCellNormal(),void 0!==e.showColumnHeader&&setColumnHeaderVisibility(e.showColumnHeader),options.enableAddRow!==e.enableAddRow&&invalidateRow(getDataLength()),options=$.extend(options,e),validateAndEnforceOptions(),$viewport.css("overflow-y",options.autoHeight?"hidden":"auto"),t||render(),setFrozenOptions(),setScroller(),zombieRowNodeFromLastMouseWheelEvent=null,setColumns(treeColumns.extractColumns()))}function validateAndEnforceOptions(){options.autoHeight&&(options.leaveSpaceForNewRows=!1),options.forceFitColumns&&(options.autosizeColsMode=Slick.GridAutosizeColsMode.LegacyForceFit,console.log("forceFitColumns option is deprecated - use autosizeColsMode"))}function setData(e,t){data=e,invalidateAllRows(),updateRowCount(),t&&scrollTo(0)}function getData(){return data}function getDataLength(){return data.getLength?data.getLength():data&&data.length||0}function getDataLengthIncludingAddNew(){return getDataLength()+(options.enableAddRow&&(!pagingActive||pagingIsLastPage)?1:0)}function getDataItem(e){return data.getItem?data.getItem(e):data[e]}function getTopPanel(){return $topPanel[0]}function setTopPanelVisibility(e,t){var i=!1!==t;options.showTopPanel!=e&&(options.showTopPanel=e,e?i?$topPanelScroller.slideDown("fast",resizeCanvas):($topPanelScroller.show(),resizeCanvas()):i?$topPanelScroller.slideUp("fast",resizeCanvas):($topPanelScroller.hide(),resizeCanvas()))}function setHeaderRowVisibility(e,t){var i=!1!==t;options.showHeaderRow!=e&&(options.showHeaderRow=e,e?i?$headerRowScroller.slideDown("fast",resizeCanvas):($headerRowScroller.show(),resizeCanvas()):i?$headerRowScroller.slideUp("fast",resizeCanvas):($headerRowScroller.hide(),resizeCanvas()))}function setColumnHeaderVisibility(e,t){options.showColumnHeader!=e&&(options.showColumnHeader=e,e?t?$headerScroller.slideDown("fast",resizeCanvas):($headerScroller.show(),resizeCanvas()):t?$headerScroller.slideUp("fast",resizeCanvas):($headerScroller.hide(),resizeCanvas()))}function setFooterRowVisibility(e,t){var i=!1!==t;options.showFooterRow!=e&&(options.showFooterRow=e,e?i?$footerRowScroller.slideDown("fast",resizeCanvas):($footerRowScroller.show(),resizeCanvas()):i?$footerRowScroller.slideUp("fast",resizeCanvas):($footerRowScroller.hide(),resizeCanvas()))}function setPreHeaderPanelVisibility(e,t){var i=!1!==t;options.showPreHeaderPanel!=e&&(options.showPreHeaderPanel=e,e?i?$preHeaderPanelScroller.slideDown("fast",resizeCanvas):($preHeaderPanelScroller.show(),resizeCanvas()):i?$preHeaderPanelScroller.slideUp("fast",resizeCanvas):($preHeaderPanelScroller.hide(),resizeCanvas()))}function getContainerNode(){return $container.get(0)}function getRowTop(e){return options.rowHeight*e-offset}function getRowFromPosition(e){return Math.floor((e+offset)/options.rowHeight)}function scrollTo(e){e=Math.max(e,0),e=Math.min(e,th-$viewportScrollContainerY.height()+(viewportHasHScroll||hasFrozenColumns()?scrollbarDimensions.height:0));var t=offset;page=Math.min(n-1,Math.floor(e/ph)),offset=Math.round(page*cj);var i=e-offset;if(offset!=t){var o=getVisibleRange(i);cleanupRows(o),updateRowPositions()}prevScrollTop!=i&&(vScrollDir=prevScrollTop+t<i+offset?1:-1,lastRenderedScrollTop=scrollTop=prevScrollTop=i,hasFrozenColumns()&&($viewportTopL[0].scrollTop=i),hasFrozenRows&&($viewportBottomL[0].scrollTop=$viewportBottomR[0].scrollTop=i),$viewportScrollContainerY[0].scrollTop=i,trigger(self.onViewportChanged,{}))}function defaultFormatter(e,t,i,n,o,s){return null==i?"":(i+"").replace(/&/g,"&amp;").replace(/</g,"&lt;").replace(/>/g,"&gt;")}function getFormatter(e,t){var i=data.getItemMetadata&&data.getItemMetadata(e),n=i&&i.columns&&(i.columns[t.id]||i.columns[getColumnIndex(t.id)]);return n&&n.formatter||i&&i.formatter||t.formatter||options.formatterFactory&&options.formatterFactory.getFormatter(t)||options.defaultFormatter}function callFormatter(e,t,i,n,o,s){var a,r=data.getItemMetadata&&data.getItemMetadata(e);if(r=r&&r.columns,r){var l=r[n.id]||r[t];a=getFormatter(e,n)(e,t,i,n,o,l)}else a=getFormatter(e,n)(e,t,i,n,o);return a}function getEditor(e,t){var i=columns[t],n=data.getItemMetadata&&data.getItemMetadata(e),o=n&&n.columns;return o&&o[i.id]&&void 0!==o[i.id].editor?o[i.id].editor:o&&o[t]&&void 0!==o[t].editor?o[t].editor:i.editor||options.editorFactory&&options.editorFactory.getEditor(i)}function getDataItemValueForColumn(e,t){return options.dataItemColumnValueExtractor?options.dataItemColumnValueExtractor(e,t):e[t.field]}function appendRowHtml(e,t,i,n,o){var s=getDataItem(i),a=i<o&&!s,r="slick-row"+(hasFrozenRows&&i<=options.frozenRow?" frozen":"")+(a?" loading":"")+(i===activeRow&&options.showCellSelection?" active":"")+(i%2==1?" odd":" even");s||(r+=" "+options.addNewRowCssClass);var l=data.getItemMetadata&&data.getItemMetadata(i);l&&l.cssClasses&&(r+=" "+l.cssClasses);var c,h,u=getFrozenRowOffset(i),d="<div class='ui-widget-content "+r+"' style='top:"+(getRowTop(i)-u)+"px'>";e.push(d),hasFrozenColumns()&&t.push(d);for(var p=0,f=columns.length;p<f;p++){if(h=columns[p],c=1,l&&l.columns){var g=l.columns[h.id]||l.columns[p];c=g&&g.colspan||1,"*"===c&&(c=f-p)}if(columnPosRight[Math.min(f-1,p+c-1)]>n.leftPx){if(!h.alwaysRenderColumn&&columnPosLeft[p]>n.rightPx)break;hasFrozenColumns()&&p>options.frozenColumn?appendCellHtml(t,i,p,c,s):appendCellHtml(e,i,p,c,s)}else(h.alwaysRenderColumn||hasFrozenColumns()&&p<=options.frozenColumn)&&appendCellHtml(e,i,p,c,s);c>1&&(p+=c-1)}e.push("</div>"),hasFrozenColumns()&&t.push("</div>")}function appendCellHtml(e,t,i,n,o){var s=columns[i],a="slick-cell l"+i+" r"+Math.min(columns.length-1,i+n-1)+(s.cssClass?" "+s.cssClass:"");for(var r in hasFrozenColumns()&&i<=options.frozenColumn&&(a+=" frozen"),t===activeRow&&i===activeCell&&options.showCellSelection&&(a+=" active"),cellCssClasses)cellCssClasses[r][t]&&cellCssClasses[r][t][s.id]&&(a+=" "+cellCssClasses[r][t][s.id]);var l=null,c="";o&&(l=getDataItemValueForColumn(o,s),c=getFormatter(t,s)(t,i,l,s,o,self),null!==c&&void 0!==c||(c=""));var h=trigger(self.onBeforeAppendCell,{row:t,cell:i,value:l,dataContext:o})||"";h+=c&&c.addClasses?(h?" ":"")+c.addClasses:"";var u=c&&c.toolTip?"title='"+c.toolTip+"'":"",d="";if(s.hasOwnProperty("cellAttrs")&&s.cellAttrs instanceof Object)for(var r in s.cellAttrs)s.cellAttrs.hasOwnProperty(r)&&(d+=" "+r+'="'+s.cellAttrs[r]+'" ');e.push("<div class='"+a+(h?" "+h:"")+"' "+u+d+">"),o&&e.push("[object Object]"!==Object.prototype.toString.call(c)?c:c.text),e.push("</div>"),rowsCache[t].cellRenderQueue.push(i),rowsCache[t].cellColSpans[i]=n}function cleanupRows(e){for(var t in rowsCache){var i=!0;hasFrozenRows&&(options.frozenBottom&&t>=actualFrozenRow||!options.frozenBottom&&t<=actualFrozenRow)&&(i=!1),(t=parseInt(t,10))!==activeRow&&(t<e.top||t>e.bottom)&&i&&removeRowFromCache(t)}options.enableAsyncPostRenderCleanup&&startPostProcessingCleanup()}function invalidate(){updateRowCount(),invalidateAllRows(),render()}function invalidateAllRows(){for(var e in currentEditor&&makeActiveCellNormal(),rowsCache)removeRowFromCache(e);options.enableAsyncPostRenderCleanup&&startPostProcessingCleanup()}function queuePostProcessedRowForCleanup(e,t,i){for(var n in postProcessgroupId++,t)t.hasOwnProperty(n)&&postProcessedCleanupQueue.push({actionType:"C",groupId:postProcessgroupId,node:e.cellNodesByColumnIdx[0|n],columnIdx:0|n,rowIdx:i});postProcessedCleanupQueue.push({actionType:"R",groupId:postProcessgroupId,node:e.rowNode}),$(e.rowNode).detach()}function queuePostProcessedCellForCleanup(e,t,i){postProcessedCleanupQueue.push({actionType:"C",groupId:postProcessgroupId,node:e,columnIdx:t,rowIdx:i}),$(e).detach()}function removeRowFromCache(e){var t=rowsCache[e];t&&(rowNodeFromLastMouseWheelEvent==t.rowNode[0]||hasFrozenColumns()&&rowNodeFromLastMouseWheelEvent==t.rowNode[1]?(t.rowNode.hide(),zombieRowNodeFromLastMouseWheelEvent=t.rowNode):t.rowNode.each((function(){this.parentElement.removeChild(this)})),delete rowsCache[e],delete postProcessedRows[e],renderedRows--,counter_rows_removed++)}function invalidateRows(e){var t,i;if(e&&e.length){for(vScrollDir=0,i=e.length,t=0;t<i;t++)currentEditor&&activeRow===e[t]&&makeActiveCellNormal(),rowsCache[e[t]]&&removeRowFromCache(e[t]);options.enableAsyncPostRenderCleanup&&startPostProcessingCleanup()}}function invalidateRow(e){(e||0===e)&&invalidateRows([e])}function applyFormatResultToCellNode(e,t,i){null!==e&&void 0!==e||(e=""),"[object Object]"===Object.prototype.toString.call(e)?(t.innerHTML=e.text,e.removeClasses&&!i&&$(t).removeClass(e.removeClasses),e.addClasses&&$(t).addClass(e.addClasses),e.toolTip&&$(t).attr("title",e.toolTip)):t.innerHTML=e}function updateCell(e,t){var i=getCellNode(e,t);if(i){var n=columns[t],o=getDataItem(e);if(currentEditor&&activeRow===e&&activeCell===t)currentEditor.loadValue(o);else{var s=o?getFormatter(e,n)(e,t,getDataItemValueForColumn(o,n),n,o,self):"";applyFormatResultToCellNode(s,i),invalidatePostProcessingResults(e)}}}function updateRow(e){var t=rowsCache[e];if(t){ensureCellNodesInRowsCache(e);var i,n=getDataItem(e);for(var o in t.cellNodesByColumnIdx)if(t.cellNodesByColumnIdx.hasOwnProperty(o)){o|=0;var s=columns[o],a=t.cellNodesByColumnIdx[o][0];e===activeRow&&o===activeCell&&currentEditor?currentEditor.loadValue(n):n?(i=getFormatter(e,s)(e,o,getDataItemValueForColumn(n,s),s,n,self),applyFormatResultToCellNode(i,a)):a.innerHTML=""}invalidatePostProcessingResults(e)}}function getViewportHeight(){if(options.autoHeight){var e=$paneHeaderL.outerHeight();e+=options.showHeaderRow?options.headerRowHeight+getVBoxDelta($headerRowScroller):0,e+=options.showFooterRow?options.footerRowHeight+getVBoxDelta($footerRowScroller):0,e+=getCanvasWidth()>viewportW?scrollbarDimensions.height:0,viewportH=options.rowHeight*getDataLengthIncludingAddNew()+(-1==options.frozenColumn?e:0)}else{var t=options.showColumnHeader?parseFloat($.css($headerScroller[0],"height"))+getVBoxDelta($headerScroller):0;topPanelH=options.showTopPanel?options.topPanelHeight+getVBoxDelta($topPanelScroller):0,headerRowH=options.showHeaderRow?options.headerRowHeight+getVBoxDelta($headerRowScroller):0,footerRowH=options.showFooterRow?options.footerRowHeight+getVBoxDelta($footerRowScroller):0;var i=options.createPreHeaderPanel&&options.showPreHeaderPanel?options.preHeaderPanelHeight+getVBoxDelta($preHeaderPanelScroller):0;viewportH=parseFloat($.css($container[0],"height",!0))-parseFloat($.css($container[0],"paddingTop",!0))-parseFloat($.css($container[0],"paddingBottom",!0))-t-topPanelH-headerRowH-footerRowH-i}return numVisibleRows=Math.ceil(viewportH/options.rowHeight),viewportH}function getViewportWidth(){viewportW=parseFloat($container.width())}function resizeCanvas(){if(initialized){paneTopH=0,paneBottomH=0,viewportTopH=0,viewportBottomH=0,getViewportWidth(),getViewportHeight(),hasFrozenRows?options.frozenBottom?(paneTopH=viewportH-frozenRowsHeight-scrollbarDimensions.height,paneBottomH=frozenRowsHeight+scrollbarDimensions.height):(paneTopH=frozenRowsHeight,paneBottomH=viewportH-frozenRowsHeight):paneTopH=viewportH,paneTopH+=topPanelH+headerRowH+footerRowH,hasFrozenColumns()&&options.autoHeight&&(paneTopH+=scrollbarDimensions.height),viewportTopH=paneTopH-topPanelH-headerRowH-footerRowH,options.autoHeight&&(hasFrozenColumns()&&$container.height(paneTopH+parseFloat($.css($headerScrollerL[0],"height"))),$paneTopL.css("position","relative")),$paneTopL.css({top:$paneHeaderL.height(),height:paneTopH});var e=$paneTopL.position().top+paneTopH;options.autoHeight||$viewportTopL.height(viewportTopH),hasFrozenColumns()?($paneTopR.css({top:$paneHeaderL.height(),height:paneTopH}),$viewportTopR.height(viewportTopH),hasFrozenRows&&($paneBottomL.css({top:e,height:paneBottomH}),$paneBottomR.css({top:e,height:paneBottomH}),$viewportBottomR.height(paneBottomH))):hasFrozenRows&&($paneBottomL.css({width:"100%",height:paneBottomH}),$paneBottomL.css("top",e)),hasFrozenRows?($viewportBottomL.height(paneBottomH),options.frozenBottom?($canvasBottomL.height(frozenRowsHeight),hasFrozenColumns()&&$canvasBottomR.height(frozenRowsHeight)):($canvasTopL.height(frozenRowsHeight),hasFrozenColumns()&&$canvasTopR.height(frozenRowsHeight))):$viewportTopR.height(viewportTopH),scrollbarDimensions&&scrollbarDimensions.width||(scrollbarDimensions=measureScrollbar()),options.autosizeColsMode===Slick.GridAutosizeColsMode.LegacyForceFit&&autosizeColumns(),updateRowCount(),handleScroll(),lastRenderedScrollLeft=-1,render()}}function updatePagingStatusFromView(e){pagingActive=0!==e.pageSize,pagingIsLastPage=e.pageNum==e.totalPages-1}function updateRowCount(){if(initialized){var e=getDataLength(),t=getDataLengthIncludingAddNew(),i=0,o=hasFrozenRows&&!options.frozenBottom?$canvasBottomL.height():$canvasTopL.height();if(hasFrozenRows)i=getDataLength()-options.frozenRow;else i=t+(options.leaveSpaceForNewRows?numVisibleRows-1:0);var s=$viewportScrollContainerY.height(),a=viewportHasVScroll;viewportHasVScroll=options.alwaysShowVerticalScroll||!options.autoHeight&&i*options.rowHeight>s,makeActiveCellNormal();var r=e-1;for(var l in rowsCache)l>r&&removeRowFromCache(l);options.enableAsyncPostRenderCleanup&&startPostProcessingCleanup(),activeCellNode&&activeRow>r&&resetActiveCell();o=h;options.autoHeight?h=options.rowHeight*i:(th=Math.max(options.rowHeight*i,s-scrollbarDimensions.height),th<maxSupportedCssHeight?(h=ph=th,n=1,cj=0):(h=maxSupportedCssHeight,ph=h/100,n=Math.floor(th/ph),cj=(th-h)/(n-1))),h!==o&&(hasFrozenRows&&!options.frozenBottom?($canvasBottomL.css("height",h),hasFrozenColumns()&&$canvasBottomR.css("height",h)):($canvasTopL.css("height",h),$canvasTopR.css("height",h)),scrollTop=$viewportScrollContainerY[0].scrollTop);var c=scrollTop+offset<=th-s;0==th||0==scrollTop?page=offset=0:scrollTo(c?scrollTop+offset:th-s),h!=o&&options.autoHeight&&resizeCanvas(),options.autosizeColsMode===Slick.GridAutosizeColsMode.LegacyForceFit&&a!=viewportHasVScroll&&autosizeColumns(),updateCanvasWidth(!1)}}function getVisibleRange(e,t){return null==e&&(e=scrollTop),null==t&&(t=scrollLeft),{top:getRowFromPosition(e),bottom:getRowFromPosition(e+viewportH)+1,leftPx:t,rightPx:t+viewportW}}function getRenderedRange(e,t){var i=getVisibleRange(e,t),n=Math.round(viewportH/options.rowHeight),o=options.minRowBuffer;return-1==vScrollDir?(i.top-=n,i.bottom+=o):1==vScrollDir?(i.top-=o,i.bottom+=n):(i.top-=o,i.bottom+=o),i.top=Math.max(0,i.top),i.bottom=Math.min(getDataLengthIncludingAddNew()-1,i.bottom),i.leftPx-=viewportW,i.rightPx+=viewportW,i.leftPx=Math.max(0,i.leftPx),i.rightPx=Math.min(canvasWidth,i.rightPx),i}function ensureCellNodesInRowsCache(e){var t=rowsCache[e];if(t&&t.cellRenderQueue.length){var i=t.rowNode.children().last();while(t.cellRenderQueue.length){var n=t.cellRenderQueue.pop();t.cellNodesByColumnIdx[n]=i,i=i.prev(),0===i.length&&(i=$(t.rowNode[0]).children().last())}}}function cleanUpCells(e,t){if(!hasFrozenRows||!(options.frozenBottom&&t>actualFrozenRow||t<=actualFrozenRow)){var i,n=rowsCache[t],o=[];for(var s in n.cellNodesByColumnIdx)if(n.cellNodesByColumnIdx.hasOwnProperty(s)&&(s|=0,!(s<=options.frozenColumn)&&!(Array.isArray(columns)&&columns[s]&&columns[s].alwaysRenderColumn))){var a=n.cellColSpans[s];(columnPosLeft[s]>e.rightPx||columnPosRight[Math.min(columns.length-1,s+a-1)]<e.leftPx)&&(t==activeRow&&s==activeCell||o.push(s))}while(null!=(i=o.pop()))n.cellNodesByColumnIdx[i][0].parentElement.removeChild(n.cellNodesByColumnIdx[i][0]),delete n.cellColSpans[i],delete n.cellNodesByColumnIdx[i],postProcessedRows[t]&&delete postProcessedRows[t][i]}}function cleanUpAndRenderCells(e){for(var t,i,n,o=[],s=[],a=e.top,r=e.bottom;a<=r;a++)if(t=rowsCache[a],t){ensureCellNodesInRowsCache(a),cleanUpCells(e,a),i=0;var l=data.getItemMetadata&&data.getItemMetadata(a);l=l&&l.columns;for(var c=getDataItem(a),h=0,u=columns.length;h<u;h++){if(columnPosLeft[h]>e.rightPx)break;if(null==(n=t.cellColSpans[h])){if(n=1,l){var d=l[columns[h].id]||l[h];n=d&&d.colspan||1,"*"===n&&(n=u-h)}columnPosRight[Math.min(u-1,h+n-1)]>e.leftPx&&(appendCellHtml(o,a,h,n,c),i++),h+=n>1?n-1:0}else h+=n>1?n-1:0}i&&(i,s.push(a))}if(o.length){var p,f,g=document.createElement("div");g.innerHTML=o.join("");while(null!=(p=s.pop())){var m;t=rowsCache[p];while(null!=(m=t.cellRenderQueue.pop()))f=g.lastChild,hasFrozenColumns()&&m>options.frozenColumn?t.rowNode[1].appendChild(f):t.rowNode[0].appendChild(f),t.cellNodesByColumnIdx[m]=$(f)}}}function renderRows(e){for(var t=[],i=[],n=[],o=!1,s=getDataLength(),a=e.top,r=e.bottom;a<=r;a++)rowsCache[a]||hasFrozenRows&&options.frozenBottom&&a==getDataLength()||(renderedRows++,n.push(a),rowsCache[a]={rowNode:null,cellColSpans:[],cellNodesByColumnIdx:[],cellRenderQueue:[]},appendRowHtml(t,i,a,e,s),activeCellNode&&activeRow===a&&(o=!0),counter_rows_rendered++);if(n.length){var l=document.createElement("div"),c=document.createElement("div");l.innerHTML=t.join(""),c.innerHTML=i.join("");for(a=0,r=n.length;a<r;a++)hasFrozenRows&&n[a]>=actualFrozenRow?hasFrozenColumns()?rowsCache[n[a]].rowNode=$().add($(l.firstChild).appendTo($canvasBottomL)).add($(c.firstChild).appendTo($canvasBottomR)):rowsCache[n[a]].rowNode=$().add($(l.firstChild).appendTo($canvasBottomL)):hasFrozenColumns()?rowsCache[n[a]].rowNode=$().add($(l.firstChild).appendTo($canvasTopL)).add($(c.firstChild).appendTo($canvasTopR)):rowsCache[n[a]].rowNode=$().add($(l.firstChild).appendTo($canvasTopL));o&&(activeCellNode=getCellNode(activeRow,activeCell))}}function startPostProcessing(){options.enableAsyncPostRender&&(clearTimeout(h_postrender),h_postrender=setTimeout(asyncPostProcessRows,options.asyncPostRenderDelay))}function startPostProcessingCleanup(){options.enableAsyncPostRenderCleanup&&(clearTimeout(h_postrenderCleanup),h_postrenderCleanup=setTimeout(asyncPostProcessCleanupRows,options.asyncPostRenderCleanupDelay))}function invalidatePostProcessingResults(e){for(var t in postProcessedRows[e])postProcessedRows[e].hasOwnProperty(t)&&(postProcessedRows[e][t]="C");postProcessFromRow=Math.min(postProcessFromRow,e),postProcessToRow=Math.max(postProcessToRow,e),startPostProcessing()}function updateRowPositions(){for(var e in rowsCache){var t=e?parseInt(e):0;rowsCache[t].rowNode[0].style.top=getRowTop(t)+"px"}}function render(){if(initialized){scrollThrottle.dequeue();var e=getVisibleRange(),t=getRenderedRange();if(cleanupRows(t),lastRenderedScrollLeft!=scrollLeft){if(hasFrozenRows){var i=jQuery.extend(!0,{},t);options.frozenBottom?(i.top=actualFrozenRow,i.bottom=getDataLength()):(i.top=0,i.bottom=options.frozenRow),cleanUpAndRenderCells(i)}cleanUpAndRenderCells(t)}renderRows(t),hasFrozenRows&&(options.frozenBottom?renderRows({top:actualFrozenRow,bottom:getDataLength()-1,leftPx:t.leftPx,rightPx:t.rightPx}):renderRows({top:0,bottom:options.frozenRow-1,leftPx:t.leftPx,rightPx:t.rightPx})),postProcessFromRow=e.top,postProcessToRow=Math.min(getDataLengthIncludingAddNew()-1,e.bottom),startPostProcessing(),lastRenderedScrollTop=scrollTop,lastRenderedScrollLeft=scrollLeft,h_render=null,trigger(self.onRendered,{startRow:e.top,endRow:e.bottom,grid:self})}}function handleHeaderScroll(){handleElementScroll($headerScrollContainer[0])}function handleHeaderRowScroll(){var e=$headerRowScrollContainer[0].scrollLeft;e!=$viewportScrollContainerX[0].scrollLeft&&($viewportScrollContainerX[0].scrollLeft=e)}function handleFooterRowScroll(){var e=$footerRowScrollContainer[0].scrollLeft;e!=$viewportScrollContainerX[0].scrollLeft&&($viewportScrollContainerX[0].scrollLeft=e)}function handlePreHeaderPanelScroll(){handleElementScroll($preHeaderPanelScroller[0])}function handleElementScroll(e){var t=e.scrollLeft;t!=$viewportScrollContainerX[0].scrollLeft&&($viewportScrollContainerX[0].scrollLeft=t)}function handleScroll(){return scrollTop=$viewportScrollContainerY[0].scrollTop,scrollLeft=$viewportScrollContainerX[0].scrollLeft,_handleScroll(!1)}function _handleScroll(e){var t=$viewportScrollContainerY[0].scrollHeight-$viewportScrollContainerY[0].clientHeight,i=$viewportScrollContainerY[0].scrollWidth-$viewportScrollContainerY[0].clientWidth;t=Math.max(0,t),i=Math.max(0,i),scrollTop>t&&(scrollTop=t),scrollLeft>i&&(scrollLeft=i);var o=Math.abs(scrollTop-prevScrollTop),s=Math.abs(scrollLeft-prevScrollLeft);if(s&&(prevScrollLeft=scrollLeft,$viewportScrollContainerX[0].scrollLeft=scrollLeft,$headerScrollContainer[0].scrollLeft=scrollLeft,$topPanelScroller[0].scrollLeft=scrollLeft,$headerRowScrollContainer[0].scrollLeft=scrollLeft,options.createFooterRow&&($footerRowScrollContainer[0].scrollLeft=scrollLeft),options.createPreHeaderPanel&&(hasFrozenColumns()?$preHeaderPanelScrollerR[0].scrollLeft=scrollLeft:$preHeaderPanelScroller[0].scrollLeft=scrollLeft),hasFrozenColumns()?hasFrozenRows&&($viewportTopR[0].scrollLeft=scrollLeft):hasFrozenRows&&($viewportTopL[0].scrollLeft=scrollLeft)),o)if(vScrollDir=prevScrollTop<scrollTop?1:-1,prevScrollTop=scrollTop,e&&($viewportScrollContainerY[0].scrollTop=scrollTop),hasFrozenColumns()&&(hasFrozenRows&&!options.frozenBottom?$viewportBottomL[0].scrollTop=scrollTop:$viewportTopL[0].scrollTop=scrollTop),o<viewportH)scrollTo(scrollTop+offset);else{var a=offset;page=h==viewportH?0:Math.min(n-1,Math.floor(scrollTop*((th-viewportH)/(h-viewportH))*(1/ph))),offset=Math.round(page*cj),a!=offset&&invalidateAllRows()}if(s||o){var r=Math.abs(lastRenderedScrollLeft-scrollLeft),l=Math.abs(lastRenderedScrollTop-scrollTop);(r>20||l>20)&&(options.forceSyncScrolling||l<viewportH&&r<viewportW?render():scrollThrottle.enqueue(),trigger(self.onViewportChanged,{}))}return trigger(self.onScroll,{scrollLeft:scrollLeft,scrollTop:scrollTop}),!(!s&&!o)}function ActionThrottle(e,t){var i=!1,n=!1;function o(){i?n=!0:a()}function s(){n=!1}function a(){i=!0,setTimeout(r,t),e()}function r(){n?(s(),a()):i=!1}return{enqueue:o,dequeue:s}}function asyncPostProcessRows(){var e=getDataLength();while(postProcessFromRow<=postProcessToRow){var t=vScrollDir>=0?postProcessFromRow++:postProcessToRow--,i=rowsCache[t];if(i&&!(t>=e)){for(var n in postProcessedRows[t]||(postProcessedRows[t]={}),ensureCellNodesInRowsCache(t),i.cellNodesByColumnIdx)if(i.cellNodesByColumnIdx.hasOwnProperty(n)){n|=0;var o=columns[n],s=postProcessedRows[t][n];if(o.asyncPostRender&&"R"!==s){var a=i.cellNodesByColumnIdx[n];a&&o.asyncPostRender(a,t,getDataItem(t),o,"C"===s),postProcessedRows[t][n]="R"}}return void(h_postrender=setTimeout(asyncPostProcessRows,options.asyncPostRenderDelay))}}}function asyncPostProcessCleanupRows(){if(postProcessedCleanupQueue.length>0){var e=postProcessedCleanupQueue[0].groupId;while(postProcessedCleanupQueue.length>0&&postProcessedCleanupQueue[0].groupId==e){var t=postProcessedCleanupQueue.shift();if("R"==t.actionType&&$(t.node).remove(),"C"==t.actionType){var i=columns[t.columnIdx];i.asyncPostRenderCleanup&&t.node&&i.asyncPostRenderCleanup(t.node,t.rowIdx,i)}}h_postrenderCleanup=setTimeout(asyncPostProcessCleanupRows,options.asyncPostRenderCleanupDelay)}}function updateCellCssStylesOnRenderedRows(e,t){var i,n,o,s;for(var a in rowsCache){if(s=t&&t[a],o=e&&e[a],s)for(n in s)o&&s[n]==o[n]||(i=getCellNode(a,getColumnIndex(n)),i&&$(i).removeClass(s[n]));if(o)for(n in o)s&&s[n]==o[n]||(i=getCellNode(a,getColumnIndex(n)),i&&$(i).addClass(o[n]))}}function addCellCssStyles(e,t){if(cellCssClasses[e])throw new Error("addCellCssStyles: cell CSS hash with key '"+e+"' already exists.");cellCssClasses[e]=t,updateCellCssStylesOnRenderedRows(t,null),trigger(self.onCellCssStylesChanged,{key:e,hash:t,grid:self})}function removeCellCssStyles(e){cellCssClasses[e]&&(updateCellCssStylesOnRenderedRows(null,cellCssClasses[e]),delete cellCssClasses[e],trigger(self.onCellCssStylesChanged,{key:e,hash:null,grid:self}))}function setCellCssStyles(e,t){var i=cellCssClasses[e];cellCssClasses[e]=t,updateCellCssStylesOnRenderedRows(t,i),trigger(self.onCellCssStylesChanged,{key:e,hash:t,grid:self})}function getCellCssStyles(e){return cellCssClasses[e]}function flashCell(e,t,i){function n(e,t){t&&setTimeout((function(){e.queue((function(){e.toggleClass(options.cellFlashingCssClass).dequeue(),n(e,t-1)}))}),i)}if(i=i||100,rowsCache[e]){var o=$(getCellNode(e,t));n(o,4)}}function handleMouseWheel(e,t,i,n){var o=$(e.target).closest(".slick-row"),s=o[0];if(s!=rowNodeFromLastMouseWheelEvent){var a=o.parents(".grid-canvas"),r=a.hasClass("grid-canvas-left");if(zombieRowNodeFromLastMouseWheelEvent&&zombieRowNodeFromLastMouseWheelEvent[r?0:1]!=s){var l=zombieRowNodeFromLastMouseWheelEvent[r||1==zombieRowNodeFromLastMouseWheelEvent.length?0:1];l.parentElement.removeChild(l),zombieRowNodeFromLastMouseWheelEvent=null}rowNodeFromLastMouseWheelEvent=s}scrollTop=Math.max(0,$viewportScrollContainerY[0].scrollTop-n*options.rowHeight),scrollLeft=$viewportScrollContainerX[0].scrollLeft+10*i;var c=_handleScroll(!0);c&&e.preventDefault()}function handleDragInit(e,t){var i=getCellFromEvent(e);if(!i||!cellExists(i.row,i.cell))return!1;var n=trigger(self.onDragInit,t,e);return!!e.isImmediatePropagationStopped()&&n}function handleDragStart(e,t){var i=getCellFromEvent(e);if(!i||!cellExists(i.row,i.cell))return!1;var n=trigger(self.onDragStart,t,e);return!!e.isImmediatePropagationStopped()&&n}function handleDrag(e,t){return trigger(self.onDrag,t,e)}function handleDragEnd(e,t){trigger(self.onDragEnd,t,e)}function handleKeyDown(e){trigger(self.onKeyDown,{row:activeRow,cell:activeCell},e);var t=e.isImmediatePropagationStopped(),i=Slick.keyCode;if(!t&&!e.shiftKey&&!e.altKey){if(options.editable&&currentEditor&&currentEditor.keyCaptureList&&currentEditor.keyCaptureList.indexOf(e.which)>-1)return;e.which==i.HOME?t=e.ctrlKey?navigateTop():navigateRowStart():e.which==i.END&&(t=e.ctrlKey?navigateBottom():navigateRowEnd())}if(!t)if(e.shiftKey||e.altKey||e.ctrlKey)e.which!=i.TAB||!e.shiftKey||e.ctrlKey||e.altKey||(t=navigatePrev());else{if(options.editable&&currentEditor&&currentEditor.keyCaptureList&&currentEditor.keyCaptureList.indexOf(e.which)>-1)return;if(e.which==i.ESCAPE){if(!getEditorLock().isActive())return;cancelEditAndSetFocus()}else e.which==i.PAGE_DOWN?(navigatePageDown(),t=!0):e.which==i.PAGE_UP?(navigatePageUp(),t=!0):e.which==i.LEFT?t=navigateLeft():e.which==i.RIGHT?t=navigateRight():e.which==i.UP?t=navigateUp():e.which==i.DOWN?t=navigateDown():e.which==i.TAB?t=navigateNext():e.which==i.ENTER&&(options.editable&&(currentEditor?activeRow===getDataLength()?navigateDown():commitEditAndSetFocus():getEditorLock().commitCurrentEdit()&&makeActiveCellEditable(void 0,void 0,e)),t=!0)}if(t){e.stopPropagation(),e.preventDefault();try{e.originalEvent.keyCode=0}catch(n){}}}function handleClick(e){currentEditor||(e.target!=document.activeElement||$(e.target).hasClass("slick-cell"))&&setFocus();var t=getCellFromEvent(e);if(t&&(null===currentEditor||activeRow!=t.row||activeCell!=t.cell)&&(trigger(self.onClick,{row:t.row,cell:t.cell},e),!e.isImmediatePropagationStopped()&&canCellBeActive(t.row,t.cell)&&(!getEditorLock().isActive()||getEditorLock().commitCurrentEdit()))){scrollRowIntoView(t.row,!1);var i=e.target&&e.target.className===Slick.preClickClassName,n=columns[t.cell],o=!!(options.editable&&n&&n.editor&&options.suppressActiveCellChangeOnEdit);setActiveCellInternal(getCellNode(t.row,t.cell),null,i,o,e)}}function handleContextMenu(e){var t=$(e.target).closest(".slick-cell",$canvas);0!==t.length&&(activeCellNode===t[0]&&null!==currentEditor||trigger(self.onContextMenu,{},e))}function handleDblClick(e){var t=getCellFromEvent(e);!t||null!==currentEditor&&activeRow==t.row&&activeCell==t.cell||(trigger(self.onDblClick,{row:t.row,cell:t.cell},e),e.isImmediatePropagationStopped()||options.editable&&gotoCell(t.row,t.cell,!0,e))}function handleHeaderMouseEnter(e){trigger(self.onHeaderMouseEnter,{column:$(this).data("column"),grid:self},e)}function handleHeaderMouseLeave(e){trigger(self.onHeaderMouseLeave,{column:$(this).data("column"),grid:self},e)}function handleHeaderContextMenu(e){var t=$(e.target).closest(".slick-header-column",".slick-header-columns"),i=t&&t.data("column");trigger(self.onHeaderContextMenu,{column:i},e)}function handleHeaderClick(e){if(!columnResizeDragging){var t=$(e.target).closest(".slick-header-column",".slick-header-columns"),i=t&&t.data("column");i&&trigger(self.onHeaderClick,{column:i},e)}}function handleFooterContextMenu(e){var t=$(e.target).closest(".slick-footerrow-column",".slick-footerrow-columns"),i=t&&t.data("column");trigger(self.onFooterContextMenu,{column:i},e)}function handleFooterClick(e){var t=$(e.target).closest(".slick-footerrow-column",".slick-footerrow-columns"),i=t&&t.data("column");trigger(self.onFooterClick,{column:i},e)}function handleMouseEnter(e){trigger(self.onMouseEnter,{},e)}function handleMouseLeave(e){trigger(self.onMouseLeave,{},e)}function cellExists(e,t){return!(e<0||e>=getDataLength()||t<0||t>=columns.length)}function getCellFromPoint(e,t){for(var i=getRowFromPosition(t),n=0,o=0,s=0;s<columns.length&&o<e;s++)o+=columns[s].width,n++;return n<0&&(n=0),{row:i,cell:n-1}}function getCellFromNode(e){var t=/l\d+/.exec(e.className);if(!t)throw new Error("getCellFromNode: cannot get cell - "+e.className);return parseInt(t[0].substr(1,t[0].length-1),10)}function getRowFromNode(e){for(var t in rowsCache)for(var i in rowsCache[t].rowNode)if(rowsCache[t].rowNode[i]===e)return t?parseInt(t):0;return null}function getFrozenRowOffset(e){var t=hasFrozenRows?options.frozenBottom?e>=actualFrozenRow?h<viewportTopH?actualFrozenRow*options.rowHeight:h:0:e>=actualFrozenRow?frozenRowsHeight:0:0;return t}function getCellFromEvent(e){var t,i,n=$(e.target).closest(".slick-cell",$canvas);if(!n.length)return null;if(t=getRowFromNode(n[0].parentNode),hasFrozenRows){var o=n.parents(".grid-canvas").offset(),s=0,a=n.parents(".grid-canvas-bottom").length;a&&(s=options.frozenBottom?$canvasTopL.height():frozenRowsHeight),t=getCellFromPoint(e.clientX-o.left,e.clientY-o.top+s+$(document).scrollTop()).row}return i=getCellFromNode(n[0]),null==t||null==i?null:{row:t,cell:i}}function getCellNodeBox(e,t){if(!cellExists(e,t))return null;for(var i=getFrozenRowOffset(e),n=getRowTop(e)-i,o=n+options.rowHeight-1,s=0,a=0;a<t;a++)s+=columns[a].width,options.frozenColumn==a&&(s=0);var r=s+columns[t].width;return{top:n,left:s,bottom:o,right:r}}function resetActiveCell(){setActiveCellInternal(null,!1)}function setFocus(){-1==tabbingDirection?$focusSink[0].focus():$focusSink2[0].focus()}function scrollCellIntoView(e,t,i){if(scrollRowIntoView(e,i),!(t<=options.frozenColumn)){var n=getColspan(e,t);internalScrollColumnIntoView(columnPosLeft[t],columnPosRight[t+(n>1?n-1:0)])}}function internalScrollColumnIntoView(e,t){var i=scrollLeft+$viewportScrollContainerX.width();e<scrollLeft?($viewportScrollContainerX.scrollLeft(e),handleScroll(),render()):t>i&&($viewportScrollContainerX.scrollLeft(Math.min(e,t-$viewportScrollContainerX[0].clientWidth)),handleScroll(),render())}function scrollColumnIntoView(e){internalScrollColumnIntoView(columnPosLeft[e],columnPosRight[e])}function setActiveCellInternal(e,t,i,n,o){null!==activeCellNode&&(makeActiveCellNormal(),$(activeCellNode).removeClass("active"),rowsCache[activeRow]&&$(rowsCache[activeRow].rowNode).removeClass("active"));if(activeCellNode=e,null!=activeCellNode){var s=$(activeCellNode),a=s.offset(),r=Math.floor(s.parents(".grid-canvas").offset().top),l=s.parents(".grid-canvas-bottom").length;hasFrozenRows&&l&&(r-=options.frozenBottom?$canvasTopL.height():frozenRowsHeight);var c=getCellFromPoint(a.left,Math.ceil(a.top)-r);activeRow=c.row,activeCell=activePosX=activeCell=activePosX=getCellFromNode(activeCellNode),null==t&&(t=activeRow==getDataLength()||options.autoEdit),options.showCellSelection&&(s.addClass("active"),rowsCache[activeRow]&&$(rowsCache[activeRow].rowNode).addClass("active")),options.editable&&t&&isCellPotentiallyEditable(activeRow,activeCell)&&(clearTimeout(h_editorLoader),options.asyncEditorLoading?h_editorLoader=setTimeout((function(){makeActiveCellEditable(void 0,i,o)}),options.asyncEditorLoadDelay):makeActiveCellEditable(void 0,i,o))}else activeRow=activeCell=null;n||trigger(self.onActiveCellChanged,getActiveCell())}function clearTextSelection(){if(document.selection&&document.selection.empty)try{document.selection.empty()}catch(t){}else if(window.getSelection){var e=window.getSelection();e&&e.removeAllRanges&&e.removeAllRanges()}}function isCellPotentiallyEditable(e,t){var i=getDataLength();return!(e<i&&!getDataItem(e))&&(!(columns[t].cannotTriggerInsert&&e>=i)&&!!getEditor(e,t))}function makeActiveCellNormal(){if(currentEditor){if(trigger(self.onBeforeCellEditorDestroy,{editor:currentEditor}),currentEditor.destroy(),currentEditor=null,activeCellNode){var e=getDataItem(activeRow);if($(activeCellNode).removeClass("editable invalid"),e){var t=columns[activeCell],i=getFormatter(activeRow,t),n=i(activeRow,activeCell,getDataItemValueForColumn(e,t),t,e,self);applyFormatResultToCellNode(n,activeCellNode),invalidatePostProcessingResults(activeRow)}}navigator.userAgent.toLowerCase().match(/msie/)&&clearTextSelection(),getEditorLock().deactivate(editController)}}function makeActiveCellEditable(e,t,i){if(activeCellNode){if(!options.editable)throw new Error("Grid : makeActiveCellEditable : should never get called when options.editable is false");if(clearTimeout(h_editorLoader),isCellPotentiallyEditable(activeRow,activeCell)){var n=columns[activeCell],o=getDataItem(activeRow);if(!1!==trigger(self.onBeforeEditCell,{row:activeRow,cell:activeCell,item:o,column:n})){getEditorLock().activate(editController),$(activeCellNode).addClass("editable");var s=e||getEditor(activeRow,activeCell);e||s.suppressClearOnEdit||(activeCellNode.innerHTML="");var a=data.getItemMetadata&&data.getItemMetadata(activeRow);a=a&&a.columns;var r=a&&(a[n.id]||a[activeCell]);currentEditor=new s({grid:self,gridPosition:absBox($container[0]),position:absBox(activeCellNode),container:activeCellNode,column:n,columnMetaData:r,item:o||{},event:i,commitChanges:commitEditAndSetFocus,cancelChanges:cancelEditAndSetFocus}),o&&(currentEditor.loadValue(o),t&&currentEditor.preClick&&currentEditor.preClick()),serializedEditorValue=currentEditor.serializeValue(),currentEditor.position&&handleActiveCellPositionChange()}else setFocus()}}}function commitEditAndSetFocus(){getEditorLock().commitCurrentEdit()&&(setFocus(),options.autoEdit&&navigateDown())}function cancelEditAndSetFocus(){getEditorLock().cancelCurrentEdit()&&setFocus()}function absBox(e){var t={top:e.offsetTop,left:e.offsetLeft,bottom:0,right:0,width:$(e).outerWidth(),height:$(e).outerHeight(),visible:!0};t.bottom=t.top+t.height,t.right=t.left+t.width;var i=e.offsetParent;while((e=e.parentNode)!=document.body){if(null==e)break;t.visible&&e.scrollHeight!=e.offsetHeight&&"visible"!=$(e).css("overflowY")&&(t.visible=t.bottom>e.scrollTop&&t.top<e.scrollTop+e.clientHeight),t.visible&&e.scrollWidth!=e.offsetWidth&&"visible"!=$(e).css("overflowX")&&(t.visible=t.right>e.scrollLeft&&t.left<e.scrollLeft+e.clientWidth),t.left-=e.scrollLeft,t.top-=e.scrollTop,e===i&&(t.left+=e.offsetLeft,t.top+=e.offsetTop,i=e.offsetParent),t.bottom=t.top+t.height,t.right=t.left+t.width}return t}function getActiveCellPosition(){return absBox(activeCellNode)}function getGridPosition(){return absBox($container[0])}function handleActiveCellPositionChange(){if(activeCellNode&&(trigger(self.onActiveCellPositionChanged,{}),currentEditor)){var e=getActiveCellPosition();currentEditor.show&&currentEditor.hide&&(e.visible?currentEditor.show():currentEditor.hide()),currentEditor.position&&currentEditor.position(e)}}function getCellEditor(){return currentEditor}function getActiveCell(){return activeCellNode?{row:activeRow,cell:activeCell}:null}function getActiveCellNode(){return activeCellNode}function scrollRowIntoView(e,t){if(!hasFrozenRows||!options.frozenBottom&&e>actualFrozenRow-1||options.frozenBottom&&e<actualFrozenRow-1){var i=$viewportScrollContainerY.height(),n=hasFrozenRows&&!options.frozenBottom?e-options.frozenRow:e,o=n*options.rowHeight,s=(n+1)*options.rowHeight-i+(viewportHasHScroll?scrollbarDimensions.height:0);(n+1)*options.rowHeight>scrollTop+i+offset?(scrollTo(t?o:s),render()):n*options.rowHeight<scrollTop+offset&&(scrollTo(t?s:o),render())}}function scrollRowToTop(e){scrollTo(e*options.rowHeight),render()}function scrollPage(e){var t=e*numVisibleRows,i=scrollTop+options.rowHeight-1;if(scrollTo((getRowFromPosition(i)+t)*options.rowHeight),render(),options.enableCellNavigation&&null!=activeRow){var n=activeRow+t,o=getDataLengthIncludingAddNew();n>=o&&(n=o-1),n<0&&(n=0);var s=0,a=null,r=activePosX;while(s<=activePosX)canCellBeActive(n,s)&&(a=s),s+=getColspan(n,s);null!==a?(setActiveCellInternal(getCellNode(n,a)),activePosX=r):resetActiveCell()}}function navigatePageDown(){scrollPage(1)}function navigatePageUp(){scrollPage(-1)}function navigateTop(){navigateToRow(0)}function navigateBottom(){navigateToRow(getDataLength()-1)}function navigateToRow(e){var t=getDataLength();if(!t)return!0;if(e<0?e=0:e>=t&&(e=t-1),scrollCellIntoView(e,0,!0),options.enableCellNavigation&&null!=activeRow){var i=0,n=null,o=activePosX;while(i<=activePosX)canCellBeActive(e,i)&&(n=i),i+=getColspan(e,i);null!==n?(setActiveCellInternal(getCellNode(e,n)),activePosX=o):resetActiveCell()}return!0}function getColspan(e,t){var i=data.getItemMetadata&&data.getItemMetadata(e);if(!i||!i.columns)return 1;var n=i.columns[columns[t].id]||i.columns[t],o=n&&n.colspan;return o="*"===o?columns.length-t:o||1,o}function findFirstFocusableCell(e){var t=0;while(t<columns.length){if(canCellBeActive(e,t))return t;t+=getColspan(e,t)}return null}function findLastFocusableCell(e){var t=0,i=null;while(t<columns.length)canCellBeActive(e,t)&&(i=t),t+=getColspan(e,t);return i}function gotoRight(e,t,i){if(t>=columns.length)return null;do{t+=getColspan(e,t)}while(t<columns.length&&!canCellBeActive(e,t));return t<columns.length?{row:e,cell:t,posX:t}:null}function gotoLeft(e,t,i){if(t<=0)return null;var n=findFirstFocusableCell(e);if(null===n||n>=t)return null;var o,s={row:e,cell:n,posX:n};while(1){if(o=gotoRight(s.row,s.cell,s.posX),!o)return null;if(o.cell>=t)return s;s=o}}function gotoDown(e,t,i){var n,o=getDataLengthIncludingAddNew();while(1){if(++e>=o)return null;n=t=0;while(t<=i)n=t,t+=getColspan(e,t);if(canCellBeActive(e,n))return{row:e,cell:n,posX:i}}}function gotoUp(e,t,i){var n;while(1){if(--e<0)return null;n=t=0;while(t<=i)n=t,t+=getColspan(e,t);if(canCellBeActive(e,n))return{row:e,cell:n,posX:i}}}function gotoNext(e,t,i){if(null==e&&null==t&&(e=t=i=0,canCellBeActive(e,t)))return{row:e,cell:t,posX:t};var n=gotoRight(e,t,i);if(n)return n;var o=null,s=getDataLengthIncludingAddNew();e===s-1&&e--;while(++e<s)if(o=findFirstFocusableCell(e),null!==o)return{row:e,cell:o,posX:o};return null}function gotoPrev(e,t,i){if(null==e&&null==t&&(e=getDataLengthIncludingAddNew()-1,t=i=columns.length-1,canCellBeActive(e,t)))return{row:e,cell:t,posX:t};var n,o;while(!n){if(n=gotoLeft(e,t,i),n)break;if(--e<0)return null;t=0,o=findLastFocusableCell(e),null!==o&&(n={row:e,cell:o,posX:o})}return n}function gotoRowStart(e,t,i){var n=findFirstFocusableCell(e);return null===n?null:{row:e,cell:n,posX:n}}function gotoRowEnd(e,t,i){var n=findLastFocusableCell(e);return null===n?null:{row:e,cell:n,posX:n}}function navigateRight(){return navigate("right")}function navigateLeft(){return navigate("left")}function navigateDown(){return navigate("down")}function navigateUp(){return navigate("up")}function navigateNext(){return navigate("next")}function navigatePrev(){return navigate("prev")}function navigateRowStart(){return navigate("home")}function navigateRowEnd(){return navigate("end")}function navigate(e){if(!options.enableCellNavigation)return!1;if(!activeCellNode&&"prev"!=e&&"next"!=e)return!1;if(!getEditorLock().commitCurrentEdit())return!0;setFocus();var t={up:-1,down:1,left:-1,right:1,prev:-1,next:1,home:-1,end:1};tabbingDirection=t[e];var i={up:gotoUp,down:gotoDown,left:gotoLeft,right:gotoRight,prev:gotoPrev,next:gotoNext,home:gotoRowStart,end:gotoRowEnd},n=i[e],o=n(activeRow,activeCell,activePosX);if(o){if(hasFrozenRows&&options.frozenBottom&o.row==getDataLength())return;var s=o.row==getDataLength();return(!options.frozenBottom&&o.row>=actualFrozenRow||options.frozenBottom&&o.row<actualFrozenRow)&&scrollCellIntoView(o.row,o.cell,!s&&options.emulatePagingWhenScrolling),setActiveCellInternal(getCellNode(o.row,o.cell)),activePosX=o.posX,!0}return setActiveCellInternal(getCellNode(activeRow,activeCell)),!1}function getCellNode(e,t){if(rowsCache[e]){ensureCellNodesInRowsCache(e);try{return rowsCache[e].cellNodesByColumnIdx.length>t?rowsCache[e].cellNodesByColumnIdx[t][0]:null}catch(i){return rowsCache[e].cellNodesByColumnIdx[t]}}return null}function setActiveCell(e,t,i,n,o){initialized&&(e>getDataLength()||e<0||t>=columns.length||t<0||options.enableCellNavigation&&(scrollCellIntoView(e,t,!1),setActiveCellInternal(getCellNode(e,t),i,n,o)))}function canCellBeActive(e,t){if(!options.enableCellNavigation||e>=getDataLengthIncludingAddNew()||e<0||t>=columns.length||t<0)return!1;var i=data.getItemMetadata&&data.getItemMetadata(e);if(i&&"undefined"!==typeof i.focusable)return!!i.focusable;var n=i&&i.columns;return n&&n[columns[t].id]&&"undefined"!==typeof n[columns[t].id].focusable?!!n[columns[t].id].focusable:n&&n[t]&&"undefined"!==typeof n[t].focusable?!!n[t].focusable:!!columns[t].focusable}function canCellBeSelected(e,t){if(e>=getDataLength()||e<0||t>=columns.length||t<0)return!1;var i=data.getItemMetadata&&data.getItemMetadata(e);if(i&&"undefined"!==typeof i.selectable)return!!i.selectable;var n=i&&i.columns&&(i.columns[columns[t].id]||i.columns[t]);return n&&"undefined"!==typeof n.selectable?!!n.selectable:!!columns[t].selectable}function gotoCell(e,t,i,n){if(initialized&&canCellBeActive(e,t)&&getEditorLock().commitCurrentEdit()){scrollCellIntoView(e,t,!1);var o=getCellNode(e,t),s=columns[t],a=!!(options.editable&&s&&s.editor&&options.suppressActiveCellChangeOnEdit);setActiveCellInternal(o,i||e===getDataLength()||options.autoEdit,null,a,n),currentEditor||setFocus()}}function commitCurrentEdit(){var e=getDataItem(activeRow),t=columns[activeCell];if(currentEditor){if(currentEditor.isValueChanged()){var i=currentEditor.validate();if(i.valid){if(activeRow<getDataLength()){var n={row:activeRow,cell:activeCell,editor:currentEditor,serializedValue:currentEditor.serializeValue(),prevSerializedValue:serializedEditorValue,execute:function(){this.editor.applyValue(e,this.serializedValue),updateRow(this.row),trigger(self.onCellChange,{row:this.row,cell:this.cell,item:e})},undo:function(){this.editor.applyValue(e,this.prevSerializedValue),updateRow(this.row),trigger(self.onCellChange,{row:this.row,cell:this.cell,item:e})}};options.editCommandHandler?(makeActiveCellNormal(),options.editCommandHandler(e,t,n)):(n.execute(),makeActiveCellNormal())}else{var o={};currentEditor.applyValue(o,currentEditor.serializeValue()),makeActiveCellNormal(),trigger(self.onAddNewRow,{item:o,column:t})}return!getEditorLock().isActive()}return $(activeCellNode).removeClass("invalid"),$(activeCellNode).width(),$(activeCellNode).addClass("invalid"),trigger(self.onValidationError,{editor:currentEditor,cellNode:activeCellNode,validationResults:i,row:activeRow,cell:activeCell,column:t}),currentEditor.focus(),!1}makeActiveCellNormal()}return!0}function cancelCurrentEdit(){return makeActiveCellNormal(),!0}function rowsToRanges(e){for(var t=[],i=columns.length-1,n=0;n<e.length;n++)t.push(new Slick.Range(e[n],0,e[n],i));return t}function getSelectedRows(){if(!selectionModel)throw new Error("Selection model is not set");return selectedRows.slice(0)}function setSelectedRows(e){if(!selectionModel)throw new Error("Selection model is not set");self&&self.getEditorLock&&!self.getEditorLock().isActive()&&selectionModel.setSelectedRanges(rowsToRanges(e))}this.debug=function(){var e="";e+="\ncounter_rows_rendered:  "+counter_rows_rendered,e+="\ncounter_rows_removed:  "+counter_rows_removed,e+="\nrenderedRows:  "+renderedRows,e+="\nnumVisibleRows:  "+numVisibleRows,e+="\nmaxSupportedCssHeight:  "+maxSupportedCssHeight,e+="\nn(umber of pages):  "+n,e+="\n(current) page:  "+page,e+="\npage height (ph):  "+ph,e+="\nvScrollDir:  "+vScrollDir,alert(e)},this.eval=function(expr){return eval(expr)},$.extend(this,{slickGridVersion:"2.4.22",onScroll:new Slick.Event,onSort:new Slick.Event,onHeaderMouseEnter:new Slick.Event,onHeaderMouseLeave:new Slick.Event,onHeaderContextMenu:new Slick.Event,onHeaderClick:new Slick.Event,onHeaderCellRendered:new Slick.Event,onBeforeHeaderCellDestroy:new Slick.Event,onHeaderRowCellRendered:new Slick.Event,onFooterRowCellRendered:new Slick.Event,onFooterContextMenu:new Slick.Event,onFooterClick:new Slick.Event,onBeforeHeaderRowCellDestroy:new Slick.Event,onBeforeFooterRowCellDestroy:new Slick.Event,onMouseEnter:new Slick.Event,onMouseLeave:new Slick.Event,onClick:new Slick.Event,onDblClick:new Slick.Event,onContextMenu:new Slick.Event,onKeyDown:new Slick.Event,onAddNewRow:new Slick.Event,onBeforeAppendCell:new Slick.Event,onValidationError:new Slick.Event,onViewportChanged:new Slick.Event,onColumnsReordered:new Slick.Event,onColumnsResized:new Slick.Event,onCellChange:new Slick.Event,onBeforeEditCell:new Slick.Event,onBeforeCellEditorDestroy:new Slick.Event,onBeforeDestroy:new Slick.Event,onActiveCellChanged:new Slick.Event,onActiveCellPositionChanged:new Slick.Event,onDragInit:new Slick.Event,onDragStart:new Slick.Event,onDrag:new Slick.Event,onDragEnd:new Slick.Event,onSelectedRowsChanged:new Slick.Event,onCellCssStylesChanged:new Slick.Event,onAutosizeColumns:new Slick.Event,onRendered:new Slick.Event,registerPlugin:registerPlugin,unregisterPlugin:unregisterPlugin,getPluginByName:getPluginByName,getColumns:getColumns,setColumns:setColumns,getColumnIndex:getColumnIndex,updateColumnHeader:updateColumnHeader,setSortColumn:setSortColumn,setSortColumns:setSortColumns,getSortColumns:getSortColumns,autosizeColumns:autosizeColumns,autosizeColumn:autosizeColumn,getOptions:getOptions,setOptions:setOptions,getData:getData,getDataLength:getDataLength,getDataItem:getDataItem,setData:setData,getSelectionModel:getSelectionModel,setSelectionModel:setSelectionModel,getSelectedRows:getSelectedRows,setSelectedRows:setSelectedRows,getContainerNode:getContainerNode,updatePagingStatusFromView:updatePagingStatusFromView,applyFormatResultToCellNode:applyFormatResultToCellNode,render:render,invalidate:invalidate,invalidateRow:invalidateRow,invalidateRows:invalidateRows,invalidateAllRows:invalidateAllRows,updateCell:updateCell,updateRow:updateRow,getViewport:getVisibleRange,getRenderedRange:getRenderedRange,resizeCanvas:resizeCanvas,updateRowCount:updateRowCount,scrollRowIntoView:scrollRowIntoView,scrollRowToTop:scrollRowToTop,scrollCellIntoView:scrollCellIntoView,scrollColumnIntoView:scrollColumnIntoView,getCanvasNode:getCanvasNode,getUID:getUID,getHeaderColumnWidthDiff:getHeaderColumnWidthDiff,getScrollbarDimensions:getScrollbarDimensions,getHeadersWidth:getHeadersWidth,getCanvasWidth:getCanvasWidth,getCanvases:getCanvases,getActiveCanvasNode:getActiveCanvasNode,setActiveCanvasNode:setActiveCanvasNode,getViewportNode:getViewportNode,getActiveViewportNode:getActiveViewportNode,setActiveViewportNode:setActiveViewportNode,focus:setFocus,scrollTo:scrollTo,getCellFromPoint:getCellFromPoint,getCellFromEvent:getCellFromEvent,getActiveCell:getActiveCell,setActiveCell:setActiveCell,getActiveCellNode:getActiveCellNode,getActiveCellPosition:getActiveCellPosition,resetActiveCell:resetActiveCell,editActiveCell:makeActiveCellEditable,getCellEditor:getCellEditor,getCellNode:getCellNode,getCellNodeBox:getCellNodeBox,canCellBeSelected:canCellBeSelected,canCellBeActive:canCellBeActive,navigatePrev:navigatePrev,navigateNext:navigateNext,navigateUp:navigateUp,navigateDown:navigateDown,navigateLeft:navigateLeft,navigateRight:navigateRight,navigatePageUp:navigatePageUp,navigatePageDown:navigatePageDown,navigateTop:navigateTop,navigateBottom:navigateBottom,navigateRowStart:navigateRowStart,navigateRowEnd:navigateRowEnd,gotoCell:gotoCell,getTopPanel:getTopPanel,setTopPanelVisibility:setTopPanelVisibility,getPreHeaderPanel:getPreHeaderPanel,getPreHeaderPanelLeft:getPreHeaderPanel,getPreHeaderPanelRight:getPreHeaderPanelRight,setPreHeaderPanelVisibility:setPreHeaderPanelVisibility,getHeader:getHeader,getHeaderColumn:getHeaderColumn,setHeaderRowVisibility:setHeaderRowVisibility,getHeaderRow:getHeaderRow,getHeaderRowColumn:getHeaderRowColumn,setFooterRowVisibility:setFooterRowVisibility,getFooterRow:getFooterRow,getFooterRowColumn:getFooterRowColumn,getGridPosition:getGridPosition,flashCell:flashCell,addCellCssStyles:addCellCssStyles,setCellCssStyles:setCellCssStyles,removeCellCssStyles:removeCellCssStyles,getCellCssStyles:getCellCssStyles,getFrozenRowOffset:getFrozenRowOffset,setColumnHeaderVisibility:setColumnHeaderVisibility,init:finishInitialization,destroy:destroy,getEditorLock:getEditorLock,getEditController:getEditController}),init()}$.extend(!0,window,{Slick:{Grid:SlickGrid}})})(jQuery)},4936:function(e,t,i){e.exports=i.p+"static/img/histogram-guid.57843081.svg"},"498a":function(e,t,i){"use strict";var n=i("23e7"),o=i("58a8").trim,s=i("c8d2");n({target:"String",proto:!0,forced:s("trim")},{trim:function(){return o(this)}})},"4de4":function(e,t,i){"use strict";var n=i("23e7"),o=i("b727").filter,s=i("1dde"),a=i("ae40"),r=s("filter"),l=a("filter");n({target:"Array",proto:!0,forced:!r||!l},{filter:function(e){return o(this,e,arguments.length>1?arguments[1]:void 0)}})},"4df4":function(e,t,i){"use strict";var n=i("0366"),o=i("7b0b"),s=i("9bdd"),a=i("e95a"),r=i("50c4"),l=i("8418"),c=i("35a1");e.exports=function(e){var t,i,h,u,d,p,f=o(e),g="function"==typeof this?this:Array,m=arguments.length,v=m>1?arguments[1]:void 0,w=void 0!==v,y=c(f),b=0;if(w&&(v=n(v,m>2?arguments[2]:void 0,2)),void 0==y||g==Array&&a(y))for(t=r(f.length),i=new g(t);t>b;b++)p=w?v(f[b],b):f[b],l(i,b,p);else for(u=y.call(f),d=u.next,i=new g;!(h=d.call(u)).done;b++)p=w?s(u,v,[h.value,b],!0):h.value,l(i,b,p);return i.length=b,i}},5181:function(e,t,i){},"532e":function(e,t,i){e.exports=i.p+"static/img/data-process-guid.beda9514.svg"},"56de":function(e,t,i){e.exports=i.p+"static/img/graph-guid.2fe2af69.svg"},"6b75":function(e,t,i){"use strict";function n(e,t){(null==t||t>e.length)&&(t=e.length);for(var i=0,n=Array(t);i<t;i++)n[i]=e[i];return n}i.d(t,"a",(function(){return n}))},"70a2":function(e,t){(function(e){function t(t){var i,n,o,s,a,r=this,l={groupItemMetadataProvider:null,inlineFilters:!1},c="id",h=[],u=[],d={},p=null,f=null,g=null,m=!1,v=!0,w={},y={},b=[],C=[],_={getter:null,formatter:null,comparer:function(e,t){return e.value===t.value?0:e.value>t.value?1:-1},predefinedValues:[],aggregators:[],aggregateEmpty:!1,aggregateCollapsed:!1,aggregateChildGroups:!1,collapsed:!1,displayTotalsRow:!0,lazyTotalsCalculation:!1},x=[],k=[],S=[],D=":|:",R=0,$=0,T=0,P=new Slick.Event,I=new Slick.Event,M=new Slick.Event,A=new Slick.Event,E=new Slick.Event,N=new Slick.Event,z=new Slick.Event,H=new Slick.Event;function L(){m=!0}function F(){m=!1,Ge()}function W(e){w=e}function O(e){o=e}function j(e){var t;e=e||0;for(var i=e,n=h.length;i<n;i++){if(t=h[i][c],void 0===t)throw new Error("Each data element must implement a unique 'id' property");d[t]=i}}function B(){for(var e,t=0,i=h.length;t<i;t++)if(e=h[t][c],void 0===e||d[e]!==t)throw new Error("Each data element must implement a unique 'id' property")}function q(){return h}function V(){return c}function G(e,t){void 0!==t&&(c=t),h=b=e,d={},j(),B(),Ge(),P.notify({idProperty:t},null,r)}function Y(e){E.notify(K(),null,r),void 0!=e.pageSize&&(R=e.pageSize,$=R?Math.min($,Math.max(0,Math.ceil(T/R)-1)):0),void 0!=e.pageNum&&($=Math.min(e.pageNum,Math.max(0,Math.ceil(T/R)-1))),N.notify(K(),null,r),Ge()}function K(){var e=R?Math.max(1,Math.ceil(T/R)):1;return{pageSize:R,pageNum:$,totalRows:T,totalPages:e,dataView:r}}function U(e,t){v=t,n=e,i=null,!1===t&&h.reverse(),h.sort(e),!1===t&&h.reverse(),d={},j(),Ge()}function X(e,t){v=t,i=e,n=null;var o=Object.prototype.toString;Object.prototype.toString="function"==typeof e?e:function(){return this[e]},!1===t&&h.reverse(),h.sort(),Object.prototype.toString=o,!1===t&&h.reverse(),d={},j(),Ge()}function Q(){n?U(n,v):i&&X(i,v)}function J(){return b}function Z(){return f}function ee(e){f=e,t.inlineFilters&&(s=Le(),a=Fe()),Ge()}function te(){return x}function ie(i){t.groupItemMetadataProvider||(t.groupItemMetadataProvider=new Slick.Data.GroupItemMetadataProvider),k=[],S=[],i=i||[],x=i instanceof Array?i:[i];for(var n=0;n<x.length;n++){var o=x[n]=e.extend(!0,{},_,x[n]);o.getterIsAFn="function"===typeof o.getter,o.compiledAccumulators=[];var s=o.aggregators.length;while(s--)o.compiledAccumulators[s]=He(o.aggregators[s]);S[n]={}}Ge()}function ne(e,t,i){ie(null!=e?{getter:e,formatter:t,comparer:i}:[])}function oe(e,t){if(!x.length)throw new Error("At least one grouping must be specified before calling setAggregators().");x[0].aggregators=e,x[0].aggregateCollapsed=t,ie(x)}function se(e){return h[e]}function ae(e){return d[e]}function re(){if(!p){p={};for(var e=0,t=u.length;e<t;e++)p[u[e][c]]=e}}function le(e){return re(),p[e[c]]}function ce(e){return re(),p[e]}function he(e){return h[d[e]]}function ue(e){var t=[];re();for(var i=0,n=e.length;i<n;i++){var o=p[e[i][c]];null!=o&&(t[t.length]=o)}return t}function de(e){var t=[];re();for(var i=0,n=e.length;i<n;i++){var o=p[e[i]];null!=o&&(t[t.length]=o)}return t}function pe(e){for(var t=[],i=0,n=e.length;i<n;i++)e[i]<u.length&&(t[t.length]=u[e[i]][c]);return t}function fe(e,t){if(void 0===d[e])throw new Error("Invalid id");if(e!==t[c]){var i=t[c];if(null==i)throw new Error("Cannot update item to associate with a null id");if(void 0!==d[i])throw new Error("Cannot update item to associate with a non-unique id");d[i]=d[e],delete d[e],g&&g[e]&&delete g[e],e=i}h[d[e]]=t,g||(g={}),g[e]=!0,Ge()}function ge(e,t){h.splice(e,0,t),j(e),Ge()}function me(e){h.push(e),j(h.length-1),Ge()}function ve(e){var t=d[e];if(void 0===t)throw new Error("Invalid id");delete d[e],h.splice(t,1),j(t),Ge()}function we(e){if(!n)throw new Error("sortedAddItem() requires a sort comparer, use sort()");ge(be(e),e)}function ye(e,t){if(void 0===d[e]||e!==t[c])throw new Error("Invalid or non-matching id "+d[e]);if(!n)throw new Error("sortedUpdateItem() requires a sort comparer, use sort()");var i=he(e);0!==n(i,t)?(ve(e),we(t)):fe(e,t)}function be(e){var t=0,i=h.length;while(t<i){var o=t+i>>>1;-1===n(h[o],e)?t=o+1:i=o}return t}function Ce(){return u.length}function _e(e){var t=u[e];if(t&&t.__group&&t.totals&&!t.totals.initialized){var i=x[t.level];i.displayTotalsRow||(Me(t.totals),t.title=i.formatter?i.formatter(t):t.value)}else t&&t.__groupTotals&&!t.initialized&&Me(t);return t}function xe(e){var i=u[e];return void 0===i?null:i.__group?t.groupItemMetadataProvider.getGroupRowMetadata(i):i.__groupTotals?t.groupItemMetadataProvider.getTotalsRowMetadata(i):null}function ke(e,t){if(null==e)for(var i=0;i<x.length;i++)S[i]={},x[i].collapsed=t,!0===t?H.notify({level:i,groupingKey:null}):z.notify({level:i,groupingKey:null});else S[e]={},x[e].collapsed=t,!0===t?H.notify({level:e,groupingKey:null}):z.notify({level:e,groupingKey:null});Ge()}function Se(e){ke(e,!0)}function De(e){ke(e,!1)}function Re(e,t,i){S[e][t]=x[e].collapsed^i,Ge()}function $e(e){var t,i,n=Array.prototype.slice.call(arguments),o=n[0];1===n.length&&-1!==o.indexOf(D)?(t=o,i=o.split(D).length-1):(t=n.join(D),i=n.length-1),Re(i,t,!0),H.notify({level:i,groupingKey:t})}function Te(e){var t,i,n=Array.prototype.slice.call(arguments),o=n[0];1===n.length&&-1!==o.indexOf(D)?(i=o.split(D).length-1,t=o):(i=n.length-1,t=n.join(D)),Re(i,t,!1),z.notify({level:i,groupingKey:t})}function Pe(){return k}function Ie(e,t){for(var i,n,o,s=[],a={},r=t?t.level+1:0,l=x[r],c=0,h=l.predefinedValues.length;c<h;c++)n=l.predefinedValues[c],i=a[n],i||(i=new Slick.Group,i.value=n,i.level=r,i.groupingKey=(t?t.groupingKey+D:"")+n,s[s.length]=i,a[n]=i);for(c=0,h=e.length;c<h;c++)o=e[c],n=l.getterIsAFn?l.getter(o):o[l.getter],i=a[n],i||(i=new Slick.Group,i.value=n,i.level=r,i.groupingKey=(t?t.groupingKey+D:"")+n,s[s.length]=i,a[n]=i),i.rows[i.count++]=o;if(r<x.length-1)for(c=0;c<s.length;c++)i=s[c],i.groups=Ie(i.rows,i);return s.length&&Ee(s),s.sort(x[r].comparer),s}function Me(e){var t,i=e.group,n=x[i.level],o=i.level==x.length,s=n.aggregators.length;if(!o&&n.aggregateChildGroups){var a=i.groups.length;while(a--)i.groups[a].totals.initialized||Me(i.groups[a].totals)}while(s--)t=n.aggregators[s],t.init(),!o&&n.aggregateChildGroups?n.compiledAccumulators[s].call(t,i.groups):n.compiledAccumulators[s].call(t,i.rows),t.storeResult(e);e.initialized=!0}function Ae(e){var t=x[e.level],i=new Slick.GroupTotals;i.group=e,e.totals=i,t.lazyTotalsCalculation||Me(i)}function Ee(e,t){t=t||0;var i,n=x[t],o=n.collapsed,s=S[t],a=e.length;while(a--)i=e[a],i.collapsed&&!n.aggregateCollapsed||(i.groups&&Ee(i.groups,t+1),n.aggregators.length&&(n.aggregateEmpty||i.rows.length||i.groups&&i.groups.length)&&Ae(i),i.collapsed=o^s[i.groupingKey],i.title=n.formatter?n.formatter(i):i.value)}function Ne(e,t){t=t||0;for(var i,n,o=x[t],s=[],a=0,r=0,l=e.length;r<l;r++){if(n=e[r],s[a++]=n,!n.collapsed){i=n.groups?Ne(n.groups,t+1):n.rows;for(var c=0,h=i.length;c<h;c++)s[a++]=i[c]}n.totals&&o.displayTotalsRow&&(!n.collapsed||o.aggregateCollapsed)&&(s[a++]=n.totals)}return s}function ze(e){var t=e.toString(),i=t.indexOf("function")>=0,n=i?/^function[^(]*\(([^)]*)\)\s*{([\s\S]*)}$/:/^[^(]*\(([^)]*)\)\s*{([\s\S]*)}$/,o=e.toString().match(n);return{params:o[1].split(","),body:o[2]}}function He(e){if(e.accumulate){var t=ze(e.accumulate),i=new Function("_items","for (var "+t.params[0]+", _i=0, _il=_items.length; _i<_il; _i++) {"+t.params[0]+" = _items[_i]; "+t.body+"}"),n="compiledAccumulatorLoop";return i.displayName=n,i.name=We(i,n),i}return function(){}}function Le(){var e=ze(f),t="{ continue _coreloop; }$1",i="{ _retval[_idx++] = $item$; continue _coreloop; }$1",n=e.body.replace(/return false\s*([;}]|\}|$)/gi,t).replace(/return!1([;}]|\}|$)/gi,t).replace(/return true\s*([;}]|\}|$)/gi,i).replace(/return!0([;}]|\}|$)/gi,i).replace(/return ([^;}]+?)\s*([;}]|$)/gi,"{ if ($1) { _retval[_idx++] = $item$; }; continue _coreloop; }$2"),o=["var _retval = [], _idx = 0; ","var $item$, $args$ = _args; ","_coreloop: ","for (var _i = 0, _il = _items.length; _i < _il; _i++) { ","$item$ = _items[_i]; ","$filter$; ","} ","return _retval; "].join("");o=o.replace(/\$filter\$/gi,n),o=o.replace(/\$item\$/gi,e.params[0]),o=o.replace(/\$args\$/gi,e.params[1]);var s=new Function("_items,_args",o),a="compiledFilter";return s.displayName=a,s.name=We(s,a),s}function Fe(){var e=ze(f),t="{ continue _coreloop; }$1",i="{ _cache[_i] = true;_retval[_idx++] = $item$; continue _coreloop; }$1",n=e.body.replace(/return false\s*([;}]|\}|$)/gi,t).replace(/return!1([;}]|\}|$)/gi,t).replace(/return true\s*([;}]|\}|$)/gi,i).replace(/return!0([;}]|\}|$)/gi,i).replace(/return ([^;}]+?)\s*([;}]|$)/gi,"{ if ((_cache[_i] = $1)) { _retval[_idx++] = $item$; }; continue _coreloop; }$2"),o=["var _retval = [], _idx = 0; ","var $item$, $args$ = _args; ","_coreloop: ","for (var _i = 0, _il = _items.length; _i < _il; _i++) { ","$item$ = _items[_i]; ","if (_cache[_i]) { ","_retval[_idx++] = $item$; ","continue _coreloop; ","} ","$filter$; ","} ","return _retval; "].join("");o=o.replace(/\$filter\$/gi,n),o=o.replace(/\$item\$/gi,e.params[0]),o=o.replace(/\$args\$/gi,e.params[1]);var s=new Function("_items,_args,_cache",o),a="compiledFilterWithCaching";return s.displayName=a,s.name=We(s,a),s}function We(e,t){try{Object.defineProperty(e,"name",{writable:!0,value:t})}catch(i){e.name=t}}function Oe(e,t){for(var i=[],n=0,o=0,s=e.length;o<s;o++)f(e[o],t)&&(i[n++]=e[o]);return i}function je(e,t,i){for(var n,o=[],s=0,a=0,r=e.length;a<r;a++)n=e[a],i[a]?o[s++]=n:f(n,t)&&(o[s++]=n,i[a]=!0);return o}function Be(e){if(f){var i=t.inlineFilters?s:Oe,n=t.inlineFilters?a:je;w.isFilterNarrowing?b=i(b,o):w.isFilterExpanding?b=n(e,o,C):w.isFilterUnchanged||(b=i(e,o))}else b=R?e:e.concat();var r;return R?(b.length<=$*R&&($=0===b.length?0:Math.floor((b.length-1)/R)),r=b.slice(R*$,R*$+R)):r=b,{totalRows:b.length,rows:r}}function qe(e,t){var i,n,o,s=[],a=0,r=Math.max(t.length,e.length);w&&w.ignoreDiffsBefore&&(a=Math.max(0,Math.min(t.length,w.ignoreDiffsBefore))),w&&w.ignoreDiffsAfter&&(r=Math.min(t.length,Math.max(0,w.ignoreDiffsAfter)));for(var l=a,h=e.length;l<r;l++)l>=h?s[s.length]=l:(i=t[l],n=e[l],(!i||x.length&&(o=i.__nonDataRow||n.__nonDataRow)&&i.__group!==n.__group||i.__group&&!i.equals(n)||o&&(i.__groupTotals||n.__groupTotals)||i[c]!=n[c]||g&&g[i[c]])&&(s[s.length]=l));return s}function Ve(e){p=null,w.isFilterNarrowing==y.isFilterNarrowing&&w.isFilterExpanding==y.isFilterExpanding||(C=[]);var t=Be(e);T=t.totalRows;var i=t.rows;k=[],x.length&&(k=Ie(i),k.length&&(i=Ne(k)));var n=qe(u,i);return u=i,n}function Ge(){if(!m){var t=e.extend(!0,{},K()),i=u.length,n=T,o=Ve(h,f);R&&T<$*R&&($=Math.max(0,Math.ceil(T/R)-1),o=Ve(h,f)),g=null,y=w,w={},n!==T&&(E.notify(t,null,r),N.notify(K(),null,r)),i!==u.length&&I.notify({previous:i,current:u.length,dataView:r,callingOnRowsChanged:o.length>0},null,r),o.length>0&&M.notify({rows:o,dataView:r,calledOnRowCountChanged:i!==u.length},null,r),(i!==u.length||o.length>0)&&A.notify({rowsDiff:o,previousRowCount:i,currentRowCount:u.length,rowCountChanged:i!==u.length,rowsChanged:o.length>0,dataView:r},null,r)}}function Ye(t,i,n){var o,s=this,a=s.mapRowsToIds(t.getSelectedRows()),r=new Slick.Event;function l(e){a.join(",")!=e.join(",")&&(a=e,r.notify({grid:t,ids:a,dataView:s},new Slick.EventData,s))}function c(){if(a.length>0){o=!0;var e=s.mapIdsToRows(a);i||l(s.mapRowsToIds(e)),t.setSelectedRows(e),o=!1}}return t.onSelectedRowsChanged.subscribe((function(i,r){if(!o){var c=s.mapRowsToIds(t.getSelectedRows());if(n&&t.getOptions().multiSelect){var h=e.grep(a,(function(e){return void 0===s.getRowById(e)}));l(h.concat(c))}else l(c)}})),this.onRowsOrCountChanged.subscribe(c),r}function Ke(e,t){var i,n;function o(e){for(var t in i={},e){var n=u[t][c];i[n]=e[t]}}function s(){if(i){n=!0,re();var o={};for(var s in i){var a=p[s];void 0!=a&&(o[a]=i[s])}e.setCellCssStyles(t,o),n=!1}}o(e.getCellCssStyles(t)),e.onCellCssStylesChanged.subscribe((function(i,a){n||t==a.key&&(a.hash?o(a.hash):(e.onCellCssStylesChanged.unsubscribe(),r.onRowsOrCountChanged.unsubscribe(s)))})),this.onRowsOrCountChanged.subscribe(s)}t=e.extend(!0,{},l,t),e.extend(this,{beginUpdate:L,endUpdate:F,setPagingOptions:Y,getPagingInfo:K,getIdPropertyName:V,getItems:q,setItems:G,setFilter:ee,getFilter:Z,getFilteredItems:J,sort:U,fastSort:X,reSort:Q,setGrouping:ie,getGrouping:te,groupBy:ne,setAggregators:oe,collapseAllGroups:Se,expandAllGroups:De,collapseGroup:$e,expandGroup:Te,getGroups:Pe,getIdxById:ae,getRowByItem:le,getRowById:ce,getItemById:he,getItemByIdx:se,mapItemsToRows:ue,mapRowsToIds:pe,mapIdsToRows:de,setRefreshHints:W,setFilterArgs:O,refresh:Ge,updateItem:fe,insertItem:ge,addItem:me,deleteItem:ve,sortedAddItem:we,sortedUpdateItem:ye,syncGridSelection:Ye,syncGridCellCssStyles:Ke,getLength:Ce,getItem:_e,getItemMetadata:xe,onSetItemsCalled:P,onRowCountChanged:I,onRowsChanged:M,onRowsOrCountChanged:A,onBeforePagingInfoChanged:E,onPagingInfoChanged:N,onGroupExpanded:z,onGroupCollapsed:H})}function i(e){this.field_=e,this.init=function(){this.count_=0,this.nonNullCount_=0,this.sum_=0},this.accumulate=function(e){var t=e[this.field_];this.count_++,null==t||""===t||isNaN(t)||(this.nonNullCount_++,this.sum_+=parseFloat(t))},this.storeResult=function(e){e.avg||(e.avg={}),0!==this.nonNullCount_&&(e.avg[this.field_]=this.sum_/this.nonNullCount_)}}function n(e){this.field_=e,this.init=function(){this.min_=null},this.accumulate=function(e){var t=e[this.field_];null==t||""===t||isNaN(t)||(null==this.min_||t<this.min_)&&(this.min_=t)},this.storeResult=function(e){e.min||(e.min={}),e.min[this.field_]=this.min_}}function o(e){this.field_=e,this.init=function(){this.max_=null},this.accumulate=function(e){var t=e[this.field_];null==t||""===t||isNaN(t)||(null==this.max_||t>this.max_)&&(this.max_=t)},this.storeResult=function(e){e.max||(e.max={}),e.max[this.field_]=this.max_}}function s(e){this.field_=e,this.init=function(){this.sum_=null},this.accumulate=function(e){var t=e[this.field_];null==t||""===t||isNaN(t)||(this.sum_+=parseFloat(t))},this.storeResult=function(e){e.sum||(e.sum={}),e.sum[this.field_]=this.sum_}}function a(e){this.field_=e,this.init=function(){},this.storeResult=function(e){e.count||(e.count={}),e.count[this.field_]=e.group.rows.length}}e.extend(!0,window,{Slick:{Data:{DataView:t,Aggregators:{Avg:i,Min:n,Max:o,Sum:s,Count:a}}}})})(jQuery)},"7d72":function(e,t){(function(e){function t(){var e=!1,t=!1;this.stopPropagation=function(){e=!0},this.isPropagationStopped=function(){return e},this.stopImmediatePropagation=function(){t=!0},this.isImmediatePropagationStopped=function(){return t}}function i(){var e=[];this.subscribe=function(t){e.push(t)},this.unsubscribe=function(t){for(var i=e.length-1;i>=0;i--)e[i]===t&&e.splice(i,1)},this.notify=function(i,n,o){var s;n=n||new t,o=o||this;for(var a=0;a<e.length&&!n.isPropagationStopped()&&!n.isImmediatePropagationStopped();a++)s=e[a].call(o,n,i);return s}}function n(){var e=[];this.subscribe=function(t,i){return e.push({event:t,handler:i}),t.subscribe(i),this},this.unsubscribe=function(t,i){var n=e.length;while(n--)if(e[n].event===t&&e[n].handler===i)return e.splice(n,1),void t.unsubscribe(i);return this},this.unsubscribeAll=function(){var t=e.length;while(t--)e[t].event.unsubscribe(e[t].handler);return e=[],this}}function o(e,t,i,n){void 0===i&&void 0===n&&(i=e,n=t),this.fromRow=Math.min(e,i),this.fromCell=Math.min(t,n),this.toRow=Math.max(e,i),this.toCell=Math.max(t,n),this.isSingleRow=function(){return this.fromRow==this.toRow},this.isSingleCell=function(){return this.fromRow==this.toRow&&this.fromCell==this.toCell},this.contains=function(e,t){return e>=this.fromRow&&e<=this.toRow&&t>=this.fromCell&&t<=this.toCell},this.toString=function(){return this.isSingleCell()?"("+this.fromRow+":"+this.fromCell+")":"("+this.fromRow+":"+this.fromCell+" - "+this.toRow+":"+this.toCell+")"}}function s(){this.__nonDataRow=!0}function a(){this.__group=!0,this.level=0,this.count=0,this.value=null,this.title=null,this.collapsed=!1,this.selectChecked=!1,this.totals=null,this.rows=[],this.groups=null,this.groupingKey=null}function r(){this.__groupTotals=!0,this.group=null,this.initialized=!1}function l(){var e=null;this.isActive=function(t){return t?e===t:null!==e},this.activate=function(t){if(t!==e){if(null!==e)throw new Error("SlickGrid.EditorLock.activate: an editController is still active, can't activate another editController");if(!t.commitCurrentEdit)throw new Error("SlickGrid.EditorLock.activate: editController must implement .commitCurrentEdit()");if(!t.cancelCurrentEdit)throw new Error("SlickGrid.EditorLock.activate: editController must implement .cancelCurrentEdit()");e=t}},this.deactivate=function(t){if(e!==t)throw new Error("SlickGrid.EditorLock.deactivate: specified editController is not the currently active one");e=null},this.commitCurrentEdit=function(){return!e||e.commitCurrentEdit()},this.cancelCurrentEdit=function(){return!e||e.cancelCurrentEdit()}}function c(t){var i={};function n(){o(t)}function o(e){e.forEach((function(e){i[e.id]=e,e.columns&&o(e.columns)}))}function s(e,t){return e.filter((function(e){var i=t.call(e);return i&&e.columns&&(e.columns=s(e.columns,t)),i&&(!e.columns||e.columns.length)}))}function a(e,t){e.sort((function(e,i){var n=r(t.getColumnIndex(e.id)),o=r(t.getColumnIndex(i.id));return n-o})).forEach((function(e){e.columns&&a(e.columns,t)}))}function r(e){return"undefined"===typeof e?-1:e}function l(e){if(!e.length)return e.columns?1+l(e.columns):1;for(var t in e)return l(e[t])}function c(e,t,i){var n=[];if(i=i||0,t==i)return e.length&&e.forEach((function(e){e.columns&&(e.extractColumns=function(){return h(e)})})),e;for(var o in e)e[o].columns&&(n=n.concat(c(e[o].columns,t,i+1)));return n}function h(e){var t=[];if(e.hasOwnProperty("length"))for(var i=0;i<e.length;i++)t=t.concat(h(e[i]));else{if(!e.hasOwnProperty("columns"))return e;t=t.concat(h(e.columns))}return t}function u(){return e.extend(!0,[],t)}n(),this.hasDepth=function(){for(var e in t)if(t[e].hasOwnProperty("columns"))return!0;return!1},this.getTreeColumns=function(){return t},this.extractColumns=function(){return this.hasDepth()?h(t):t},this.getDepth=function(){return l(t)},this.getColumnsInDepth=function(e){return c(t,e)},this.getColumnsInGroup=function(e){return h(e)},this.visibleColumns=function(){return s(u(),(function(){return this.visible}))},this.filter=function(e){return s(u(),e)},this.reOrder=function(e){return a(t,e)},this.getById=function(e){return i[e]},this.getInIds=function(e){return e.map((function(e){return i[e]}))}}e.extend(!0,window,{Slick:{Event:i,EventData:t,EventHandler:n,Range:o,NonDataRow:s,Group:a,GroupTotals:r,EditorLock:l,GlobalEditorLock:new l,TreeColumns:c,keyCode:{SPACE:8,BACKSPACE:8,DELETE:46,DOWN:40,END:35,ENTER:13,ESCAPE:27,HOME:36,INSERT:45,LEFT:37,PAGE_DOWN:34,PAGE_UP:33,RIGHT:39,TAB:9,UP:38,A:65},preClickClassName:"slick-edit-preclick",GridAutosizeColsMode:{None:"NOA",LegacyOff:"LOF",LegacyForceFit:"LFF",IgnoreViewport:"IGV",FitColsToViewport:"FCV",FitViewportToCols:"FVC"},ColAutosizeMode:{Locked:"LCK",Guide:"GUI",Content:"CON",ContentIntelligent:"CTI"},RowSelectionMode:{FirstRow:"FS1",FirstNRows:"FSN",AllRows:"ALL",LastRow:"LS1"},ValueFilterMode:{None:"NONE",DeDuplicate:"DEDP",GetGreatestAndSub:"GR8T",GetLongestTextAndSub:"LNSB",GetLongestText:"LNSC"},WidthEvalMode:{CanvasTextSize:"CANV",HTML:"HTML"}}}),a.prototype=new s,a.prototype.equals=function(e){return this.value===e.value&&this.count===e.count&&this.collapsed===e.collapsed&&this.title===e.title},r.prototype=new s})(jQuery)},"97a7":function(e,t,i){e.exports=i.p+"static/img/scalar-guid.111b3272.svg"},"98c0":function(e,t,i){e.exports=i.p+"static/img/tensor-guid.c9811239.svg"},a15b:function(e,t,i){"use strict";var n=i("23e7"),o=i("44ad"),s=i("fc6a"),a=i("a640"),r=[].join,l=o!=Object,c=a("join",",");n({target:"Array",proto:!0,forced:l||!c},{join:function(e){return r.call(s(this),void 0===e?",":e)}})},a434:function(e,t,i){"use strict";var n=i("23e7"),o=i("23cb"),s=i("a691"),a=i("50c4"),r=i("7b0b"),l=i("65f0"),c=i("8418"),h=i("1dde"),u=i("ae40"),d=h("splice"),p=u("splice",{ACCESSORS:!0,0:0,1:2}),f=Math.max,g=Math.min,m=9007199254740991,v="Maximum allowed length exceeded";n({target:"Array",proto:!0,forced:!d||!p},{splice:function(e,t){var i,n,h,u,d,p,w=r(this),y=a(w.length),b=o(e,y),C=arguments.length;if(0===C?i=n=0:1===C?(i=0,n=y-b):(i=C-2,n=g(f(s(t),0),y-b)),y+i-n>m)throw TypeError(v);for(h=l(w,n),u=0;u<n;u++)d=b+u,d in w&&c(h,u,w[d]);if(h.length=n,i<n){for(u=b;u<y-n;u++)d=u+n,p=u+i,d in w?w[p]=w[d]:delete w[p];for(u=y;u>y-n+i;u--)delete w[u-1]}else if(i>n)for(u=y-n;u>b;u--)d=u+n-1,p=u+i-1,d in w?w[p]=w[d]:delete w[p];for(u=0;u<i;u++)w[u+b]=arguments[u+2];return w.length=y-n+i,h}})},a630:function(e,t,i){var n=i("23e7"),o=i("4df4"),s=i("1c7e"),a=!s((function(e){Array.from(e)}));n({target:"Array",stat:!0,forced:a},{from:o})},ad6c:function(e,t,i){},b0c0:function(e,t,i){var n=i("83ab"),o=i("9bf2").f,s=Function.prototype,a=s.toString,r=/^\s*function ([^ (]*)/,l="name";n&&!(l in s)&&o(s,l,{configurable:!0,get:function(){try{return a.call(this).match(r)[1]}catch(e){return""}}})},c8d2:function(e,t,i){var n=i("d039"),o=i("5899"),s="​᠎";e.exports=function(e){return n((function(){return!!o[e]()||s[e]()!=s||o[e].name!==e}))}},cc22:function(e,t,i){"use strict";i.r(t);var n=function(){var e=this,t=e.$createElement,n=e._self._c||t;return n("div",{staticClass:"cl-dashboard"},[n("div",{staticClass:"cl-dashboard-top"},[n("div",{staticClass:"cl-dashboard-top-title"},[e._v(" "+e._s(e.$t("trainingDashboard.trainingDashboardTitle"))+" ")]),e.trainJobCached?n("div",{staticClass:"loading-icon",class:e.trainJobCached>1?"el-icon-loading":"el-icon-time",attrs:{title:e.trainJobCached>1?e.$t("trainingDashboard.loadingTip"):e.$t("trainingDashboard.waitLoading")}}):e._e(),n("div",{staticClass:"path-message"},[n("span",[e._v(e._s(e.$t("symbols.leftbracket")))]),n("span",[e._v(e._s(e.$t("trainingDashboard.summaryDirPath")))]),n("span",{staticClass:"path-name",attrs:{title:this.summaryPath}},[e._v(e._s(e.summaryPath))]),n("span",[e._v(e._s(e.$t("symbols.rightbracket")))])])]),n("div",{staticClass:"cl-dashboard-center"},[n("div",{staticClass:"cl-dashboard-con-up",class:e.curPageArr.length&&!e.wrongPlugin?"":"no-data-hover",on:{click:e.viewMoreScalars}},[n("div",{staticClass:"cl-dashboard-title"},[e._v(" "+e._s(e.$t("trainingDashboard.trainingScalar")))]),n("div",{staticClass:"cl-module"},[n("div",{directives:[{name:"show",rawName:"v-show",value:e.curPageArr.length&&!e.wrongPlugin,expression:"curPageArr.length && !wrongPlugin"}],staticClass:"cl-scalar-tagName"},e._l(e.curPageArr,(function(t,i){return n("div",{key:i,class:["tagNameLeft",1==i?"tagNameRight":""]},[e._v(" "+e._s(t.tagName)+" ")])})),0),n("div",{directives:[{name:"show",rawName:"v-show",value:e.curPageArr.length&&!e.wrongPlugin,expression:"curPageArr.length && !wrongPlugin"}],key:"chart-data",attrs:{id:"module-chart"}}),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.curPageArr.length||e.wrongPlugin,expression:"!curPageArr.length || wrongPlugin"}],key:"no-chart-data",staticClass:"no-data-img"},[n("img",{attrs:{src:i("97a7"),alt:""}}),e.initOverKey.scalar?n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("summaryManage.guidTipFirst"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.guidUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.guidTipSecond")))]),e._v(" "+e._s(e.$t("summaryManage.guidTipThird"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.scalarUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.scalarTipFirst")))]),e._v(" "+e._s(e.$t("summaryManage.guidTipForth"))+" ")]):n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("public.dataLoading"))+" ")])])])]),n("div",{staticClass:"cl-dashboard-con-up",class:e.histogramTag&&!e.wrongPlugin?"":"no-data-hover",on:{click:e.viewMoreHistogram}},[n("div",{staticClass:"cl-dashboard-title"},[e._v(e._s(e.$t("histogram.titleText")))]),n("div",{staticClass:"cl-module"},[n("div",{directives:[{name:"show",rawName:"v-show",value:!!e.histogramTag&&!e.wrongPlugin,expression:"!!histogramTag && !wrongPlugin"}],staticClass:"histogram-char-container"},[n("div",{attrs:{id:"distribution-chart"}}),n("div",{staticClass:"tag-text",attrs:{title:e.histogramTag}},[e._v(e._s(e.histogramTag))])]),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.histogramTag||e.wrongPlugin,expression:"!histogramTag || wrongPlugin"}],key:"no-chart-data",staticClass:"no-data-img"},[n("img",{attrs:{src:i("4936"),alt:""}}),e.initOverKey.histogram?n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("summaryManage.guidTipFirst"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.guidUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.guidTipSecond")))]),e._v(" "+e._s(e.$t("summaryManage.guidTipThird"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.histogramUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.histogramGuidTipFirst")))]),e._v(" "+e._s(e.$t("summaryManage.guidTipForth"))+" ")]):n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("public.dataLoading"))+" ")])])])]),n("div",{staticClass:"cl-dashboard-con-up",class:e.tensorTag&&!e.wrongPlugin?"":"no-data-hover",on:{mousedown:function(t){return e.viewMoreTensors(t)},mouseup:function(t){return e.viewMoreTensors(t)},click:function(t){return e.viewMoreTensors(t)}}},[n("div",{staticClass:"cl-dashboard-title"},[e._v(e._s(e.$t("tensors.titleText")))]),n("div",{staticClass:"cl-module"},[n("div",{directives:[{name:"show",rawName:"v-show",value:!!e.tensorTag&&!e.wrongPlugin,expression:"!!tensorTag && !wrongPlugin"}],staticClass:"tensor-char-container"},[n("div",{attrs:{id:"tensor-chart-container"}},[n("gridTableComponents",{ref:"tensorChart",attrs:{showOperate:!1,fullData:e.tensorData}})],1),n("div",{staticClass:"tag-text",attrs:{title:e.tensorTag}},[e._v(e._s(e.tensorTag))])]),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.tensorTag||e.wrongPlugin,expression:"!tensorTag || wrongPlugin"}],key:"no-chart-data",staticClass:"no-data-img"},[n("img",{attrs:{src:i("98c0"),alt:""}}),e.initOverKey.tensor?n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("summaryManage.guidTipFirst"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.guidUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.guidTipSecond")))]),e._v(" "+e._s(e.$t("summaryManage.guidTipThird"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.tensorUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.tensorTipFirst")))]),e._v(" "+e._s(e.$t("summaryManage.guidTipForth"))+" ")]):n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("public.dataLoading"))+" ")])])])]),n("div",{staticClass:"cl-dashboard-con-up",class:e.firstFloorNodes.length&&!e.wrongPlugin?"":"no-data-hover",on:{click:e.jumpToGraph}},[n("div",{staticClass:"cl-dashboard-title"},[e._v(" "+e._s(e.$t("trainingDashboard.calculationChart"))+" ")]),n("div",{staticClass:"cl-module"},[n("div",{directives:[{name:"show",rawName:"v-show",value:e.firstFloorNodes.length&&!e.wrongPlugin,expression:"firstFloorNodes.length && !wrongPlugin"}],staticClass:"graph",attrs:{id:"graph"}}),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.firstFloorNodes.length||e.wrongPlugin,expression:"!firstFloorNodes.length || wrongPlugin"}],staticClass:"no-data-img"},[n("img",{attrs:{src:i("56de"),alt:""}}),e.initOverKey.graph?n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("summaryManage.guidTipFirst"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.guidUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.guidTipSecond")))]),e._v(" "+e._s(e.$t("summaryManage.guidTipThird"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.graphUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.graphTipFirst")))]),e._v(" "+e._s(e.$t("summaryManage.graphTipSecond"))+" ")]):n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("public.dataLoading"))+" ")])])])]),n("div",{staticClass:"cl-dashboard-con-up",class:e.showDatasetGraph&&!e.wrongPlugin?"":"no-data-hover",on:{click:e.jumpToDataMap}},[n("div",{staticClass:"cl-dashboard-title"},[e._v(" "+e._s(e.$t("trainingDashboard.dataMap")))]),n("div",{staticClass:"cl-module"},[n("div",{directives:[{name:"show",rawName:"v-show",value:e.showDatasetGraph&&!e.wrongPlugin,expression:"showDatasetGraph && !wrongPlugin"}],staticClass:"graph",attrs:{id:"dataMapGraph"}}),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.showDatasetGraph||e.wrongPlugin,expression:"!showDatasetGraph || wrongPlugin"}],key:"no-chart-data",staticClass:"no-data-img"},[n("img",{attrs:{src:i("532e"),alt:""}}),e.initOverKey.dataMap?n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("summaryManage.guidTipFirst"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.guidUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.guidTipSecond")))]),e._v(" "+e._s(e.$t("summaryManage.guidTipThird"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.dataProcessUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.dataProcessTipFirst")))]),e._v(" "+e._s(e.$t("symbols.fullStop"))+" ")]):n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("public.dataLoading"))+" ")])])])]),n("div",{staticClass:"cl-dashboard-con-up",class:e.originImageDataArr.length&&!e.wrongPlugin?"":"no-data-hover",on:{click:function(t){return e.linkToImage(t)}}},[n("div",{staticClass:"cl-dashboard-title"},[n("div",{staticClass:"cl-dashboard-title-left"},[e._v(" "+e._s(e.$t("trainingDashboard.samplingData")))]),n("div",{staticClass:"cl-dashboard-title-right title-height"},[n("el-button",{attrs:{disabled:e.originImageDataArr.length<=1,type:"text"},on:{click:function(t){return e.changeImage(t)}}},[n("span",{staticClass:"el-icon-refresh"}),e._v(e._s(e.$t("trainingDashboard.imagesampleSwitch"))+" ")])],1)]),n("div",{staticClass:"cl-module"},[n("div",{staticClass:"image-container",class:e.originImageDataArr.length&&!e.wrongPlugin?"":"no-data-img"},[n("img",{directives:[{name:"show",rawName:"v-show",value:e.originImageDataArr.length&&!e.wrongPlugin,expression:"originImageDataArr.length && !wrongPlugin"}],staticClass:"sample-img select-disable",attrs:{src:e.curImageShowSample.curImgUrl}}),n("img",{directives:[{name:"show",rawName:"v-show",value:!e.originImageDataArr.length||e.wrongPlugin,expression:"!originImageDataArr.length || wrongPlugin"}],attrs:{src:i("daff"),alt:""}}),n("p",{directives:[{name:"show",rawName:"v-show",value:(!e.originImageDataArr.length||e.wrongPlugin)&&e.initOverKey.image,expression:"(!originImageDataArr.length || wrongPlugin) && initOverKey.image"}],staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("summaryManage.guidTipFirst"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.guidUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.guidTipSecond")))]),e._v(" "+e._s(e.$t("summaryManage.guidTipThird"))+" "),n("a",{staticClass:"link-text",attrs:{href:e.$t("summaryManage.imageUrl"),target:"_blank"}},[e._v(e._s(e.$t("summaryManage.imageTipFirst")))]),e._v(" "+e._s(e.$t("symbols.fullStop"))+" ")]),n("p",{directives:[{name:"show",rawName:"v-show",value:(!e.originImageDataArr.length||e.wrongPlugin)&&!e.initOverKey.image,expression:"(!originImageDataArr.length || wrongPlugin) && !initOverKey.image"}],staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("public.dataLoading"))+" ")])])])]),n("div",{staticClass:"cl-dashboard-con-up",class:e.show3dGraph&&!e.wrongPlugin?"":"no-data-hover",on:{click:e.jumpToLossAnalysis}},[n("div",{staticClass:"cl-dashboard-title"},[e._v(" "+e._s(e.$t("lossAnalysis.titleText")))]),n("div",{staticClass:"cl-module cover-container"},[n("div",{directives:[{name:"show",rawName:"v-show",value:e.show3dGraph&&!e.wrongPlugin,expression:"show3dGraph && !wrongPlugin"}],staticClass:"diagram-container"},[n("diagram3D",{ref:"diagram3D",attrs:{oriData:e.diagramOriDate,styleSetting:e.diagram3DSetting}})],1),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.show3dGraph||e.wrongPlugin,expression:"!show3dGraph || wrongPlugin"}],key:"no-chart-data",staticClass:"no-data-img"},[n("img",{attrs:{src:i("9485"),alt:""}}),n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("public.noData"))+" ")])]),n("div",{staticClass:"diagram-cover-item"})])]),n("div",{staticClass:"cl-dashboard-con-up",class:e.noOperatorData||e.wrongPlugin?"no-data-hover":"",on:{click:e.jumpOperatorPrecision}},[n("div",{staticClass:"cl-dashboard-title"},[e._v(" "+e._s(e.$t("operatorPrecision.title"))+" ")]),n("div",{directives:[{name:"show",rawName:"v-show",value:!e.noOperatorData,expression:"!noOperatorData"}],staticClass:"cl-operator-table"},[n("el-table",{staticStyle:{width:"100%"},attrs:{data:e.getTableData,"header-cell-style":e.headerStyle,"cell-style":e.cellStyle,"row-class-name":e.tableRowClassName}},[n("el-table-column",{attrs:{prop:"id",label:"ID",width:"55"}}),n("el-table-column",{attrs:{prop:"op_name",label:this.$t("operatorPrecision.opName"),"min-width":"10%"}}),n("el-table-column",{attrs:{prop:"type",label:this.$t("operatorPrecision.opType"),"min-width":"10%"}}),n("el-table-column",{attrs:{prop:"precision_flag",label:this.$t("operatorPrecision.precisionFlag"),"min-width":"12%"}}),n("el-table-column",{attrs:{label:this.$t("operatorPrecision.isReduceOp"),"min-width":"15%"},scopedSlots:e._u([{key:"default",fn:function(t){return[n("div",[e._v(e._s(t.row.reduce_flag))])]}}])})],1)],1),n("div",{directives:[{name:"show",rawName:"v-show",value:e.noOperatorData,expression:"noOperatorData"}],key:"no-chart-data",staticClass:"no-data-img"},[n("img",{attrs:{src:i("9485"),alt:""}}),n("p",{staticClass:"no-data-text"},[e._v(" "+e._s(e.$t("public.noData"))+" ")])])])])])},o=[],s=(i("99af"),i("4de4"),i("caad"),i("e260"),i("a15b"),i("d81d"),i("fb6a"),i("4e82"),i("a434"),i("b0c0"),i("e9c4"),i("a9e3"),i("f00c"),i("b680"),i("b64b"),i("d3b7"),i("ac1f"),i("00b4"),i("25f0"),i("2532"),i("3ca3"),i("5319"),i("159b"),i("ddb0"),i("9fac")),a=i("3778"),r=i("f756"),l=i("2dd7"),c=i("5698"),h=(i("6d21"),i("cd2f")),u=i("f633"),d={select:c["select"],selectAll:c["selectAll"],format:c["format"],precisionRound:c["precisionRound"]},p={data:function(){return{trainingJobId:"",summaryPath:"",defColorCount:l["a"].commonColorArr[this.$store.state.themeIndex].length,colorNum:0,charObj:null,histogramObj:null,histogramOption:{},histogramData:[],histogramOriData:{},charOption:{},charData:[],originImageDataArr:[],curImageShowSample:{},imageRandomLoopCount:0,imageBasePath:"v1/mindinsight/datavisual/image/single-image?",autoUpdateTimer:null,histogramTag:"",allGraphData:{},graphviz:null,firstFloorNodes:[],curColorIndex:0,totalMemory:33554432,curPageArr:[],allDatasetGraphData:{},showDatasetGraph:!1,datasetGraphviz:{},reloadStopTime:1e3,wrongPlugin:!1,fileTag:"",tensorData:[],tensorTag:"",tensorNewDataFlag:!0,tensorMouseData:{timeStamp:0,pageX:0,pageY:0,isClick:!1},mouseEventKey:{click:"click",mousedown:"mousedown",mouseup:"mouseup"},trainJobCached:0,cacheKey:{notInCache:"NOT_IN_CACHE",caching:"CACHING",cached:"CACHED"},initOverKey:{histogram:!1,image:!1,tensor:!1,scalar:!1,graph:!1,dataMap:!1},show3dGraph:!1,diagramOriDate:{},diagram3DSetting:{camera:JSON.parse(JSON.stringify(l["a"].lossCommonStyle.camera)),light:JSON.parse(JSON.stringify(l["a"].lossCommonStyle.light)),line:JSON.parse(JSON.stringify(l["a"].lossCommonStyle.line)),opacity:JSON.parse(JSON.stringify(l["a"].lossCommonStyle.opacity)),surface:{colorscale:JSON.parse(JSON.stringify(l["a"].lossColorscale[0]))}},themeIndex:this.$store.state.themeIndex,tableData:[],trainJobId:this.$route.query.id,noOperatorData:!0,operatorInfoFile:null}},computed:{isReload:function(){return this.$store.state.isReload},isTimeReload:function(){return this.$store.state.isTimeReload},timeReloadValue:function(){return this.$store.state.timeReloadValue},getTableData:function(){var e=this.tableData.slice(0,4);return e.push({id:"...",op_name:"...",name:"",type:"...",precision_flag:"",reduce_flag:""}),e}},components:{gridTableComponents:h["a"],diagram3D:u["a"]},watch:{isReload:function(e){var t=this;e&&(this.isTimeReload&&this.startAutoUpdate(),this.getDatavisualPlugins(!0),this.queryDatasetGraph(),this.queryTrainJobCacheState(),setTimeout((function(){t.$store.commit("setIsReload",!1)}),this.reloadStopTime))},isTimeReload:function(e){e?this.startAutoUpdate():this.stopAutoUpdate()},timeReloadValue:function(){this.isTimeReload&&this.startAutoUpdate()}},destroyed:function(){this.stopAutoUpdate(),window.removeEventListener("resize",this.resizeCallback,!1)},mounted:function(){var e=this;this.$nextTick((function(){window.addEventListener("resize",e.resizeCallback,!1),e.init()}))},methods:{queryAllGraphNodesData:function(){var e=this,t={train_id:this.$route.query.id,tag:this.operatorInfoFile,mode:"normal"};a["a"].queryAllGraphNodesData(t).then((function(t){if(t&&t.data){var i=t.data.all_nodes_detail,n=[];i.forEach((function(t,i){var o=t.name,s=o.split("/").length>1?o.split("/")[o.split("/").length-1]:"",a=t.type,r=JSON.stringify(t.input),l=JSON.stringify(t.output),c=t.attr.hasOwnProperty("precision_flag")?t.attr.precision_flag:"",h="",u=i+1;""!=c?(h=-1!=c.indexOf("reduce")?e.$t("operatorPrecision.yes"):e.$t("operatorPrecision.no"),n.unshift({id:u,op_name:s,name:o,type:a,precision_flag:c,reduce_flag:h,input:r,output:l})):n.push({id:u,op_name:s,name:o,type:a,precision_flag:c,reduce_flag:h,input:r,output:l})})),e.tableData=n,e.tableData.length>0&&(e.noOperatorData=!1)}else e.noOperatorData=!1}),(function(t){e.loading.show=!1,e.noOperatorData=!1})).catch((function(t){e.noOperatorData=!1}))},tableRowClassName:function(e){return e%2==0?"waring-row":"success-row"},cellStyle:function(e){e.row;var t=e.rowIndex;return this.tableData.length<5&&t==this.tableData.length||4==t?"font-weight: bold":void 0},headerStyle:function(){return{background:"0"==this.themeIndex?"#8df1f2":"#000",color:"#282B33"}},jumpOperatorPrecision:function(){this.$router.push({path:"/train-manage/OperatorPrecision",query:{train_id:this.trainingJobId,summaryPath:this.summaryPath}})},resizeCallback:function(){var e=this;this.charResizeTimer&&(clearTimeout(this.charResizeTimer),this.charResizeTimer=null),this.charResizeTimer=setTimeout((function(){e.charObj&&e.charObj.resize(),e.histogramObj&&e.histogramObj.resize();var t=e.$refs.tensorChart;t&&t.resizeView();var i=e.$refs.diagram3D;i&&i.resize()}),500)},init:function(){this.$route.query&&this.$route.query.id?(this.trainingJobId=this.$route.query.id,this.summaryPath=this.trainingJobId,document.title=this.summaryPath+"-"+this.$t("trainingDashboard.trainingDashboardTitle")+"-MindInsight"):(document.title=this.$t("trainingDashboard.trainingDashboardTitle")+"-MindInsight",this.trainingJobId="",this.$message.error(this.$t("trainingDashboard.invalidId"))),this.getDatavisualPlugins(!0),this.isTimeReload&&this.startAutoUpdate(),this.queryDatasetGraph(),this.queryTrainJobCacheState(),this.getEpochIntervals()},getEpochIntervals:function(){var e=this,t={train_id:this.trainingJobId};a["a"].queryEpochIntervals(t).then((function(t){if(t&&t.data){var i=t.data.intervals;if(i instanceof Array&&i.length){var n={train_id:e.trainingJobId,type:"interval",metadata:!0,interval_id:i[0].id};e.getDataOfLossGraph(n)}}else e.show3dGraph=!1}),(function(){e.diagramNewDataFlag=!0,e.show3dGraph=!1}))},getDataOfLossGraph:function(e){var t=this;a["a"].queryLossGraph(e).then((function(e){e&&e.data&&e.data.landscapes&&e.data.landscapes.length&&(t.show3dGraph=!0,t.diagramOriDate=JSON.parse(JSON.stringify(e.data.landscapes[0])),t.$nextTick((function(){var e=t.$refs.diagram3D;e&&(e.updateView(t.diagramNewDataFlag),t.diagramNewDataFlag=!1)})))}),(function(){t.diagramNewDataFlag=!0,t.show3dGraph=!1}))},getDatavisualPlugins:function(e){var t=this,i={train_id:this.trainingJobId,manual_update:e||!1};a["a"].getDatavisualPlugins(i).then((function(e){if(t.wrongPlugin=!1,e&&e.data&&e.data.plugins){var i=e.data.plugins,n=i.image||[],o=i.scalar||[],s=i.graph||[],a=i.tensor||[];s.length&&(t.operatorInfoFile=s[0],t.fileTag=s[0],t.queryAllGraphNodesData());var r=i.histogram||[];t.getHistogramTag(r),t.dealImageData(n),t.getScalarList(o),t.dealTensorData(a),!t.firstFloorNodes.length&&s.length?t.queryGraphData():t.initOverKey.graph=!0}else t.initOverKey={histogram:!0,image:!0,tensor:!0,scalar:!0,graph:!0,dataMap:!0}})).catch((function(e){t.initOverKey={histogram:!0,image:!0,tensor:!0,scalar:!0,graph:!0,dataMap:!0},e.response&&e.response.data&&e.response.data.error_code&&"50545005"===e.response.data.error_code.toString()&&(t.wrongPlugin=!0)}))},viewMoreScalars:function(){this.curPageArr.length&&this.$router.push({path:"/train-manage/scalar",query:{train_id:this.trainingJobId,summaryPath:this.summaryPath}})},viewMoreHistogram:function(){this.histogramTag&&this.$router.push({path:"/train-manage/histogram",query:{train_id:this.trainingJobId,summaryPath:this.summaryPath}})},viewMoreTensors:function(e){if(this.tensorTag)if(e.type===this.mouseEventKey.mousedown)this.tensorMouseData.isClick=!1,this.tensorMouseData.pageX=e.pageX,this.tensorMouseData.pageY=e.pageY;else if(e.type===this.mouseEventKey.mouseup){var t=3;Math.abs(e.pageX-this.tensorMouseData.pageX)<=t&&Math.abs(e.pageY-this.tensorMouseData.pageY)<=t&&(this.tensorMouseData.isClick=!0,this.tensorMouseData.timeStamp=e.timeStamp)}else e.type===this.mouseEventKey.click&&this.tensorMouseData.isClick&&e.timeStamp===this.tensorMouseData.timeStamp&&this.$router.push({path:"/train-manage/tensor",query:{train_id:this.trainingJobId,summaryPath:this.summaryPath}})},jumpToDataMap:function(){Object.keys(this.allDatasetGraphData).length&&this.$router.push({path:"/train-manage/data-map",query:{train_id:this.trainingJobId,summaryPath:this.summaryPath}})},jumpToLossAnalysis:function(){this.show3dGraph&&!this.wrongPlugin&&this.$router.push({path:"/train-manage/loss-analysis",query:{train_id:this.trainingJobId}})},jumpToGraph:function(){this.firstFloorNodes.length&&this.$router.push({path:"/train-manage/graph",query:{train_id:this.trainingJobId,summaryPath:this.summaryPath}})},startAutoUpdate:function(){var e=this;this.autoUpdateTimer&&(clearInterval(this.autoUpdateTimer),this.autoUpdateTimer=null),this.autoUpdateTimer=setInterval((function(){e.$store.commit("clearToken"),e.getDatavisualPlugins(),Object.keys(e.allDatasetGraphData).length||e.queryDatasetGraph(),e.queryTrainJobCacheState()}),1e3*this.timeReloadValue)},stopAutoUpdate:function(){this.autoUpdateTimer&&(clearInterval(this.autoUpdateTimer),this.autoUpdateTimer=null)},getScalarList:function(e){var t=this;if(!e.length)return this.colorNum=0,this.charObj=null,this.charOption={},this.charData=[],this.curPageArr=[],void(this.initOverKey.scalar=!0);var i={plugin_name:"scalar",train_id:this.trainingJobId};a["a"].getSingleTrainJob(i,!0).then((function(e){if(e&&e.data&&e.data.train_jobs&&e.data.train_jobs.length)if(e.data&&e.data.error_code)t.$message.error("".concat(e.data.error_code," : ").concat(t.$t("error")[e.data.error_code]));else{var i=[],n=[],o=e.data.train_jobs;o.forEach((function(e,t){n.push({id:e.id,label:e.name}),e.tags.forEach((function(t){var n=-1;if(i.some((function(e,i){if(e.tagName===t)return n=i,!0})),-1===n)i.push({tagName:t,runNames:[e.name],runId:[e.id],colors:[]});else{var o=i[n];o.runNames.push(e.name),o.runId.push(e.id)}}))})),t.runList=n,i.length&&(i=i.slice(0,2),t.curPageArr=i,t.updateTagInPage())}else t.initOverKey.scalar=!0}),this.errorScalar).catch((function(e){t.initOverKey.scalar=!0}))},updateTagInPage:function(){var e=this,t=[];this.curPageArr.forEach((function(i,n){var o=i.runId.length;if(0!==o){var s={train_id:i.runId[0],tag:i.tagName};t.push(e.addAjax(s,n))}})),Promise.all(t).then((function(t){if(t){e.charData=[];for(var i,n=function(){if(!t[o]||!t[o].data)return{v:void 0};var i=t[o].data,n={valueData:{stepData:[],absData:[],relativeData:[]},yAxisIndex:t[o].yIndex,color:l["a"].commonColorArr[e.themeIndex][e.colorNum]?l["a"].commonColorArr[e.themeIndex][e.colorNum]:l["a"].commonColorArr[e.themeIndex][e.defColorCount-1],tagName:t[o].params.tag},s=0;i.metadatas.length&&(s=i.metadatas[0].wall_time),i.metadatas.forEach((function(e){n.valueData.stepData.push([e.step,e.value]),n.valueData.absData.push([e.wall_time,e.value]),n.valueData.relativeData.push([e.wall_time-s,e.value])})),e.colorNum++,e.charData.push(n)},o=0;o<t.length;o++)if(i=n(),i)return i.v;e.charOption=e.formateCharOption(),e.colorNum=0,e.updateOrCreateChar()}}))},formateSmoothData:function(e){if(!e||e.length<2)return e;for(var t=JSON.parse(JSON.stringify(e)),i=e.length,n=0,o=0,s=0,a=t[0][1],r=t.every((function(e){return e[1]===a})),l=0;l<i;l++){var c=t[l][1];if(!r&&Number.isFinite(c)){n=n*o+(1-o)*c,s++;var h=1;1!==o&&(h=1-Math.pow(o,s)),t[l][1]=n/h}}return t},updateOrCreateChar:function(e){this.charObj?this.charObj.setOption(this.charOption,!0):document.getElementById("module-chart")&&(this.charObj=s["a"].init(document.getElementById("module-chart"),s["b"]),this.charObj.setOption(this.charOption,!0))},formateCharOption:function(){var e=this,t=[],i=[],n=[];this.curPageArr.forEach((function(e){var t={type:"value",scale:!0,axisLabel:{formatter:function(e){var t=Math.abs(e);return t.toString().length>6?e.toExponential(0):e>=1e3||e<=-1e3?parseFloat((e/1e3).toFixed(2))+"k":e>0?e:parseFloat(e.toFixed(3))}}};n.push(t)})),this.charData.forEach((function(n,o){var s={name:n.tagName,data:[],type:"line",showSymbol:!1,lineStyle:{color:n.color},color:n.color,yAxisIndex:n.yAxisIndex};s.data=e.formateSmoothData(n.valueData["stepData"]),t.push(s),i.push(s.name)}));var o={legend:{data:i,selectedMode:!1,icon:"circle",bottom:0},grid:{top:20,bottom:66,left:60,right:60},xAxis:[{type:"value",show:!0,scale:!0,nameGap:30,minInterval:1,axisLabel:{interval:0,formatter:function(e){var t=Math.abs(e);return t.toString().length>6?e.toExponential(0):e>=1e3||e<=-1e3?parseFloat((e/1e3).toFixed(2))+"k":e>0?e:parseFloat(e.toFixed(3))}}}],yAxis:n,animation:!0,series:t};return o},addAjax:function(e,t){return new Promise((function(i,n){a["a"].getScalarsSample(e).then((function(n){n&&(n.params=e,n.yIndex=t,i(n))})).catch((function(e){e&&n(e)}))}))},dealImageData:function(e){var t=this;if(!e.length)return this.curImageShowSample={},this.originImageDataArr=[],void(this.initOverKey.image=!0);if("{}"===JSON.stringify(this.curImageShowSample)||-1===e.indexOf(this.curImageShowSample.tagName)){var i=[];e.forEach((function(e){var n={runId:t.trainingJobId,tagName:e,sampleData:[],curImgUrl:""};i.push(n)})),this.originImageDataArr=i,this.getSampleRandomly()}else this.updateImageSample()},dealTensorData:function(e){var t=this.tensorTag;e.length?this.tensorTag=e[0]:this.tensorTag="",this.tensorTag!==t?this.tensorNewDataFlag=!0:this.tensorNewDataFlag=!1,this.tensorTag?this.getTensorGridData():this.initOverKey.tensor=!0},getTensorGridData:function(){var e=this,t={train_id:this.trainingJobId,tag:this.tensorTag,detail:"stats"};a["a"].getTensorsSample(t).then((function(t){if(e.initOverKey.tensor=!0,t&&t.data&&t.data.tensors.length){var i=JSON.parse(JSON.stringify(t.data.tensors[0]));if(!i.values.length)return e.tensorData=[],void e.$nextTick((function(){var t=e.$refs.tensorChart;t&&t.updateGridData()}));var n=i.values[i.values.length-1],o=e.initFilterStr(n.value.dims);e.freshtMartixData(n.step,o)}}),(function(){e.tensorData=[],e.initOverKey.tensor=!0,e.$nextTick((function(){var t=e.$refs.tensorChart;t&&t.updateGridData()}))}))},initFilterStr:function(e){if(!e)return[];for(var t=e.length-2,i=[],n=1e3,o=0;o<e.length;o++)i.push(o>=t?":":"0");if(i.length){var s=i.length-1,a=i[s];a&&e[s]>n&&(i[s]="0:".concat(n))}return"[".concat(i.toString(),"]")},freshtMartixData:function(e,t){var i=this,n={train_id:this.trainingJobId,tag:this.tensorTag,detail:"data",step:e,dims:t};a["a"].getTensorsSample(n).then((function(e){if(e&&e.data&&e.data.tensors.length){var t=e.data.tensors[0],n=t.values[0],o={};n?(i.tensorData=n.value.data instanceof Array?n.value.data:[n.value.data],o=n.value.statistics):i.tensorData=[[]],i.$nextTick((function(){var e=i.$refs.tensorChart;e&&e.updateGridData(i.tensorNewDataFlag,n.value.dims,o)}))}}),(function(e){i.tensorData=[];var t=!1,n="";e.response&&e.response.data&&e.response.data.error_code&&("50545013"===e.response.data.error_code.toString()||"50545014"===e.response.data.error_code.toString()||"50545016"===e.response.data.error_code.toString())&&(t=!0,n="50545013"===e.response.data.error_code.toString()?i.$t("tensors.tensorDashboardLimitErrorMsg"):i.$t("error")[e.response.data.error_code]),i.$nextTick((function(){var e=i.$refs.tensorChart;e&&(t?e.showRequestErrorMessage(n):e.updateGridData())}))}))},getHistogramTag:function(e){var t=this;if(e){var i="";if(i=this.histogramTag&&-1!==e.indexOf(this.histogramTag)?this.histogramTag:e[0]||"",!i)return this.histogramTag=i,void(this.initOverKey.histogram=!0);var n={train_id:this.trainingJobId,tag:i};a["a"].getHistogramData(n).then((function(e){if(t.initOverKey.histogram=!0,e&&e.data&&e.data.histograms&&e.data.histograms.length){var n=JSON.parse(JSON.stringify(e.data));t.histogramTag=i,t.histogramData=t.formOriData(n),t.formatDataToChar(),t.updateHistogramSampleData()}}),(function(e){t.initOverKey.histogram=!0,t.histogramTag=""}))}else this.initOverKey.histogram=!0},formOriData:function(e){var t=[];return e.histograms.forEach((function(e,i){var n=e.step.toString(),o={wall_time:e.wall_time,step:n,items:[]},s=[];if(e.buckets.forEach((function(t){var i=t[0]+t[1]/2,o=s.filter((function(e){return e[0]===i}));o.length||s.push([e.wall_time,n,i,Math.floor(t[2])])})),s.sort((function(e,t){return e[0]-t[0]})),s.length){var a=s[0][2],r=s[s.length-1][2],l=[[e.wall_time,n,a,0]].concat(s,[[e.wall_time,n,r,0]]);o.items=l,t.push(o)}})),t},formatDataToChar:function(){var e=this.histogramData,t=[],i=-1/0,n=1/0,o=-1/0,s=1/0,a=[];e&&e.length&&e.forEach((function(e){var r=[];a.push(e.step),e.items.forEach((function(t){r.push(t[2],e.step,t[3]),i=Math.max(i,t[2]),n=Math.min(n,t[2]),s=Math.min(s,t[3]),o=Math.max(o,t[3])})),t.push(r)})),this.histogramOriData={seriesData:t,maxX:i,minX:n,maxZ:o,minZ:s,gridData:a}},formatCharOption:function(){var e=this,t=l["a"].histogramThemes[this.themeIndex],i=l["a"].commonChartTheme[this.themeIndex],n=t.lineMinColor,o=t.lineMaxColor,s=this.histogramOriData,a=this.getGradientColor(n,o,s.seriesData.length),r=[];s.seriesData.forEach((function(e,t){var i={name:e[1],value:e,itemStyle:{color:a[t]}};r.push(i)}));var c={grid:{left:24,top:120,right:40,bottom:43},xAxis:{max:s.maxX,min:s.minX,axisLine:{onZero:!1,lineStyle:{color:i.axisTextColor}},axisLabel:{fontSize:"11",formatter:function(e){return e.toString().length>=6?e.toExponential(3):Math.round(1e3*e)/1e3}},splitLine:{show:!1}},yAxis:{type:"category",position:"right",axisLine:{onZero:!1,show:!1},splitLine:{show:!0},inverse:!0,axisTick:{show:!1},boundaryGap:!1,axisLabel:{fontSize:"11"}},series:[{type:"custom",dimensions:["x","y"],renderItem:function(i,n){var o=e.makePolyPoints(i.dataIndex,n.coord,i.coordSys.y-10);return{type:"polyline",z2:i.dataIndex,silent:!0,shape:{points:o},style:n.style({stroke:t.polylineBorderColor,lineWidth:1})}},data:r}]};return c},formatColor:function(e){if(e){var t=/^([0-9a-fA-F]{3}|[0-9a-fA-F]{6})$/,i=e.toLowerCase().slice(1);if(t.test(i)){var n="";if(3===i.length){for(var o=0;o<3;o++)n+=n.slice(o,o+1).concat(n.slice(o,o+1));i=n}for(var s=[],a=0;a<6;a+=2)s.push(parseInt("0x".concat(i.slice(a,a+2))));return s}return i}},formatColorToHex:function(e){var t=/^(rgb|RGB)/g;if(t.test(e)){for(var i=e.replace(/(?:(|)|rgb|RGB)*/g,"").split(","),n="",o=0;o<i.length;o++){var s=Number(i[o]).toString(16);s=s<10?"0".concat(s):s,"0"===s&&(s+=s),n+=s}return 6!==n.length&&(n=e),n}},getGradientColor:function(e,t,i){for(var n=this.formatColor(e),o=this.formatColor(t),s=(o[0]-n[0])/i,a=(o[1]-n[1])/i,r=(o[2]-n[2])/i,l=[],c=0;c<i;c++){var h=parseInt(s*c+n[0]),u=parseInt(a*c+n[1]),d=parseInt(r*c+n[2]),p=this.formatColorToHex("rgb(".concat(h,",").concat(u,",").concat(d,")"));l.push(p)}return l},updateHistogramSampleData:function(){var e=this;this.histogramOption=this.formatCharOption(),setTimeout((function(){e.histogramObj||(e.histogramObj=s["a"].init(document.getElementById("distribution-chart"),s["b"])),e.histogramObj.setOption(e.histogramOption,!0)}),100)},getValue:function(e,t,i){return e[t][i]},makePolyPoints:function(e,t,i){for(var n=[],o=this.histogramOriData.seriesData,s=this.histogramOriData.maxZ,a=this.histogramOriData.minZ,r=0;r<o[e].length;){var l=this.getValue(o,e,r++),c=this.getValue(o,e,r++),h=this.getValue(o,e,r++),u=t([l,c]);u[1]-=(h-a)/(s-a)*i,n.push(u)}return n},errorScalar:function(e){e.response&&e.response.data?this.scalarClearAllData():"ECONNABORTED"===e.code&&/^timeout/.test(e.message)||this.scalarClearAllData()},scalarClearAllData:function(){this.colorNum=0,this.charObj=null,this.curPageArr=[],this.charOption={},this.charData=[]},getSampleRandomly:function(){if(this.originImageDataArr.length)if(this.originImageDataArr.length>1){var e=Math.floor(Math.random()*this.originImageDataArr.length),t=this.originImageDataArr[e];if(!(t.tagName!==this.curImageShowSample.tagName||this.imageRandomLoopCount>=15))return this.imageRandomLoopCount++,void this.getSampleRandomly();this.curImageShowSample=t,this.imageRandomLoopCount=0}else this.imageRandomLoopCount=0,this.curImageShowSample=this.originImageDataArr[0];else this.imageRandomLoopCount=0,this.curImageShowSample={};this.updateImageSample()},updateImageSample:function(){var e=this;if("{}"!==JSON.stringify(this.curImageShowSample)){var t=this.curImageShowSample,i={train_id:t.runId,tag:t.tagName};a["a"].getImageMetadatas(i).then((function(i){if(e.initOverKey.image=!0,i&&i.data&&i.data.metadatas&&t.runId===e.curImageShowSample.runId&&t.tagName===e.curImageShowSample.tagName){var n=i.data.metadatas;if(t.sampleData=n,t.sampleData.length){var o=t.sampleData.length-1,s=t.sampleData[o].wall_time;t.curImgUrl="".concat(r["a"]).concat(e.imageBasePath)+"train_id=".concat(Object(r["c"])(t.runId),"&tag=").concat(Object(r["c"])(t.tagName))+"&step=-1&wt=".concat(s)}else e.curImageShowSample={}}}),(function(t){e.initOverKey.image=!0,e.curImageShowSample={}})).catch((function(t){e.$message.error(e.$t("public.dataError"))}))}else this.initOverKey.image=!0},linkToImage:function(e){if(this.originImageDataArr.length){if(1===this.originImageDataArr.length){var t=e.path||e.composedPath&&e.composedPath(),i=t.some((function(e){if(e.className)return-1!==e.className.indexOf("el-button")}));if(i)return}this.$router.push({path:"/train-manage/image",query:{train_id:this.trainingJobId,summaryPath:this.summaryPath}})}},changeImage:function(e){e.stopPropagation(),e.preventDefault(),this.getSampleRandomly()},initGraph:function(e){var t=this;this.graphviz=d.select("#graph").graphviz({useWorker:!1,totalMemory:this.totalMemory}).dot(e).attributer(this.attributer).render((function(){t.initOverKey.graph=!0,d.select("#graph svg")&&d.select("#graph svg").on(".zoom",null),d.selectAll("#graph title").remove(),setTimeout((function(){t.graphviz._data=null,t.graphviz._dictionary=null,t.graphviz=null}),200)}))},queryGraphData:function(){var e=this,t={train_id:this.trainingJobId,type:"name_scope",tag:this.fileTag};a["a"].queryGraphData(t).then((function(t){if(t&&t.data&&t.data.nodes&&t.data.nodes.length){var i=t.data.nodes;e.packageDataToObject(i);var n=e.packageGraphData();e.initGraph(n)}else e.initOverKey.graph=!0}),(function(t){e.initOverKey.graph=!0})).catch((function(t){e.initOverKey.graph=!0}))},packageDataToObject:function(e){var t=this;this.allGraphData={},this.firstFloorNodes=[],e.forEach((function(e){t.allGraphData[e.name]=e,t.firstFloorNodes.push(e.name)}))},getChildNodesByName:function(){var e=this,t=this.firstFloorNodes,i=t.map((function(t){return e.allGraphData[t]}));return i},packageGraphData:function(){var e='node[style="filled";fontsize="10px"];edge[fontsize="6px";];';return"digraph {".concat(e).concat(this.packageNodes()).concat(this.packageEdges(),"}")},packageNodes:function(){var e=this,t=this.getChildNodesByName(),i="";return t.forEach((function(t){var n=t.name.split("/").pop();if("polymeric_scope"===t.type)i+="<".concat(t.name,'>[id="').concat(t.name,'";shape="octagon";')+'label="'.concat(n,'";class="polymeric"];');else if("name_scope"===t.type){var o=l["a"].graphColorArr[e.$store.state.themeIndex][e.curColorIndex];e.curColorIndex=e.curColorIndex%4,e.curColorIndex++,i+="<".concat(t.name,'>[id="').concat(t.name,'";fillcolor="').concat(o,'";')+'shape="polygon";label="'.concat(n,'";class="cluster"];')}else"Const"===t.type?i+="<".concat(t.name,'>[id="').concat(t.name,'";label="').concat(n,'\n\n\n";')+'shape="circle";width="0.14";height="0.14";fixedsize=true;];':i+="<".concat(t.name,'>[id="').concat(t.name,'";shape="ellipse";')+'label="'.concat(n,'";];');for(var s=Object.keys(t.polymeric_input||{}),a=t.name,r="",c=!1,h=0;h<Math.min(5,s.length);h++){r=s[h],c=!(!e.allGraphData[s[h]]||"Const"!==e.allGraphData[s[h]].type);var u=c?'shape="circle";width="0.14";height="0.14";fixedsize=true;'+'label="'.concat(r.split("/").pop(),'\n\n\n";'):'shape="Mrecord";label="'.concat(r.split("/").pop(),'";');i+="<".concat(r,"^").concat(a,'>[id="').concat(r,"^").concat(a,'";')+"".concat(u,'class="plain"];')}s.length>5&&(i+="<analogNodesInput^".concat(a,'>[id="analogNodesInput^')+"".concat(a,'";label="').concat(s.length-5,' more...";shape="Mrecord";')+'class="plain";];'),s=Object.keys(t.polymeric_output||{}),r=t.name;for(var d=0;d<Math.min(5,s.length);d++){a=s[d];var p=c?'shape="circle";width="0.14";height="0.14";fixedsize=true;'+'label="'.concat(a.split("/").pop(),'\n\n\n";'):'shape="Mrecord";label="'.concat(a.split("/").pop(),'";');i+="<".concat(a,"^").concat(r,'>[id="').concat(a,"^").concat(r,'";')+"".concat(p,'class="plain";];')}s.length>5&&(i+="<analogNodesOutput^".concat(r,'>[id="analogNodesOutput^')+"".concat(r,'";shape="Mrecord";label="').concat(s.length-5,' more...";')+'class="plain";];')})),i},packageEdges:function(){var e=this,t=this.getChildNodesByName(),i="",n=[];return t.forEach((function(e){var t=e.input||{},i=Object.keys(t);i.forEach((function(i){if(t[i]){var o="polymeric_scope"===t[i].scope?i.substring(0,i.lastIndexOf("/")):i,s=o.split("/")[0],a="polymeric_scope"===e.type||e.polymeric_scope_name?"".concat(e.name.substring(0,e.name.lastIndexOf("/"))):e.name;if(s&&a&&!a.includes(s)&&!s.includes(a)){var r={source:s,target:a,shape:t[i].shape,edge_type:t[i].edge_type,count:1};n.push(r)}}}));for(var o=Object.keys(e.polymeric_input||{}),s=0;s<Math.min(5,o.length);s++){var a=e.name,r=o[s],l={source:"".concat(r,"^").concat(a),target:a,shape:[],edge_type:"polymeric_scope"===e.type||e.polymeric_scope_name?"":"polymeric",count:1};n.push(l)}if(o.length>5){var c={source:"analogNodesInput^".concat(e.name),target:e.name,shape:[],edge_type:"polymeric_scope"===e.type||e.polymeric_scope_name?"":"polymeric",count:1};n.push(c)}o=Object.keys(e.polymeric_output||{});for(var h=0;h<Math.min(5,o.length);h++){var u=e.name,d=o[h],p={source:u,target:"".concat(d,"^").concat(u),shape:[],edge_type:"polymeric_scope"===e.type||e.polymeric_scope_name?"":"polymeric",count:1};n.push(p)}if(o.length>5){var f={source:e.name,target:"analogNodesOutput^".concat(e.name),shape:[],edge_type:"polymeric_scope"===e.type||e.polymeric_scope_name?"":"polymeric",count:1};n.push(f)}})),this.uniqueEdges(n),n.forEach((function(t){i+="<".concat(t.source,">-><").concat(t.target,'>[id="').concat(t.source,"->")+"".concat(t.target,'";label="').concat(e.getEdgeLabel(t),'";')+"".concat("control"===t.edge_type?"style=dashed":"","];")})),i},uniqueEdges:function(e){for(var t=0;t<e.length-1;t++)for(var i=t+1;i<e.length;i++){var n=e[t].source===e[i].source&&e[t].target===e[i].target&&e[t].edge_type===e[i].edge_type;n&&(e[t].count+=e[i].count,e.splice(i--,1))}},getEdgeLabel:function(e){if("control"===e.edge_type)return"";var t="";if(e.count&&1!==e.count)t="".concat(e.count,"tensors");else if(e.shape&&e.shape.length){var i=e.shape.some((function(e){return"number"!==typeof e}));t=i?"tuple(".concat(e.shape.length," items)"):e.shape.join("x")}return t},attributer:function(e,t,i){var n="ellipse"===e.tag||"circle"===e.tag||"polygon"===e.tag&&"transparent"!==e.attributes.stroke;if("svg"===e.tag){var o="100%",s="100%";e.attributes.width=o,e.attributes.height=s}else n&&(e.attributes.stroke="rgb(120, 120, 120)")},queryDatasetGraph:function(){var e=this,t={train_id:this.trainingJobId};a["a"].queryDatasetGraph(t).then((function(t){if(t&&t.data&&t.data.dataset_graph&&Object.keys(t.data.dataset_graph).length){var i=JSON.parse(JSON.stringify(t.data.dataset_graph));if(e.dealDatasetGraph(i),Object.keys(e.allDatasetGraphData).length){var n=e.packageDatasetGraph();e.initDatasetGraph(n)}}else e.initOverKey.dataMap=!0}),(function(t){e.initOverKey.dataMap=!0})).catch((function(t){e.initOverKey.dataMap=!0}))},dealDatasetGraph:function(e){var t=this,i=arguments.length>1&&void 0!==arguments[1]?arguments[1]:"",n=arguments.length>2&&void 0!==arguments[2]?arguments[2]:0;if(e){var o="".concat(i?i+"/":"").concat(e.op_type||"","_").concat(n),s={key:o,id:""};Object.keys(e).forEach((function(i){"children"!==i?s[i]=JSON.parse(JSON.stringify(e[i])):(s.children=[],e.children&&e.children.length&&e.children.forEach((function(e,i){s.children.push("".concat(s.key,"/").concat(e.op_type,"_").concat(i)),t.dealDatasetGraph(e,s.key,i)})))})),this.allDatasetGraphData[o]=s}},packageDatasetGraph:function(){var e=this,t=["BatchDataset","ShuffleDataset","RepeatDataset","MapDataset","Batch","Shuffle","Repeat","Map"],i=["Map","MapDataset"],n="",o="";Object.keys(this.allDatasetGraphData).forEach((function(o){var s=e.allDatasetGraphData[o];i.includes(s.op_type)?n+=e.packageSubGraph(o):(s.id=o,n+="<".concat(s.key,'>[id="').concat(s.key,'";label="').concat(s.op_type,'";')+"class=".concat(t.includes(s.op_type)?s.op_type.replace("Dataset",""):"Create",';shape=rect;fillcolor="#9cc3e5";];'))})),Object.keys(this.allDatasetGraphData).forEach((function(t){var n=e.allDatasetGraphData[t];n.children.forEach((function(t){var s=e.allDatasetGraphData[t];o+="<".concat(s.id,">-><").concat(n.id,">[").concat(i.includes(s.op_type)?"ltail=<cluster_".concat(s.key,">;"):"").concat(i.includes(n.op_type)?"lhead=<cluster_".concat(n.key,">;"):"","];")}))}));var s='node[style="filled";fontsize="10px"];edge[fontsize="6px";];';return"digraph {compound=true;rankdir=LR;".concat(s).concat(n).concat(o,"}")},packageSubGraph:function(e){var t=this.allDatasetGraphData[e],i="";if(t.operations&&t.operations.length){var n="";t.operations.forEach((function(e,i){var o="".concat(t.key,"/").concat(e.tensor_op_name,"_").concat(i);e.key=o,n+="<".concat(o,'>[id="').concat(o,'";class=Operator;label="').concat(e.tensor_op_name,'";fillcolor="#c5e0b3"];'),t.id||(t.id=o)})),i+="subgraph <cluster_".concat(e,'>{style="filled";id="').concat(e,'";')+'label="'.concat(t.op_type,'";fillcolor="#9cc3e5";').concat(n,"};")}return i},initDatasetGraph:function(e){this.datasetGraphviz=d.select("#dataMapGraph").graphviz({useWorker:!1,totalMemory:this.totalMemory}).dot(e).attributer((function(e,t,i){if("svg"===e.tag){var n="100%",o="100%";e.attributes.width=n,e.attributes.height=o}})).render(this.afterinitDatasetGraph)},afterinitDatasetGraph:function(){var e=this;this.showDatasetGraph=!0,this.initOverKey.dataMap=!0,d.select("#dataMapGraph svg")&&d.select("#dataMapGraph svg").on(".zoom",null),setTimeout((function(){e.datasetGraphviz&&(e.datasetGraphviz._data=null,e.datasetGraphviz._dictionary=null,e.datasetGraphviz=null)}),100),d.select("#dataMapGraph").selectAll("title").remove()},queryTrainJobCacheState:function(){var e=this,t={train_id:this.trainingJobId};a["a"].querySummaryList(t,!0).then((function(t){if(t&&t.data&&t.data.train_jobs&&t.data.train_jobs.length){var i=t.data.train_jobs[0],n=i.graph_files||i.summary_files;i.is_integrity?n&&i.cache_status===e.cacheKey.notInCache?e.trainJobCached=1:n&&i.cache_status===e.cacheKey.caching?e.trainJobCached=2:e.trainJobCached=0:e.$message({showClose:!0,message:e.$t("trainingDashboard.integrityInfo"),type:"warning",duration:5e3})}else e.trainJobCached=0}))}}},f=p,g=(i("32c7"),i("2877")),m=Object(g["a"])(f,n,o,!1,null,null,null);t["default"]=m.exports},cd2f:function(e,t,i){"use strict";var n=function(){var e=this,t=e.$createElement,i=e._self._c||t;return i("div",{staticClass:"cl-slickgrid-container"},[i("div",{staticClass:"data-show-container"},[i("div",{directives:[{name:"show",rawName:"v-show",value:e.incorrectData,expression:"incorrectData"}],staticClass:"error-msg-container"},[e._v(" "+e._s(e.$t("components.gridIncorrectDataError"))+" ")]),i("div",{directives:[{name:"show",rawName:"v-show",value:!e.incorrectData&&e.requestError,expression:"!incorrectData && requestError"}],staticClass:"error-msg-container"},[e._v(" "+e._s(e.errorMsg)+" ")]),i("div",{directives:[{name:"show",rawName:"v-show",value:!e.fullData.length&&e.updated&&!e.incorrectData&&!e.requestError,expression:"!fullData.length && updated && !incorrectData && !requestError"}],staticClass:"error-msg-container"},[e._v(" "+e._s(e.$t("components.gridTableNoData"))+" ")]),i("div",{directives:[{name:"show",rawName:"v-show",value:!!e.fullData.length&&!e.incorrectData,expression:"!!fullData.length && !incorrectData"}],staticClass:"grid-item",attrs:{id:e.itemId}})]),e.showOperate&&(e.fullData.length||e.requestError)?i("div",{staticClass:"operate-container"},[i("div",{staticClass:"filter-container",on:{keyup:function(t){return!t.type.indexOf("key")&&e._k(t.keyCode,"enter",13,t.key,"Enter")?null:e.filterChange.apply(null,arguments)}}},[i("div",{staticClass:"filter-input-title"},[e._v(e._s(e.$t("components.dimsFilterInputTitle"))+" "),i("span",{staticClass:"el-icon-info",attrs:{title:e.$t("components.dimsFilterInputTip")}})]),e._l(e.filterArr,(function(t,n){return i("div",{key:n},[i("el-input",{staticClass:"filter-input",class:[t.showError?"error-border":"",e.fullScreen?"long-input":"short-input"],model:{value:t.model,callback:function(i){e.$set(t,"model",i)},expression:"item.model"}}),n===e.filterArr.length-1?i("span",{staticClass:"input-behind"},[e._v(e._s(e.$t("symbols.slashes")))]):i("span",{staticClass:"input-behind"},[e._v(e._s(e.$t("symbols.point")))])],1)})),e.filterArr.length?i("el-button",{staticClass:"filter-check",attrs:{size:"mini"},on:{click:e.filterChange}},[i("i",{staticClass:"el-icon-check"})]):e._e(),e.filterCorrect?e._e():i("span",{staticClass:"filter-incorrect-text"},[e._v(e._s(e.$t("components.inCorrectInput")))])],2),i("div",{staticClass:"accuracy-container"},[e._v(" "+e._s(e.$t("components.category"))),i("el-select",{staticClass:"select-category",on:{change:e.accuracyChange},model:{value:e.category,callback:function(t){e.category=t},expression:"category"}},e._l(e.categoryArr,(function(e){return i("el-option",{key:e.label,attrs:{label:e.label,value:e.value}})})),1),e._v(" "+e._s(e.$t("components.gridAccuracy"))+" "),i("span",{staticClass:"el-icon-info",attrs:{title:e.$t("components.accuracyTips")}}),i("el-select",{staticClass:"select-item",on:{change:e.accuracyChange},model:{value:e.accuracy,callback:function(t){e.accuracy=t},expression:"accuracy"}},e._l(e.accuracyArr,(function(e){return i("el-option",{key:e.label,attrs:{label:e.label,value:e.value}})})),1),i("div",{staticClass:"full-screen-icon",class:e.fullScreen?"active-color":"",attrs:{title:e.$t("scalar.fullScreen")},on:{click:e.toggleFullScreen}},[i("i",{staticClass:"el-icon-full-screen"})])],1)]):e._e()])},o=[],s=(i("99af"),i("fb6a"),i("a9e3"),i("b680"),i("d3b7"),i("498a"),i("159b"),i("326d"),i("f411"),i("dc1b"),i("f06f"),i("ed3a"),i("7d72"),i("70a2"),i("3a3f"),i("2dd7")),a={props:{fullData:{type:Array,default:function(){return[]}},showOperate:{type:Boolean,default:!0},fullScreen:{type:Boolean,default:!1},columnLimitNum:{type:Number,default:-1}},data:function(){return{itemId:"",gridObj:null,columnsData:[],columnsLength:0,filterArr:[],formateData:[],formateArr:[],statistics:{},accuracy:5,incorrectData:!1,updated:!1,scrollTop:!1,filterCorrect:!0,requestError:!1,errorMsg:"",viewResizeFlag:!1,accuracyArr:[{label:0,value:0},{label:1,value:1},{label:2,value:2},{label:3,value:3},{label:4,value:4},{label:5,value:5}],optionObj:{enableColumnReorder:!1,enableCellNavigation:!0,frozenColumn:0,frozenRow:0},tableStartIndex:{rowStartIndex:0,colStartIndex:0},categoryArr:[{label:this.$t("components.value"),value:"value"},{label:this.$t("components.scientificCounting"),value:"science"}],category:"value",gridTableThemeObj:s["a"].tensorThemes[this.$store.state.themeIndex]}},computed:{},watch:{},mounted:function(){this.init()},methods:{init:function(){this.itemId="".concat((new Date).getTime())+"".concat(this.$store.state.componentsCount),this.$store.commit("componentsNum")},initializeFilterArr:function(e,t){if(this.filterCorrect=!0,t){for(var i=t.slice(1,t.length-1).split(","),n=[],o=[],s=0;s<i.length;s++)if(n.push({model:i[s],max:e[s]-1,showError:!1}),-1!==i[s].indexOf(":")){var a=i[s].split(":");if(a[0]){var r=Number(a[0]);r=r<0?e[s]+r:r,o.push(r)}else o.push(0)}this.filterArr=n,o.length?o.length>=2?this.tableStartIndex={rowStartIndex:o[0],colStartIndex:o[1]}:this.tableStartIndex={rowStartIndex:0,colStartIndex:o[0]}:this.tableStartIndex={rowStartIndex:0,colStartIndex:0}}else this.filterArr=[]},formateColumnsData:function(){var e=this;this.columnsData=[{id:-1,name:" ",field:-1,width:120,headerCssClass:"headerStyle"}];var t=this.formateData[0];t?t.forEach((function(t,i){var n=i+e.tableStartIndex.colStartIndex;e.columnsData.push({id:n,name:n,field:n,width:120,headerCssClass:"headerStyle",formatter:e.formateValueColor})})):this.columnsData=[]},formateValueColor:function(e,t,i,n,o){return t&&i&&!isNaN(i)&&i!==1/0&&i!==-1/0?i<0?'<span class="table-item-span" style="background:rgba('.concat(this.gridTableThemeObj.negativeColor,", ").concat(i/this.statistics.overall_min,')">').concat(i,"</span>"):'<span class="table-item-span" style="background:rgba('.concat(this.gridTableThemeObj.positiveColor,", ").concat(i/this.statistics.overall_max,')">').concat(i,"</span>"):i},formateGridArray:function(){var e=this;this.fullData instanceof Array?this.fullData.length?this.fullData[0]instanceof Array?this.formateData=this.fullData:this.formateData=[this.fullData]:(this.formateData=[[]],this.columnsData=[]):this.formateData=[[this.fullData]];var t=[];this.formateData.forEach((function(i,n){var o={"-1":n+e.tableStartIndex.rowStartIndex};i.forEach((function(t,i){var n=i+e.tableStartIndex.colStartIndex;isNaN(t)?o[n]=t:e.category===e.categoryArr[0].value?o[n]=t.toFixed(e.accuracy):o[n]=t.toExponential(e.accuracy)})),t.push(o)})),this.formateArr=t},updateGrid:function(){var e=this;this.$nextTick((function(){e.gridObj||(e.gridObj=new window.Slick.Grid("#".concat(e.itemId),e.formateArr,e.columnsData,e.optionObj),e.columnsLength=e.columnsData.length),e.gridObj.setData(e.formateArr,e.scrollTop),e.scrollTop=!1;var t=e.columnsData.length;(e.columnsLength!==t||e.viewResizeFlag)&&(e.gridObj.setColumns(e.columnsData),e.columnsLength=t,e.viewResizeFlag=!1),e.gridObj.render()}))},accuracyChange:function(){this.formateGridArray(),this.requestError||this.incorrectData||this.updateGrid()},filterChange:function(){var e=this,t=!0,i=!1,n=2,o=[],s=[];if(this.filterArr.forEach((function(a,r){var l=a.model.trim();if(isNaN(l))if(-1!==l.indexOf(":")){o.push(r);var c=e.checkCombinatorialInput(a);c?(a.showError=!1,n?n--:i=!0):(a.showError=!0,t=!1)}else a.showError=!0,t=!1;else l<-(a.max+1)||l>a.max||""===l||l%1?(a.showError=!0,t=!1):(a.showError=!1,l=Number(l));s.push(l)})),o.length){var a=o.pop(),r=this.filterArr[a];if(this.columnLimitNum>0&&r&&!r.showError&&r.max>=this.columnLimitNum){var l=this.checkFilterLimitOver(r);l&&(r.showError=!0,t=!1)}}this.filterCorrect=t,i&&t?this.incorrectData=!0:(this.incorrectData=!1,t&&(this.viewResizeFlag=!0,this.$emit("martixFilterChange",s)))},checkFilterLimitOver:function(e){var t=!1,i=e.model.trim(),n=i.split(":"),o=n[0]?n[0]:0,s=n[1]?n[1]:e.max+1;return o=o<0?e.max+Number(o)+1:Number(o),s=s<0?e.max+Number(s)+1:Number(s),s-o>this.columnLimitNum&&(t=!0),t},checkCombinatorialInput:function(e){var t=e.model.trim(),i=t.split(":"),n=i[0],o=i[1],s=2;if(n&&(isNaN(n)||n<-(e+1)||n>e.max))return!1;if(o&&(isNaN(o)||o<=-(e.max+1)||o>e.max+1||!Number(o)))return!1;if(i.length>s)return!1;if(n||o){if(n&&o){var a=n<0?e.max+Number(n)+1:Number(n),r=o<0?e.max+Number(o)+1:Number(o);return!(r<=a)}return!0}return!0},updateGridData:function(e,t,i,n){var o=this;this.updated=!0,this.requestError=!1,this.$nextTick((function(){o.fullData&&o.fullData.length&&(e&&(o.initializeFilterArr(t,n),o.scrollTop=!0),(e||void 0===o.statistics.overall_max)&&(o.statistics=i),o.formateGridArray(),o.formateColumnsData(),o.incorrectData||o.updateGrid())}))},resizeView:function(){var e=this;this.gridObj&&(this.incorrectData||this.requestError?this.viewResizeFlag=!0:this.$nextTick((function(){e.gridObj.resizeCanvas(),e.gridObj.render()})))},toggleFullScreen:function(){this.$emit("toggleFullScreen")},showRequestErrorMessage:function(e,t,i){this.errorMsg=e,!this.filterArr.length&&t&&i&&this.initializeFilterArr(t,i),this.requestError=!0}},destroyed:function(){}},r=a,l=(i("dc82"),i("2877")),c=Object(l["a"])(r,n,o,!1,null,null,null);t["a"]=c.exports},d28b:function(e,t,i){var n=i("746f");n("iterator")},daff:function(e,t,i){e.exports=i.p+"static/img/image-guid.33f41c3e.svg"},dc1b:function(e,t,i){var n,o;
/*!
 * jQuery JavaScript Library v3.1.0
 * https://jquery.com/
 *
 * Includes Sizzle.js
 * https://sizzlejs.com/
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license
 * https://jquery.org/license
 *
 * Date: 2016-07-07T21:44Z
 */(function(t,i){"use strict";"object"===typeof e.exports?e.exports=t.document?i(t,!0):function(e){if(!e.document)throw new Error("jQuery requires a window with a document");return i(e)}:i(t)})("undefined"!==typeof window?window:this,(function(i,s){"use strict";var a=[],r=i.document,l=Object.getPrototypeOf,c=a.slice,h=a.concat,u=a.push,d=a.indexOf,p={},f=p.toString,g=p.hasOwnProperty,m=g.toString,v=m.call(Object),w={};function y(e,t){t=t||r;var i=t.createElement("script");i.text=e,t.head.appendChild(i).parentNode.removeChild(i)}var b="3.1.0",C=function(e,t){return new C.fn.init(e,t)},_=/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,x=/^-ms-/,k=/-([a-z])/g,S=function(e,t){return t.toUpperCase()};function D(e){var t=!!e&&"length"in e&&e.length,i=C.type(e);return"function"!==i&&!C.isWindow(e)&&("array"===i||0===t||"number"===typeof t&&t>0&&t-1 in e)}C.fn=C.prototype={jquery:b,constructor:C,length:0,toArray:function(){return c.call(this)},get:function(e){return null!=e?e<0?this[e+this.length]:this[e]:c.call(this)},pushStack:function(e){var t=C.merge(this.constructor(),e);return t.prevObject=this,t},each:function(e){return C.each(this,e)},map:function(e){return this.pushStack(C.map(this,(function(t,i){return e.call(t,i,t)})))},slice:function(){return this.pushStack(c.apply(this,arguments))},first:function(){return this.eq(0)},last:function(){return this.eq(-1)},eq:function(e){var t=this.length,i=+e+(e<0?t:0);return this.pushStack(i>=0&&i<t?[this[i]]:[])},end:function(){return this.prevObject||this.constructor()},push:u,sort:a.sort,splice:a.splice},C.extend=C.fn.extend=function(){var e,t,i,n,o,s,a=arguments[0]||{},r=1,l=arguments.length,c=!1;for("boolean"===typeof a&&(c=a,a=arguments[r]||{},r++),"object"===typeof a||C.isFunction(a)||(a={}),r===l&&(a=this,r--);r<l;r++)if(null!=(e=arguments[r]))for(t in e)i=a[t],n=e[t],a!==n&&(c&&n&&(C.isPlainObject(n)||(o=C.isArray(n)))?(o?(o=!1,s=i&&C.isArray(i)?i:[]):s=i&&C.isPlainObject(i)?i:{},a[t]=C.extend(c,s,n)):void 0!==n&&(a[t]=n));return a},C.extend({expando:"jQuery"+(b+Math.random()).replace(/\D/g,""),isReady:!0,error:function(e){throw new Error(e)},noop:function(){},isFunction:function(e){return"function"===C.type(e)},isArray:Array.isArray,isWindow:function(e){return null!=e&&e===e.window},isNumeric:function(e){var t=C.type(e);return("number"===t||"string"===t)&&!isNaN(e-parseFloat(e))},isPlainObject:function(e){var t,i;return!(!e||"[object Object]"!==f.call(e))&&(t=l(e),!t||(i=g.call(t,"constructor")&&t.constructor,"function"===typeof i&&m.call(i)===v))},isEmptyObject:function(e){var t;for(t in e)return!1;return!0},type:function(e){return null==e?e+"":"object"===typeof e||"function"===typeof e?p[f.call(e)]||"object":typeof e},globalEval:function(e){y(e)},camelCase:function(e){return e.replace(x,"ms-").replace(k,S)},nodeName:function(e,t){return e.nodeName&&e.nodeName.toLowerCase()===t.toLowerCase()},each:function(e,t){var i,n=0;if(D(e)){for(i=e.length;n<i;n++)if(!1===t.call(e[n],n,e[n]))break}else for(n in e)if(!1===t.call(e[n],n,e[n]))break;return e},trim:function(e){return null==e?"":(e+"").replace(_,"")},makeArray:function(e,t){var i=t||[];return null!=e&&(D(Object(e))?C.merge(i,"string"===typeof e?[e]:e):u.call(i,e)),i},inArray:function(e,t,i){return null==t?-1:d.call(t,e,i)},merge:function(e,t){for(var i=+t.length,n=0,o=e.length;n<i;n++)e[o++]=t[n];return e.length=o,e},grep:function(e,t,i){for(var n,o=[],s=0,a=e.length,r=!i;s<a;s++)n=!t(e[s],s),n!==r&&o.push(e[s]);return o},map:function(e,t,i){var n,o,s=0,a=[];if(D(e))for(n=e.length;s<n;s++)o=t(e[s],s,i),null!=o&&a.push(o);else for(s in e)o=t(e[s],s,i),null!=o&&a.push(o);return h.apply([],a)},guid:1,proxy:function(e,t){var i,n,o;if("string"===typeof t&&(i=e[t],t=e,e=i),C.isFunction(e))return n=c.call(arguments,2),o=function(){return e.apply(t||this,n.concat(c.call(arguments)))},o.guid=e.guid=e.guid||C.guid++,o},now:Date.now,support:w}),"function"===typeof Symbol&&(C.fn[Symbol.iterator]=a[Symbol.iterator]),C.each("Boolean Number String Function Array Date RegExp Object Error Symbol".split(" "),(function(e,t){p["[object "+t+"]"]=t.toLowerCase()}));var R=
/*!
 * Sizzle CSS Selector Engine v2.3.0
 * https://sizzlejs.com/
 *
 * Copyright jQuery Foundation and other contributors
 * Released under the MIT license
 * http://jquery.org/license
 *
 * Date: 2016-01-04
 */
function(e){var t,i,n,o,s,a,r,l,c,h,u,d,p,f,g,m,v,w,y,b="sizzle"+1*new Date,C=e.document,_=0,x=0,k=ae(),S=ae(),D=ae(),R=function(e,t){return e===t&&(u=!0),0},$={}.hasOwnProperty,T=[],P=T.pop,I=T.push,M=T.push,A=T.slice,E=function(e,t){for(var i=0,n=e.length;i<n;i++)if(e[i]===t)return i;return-1},N="checked|selected|async|autofocus|autoplay|controls|defer|disabled|hidden|ismap|loop|multiple|open|readonly|required|scoped",z="[\\x20\\t\\r\\n\\f]",H="(?:\\\\.|[\\w-]|[^\0-\\xa0])+",L="\\["+z+"*("+H+")(?:"+z+"*([*^$|!~]?=)"+z+"*(?:'((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\"|("+H+"))|)"+z+"*\\]",F=":("+H+")(?:\\((('((?:\\\\.|[^\\\\'])*)'|\"((?:\\\\.|[^\\\\\"])*)\")|((?:\\\\.|[^\\\\()[\\]]|"+L+")*)|.*)\\)|)",W=new RegExp(z+"+","g"),O=new RegExp("^"+z+"+|((?:^|[^\\\\])(?:\\\\.)*)"+z+"+$","g"),j=new RegExp("^"+z+"*,"+z+"*"),B=new RegExp("^"+z+"*([>+~]|"+z+")"+z+"*"),q=new RegExp("="+z+"*([^\\]'\"]*?)"+z+"*\\]","g"),V=new RegExp(F),G=new RegExp("^"+H+"$"),Y={ID:new RegExp("^#("+H+")"),CLASS:new RegExp("^\\.("+H+")"),TAG:new RegExp("^("+H+"|[*])"),ATTR:new RegExp("^"+L),PSEUDO:new RegExp("^"+F),CHILD:new RegExp("^:(only|first|last|nth|nth-last)-(child|of-type)(?:\\("+z+"*(even|odd|(([+-]|)(\\d*)n|)"+z+"*(?:([+-]|)"+z+"*(\\d+)|))"+z+"*\\)|)","i"),bool:new RegExp("^(?:"+N+")$","i"),needsContext:new RegExp("^"+z+"*[>+~]|:(even|odd|eq|gt|lt|nth|first|last)(?:\\("+z+"*((?:-\\d)?\\d*)"+z+"*\\)|)(?=[^-]|$)","i")},K=/^(?:input|select|textarea|button)$/i,U=/^h\d$/i,X=/^[^{]+\{\s*\[native \w/,Q=/^(?:#([\w-]+)|(\w+)|\.([\w-]+))$/,J=/[+~]/,Z=new RegExp("\\\\([\\da-f]{1,6}"+z+"?|("+z+")|.)","ig"),ee=function(e,t,i){var n="0x"+t-65536;return n!==n||i?t:n<0?String.fromCharCode(n+65536):String.fromCharCode(n>>10|55296,1023&n|56320)},te=/([\0-\x1f\x7f]|^-?\d)|^-$|[^\x80-\uFFFF\w-]/g,ie=function(e,t){return t?"\0"===e?"�":e.slice(0,-1)+"\\"+e.charCodeAt(e.length-1).toString(16)+" ":"\\"+e},ne=function(){d()},oe=we((function(e){return!0===e.disabled}),{dir:"parentNode",next:"legend"});try{M.apply(T=A.call(C.childNodes),C.childNodes),T[C.childNodes.length].nodeType}catch(Se){M={apply:T.length?function(e,t){I.apply(e,A.call(t))}:function(e,t){var i=e.length,n=0;while(e[i++]=t[n++]);e.length=i-1}}}function se(e,t,n,o){var s,r,c,h,u,f,v,w=t&&t.ownerDocument,_=t?t.nodeType:9;if(n=n||[],"string"!==typeof e||!e||1!==_&&9!==_&&11!==_)return n;if(!o&&((t?t.ownerDocument||t:C)!==p&&d(t),t=t||p,g)){if(11!==_&&(u=Q.exec(e)))if(s=u[1]){if(9===_){if(!(c=t.getElementById(s)))return n;if(c.id===s)return n.push(c),n}else if(w&&(c=w.getElementById(s))&&y(t,c)&&c.id===s)return n.push(c),n}else{if(u[2])return M.apply(n,t.getElementsByTagName(e)),n;if((s=u[3])&&i.getElementsByClassName&&t.getElementsByClassName)return M.apply(n,t.getElementsByClassName(s)),n}if(i.qsa&&!D[e+" "]&&(!m||!m.test(e))){if(1!==_)w=t,v=e;else if("object"!==t.nodeName.toLowerCase()){(h=t.getAttribute("id"))?h=h.replace(te,ie):t.setAttribute("id",h=b),f=a(e),r=f.length;while(r--)f[r]="#"+h+" "+ve(f[r]);v=f.join(","),w=J.test(e)&&ge(t.parentNode)||t}if(v)try{return M.apply(n,w.querySelectorAll(v)),n}catch(x){}finally{h===b&&t.removeAttribute("id")}}}return l(e.replace(O,"$1"),t,n,o)}function ae(){var e=[];function t(i,o){return e.push(i+" ")>n.cacheLength&&delete t[e.shift()],t[i+" "]=o}return t}function re(e){return e[b]=!0,e}function le(e){var t=p.createElement("fieldset");try{return!!e(t)}catch(Se){return!1}finally{t.parentNode&&t.parentNode.removeChild(t),t=null}}function ce(e,t){var i=e.split("|"),o=i.length;while(o--)n.attrHandle[i[o]]=t}function he(e,t){var i=t&&e,n=i&&1===e.nodeType&&1===t.nodeType&&e.sourceIndex-t.sourceIndex;if(n)return n;if(i)while(i=i.nextSibling)if(i===t)return-1;return e?1:-1}function ue(e){return function(t){var i=t.nodeName.toLowerCase();return"input"===i&&t.type===e}}function de(e){return function(t){var i=t.nodeName.toLowerCase();return("input"===i||"button"===i)&&t.type===e}}function pe(e){return function(t){return"label"in t&&t.disabled===e||"form"in t&&t.disabled===e||"form"in t&&!1===t.disabled&&(t.isDisabled===e||t.isDisabled!==!e&&("label"in t||!oe(t))!==e)}}function fe(e){return re((function(t){return t=+t,re((function(i,n){var o,s=e([],i.length,t),a=s.length;while(a--)i[o=s[a]]&&(i[o]=!(n[o]=i[o]))}))}))}function ge(e){return e&&"undefined"!==typeof e.getElementsByTagName&&e}for(t in i=se.support={},s=se.isXML=function(e){var t=e&&(e.ownerDocument||e).documentElement;return!!t&&"HTML"!==t.nodeName},d=se.setDocument=function(e){var t,o,a=e?e.ownerDocument||e:C;return a!==p&&9===a.nodeType&&a.documentElement?(p=a,f=p.documentElement,g=!s(p),C!==p&&(o=p.defaultView)&&o.top!==o&&(o.addEventListener?o.addEventListener("unload",ne,!1):o.attachEvent&&o.attachEvent("onunload",ne)),i.attributes=le((function(e){return e.className="i",!e.getAttribute("className")})),i.getElementsByTagName=le((function(e){return e.appendChild(p.createComment("")),!e.getElementsByTagName("*").length})),i.getElementsByClassName=X.test(p.getElementsByClassName),i.getById=le((function(e){return f.appendChild(e).id=b,!p.getElementsByName||!p.getElementsByName(b).length})),i.getById?(n.find["ID"]=function(e,t){if("undefined"!==typeof t.getElementById&&g){var i=t.getElementById(e);return i?[i]:[]}},n.filter["ID"]=function(e){var t=e.replace(Z,ee);return function(e){return e.getAttribute("id")===t}}):(delete n.find["ID"],n.filter["ID"]=function(e){var t=e.replace(Z,ee);return function(e){var i="undefined"!==typeof e.getAttributeNode&&e.getAttributeNode("id");return i&&i.value===t}}),n.find["TAG"]=i.getElementsByTagName?function(e,t){return"undefined"!==typeof t.getElementsByTagName?t.getElementsByTagName(e):i.qsa?t.querySelectorAll(e):void 0}:function(e,t){var i,n=[],o=0,s=t.getElementsByTagName(e);if("*"===e){while(i=s[o++])1===i.nodeType&&n.push(i);return n}return s},n.find["CLASS"]=i.getElementsByClassName&&function(e,t){if("undefined"!==typeof t.getElementsByClassName&&g)return t.getElementsByClassName(e)},v=[],m=[],(i.qsa=X.test(p.querySelectorAll))&&(le((function(e){f.appendChild(e).innerHTML="<a id='"+b+"'></a><select id='"+b+"-\r\\' msallowcapture=''><option selected=''></option></select>",e.querySelectorAll("[msallowcapture^='']").length&&m.push("[*^$]="+z+"*(?:''|\"\")"),e.querySelectorAll("[selected]").length||m.push("\\["+z+"*(?:value|"+N+")"),e.querySelectorAll("[id~="+b+"-]").length||m.push("~="),e.querySelectorAll(":checked").length||m.push(":checked"),e.querySelectorAll("a#"+b+"+*").length||m.push(".#.+[+~]")})),le((function(e){e.innerHTML="<a href='' disabled='disabled'></a><select disabled='disabled'><option/></select>";var t=p.createElement("input");t.setAttribute("type","hidden"),e.appendChild(t).setAttribute("name","D"),e.querySelectorAll("[name=d]").length&&m.push("name"+z+"*[*^$|!~]?="),2!==e.querySelectorAll(":enabled").length&&m.push(":enabled",":disabled"),f.appendChild(e).disabled=!0,2!==e.querySelectorAll(":disabled").length&&m.push(":enabled",":disabled"),e.querySelectorAll("*,:x"),m.push(",.*:")}))),(i.matchesSelector=X.test(w=f.matches||f.webkitMatchesSelector||f.mozMatchesSelector||f.oMatchesSelector||f.msMatchesSelector))&&le((function(e){i.disconnectedMatch=w.call(e,"*"),w.call(e,"[s!='']:x"),v.push("!=",F)})),m=m.length&&new RegExp(m.join("|")),v=v.length&&new RegExp(v.join("|")),t=X.test(f.compareDocumentPosition),y=t||X.test(f.contains)?function(e,t){var i=9===e.nodeType?e.documentElement:e,n=t&&t.parentNode;return e===n||!(!n||1!==n.nodeType||!(i.contains?i.contains(n):e.compareDocumentPosition&&16&e.compareDocumentPosition(n)))}:function(e,t){if(t)while(t=t.parentNode)if(t===e)return!0;return!1},R=t?function(e,t){if(e===t)return u=!0,0;var n=!e.compareDocumentPosition-!t.compareDocumentPosition;return n||(n=(e.ownerDocument||e)===(t.ownerDocument||t)?e.compareDocumentPosition(t):1,1&n||!i.sortDetached&&t.compareDocumentPosition(e)===n?e===p||e.ownerDocument===C&&y(C,e)?-1:t===p||t.ownerDocument===C&&y(C,t)?1:h?E(h,e)-E(h,t):0:4&n?-1:1)}:function(e,t){if(e===t)return u=!0,0;var i,n=0,o=e.parentNode,s=t.parentNode,a=[e],r=[t];if(!o||!s)return e===p?-1:t===p?1:o?-1:s?1:h?E(h,e)-E(h,t):0;if(o===s)return he(e,t);i=e;while(i=i.parentNode)a.unshift(i);i=t;while(i=i.parentNode)r.unshift(i);while(a[n]===r[n])n++;return n?he(a[n],r[n]):a[n]===C?-1:r[n]===C?1:0},p):p},se.matches=function(e,t){return se(e,null,null,t)},se.matchesSelector=function(e,t){if((e.ownerDocument||e)!==p&&d(e),t=t.replace(q,"='$1']"),i.matchesSelector&&g&&!D[t+" "]&&(!v||!v.test(t))&&(!m||!m.test(t)))try{var n=w.call(e,t);if(n||i.disconnectedMatch||e.document&&11!==e.document.nodeType)return n}catch(Se){}return se(t,p,null,[e]).length>0},se.contains=function(e,t){return(e.ownerDocument||e)!==p&&d(e),y(e,t)},se.attr=function(e,t){(e.ownerDocument||e)!==p&&d(e);var o=n.attrHandle[t.toLowerCase()],s=o&&$.call(n.attrHandle,t.toLowerCase())?o(e,t,!g):void 0;return void 0!==s?s:i.attributes||!g?e.getAttribute(t):(s=e.getAttributeNode(t))&&s.specified?s.value:null},se.escape=function(e){return(e+"").replace(te,ie)},se.error=function(e){throw new Error("Syntax error, unrecognized expression: "+e)},se.uniqueSort=function(e){var t,n=[],o=0,s=0;if(u=!i.detectDuplicates,h=!i.sortStable&&e.slice(0),e.sort(R),u){while(t=e[s++])t===e[s]&&(o=n.push(s));while(o--)e.splice(n[o],1)}return h=null,e},o=se.getText=function(e){var t,i="",n=0,s=e.nodeType;if(s){if(1===s||9===s||11===s){if("string"===typeof e.textContent)return e.textContent;for(e=e.firstChild;e;e=e.nextSibling)i+=o(e)}else if(3===s||4===s)return e.nodeValue}else while(t=e[n++])i+=o(t);return i},n=se.selectors={cacheLength:50,createPseudo:re,match:Y,attrHandle:{},find:{},relative:{">":{dir:"parentNode",first:!0}," ":{dir:"parentNode"},"+":{dir:"previousSibling",first:!0},"~":{dir:"previousSibling"}},preFilter:{ATTR:function(e){return e[1]=e[1].replace(Z,ee),e[3]=(e[3]||e[4]||e[5]||"").replace(Z,ee),"~="===e[2]&&(e[3]=" "+e[3]+" "),e.slice(0,4)},CHILD:function(e){return e[1]=e[1].toLowerCase(),"nth"===e[1].slice(0,3)?(e[3]||se.error(e[0]),e[4]=+(e[4]?e[5]+(e[6]||1):2*("even"===e[3]||"odd"===e[3])),e[5]=+(e[7]+e[8]||"odd"===e[3])):e[3]&&se.error(e[0]),e},PSEUDO:function(e){var t,i=!e[6]&&e[2];return Y["CHILD"].test(e[0])?null:(e[3]?e[2]=e[4]||e[5]||"":i&&V.test(i)&&(t=a(i,!0))&&(t=i.indexOf(")",i.length-t)-i.length)&&(e[0]=e[0].slice(0,t),e[2]=i.slice(0,t)),e.slice(0,3))}},filter:{TAG:function(e){var t=e.replace(Z,ee).toLowerCase();return"*"===e?function(){return!0}:function(e){return e.nodeName&&e.nodeName.toLowerCase()===t}},CLASS:function(e){var t=k[e+" "];return t||(t=new RegExp("(^|"+z+")"+e+"("+z+"|$)"))&&k(e,(function(e){return t.test("string"===typeof e.className&&e.className||"undefined"!==typeof e.getAttribute&&e.getAttribute("class")||"")}))},ATTR:function(e,t,i){return function(n){var o=se.attr(n,e);return null==o?"!="===t:!t||(o+="","="===t?o===i:"!="===t?o!==i:"^="===t?i&&0===o.indexOf(i):"*="===t?i&&o.indexOf(i)>-1:"$="===t?i&&o.slice(-i.length)===i:"~="===t?(" "+o.replace(W," ")+" ").indexOf(i)>-1:"|="===t&&(o===i||o.slice(0,i.length+1)===i+"-"))}},CHILD:function(e,t,i,n,o){var s="nth"!==e.slice(0,3),a="last"!==e.slice(-4),r="of-type"===t;return 1===n&&0===o?function(e){return!!e.parentNode}:function(t,i,l){var c,h,u,d,p,f,g=s!==a?"nextSibling":"previousSibling",m=t.parentNode,v=r&&t.nodeName.toLowerCase(),w=!l&&!r,y=!1;if(m){if(s){while(g){d=t;while(d=d[g])if(r?d.nodeName.toLowerCase()===v:1===d.nodeType)return!1;f=g="only"===e&&!f&&"nextSibling"}return!0}if(f=[a?m.firstChild:m.lastChild],a&&w){d=m,u=d[b]||(d[b]={}),h=u[d.uniqueID]||(u[d.uniqueID]={}),c=h[e]||[],p=c[0]===_&&c[1],y=p&&c[2],d=p&&m.childNodes[p];while(d=++p&&d&&d[g]||(y=p=0)||f.pop())if(1===d.nodeType&&++y&&d===t){h[e]=[_,p,y];break}}else if(w&&(d=t,u=d[b]||(d[b]={}),h=u[d.uniqueID]||(u[d.uniqueID]={}),c=h[e]||[],p=c[0]===_&&c[1],y=p),!1===y)while(d=++p&&d&&d[g]||(y=p=0)||f.pop())if((r?d.nodeName.toLowerCase()===v:1===d.nodeType)&&++y&&(w&&(u=d[b]||(d[b]={}),h=u[d.uniqueID]||(u[d.uniqueID]={}),h[e]=[_,y]),d===t))break;return y-=o,y===n||y%n===0&&y/n>=0}}},PSEUDO:function(e,t){var i,o=n.pseudos[e]||n.setFilters[e.toLowerCase()]||se.error("unsupported pseudo: "+e);return o[b]?o(t):o.length>1?(i=[e,e,"",t],n.setFilters.hasOwnProperty(e.toLowerCase())?re((function(e,i){var n,s=o(e,t),a=s.length;while(a--)n=E(e,s[a]),e[n]=!(i[n]=s[a])})):function(e){return o(e,0,i)}):o}},pseudos:{not:re((function(e){var t=[],i=[],n=r(e.replace(O,"$1"));return n[b]?re((function(e,t,i,o){var s,a=n(e,null,o,[]),r=e.length;while(r--)(s=a[r])&&(e[r]=!(t[r]=s))})):function(e,o,s){return t[0]=e,n(t,null,s,i),t[0]=null,!i.pop()}})),has:re((function(e){return function(t){return se(e,t).length>0}})),contains:re((function(e){return e=e.replace(Z,ee),function(t){return(t.textContent||t.innerText||o(t)).indexOf(e)>-1}})),lang:re((function(e){return G.test(e||"")||se.error("unsupported lang: "+e),e=e.replace(Z,ee).toLowerCase(),function(t){var i;do{if(i=g?t.lang:t.getAttribute("xml:lang")||t.getAttribute("lang"))return i=i.toLowerCase(),i===e||0===i.indexOf(e+"-")}while((t=t.parentNode)&&1===t.nodeType);return!1}})),target:function(t){var i=e.location&&e.location.hash;return i&&i.slice(1)===t.id},root:function(e){return e===f},focus:function(e){return e===p.activeElement&&(!p.hasFocus||p.hasFocus())&&!!(e.type||e.href||~e.tabIndex)},enabled:pe(!1),disabled:pe(!0),checked:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&!!e.checked||"option"===t&&!!e.selected},selected:function(e){return e.parentNode&&e.parentNode.selectedIndex,!0===e.selected},empty:function(e){for(e=e.firstChild;e;e=e.nextSibling)if(e.nodeType<6)return!1;return!0},parent:function(e){return!n.pseudos["empty"](e)},header:function(e){return U.test(e.nodeName)},input:function(e){return K.test(e.nodeName)},button:function(e){var t=e.nodeName.toLowerCase();return"input"===t&&"button"===e.type||"button"===t},text:function(e){var t;return"input"===e.nodeName.toLowerCase()&&"text"===e.type&&(null==(t=e.getAttribute("type"))||"text"===t.toLowerCase())},first:fe((function(){return[0]})),last:fe((function(e,t){return[t-1]})),eq:fe((function(e,t,i){return[i<0?i+t:i]})),even:fe((function(e,t){for(var i=0;i<t;i+=2)e.push(i);return e})),odd:fe((function(e,t){for(var i=1;i<t;i+=2)e.push(i);return e})),lt:fe((function(e,t,i){for(var n=i<0?i+t:i;--n>=0;)e.push(n);return e})),gt:fe((function(e,t,i){for(var n=i<0?i+t:i;++n<t;)e.push(n);return e}))}},n.pseudos["nth"]=n.pseudos["eq"],{radio:!0,checkbox:!0,file:!0,password:!0,image:!0})n.pseudos[t]=ue(t);for(t in{submit:!0,reset:!0})n.pseudos[t]=de(t);function me(){}function ve(e){for(var t=0,i=e.length,n="";t<i;t++)n+=e[t].value;return n}function we(e,t,i){var n=t.dir,o=t.next,s=o||n,a=i&&"parentNode"===s,r=x++;return t.first?function(t,i,o){while(t=t[n])if(1===t.nodeType||a)return e(t,i,o)}:function(t,i,l){var c,h,u,d=[_,r];if(l){while(t=t[n])if((1===t.nodeType||a)&&e(t,i,l))return!0}else while(t=t[n])if(1===t.nodeType||a)if(u=t[b]||(t[b]={}),h=u[t.uniqueID]||(u[t.uniqueID]={}),o&&o===t.nodeName.toLowerCase())t=t[n]||t;else{if((c=h[s])&&c[0]===_&&c[1]===r)return d[2]=c[2];if(h[s]=d,d[2]=e(t,i,l))return!0}}}function ye(e){return e.length>1?function(t,i,n){var o=e.length;while(o--)if(!e[o](t,i,n))return!1;return!0}:e[0]}function be(e,t,i){for(var n=0,o=t.length;n<o;n++)se(e,t[n],i);return i}function Ce(e,t,i,n,o){for(var s,a=[],r=0,l=e.length,c=null!=t;r<l;r++)(s=e[r])&&(i&&!i(s,n,o)||(a.push(s),c&&t.push(r)));return a}function _e(e,t,i,n,o,s){return n&&!n[b]&&(n=_e(n)),o&&!o[b]&&(o=_e(o,s)),re((function(s,a,r,l){var c,h,u,d=[],p=[],f=a.length,g=s||be(t||"*",r.nodeType?[r]:r,[]),m=!e||!s&&t?g:Ce(g,d,e,r,l),v=i?o||(s?e:f||n)?[]:a:m;if(i&&i(m,v,r,l),n){c=Ce(v,p),n(c,[],r,l),h=c.length;while(h--)(u=c[h])&&(v[p[h]]=!(m[p[h]]=u))}if(s){if(o||e){if(o){c=[],h=v.length;while(h--)(u=v[h])&&c.push(m[h]=u);o(null,v=[],c,l)}h=v.length;while(h--)(u=v[h])&&(c=o?E(s,u):d[h])>-1&&(s[c]=!(a[c]=u))}}else v=Ce(v===a?v.splice(f,v.length):v),o?o(null,a,v,l):M.apply(a,v)}))}function xe(e){for(var t,i,o,s=e.length,a=n.relative[e[0].type],r=a||n.relative[" "],l=a?1:0,h=we((function(e){return e===t}),r,!0),u=we((function(e){return E(t,e)>-1}),r,!0),d=[function(e,i,n){var o=!a&&(n||i!==c)||((t=i).nodeType?h(e,i,n):u(e,i,n));return t=null,o}];l<s;l++)if(i=n.relative[e[l].type])d=[we(ye(d),i)];else{if(i=n.filter[e[l].type].apply(null,e[l].matches),i[b]){for(o=++l;o<s;o++)if(n.relative[e[o].type])break;return _e(l>1&&ye(d),l>1&&ve(e.slice(0,l-1).concat({value:" "===e[l-2].type?"*":""})).replace(O,"$1"),i,l<o&&xe(e.slice(l,o)),o<s&&xe(e=e.slice(o)),o<s&&ve(e))}d.push(i)}return ye(d)}function ke(e,t){var i=t.length>0,o=e.length>0,s=function(s,a,r,l,h){var u,f,m,v=0,w="0",y=s&&[],b=[],C=c,x=s||o&&n.find["TAG"]("*",h),k=_+=null==C?1:Math.random()||.1,S=x.length;for(h&&(c=a===p||a||h);w!==S&&null!=(u=x[w]);w++){if(o&&u){f=0,a||u.ownerDocument===p||(d(u),r=!g);while(m=e[f++])if(m(u,a||p,r)){l.push(u);break}h&&(_=k)}i&&((u=!m&&u)&&v--,s&&y.push(u))}if(v+=w,i&&w!==v){f=0;while(m=t[f++])m(y,b,a,r);if(s){if(v>0)while(w--)y[w]||b[w]||(b[w]=P.call(l));b=Ce(b)}M.apply(l,b),h&&!s&&b.length>0&&v+t.length>1&&se.uniqueSort(l)}return h&&(_=k,c=C),y};return i?re(s):s}return me.prototype=n.filters=n.pseudos,n.setFilters=new me,a=se.tokenize=function(e,t){var i,o,s,a,r,l,c,h=S[e+" "];if(h)return t?0:h.slice(0);r=e,l=[],c=n.preFilter;while(r){for(a in i&&!(o=j.exec(r))||(o&&(r=r.slice(o[0].length)||r),l.push(s=[])),i=!1,(o=B.exec(r))&&(i=o.shift(),s.push({value:i,type:o[0].replace(O," ")}),r=r.slice(i.length)),n.filter)!(o=Y[a].exec(r))||c[a]&&!(o=c[a](o))||(i=o.shift(),s.push({value:i,type:a,matches:o}),r=r.slice(i.length));if(!i)break}return t?r.length:r?se.error(e):S(e,l).slice(0)},r=se.compile=function(e,t){var i,n=[],o=[],s=D[e+" "];if(!s){t||(t=a(e)),i=t.length;while(i--)s=xe(t[i]),s[b]?n.push(s):o.push(s);s=D(e,ke(o,n)),s.selector=e}return s},l=se.select=function(e,t,o,s){var l,c,h,u,d,p="function"===typeof e&&e,f=!s&&a(e=p.selector||e);if(o=o||[],1===f.length){if(c=f[0]=f[0].slice(0),c.length>2&&"ID"===(h=c[0]).type&&i.getById&&9===t.nodeType&&g&&n.relative[c[1].type]){if(t=(n.find["ID"](h.matches[0].replace(Z,ee),t)||[])[0],!t)return o;p&&(t=t.parentNode),e=e.slice(c.shift().value.length)}l=Y["needsContext"].test(e)?0:c.length;while(l--){if(h=c[l],n.relative[u=h.type])break;if((d=n.find[u])&&(s=d(h.matches[0].replace(Z,ee),J.test(c[0].type)&&ge(t.parentNode)||t))){if(c.splice(l,1),e=s.length&&ve(c),!e)return M.apply(o,s),o;break}}}return(p||r(e,f))(s,t,!g,o,!t||J.test(e)&&ge(t.parentNode)||t),o},i.sortStable=b.split("").sort(R).join("")===b,i.detectDuplicates=!!u,d(),i.sortDetached=le((function(e){return 1&e.compareDocumentPosition(p.createElement("fieldset"))})),le((function(e){return e.innerHTML="<a href='#'></a>","#"===e.firstChild.getAttribute("href")}))||ce("type|href|height|width",(function(e,t,i){if(!i)return e.getAttribute(t,"type"===t.toLowerCase()?1:2)})),i.attributes&&le((function(e){return e.innerHTML="<input/>",e.firstChild.setAttribute("value",""),""===e.firstChild.getAttribute("value")}))||ce("value",(function(e,t,i){if(!i&&"input"===e.nodeName.toLowerCase())return e.defaultValue})),le((function(e){return null==e.getAttribute("disabled")}))||ce(N,(function(e,t,i){var n;if(!i)return!0===e[t]?t.toLowerCase():(n=e.getAttributeNode(t))&&n.specified?n.value:null})),se}(i);C.find=R,C.expr=R.selectors,C.expr[":"]=C.expr.pseudos,C.uniqueSort=C.unique=R.uniqueSort,C.text=R.getText,C.isXMLDoc=R.isXML,C.contains=R.contains,C.escapeSelector=R.escape;var $=function(e,t,i){var n=[],o=void 0!==i;while((e=e[t])&&9!==e.nodeType)if(1===e.nodeType){if(o&&C(e).is(i))break;n.push(e)}return n},T=function(e,t){for(var i=[];e;e=e.nextSibling)1===e.nodeType&&e!==t&&i.push(e);return i},P=C.expr.match.needsContext,I=/^<([a-z][^\/\0>:\x20\t\r\n\f]*)[\x20\t\r\n\f]*\/?>(?:<\/\1>|)$/i,M=/^.[^:#\[\.,]*$/;function A(e,t,i){if(C.isFunction(t))return C.grep(e,(function(e,n){return!!t.call(e,n,e)!==i}));if(t.nodeType)return C.grep(e,(function(e){return e===t!==i}));if("string"===typeof t){if(M.test(t))return C.filter(t,e,i);t=C.filter(t,e)}return C.grep(e,(function(e){return d.call(t,e)>-1!==i&&1===e.nodeType}))}C.filter=function(e,t,i){var n=t[0];return i&&(e=":not("+e+")"),1===t.length&&1===n.nodeType?C.find.matchesSelector(n,e)?[n]:[]:C.find.matches(e,C.grep(t,(function(e){return 1===e.nodeType})))},C.fn.extend({find:function(e){var t,i,n=this.length,o=this;if("string"!==typeof e)return this.pushStack(C(e).filter((function(){for(t=0;t<n;t++)if(C.contains(o[t],this))return!0})));for(i=this.pushStack([]),t=0;t<n;t++)C.find(e,o[t],i);return n>1?C.uniqueSort(i):i},filter:function(e){return this.pushStack(A(this,e||[],!1))},not:function(e){return this.pushStack(A(this,e||[],!0))},is:function(e){return!!A(this,"string"===typeof e&&P.test(e)?C(e):e||[],!1).length}});var E,N=/^(?:\s*(<[\w\W]+>)[^>]*|#([\w-]+))$/,z=C.fn.init=function(e,t,i){var n,o;if(!e)return this;if(i=i||E,"string"===typeof e){if(n="<"===e[0]&&">"===e[e.length-1]&&e.length>=3?[null,e,null]:N.exec(e),!n||!n[1]&&t)return!t||t.jquery?(t||i).find(e):this.constructor(t).find(e);if(n[1]){if(t=t instanceof C?t[0]:t,C.merge(this,C.parseHTML(n[1],t&&t.nodeType?t.ownerDocument||t:r,!0)),I.test(n[1])&&C.isPlainObject(t))for(n in t)C.isFunction(this[n])?this[n](t[n]):this.attr(n,t[n]);return this}return o=r.getElementById(n[2]),o&&(this[0]=o,this.length=1),this}return e.nodeType?(this[0]=e,this.length=1,this):C.isFunction(e)?void 0!==i.ready?i.ready(e):e(C):C.makeArray(e,this)};z.prototype=C.fn,E=C(r);var H=/^(?:parents|prev(?:Until|All))/,L={children:!0,contents:!0,next:!0,prev:!0};function F(e,t){while((e=e[t])&&1!==e.nodeType);return e}C.fn.extend({has:function(e){var t=C(e,this),i=t.length;return this.filter((function(){for(var e=0;e<i;e++)if(C.contains(this,t[e]))return!0}))},closest:function(e,t){var i,n=0,o=this.length,s=[],a="string"!==typeof e&&C(e);if(!P.test(e))for(;n<o;n++)for(i=this[n];i&&i!==t;i=i.parentNode)if(i.nodeType<11&&(a?a.index(i)>-1:1===i.nodeType&&C.find.matchesSelector(i,e))){s.push(i);break}return this.pushStack(s.length>1?C.uniqueSort(s):s)},index:function(e){return e?"string"===typeof e?d.call(C(e),this[0]):d.call(this,e.jquery?e[0]:e):this[0]&&this[0].parentNode?this.first().prevAll().length:-1},add:function(e,t){return this.pushStack(C.uniqueSort(C.merge(this.get(),C(e,t))))},addBack:function(e){return this.add(null==e?this.prevObject:this.prevObject.filter(e))}}),C.each({parent:function(e){var t=e.parentNode;return t&&11!==t.nodeType?t:null},parents:function(e){return $(e,"parentNode")},parentsUntil:function(e,t,i){return $(e,"parentNode",i)},next:function(e){return F(e,"nextSibling")},prev:function(e){return F(e,"previousSibling")},nextAll:function(e){return $(e,"nextSibling")},prevAll:function(e){return $(e,"previousSibling")},nextUntil:function(e,t,i){return $(e,"nextSibling",i)},prevUntil:function(e,t,i){return $(e,"previousSibling",i)},siblings:function(e){return T((e.parentNode||{}).firstChild,e)},children:function(e){return T(e.firstChild)},contents:function(e){return e.contentDocument||C.merge([],e.childNodes)}},(function(e,t){C.fn[e]=function(i,n){var o=C.map(this,t,i);return"Until"!==e.slice(-5)&&(n=i),n&&"string"===typeof n&&(o=C.filter(n,o)),this.length>1&&(L[e]||C.uniqueSort(o),H.test(e)&&o.reverse()),this.pushStack(o)}}));var W=/\S+/g;function O(e){var t={};return C.each(e.match(W)||[],(function(e,i){t[i]=!0})),t}function j(e){return e}function B(e){throw e}function q(e,t,i){var n;try{e&&C.isFunction(n=e.promise)?n.call(e).done(t).fail(i):e&&C.isFunction(n=e.then)?n.call(e,t,i):t.call(void 0,e)}catch(e){i.call(void 0,e)}}C.Callbacks=function(e){e="string"===typeof e?O(e):C.extend({},e);var t,i,n,o,s=[],a=[],r=-1,l=function(){for(o=e.once,n=t=!0;a.length;r=-1){i=a.shift();while(++r<s.length)!1===s[r].apply(i[0],i[1])&&e.stopOnFalse&&(r=s.length,i=!1)}e.memory||(i=!1),t=!1,o&&(s=i?[]:"")},c={add:function(){return s&&(i&&!t&&(r=s.length-1,a.push(i)),function t(i){C.each(i,(function(i,n){C.isFunction(n)?e.unique&&c.has(n)||s.push(n):n&&n.length&&"string"!==C.type(n)&&t(n)}))}(arguments),i&&!t&&l()),this},remove:function(){return C.each(arguments,(function(e,t){var i;while((i=C.inArray(t,s,i))>-1)s.splice(i,1),i<=r&&r--})),this},has:function(e){return e?C.inArray(e,s)>-1:s.length>0},empty:function(){return s&&(s=[]),this},disable:function(){return o=a=[],s=i="",this},disabled:function(){return!s},lock:function(){return o=a=[],i||t||(s=i=""),this},locked:function(){return!!o},fireWith:function(e,i){return o||(i=i||[],i=[e,i.slice?i.slice():i],a.push(i),t||l()),this},fire:function(){return c.fireWith(this,arguments),this},fired:function(){return!!n}};return c},C.extend({Deferred:function(e){var t=[["notify","progress",C.Callbacks("memory"),C.Callbacks("memory"),2],["resolve","done",C.Callbacks("once memory"),C.Callbacks("once memory"),0,"resolved"],["reject","fail",C.Callbacks("once memory"),C.Callbacks("once memory"),1,"rejected"]],n="pending",o={state:function(){return n},always:function(){return s.done(arguments).fail(arguments),this},catch:function(e){return o.then(null,e)},pipe:function(){var e=arguments;return C.Deferred((function(i){C.each(t,(function(t,n){var o=C.isFunction(e[n[4]])&&e[n[4]];s[n[1]]((function(){var e=o&&o.apply(this,arguments);e&&C.isFunction(e.promise)?e.promise().progress(i.notify).done(i.resolve).fail(i.reject):i[n[0]+"With"](this,o?[e]:arguments)}))})),e=null})).promise()},then:function(e,n,o){var s=0;function a(e,t,n,o){return function(){var r=this,l=arguments,c=function(){var i,c;if(!(e<s)){if(i=n.apply(r,l),i===t.promise())throw new TypeError("Thenable self-resolution");c=i&&("object"===typeof i||"function"===typeof i)&&i.then,C.isFunction(c)?o?c.call(i,a(s,t,j,o),a(s,t,B,o)):(s++,c.call(i,a(s,t,j,o),a(s,t,B,o),a(s,t,j,t.notifyWith))):(n!==j&&(r=void 0,l=[i]),(o||t.resolveWith)(r,l))}},h=o?c:function(){try{c()}catch(i){C.Deferred.exceptionHook&&C.Deferred.exceptionHook(i,h.stackTrace),e+1>=s&&(n!==B&&(r=void 0,l=[i]),t.rejectWith(r,l))}};e?h():(C.Deferred.getStackHook&&(h.stackTrace=C.Deferred.getStackHook()),i.setTimeout(h))}}return C.Deferred((function(i){t[0][3].add(a(0,i,C.isFunction(o)?o:j,i.notifyWith)),t[1][3].add(a(0,i,C.isFunction(e)?e:j)),t[2][3].add(a(0,i,C.isFunction(n)?n:B))})).promise()},promise:function(e){return null!=e?C.extend(e,o):o}},s={};return C.each(t,(function(e,i){var a=i[2],r=i[5];o[i[1]]=a.add,r&&a.add((function(){n=r}),t[3-e][2].disable,t[0][2].lock),a.add(i[3].fire),s[i[0]]=function(){return s[i[0]+"With"](this===s?void 0:this,arguments),this},s[i[0]+"With"]=a.fireWith})),o.promise(s),e&&e.call(s,s),s},when:function(e){var t=arguments.length,i=t,n=Array(i),o=c.call(arguments),s=C.Deferred(),a=function(e){return function(i){n[e]=this,o[e]=arguments.length>1?c.call(arguments):i,--t||s.resolveWith(n,o)}};if(t<=1&&(q(e,s.done(a(i)).resolve,s.reject),"pending"===s.state()||C.isFunction(o[i]&&o[i].then)))return s.then();while(i--)q(o[i],a(i),s.reject);return s.promise()}});var V=/^(Eval|Internal|Range|Reference|Syntax|Type|URI)Error$/;C.Deferred.exceptionHook=function(e,t){i.console&&i.console.warn&&e&&V.test(e.name)&&i.console.warn("jQuery.Deferred exception: "+e.message,e.stack,t)},C.readyException=function(e){i.setTimeout((function(){throw e}))};var G=C.Deferred();function Y(){r.removeEventListener("DOMContentLoaded",Y),i.removeEventListener("load",Y),C.ready()}C.fn.ready=function(e){return G.then(e).catch((function(e){C.readyException(e)})),this},C.extend({isReady:!1,readyWait:1,holdReady:function(e){e?C.readyWait++:C.ready(!0)},ready:function(e){(!0===e?--C.readyWait:C.isReady)||(C.isReady=!0,!0!==e&&--C.readyWait>0||G.resolveWith(r,[C]))}}),C.ready.then=G.then,"complete"===r.readyState||"loading"!==r.readyState&&!r.documentElement.doScroll?i.setTimeout(C.ready):(r.addEventListener("DOMContentLoaded",Y),i.addEventListener("load",Y));var K=function(e,t,i,n,o,s,a){var r=0,l=e.length,c=null==i;if("object"===C.type(i))for(r in o=!0,i)K(e,t,r,i[r],!0,s,a);else if(void 0!==n&&(o=!0,C.isFunction(n)||(a=!0),c&&(a?(t.call(e,n),t=null):(c=t,t=function(e,t,i){return c.call(C(e),i)})),t))for(;r<l;r++)t(e[r],i,a?n:n.call(e[r],r,t(e[r],i)));return o?e:c?t.call(e):l?t(e[0],i):s},U=function(e){return 1===e.nodeType||9===e.nodeType||!+e.nodeType};function X(){this.expando=C.expando+X.uid++}X.uid=1,X.prototype={cache:function(e){var t=e[this.expando];return t||(t={},U(e)&&(e.nodeType?e[this.expando]=t:Object.defineProperty(e,this.expando,{value:t,configurable:!0}))),t},set:function(e,t,i){var n,o=this.cache(e);if("string"===typeof t)o[C.camelCase(t)]=i;else for(n in t)o[C.camelCase(n)]=t[n];return o},get:function(e,t){return void 0===t?this.cache(e):e[this.expando]&&e[this.expando][C.camelCase(t)]},access:function(e,t,i){return void 0===t||t&&"string"===typeof t&&void 0===i?this.get(e,t):(this.set(e,t,i),void 0!==i?i:t)},remove:function(e,t){var i,n=e[this.expando];if(void 0!==n){if(void 0!==t){C.isArray(t)?t=t.map(C.camelCase):(t=C.camelCase(t),t=t in n?[t]:t.match(W)||[]),i=t.length;while(i--)delete n[t[i]]}(void 0===t||C.isEmptyObject(n))&&(e.nodeType?e[this.expando]=void 0:delete e[this.expando])}},hasData:function(e){var t=e[this.expando];return void 0!==t&&!C.isEmptyObject(t)}};var Q=new X,J=new X,Z=/^(?:\{[\w\W]*\}|\[[\w\W]*\])$/,ee=/[A-Z]/g;function te(e,t,i){var n;if(void 0===i&&1===e.nodeType)if(n="data-"+t.replace(ee,"-$&").toLowerCase(),i=e.getAttribute(n),"string"===typeof i){try{i="true"===i||"false"!==i&&("null"===i?null:+i+""===i?+i:Z.test(i)?JSON.parse(i):i)}catch(o){}J.set(e,t,i)}else i=void 0;return i}C.extend({hasData:function(e){return J.hasData(e)||Q.hasData(e)},data:function(e,t,i){return J.access(e,t,i)},removeData:function(e,t){J.remove(e,t)},_data:function(e,t,i){return Q.access(e,t,i)},_removeData:function(e,t){Q.remove(e,t)}}),C.fn.extend({data:function(e,t){var i,n,o,s=this[0],a=s&&s.attributes;if(void 0===e){if(this.length&&(o=J.get(s),1===s.nodeType&&!Q.get(s,"hasDataAttrs"))){i=a.length;while(i--)a[i]&&(n=a[i].name,0===n.indexOf("data-")&&(n=C.camelCase(n.slice(5)),te(s,n,o[n])));Q.set(s,"hasDataAttrs",!0)}return o}return"object"===typeof e?this.each((function(){J.set(this,e)})):K(this,(function(t){var i;if(s&&void 0===t)return i=J.get(s,e),void 0!==i?i:(i=te(s,e),void 0!==i?i:void 0);this.each((function(){J.set(this,e,t)}))}),null,t,arguments.length>1,null,!0)},removeData:function(e){return this.each((function(){J.remove(this,e)}))}}),C.extend({queue:function(e,t,i){var n;if(e)return t=(t||"fx")+"queue",n=Q.get(e,t),i&&(!n||C.isArray(i)?n=Q.access(e,t,C.makeArray(i)):n.push(i)),n||[]},dequeue:function(e,t){t=t||"fx";var i=C.queue(e,t),n=i.length,o=i.shift(),s=C._queueHooks(e,t),a=function(){C.dequeue(e,t)};"inprogress"===o&&(o=i.shift(),n--),o&&("fx"===t&&i.unshift("inprogress"),delete s.stop,o.call(e,a,s)),!n&&s&&s.empty.fire()},_queueHooks:function(e,t){var i=t+"queueHooks";return Q.get(e,i)||Q.access(e,i,{empty:C.Callbacks("once memory").add((function(){Q.remove(e,[t+"queue",i])}))})}}),C.fn.extend({queue:function(e,t){var i=2;return"string"!==typeof e&&(t=e,e="fx",i--),arguments.length<i?C.queue(this[0],e):void 0===t?this:this.each((function(){var i=C.queue(this,e,t);C._queueHooks(this,e),"fx"===e&&"inprogress"!==i[0]&&C.dequeue(this,e)}))},dequeue:function(e){return this.each((function(){C.dequeue(this,e)}))},clearQueue:function(e){return this.queue(e||"fx",[])},promise:function(e,t){var i,n=1,o=C.Deferred(),s=this,a=this.length,r=function(){--n||o.resolveWith(s,[s])};"string"!==typeof e&&(t=e,e=void 0),e=e||"fx";while(a--)i=Q.get(s[a],e+"queueHooks"),i&&i.empty&&(n++,i.empty.add(r));return r(),o.promise(t)}});var ie=/[+-]?(?:\d*\.|)\d+(?:[eE][+-]?\d+|)/.source,ne=new RegExp("^(?:([+-])=|)("+ie+")([a-z%]*)$","i"),oe=["Top","Right","Bottom","Left"],se=function(e,t){return e=t||e,"none"===e.style.display||""===e.style.display&&C.contains(e.ownerDocument,e)&&"none"===C.css(e,"display")},ae=function(e,t,i,n){var o,s,a={};for(s in t)a[s]=e.style[s],e.style[s]=t[s];for(s in o=i.apply(e,n||[]),t)e.style[s]=a[s];return o};function re(e,t,i,n){var o,s=1,a=20,r=n?function(){return n.cur()}:function(){return C.css(e,t,"")},l=r(),c=i&&i[3]||(C.cssNumber[t]?"":"px"),h=(C.cssNumber[t]||"px"!==c&&+l)&&ne.exec(C.css(e,t));if(h&&h[3]!==c){c=c||h[3],i=i||[],h=+l||1;do{s=s||".5",h/=s,C.style(e,t,h+c)}while(s!==(s=r()/l)&&1!==s&&--a)}return i&&(h=+h||+l||0,o=i[1]?h+(i[1]+1)*i[2]:+i[2],n&&(n.unit=c,n.start=h,n.end=o)),o}var le={};function ce(e){var t,i=e.ownerDocument,n=e.nodeName,o=le[n];return o||(t=i.body.appendChild(i.createElement(n)),o=C.css(t,"display"),t.parentNode.removeChild(t),"none"===o&&(o="block"),le[n]=o,o)}function he(e,t){for(var i,n,o=[],s=0,a=e.length;s<a;s++)n=e[s],n.style&&(i=n.style.display,t?("none"===i&&(o[s]=Q.get(n,"display")||null,o[s]||(n.style.display="")),""===n.style.display&&se(n)&&(o[s]=ce(n))):"none"!==i&&(o[s]="none",Q.set(n,"display",i)));for(s=0;s<a;s++)null!=o[s]&&(e[s].style.display=o[s]);return e}C.fn.extend({show:function(){return he(this,!0)},hide:function(){return he(this)},toggle:function(e){return"boolean"===typeof e?e?this.show():this.hide():this.each((function(){se(this)?C(this).show():C(this).hide()}))}});var ue=/^(?:checkbox|radio)$/i,de=/<([a-z][^\/\0>\x20\t\r\n\f]+)/i,pe=/^$|\/(?:java|ecma)script/i,fe={option:[1,"<select multiple='multiple'>","</select>"],thead:[1,"<table>","</table>"],col:[2,"<table><colgroup>","</colgroup></table>"],tr:[2,"<table><tbody>","</tbody></table>"],td:[3,"<table><tbody><tr>","</tr></tbody></table>"],_default:[0,"",""]};function ge(e,t){var i="undefined"!==typeof e.getElementsByTagName?e.getElementsByTagName(t||"*"):"undefined"!==typeof e.querySelectorAll?e.querySelectorAll(t||"*"):[];return void 0===t||t&&C.nodeName(e,t)?C.merge([e],i):i}function me(e,t){for(var i=0,n=e.length;i<n;i++)Q.set(e[i],"globalEval",!t||Q.get(t[i],"globalEval"))}fe.optgroup=fe.option,fe.tbody=fe.tfoot=fe.colgroup=fe.caption=fe.thead,fe.th=fe.td;var ve=/<|&#?\w+;/;function we(e,t,i,n,o){for(var s,a,r,l,c,h,u=t.createDocumentFragment(),d=[],p=0,f=e.length;p<f;p++)if(s=e[p],s||0===s)if("object"===C.type(s))C.merge(d,s.nodeType?[s]:s);else if(ve.test(s)){a=a||u.appendChild(t.createElement("div")),r=(de.exec(s)||["",""])[1].toLowerCase(),l=fe[r]||fe._default,a.innerHTML=l[1]+C.htmlPrefilter(s)+l[2],h=l[0];while(h--)a=a.lastChild;C.merge(d,a.childNodes),a=u.firstChild,a.textContent=""}else d.push(t.createTextNode(s));u.textContent="",p=0;while(s=d[p++])if(n&&C.inArray(s,n)>-1)o&&o.push(s);else if(c=C.contains(s.ownerDocument,s),a=ge(u.appendChild(s),"script"),c&&me(a),i){h=0;while(s=a[h++])pe.test(s.type||"")&&i.push(s)}return u}(function(){var e=r.createDocumentFragment(),t=e.appendChild(r.createElement("div")),i=r.createElement("input");i.setAttribute("type","radio"),i.setAttribute("checked","checked"),i.setAttribute("name","t"),t.appendChild(i),w.checkClone=t.cloneNode(!0).cloneNode(!0).lastChild.checked,t.innerHTML="<textarea>x</textarea>",w.noCloneChecked=!!t.cloneNode(!0).lastChild.defaultValue})();var ye=r.documentElement,be=/^key/,Ce=/^(?:mouse|pointer|contextmenu|drag|drop)|click/,_e=/^([^.]*)(?:\.(.+)|)/;function xe(){return!0}function ke(){return!1}function Se(){try{return r.activeElement}catch(e){}}function De(e,t,i,n,o,s){var a,r;if("object"===typeof t){for(r in"string"!==typeof i&&(n=n||i,i=void 0),t)De(e,r,i,n,t[r],s);return e}if(null==n&&null==o?(o=i,n=i=void 0):null==o&&("string"===typeof i?(o=n,n=void 0):(o=n,n=i,i=void 0)),!1===o)o=ke;else if(!o)return e;return 1===s&&(a=o,o=function(e){return C().off(e),a.apply(this,arguments)},o.guid=a.guid||(a.guid=C.guid++)),e.each((function(){C.event.add(this,t,o,n,i)}))}C.event={global:{},add:function(e,t,i,n,o){var s,a,r,l,c,h,u,d,p,f,g,m=Q.get(e);if(m){i.handler&&(s=i,i=s.handler,o=s.selector),o&&C.find.matchesSelector(ye,o),i.guid||(i.guid=C.guid++),(l=m.events)||(l=m.events={}),(a=m.handle)||(a=m.handle=function(t){return"undefined"!==typeof C&&C.event.triggered!==t.type?C.event.dispatch.apply(e,arguments):void 0}),t=(t||"").match(W)||[""],c=t.length;while(c--)r=_e.exec(t[c])||[],p=g=r[1],f=(r[2]||"").split(".").sort(),p&&(u=C.event.special[p]||{},p=(o?u.delegateType:u.bindType)||p,u=C.event.special[p]||{},h=C.extend({type:p,origType:g,data:n,handler:i,guid:i.guid,selector:o,needsContext:o&&C.expr.match.needsContext.test(o),namespace:f.join(".")},s),(d=l[p])||(d=l[p]=[],d.delegateCount=0,u.setup&&!1!==u.setup.call(e,n,f,a)||e.addEventListener&&e.addEventListener(p,a)),u.add&&(u.add.call(e,h),h.handler.guid||(h.handler.guid=i.guid)),o?d.splice(d.delegateCount++,0,h):d.push(h),C.event.global[p]=!0)}},remove:function(e,t,i,n,o){var s,a,r,l,c,h,u,d,p,f,g,m=Q.hasData(e)&&Q.get(e);if(m&&(l=m.events)){t=(t||"").match(W)||[""],c=t.length;while(c--)if(r=_e.exec(t[c])||[],p=g=r[1],f=(r[2]||"").split(".").sort(),p){u=C.event.special[p]||{},p=(n?u.delegateType:u.bindType)||p,d=l[p]||[],r=r[2]&&new RegExp("(^|\\.)"+f.join("\\.(?:.*\\.|)")+"(\\.|$)"),a=s=d.length;while(s--)h=d[s],!o&&g!==h.origType||i&&i.guid!==h.guid||r&&!r.test(h.namespace)||n&&n!==h.selector&&("**"!==n||!h.selector)||(d.splice(s,1),h.selector&&d.delegateCount--,u.remove&&u.remove.call(e,h));a&&!d.length&&(u.teardown&&!1!==u.teardown.call(e,f,m.handle)||C.removeEvent(e,p,m.handle),delete l[p])}else for(p in l)C.event.remove(e,p+t[c],i,n,!0);C.isEmptyObject(l)&&Q.remove(e,"handle events")}},dispatch:function(e){var t,i,n,o,s,a,r=C.event.fix(e),l=new Array(arguments.length),c=(Q.get(this,"events")||{})[r.type]||[],h=C.event.special[r.type]||{};for(l[0]=r,t=1;t<arguments.length;t++)l[t]=arguments[t];if(r.delegateTarget=this,!h.preDispatch||!1!==h.preDispatch.call(this,r)){a=C.event.handlers.call(this,r,c),t=0;while((o=a[t++])&&!r.isPropagationStopped()){r.currentTarget=o.elem,i=0;while((s=o.handlers[i++])&&!r.isImmediatePropagationStopped())r.rnamespace&&!r.rnamespace.test(s.namespace)||(r.handleObj=s,r.data=s.data,n=((C.event.special[s.origType]||{}).handle||s.handler).apply(o.elem,l),void 0!==n&&!1===(r.result=n)&&(r.preventDefault(),r.stopPropagation()))}return h.postDispatch&&h.postDispatch.call(this,r),r.result}},handlers:function(e,t){var i,n,o,s,a=[],r=t.delegateCount,l=e.target;if(r&&l.nodeType&&("click"!==e.type||isNaN(e.button)||e.button<1))for(;l!==this;l=l.parentNode||this)if(1===l.nodeType&&(!0!==l.disabled||"click"!==e.type)){for(n=[],i=0;i<r;i++)s=t[i],o=s.selector+" ",void 0===n[o]&&(n[o]=s.needsContext?C(o,this).index(l)>-1:C.find(o,this,null,[l]).length),n[o]&&n.push(s);n.length&&a.push({elem:l,handlers:n})}return r<t.length&&a.push({elem:this,handlers:t.slice(r)}),a},addProp:function(e,t){Object.defineProperty(C.Event.prototype,e,{enumerable:!0,configurable:!0,get:C.isFunction(t)?function(){if(this.originalEvent)return t(this.originalEvent)}:function(){if(this.originalEvent)return this.originalEvent[e]},set:function(t){Object.defineProperty(this,e,{enumerable:!0,configurable:!0,writable:!0,value:t})}})},fix:function(e){return e[C.expando]?e:new C.Event(e)},special:{load:{noBubble:!0},focus:{trigger:function(){if(this!==Se()&&this.focus)return this.focus(),!1},delegateType:"focusin"},blur:{trigger:function(){if(this===Se()&&this.blur)return this.blur(),!1},delegateType:"focusout"},click:{trigger:function(){if("checkbox"===this.type&&this.click&&C.nodeName(this,"input"))return this.click(),!1},_default:function(e){return C.nodeName(e.target,"a")}},beforeunload:{postDispatch:function(e){void 0!==e.result&&e.originalEvent&&(e.originalEvent.returnValue=e.result)}}}},C.removeEvent=function(e,t,i){e.removeEventListener&&e.removeEventListener(t,i)},C.Event=function(e,t){if(!(this instanceof C.Event))return new C.Event(e,t);e&&e.type?(this.originalEvent=e,this.type=e.type,this.isDefaultPrevented=e.defaultPrevented||void 0===e.defaultPrevented&&!1===e.returnValue?xe:ke,this.target=e.target&&3===e.target.nodeType?e.target.parentNode:e.target,this.currentTarget=e.currentTarget,this.relatedTarget=e.relatedTarget):this.type=e,t&&C.extend(this,t),this.timeStamp=e&&e.timeStamp||C.now(),this[C.expando]=!0},C.Event.prototype={constructor:C.Event,isDefaultPrevented:ke,isPropagationStopped:ke,isImmediatePropagationStopped:ke,isSimulated:!1,preventDefault:function(){var e=this.originalEvent;this.isDefaultPrevented=xe,e&&!this.isSimulated&&e.preventDefault()},stopPropagation:function(){var e=this.originalEvent;this.isPropagationStopped=xe,e&&!this.isSimulated&&e.stopPropagation()},stopImmediatePropagation:function(){var e=this.originalEvent;this.isImmediatePropagationStopped=xe,e&&!this.isSimulated&&e.stopImmediatePropagation(),this.stopPropagation()}},C.each({altKey:!0,bubbles:!0,cancelable:!0,changedTouches:!0,ctrlKey:!0,detail:!0,eventPhase:!0,metaKey:!0,pageX:!0,pageY:!0,shiftKey:!0,view:!0,char:!0,charCode:!0,key:!0,keyCode:!0,button:!0,buttons:!0,clientX:!0,clientY:!0,offsetX:!0,offsetY:!0,pointerId:!0,pointerType:!0,screenX:!0,screenY:!0,targetTouches:!0,toElement:!0,touches:!0,which:function(e){var t=e.button;return null==e.which&&be.test(e.type)?null!=e.charCode?e.charCode:e.keyCode:!e.which&&void 0!==t&&Ce.test(e.type)?1&t?1:2&t?3:4&t?2:0:e.which}},C.event.addProp),C.each({mouseenter:"mouseover",mouseleave:"mouseout",pointerenter:"pointerover",pointerleave:"pointerout"},(function(e,t){C.event.special[e]={delegateType:t,bindType:t,handle:function(e){var i,n=this,o=e.relatedTarget,s=e.handleObj;return o&&(o===n||C.contains(n,o))||(e.type=s.origType,i=s.handler.apply(this,arguments),e.type=t),i}}})),C.fn.extend({on:function(e,t,i,n){return De(this,e,t,i,n)},one:function(e,t,i,n){return De(this,e,t,i,n,1)},off:function(e,t,i){var n,o;if(e&&e.preventDefault&&e.handleObj)return n=e.handleObj,C(e.delegateTarget).off(n.namespace?n.origType+"."+n.namespace:n.origType,n.selector,n.handler),this;if("object"===typeof e){for(o in e)this.off(o,t,e[o]);return this}return!1!==t&&"function"!==typeof t||(i=t,t=void 0),!1===i&&(i=ke),this.each((function(){C.event.remove(this,e,i,t)}))}});var Re=/<(?!area|br|col|embed|hr|img|input|link|meta|param)(([a-z][^\/\0>\x20\t\r\n\f]*)[^>]*)\/>/gi,$e=/<script|<style|<link/i,Te=/checked\s*(?:[^=]|=\s*.checked.)/i,Pe=/^true\/(.*)/,Ie=/^\s*<!(?:\[CDATA\[|--)|(?:\]\]|--)>\s*$/g;function Me(e,t){return C.nodeName(e,"table")&&C.nodeName(11!==t.nodeType?t:t.firstChild,"tr")&&e.getElementsByTagName("tbody")[0]||e}function Ae(e){return e.type=(null!==e.getAttribute("type"))+"/"+e.type,e}function Ee(e){var t=Pe.exec(e.type);return t?e.type=t[1]:e.removeAttribute("type"),e}function Ne(e,t){var i,n,o,s,a,r,l,c;if(1===t.nodeType){if(Q.hasData(e)&&(s=Q.access(e),a=Q.set(t,s),c=s.events,c))for(o in delete a.handle,a.events={},c)for(i=0,n=c[o].length;i<n;i++)C.event.add(t,o,c[o][i]);J.hasData(e)&&(r=J.access(e),l=C.extend({},r),J.set(t,l))}}function ze(e,t){var i=t.nodeName.toLowerCase();"input"===i&&ue.test(e.type)?t.checked=e.checked:"input"!==i&&"textarea"!==i||(t.defaultValue=e.defaultValue)}function He(e,t,i,n){t=h.apply([],t);var o,s,a,r,l,c,u=0,d=e.length,p=d-1,f=t[0],g=C.isFunction(f);if(g||d>1&&"string"===typeof f&&!w.checkClone&&Te.test(f))return e.each((function(o){var s=e.eq(o);g&&(t[0]=f.call(this,o,s.html())),He(s,t,i,n)}));if(d&&(o=we(t,e[0].ownerDocument,!1,e,n),s=o.firstChild,1===o.childNodes.length&&(o=s),s||n)){for(a=C.map(ge(o,"script"),Ae),r=a.length;u<d;u++)l=o,u!==p&&(l=C.clone(l,!0,!0),r&&C.merge(a,ge(l,"script"))),i.call(e[u],l,u);if(r)for(c=a[a.length-1].ownerDocument,C.map(a,Ee),u=0;u<r;u++)l=a[u],pe.test(l.type||"")&&!Q.access(l,"globalEval")&&C.contains(c,l)&&(l.src?C._evalUrl&&C._evalUrl(l.src):y(l.textContent.replace(Ie,""),c))}return e}function Le(e,t,i){for(var n,o=t?C.filter(t,e):e,s=0;null!=(n=o[s]);s++)i||1!==n.nodeType||C.cleanData(ge(n)),n.parentNode&&(i&&C.contains(n.ownerDocument,n)&&me(ge(n,"script")),n.parentNode.removeChild(n));return e}C.extend({htmlPrefilter:function(e){return e.replace(Re,"<$1></$2>")},clone:function(e,t,i){var n,o,s,a,r=e.cloneNode(!0),l=C.contains(e.ownerDocument,e);if(!w.noCloneChecked&&(1===e.nodeType||11===e.nodeType)&&!C.isXMLDoc(e))for(a=ge(r),s=ge(e),n=0,o=s.length;n<o;n++)ze(s[n],a[n]);if(t)if(i)for(s=s||ge(e),a=a||ge(r),n=0,o=s.length;n<o;n++)Ne(s[n],a[n]);else Ne(e,r);return a=ge(r,"script"),a.length>0&&me(a,!l&&ge(e,"script")),r},cleanData:function(e){for(var t,i,n,o=C.event.special,s=0;void 0!==(i=e[s]);s++)if(U(i)){if(t=i[Q.expando]){if(t.events)for(n in t.events)o[n]?C.event.remove(i,n):C.removeEvent(i,n,t.handle);i[Q.expando]=void 0}i[J.expando]&&(i[J.expando]=void 0)}}}),C.fn.extend({detach:function(e){return Le(this,e,!0)},remove:function(e){return Le(this,e)},text:function(e){return K(this,(function(e){return void 0===e?C.text(this):this.empty().each((function(){1!==this.nodeType&&11!==this.nodeType&&9!==this.nodeType||(this.textContent=e)}))}),null,e,arguments.length)},append:function(){return He(this,arguments,(function(e){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var t=Me(this,e);t.appendChild(e)}}))},prepend:function(){return He(this,arguments,(function(e){if(1===this.nodeType||11===this.nodeType||9===this.nodeType){var t=Me(this,e);t.insertBefore(e,t.firstChild)}}))},before:function(){return He(this,arguments,(function(e){this.parentNode&&this.parentNode.insertBefore(e,this)}))},after:function(){return He(this,arguments,(function(e){this.parentNode&&this.parentNode.insertBefore(e,this.nextSibling)}))},empty:function(){for(var e,t=0;null!=(e=this[t]);t++)1===e.nodeType&&(C.cleanData(ge(e,!1)),e.textContent="");return this},clone:function(e,t){return e=null!=e&&e,t=null==t?e:t,this.map((function(){return C.clone(this,e,t)}))},html:function(e){return K(this,(function(e){var t=this[0]||{},i=0,n=this.length;if(void 0===e&&1===t.nodeType)return t.innerHTML;if("string"===typeof e&&!$e.test(e)&&!fe[(de.exec(e)||["",""])[1].toLowerCase()]){e=C.htmlPrefilter(e);try{for(;i<n;i++)t=this[i]||{},1===t.nodeType&&(C.cleanData(ge(t,!1)),t.innerHTML=e);t=0}catch(o){}}t&&this.empty().append(e)}),null,e,arguments.length)},replaceWith:function(){var e=[];return He(this,arguments,(function(t){var i=this.parentNode;C.inArray(this,e)<0&&(C.cleanData(ge(this)),i&&i.replaceChild(t,this))}),e)}}),C.each({appendTo:"append",prependTo:"prepend",insertBefore:"before",insertAfter:"after",replaceAll:"replaceWith"},(function(e,t){C.fn[e]=function(e){for(var i,n=[],o=C(e),s=o.length-1,a=0;a<=s;a++)i=a===s?this:this.clone(!0),C(o[a])[t](i),u.apply(n,i.get());return this.pushStack(n)}}));var Fe=/^margin/,We=new RegExp("^("+ie+")(?!px)[a-z%]+$","i"),Oe=function(e){var t=e.ownerDocument.defaultView;return t&&t.opener||(t=i),t.getComputedStyle(e)};function je(e,t,i){var n,o,s,a,r=e.style;return i=i||Oe(e),i&&(a=i.getPropertyValue(t)||i[t],""!==a||C.contains(e.ownerDocument,e)||(a=C.style(e,t)),!w.pixelMarginRight()&&We.test(a)&&Fe.test(t)&&(n=r.width,o=r.minWidth,s=r.maxWidth,r.minWidth=r.maxWidth=r.width=a,a=i.width,r.width=n,r.minWidth=o,r.maxWidth=s)),void 0!==a?a+"":a}function Be(e,t){return{get:function(){if(!e())return(this.get=t).apply(this,arguments);delete this.get}}}(function(){function e(){if(l){l.style.cssText="box-sizing:border-box;position:relative;display:block;margin:auto;border:1px;padding:1px;top:1%;width:50%",l.innerHTML="",ye.appendChild(a);var e=i.getComputedStyle(l);t="1%"!==e.top,s="2px"===e.marginLeft,n="4px"===e.width,l.style.marginRight="50%",o="4px"===e.marginRight,ye.removeChild(a),l=null}}var t,n,o,s,a=r.createElement("div"),l=r.createElement("div");l.style&&(l.style.backgroundClip="content-box",l.cloneNode(!0).style.backgroundClip="",w.clearCloneStyle="content-box"===l.style.backgroundClip,a.style.cssText="border:0;width:8px;height:0;top:0;left:-9999px;padding:0;margin-top:1px;position:absolute",a.appendChild(l),C.extend(w,{pixelPosition:function(){return e(),t},boxSizingReliable:function(){return e(),n},pixelMarginRight:function(){return e(),o},reliableMarginLeft:function(){return e(),s}}))})();var qe=/^(none|table(?!-c[ea]).+)/,Ve={position:"absolute",visibility:"hidden",display:"block"},Ge={letterSpacing:"0",fontWeight:"400"},Ye=["Webkit","Moz","ms"],Ke=r.createElement("div").style;function Ue(e){if(e in Ke)return e;var t=e[0].toUpperCase()+e.slice(1),i=Ye.length;while(i--)if(e=Ye[i]+t,e in Ke)return e}function Xe(e,t,i){var n=ne.exec(t);return n?Math.max(0,n[2]-(i||0))+(n[3]||"px"):t}function Qe(e,t,i,n,o){for(var s=i===(n?"border":"content")?4:"width"===t?1:0,a=0;s<4;s+=2)"margin"===i&&(a+=C.css(e,i+oe[s],!0,o)),n?("content"===i&&(a-=C.css(e,"padding"+oe[s],!0,o)),"margin"!==i&&(a-=C.css(e,"border"+oe[s]+"Width",!0,o))):(a+=C.css(e,"padding"+oe[s],!0,o),"padding"!==i&&(a+=C.css(e,"border"+oe[s]+"Width",!0,o)));return a}function Je(e,t,i){var n,o=!0,s=Oe(e),a="border-box"===C.css(e,"boxSizing",!1,s);if(e.getClientRects().length&&(n=e.getBoundingClientRect()[t]),n<=0||null==n){if(n=je(e,t,s),(n<0||null==n)&&(n=e.style[t]),We.test(n))return n;o=a&&(w.boxSizingReliable()||n===e.style[t]),n=parseFloat(n)||0}return n+Qe(e,t,i||(a?"border":"content"),o,s)+"px"}function Ze(e,t,i,n,o){return new Ze.prototype.init(e,t,i,n,o)}C.extend({cssHooks:{opacity:{get:function(e,t){if(t){var i=je(e,"opacity");return""===i?"1":i}}}},cssNumber:{animationIterationCount:!0,columnCount:!0,fillOpacity:!0,flexGrow:!0,flexShrink:!0,fontWeight:!0,lineHeight:!0,opacity:!0,order:!0,orphans:!0,widows:!0,zIndex:!0,zoom:!0},cssProps:{float:"cssFloat"},style:function(e,t,i,n){if(e&&3!==e.nodeType&&8!==e.nodeType&&e.style){var o,s,a,r=C.camelCase(t),l=e.style;if(t=C.cssProps[r]||(C.cssProps[r]=Ue(r)||r),a=C.cssHooks[t]||C.cssHooks[r],void 0===i)return a&&"get"in a&&void 0!==(o=a.get(e,!1,n))?o:l[t];s=typeof i,"string"===s&&(o=ne.exec(i))&&o[1]&&(i=re(e,t,o),s="number"),null!=i&&i===i&&("number"===s&&(i+=o&&o[3]||(C.cssNumber[r]?"":"px")),w.clearCloneStyle||""!==i||0!==t.indexOf("background")||(l[t]="inherit"),a&&"set"in a&&void 0===(i=a.set(e,i,n))||(l[t]=i))}},css:function(e,t,i,n){var o,s,a,r=C.camelCase(t);return t=C.cssProps[r]||(C.cssProps[r]=Ue(r)||r),a=C.cssHooks[t]||C.cssHooks[r],a&&"get"in a&&(o=a.get(e,!0,i)),void 0===o&&(o=je(e,t,n)),"normal"===o&&t in Ge&&(o=Ge[t]),""===i||i?(s=parseFloat(o),!0===i||isFinite(s)?s||0:o):o}}),C.each(["height","width"],(function(e,t){C.cssHooks[t]={get:function(e,i,n){if(i)return!qe.test(C.css(e,"display"))||e.getClientRects().length&&e.getBoundingClientRect().width?Je(e,t,n):ae(e,Ve,(function(){return Je(e,t,n)}))},set:function(e,i,n){var o,s=n&&Oe(e),a=n&&Qe(e,t,n,"border-box"===C.css(e,"boxSizing",!1,s),s);return a&&(o=ne.exec(i))&&"px"!==(o[3]||"px")&&(e.style[t]=i,i=C.css(e,t)),Xe(e,i,a)}}})),C.cssHooks.marginLeft=Be(w.reliableMarginLeft,(function(e,t){if(t)return(parseFloat(je(e,"marginLeft"))||e.getBoundingClientRect().left-ae(e,{marginLeft:0},(function(){return e.getBoundingClientRect().left})))+"px"})),C.each({margin:"",padding:"",border:"Width"},(function(e,t){C.cssHooks[e+t]={expand:function(i){for(var n=0,o={},s="string"===typeof i?i.split(" "):[i];n<4;n++)o[e+oe[n]+t]=s[n]||s[n-2]||s[0];return o}},Fe.test(e)||(C.cssHooks[e+t].set=Xe)})),C.fn.extend({css:function(e,t){return K(this,(function(e,t,i){var n,o,s={},a=0;if(C.isArray(t)){for(n=Oe(e),o=t.length;a<o;a++)s[t[a]]=C.css(e,t[a],!1,n);return s}return void 0!==i?C.style(e,t,i):C.css(e,t)}),e,t,arguments.length>1)}}),C.Tween=Ze,Ze.prototype={constructor:Ze,init:function(e,t,i,n,o,s){this.elem=e,this.prop=i,this.easing=o||C.easing._default,this.options=t,this.start=this.now=this.cur(),this.end=n,this.unit=s||(C.cssNumber[i]?"":"px")},cur:function(){var e=Ze.propHooks[this.prop];return e&&e.get?e.get(this):Ze.propHooks._default.get(this)},run:function(e){var t,i=Ze.propHooks[this.prop];return this.options.duration?this.pos=t=C.easing[this.easing](e,this.options.duration*e,0,1,this.options.duration):this.pos=t=e,this.now=(this.end-this.start)*t+this.start,this.options.step&&this.options.step.call(this.elem,this.now,this),i&&i.set?i.set(this):Ze.propHooks._default.set(this),this}},Ze.prototype.init.prototype=Ze.prototype,Ze.propHooks={_default:{get:function(e){var t;return 1!==e.elem.nodeType||null!=e.elem[e.prop]&&null==e.elem.style[e.prop]?e.elem[e.prop]:(t=C.css(e.elem,e.prop,""),t&&"auto"!==t?t:0)},set:function(e){C.fx.step[e.prop]?C.fx.step[e.prop](e):1!==e.elem.nodeType||null==e.elem.style[C.cssProps[e.prop]]&&!C.cssHooks[e.prop]?e.elem[e.prop]=e.now:C.style(e.elem,e.prop,e.now+e.unit)}}},Ze.propHooks.scrollTop=Ze.propHooks.scrollLeft={set:function(e){e.elem.nodeType&&e.elem.parentNode&&(e.elem[e.prop]=e.now)}},C.easing={linear:function(e){return e},swing:function(e){return.5-Math.cos(e*Math.PI)/2},_default:"swing"},C.fx=Ze.prototype.init,C.fx.step={};var et,tt,it=/^(?:toggle|show|hide)$/,nt=/queueHooks$/;function ot(){tt&&(i.requestAnimationFrame(ot),C.fx.tick())}function st(){return i.setTimeout((function(){et=void 0})),et=C.now()}function at(e,t){var i,n=0,o={height:e};for(t=t?1:0;n<4;n+=2-t)i=oe[n],o["margin"+i]=o["padding"+i]=e;return t&&(o.opacity=o.width=e),o}function rt(e,t,i){for(var n,o=(ht.tweeners[t]||[]).concat(ht.tweeners["*"]),s=0,a=o.length;s<a;s++)if(n=o[s].call(i,t,e))return n}function lt(e,t,i){var n,o,s,a,r,l,c,h,u="width"in t||"height"in t,d=this,p={},f=e.style,g=e.nodeType&&se(e),m=Q.get(e,"fxshow");for(n in i.queue||(a=C._queueHooks(e,"fx"),null==a.unqueued&&(a.unqueued=0,r=a.empty.fire,a.empty.fire=function(){a.unqueued||r()}),a.unqueued++,d.always((function(){d.always((function(){a.unqueued--,C.queue(e,"fx").length||a.empty.fire()}))}))),t)if(o=t[n],it.test(o)){if(delete t[n],s=s||"toggle"===o,o===(g?"hide":"show")){if("show"!==o||!m||void 0===m[n])continue;g=!0}p[n]=m&&m[n]||C.style(e,n)}if(l=!C.isEmptyObject(t),l||!C.isEmptyObject(p))for(n in u&&1===e.nodeType&&(i.overflow=[f.overflow,f.overflowX,f.overflowY],c=m&&m.display,null==c&&(c=Q.get(e,"display")),h=C.css(e,"display"),"none"===h&&(c?h=c:(he([e],!0),c=e.style.display||c,h=C.css(e,"display"),he([e]))),("inline"===h||"inline-block"===h&&null!=c)&&"none"===C.css(e,"float")&&(l||(d.done((function(){f.display=c})),null==c&&(h=f.display,c="none"===h?"":h)),f.display="inline-block")),i.overflow&&(f.overflow="hidden",d.always((function(){f.overflow=i.overflow[0],f.overflowX=i.overflow[1],f.overflowY=i.overflow[2]}))),l=!1,p)l||(m?"hidden"in m&&(g=m.hidden):m=Q.access(e,"fxshow",{display:c}),s&&(m.hidden=!g),g&&he([e],!0),d.done((function(){for(n in g||he([e]),Q.remove(e,"fxshow"),p)C.style(e,n,p[n])}))),l=rt(g?m[n]:0,n,d),n in m||(m[n]=l.start,g&&(l.end=l.start,l.start=0))}function ct(e,t){var i,n,o,s,a;for(i in e)if(n=C.camelCase(i),o=t[n],s=e[i],C.isArray(s)&&(o=s[1],s=e[i]=s[0]),i!==n&&(e[n]=s,delete e[i]),a=C.cssHooks[n],a&&"expand"in a)for(i in s=a.expand(s),delete e[n],s)i in e||(e[i]=s[i],t[i]=o);else t[n]=o}function ht(e,t,i){var n,o,s=0,a=ht.prefilters.length,r=C.Deferred().always((function(){delete l.elem})),l=function(){if(o)return!1;for(var t=et||st(),i=Math.max(0,c.startTime+c.duration-t),n=i/c.duration||0,s=1-n,a=0,l=c.tweens.length;a<l;a++)c.tweens[a].run(s);return r.notifyWith(e,[c,s,i]),s<1&&l?i:(r.resolveWith(e,[c]),!1)},c=r.promise({elem:e,props:C.extend({},t),opts:C.extend(!0,{specialEasing:{},easing:C.easing._default},i),originalProperties:t,originalOptions:i,startTime:et||st(),duration:i.duration,tweens:[],createTween:function(t,i){var n=C.Tween(e,c.opts,t,i,c.opts.specialEasing[t]||c.opts.easing);return c.tweens.push(n),n},stop:function(t){var i=0,n=t?c.tweens.length:0;if(o)return this;for(o=!0;i<n;i++)c.tweens[i].run(1);return t?(r.notifyWith(e,[c,1,0]),r.resolveWith(e,[c,t])):r.rejectWith(e,[c,t]),this}}),h=c.props;for(ct(h,c.opts.specialEasing);s<a;s++)if(n=ht.prefilters[s].call(c,e,h,c.opts),n)return C.isFunction(n.stop)&&(C._queueHooks(c.elem,c.opts.queue).stop=C.proxy(n.stop,n)),n;return C.map(h,rt,c),C.isFunction(c.opts.start)&&c.opts.start.call(e,c),C.fx.timer(C.extend(l,{elem:e,anim:c,queue:c.opts.queue})),c.progress(c.opts.progress).done(c.opts.done,c.opts.complete).fail(c.opts.fail).always(c.opts.always)}C.Animation=C.extend(ht,{tweeners:{"*":[function(e,t){var i=this.createTween(e,t);return re(i.elem,e,ne.exec(t),i),i}]},tweener:function(e,t){C.isFunction(e)?(t=e,e=["*"]):e=e.match(W);for(var i,n=0,o=e.length;n<o;n++)i=e[n],ht.tweeners[i]=ht.tweeners[i]||[],ht.tweeners[i].unshift(t)},prefilters:[lt],prefilter:function(e,t){t?ht.prefilters.unshift(e):ht.prefilters.push(e)}}),C.speed=function(e,t,i){var n=e&&"object"===typeof e?C.extend({},e):{complete:i||!i&&t||C.isFunction(e)&&e,duration:e,easing:i&&t||t&&!C.isFunction(t)&&t};return C.fx.off||r.hidden?n.duration=0:n.duration="number"===typeof n.duration?n.duration:n.duration in C.fx.speeds?C.fx.speeds[n.duration]:C.fx.speeds._default,null!=n.queue&&!0!==n.queue||(n.queue="fx"),n.old=n.complete,n.complete=function(){C.isFunction(n.old)&&n.old.call(this),n.queue&&C.dequeue(this,n.queue)},n},C.fn.extend({fadeTo:function(e,t,i,n){return this.filter(se).css("opacity",0).show().end().animate({opacity:t},e,i,n)},animate:function(e,t,i,n){var o=C.isEmptyObject(e),s=C.speed(t,i,n),a=function(){var t=ht(this,C.extend({},e),s);(o||Q.get(this,"finish"))&&t.stop(!0)};return a.finish=a,o||!1===s.queue?this.each(a):this.queue(s.queue,a)},stop:function(e,t,i){var n=function(e){var t=e.stop;delete e.stop,t(i)};return"string"!==typeof e&&(i=t,t=e,e=void 0),t&&!1!==e&&this.queue(e||"fx",[]),this.each((function(){var t=!0,o=null!=e&&e+"queueHooks",s=C.timers,a=Q.get(this);if(o)a[o]&&a[o].stop&&n(a[o]);else for(o in a)a[o]&&a[o].stop&&nt.test(o)&&n(a[o]);for(o=s.length;o--;)s[o].elem!==this||null!=e&&s[o].queue!==e||(s[o].anim.stop(i),t=!1,s.splice(o,1));!t&&i||C.dequeue(this,e)}))},finish:function(e){return!1!==e&&(e=e||"fx"),this.each((function(){var t,i=Q.get(this),n=i[e+"queue"],o=i[e+"queueHooks"],s=C.timers,a=n?n.length:0;for(i.finish=!0,C.queue(this,e,[]),o&&o.stop&&o.stop.call(this,!0),t=s.length;t--;)s[t].elem===this&&s[t].queue===e&&(s[t].anim.stop(!0),s.splice(t,1));for(t=0;t<a;t++)n[t]&&n[t].finish&&n[t].finish.call(this);delete i.finish}))}}),C.each(["toggle","show","hide"],(function(e,t){var i=C.fn[t];C.fn[t]=function(e,n,o){return null==e||"boolean"===typeof e?i.apply(this,arguments):this.animate(at(t,!0),e,n,o)}})),C.each({slideDown:at("show"),slideUp:at("hide"),slideToggle:at("toggle"),fadeIn:{opacity:"show"},fadeOut:{opacity:"hide"},fadeToggle:{opacity:"toggle"}},(function(e,t){C.fn[e]=function(e,i,n){return this.animate(t,e,i,n)}})),C.timers=[],C.fx.tick=function(){var e,t=0,i=C.timers;for(et=C.now();t<i.length;t++)e=i[t],e()||i[t]!==e||i.splice(t--,1);i.length||C.fx.stop(),et=void 0},C.fx.timer=function(e){C.timers.push(e),e()?C.fx.start():C.timers.pop()},C.fx.interval=13,C.fx.start=function(){tt||(tt=i.requestAnimationFrame?i.requestAnimationFrame(ot):i.setInterval(C.fx.tick,C.fx.interval))},C.fx.stop=function(){i.cancelAnimationFrame?i.cancelAnimationFrame(tt):i.clearInterval(tt),tt=null},C.fx.speeds={slow:600,fast:200,_default:400},C.fn.delay=function(e,t){return e=C.fx&&C.fx.speeds[e]||e,t=t||"fx",this.queue(t,(function(t,n){var o=i.setTimeout(t,e);n.stop=function(){i.clearTimeout(o)}}))},function(){var e=r.createElement("input"),t=r.createElement("select"),i=t.appendChild(r.createElement("option"));e.type="checkbox",w.checkOn=""!==e.value,w.optSelected=i.selected,e=r.createElement("input"),e.value="t",e.type="radio",w.radioValue="t"===e.value}();var ut,dt=C.expr.attrHandle;C.fn.extend({attr:function(e,t){return K(this,C.attr,e,t,arguments.length>1)},removeAttr:function(e){return this.each((function(){C.removeAttr(this,e)}))}}),C.extend({attr:function(e,t,i){var n,o,s=e.nodeType;if(3!==s&&8!==s&&2!==s)return"undefined"===typeof e.getAttribute?C.prop(e,t,i):(1===s&&C.isXMLDoc(e)||(o=C.attrHooks[t.toLowerCase()]||(C.expr.match.bool.test(t)?ut:void 0)),void 0!==i?null===i?void C.removeAttr(e,t):o&&"set"in o&&void 0!==(n=o.set(e,i,t))?n:(e.setAttribute(t,i+""),i):o&&"get"in o&&null!==(n=o.get(e,t))?n:(n=C.find.attr(e,t),null==n?void 0:n))},attrHooks:{type:{set:function(e,t){if(!w.radioValue&&"radio"===t&&C.nodeName(e,"input")){var i=e.value;return e.setAttribute("type",t),i&&(e.value=i),t}}}},removeAttr:function(e,t){var i,n=0,o=t&&t.match(W);if(o&&1===e.nodeType)while(i=o[n++])e.removeAttribute(i)}}),ut={set:function(e,t,i){return!1===t?C.removeAttr(e,i):e.setAttribute(i,i),i}},C.each(C.expr.match.bool.source.match(/\w+/g),(function(e,t){var i=dt[t]||C.find.attr;dt[t]=function(e,t,n){var o,s,a=t.toLowerCase();return n||(s=dt[a],dt[a]=o,o=null!=i(e,t,n)?a:null,dt[a]=s),o}}));var pt=/^(?:input|select|textarea|button)$/i,ft=/^(?:a|area)$/i;C.fn.extend({prop:function(e,t){return K(this,C.prop,e,t,arguments.length>1)},removeProp:function(e){return this.each((function(){delete this[C.propFix[e]||e]}))}}),C.extend({prop:function(e,t,i){var n,o,s=e.nodeType;if(3!==s&&8!==s&&2!==s)return 1===s&&C.isXMLDoc(e)||(t=C.propFix[t]||t,o=C.propHooks[t]),void 0!==i?o&&"set"in o&&void 0!==(n=o.set(e,i,t))?n:e[t]=i:o&&"get"in o&&null!==(n=o.get(e,t))?n:e[t]},propHooks:{tabIndex:{get:function(e){var t=C.find.attr(e,"tabindex");return t?parseInt(t,10):pt.test(e.nodeName)||ft.test(e.nodeName)&&e.href?0:-1}}},propFix:{for:"htmlFor",class:"className"}}),w.optSelected||(C.propHooks.selected={get:function(e){var t=e.parentNode;return t&&t.parentNode&&t.parentNode.selectedIndex,null},set:function(e){var t=e.parentNode;t&&(t.selectedIndex,t.parentNode&&t.parentNode.selectedIndex)}}),C.each(["tabIndex","readOnly","maxLength","cellSpacing","cellPadding","rowSpan","colSpan","useMap","frameBorder","contentEditable"],(function(){C.propFix[this.toLowerCase()]=this}));var gt=/[\t\r\n\f]/g;function mt(e){return e.getAttribute&&e.getAttribute("class")||""}C.fn.extend({addClass:function(e){var t,i,n,o,s,a,r,l=0;if(C.isFunction(e))return this.each((function(t){C(this).addClass(e.call(this,t,mt(this)))}));if("string"===typeof e&&e){t=e.match(W)||[];while(i=this[l++])if(o=mt(i),n=1===i.nodeType&&(" "+o+" ").replace(gt," "),n){a=0;while(s=t[a++])n.indexOf(" "+s+" ")<0&&(n+=s+" ");r=C.trim(n),o!==r&&i.setAttribute("class",r)}}return this},removeClass:function(e){var t,i,n,o,s,a,r,l=0;if(C.isFunction(e))return this.each((function(t){C(this).removeClass(e.call(this,t,mt(this)))}));if(!arguments.length)return this.attr("class","");if("string"===typeof e&&e){t=e.match(W)||[];while(i=this[l++])if(o=mt(i),n=1===i.nodeType&&(" "+o+" ").replace(gt," "),n){a=0;while(s=t[a++])while(n.indexOf(" "+s+" ")>-1)n=n.replace(" "+s+" "," ");r=C.trim(n),o!==r&&i.setAttribute("class",r)}}return this},toggleClass:function(e,t){var i=typeof e;return"boolean"===typeof t&&"string"===i?t?this.addClass(e):this.removeClass(e):C.isFunction(e)?this.each((function(i){C(this).toggleClass(e.call(this,i,mt(this),t),t)})):this.each((function(){var t,n,o,s;if("string"===i){n=0,o=C(this),s=e.match(W)||[];while(t=s[n++])o.hasClass(t)?o.removeClass(t):o.addClass(t)}else void 0!==e&&"boolean"!==i||(t=mt(this),t&&Q.set(this,"__className__",t),this.setAttribute&&this.setAttribute("class",t||!1===e?"":Q.get(this,"__className__")||""))}))},hasClass:function(e){var t,i,n=0;t=" "+e+" ";while(i=this[n++])if(1===i.nodeType&&(" "+mt(i)+" ").replace(gt," ").indexOf(t)>-1)return!0;return!1}});var vt=/\r/g,wt=/[\x20\t\r\n\f]+/g;C.fn.extend({val:function(e){var t,i,n,o=this[0];return arguments.length?(n=C.isFunction(e),this.each((function(i){var o;1===this.nodeType&&(o=n?e.call(this,i,C(this).val()):e,null==o?o="":"number"===typeof o?o+="":C.isArray(o)&&(o=C.map(o,(function(e){return null==e?"":e+""}))),t=C.valHooks[this.type]||C.valHooks[this.nodeName.toLowerCase()],t&&"set"in t&&void 0!==t.set(this,o,"value")||(this.value=o))}))):o?(t=C.valHooks[o.type]||C.valHooks[o.nodeName.toLowerCase()],t&&"get"in t&&void 0!==(i=t.get(o,"value"))?i:(i=o.value,"string"===typeof i?i.replace(vt,""):null==i?"":i)):void 0}}),C.extend({valHooks:{option:{get:function(e){var t=C.find.attr(e,"value");return null!=t?t:C.trim(C.text(e)).replace(wt," ")}},select:{get:function(e){for(var t,i,n=e.options,o=e.selectedIndex,s="select-one"===e.type,a=s?null:[],r=s?o+1:n.length,l=o<0?r:s?o:0;l<r;l++)if(i=n[l],(i.selected||l===o)&&!i.disabled&&(!i.parentNode.disabled||!C.nodeName(i.parentNode,"optgroup"))){if(t=C(i).val(),s)return t;a.push(t)}return a},set:function(e,t){var i,n,o=e.options,s=C.makeArray(t),a=o.length;while(a--)n=o[a],(n.selected=C.inArray(C.valHooks.option.get(n),s)>-1)&&(i=!0);return i||(e.selectedIndex=-1),s}}}}),C.each(["radio","checkbox"],(function(){C.valHooks[this]={set:function(e,t){if(C.isArray(t))return e.checked=C.inArray(C(e).val(),t)>-1}},w.checkOn||(C.valHooks[this].get=function(e){return null===e.getAttribute("value")?"on":e.value})}));var yt=/^(?:focusinfocus|focusoutblur)$/;C.extend(C.event,{trigger:function(e,t,n,o){var s,a,l,c,h,u,d,p=[n||r],f=g.call(e,"type")?e.type:e,m=g.call(e,"namespace")?e.namespace.split("."):[];if(a=l=n=n||r,3!==n.nodeType&&8!==n.nodeType&&!yt.test(f+C.event.triggered)&&(f.indexOf(".")>-1&&(m=f.split("."),f=m.shift(),m.sort()),h=f.indexOf(":")<0&&"on"+f,e=e[C.expando]?e:new C.Event(f,"object"===typeof e&&e),e.isTrigger=o?2:3,e.namespace=m.join("."),e.rnamespace=e.namespace?new RegExp("(^|\\.)"+m.join("\\.(?:.*\\.|)")+"(\\.|$)"):null,e.result=void 0,e.target||(e.target=n),t=null==t?[e]:C.makeArray(t,[e]),d=C.event.special[f]||{},o||!d.trigger||!1!==d.trigger.apply(n,t))){if(!o&&!d.noBubble&&!C.isWindow(n)){for(c=d.delegateType||f,yt.test(c+f)||(a=a.parentNode);a;a=a.parentNode)p.push(a),l=a;l===(n.ownerDocument||r)&&p.push(l.defaultView||l.parentWindow||i)}s=0;while((a=p[s++])&&!e.isPropagationStopped())e.type=s>1?c:d.bindType||f,u=(Q.get(a,"events")||{})[e.type]&&Q.get(a,"handle"),u&&u.apply(a,t),u=h&&a[h],u&&u.apply&&U(a)&&(e.result=u.apply(a,t),!1===e.result&&e.preventDefault());return e.type=f,o||e.isDefaultPrevented()||d._default&&!1!==d._default.apply(p.pop(),t)||!U(n)||h&&C.isFunction(n[f])&&!C.isWindow(n)&&(l=n[h],l&&(n[h]=null),C.event.triggered=f,n[f](),C.event.triggered=void 0,l&&(n[h]=l)),e.result}},simulate:function(e,t,i){var n=C.extend(new C.Event,i,{type:e,isSimulated:!0});C.event.trigger(n,null,t)}}),C.fn.extend({trigger:function(e,t){return this.each((function(){C.event.trigger(e,t,this)}))},triggerHandler:function(e,t){var i=this[0];if(i)return C.event.trigger(e,t,i,!0)}}),C.each("blur focus focusin focusout resize scroll click dblclick mousedown mouseup mousemove mouseover mouseout mouseenter mouseleave change select submit keydown keypress keyup contextmenu".split(" "),(function(e,t){C.fn[t]=function(e,i){return arguments.length>0?this.on(t,null,e,i):this.trigger(t)}})),C.fn.extend({hover:function(e,t){return this.mouseenter(e).mouseleave(t||e)}}),w.focusin="onfocusin"in i,w.focusin||C.each({focus:"focusin",blur:"focusout"},(function(e,t){var i=function(e){C.event.simulate(t,e.target,C.event.fix(e))};C.event.special[t]={setup:function(){var n=this.ownerDocument||this,o=Q.access(n,t);o||n.addEventListener(e,i,!0),Q.access(n,t,(o||0)+1)},teardown:function(){var n=this.ownerDocument||this,o=Q.access(n,t)-1;o?Q.access(n,t,o):(n.removeEventListener(e,i,!0),Q.remove(n,t))}}}));var bt=i.location,Ct=C.now(),_t=/\?/;C.parseXML=function(e){var t;if(!e||"string"!==typeof e)return null;try{t=(new i.DOMParser).parseFromString(e,"text/xml")}catch(n){t=void 0}return t&&!t.getElementsByTagName("parsererror").length||C.error("Invalid XML: "+e),t};var xt=/\[\]$/,kt=/\r?\n/g,St=/^(?:submit|button|image|reset|file)$/i,Dt=/^(?:input|select|textarea|keygen)/i;function Rt(e,t,i,n){var o;if(C.isArray(t))C.each(t,(function(t,o){i||xt.test(e)?n(e,o):Rt(e+"["+("object"===typeof o&&null!=o?t:"")+"]",o,i,n)}));else if(i||"object"!==C.type(t))n(e,t);else for(o in t)Rt(e+"["+o+"]",t[o],i,n)}C.param=function(e,t){var i,n=[],o=function(e,t){var i=C.isFunction(t)?t():t;n[n.length]=encodeURIComponent(e)+"="+encodeURIComponent(null==i?"":i)};if(C.isArray(e)||e.jquery&&!C.isPlainObject(e))C.each(e,(function(){o(this.name,this.value)}));else for(i in e)Rt(i,e[i],t,o);return n.join("&")},C.fn.extend({serialize:function(){return C.param(this.serializeArray())},serializeArray:function(){return this.map((function(){var e=C.prop(this,"elements");return e?C.makeArray(e):this})).filter((function(){var e=this.type;return this.name&&!C(this).is(":disabled")&&Dt.test(this.nodeName)&&!St.test(e)&&(this.checked||!ue.test(e))})).map((function(e,t){var i=C(this).val();return null==i?null:C.isArray(i)?C.map(i,(function(e){return{name:t.name,value:e.replace(kt,"\r\n")}})):{name:t.name,value:i.replace(kt,"\r\n")}})).get()}});var $t=/%20/g,Tt=/#.*$/,Pt=/([?&])_=[^&]*/,It=/^(.*?):[ \t]*([^\r\n]*)$/gm,Mt=/^(?:about|app|app-storage|.+-extension|file|res|widget):$/,At=/^(?:GET|HEAD)$/,Et=/^\/\//,Nt={},zt={},Ht="*/".concat("*"),Lt=r.createElement("a");function Ft(e){return function(t,i){"string"!==typeof t&&(i=t,t="*");var n,o=0,s=t.toLowerCase().match(W)||[];if(C.isFunction(i))while(n=s[o++])"+"===n[0]?(n=n.slice(1)||"*",(e[n]=e[n]||[]).unshift(i)):(e[n]=e[n]||[]).push(i)}}function Wt(e,t,i,n){var o={},s=e===zt;function a(r){var l;return o[r]=!0,C.each(e[r]||[],(function(e,r){var c=r(t,i,n);return"string"!==typeof c||s||o[c]?s?!(l=c):void 0:(t.dataTypes.unshift(c),a(c),!1)})),l}return a(t.dataTypes[0])||!o["*"]&&a("*")}function Ot(e,t){var i,n,o=C.ajaxSettings.flatOptions||{};for(i in t)void 0!==t[i]&&((o[i]?e:n||(n={}))[i]=t[i]);return n&&C.extend(!0,e,n),e}function jt(e,t,i){var n,o,s,a,r=e.contents,l=e.dataTypes;while("*"===l[0])l.shift(),void 0===n&&(n=e.mimeType||t.getResponseHeader("Content-Type"));if(n)for(o in r)if(r[o]&&r[o].test(n)){l.unshift(o);break}if(l[0]in i)s=l[0];else{for(o in i){if(!l[0]||e.converters[o+" "+l[0]]){s=o;break}a||(a=o)}s=s||a}if(s)return s!==l[0]&&l.unshift(s),i[s]}function Bt(e,t,i,n){var o,s,a,r,l,c={},h=e.dataTypes.slice();if(h[1])for(a in e.converters)c[a.toLowerCase()]=e.converters[a];s=h.shift();while(s)if(e.responseFields[s]&&(i[e.responseFields[s]]=t),!l&&n&&e.dataFilter&&(t=e.dataFilter(t,e.dataType)),l=s,s=h.shift(),s)if("*"===s)s=l;else if("*"!==l&&l!==s){if(a=c[l+" "+s]||c["* "+s],!a)for(o in c)if(r=o.split(" "),r[1]===s&&(a=c[l+" "+r[0]]||c["* "+r[0]],a)){!0===a?a=c[o]:!0!==c[o]&&(s=r[0],h.unshift(r[1]));break}if(!0!==a)if(a&&e.throws)t=a(t);else try{t=a(t)}catch(u){return{state:"parsererror",error:a?u:"No conversion from "+l+" to "+s}}}return{state:"success",data:t}}Lt.href=bt.href,C.extend({active:0,lastModified:{},etag:{},ajaxSettings:{url:bt.href,type:"GET",isLocal:Mt.test(bt.protocol),global:!0,processData:!0,async:!0,contentType:"application/x-www-form-urlencoded; charset=UTF-8",accepts:{"*":Ht,text:"text/plain",html:"text/html",xml:"application/xml, text/xml",json:"application/json, text/javascript"},contents:{xml:/\bxml\b/,html:/\bhtml/,json:/\bjson\b/},responseFields:{xml:"responseXML",text:"responseText",json:"responseJSON"},converters:{"* text":String,"text html":!0,"text json":JSON.parse,"text xml":C.parseXML},flatOptions:{url:!0,context:!0}},ajaxSetup:function(e,t){return t?Ot(Ot(e,C.ajaxSettings),t):Ot(C.ajaxSettings,e)},ajaxPrefilter:Ft(Nt),ajaxTransport:Ft(zt),ajax:function(e,t){"object"===typeof e&&(t=e,e=void 0),t=t||{};var n,o,s,a,l,c,h,u,d,p,f=C.ajaxSetup({},t),g=f.context||f,m=f.context&&(g.nodeType||g.jquery)?C(g):C.event,v=C.Deferred(),w=C.Callbacks("once memory"),y=f.statusCode||{},b={},_={},x="canceled",k={readyState:0,getResponseHeader:function(e){var t;if(h){if(!a){a={};while(t=It.exec(s))a[t[1].toLowerCase()]=t[2]}t=a[e.toLowerCase()]}return null==t?null:t},getAllResponseHeaders:function(){return h?s:null},setRequestHeader:function(e,t){return null==h&&(e=_[e.toLowerCase()]=_[e.toLowerCase()]||e,b[e]=t),this},overrideMimeType:function(e){return null==h&&(f.mimeType=e),this},statusCode:function(e){var t;if(e)if(h)k.always(e[k.status]);else for(t in e)y[t]=[y[t],e[t]];return this},abort:function(e){var t=e||x;return n&&n.abort(t),S(0,t),this}};if(v.promise(k),f.url=((e||f.url||bt.href)+"").replace(Et,bt.protocol+"//"),f.type=t.method||t.type||f.method||f.type,f.dataTypes=(f.dataType||"*").toLowerCase().match(W)||[""],null==f.crossDomain){c=r.createElement("a");try{c.href=f.url,c.href=c.href,f.crossDomain=Lt.protocol+"//"+Lt.host!==c.protocol+"//"+c.host}catch(D){f.crossDomain=!0}}if(f.data&&f.processData&&"string"!==typeof f.data&&(f.data=C.param(f.data,f.traditional)),Wt(Nt,f,t,k),h)return k;for(d in u=C.event&&f.global,u&&0===C.active++&&C.event.trigger("ajaxStart"),f.type=f.type.toUpperCase(),f.hasContent=!At.test(f.type),o=f.url.replace(Tt,""),f.hasContent?f.data&&f.processData&&0===(f.contentType||"").indexOf("application/x-www-form-urlencoded")&&(f.data=f.data.replace($t,"+")):(p=f.url.slice(o.length),f.data&&(o+=(_t.test(o)?"&":"?")+f.data,delete f.data),!1===f.cache&&(o=o.replace(Pt,""),p=(_t.test(o)?"&":"?")+"_="+Ct+++p),f.url=o+p),f.ifModified&&(C.lastModified[o]&&k.setRequestHeader("If-Modified-Since",C.lastModified[o]),C.etag[o]&&k.setRequestHeader("If-None-Match",C.etag[o])),(f.data&&f.hasContent&&!1!==f.contentType||t.contentType)&&k.setRequestHeader("Content-Type",f.contentType),k.setRequestHeader("Accept",f.dataTypes[0]&&f.accepts[f.dataTypes[0]]?f.accepts[f.dataTypes[0]]+("*"!==f.dataTypes[0]?", "+Ht+"; q=0.01":""):f.accepts["*"]),f.headers)k.setRequestHeader(d,f.headers[d]);if(f.beforeSend&&(!1===f.beforeSend.call(g,k,f)||h))return k.abort();if(x="abort",w.add(f.complete),k.done(f.success),k.fail(f.error),n=Wt(zt,f,t,k),n){if(k.readyState=1,u&&m.trigger("ajaxSend",[k,f]),h)return k;f.async&&f.timeout>0&&(l=i.setTimeout((function(){k.abort("timeout")}),f.timeout));try{h=!1,n.send(b,S)}catch(D){if(h)throw D;S(-1,D)}}else S(-1,"No Transport");function S(e,t,a,r){var c,d,p,b,_,x=t;h||(h=!0,l&&i.clearTimeout(l),n=void 0,s=r||"",k.readyState=e>0?4:0,c=e>=200&&e<300||304===e,a&&(b=jt(f,k,a)),b=Bt(f,b,k,c),c?(f.ifModified&&(_=k.getResponseHeader("Last-Modified"),_&&(C.lastModified[o]=_),_=k.getResponseHeader("etag"),_&&(C.etag[o]=_)),204===e||"HEAD"===f.type?x="nocontent":304===e?x="notmodified":(x=b.state,d=b.data,p=b.error,c=!p)):(p=x,!e&&x||(x="error",e<0&&(e=0))),k.status=e,k.statusText=(t||x)+"",c?v.resolveWith(g,[d,x,k]):v.rejectWith(g,[k,x,p]),k.statusCode(y),y=void 0,u&&m.trigger(c?"ajaxSuccess":"ajaxError",[k,f,c?d:p]),w.fireWith(g,[k,x]),u&&(m.trigger("ajaxComplete",[k,f]),--C.active||C.event.trigger("ajaxStop")))}return k},getJSON:function(e,t,i){return C.get(e,t,i,"json")},getScript:function(e,t){return C.get(e,void 0,t,"script")}}),C.each(["get","post"],(function(e,t){C[t]=function(e,i,n,o){return C.isFunction(i)&&(o=o||n,n=i,i=void 0),C.ajax(C.extend({url:e,type:t,dataType:o,data:i,success:n},C.isPlainObject(e)&&e))}})),C._evalUrl=function(e){return C.ajax({url:e,type:"GET",dataType:"script",cache:!0,async:!1,global:!1,throws:!0})},C.fn.extend({wrapAll:function(e){var t;return this[0]&&(C.isFunction(e)&&(e=e.call(this[0])),t=C(e,this[0].ownerDocument).eq(0).clone(!0),this[0].parentNode&&t.insertBefore(this[0]),t.map((function(){var e=this;while(e.firstElementChild)e=e.firstElementChild;return e})).append(this)),this},wrapInner:function(e){return C.isFunction(e)?this.each((function(t){C(this).wrapInner(e.call(this,t))})):this.each((function(){var t=C(this),i=t.contents();i.length?i.wrapAll(e):t.append(e)}))},wrap:function(e){var t=C.isFunction(e);return this.each((function(i){C(this).wrapAll(t?e.call(this,i):e)}))},unwrap:function(e){return this.parent(e).not("body").each((function(){C(this).replaceWith(this.childNodes)})),this}}),C.expr.pseudos.hidden=function(e){return!C.expr.pseudos.visible(e)},C.expr.pseudos.visible=function(e){return!!(e.offsetWidth||e.offsetHeight||e.getClientRects().length)},C.ajaxSettings.xhr=function(){try{return new i.XMLHttpRequest}catch(e){}};var qt={0:200,1223:204},Vt=C.ajaxSettings.xhr();w.cors=!!Vt&&"withCredentials"in Vt,w.ajax=Vt=!!Vt,C.ajaxTransport((function(e){var t,n;if(w.cors||Vt&&!e.crossDomain)return{send:function(o,s){var a,r=e.xhr();if(r.open(e.type,e.url,e.async,e.username,e.password),e.xhrFields)for(a in e.xhrFields)r[a]=e.xhrFields[a];for(a in e.mimeType&&r.overrideMimeType&&r.overrideMimeType(e.mimeType),e.crossDomain||o["X-Requested-With"]||(o["X-Requested-With"]="XMLHttpRequest"),o)r.setRequestHeader(a,o[a]);t=function(e){return function(){t&&(t=n=r.onload=r.onerror=r.onabort=r.onreadystatechange=null,"abort"===e?r.abort():"error"===e?"number"!==typeof r.status?s(0,"error"):s(r.status,r.statusText):s(qt[r.status]||r.status,r.statusText,"text"!==(r.responseType||"text")||"string"!==typeof r.responseText?{binary:r.response}:{text:r.responseText},r.getAllResponseHeaders()))}},r.onload=t(),n=r.onerror=t("error"),void 0!==r.onabort?r.onabort=n:r.onreadystatechange=function(){4===r.readyState&&i.setTimeout((function(){t&&n()}))},t=t("abort");try{r.send(e.hasContent&&e.data||null)}catch(l){if(t)throw l}},abort:function(){t&&t()}}})),C.ajaxPrefilter((function(e){e.crossDomain&&(e.contents.script=!1)})),C.ajaxSetup({accepts:{script:"text/javascript, application/javascript, application/ecmascript, application/x-ecmascript"},contents:{script:/\b(?:java|ecma)script\b/},converters:{"text script":function(e){return C.globalEval(e),e}}}),C.ajaxPrefilter("script",(function(e){void 0===e.cache&&(e.cache=!1),e.crossDomain&&(e.type="GET")})),C.ajaxTransport("script",(function(e){var t,i;if(e.crossDomain)return{send:function(n,o){t=C("<script>").prop({charset:e.scriptCharset,src:e.url}).on("load error",i=function(e){t.remove(),i=null,e&&o("error"===e.type?404:200,e.type)}),r.head.appendChild(t[0])},abort:function(){i&&i()}}}));var Gt=[],Yt=/(=)\?(?=&|$)|\?\?/;function Kt(e){return C.isWindow(e)?e:9===e.nodeType&&e.defaultView}C.ajaxSetup({jsonp:"callback",jsonpCallback:function(){var e=Gt.pop()||C.expando+"_"+Ct++;return this[e]=!0,e}}),C.ajaxPrefilter("json jsonp",(function(e,t,n){var o,s,a,r=!1!==e.jsonp&&(Yt.test(e.url)?"url":"string"===typeof e.data&&0===(e.contentType||"").indexOf("application/x-www-form-urlencoded")&&Yt.test(e.data)&&"data");if(r||"jsonp"===e.dataTypes[0])return o=e.jsonpCallback=C.isFunction(e.jsonpCallback)?e.jsonpCallback():e.jsonpCallback,r?e[r]=e[r].replace(Yt,"$1"+o):!1!==e.jsonp&&(e.url+=(_t.test(e.url)?"&":"?")+e.jsonp+"="+o),e.converters["script json"]=function(){return a||C.error(o+" was not called"),a[0]},e.dataTypes[0]="json",s=i[o],i[o]=function(){a=arguments},n.always((function(){void 0===s?C(i).removeProp(o):i[o]=s,e[o]&&(e.jsonpCallback=t.jsonpCallback,Gt.push(o)),a&&C.isFunction(s)&&s(a[0]),a=s=void 0})),"script"})),w.createHTMLDocument=function(){var e=r.implementation.createHTMLDocument("").body;return e.innerHTML="<form></form><form></form>",2===e.childNodes.length}(),C.parseHTML=function(e,t,i){return"string"!==typeof e?[]:("boolean"===typeof t&&(i=t,t=!1),t||(w.createHTMLDocument?(t=r.implementation.createHTMLDocument(""),n=t.createElement("base"),n.href=r.location.href,t.head.appendChild(n)):t=r),o=I.exec(e),s=!i&&[],o?[t.createElement(o[1])]:(o=we([e],t,s),s&&s.length&&C(s).remove(),C.merge([],o.childNodes)));var n,o,s},C.fn.load=function(e,t,i){var n,o,s,a=this,r=e.indexOf(" ");return r>-1&&(n=C.trim(e.slice(r)),e=e.slice(0,r)),C.isFunction(t)?(i=t,t=void 0):t&&"object"===typeof t&&(o="POST"),a.length>0&&C.ajax({url:e,type:o||"GET",dataType:"html",data:t}).done((function(e){s=arguments,a.html(n?C("<div>").append(C.parseHTML(e)).find(n):e)})).always(i&&function(e,t){a.each((function(){i.apply(this,s||[e.responseText,t,e])}))}),this},C.each(["ajaxStart","ajaxStop","ajaxComplete","ajaxError","ajaxSuccess","ajaxSend"],(function(e,t){C.fn[t]=function(e){return this.on(t,e)}})),C.expr.pseudos.animated=function(e){return C.grep(C.timers,(function(t){return e===t.elem})).length},C.offset={setOffset:function(e,t,i){var n,o,s,a,r,l,c,h=C.css(e,"position"),u=C(e),d={};"static"===h&&(e.style.position="relative"),r=u.offset(),s=C.css(e,"top"),l=C.css(e,"left"),c=("absolute"===h||"fixed"===h)&&(s+l).indexOf("auto")>-1,c?(n=u.position(),a=n.top,o=n.left):(a=parseFloat(s)||0,o=parseFloat(l)||0),C.isFunction(t)&&(t=t.call(e,i,C.extend({},r))),null!=t.top&&(d.top=t.top-r.top+a),null!=t.left&&(d.left=t.left-r.left+o),"using"in t?t.using.call(e,d):u.css(d)}},C.fn.extend({offset:function(e){if(arguments.length)return void 0===e?this:this.each((function(t){C.offset.setOffset(this,e,t)}));var t,i,n,o,s=this[0];return s?s.getClientRects().length?(n=s.getBoundingClientRect(),n.width||n.height?(o=s.ownerDocument,i=Kt(o),t=o.documentElement,{top:n.top+i.pageYOffset-t.clientTop,left:n.left+i.pageXOffset-t.clientLeft}):n):{top:0,left:0}:void 0},position:function(){if(this[0]){var e,t,i=this[0],n={top:0,left:0};return"fixed"===C.css(i,"position")?t=i.getBoundingClientRect():(e=this.offsetParent(),t=this.offset(),C.nodeName(e[0],"html")||(n=e.offset()),n={top:n.top+C.css(e[0],"borderTopWidth",!0),left:n.left+C.css(e[0],"borderLeftWidth",!0)}),{top:t.top-n.top-C.css(i,"marginTop",!0),left:t.left-n.left-C.css(i,"marginLeft",!0)}}},offsetParent:function(){return this.map((function(){var e=this.offsetParent;while(e&&"static"===C.css(e,"position"))e=e.offsetParent;return e||ye}))}}),C.each({scrollLeft:"pageXOffset",scrollTop:"pageYOffset"},(function(e,t){var i="pageYOffset"===t;C.fn[e]=function(n){return K(this,(function(e,n,o){var s=Kt(e);if(void 0===o)return s?s[t]:e[n];s?s.scrollTo(i?s.pageXOffset:o,i?o:s.pageYOffset):e[n]=o}),e,n,arguments.length)}})),C.each(["top","left"],(function(e,t){C.cssHooks[t]=Be(w.pixelPosition,(function(e,i){if(i)return i=je(e,t),We.test(i)?C(e).position()[t]+"px":i}))})),C.each({Height:"height",Width:"width"},(function(e,t){C.each({padding:"inner"+e,content:t,"":"outer"+e},(function(i,n){C.fn[n]=function(o,s){var a=arguments.length&&(i||"boolean"!==typeof o),r=i||(!0===o||!0===s?"margin":"border");return K(this,(function(t,i,o){var s;return C.isWindow(t)?0===n.indexOf("outer")?t["inner"+e]:t.document.documentElement["client"+e]:9===t.nodeType?(s=t.documentElement,Math.max(t.body["scroll"+e],s["scroll"+e],t.body["offset"+e],s["offset"+e],s["client"+e])):void 0===o?C.css(t,i,r):C.style(t,i,o,r)}),t,a?o:void 0,a)}}))})),C.fn.extend({bind:function(e,t,i){return this.on(e,null,t,i)},unbind:function(e,t){return this.off(e,null,t)},delegate:function(e,t,i,n){return this.on(t,e,i,n)},undelegate:function(e,t,i){return 1===arguments.length?this.off(e,"**"):this.off(t,e||"**",i)}}),C.parseJSON=JSON.parse,n=[],o=function(){return C}.apply(t,n),void 0===o||(e.exports=o);var Ut=i.jQuery,Xt=i.$;return C.noConflict=function(e){return i.$===C&&(i.$=Xt),e&&i.jQuery===C&&(i.jQuery=Ut),C},s||(i.jQuery=i.$=C),C}))},dc82:function(e,t,i){"use strict";i("ad6c")},e01a:function(e,t,i){"use strict";var n=i("23e7"),o=i("83ab"),s=i("da84"),a=i("5135"),r=i("861d"),l=i("9bf2").f,c=i("e893"),h=s.Symbol;if(o&&"function"==typeof h&&(!("description"in h.prototype)||void 0!==h().description)){var u={},d=function(){var e=arguments.length<1||void 0===arguments[0]?void 0:String(arguments[0]),t=this instanceof d?new h(e):void 0===e?h():h(e);return""===e&&(u[t]=!0),t};c(d,h);var p=d.prototype=h.prototype;p.constructor=d;var f=p.toString,g="Symbol(test)"==String(h("test")),m=/^Symbol\((.*)\)[^)]+$/;l(p,"description",{configurable:!0,get:function(){var e=r(this)?this.valueOf():this,t=f.call(e);if(a(u,e))return"";var i=g?t.slice(7,-1):t.replace(m,"$1");return""===i?void 0:i}}),n({global:!0,forced:!0},{Symbol:d})}},e285:function(e,t,i){var n=i("da84"),o=n.isFinite;e.exports=Number.isFinite||function(e){return"number"==typeof e&&o(e)}},e9c4:function(e,t,i){var n=i("23e7"),o=i("d066"),s=i("d039"),a=o("JSON","stringify"),r=/[\uD800-\uDFFF]/g,l=/^[\uD800-\uDBFF]$/,c=/^[\uDC00-\uDFFF]$/,h=function(e,t,i){var n=i.charAt(t-1),o=i.charAt(t+1);return l.test(e)&&!c.test(o)||c.test(e)&&!l.test(n)?"\\u"+e.charCodeAt(0).toString(16):e},u=s((function(){return'"\\udf06\\ud834"'!==a("\udf06\ud834")||'"\\udead"'!==a("\udead")}));a&&n({target:"JSON",stat:!0,forced:u},{stringify:function(e,t,i){var n=a.apply(null,arguments);return"string"==typeof n?n.replace(r,h):n}})},ed3a:function(e,t){(function(e){e.fn.drag=function(t,i,n){var o="string"==typeof t?t:"",s=e.isFunction(t)?t:e.isFunction(i)?i:null;return 0!==o.indexOf("drag")&&(o="drag"+o),n=(t==s?i:n)||{},s?this.on(o,n,s):this.trigger(o)};var t=e.event,i=t.special,n=i.drag={defaults:{which:1,distance:0,not:":input",handle:null,relative:!1,drop:!0,click:!1},datakey:"dragdata",noBubble:!0,add:function(t){var i=e.data(this,n.datakey),o=t.data||{};i.related+=1,e.each(n.defaults,(function(e,t){void 0!==o[e]&&(i[e]=o[e])}))},remove:function(){e.data(this,n.datakey).related-=1},setup:function(){if(!e.data(this,n.datakey)){var i=e.extend({related:0},n.defaults);e.data(this,n.datakey,i),t.add(this,"touchstart mousedown",n.init,i),this.attachEvent&&this.attachEvent("ondragstart",n.dontstart)}},teardown:function(){var i=e.data(this,n.datakey)||{};i.related||(e.removeData(this,n.datakey),t.remove(this,"touchstart mousedown",n.init),n.textselect(!0),this.detachEvent&&this.detachEvent("ondragstart",n.dontstart))},init:function(o){if(!n.touched){var s,a=o.data;if(!(0!=o.which&&a.which>0&&o.which!=a.which)){var r=e(o.target).attr("class")||"";if(!e(o.target).is(a.not)&&r&&-1!==r.toString().indexOf("slick")&&(!a.handle||e(o.target).closest(a.handle,o.currentTarget).length)&&(n.touched="touchstart"==o.type?this:null,a.propagates=1,a.mousedown=this,a.interactions=[n.interaction(this,a)],a.target=o.target,a.pageX=o.pageX,a.pageY=o.pageY,a.dragging=null,s=n.hijack(o,"draginit",a),a.propagates))return s=n.flatten(s),s&&s.length&&(a.interactions=[],e.each(s,(function(){a.interactions.push(n.interaction(this,a))}))),a.propagates=a.interactions.length,!1!==a.drop&&i.drop&&i.drop.handler(o,a),n.textselect(!1),n.touched?t.add(n.touched,"touchmove touchend",n.handler,a):t.add(document,"mousemove mouseup",n.handler,a),!(!n.touched||a.live)&&void 0}}},interaction:function(t,i){var o=t&&t.ownerDocument&&e(t)[i.relative?"position":"offset"]()||{top:0,left:0};return{drag:t,callback:new n.callback,droppable:[],offset:o}},handler:function(o){var s=o.data;switch(o.type){case!s.dragging&&"touchmove":o.preventDefault();case!s.dragging&&"mousemove":if(Math.pow(o.pageX-s.pageX,2)+Math.pow(o.pageY-s.pageY,2)<Math.pow(s.distance,2))break;o.target=s.target,n.hijack(o,"dragstart",s),s.propagates&&(s.dragging=!0);case"touchmove":o.preventDefault();case"mousemove":if(s.dragging){if(n.hijack(o,"drag",s),s.propagates){!1!==s.drop&&i.drop&&i.drop.handler(o,s);break}o.type="mouseup"}case"touchend":case"mouseup":default:n.touched?t.remove(n.touched,"touchmove touchend",n.handler):t.remove(document,"mousemove mouseup",n.handler),s.dragging&&(!1!==s.drop&&i.drop&&i.drop.handler(o,s),n.hijack(o,"dragend",s)),n.textselect(!0),!1===s.click&&s.dragging&&e.data(s.mousedown,"suppress.click",(new Date).getTime()+5),s.dragging=n.touched=!1;break}},hijack:function(i,o,s,a,r){if(s){var l,c,h,u={event:i.originalEvent,type:i.type},d=o.indexOf("drop")?"drag":"drop",p=a||0,f=isNaN(a)?s.interactions.length:a;i.type=o;var g=function(){};i.originalEvent=new jQuery.Event(u.event,{preventDefault:g,stopPropagation:g,stopImmediatePropagation:g}),s.results=[];do{if(c=s.interactions[p]){if("dragend"!==o&&c.cancelled)continue;h=n.properties(i,s,c),c.results=[],e(r||c[d]||s.droppable).each((function(a,r){if(h.target=r,i.isPropagationStopped=function(){return!1},l=r?t.dispatch.call(r,i,h):null,!1===l?("drag"==d&&(c.cancelled=!0,s.propagates-=1),"drop"==o&&(c[d][a]=null)):"dropinit"==o&&c.droppable.push(n.element(l)||r),"dragstart"==o&&(c.proxy=e(n.element(l)||c.drag)[0]),c.results.push(l),delete i.result,"dropinit"!==o)return l})),s.results[p]=n.flatten(c.results),"dropinit"==o&&(c.droppable=n.flatten(c.droppable)),"dragstart"!=o||c.cancelled||h.update()}}while(++p<f);return i.type=u.type,i.originalEvent=u.event,n.flatten(s.results)}},properties:function(e,t,i){var o=i.callback;return o.drag=i.drag,o.proxy=i.proxy||i.drag,o.startX=t.pageX,o.startY=t.pageY,o.deltaX=e.pageX-t.pageX,o.deltaY=e.pageY-t.pageY,o.originalX=i.offset.left,o.originalY=i.offset.top,o.offsetX=o.originalX+o.deltaX,o.offsetY=o.originalY+o.deltaY,o.drop=n.flatten((i.drop||[]).slice()),o.available=n.flatten((i.droppable||[]).slice()),o},element:function(e){if(e&&(e.jquery||1==e.nodeType))return e},flatten:function(t){return e.map(t,(function(t){return t&&t.jquery?e.makeArray(t):t&&t.length?n.flatten(t):t}))},textselect:function(t){e(document)[t?"off":"on"]("selectstart",n.dontstart).css("MozUserSelect",t?"":"none"),document.unselectable=t?"off":"on"},dontstart:function(){return!1},callback:function(){}};n.callback.prototype={update:function(){i.drop&&this.available.length&&e.each(this.available,(function(e){i.drop.locate(this,e)}))}};var o=t.dispatch;t.dispatch=function(t){if(!(e.data(this,"suppress."+t.type)-(new Date).getTime()>0))return o.apply(this,arguments);e.removeData(this,"suppress."+t.type)},i.draginit=i.dragstart=i.dragend=n})(jQuery)},f00c:function(e,t,i){var n=i("23e7"),o=i("e285");n({target:"Number",stat:!0},{isFinite:o})},f06f:function(module,exports){
/*! jQuery UI - v1.9.2 - 2012-11-23
* http://jqueryui.com
* Includes: jquery.ui.core.js, jquery.ui.widget.js, jquery.ui.mouse.js, jquery.ui.draggable.js, jquery.ui.droppable.js, jquery.ui.resizable.js, jquery.ui.selectable.js, jquery.ui.sortable.js, jquery.ui.effect.js, jquery.ui.accordion.js, jquery.ui.autocomplete.js, jquery.ui.button.js, jquery.ui.datepicker.js, jquery.ui.dialog.js, jquery.ui.effect-blind.js, jquery.ui.effect-bounce.js, jquery.ui.effect-clip.js, jquery.ui.effect-drop.js, jquery.ui.effect-explode.js, jquery.ui.effect-fade.js, jquery.ui.effect-fold.js, jquery.ui.effect-highlight.js, jquery.ui.effect-pulsate.js, jquery.ui.effect-scale.js, jquery.ui.effect-shake.js, jquery.ui.effect-slide.js, jquery.ui.effect-transfer.js, jquery.ui.menu.js, jquery.ui.position.js, jquery.ui.progressbar.js, jquery.ui.slider.js, jquery.ui.spinner.js, jquery.ui.tabs.js, jquery.ui.tooltip.js
* Copyright 2012 jQuery Foundation and other contributors; Licensed MIT */
(function(e,t){var i=0,n=/^ui-id-\d+$/;function o(t,i){var n,o,a,r=t.nodeName.toLowerCase();return"area"===r?(n=t.parentNode,o=n.name,!(!t.href||!o||"map"!==n.nodeName.toLowerCase())&&(a=e("img[usemap=#"+o+"]")[0],!!a&&s(a))):(/input|select|textarea|button|object/.test(r)?!t.disabled:"a"===r&&t.href||i)&&s(t)}function s(t){return e.expr.filters.visible(t)&&!e(t).parents().andSelf().filter((function(){return"hidden"===e.css(this,"visibility")})).length}e.ui=e.ui||{},e.ui.version||(e.extend(e.ui,{version:"1.9.2",keyCode:{BACKSPACE:8,COMMA:188,DELETE:46,DOWN:40,END:35,ENTER:13,ESCAPE:27,HOME:36,LEFT:37,NUMPAD_ADD:107,NUMPAD_DECIMAL:110,NUMPAD_DIVIDE:111,NUMPAD_ENTER:108,NUMPAD_MULTIPLY:106,NUMPAD_SUBTRACT:109,PAGE_DOWN:34,PAGE_UP:33,PERIOD:190,RIGHT:39,SPACE:32,TAB:9,UP:38}}),e.fn.extend({_focus:e.fn.focus,focus:function(t,i){return"number"===typeof t?this.each((function(){var n=this;setTimeout((function(){e(n).focus(),i&&i.call(n)}),t)})):this._focus.apply(this,arguments)},scrollParent:function(){var t;return t=e.ui.ie&&/(static|relative)/.test(this.css("position"))||/absolute/.test(this.css("position"))?this.parents().filter((function(){return/(relative|absolute|fixed)/.test(e.css(this,"position"))&&/(auto|scroll)/.test(e.css(this,"overflow")+e.css(this,"overflow-y")+e.css(this,"overflow-x"))})).eq(0):this.parents().filter((function(){return/(auto|scroll)/.test(e.css(this,"overflow")+e.css(this,"overflow-y")+e.css(this,"overflow-x"))})).eq(0),/fixed/.test(this.css("position"))||!t.length?e(document):t},zIndex:function(i){if(i!==t)return this.css("zIndex",i);if(this.length){var n,o,s=e(this[0]);while(s.length&&s[0]!==document){if(n=s.css("position"),("absolute"===n||"relative"===n||"fixed"===n)&&(o=parseInt(s.css("zIndex"),10),!isNaN(o)&&0!==o))return o;s=s.parent()}}return 0},uniqueId:function(){return this.each((function(){this.id||(this.id="ui-id-"+ ++i)}))},removeUniqueId:function(){return this.each((function(){n.test(this.id)&&e(this).removeAttr("id")}))}}),e.extend(e.expr[":"],{data:e.expr.createPseudo?e.expr.createPseudo((function(t){return function(i){return!!e.data(i,t)}})):function(t,i,n){return!!e.data(t,n[3])},focusable:function(t){return o(t,!isNaN(e.attr(t,"tabindex")))},tabbable:function(t){var i=e.attr(t,"tabindex"),n=isNaN(i);return(n||i>=0)&&o(t,!n)}}),e((function(){var t=document.body,i=t.appendChild(i=document.createElement("div"));i.offsetHeight,e.extend(i.style,{minHeight:"100px",height:"auto",padding:0,borderWidth:0}),e.support.minHeight=100===i.offsetHeight,e.support.selectstart="onselectstart"in i,t.removeChild(i).style.display="none"})),e("<a>").outerWidth(1).jquery||e.each(["Width","Height"],(function(i,n){var o="Width"===n?["Left","Right"]:["Top","Bottom"],s=n.toLowerCase(),a={innerWidth:e.fn.innerWidth,innerHeight:e.fn.innerHeight,outerWidth:e.fn.outerWidth,outerHeight:e.fn.outerHeight};function r(t,i,n,s){return e.each(o,(function(){i-=parseFloat(e.css(t,"padding"+this))||0,n&&(i-=parseFloat(e.css(t,"border"+this+"Width"))||0),s&&(i-=parseFloat(e.css(t,"margin"+this))||0)})),i}e.fn["inner"+n]=function(i){return i===t?a["inner"+n].call(this):this.each((function(){e(this).css(s,r(this,i)+"px")}))},e.fn["outer"+n]=function(t,i){return"number"!==typeof t?a["outer"+n].call(this,t):this.each((function(){e(this).css(s,r(this,t,!0,i)+"px")}))}})),e("<a>").data("a-b","a").removeData("a-b").data("a-b")&&(e.fn.removeData=function(t){return function(i){return arguments.length?t.call(this,e.camelCase(i)):t.call(this)}}(e.fn.removeData)),function(){var t=/msie ([\w.]+)/.exec(navigator.userAgent.toLowerCase())||[];e.ui.ie=!!t.length,e.ui.ie6=6===parseFloat(t[1],10)}(),e.fn.extend({disableSelection:function(){return this.bind((e.support.selectstart?"selectstart":"mousedown")+".ui-disableSelection",(function(e){e.preventDefault()}))},enableSelection:function(){return this.unbind(".ui-disableSelection")}}),e.extend(e.ui,{plugin:{add:function(t,i,n){var o,s=e.ui[t].prototype;for(o in n)s.plugins[o]=s.plugins[o]||[],s.plugins[o].push([i,n[o]])},call:function(e,t,i){var n,o=e.plugins[t];if(o&&e.element[0].parentNode&&11!==e.element[0].parentNode.nodeType)for(n=0;n<o.length;n++)e.options[o[n][0]]&&o[n][1].apply(e.element,i)}},contains:e.contains,hasScroll:function(t,i){if("hidden"===e(t).css("overflow"))return!1;var n=i&&"left"===i?"scrollLeft":"scrollTop",o=!1;return t[n]>0||(t[n]=1,o=t[n]>0,t[n]=0,o)},isOverAxis:function(e,t,i){return e>t&&e<t+i},isOver:function(t,i,n,o,s,a){return e.ui.isOverAxis(t,n,s)&&e.ui.isOverAxis(i,o,a)}}))})(jQuery),function(e,t){var i=0,n=Array.prototype.slice,o=e.cleanData;e.cleanData=function(t){for(var i,n=0;null!=(i=t[n]);n++)try{e(i).triggerHandler("remove")}catch(s){}o(t)},e.widget=function(t,i,n){var o,s,a,r,l=t.split(".")[0];t=t.split(".")[1],o=l+"-"+t,n||(n=i,i=e.Widget),e.expr[":"][o.toLowerCase()]=function(t){return!!e.data(t,o)},e[l]=e[l]||{},s=e[l][t],a=e[l][t]=function(e,t){if(!this._createWidget)return new a(e,t);arguments.length&&this._createWidget(e,t)},e.extend(a,s,{version:n.version,_proto:e.extend({},n),_childConstructors:[]}),r=new i,r.options=e.widget.extend({},r.options),e.each(n,(function(t,o){e.isFunction(o)&&(n[t]=function(){var e=function(){return i.prototype[t].apply(this,arguments)},n=function(e){return i.prototype[t].apply(this,e)};return function(){var t,i=this._super,s=this._superApply;return this._super=e,this._superApply=n,t=o.apply(this,arguments),this._super=i,this._superApply=s,t}}())})),a.prototype=e.widget.extend(r,{widgetEventPrefix:s?r.widgetEventPrefix:t},n,{constructor:a,namespace:l,widgetName:t,widgetBaseClass:o,widgetFullName:o}),s?(e.each(s._childConstructors,(function(t,i){var n=i.prototype;e.widget(n.namespace+"."+n.widgetName,a,i._proto)})),delete s._childConstructors):i._childConstructors.push(a),e.widget.bridge(t,a)},e.widget.extend=function(i){for(var o,s,a=n.call(arguments,1),r=0,l=a.length;r<l;r++)for(o in a[r])s=a[r][o],a[r].hasOwnProperty(o)&&s!==t&&(e.isPlainObject(s)?i[o]=e.isPlainObject(i[o])?e.widget.extend({},i[o],s):e.widget.extend({},s):i[o]=s);return i},e.widget.bridge=function(i,o){var s=o.prototype.widgetFullName||i;e.fn[i]=function(a){var r="string"===typeof a,l=n.call(arguments,1),c=this;return a=!r&&l.length?e.widget.extend.apply(null,[a].concat(l)):a,r?this.each((function(){var n,o=e.data(this,s);return o?e.isFunction(o[a])&&"_"!==a.charAt(0)?(n=o[a].apply(o,l),n!==o&&n!==t?(c=n&&n.jquery?c.pushStack(n.get()):n,!1):void 0):e.error("no such method '"+a+"' for "+i+" widget instance"):e.error("cannot call methods on "+i+" prior to initialization; attempted to call method '"+a+"'")})):this.each((function(){var t=e.data(this,s);t?t.option(a||{})._init():e.data(this,s,new o(a,this))})),c}},e.Widget=function(){},e.Widget._childConstructors=[],e.Widget.prototype={widgetName:"widget",widgetEventPrefix:"",defaultElement:"<div>",options:{disabled:!1,create:null},_createWidget:function(t,n){n=e(n||this.defaultElement||this)[0],this.element=e(n),this.uuid=i++,this.eventNamespace="."+this.widgetName+this.uuid,this.options=e.widget.extend({},this.options,this._getCreateOptions(),t),this.bindings=e(),this.hoverable=e(),this.focusable=e(),n!==this&&(e.data(n,this.widgetName,this),e.data(n,this.widgetFullName,this),this._on(!0,this.element,{remove:function(e){e.target===n&&this.destroy()}}),this.document=e(n.style?n.ownerDocument:n.document||n),this.window=e(this.document[0].defaultView||this.document[0].parentWindow)),this._create(),this._trigger("create",null,this._getCreateEventData()),this._init()},_getCreateOptions:e.noop,_getCreateEventData:e.noop,_create:e.noop,_init:e.noop,destroy:function(){this._destroy(),this.element.unbind(this.eventNamespace).removeData(this.widgetName).removeData(this.widgetFullName).removeData(e.camelCase(this.widgetFullName)),this.widget().unbind(this.eventNamespace).removeAttr("aria-disabled").removeClass(this.widgetFullName+"-disabled ui-state-disabled"),this.bindings.unbind(this.eventNamespace),this.hoverable.removeClass("ui-state-hover"),this.focusable.removeClass("ui-state-focus")},_destroy:e.noop,widget:function(){return this.element},option:function(i,n){var o,s,a,r=i;if(0===arguments.length)return e.widget.extend({},this.options);if("string"===typeof i)if(r={},o=i.split("."),i=o.shift(),o.length){for(s=r[i]=e.widget.extend({},this.options[i]),a=0;a<o.length-1;a++)s[o[a]]=s[o[a]]||{},s=s[o[a]];if(i=o.pop(),n===t)return s[i]===t?null:s[i];s[i]=n}else{if(n===t)return this.options[i]===t?null:this.options[i];r[i]=n}return this._setOptions(r),this},_setOptions:function(e){var t;for(t in e)this._setOption(t,e[t]);return this},_setOption:function(e,t){return this.options[e]=t,"disabled"===e&&(this.widget().toggleClass(this.widgetFullName+"-disabled ui-state-disabled",!!t).attr("aria-disabled",t),this.hoverable.removeClass("ui-state-hover"),this.focusable.removeClass("ui-state-focus")),this},enable:function(){return this._setOption("disabled",!1)},disable:function(){return this._setOption("disabled",!0)},_on:function(t,i,n){var o,s=this;"boolean"!==typeof t&&(n=i,i=t,t=!1),n?(i=o=e(i),this.bindings=this.bindings.add(i)):(n=i,i=this.element,o=this.widget()),e.each(n,(function(n,a){function r(){if(t||!0!==s.options.disabled&&!e(this).hasClass("ui-state-disabled"))return("string"===typeof a?s[a]:a).apply(s,arguments)}"string"!==typeof a&&(r.guid=a.guid=a.guid||r.guid||e.guid++);var l=n.match(/^(\w+)\s*(.*)$/),c=l[1]+s.eventNamespace,h=l[2];h?o.delegate(h,c,r):i.bind(c,r)}))},_off:function(e,t){t=(t||"").split(" ").join(this.eventNamespace+" ")+this.eventNamespace,e.unbind(t).undelegate(t)},_delay:function(e,t){function i(){return("string"===typeof e?n[e]:e).apply(n,arguments)}var n=this;return setTimeout(i,t||0)},_hoverable:function(t){this.hoverable=this.hoverable.add(t),this._on(t,{mouseenter:function(t){e(t.currentTarget).addClass("ui-state-hover")},mouseleave:function(t){e(t.currentTarget).removeClass("ui-state-hover")}})},_focusable:function(t){this.focusable=this.focusable.add(t),this._on(t,{focusin:function(t){e(t.currentTarget).addClass("ui-state-focus")},focusout:function(t){e(t.currentTarget).removeClass("ui-state-focus")}})},_trigger:function(t,i,n){var o,s,a=this.options[t];if(n=n||{},i=e.Event(i),i.type=(t===this.widgetEventPrefix?t:this.widgetEventPrefix+t).toLowerCase(),i.target=this.element[0],s=i.originalEvent,s)for(o in s)o in i||(i[o]=s[o]);return this.element.trigger(i,n),!(e.isFunction(a)&&!1===a.apply(this.element[0],[i].concat(n))||i.isDefaultPrevented())}},e.each({show:"fadeIn",hide:"fadeOut"},(function(t,i){e.Widget.prototype["_"+t]=function(n,o,s){"string"===typeof o&&(o={effect:o});var a,r=o?!0===o||"number"===typeof o?i:o.effect||i:t;o=o||{},"number"===typeof o&&(o={duration:o}),a=!e.isEmptyObject(o),o.complete=s,o.delay&&n.delay(o.delay),a&&e.effects&&(e.effects.effect[r]||!1!==e.uiBackCompat&&e.effects[r])?n[t](o):r!==t&&n[r]?n[r](o.duration,o.easing,s):n.queue((function(i){e(this)[t](),s&&s.call(n[0]),i()}))}})),!1!==e.uiBackCompat&&(e.Widget.prototype._getCreateOptions=function(){return e.metadata&&e.metadata.get(this.element[0])[this.widgetName]})}(jQuery),function(e,t){var i=!1;e(document).mouseup((function(e){i=!1})),e.widget("ui.mouse",{version:"1.9.2",options:{cancel:"input,textarea,button,select,option",distance:1,delay:0},_mouseInit:function(){var t=this;this.element.bind("mousedown."+this.widgetName,(function(e){return t._mouseDown(e)})).bind("click."+this.widgetName,(function(i){if(!0===e.data(i.target,t.widgetName+".preventClickEvent"))return e.removeData(i.target,t.widgetName+".preventClickEvent"),i.stopImmediatePropagation(),!1})),this.started=!1},_mouseDestroy:function(){this.element.unbind("."+this.widgetName),this._mouseMoveDelegate&&e(document).unbind("mousemove."+this.widgetName,this._mouseMoveDelegate).unbind("mouseup."+this.widgetName,this._mouseUpDelegate)},_mouseDown:function(t){if(!i){this._mouseStarted&&this._mouseUp(t),this._mouseDownEvent=t;var n=this,o=1===t.which,s=!("string"!==typeof this.options.cancel||!t.target.nodeName)&&e(t.target).closest(this.options.cancel).length;return!(o&&!s&&this._mouseCapture(t))||(this.mouseDelayMet=!this.options.delay,this.mouseDelayMet||(this._mouseDelayTimer=setTimeout((function(){n.mouseDelayMet=!0}),this.options.delay)),this._mouseDistanceMet(t)&&this._mouseDelayMet(t)&&(this._mouseStarted=!1!==this._mouseStart(t),!this._mouseStarted)?(t.preventDefault(),!0):(!0===e.data(t.target,this.widgetName+".preventClickEvent")&&e.removeData(t.target,this.widgetName+".preventClickEvent"),this._mouseMoveDelegate=function(e){return n._mouseMove(e)},this._mouseUpDelegate=function(e){return n._mouseUp(e)},e(document).bind("mousemove."+this.widgetName,this._mouseMoveDelegate).bind("mouseup."+this.widgetName,this._mouseUpDelegate),t.preventDefault(),i=!0,!0))}},_mouseMove:function(t){return!e.ui.ie||document.documentMode>=9||t.button?this._mouseStarted?(this._mouseDrag(t),t.preventDefault()):(this._mouseDistanceMet(t)&&this._mouseDelayMet(t)&&(this._mouseStarted=!1!==this._mouseStart(this._mouseDownEvent,t),this._mouseStarted?this._mouseDrag(t):this._mouseUp(t)),!this._mouseStarted):this._mouseUp(t)},_mouseUp:function(t){return e(document).unbind("mousemove."+this.widgetName,this._mouseMoveDelegate).unbind("mouseup."+this.widgetName,this._mouseUpDelegate),this._mouseStarted&&(this._mouseStarted=!1,t.target===this._mouseDownEvent.target&&e.data(t.target,this.widgetName+".preventClickEvent",!0),this._mouseStop(t)),!1},_mouseDistanceMet:function(e){return Math.max(Math.abs(this._mouseDownEvent.pageX-e.pageX),Math.abs(this._mouseDownEvent.pageY-e.pageY))>=this.options.distance},_mouseDelayMet:function(e){return this.mouseDelayMet},_mouseStart:function(e){},_mouseDrag:function(e){},_mouseStop:function(e){},_mouseCapture:function(e){return!0}})}(jQuery),function(e,t){e.widget("ui.draggable",e.ui.mouse,{version:"1.9.2",widgetEventPrefix:"drag",options:{addClasses:!0,appendTo:"parent",axis:!1,connectToSortable:!1,containment:!1,cursor:"auto",cursorAt:!1,grid:!1,handle:!1,helper:"original",iframeFix:!1,opacity:!1,refreshPositions:!1,revert:!1,revertDuration:500,scope:"default",scroll:!0,scrollSensitivity:20,scrollSpeed:20,snap:!1,snapMode:"both",snapTolerance:20,stack:!1,zIndex:!1},_create:function(){"original"!=this.options.helper||/^(?:r|a|f)/.test(this.element.css("position"))||(this.element[0].style.position="relative"),this.options.addClasses&&this.element.addClass("ui-draggable"),this.options.disabled&&this.element.addClass("ui-draggable-disabled"),this._mouseInit()},_destroy:function(){this.element.removeClass("ui-draggable ui-draggable-dragging ui-draggable-disabled"),this._mouseDestroy()},_mouseCapture:function(t){var i=this.options;return!(this.helper||i.disabled||e(t.target).is(".ui-resizable-handle"))&&(this.handle=this._getHandle(t),!!this.handle&&(e(!0===i.iframeFix?"iframe":i.iframeFix).each((function(){e('<div class="ui-draggable-iframeFix" style="background: #fff;"></div>').css({width:this.offsetWidth+"px",height:this.offsetHeight+"px",position:"absolute",opacity:"0.001",zIndex:1e3}).css(e(this).offset()).appendTo("body")})),!0))},_mouseStart:function(t){var i=this.options;return this.helper=this._createHelper(t),this.helper.addClass("ui-draggable-dragging"),this._cacheHelperProportions(),e.ui.ddmanager&&(e.ui.ddmanager.current=this),this._cacheMargins(),this.cssPosition=this.helper.css("position"),this.scrollParent=this.helper.scrollParent(),this.offset=this.positionAbs=this.element.offset(),this.offset={top:this.offset.top-this.margins.top,left:this.offset.left-this.margins.left},e.extend(this.offset,{click:{left:t.pageX-this.offset.left,top:t.pageY-this.offset.top},parent:this._getParentOffset(),relative:this._getRelativeOffset()}),this.originalPosition=this.position=this._generatePosition(t),this.originalPageX=t.pageX,this.originalPageY=t.pageY,i.cursorAt&&this._adjustOffsetFromHelper(i.cursorAt),i.containment&&this._setContainment(),!1===this._trigger("start",t)?(this._clear(),!1):(this._cacheHelperProportions(),e.ui.ddmanager&&!i.dropBehaviour&&e.ui.ddmanager.prepareOffsets(this,t),this._mouseDrag(t,!0),e.ui.ddmanager&&e.ui.ddmanager.dragStart(this,t),!0)},_mouseDrag:function(t,i){if(this.position=this._generatePosition(t),this.positionAbs=this._convertPositionTo("absolute"),!i){var n=this._uiHash();if(!1===this._trigger("drag",t,n))return this._mouseUp({}),!1;this.position=n.position}return this.options.axis&&"y"==this.options.axis||(this.helper[0].style.left=this.position.left+"px"),this.options.axis&&"x"==this.options.axis||(this.helper[0].style.top=this.position.top+"px"),e.ui.ddmanager&&e.ui.ddmanager.drag(this,t),!1},_mouseStop:function(t){var i=!1;e.ui.ddmanager&&!this.options.dropBehaviour&&(i=e.ui.ddmanager.drop(this,t)),this.dropped&&(i=this.dropped,this.dropped=!1);var n=this.element[0],o=!1;while(n&&(n=n.parentNode))n==document&&(o=!0);if(!o&&"original"===this.options.helper)return!1;if("invalid"==this.options.revert&&!i||"valid"==this.options.revert&&i||!0===this.options.revert||e.isFunction(this.options.revert)&&this.options.revert.call(this.element,i)){var s=this;e(this.helper).animate(this.originalPosition,parseInt(this.options.revertDuration,10),(function(){!1!==s._trigger("stop",t)&&s._clear()}))}else!1!==this._trigger("stop",t)&&this._clear();return!1},_mouseUp:function(t){return e("div.ui-draggable-iframeFix").each((function(){this.parentNode.removeChild(this)})),e.ui.ddmanager&&e.ui.ddmanager.dragStop(this,t),e.ui.mouse.prototype._mouseUp.call(this,t)},cancel:function(){return this.helper.is(".ui-draggable-dragging")?this._mouseUp({}):this._clear(),this},_getHandle:function(t){var i=!this.options.handle||!e(this.options.handle,this.element).length;return e(this.options.handle,this.element).find("*").andSelf().each((function(){this==t.target&&(i=!0)})),i},_createHelper:function(t){var i=this.options,n=e.isFunction(i.helper)?e(i.helper.apply(this.element[0],[t])):"clone"==i.helper?this.element.clone().removeAttr("id"):this.element;return n.parents("body").length||n.appendTo("parent"==i.appendTo?this.element[0].parentNode:i.appendTo),n[0]==this.element[0]||/(fixed|absolute)/.test(n.css("position"))||n.css("position","absolute"),n},_adjustOffsetFromHelper:function(t){"string"==typeof t&&(t=t.split(" ")),e.isArray(t)&&(t={left:+t[0],top:+t[1]||0}),"left"in t&&(this.offset.click.left=t.left+this.margins.left),"right"in t&&(this.offset.click.left=this.helperProportions.width-t.right+this.margins.left),"top"in t&&(this.offset.click.top=t.top+this.margins.top),"bottom"in t&&(this.offset.click.top=this.helperProportions.height-t.bottom+this.margins.top)},_getParentOffset:function(){this.offsetParent=this.helper.offsetParent();var t=this.offsetParent.offset();return"absolute"==this.cssPosition&&this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])&&(t.left+=this.scrollParent.scrollLeft(),t.top+=this.scrollParent.scrollTop()),(this.offsetParent[0]==document.body||this.offsetParent[0].tagName&&"html"==this.offsetParent[0].tagName.toLowerCase()&&e.ui.ie)&&(t={top:0,left:0}),{top:t.top+(parseInt(this.offsetParent.css("borderTopWidth"),10)||0),left:t.left+(parseInt(this.offsetParent.css("borderLeftWidth"),10)||0)}},_getRelativeOffset:function(){if("relative"==this.cssPosition){var e=this.element.position();return{top:e.top-(parseInt(this.helper.css("top"),10)||0)+this.scrollParent.scrollTop(),left:e.left-(parseInt(this.helper.css("left"),10)||0)+this.scrollParent.scrollLeft()}}return{top:0,left:0}},_cacheMargins:function(){this.margins={left:parseInt(this.element.css("marginLeft"),10)||0,top:parseInt(this.element.css("marginTop"),10)||0,right:parseInt(this.element.css("marginRight"),10)||0,bottom:parseInt(this.element.css("marginBottom"),10)||0}},_cacheHelperProportions:function(){this.helperProportions={width:this.helper.outerWidth(),height:this.helper.outerHeight()}},_setContainment:function(){var t=this.options;if("parent"==t.containment&&(t.containment=this.helper[0].parentNode),"document"!=t.containment&&"window"!=t.containment||(this.containment=["document"==t.containment?0:e(window).scrollLeft()-this.offset.relative.left-this.offset.parent.left,"document"==t.containment?0:e(window).scrollTop()-this.offset.relative.top-this.offset.parent.top,("document"==t.containment?0:e(window).scrollLeft())+e("document"==t.containment?document:window).width()-this.helperProportions.width-this.margins.left,("document"==t.containment?0:e(window).scrollTop())+(e("document"==t.containment?document:window).height()||document.body.parentNode.scrollHeight)-this.helperProportions.height-this.margins.top]),/^(document|window|parent)$/.test(t.containment)||t.containment.constructor==Array)t.containment.constructor==Array&&(this.containment=t.containment);else{var i=e(t.containment),n=i[0];if(!n)return;i.offset();var o="hidden"!=e(n).css("overflow");this.containment=[(parseInt(e(n).css("borderLeftWidth"),10)||0)+(parseInt(e(n).css("paddingLeft"),10)||0),(parseInt(e(n).css("borderTopWidth"),10)||0)+(parseInt(e(n).css("paddingTop"),10)||0),(o?Math.max(n.scrollWidth,n.offsetWidth):n.offsetWidth)-(parseInt(e(n).css("borderLeftWidth"),10)||0)-(parseInt(e(n).css("paddingRight"),10)||0)-this.helperProportions.width-this.margins.left-this.margins.right,(o?Math.max(n.scrollHeight,n.offsetHeight):n.offsetHeight)-(parseInt(e(n).css("borderTopWidth"),10)||0)-(parseInt(e(n).css("paddingBottom"),10)||0)-this.helperProportions.height-this.margins.top-this.margins.bottom],this.relative_container=i}},_convertPositionTo:function(t,i){i||(i=this.position);var n="absolute"==t?1:-1,o=(this.options,"absolute"!=this.cssPosition||this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent),s=/(html|body)/i.test(o[0].tagName);return{top:i.top+this.offset.relative.top*n+this.offset.parent.top*n-("fixed"==this.cssPosition?-this.scrollParent.scrollTop():s?0:o.scrollTop())*n,left:i.left+this.offset.relative.left*n+this.offset.parent.left*n-("fixed"==this.cssPosition?-this.scrollParent.scrollLeft():s?0:o.scrollLeft())*n}},_generatePosition:function(t){var i=this.options,n="absolute"!=this.cssPosition||this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent,o=/(html|body)/i.test(n[0].tagName),s=t.pageX,a=t.pageY;if(this.originalPosition){var r;if(this.containment){if(this.relative_container){var l=this.relative_container.offset();r=[this.containment[0]+l.left,this.containment[1]+l.top,this.containment[2]+l.left,this.containment[3]+l.top]}else r=this.containment;t.pageX-this.offset.click.left<r[0]&&(s=r[0]+this.offset.click.left),t.pageY-this.offset.click.top<r[1]&&(a=r[1]+this.offset.click.top),t.pageX-this.offset.click.left>r[2]&&(s=r[2]+this.offset.click.left),t.pageY-this.offset.click.top>r[3]&&(a=r[3]+this.offset.click.top)}if(i.grid){var c=i.grid[1]?this.originalPageY+Math.round((a-this.originalPageY)/i.grid[1])*i.grid[1]:this.originalPageY;a=r&&(c-this.offset.click.top<r[1]||c-this.offset.click.top>r[3])?c-this.offset.click.top<r[1]?c+i.grid[1]:c-i.grid[1]:c;var h=i.grid[0]?this.originalPageX+Math.round((s-this.originalPageX)/i.grid[0])*i.grid[0]:this.originalPageX;s=r&&(h-this.offset.click.left<r[0]||h-this.offset.click.left>r[2])?h-this.offset.click.left<r[0]?h+i.grid[0]:h-i.grid[0]:h}}return{top:a-this.offset.click.top-this.offset.relative.top-this.offset.parent.top+("fixed"==this.cssPosition?-this.scrollParent.scrollTop():o?0:n.scrollTop()),left:s-this.offset.click.left-this.offset.relative.left-this.offset.parent.left+("fixed"==this.cssPosition?-this.scrollParent.scrollLeft():o?0:n.scrollLeft())}},_clear:function(){this.helper.removeClass("ui-draggable-dragging"),this.helper[0]==this.element[0]||this.cancelHelperRemoval||this.helper.remove(),this.helper=null,this.cancelHelperRemoval=!1},_trigger:function(t,i,n){return n=n||this._uiHash(),e.ui.plugin.call(this,t,[i,n]),"drag"==t&&(this.positionAbs=this._convertPositionTo("absolute")),e.Widget.prototype._trigger.call(this,t,i,n)},plugins:{},_uiHash:function(e){return{helper:this.helper,position:this.position,originalPosition:this.originalPosition,offset:this.positionAbs}}}),e.ui.plugin.add("draggable","connectToSortable",{start:function(t,i){var n=e(this).data("draggable"),o=n.options,s=e.extend({},i,{item:n.element});n.sortables=[],e(o.connectToSortable).each((function(){var i=e.data(this,"sortable");i&&!i.options.disabled&&(n.sortables.push({instance:i,shouldRevert:i.options.revert}),i.refreshPositions(),i._trigger("activate",t,s))}))},stop:function(t,i){var n=e(this).data("draggable"),o=e.extend({},i,{item:n.element});e.each(n.sortables,(function(){this.instance.isOver?(this.instance.isOver=0,n.cancelHelperRemoval=!0,this.instance.cancelHelperRemoval=!1,this.shouldRevert&&(this.instance.options.revert=!0),this.instance._mouseStop(t),this.instance.options.helper=this.instance.options._helper,"original"==n.options.helper&&this.instance.currentItem.css({top:"auto",left:"auto"})):(this.instance.cancelHelperRemoval=!1,this.instance._trigger("deactivate",t,o))}))},drag:function(t,i){var n=e(this).data("draggable"),o=this;e.each(n.sortables,(function(s){var a=!1,r=this;this.instance.positionAbs=n.positionAbs,this.instance.helperProportions=n.helperProportions,this.instance.offset.click=n.offset.click,this.instance._intersectsWith(this.instance.containerCache)&&(a=!0,e.each(n.sortables,(function(){return this.instance.positionAbs=n.positionAbs,this.instance.helperProportions=n.helperProportions,this.instance.offset.click=n.offset.click,this!=r&&this.instance._intersectsWith(this.instance.containerCache)&&e.ui.contains(r.instance.element[0],this.instance.element[0])&&(a=!1),a}))),a?(this.instance.isOver||(this.instance.isOver=1,this.instance.currentItem=e(o).clone().removeAttr("id").appendTo(this.instance.element).data("sortable-item",!0),this.instance.options._helper=this.instance.options.helper,this.instance.options.helper=function(){return i.helper[0]},t.target=this.instance.currentItem[0],this.instance._mouseCapture(t,!0),this.instance._mouseStart(t,!0,!0),this.instance.offset.click.top=n.offset.click.top,this.instance.offset.click.left=n.offset.click.left,this.instance.offset.parent.left-=n.offset.parent.left-this.instance.offset.parent.left,this.instance.offset.parent.top-=n.offset.parent.top-this.instance.offset.parent.top,n._trigger("toSortable",t),n.dropped=this.instance.element,n.currentItem=n.element,this.instance.fromOutside=n),this.instance.currentItem&&this.instance._mouseDrag(t)):this.instance.isOver&&(this.instance.isOver=0,this.instance.cancelHelperRemoval=!0,this.instance.options.revert=!1,this.instance._trigger("out",t,this.instance._uiHash(this.instance)),this.instance._mouseStop(t,!0),this.instance.options.helper=this.instance.options._helper,this.instance.currentItem.remove(),this.instance.placeholder&&this.instance.placeholder.remove(),n._trigger("fromSortable",t),n.dropped=!1)}))}}),e.ui.plugin.add("draggable","cursor",{start:function(t,i){var n=e("body"),o=e(this).data("draggable").options;n.css("cursor")&&(o._cursor=n.css("cursor")),n.css("cursor",o.cursor)},stop:function(t,i){var n=e(this).data("draggable").options;n._cursor&&e("body").css("cursor",n._cursor)}}),e.ui.plugin.add("draggable","opacity",{start:function(t,i){var n=e(i.helper),o=e(this).data("draggable").options;n.css("opacity")&&(o._opacity=n.css("opacity")),n.css("opacity",o.opacity)},stop:function(t,i){var n=e(this).data("draggable").options;n._opacity&&e(i.helper).css("opacity",n._opacity)}}),e.ui.plugin.add("draggable","scroll",{start:function(t,i){var n=e(this).data("draggable");n.scrollParent[0]!=document&&"HTML"!=n.scrollParent[0].tagName&&(n.overflowOffset=n.scrollParent.offset())},drag:function(t,i){var n=e(this).data("draggable"),o=n.options,s=!1;n.scrollParent[0]!=document&&"HTML"!=n.scrollParent[0].tagName?(o.axis&&"x"==o.axis||(n.overflowOffset.top+n.scrollParent[0].offsetHeight-t.pageY<o.scrollSensitivity?n.scrollParent[0].scrollTop=s=n.scrollParent[0].scrollTop+o.scrollSpeed:t.pageY-n.overflowOffset.top<o.scrollSensitivity&&(n.scrollParent[0].scrollTop=s=n.scrollParent[0].scrollTop-o.scrollSpeed)),o.axis&&"y"==o.axis||(n.overflowOffset.left+n.scrollParent[0].offsetWidth-t.pageX<o.scrollSensitivity?n.scrollParent[0].scrollLeft=s=n.scrollParent[0].scrollLeft+o.scrollSpeed:t.pageX-n.overflowOffset.left<o.scrollSensitivity&&(n.scrollParent[0].scrollLeft=s=n.scrollParent[0].scrollLeft-o.scrollSpeed))):(o.axis&&"x"==o.axis||(t.pageY-e(document).scrollTop()<o.scrollSensitivity?s=e(document).scrollTop(e(document).scrollTop()-o.scrollSpeed):e(window).height()-(t.pageY-e(document).scrollTop())<o.scrollSensitivity&&(s=e(document).scrollTop(e(document).scrollTop()+o.scrollSpeed))),o.axis&&"y"==o.axis||(t.pageX-e(document).scrollLeft()<o.scrollSensitivity?s=e(document).scrollLeft(e(document).scrollLeft()-o.scrollSpeed):e(window).width()-(t.pageX-e(document).scrollLeft())<o.scrollSensitivity&&(s=e(document).scrollLeft(e(document).scrollLeft()+o.scrollSpeed)))),!1!==s&&e.ui.ddmanager&&!o.dropBehaviour&&e.ui.ddmanager.prepareOffsets(n,t)}}),e.ui.plugin.add("draggable","snap",{start:function(t,i){var n=e(this).data("draggable"),o=n.options;n.snapElements=[],e(o.snap.constructor!=String?o.snap.items||":data(draggable)":o.snap).each((function(){var t=e(this),i=t.offset();this!=n.element[0]&&n.snapElements.push({item:this,width:t.outerWidth(),height:t.outerHeight(),top:i.top,left:i.left})}))},drag:function(t,i){for(var n=e(this).data("draggable"),o=n.options,s=o.snapTolerance,a=i.offset.left,r=a+n.helperProportions.width,l=i.offset.top,c=l+n.helperProportions.height,h=n.snapElements.length-1;h>=0;h--){var u=n.snapElements[h].left,d=u+n.snapElements[h].width,p=n.snapElements[h].top,f=p+n.snapElements[h].height;if(u-s<a&&a<d+s&&p-s<l&&l<f+s||u-s<a&&a<d+s&&p-s<c&&c<f+s||u-s<r&&r<d+s&&p-s<l&&l<f+s||u-s<r&&r<d+s&&p-s<c&&c<f+s){if("inner"!=o.snapMode){var g=Math.abs(p-c)<=s,m=Math.abs(f-l)<=s,v=Math.abs(u-r)<=s,w=Math.abs(d-a)<=s;g&&(i.position.top=n._convertPositionTo("relative",{top:p-n.helperProportions.height,left:0}).top-n.margins.top),m&&(i.position.top=n._convertPositionTo("relative",{top:f,left:0}).top-n.margins.top),v&&(i.position.left=n._convertPositionTo("relative",{top:0,left:u-n.helperProportions.width}).left-n.margins.left),w&&(i.position.left=n._convertPositionTo("relative",{top:0,left:d}).left-n.margins.left)}var y=g||m||v||w;if("outer"!=o.snapMode){g=Math.abs(p-l)<=s,m=Math.abs(f-c)<=s,v=Math.abs(u-a)<=s,w=Math.abs(d-r)<=s;g&&(i.position.top=n._convertPositionTo("relative",{top:p,left:0}).top-n.margins.top),m&&(i.position.top=n._convertPositionTo("relative",{top:f-n.helperProportions.height,left:0}).top-n.margins.top),v&&(i.position.left=n._convertPositionTo("relative",{top:0,left:u}).left-n.margins.left),w&&(i.position.left=n._convertPositionTo("relative",{top:0,left:d-n.helperProportions.width}).left-n.margins.left)}!n.snapElements[h].snapping&&(g||m||v||w||y)&&n.options.snap.snap&&n.options.snap.snap.call(n.element,t,e.extend(n._uiHash(),{snapItem:n.snapElements[h].item})),n.snapElements[h].snapping=g||m||v||w||y}else n.snapElements[h].snapping&&n.options.snap.release&&n.options.snap.release.call(n.element,t,e.extend(n._uiHash(),{snapItem:n.snapElements[h].item})),n.snapElements[h].snapping=!1}}}),e.ui.plugin.add("draggable","stack",{start:function(t,i){var n=e(this).data("draggable").options,o=e.makeArray(e(n.stack)).sort((function(t,i){return(parseInt(e(t).css("zIndex"),10)||0)-(parseInt(e(i).css("zIndex"),10)||0)}));if(o.length){var s=parseInt(o[0].style.zIndex)||0;e(o).each((function(e){this.style.zIndex=s+e})),this[0].style.zIndex=s+o.length}}}),e.ui.plugin.add("draggable","zIndex",{start:function(t,i){var n=e(i.helper),o=e(this).data("draggable").options;n.css("zIndex")&&(o._zIndex=n.css("zIndex")),n.css("zIndex",o.zIndex)},stop:function(t,i){var n=e(this).data("draggable").options;n._zIndex&&e(i.helper).css("zIndex",n._zIndex)}})}(jQuery),function(e,t){e.widget("ui.droppable",{version:"1.9.2",widgetEventPrefix:"drop",options:{accept:"*",activeClass:!1,addClasses:!0,greedy:!1,hoverClass:!1,scope:"default",tolerance:"intersect"},_create:function(){var t=this.options,i=t.accept;this.isover=0,this.isout=1,this.accept=e.isFunction(i)?i:function(e){return e.is(i)},this.proportions={width:this.element[0].offsetWidth,height:this.element[0].offsetHeight},e.ui.ddmanager.droppables[t.scope]=e.ui.ddmanager.droppables[t.scope]||[],e.ui.ddmanager.droppables[t.scope].push(this),t.addClasses&&this.element.addClass("ui-droppable")},_destroy:function(){for(var t=e.ui.ddmanager.droppables[this.options.scope],i=0;i<t.length;i++)t[i]==this&&t.splice(i,1);this.element.removeClass("ui-droppable ui-droppable-disabled")},_setOption:function(t,i){"accept"==t&&(this.accept=e.isFunction(i)?i:function(e){return e.is(i)}),e.Widget.prototype._setOption.apply(this,arguments)},_activate:function(t){var i=e.ui.ddmanager.current;this.options.activeClass&&this.element.addClass(this.options.activeClass),i&&this._trigger("activate",t,this.ui(i))},_deactivate:function(t){var i=e.ui.ddmanager.current;this.options.activeClass&&this.element.removeClass(this.options.activeClass),i&&this._trigger("deactivate",t,this.ui(i))},_over:function(t){var i=e.ui.ddmanager.current;i&&(i.currentItem||i.element)[0]!=this.element[0]&&this.accept.call(this.element[0],i.currentItem||i.element)&&(this.options.hoverClass&&this.element.addClass(this.options.hoverClass),this._trigger("over",t,this.ui(i)))},_out:function(t){var i=e.ui.ddmanager.current;i&&(i.currentItem||i.element)[0]!=this.element[0]&&this.accept.call(this.element[0],i.currentItem||i.element)&&(this.options.hoverClass&&this.element.removeClass(this.options.hoverClass),this._trigger("out",t,this.ui(i)))},_drop:function(t,i){var n=i||e.ui.ddmanager.current;if(!n||(n.currentItem||n.element)[0]==this.element[0])return!1;var o=!1;return this.element.find(":data(droppable)").not(".ui-draggable-dragging").each((function(){var t=e.data(this,"droppable");if(t.options.greedy&&!t.options.disabled&&t.options.scope==n.options.scope&&t.accept.call(t.element[0],n.currentItem||n.element)&&e.ui.intersect(n,e.extend(t,{offset:t.element.offset()}),t.options.tolerance))return o=!0,!1})),!o&&(!!this.accept.call(this.element[0],n.currentItem||n.element)&&(this.options.activeClass&&this.element.removeClass(this.options.activeClass),this.options.hoverClass&&this.element.removeClass(this.options.hoverClass),this._trigger("drop",t,this.ui(n)),this.element))},ui:function(e){return{draggable:e.currentItem||e.element,helper:e.helper,position:e.position,offset:e.positionAbs}}}),e.ui.intersect=function(t,i,n){if(!i.offset)return!1;var o=(t.positionAbs||t.position.absolute).left,s=o+t.helperProportions.width,a=(t.positionAbs||t.position.absolute).top,r=a+t.helperProportions.height,l=i.offset.left,c=l+i.proportions.width,h=i.offset.top,u=h+i.proportions.height;switch(n){case"fit":return l<=o&&s<=c&&h<=a&&r<=u;case"intersect":return l<o+t.helperProportions.width/2&&s-t.helperProportions.width/2<c&&h<a+t.helperProportions.height/2&&r-t.helperProportions.height/2<u;case"pointer":var d=(t.positionAbs||t.position.absolute).left+(t.clickOffset||t.offset.click).left,p=(t.positionAbs||t.position.absolute).top+(t.clickOffset||t.offset.click).top,f=e.ui.isOver(p,d,h,l,i.proportions.height,i.proportions.width);return f;case"touch":return(a>=h&&a<=u||r>=h&&r<=u||a<h&&r>u)&&(o>=l&&o<=c||s>=l&&s<=c||o<l&&s>c);default:return!1}},e.ui.ddmanager={current:null,droppables:{default:[]},prepareOffsets:function(t,i){var n=e.ui.ddmanager.droppables[t.options.scope]||[],o=i?i.type:null,s=(t.currentItem||t.element).find(":data(droppable)").andSelf();e:for(var a=0;a<n.length;a++)if(!(n[a].options.disabled||t&&!n[a].accept.call(n[a].element[0],t.currentItem||t.element))){for(var r=0;r<s.length;r++)if(s[r]==n[a].element[0]){n[a].proportions.height=0;continue e}n[a].visible="none"!=n[a].element.css("display"),n[a].visible&&("mousedown"==o&&n[a]._activate.call(n[a],i),n[a].offset=n[a].element.offset(),n[a].proportions={width:n[a].element[0].offsetWidth,height:n[a].element[0].offsetHeight})}},drop:function(t,i){var n=!1;return e.each(e.ui.ddmanager.droppables[t.options.scope]||[],(function(){this.options&&(!this.options.disabled&&this.visible&&e.ui.intersect(t,this,this.options.tolerance)&&(n=this._drop.call(this,i)||n),!this.options.disabled&&this.visible&&this.accept.call(this.element[0],t.currentItem||t.element)&&(this.isout=1,this.isover=0,this._deactivate.call(this,i)))})),n},dragStart:function(t,i){t.element.parentsUntil("body").bind("scroll.droppable",(function(){t.options.refreshPositions||e.ui.ddmanager.prepareOffsets(t,i)}))},drag:function(t,i){t.options.refreshPositions&&e.ui.ddmanager.prepareOffsets(t,i),e.each(e.ui.ddmanager.droppables[t.options.scope]||[],(function(){if(!this.options.disabled&&!this.greedyChild&&this.visible){var n=e.ui.intersect(t,this,this.options.tolerance),o=n||1!=this.isover?n&&0==this.isover?"isover":null:"isout";if(o){var s;if(this.options.greedy){var a=this.options.scope,r=this.element.parents(":data(droppable)").filter((function(){return e.data(this,"droppable").options.scope===a}));r.length&&(s=e.data(r[0],"droppable"),s.greedyChild="isover"==o?1:0)}s&&"isover"==o&&(s["isover"]=0,s["isout"]=1,s._out.call(s,i)),this[o]=1,this["isout"==o?"isover":"isout"]=0,this["isover"==o?"_over":"_out"].call(this,i),s&&"isout"==o&&(s["isout"]=0,s["isover"]=1,s._over.call(s,i))}}}))},dragStop:function(t,i){t.element.parentsUntil("body").unbind("scroll.droppable"),t.options.refreshPositions||e.ui.ddmanager.prepareOffsets(t,i)}}}(jQuery),function(e,t){e.widget("ui.resizable",e.ui.mouse,{version:"1.9.2",widgetEventPrefix:"resize",options:{alsoResize:!1,animate:!1,animateDuration:"slow",animateEasing:"swing",aspectRatio:!1,autoHide:!1,containment:!1,ghost:!1,grid:!1,handles:"e,s,se",helper:!1,maxHeight:null,maxWidth:null,minHeight:10,minWidth:10,zIndex:1e3},_create:function(){var t=this,i=this.options;if(this.element.addClass("ui-resizable"),e.extend(this,{_aspectRatio:!!i.aspectRatio,aspectRatio:i.aspectRatio,originalElement:this.element,_proportionallyResizeElements:[],_helper:i.helper||i.ghost||i.animate?i.helper||"ui-resizable-helper":null}),this.element[0].nodeName.match(/canvas|textarea|input|select|button|img/i)&&(this.element.wrap(e('<div class="ui-wrapper" style="overflow: hidden;"></div>').css({position:this.element.css("position"),width:this.element.outerWidth(),height:this.element.outerHeight(),top:this.element.css("top"),left:this.element.css("left")})),this.element=this.element.parent().data("resizable",this.element.data("resizable")),this.elementIsWrapper=!0,this.element.css({marginLeft:this.originalElement.css("marginLeft"),marginTop:this.originalElement.css("marginTop"),marginRight:this.originalElement.css("marginRight"),marginBottom:this.originalElement.css("marginBottom")}),this.originalElement.css({marginLeft:0,marginTop:0,marginRight:0,marginBottom:0}),this.originalResizeStyle=this.originalElement.css("resize"),this.originalElement.css("resize","none"),this._proportionallyResizeElements.push(this.originalElement.css({position:"static",zoom:1,display:"block"})),this.originalElement.css({margin:this.originalElement.css("margin")}),this._proportionallyResize()),this.handles=i.handles||(e(".ui-resizable-handle",this.element).length?{n:".ui-resizable-n",e:".ui-resizable-e",s:".ui-resizable-s",w:".ui-resizable-w",se:".ui-resizable-se",sw:".ui-resizable-sw",ne:".ui-resizable-ne",nw:".ui-resizable-nw"}:"e,s,se"),this.handles.constructor==String){"all"==this.handles&&(this.handles="n,e,s,w,se,sw,ne,nw");var n=this.handles.split(",");this.handles={};for(var o=0;o<n.length;o++){var s=e.trim(n[o]),a="ui-resizable-"+s,r=e('<div class="ui-resizable-handle '+a+'"></div>');r.css({zIndex:i.zIndex}),"se"==s&&r.addClass("ui-icon ui-icon-gripsmall-diagonal-se"),this.handles[s]=".ui-resizable-"+s,this.element.append(r)}}this._renderAxis=function(t){for(var i in t=t||this.element,this.handles){if(this.handles[i].constructor==String&&(this.handles[i]=e(this.handles[i],this.element).show()),this.elementIsWrapper&&this.originalElement[0].nodeName.match(/textarea|input|select|button/i)){var n=e(this.handles[i],this.element),o=0;o=/sw|ne|nw|se|n|s/.test(i)?n.outerHeight():n.outerWidth();var s=["padding",/ne|nw|n/.test(i)?"Top":/se|sw|s/.test(i)?"Bottom":/^e$/.test(i)?"Right":"Left"].join("");t.css(s,o),this._proportionallyResize()}e(this.handles[i]).length}},this._renderAxis(this.element),this._handles=e(".ui-resizable-handle",this.element).disableSelection(),this._handles.mouseover((function(){if(!t.resizing){if(this.className)var e=this.className.match(/ui-resizable-(se|sw|ne|nw|n|e|s|w)/i);t.axis=e&&e[1]?e[1]:"se"}})),i.autoHide&&(this._handles.hide(),e(this.element).addClass("ui-resizable-autohide").mouseenter((function(){i.disabled||(e(this).removeClass("ui-resizable-autohide"),t._handles.show())})).mouseleave((function(){i.disabled||t.resizing||(e(this).addClass("ui-resizable-autohide"),t._handles.hide())}))),this._mouseInit()},_destroy:function(){this._mouseDestroy();var t=function(t){e(t).removeClass("ui-resizable ui-resizable-disabled ui-resizable-resizing").removeData("resizable").removeData("ui-resizable").unbind(".resizable").find(".ui-resizable-handle").remove()};if(this.elementIsWrapper){t(this.element);var i=this.element;this.originalElement.css({position:i.css("position"),width:i.outerWidth(),height:i.outerHeight(),top:i.css("top"),left:i.css("left")}).insertAfter(i),i.remove()}return this.originalElement.css("resize",this.originalResizeStyle),t(this.originalElement),this},_mouseCapture:function(t){var i=!1;for(var n in this.handles)e(this.handles[n])[0]==t.target&&(i=!0);return!this.options.disabled&&i},_mouseStart:function(t){var n=this.options,o=this.element.position(),s=this.element;this.resizing=!0,this.documentScroll={top:e(document).scrollTop(),left:e(document).scrollLeft()},(s.is(".ui-draggable")||/absolute/.test(s.css("position")))&&s.css({position:"absolute",top:o.top,left:o.left}),this._renderProxy();var a=i(this.helper.css("left")),r=i(this.helper.css("top"));n.containment&&(a+=e(n.containment).scrollLeft()||0,r+=e(n.containment).scrollTop()||0),this.offset=this.helper.offset(),this.position={left:a,top:r},this.size=this._helper?{width:s.outerWidth(),height:s.outerHeight()}:{width:s.width(),height:s.height()},this.originalSize=this._helper?{width:s.outerWidth(),height:s.outerHeight()}:{width:s.width(),height:s.height()},this.originalPosition={left:a,top:r},this.sizeDiff={width:s.outerWidth()-s.width(),height:s.outerHeight()-s.height()},this.originalMousePosition={left:t.pageX,top:t.pageY},this.aspectRatio="number"==typeof n.aspectRatio?n.aspectRatio:this.originalSize.width/this.originalSize.height||1;var l=e(".ui-resizable-"+this.axis).css("cursor");return e("body").css("cursor","auto"==l?this.axis+"-resize":l),s.addClass("ui-resizable-resizing"),this._propagate("start",t),!0},_mouseDrag:function(e){var t=this.helper,i=(this.options,this.originalMousePosition),n=this.axis,o=e.pageX-i.left||0,s=e.pageY-i.top||0,a=this._change[n];if(!a)return!1;var r=a.apply(this,[e,o,s]);return this._updateVirtualBoundaries(e.shiftKey),(this._aspectRatio||e.shiftKey)&&(r=this._updateRatio(r,e)),r=this._respectSize(r,e),this._propagate("resize",e),t.css({top:this.position.top+"px",left:this.position.left+"px",width:this.size.width+"px",height:this.size.height+"px"}),!this._helper&&this._proportionallyResizeElements.length&&this._proportionallyResize(),this._updateCache(r),this._trigger("resize",e,this.ui()),!1},_mouseStop:function(t){this.resizing=!1;var i=this.options,n=this;if(this._helper){var o=this._proportionallyResizeElements,s=o.length&&/textarea/i.test(o[0].nodeName),a=s&&e.ui.hasScroll(o[0],"left")?0:n.sizeDiff.height,r=s?0:n.sizeDiff.width,l={width:n.helper.width()-r,height:n.helper.height()-a},c=parseInt(n.element.css("left"),10)+(n.position.left-n.originalPosition.left)||null,h=parseInt(n.element.css("top"),10)+(n.position.top-n.originalPosition.top)||null;i.animate||this.element.css(e.extend(l,{top:h,left:c})),n.helper.height(n.size.height),n.helper.width(n.size.width),this._helper&&!i.animate&&this._proportionallyResize()}return e("body").css("cursor","auto"),this.element.removeClass("ui-resizable-resizing"),this._propagate("stop",t),this._helper&&this.helper.remove(),!1},_updateVirtualBoundaries:function(e){var t,i,o,s,a,r=this.options;a={minWidth:n(r.minWidth)?r.minWidth:0,maxWidth:n(r.maxWidth)?r.maxWidth:1/0,minHeight:n(r.minHeight)?r.minHeight:0,maxHeight:n(r.maxHeight)?r.maxHeight:1/0},(this._aspectRatio||e)&&(t=a.minHeight*this.aspectRatio,o=a.minWidth/this.aspectRatio,i=a.maxHeight*this.aspectRatio,s=a.maxWidth/this.aspectRatio,t>a.minWidth&&(a.minWidth=t),o>a.minHeight&&(a.minHeight=o),i<a.maxWidth&&(a.maxWidth=i),s<a.maxHeight&&(a.maxHeight=s)),this._vBoundaries=a},_updateCache:function(e){this.options;this.offset=this.helper.offset(),n(e.left)&&(this.position.left=e.left),n(e.top)&&(this.position.top=e.top),n(e.height)&&(this.size.height=e.height),n(e.width)&&(this.size.width=e.width)},_updateRatio:function(e,t){this.options;var i=this.position,o=this.size,s=this.axis;return n(e.height)?e.width=e.height*this.aspectRatio:n(e.width)&&(e.height=e.width/this.aspectRatio),"sw"==s&&(e.left=i.left+(o.width-e.width),e.top=null),"nw"==s&&(e.top=i.top+(o.height-e.height),e.left=i.left+(o.width-e.width)),e},_respectSize:function(e,t){this.helper;var i=this._vBoundaries,o=(this._aspectRatio||t.shiftKey,this.axis),s=n(e.width)&&i.maxWidth&&i.maxWidth<e.width,a=n(e.height)&&i.maxHeight&&i.maxHeight<e.height,r=n(e.width)&&i.minWidth&&i.minWidth>e.width,l=n(e.height)&&i.minHeight&&i.minHeight>e.height;r&&(e.width=i.minWidth),l&&(e.height=i.minHeight),s&&(e.width=i.maxWidth),a&&(e.height=i.maxHeight);var c=this.originalPosition.left+this.originalSize.width,h=this.position.top+this.size.height,u=/sw|nw|w/.test(o),d=/nw|ne|n/.test(o);r&&u&&(e.left=c-i.minWidth),s&&u&&(e.left=c-i.maxWidth),l&&d&&(e.top=h-i.minHeight),a&&d&&(e.top=h-i.maxHeight);var p=!e.width&&!e.height;return p&&!e.left&&e.top?e.top=null:p&&!e.top&&e.left&&(e.left=null),e},_proportionallyResize:function(){this.options;if(this._proportionallyResizeElements.length)for(var t=this.helper||this.element,i=0;i<this._proportionallyResizeElements.length;i++){var n=this._proportionallyResizeElements[i];if(!this.borderDif){var o=[n.css("borderTopWidth"),n.css("borderRightWidth"),n.css("borderBottomWidth"),n.css("borderLeftWidth")],s=[n.css("paddingTop"),n.css("paddingRight"),n.css("paddingBottom"),n.css("paddingLeft")];this.borderDif=e.map(o,(function(e,t){var i=parseInt(e,10)||0,n=parseInt(s[t],10)||0;return i+n}))}n.css({height:t.height()-this.borderDif[0]-this.borderDif[2]||0,width:t.width()-this.borderDif[1]-this.borderDif[3]||0})}},_renderProxy:function(){var t=this.element,i=this.options;if(this.elementOffset=t.offset(),this._helper){this.helper=this.helper||e('<div style="overflow:hidden;"></div>');var n=e.ui.ie6?1:0,o=e.ui.ie6?2:-1;this.helper.addClass(this._helper).css({width:this.element.outerWidth()+o,height:this.element.outerHeight()+o,position:"absolute",left:this.elementOffset.left-n+"px",top:this.elementOffset.top-n+"px",zIndex:++i.zIndex}),this.helper.appendTo("body").disableSelection()}else this.helper=this.element},_change:{e:function(e,t,i){return{width:this.originalSize.width+t}},w:function(e,t,i){this.options;var n=this.originalSize,o=this.originalPosition;return{left:o.left+t,width:n.width-t}},n:function(e,t,i){this.options;var n=this.originalSize,o=this.originalPosition;return{top:o.top+i,height:n.height-i}},s:function(e,t,i){return{height:this.originalSize.height+i}},se:function(t,i,n){return e.extend(this._change.s.apply(this,arguments),this._change.e.apply(this,[t,i,n]))},sw:function(t,i,n){return e.extend(this._change.s.apply(this,arguments),this._change.w.apply(this,[t,i,n]))},ne:function(t,i,n){return e.extend(this._change.n.apply(this,arguments),this._change.e.apply(this,[t,i,n]))},nw:function(t,i,n){return e.extend(this._change.n.apply(this,arguments),this._change.w.apply(this,[t,i,n]))}},_propagate:function(t,i){e.ui.plugin.call(this,t,[i,this.ui()]),"resize"!=t&&this._trigger(t,i,this.ui())},plugins:{},ui:function(){return{originalElement:this.originalElement,element:this.element,helper:this.helper,position:this.position,size:this.size,originalSize:this.originalSize,originalPosition:this.originalPosition}}}),e.ui.plugin.add("resizable","alsoResize",{start:function(t,i){var n=e(this).data("resizable"),o=n.options,s=function(t){e(t).each((function(){var t=e(this);t.data("resizable-alsoresize",{width:parseInt(t.width(),10),height:parseInt(t.height(),10),left:parseInt(t.css("left"),10),top:parseInt(t.css("top"),10)})}))};"object"!=typeof o.alsoResize||o.alsoResize.parentNode?s(o.alsoResize):o.alsoResize.length?(o.alsoResize=o.alsoResize[0],s(o.alsoResize)):e.each(o.alsoResize,(function(e){s(e)}))},resize:function(t,i){var n=e(this).data("resizable"),o=n.options,s=n.originalSize,a=n.originalPosition,r={height:n.size.height-s.height||0,width:n.size.width-s.width||0,top:n.position.top-a.top||0,left:n.position.left-a.left||0},l=function(t,n){e(t).each((function(){var t=e(this),o=e(this).data("resizable-alsoresize"),s={},a=n&&n.length?n:t.parents(i.originalElement[0]).length?["width","height"]:["width","height","top","left"];e.each(a,(function(e,t){var i=(o[t]||0)+(r[t]||0);i&&i>=0&&(s[t]=i||null)})),t.css(s)}))};"object"!=typeof o.alsoResize||o.alsoResize.nodeType?l(o.alsoResize):e.each(o.alsoResize,(function(e,t){l(e,t)}))},stop:function(t,i){e(this).removeData("resizable-alsoresize")}}),e.ui.plugin.add("resizable","animate",{stop:function(t,i){var n=e(this).data("resizable"),o=n.options,s=n._proportionallyResizeElements,a=s.length&&/textarea/i.test(s[0].nodeName),r=a&&e.ui.hasScroll(s[0],"left")?0:n.sizeDiff.height,l=a?0:n.sizeDiff.width,c={width:n.size.width-l,height:n.size.height-r},h=parseInt(n.element.css("left"),10)+(n.position.left-n.originalPosition.left)||null,u=parseInt(n.element.css("top"),10)+(n.position.top-n.originalPosition.top)||null;n.element.animate(e.extend(c,u&&h?{top:u,left:h}:{}),{duration:o.animateDuration,easing:o.animateEasing,step:function(){var i={width:parseInt(n.element.css("width"),10),height:parseInt(n.element.css("height"),10),top:parseInt(n.element.css("top"),10),left:parseInt(n.element.css("left"),10)};s&&s.length&&e(s[0]).css({width:i.width,height:i.height}),n._updateCache(i),n._propagate("resize",t)}})}}),e.ui.plugin.add("resizable","containment",{start:function(t,n){var o=e(this).data("resizable"),s=o.options,a=o.element,r=s.containment,l=r instanceof e?r.get(0):/parent/.test(r)?a.parent().get(0):r;if(l)if(o.containerElement=e(l),/document/.test(r)||r==document)o.containerOffset={left:0,top:0},o.containerPosition={left:0,top:0},o.parentData={element:e(document),left:0,top:0,width:e(document).width(),height:e(document).height()||document.body.parentNode.scrollHeight};else{var c=e(l),h=[];e(["Top","Right","Left","Bottom"]).each((function(e,t){h[e]=i(c.css("padding"+t))})),o.containerOffset=c.offset(),o.containerPosition=c.position(),o.containerSize={height:c.innerHeight()-h[3],width:c.innerWidth()-h[1]};var u=o.containerOffset,d=o.containerSize.height,p=o.containerSize.width,f=e.ui.hasScroll(l,"left")?l.scrollWidth:p,g=e.ui.hasScroll(l)?l.scrollHeight:d;o.parentData={element:l,left:u.left,top:u.top,width:f,height:g}}},resize:function(t,i){var n=e(this).data("resizable"),o=n.options,s=(n.containerSize,n.containerOffset),a=(n.size,n.position),r=n._aspectRatio||t.shiftKey,l={top:0,left:0},c=n.containerElement;c[0]!=document&&/static/.test(c.css("position"))&&(l=s),a.left<(n._helper?s.left:0)&&(n.size.width=n.size.width+(n._helper?n.position.left-s.left:n.position.left-l.left),r&&(n.size.height=n.size.width/n.aspectRatio),n.position.left=o.helper?s.left:0),a.top<(n._helper?s.top:0)&&(n.size.height=n.size.height+(n._helper?n.position.top-s.top:n.position.top),r&&(n.size.width=n.size.height*n.aspectRatio),n.position.top=n._helper?s.top:0),n.offset.left=n.parentData.left+n.position.left,n.offset.top=n.parentData.top+n.position.top;var h=Math.abs((n._helper,n.offset.left-l.left+n.sizeDiff.width)),u=Math.abs((n._helper?n.offset.top-l.top:n.offset.top-s.top)+n.sizeDiff.height),d=n.containerElement.get(0)==n.element.parent().get(0),p=/relative|absolute/.test(n.containerElement.css("position"));d&&p&&(h-=n.parentData.left),h+n.size.width>=n.parentData.width&&(n.size.width=n.parentData.width-h,r&&(n.size.height=n.size.width/n.aspectRatio)),u+n.size.height>=n.parentData.height&&(n.size.height=n.parentData.height-u,r&&(n.size.width=n.size.height*n.aspectRatio))},stop:function(t,i){var n=e(this).data("resizable"),o=n.options,s=(n.position,n.containerOffset),a=n.containerPosition,r=n.containerElement,l=e(n.helper),c=l.offset(),h=l.outerWidth()-n.sizeDiff.width,u=l.outerHeight()-n.sizeDiff.height;n._helper&&!o.animate&&/relative/.test(r.css("position"))&&e(this).css({left:c.left-a.left-s.left,width:h,height:u}),n._helper&&!o.animate&&/static/.test(r.css("position"))&&e(this).css({left:c.left-a.left-s.left,width:h,height:u})}}),e.ui.plugin.add("resizable","ghost",{start:function(t,i){var n=e(this).data("resizable"),o=n.options,s=n.size;n.ghost=n.originalElement.clone(),n.ghost.css({opacity:.25,display:"block",position:"relative",height:s.height,width:s.width,margin:0,left:0,top:0}).addClass("ui-resizable-ghost").addClass("string"==typeof o.ghost?o.ghost:""),n.ghost.appendTo(n.helper)},resize:function(t,i){var n=e(this).data("resizable");n.options;n.ghost&&n.ghost.css({position:"relative",height:n.size.height,width:n.size.width})},stop:function(t,i){var n=e(this).data("resizable");n.options;n.ghost&&n.helper&&n.helper.get(0).removeChild(n.ghost.get(0))}}),e.ui.plugin.add("resizable","grid",{resize:function(t,i){var n=e(this).data("resizable"),o=n.options,s=n.size,a=n.originalSize,r=n.originalPosition,l=n.axis;o._aspectRatio||t.shiftKey;o.grid="number"==typeof o.grid?[o.grid,o.grid]:o.grid;var c=Math.round((s.width-a.width)/(o.grid[0]||1))*(o.grid[0]||1),h=Math.round((s.height-a.height)/(o.grid[1]||1))*(o.grid[1]||1);/^(se|s|e)$/.test(l)?(n.size.width=a.width+c,n.size.height=a.height+h):/^(ne)$/.test(l)?(n.size.width=a.width+c,n.size.height=a.height+h,n.position.top=r.top-h):/^(sw)$/.test(l)?(n.size.width=a.width+c,n.size.height=a.height+h,n.position.left=r.left-c):(n.size.width=a.width+c,n.size.height=a.height+h,n.position.top=r.top-h,n.position.left=r.left-c)}});var i=function(e){return parseInt(e,10)||0},n=function(e){return!isNaN(parseInt(e,10))}}(jQuery),function(e,t){e.widget("ui.selectable",e.ui.mouse,{version:"1.9.2",options:{appendTo:"body",autoRefresh:!0,distance:0,filter:"*",tolerance:"touch"},_create:function(){var t,i=this;this.element.addClass("ui-selectable"),this.dragged=!1,this.refresh=function(){t=e(i.options.filter,i.element[0]),t.addClass("ui-selectee"),t.each((function(){var t=e(this),i=t.offset();e.data(this,"selectable-item",{element:this,$element:t,left:i.left,top:i.top,right:i.left+t.outerWidth(),bottom:i.top+t.outerHeight(),startselected:!1,selected:t.hasClass("ui-selected"),selecting:t.hasClass("ui-selecting"),unselecting:t.hasClass("ui-unselecting")})}))},this.refresh(),this.selectees=t.addClass("ui-selectee"),this._mouseInit(),this.helper=e("<div class='ui-selectable-helper'></div>")},_destroy:function(){this.selectees.removeClass("ui-selectee").removeData("selectable-item"),this.element.removeClass("ui-selectable ui-selectable-disabled"),this._mouseDestroy()},_mouseStart:function(t){var i=this;if(this.opos=[t.pageX,t.pageY],!this.options.disabled){var n=this.options;this.selectees=e(n.filter,this.element[0]),this._trigger("start",t),e(n.appendTo).append(this.helper),this.helper.css({left:t.clientX,top:t.clientY,width:0,height:0}),n.autoRefresh&&this.refresh(),this.selectees.filter(".ui-selected").each((function(){var n=e.data(this,"selectable-item");n.startselected=!0,t.metaKey||t.ctrlKey||(n.$element.removeClass("ui-selected"),n.selected=!1,n.$element.addClass("ui-unselecting"),n.unselecting=!0,i._trigger("unselecting",t,{unselecting:n.element}))})),e(t.target).parents().andSelf().each((function(){var n=e.data(this,"selectable-item");if(n){var o=!t.metaKey&&!t.ctrlKey||!n.$element.hasClass("ui-selected");return n.$element.removeClass(o?"ui-unselecting":"ui-selected").addClass(o?"ui-selecting":"ui-unselecting"),n.unselecting=!o,n.selecting=o,n.selected=o,o?i._trigger("selecting",t,{selecting:n.element}):i._trigger("unselecting",t,{unselecting:n.element}),!1}}))}},_mouseDrag:function(t){var i=this;if(this.dragged=!0,!this.options.disabled){var n=this.options,o=this.opos[0],s=this.opos[1],a=t.pageX,r=t.pageY;if(o>a){var l=a;a=o,o=l}if(s>r){l=r;r=s,s=l}return this.helper.css({left:o,top:s,width:a-o,height:r-s}),this.selectees.each((function(){var l=e.data(this,"selectable-item");if(l&&l.element!=i.element[0]){var c=!1;"touch"==n.tolerance?c=!(l.left>a||l.right<o||l.top>r||l.bottom<s):"fit"==n.tolerance&&(c=l.left>o&&l.right<a&&l.top>s&&l.bottom<r),c?(l.selected&&(l.$element.removeClass("ui-selected"),l.selected=!1),l.unselecting&&(l.$element.removeClass("ui-unselecting"),l.unselecting=!1),l.selecting||(l.$element.addClass("ui-selecting"),l.selecting=!0,i._trigger("selecting",t,{selecting:l.element}))):(l.selecting&&((t.metaKey||t.ctrlKey)&&l.startselected?(l.$element.removeClass("ui-selecting"),l.selecting=!1,l.$element.addClass("ui-selected"),l.selected=!0):(l.$element.removeClass("ui-selecting"),l.selecting=!1,l.startselected&&(l.$element.addClass("ui-unselecting"),l.unselecting=!0),i._trigger("unselecting",t,{unselecting:l.element}))),l.selected&&(t.metaKey||t.ctrlKey||l.startselected||(l.$element.removeClass("ui-selected"),l.selected=!1,l.$element.addClass("ui-unselecting"),l.unselecting=!0,i._trigger("unselecting",t,{unselecting:l.element}))))}})),!1}},_mouseStop:function(t){var i=this;this.dragged=!1;this.options;return e(".ui-unselecting",this.element[0]).each((function(){var n=e.data(this,"selectable-item");n.$element.removeClass("ui-unselecting"),n.unselecting=!1,n.startselected=!1,i._trigger("unselected",t,{unselected:n.element})})),e(".ui-selecting",this.element[0]).each((function(){var n=e.data(this,"selectable-item");n.$element.removeClass("ui-selecting").addClass("ui-selected"),n.selecting=!1,n.selected=!0,n.startselected=!0,i._trigger("selected",t,{selected:n.element})})),this._trigger("stop",t),this.helper.remove(),!1}})}(jQuery),function(e,t){e.widget("ui.sortable",e.ui.mouse,{version:"1.9.2",widgetEventPrefix:"sort",ready:!1,options:{appendTo:"parent",axis:!1,connectWith:!1,containment:!1,cursor:"auto",cursorAt:!1,dropOnEmpty:!0,forcePlaceholderSize:!1,forceHelperSize:!1,grid:!1,handle:!1,helper:"original",items:"> *",opacity:!1,placeholder:!1,revert:!1,scroll:!0,scrollSensitivity:20,scrollSpeed:20,scope:"default",tolerance:"intersect",zIndex:1e3},_create:function(){var e=this.options;this.containerCache={},this.element.addClass("ui-sortable"),this.refresh(),this.floating=!!this.items.length&&("x"===e.axis||/left|right/.test(this.items[0].item.css("float"))||/inline|table-cell/.test(this.items[0].item.css("display"))),this.offset=this.element.offset(),this._mouseInit(),this.ready=!0},_destroy:function(){this.element.removeClass("ui-sortable ui-sortable-disabled"),this._mouseDestroy();for(var e=this.items.length-1;e>=0;e--)this.items[e].item.removeData(this.widgetName+"-item");return this},_setOption:function(t,i){"disabled"===t?(this.options[t]=i,this.widget().toggleClass("ui-sortable-disabled",!!i)):e.Widget.prototype._setOption.apply(this,arguments)},_mouseCapture:function(t,i){var n=this;if(this.reverting)return!1;if(this.options.disabled||"static"==this.options.type)return!1;this._refreshItems(t);var o=null;e(t.target).parents().each((function(){if(e.data(this,n.widgetName+"-item")==n)return o=e(this),!1}));if(e.data(t.target,n.widgetName+"-item")==n&&(o=e(t.target)),!o)return!1;if(this.options.handle&&!i){var s=!1;if(e(this.options.handle,o).find("*").andSelf().each((function(){this==t.target&&(s=!0)})),!s)return!1}return this.currentItem=o,this._removeCurrentsFromItems(),!0},_mouseStart:function(t,i,n){var o=this.options;if(this.currentContainer=this,this.refreshPositions(),this.helper=this._createHelper(t),this._cacheHelperProportions(),this._cacheMargins(),this.scrollParent=this.helper.scrollParent(),this.offset=this.currentItem.offset(),this.offset={top:this.offset.top-this.margins.top,left:this.offset.left-this.margins.left},e.extend(this.offset,{click:{left:t.pageX-this.offset.left,top:t.pageY-this.offset.top},parent:this._getParentOffset(),relative:this._getRelativeOffset()}),this.helper.css("position","absolute"),this.cssPosition=this.helper.css("position"),this.originalPosition=this._generatePosition(t),this.originalPageX=t.pageX,this.originalPageY=t.pageY,o.cursorAt&&this._adjustOffsetFromHelper(o.cursorAt),this.domPosition={prev:this.currentItem.prev()[0],parent:this.currentItem.parent()[0]},this.helper[0]!=this.currentItem[0]&&this.currentItem.hide(),this._createPlaceholder(),o.containment&&this._setContainment(),o.cursor&&(e("body").css("cursor")&&(this._storedCursor=e("body").css("cursor")),e("body").css("cursor",o.cursor)),o.opacity&&(this.helper.css("opacity")&&(this._storedOpacity=this.helper.css("opacity")),this.helper.css("opacity",o.opacity)),o.zIndex&&(this.helper.css("zIndex")&&(this._storedZIndex=this.helper.css("zIndex")),this.helper.css("zIndex",o.zIndex)),this.scrollParent[0]!=document&&"HTML"!=this.scrollParent[0].tagName&&(this.overflowOffset=this.scrollParent.offset()),this._trigger("start",t,this._uiHash()),this._preserveHelperProportions||this._cacheHelperProportions(),!n)for(var s=this.containers.length-1;s>=0;s--)this.containers[s]._trigger("activate",t,this._uiHash(this));return e.ui.ddmanager&&(e.ui.ddmanager.current=this),e.ui.ddmanager&&!o.dropBehaviour&&e.ui.ddmanager.prepareOffsets(this,t),this.dragging=!0,this.helper.addClass("ui-sortable-helper"),this._mouseDrag(t),!0},_mouseDrag:function(t){if(this.position=this._generatePosition(t),this.positionAbs=this._convertPositionTo("absolute"),this.lastPositionAbs||(this.lastPositionAbs=this.positionAbs),this.options.scroll){var i=this.options,n=!1;this.scrollParent[0]!=document&&"HTML"!=this.scrollParent[0].tagName?(this.overflowOffset.top+this.scrollParent[0].offsetHeight-t.pageY<i.scrollSensitivity?this.scrollParent[0].scrollTop=n=this.scrollParent[0].scrollTop+i.scrollSpeed:t.pageY-this.overflowOffset.top<i.scrollSensitivity&&(this.scrollParent[0].scrollTop=n=this.scrollParent[0].scrollTop-i.scrollSpeed),this.overflowOffset.left+this.scrollParent[0].offsetWidth-t.pageX<i.scrollSensitivity?this.scrollParent[0].scrollLeft=n=this.scrollParent[0].scrollLeft+i.scrollSpeed:t.pageX-this.overflowOffset.left<i.scrollSensitivity&&(this.scrollParent[0].scrollLeft=n=this.scrollParent[0].scrollLeft-i.scrollSpeed)):(t.pageY-e(document).scrollTop()<i.scrollSensitivity?n=e(document).scrollTop(e(document).scrollTop()-i.scrollSpeed):e(window).height()-(t.pageY-e(document).scrollTop())<i.scrollSensitivity&&(n=e(document).scrollTop(e(document).scrollTop()+i.scrollSpeed)),t.pageX-e(document).scrollLeft()<i.scrollSensitivity?n=e(document).scrollLeft(e(document).scrollLeft()-i.scrollSpeed):e(window).width()-(t.pageX-e(document).scrollLeft())<i.scrollSensitivity&&(n=e(document).scrollLeft(e(document).scrollLeft()+i.scrollSpeed))),!1!==n&&e.ui.ddmanager&&!i.dropBehaviour&&e.ui.ddmanager.prepareOffsets(this,t)}this.positionAbs=this._convertPositionTo("absolute"),this.options.axis&&"y"==this.options.axis||(this.helper[0].style.left=this.position.left+"px"),this.options.axis&&"x"==this.options.axis||(this.helper[0].style.top=this.position.top+"px");for(var o=this.items.length-1;o>=0;o--){var s=this.items[o],a=s.item[0],r=this._intersectsWithPointer(s);if(r&&(s.instance===this.currentContainer&&a!=this.currentItem[0]&&this.placeholder[1==r?"next":"prev"]()[0]!=a&&!e.contains(this.placeholder[0],a)&&("semi-dynamic"!=this.options.type||!e.contains(this.element[0],a)))){if(this.direction=1==r?"down":"up","pointer"!=this.options.tolerance&&!this._intersectsWithSides(s))break;this._rearrange(t,s),this._trigger("change",t,this._uiHash());break}}return this._contactContainers(t),e.ui.ddmanager&&e.ui.ddmanager.drag(this,t),this._trigger("sort",t,this._uiHash()),this.lastPositionAbs=this.positionAbs,!1},_mouseStop:function(t,i){if(t){if(e.ui.ddmanager&&!this.options.dropBehaviour&&e.ui.ddmanager.drop(this,t),this.options.revert){var n=this,o=this.placeholder.offset();this.reverting=!0,e(this.helper).animate({left:o.left-this.offset.parent.left-this.margins.left+(this.offsetParent[0]==document.body?0:this.offsetParent[0].scrollLeft),top:o.top-this.offset.parent.top-this.margins.top+(this.offsetParent[0]==document.body?0:this.offsetParent[0].scrollTop)},parseInt(this.options.revert,10)||500,(function(){n._clear(t)}))}else this._clear(t,i);return!1}},cancel:function(){if(this.dragging){this._mouseUp({target:null}),"original"==this.options.helper?this.currentItem.css(this._storedCSS).removeClass("ui-sortable-helper"):this.currentItem.show();for(var t=this.containers.length-1;t>=0;t--)this.containers[t]._trigger("deactivate",null,this._uiHash(this)),this.containers[t].containerCache.over&&(this.containers[t]._trigger("out",null,this._uiHash(this)),this.containers[t].containerCache.over=0)}return this.placeholder&&(this.placeholder[0].parentNode&&this.placeholder[0].parentNode.removeChild(this.placeholder[0]),"original"!=this.options.helper&&this.helper&&this.helper[0].parentNode&&this.helper.remove(),e.extend(this,{helper:null,dragging:!1,reverting:!1,_noFinalSort:null}),this.domPosition.prev?e(this.domPosition.prev).after(this.currentItem):e(this.domPosition.parent).prepend(this.currentItem)),this},serialize:function(t){var i=this._getItemsAsjQuery(t&&t.connected),n=[];return t=t||{},e(i).each((function(){var i=(e(t.item||this).attr(t.attribute||"id")||"").match(t.expression||/(.+)[-=_](.+)/);i&&n.push((t.key||i[1]+"[]")+"="+(t.key&&t.expression?i[1]:i[2]))})),!n.length&&t.key&&n.push(t.key+"="),n.join("&")},toArray:function(t){var i=this._getItemsAsjQuery(t&&t.connected),n=[];return t=t||{},i.each((function(){n.push(e(t.item||this).attr(t.attribute||"id")||"")})),n},_intersectsWith:function(e){var t=this.positionAbs.left,i=t+this.helperProportions.width,n=this.positionAbs.top,o=n+this.helperProportions.height,s=e.left,a=s+e.width,r=e.top,l=r+e.height,c=this.offset.click.top,h=this.offset.click.left,u=n+c>r&&n+c<l&&t+h>s&&t+h<a;return"pointer"==this.options.tolerance||this.options.forcePointerForContainers||"pointer"!=this.options.tolerance&&this.helperProportions[this.floating?"width":"height"]>e[this.floating?"width":"height"]?u:s<t+this.helperProportions.width/2&&i-this.helperProportions.width/2<a&&r<n+this.helperProportions.height/2&&o-this.helperProportions.height/2<l},_intersectsWithPointer:function(t){var i="x"===this.options.axis||e.ui.isOverAxis(this.positionAbs.top+this.offset.click.top,t.top,t.height),n="y"===this.options.axis||e.ui.isOverAxis(this.positionAbs.left+this.offset.click.left,t.left,t.width),o=i&&n,s=this._getDragVerticalDirection(),a=this._getDragHorizontalDirection();return!!o&&(this.floating?a&&"right"==a||"down"==s?2:1:s&&("down"==s?2:1))},_intersectsWithSides:function(t){var i=e.ui.isOverAxis(this.positionAbs.top+this.offset.click.top,t.top+t.height/2,t.height),n=e.ui.isOverAxis(this.positionAbs.left+this.offset.click.left,t.left+t.width/2,t.width),o=this._getDragVerticalDirection(),s=this._getDragHorizontalDirection();return this.floating&&s?"right"==s&&n||"left"==s&&!n:o&&("down"==o&&i||"up"==o&&!i)},_getDragVerticalDirection:function(){var e=this.positionAbs.top-this.lastPositionAbs.top;return 0!=e&&(e>0?"down":"up")},_getDragHorizontalDirection:function(){var e=this.positionAbs.left-this.lastPositionAbs.left;return 0!=e&&(e>0?"right":"left")},refresh:function(e){return this._refreshItems(e),this.refreshPositions(),this},_connectWith:function(){var e=this.options;return e.connectWith.constructor==String?[e.connectWith]:e.connectWith},_getItemsAsjQuery:function(t){var i=[],n=[],o=this._connectWith();if(o&&t)for(var s=o.length-1;s>=0;s--)for(var a=e(o[s]),r=a.length-1;r>=0;r--){var l=e.data(a[r],this.widgetName);l&&l!=this&&!l.options.disabled&&n.push([e.isFunction(l.options.items)?l.options.items.call(l.element):e(l.options.items,l.element).not(".ui-sortable-helper").not(".ui-sortable-placeholder"),l])}n.push([e.isFunction(this.options.items)?this.options.items.call(this.element,null,{options:this.options,item:this.currentItem}):e(this.options.items,this.element).not(".ui-sortable-helper").not(".ui-sortable-placeholder"),this]);for(s=n.length-1;s>=0;s--)n[s][0].each((function(){i.push(this)}));return e(i)},_removeCurrentsFromItems:function(){var t=this.currentItem.find(":data("+this.widgetName+"-item)");this.items=e.grep(this.items,(function(e){for(var i=0;i<t.length;i++)if(t[i]==e.item[0])return!1;return!0}))},_refreshItems:function(t){this.items=[],this.containers=[this];var i=this.items,n=[[e.isFunction(this.options.items)?this.options.items.call(this.element[0],t,{item:this.currentItem}):e(this.options.items,this.element),this]],o=this._connectWith();if(o&&this.ready)for(var s=o.length-1;s>=0;s--)for(var a=e(o[s]),r=a.length-1;r>=0;r--){var l=e.data(a[r],this.widgetName);l&&l!=this&&!l.options.disabled&&(n.push([e.isFunction(l.options.items)?l.options.items.call(l.element[0],t,{item:this.currentItem}):e(l.options.items,l.element),l]),this.containers.push(l))}for(s=n.length-1;s>=0;s--)for(var c=n[s][1],h=n[s][0],u=(r=0,h.length);r<u;r++){var d=e(h[r]);d.data(this.widgetName+"-item",c),i.push({item:d,instance:c,width:0,height:0,left:0,top:0})}},refreshPositions:function(t){this.offsetParent&&this.helper&&(this.offset.parent=this._getParentOffset());for(var i=this.items.length-1;i>=0;i--){var n=this.items[i];if(n.instance==this.currentContainer||!this.currentContainer||n.item[0]==this.currentItem[0]){var o=this.options.toleranceElement?e(this.options.toleranceElement,n.item):n.item;t||(n.width=o.outerWidth(),n.height=o.outerHeight());var s=o.offset();n.left=s.left,n.top=s.top}}if(this.options.custom&&this.options.custom.refreshContainers)this.options.custom.refreshContainers.call(this);else for(i=this.containers.length-1;i>=0;i--){s=this.containers[i].element.offset();this.containers[i].containerCache.left=s.left,this.containers[i].containerCache.top=s.top,this.containers[i].containerCache.width=this.containers[i].element.outerWidth(),this.containers[i].containerCache.height=this.containers[i].element.outerHeight()}return this},_createPlaceholder:function(t){t=t||this;var i=t.options;if(!i.placeholder||i.placeholder.constructor==String){var n=i.placeholder;i.placeholder={element:function(){var i=e(document.createElement(t.currentItem[0].nodeName)).addClass(n||t.currentItem[0].className+" ui-sortable-placeholder").removeClass("ui-sortable-helper")[0];return n||(i.style.visibility="hidden"),i},update:function(e,o){n&&!i.forcePlaceholderSize||(o.height()||o.height(t.currentItem.innerHeight()-parseInt(t.currentItem.css("paddingTop")||0,10)-parseInt(t.currentItem.css("paddingBottom")||0,10)),o.width()||o.width(t.currentItem.innerWidth()-parseInt(t.currentItem.css("paddingLeft")||0,10)-parseInt(t.currentItem.css("paddingRight")||0,10)))}}}t.placeholder=e(i.placeholder.element.call(t.element,t.currentItem)),t.currentItem.after(t.placeholder),i.placeholder.update(t,t.placeholder)},_contactContainers:function(t){for(var i=null,n=null,o=this.containers.length-1;o>=0;o--)if(!e.contains(this.currentItem[0],this.containers[o].element[0]))if(this._intersectsWith(this.containers[o].containerCache)){if(i&&e.contains(this.containers[o].element[0],i.element[0]))continue;i=this.containers[o],n=o}else this.containers[o].containerCache.over&&(this.containers[o]._trigger("out",t,this._uiHash(this)),this.containers[o].containerCache.over=0);if(i)if(1===this.containers.length)this.containers[n]._trigger("over",t,this._uiHash(this)),this.containers[n].containerCache.over=1;else{for(var s=1e4,a=null,r=this.containers[n].floating?"left":"top",l=this.containers[n].floating?"width":"height",c=this.positionAbs[r]+this.offset.click[r],h=this.items.length-1;h>=0;h--)if(e.contains(this.containers[n].element[0],this.items[h].item[0])&&this.items[h].item[0]!=this.currentItem[0]){var u=this.items[h].item.offset()[r],d=!1;Math.abs(u-c)>Math.abs(u+this.items[h][l]-c)&&(d=!0,u+=this.items[h][l]),Math.abs(u-c)<s&&(s=Math.abs(u-c),a=this.items[h],this.direction=d?"up":"down")}if(!a&&!this.options.dropOnEmpty)return;this.currentContainer=this.containers[n],a?this._rearrange(t,a,null,!0):this._rearrange(t,null,this.containers[n].element,!0),this._trigger("change",t,this._uiHash()),this.containers[n]._trigger("change",t,this._uiHash(this)),this.options.placeholder.update(this.currentContainer,this.placeholder),this.containers[n]._trigger("over",t,this._uiHash(this)),this.containers[n].containerCache.over=1}},_createHelper:function(t){var i=this.options,n=e.isFunction(i.helper)?e(i.helper.apply(this.element[0],[t,this.currentItem])):"clone"==i.helper?this.currentItem.clone():this.currentItem;return n.parents("body").length||e("parent"!=i.appendTo?i.appendTo:this.currentItem[0].parentNode)[0].appendChild(n[0]),n[0]==this.currentItem[0]&&(this._storedCSS={width:this.currentItem[0].style.width,height:this.currentItem[0].style.height,position:this.currentItem.css("position"),top:this.currentItem.css("top"),left:this.currentItem.css("left")}),(""==n[0].style.width||i.forceHelperSize)&&n.width(this.currentItem.width()),(""==n[0].style.height||i.forceHelperSize)&&n.height(this.currentItem.height()),n},_adjustOffsetFromHelper:function(t){"string"==typeof t&&(t=t.split(" ")),e.isArray(t)&&(t={left:+t[0],top:+t[1]||0}),"left"in t&&(this.offset.click.left=t.left+this.margins.left),"right"in t&&(this.offset.click.left=this.helperProportions.width-t.right+this.margins.left),"top"in t&&(this.offset.click.top=t.top+this.margins.top),"bottom"in t&&(this.offset.click.top=this.helperProportions.height-t.bottom+this.margins.top)},_getParentOffset:function(){this.offsetParent=this.helper.offsetParent();var t=this.offsetParent.offset();return"absolute"==this.cssPosition&&this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])&&(t.left+=this.scrollParent.scrollLeft(),t.top+=this.scrollParent.scrollTop()),(this.offsetParent[0]==document.body||this.offsetParent[0].tagName&&"html"==this.offsetParent[0].tagName.toLowerCase()&&e.ui.ie)&&(t={top:0,left:0}),{top:t.top+(parseInt(this.offsetParent.css("borderTopWidth"),10)||0),left:t.left+(parseInt(this.offsetParent.css("borderLeftWidth"),10)||0)}},_getRelativeOffset:function(){if("relative"==this.cssPosition){var e=this.currentItem.position();return{top:e.top-(parseInt(this.helper.css("top"),10)||0)+this.scrollParent.scrollTop(),left:e.left-(parseInt(this.helper.css("left"),10)||0)+this.scrollParent.scrollLeft()}}return{top:0,left:0}},_cacheMargins:function(){this.margins={left:parseInt(this.currentItem.css("marginLeft"),10)||0,top:parseInt(this.currentItem.css("marginTop"),10)||0}},_cacheHelperProportions:function(){this.helperProportions={width:this.helper.outerWidth(),height:this.helper.outerHeight()}},_setContainment:function(){var t=this.options;if("parent"==t.containment&&(t.containment=this.helper[0].parentNode),"document"!=t.containment&&"window"!=t.containment||(this.containment=[0-this.offset.relative.left-this.offset.parent.left,0-this.offset.relative.top-this.offset.parent.top,e("document"==t.containment?document:window).width()-this.helperProportions.width-this.margins.left,(e("document"==t.containment?document:window).height()||document.body.parentNode.scrollHeight)-this.helperProportions.height-this.margins.top]),!/^(document|window|parent)$/.test(t.containment)){var i=e(t.containment)[0],n=e(t.containment).offset(),o="hidden"!=e(i).css("overflow");this.containment=[n.left+(parseInt(e(i).css("borderLeftWidth"),10)||0)+(parseInt(e(i).css("paddingLeft"),10)||0)-this.margins.left,n.top+(parseInt(e(i).css("borderTopWidth"),10)||0)+(parseInt(e(i).css("paddingTop"),10)||0)-this.margins.top,n.left+(o?Math.max(i.scrollWidth,i.offsetWidth):i.offsetWidth)-(parseInt(e(i).css("borderLeftWidth"),10)||0)-(parseInt(e(i).css("paddingRight"),10)||0)-this.helperProportions.width-this.margins.left,n.top+(o?Math.max(i.scrollHeight,i.offsetHeight):i.offsetHeight)-(parseInt(e(i).css("borderTopWidth"),10)||0)-(parseInt(e(i).css("paddingBottom"),10)||0)-this.helperProportions.height-this.margins.top]}},_convertPositionTo:function(t,i){i||(i=this.position);var n="absolute"==t?1:-1,o=(this.options,"absolute"!=this.cssPosition||this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent),s=/(html|body)/i.test(o[0].tagName);return{top:i.top+this.offset.relative.top*n+this.offset.parent.top*n-("fixed"==this.cssPosition?-this.scrollParent.scrollTop():s?0:o.scrollTop())*n,left:i.left+this.offset.relative.left*n+this.offset.parent.left*n-("fixed"==this.cssPosition?-this.scrollParent.scrollLeft():s?0:o.scrollLeft())*n}},_generatePosition:function(t){var i=this.options,n="absolute"!=this.cssPosition||this.scrollParent[0]!=document&&e.contains(this.scrollParent[0],this.offsetParent[0])?this.scrollParent:this.offsetParent,o=/(html|body)/i.test(n[0].tagName);"relative"!=this.cssPosition||this.scrollParent[0]!=document&&this.scrollParent[0]!=this.offsetParent[0]||(this.offset.relative=this._getRelativeOffset());var s=t.pageX,a=t.pageY;if(this.originalPosition&&(this.containment&&(t.pageX-this.offset.click.left<this.containment[0]&&(s=this.containment[0]+this.offset.click.left),t.pageY-this.offset.click.top<this.containment[1]&&(a=this.containment[1]+this.offset.click.top),t.pageX-this.offset.click.left>this.containment[2]&&(s=this.containment[2]+this.offset.click.left),t.pageY-this.offset.click.top>this.containment[3]&&(a=this.containment[3]+this.offset.click.top)),i.grid)){var r=this.originalPageY+Math.round((a-this.originalPageY)/i.grid[1])*i.grid[1];a=this.containment&&(r-this.offset.click.top<this.containment[1]||r-this.offset.click.top>this.containment[3])?r-this.offset.click.top<this.containment[1]?r+i.grid[1]:r-i.grid[1]:r;var l=this.originalPageX+Math.round((s-this.originalPageX)/i.grid[0])*i.grid[0];s=this.containment&&(l-this.offset.click.left<this.containment[0]||l-this.offset.click.left>this.containment[2])?l-this.offset.click.left<this.containment[0]?l+i.grid[0]:l-i.grid[0]:l}return{top:a-this.offset.click.top-this.offset.relative.top-this.offset.parent.top+("fixed"==this.cssPosition?-this.scrollParent.scrollTop():o?0:n.scrollTop()),left:s-this.offset.click.left-this.offset.relative.left-this.offset.parent.left+("fixed"==this.cssPosition?-this.scrollParent.scrollLeft():o?0:n.scrollLeft())}},_rearrange:function(e,t,i,n){i?i[0].appendChild(this.placeholder[0]):t.item[0].parentNode.insertBefore(this.placeholder[0],"down"==this.direction?t.item[0]:t.item[0].nextSibling),this.counter=this.counter?++this.counter:1;var o=this.counter;this._delay((function(){o==this.counter&&this.refreshPositions(!n)}))},_clear:function(t,i){this.reverting=!1;var n=[];if(!this._noFinalSort&&this.currentItem.parent().length&&this.placeholder.before(this.currentItem),this._noFinalSort=null,this.helper[0]==this.currentItem[0]){for(var o in this._storedCSS)"auto"!=this._storedCSS[o]&&"static"!=this._storedCSS[o]||(this._storedCSS[o]="");this.currentItem.css(this._storedCSS).removeClass("ui-sortable-helper")}else this.currentItem.show();this.fromOutside&&!i&&n.push((function(e){this._trigger("receive",e,this._uiHash(this.fromOutside))})),!this.fromOutside&&this.domPosition.prev==this.currentItem.prev().not(".ui-sortable-helper")[0]&&this.domPosition.parent==this.currentItem.parent()[0]||i||n.push((function(e){this._trigger("update",e,this._uiHash())})),this!==this.currentContainer&&(i||(n.push((function(e){this._trigger("remove",e,this._uiHash())})),n.push(function(e){return function(t){e._trigger("receive",t,this._uiHash(this))}}.call(this,this.currentContainer)),n.push(function(e){return function(t){e._trigger("update",t,this._uiHash(this))}}.call(this,this.currentContainer))));for(o=this.containers.length-1;o>=0;o--)i||n.push(function(e){return function(t){e._trigger("deactivate",t,this._uiHash(this))}}.call(this,this.containers[o])),this.containers[o].containerCache.over&&(n.push(function(e){return function(t){e._trigger("out",t,this._uiHash(this))}}.call(this,this.containers[o])),this.containers[o].containerCache.over=0);if(this._storedCursor&&e("body").css("cursor",this._storedCursor),this._storedOpacity&&this.helper.css("opacity",this._storedOpacity),this._storedZIndex&&this.helper.css("zIndex","auto"==this._storedZIndex?"":this._storedZIndex),this.dragging=!1,this.cancelHelperRemoval){if(!i){this._trigger("beforeStop",t,this._uiHash());for(o=0;o<n.length;o++)n[o].call(this,t);this._trigger("stop",t,this._uiHash())}return this.fromOutside=!1,!1}if(i||this._trigger("beforeStop",t,this._uiHash()),this.placeholder[0].parentNode.removeChild(this.placeholder[0]),this.helper[0]!=this.currentItem[0]&&this.helper.remove(),this.helper=null,!i){for(o=0;o<n.length;o++)n[o].call(this,t);this._trigger("stop",t,this._uiHash())}return this.fromOutside=!1,!0},_trigger:function(){!1===e.Widget.prototype._trigger.apply(this,arguments)&&this.cancel()},_uiHash:function(t){var i=t||this;return{helper:i.helper,placeholder:i.placeholder||e([]),position:i.position,originalPosition:i.originalPosition,offset:i.positionAbs,item:i.currentItem,sender:t?t.element:null}}})}(jQuery),jQuery.effects||function(e,t){var i=!1!==e.uiBackCompat,n="ui-effects-";e.effects={effect:{}},
/*!
 * jQuery Color Animations v2.0.0
 * http://jquery.com/
 *
 * Copyright 2012 jQuery Foundation and other contributors
 * Released under the MIT license.
 * http://jquery.org/license
 *
 * Date: Mon Aug 13 13:41:02 2012 -0500
 */
function(t,i){var n,o="backgroundColor borderBottomColor borderLeftColor borderRightColor borderTopColor color columnRuleColor outlineColor textDecorationColor textEmphasisColor".split(" "),s=/^([\-+])=\s*(\d+\.?\d*)/,a=[{re:/rgba?\(\s*(\d{1,3})\s*,\s*(\d{1,3})\s*,\s*(\d{1,3})\s*(?:,\s*(\d+(?:\.\d+)?)\s*)?\)/,parse:function(e){return[e[1],e[2],e[3],e[4]]}},{re:/rgba?\(\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*(?:,\s*(\d+(?:\.\d+)?)\s*)?\)/,parse:function(e){return[2.55*e[1],2.55*e[2],2.55*e[3],e[4]]}},{re:/#([a-f0-9]{2})([a-f0-9]{2})([a-f0-9]{2})/,parse:function(e){return[parseInt(e[1],16),parseInt(e[2],16),parseInt(e[3],16)]}},{re:/#([a-f0-9])([a-f0-9])([a-f0-9])/,parse:function(e){return[parseInt(e[1]+e[1],16),parseInt(e[2]+e[2],16),parseInt(e[3]+e[3],16)]}},{re:/hsla?\(\s*(\d+(?:\.\d+)?)\s*,\s*(\d+(?:\.\d+)?)\%\s*,\s*(\d+(?:\.\d+)?)\%\s*(?:,\s*(\d+(?:\.\d+)?)\s*)?\)/,space:"hsla",parse:function(e){return[e[1],e[2]/100,e[3]/100,e[4]]}}],r=t.Color=function(e,i,n,o){return new t.Color.fn.parse(e,i,n,o)},l={rgba:{props:{red:{idx:0,type:"byte"},green:{idx:1,type:"byte"},blue:{idx:2,type:"byte"}}},hsla:{props:{hue:{idx:0,type:"degrees"},saturation:{idx:1,type:"percent"},lightness:{idx:2,type:"percent"}}}},c={byte:{floor:!0,max:255},percent:{max:1},degrees:{mod:360,floor:!0}},h=r.support={},u=t("<p>")[0],d=t.each;function p(e,t,i){var n=c[t.type]||{};return null==e?i||!t.def?null:t.def:(e=n.floor?~~e:parseFloat(e),isNaN(e)?t.def:n.mod?(e+n.mod)%n.mod:0>e?0:n.max<e?n.max:e)}function f(e){var i=r(),o=i._rgba=[];return e=e.toLowerCase(),d(a,(function(t,n){var s,a=n.re.exec(e),r=a&&n.parse(a),c=n.space||"rgba";if(r)return s=i[c](r),i[l[c].cache]=s[l[c].cache],o=i._rgba=s._rgba,!1})),o.length?("0,0,0,0"===o.join()&&t.extend(o,n.transparent),i):n[e]}function g(e,t,i){return i=(i+1)%1,6*i<1?e+(t-e)*i*6:2*i<1?t:3*i<2?e+(t-e)*(2/3-i)*6:e}u.style.cssText="background-color:rgba(1,1,1,.5)",h.rgba=u.style.backgroundColor.indexOf("rgba")>-1,d(l,(function(e,t){t.cache="_"+e,t.props.alpha={idx:3,type:"percent",def:1}})),r.fn=t.extend(r.prototype,{parse:function(o,s,a,c){if(o===i)return this._rgba=[null,null,null,null],this;(o.jquery||o.nodeType)&&(o=t(o).css(s),s=i);var h=this,u=t.type(o),g=this._rgba=[];return s!==i&&(o=[o,s,a,c],u="array"),"string"===u?this.parse(f(o)||n._default):"array"===u?(d(l.rgba.props,(function(e,t){g[t.idx]=p(o[t.idx],t)})),this):"object"===u?(d(l,o instanceof r?function(e,t){o[t.cache]&&(h[t.cache]=o[t.cache].slice())}:function(t,i){var n=i.cache;d(i.props,(function(e,t){if(!h[n]&&i.to){if("alpha"===e||null==o[e])return;h[n]=i.to(h._rgba)}h[n][t.idx]=p(o[e],t,!0)})),h[n]&&e.inArray(null,h[n].slice(0,3))<0&&(h[n][3]=1,i.from&&(h._rgba=i.from(h[n])))}),this):void 0},is:function(e){var t=r(e),i=!0,n=this;return d(l,(function(e,o){var s,a=t[o.cache];return a&&(s=n[o.cache]||o.to&&o.to(n._rgba)||[],d(o.props,(function(e,t){if(null!=a[t.idx])return i=a[t.idx]===s[t.idx],i}))),i})),i},_space:function(){var e=[],t=this;return d(l,(function(i,n){t[n.cache]&&e.push(i)})),e.pop()},transition:function(e,t){var i=r(e),n=i._space(),o=l[n],s=0===this.alpha()?r("transparent"):this,a=s[o.cache]||o.to(s._rgba),h=a.slice();return i=i[o.cache],d(o.props,(function(e,n){var o=n.idx,s=a[o],r=i[o],l=c[n.type]||{};null!==r&&(null===s?h[o]=r:(l.mod&&(r-s>l.mod/2?s+=l.mod:s-r>l.mod/2&&(s-=l.mod)),h[o]=p((r-s)*t+s,n)))})),this[n](h)},blend:function(e){if(1===this._rgba[3])return this;var i=this._rgba.slice(),n=i.pop(),o=r(e)._rgba;return r(t.map(i,(function(e,t){return(1-n)*o[t]+n*e})))},toRgbaString:function(){var e="rgba(",i=t.map(this._rgba,(function(e,t){return null==e?t>2?1:0:e}));return 1===i[3]&&(i.pop(),e="rgb("),e+i.join()+")"},toHslaString:function(){var e="hsla(",i=t.map(this.hsla(),(function(e,t){return null==e&&(e=t>2?1:0),t&&t<3&&(e=Math.round(100*e)+"%"),e}));return 1===i[3]&&(i.pop(),e="hsl("),e+i.join()+")"},toHexString:function(e){var i=this._rgba.slice(),n=i.pop();return e&&i.push(~~(255*n)),"#"+t.map(i,(function(e){return e=(e||0).toString(16),1===e.length?"0"+e:e})).join("")},toString:function(){return 0===this._rgba[3]?"transparent":this.toRgbaString()}}),r.fn.parse.prototype=r.fn,l.hsla.to=function(e){if(null==e[0]||null==e[1]||null==e[2])return[null,null,null,e[3]];var t,i,n=e[0]/255,o=e[1]/255,s=e[2]/255,a=e[3],r=Math.max(n,o,s),l=Math.min(n,o,s),c=r-l,h=r+l,u=.5*h;return t=l===r?0:n===r?60*(o-s)/c+360:o===r?60*(s-n)/c+120:60*(n-o)/c+240,i=0===u||1===u?u:u<=.5?c/h:c/(2-h),[Math.round(t)%360,i,u,null==a?1:a]},l.hsla.from=function(e){if(null==e[0]||null==e[1]||null==e[2])return[null,null,null,e[3]];var t=e[0]/360,i=e[1],n=e[2],o=e[3],s=n<=.5?n*(1+i):n+i-n*i,a=2*n-s;return[Math.round(255*g(a,s,t+1/3)),Math.round(255*g(a,s,t)),Math.round(255*g(a,s,t-1/3)),o]},d(l,(function(e,n){var o=n.props,a=n.cache,l=n.to,c=n.from;r.fn[e]=function(e){if(l&&!this[a]&&(this[a]=l(this._rgba)),e===i)return this[a].slice();var n,s=t.type(e),h="array"===s||"object"===s?e:arguments,u=this[a].slice();return d(o,(function(e,t){var i=h["object"===s?e:t.idx];null==i&&(i=u[t.idx]),u[t.idx]=p(i,t)})),c?(n=r(c(u)),n[a]=u,n):r(u)},d(o,(function(i,n){r.fn[i]||(r.fn[i]=function(o){var a,r=t.type(o),l="alpha"===i?this._hsla?"hsla":"rgba":e,c=this[l](),h=c[n.idx];return"undefined"===r?h:("function"===r&&(o=o.call(this,h),r=t.type(o)),null==o&&n.empty?this:("string"===r&&(a=s.exec(o),a&&(o=h+parseFloat(a[2])*("+"===a[1]?1:-1))),c[n.idx]=o,this[l](c)))})}))})),d(o,(function(e,i){t.cssHooks[i]={set:function(e,n){var o,s,a="";if("string"!==t.type(n)||(o=f(n))){if(n=r(o||n),!h.rgba&&1!==n._rgba[3]){s="backgroundColor"===i?e.parentNode:e;while((""===a||"transparent"===a)&&s&&s.style)try{a=t.css(s,"backgroundColor"),s=s.parentNode}catch(l){}n=n.blend(a&&"transparent"!==a?a:"_default")}n=n.toRgbaString()}try{e.style[i]=n}catch(c){}}},t.fx.step[i]=function(e){e.colorInit||(e.start=r(e.elem,i),e.end=r(e.end),e.colorInit=!0),t.cssHooks[i].set(e.elem,e.start.transition(e.end,e.pos))}})),t.cssHooks.borderColor={expand:function(e){var t={};return d(["Top","Right","Bottom","Left"],(function(i,n){t["border"+n+"Color"]=e})),t}},n=t.Color.names={aqua:"#00ffff",black:"#000000",blue:"#0000ff",fuchsia:"#ff00ff",gray:"#808080",green:"#008000",lime:"#00ff00",maroon:"#800000",navy:"#000080",olive:"#808000",purple:"#800080",red:"#ff0000",silver:"#c0c0c0",teal:"#008080",white:"#ffffff",yellow:"#ffff00",transparent:[null,null,null,0],_default:"#ffffff"}}(jQuery),function(){var i=["add","remove","toggle"],n={border:1,borderBottom:1,borderColor:1,borderLeft:1,borderRight:1,borderTop:1,borderWidth:1,margin:1,padding:1};function o(){var t,i,n=this.ownerDocument.defaultView?this.ownerDocument.defaultView.getComputedStyle(this,null):this.currentStyle,o={};if(n&&n.length&&n[0]&&n[n[0]]){i=n.length;while(i--)t=n[i],"string"===typeof n[t]&&(o[e.camelCase(t)]=n[t])}else for(t in n)"string"===typeof n[t]&&(o[t]=n[t]);return o}function s(t,i){var o,s,a={};for(o in i)s=i[o],t[o]!==s&&(n[o]||!e.fx.step[o]&&isNaN(parseFloat(s))||(a[o]=s));return a}e.each(["borderLeftStyle","borderRightStyle","borderBottomStyle","borderTopStyle"],(function(t,i){e.fx.step[i]=function(e){("none"!==e.end&&!e.setAttr||1===e.pos&&!e.setAttr)&&(jQuery.style(e.elem,i,e.end),e.setAttr=!0)}})),e.effects.animateClass=function(t,n,a,r){var l=e.speed(n,a,r);return this.queue((function(){var n,a=e(this),r=a.attr("class")||"",c=l.children?a.find("*").andSelf():a;c=c.map((function(){var t=e(this);return{el:t,start:o.call(this)}})),n=function(){e.each(i,(function(e,i){t[i]&&a[i+"Class"](t[i])}))},n(),c=c.map((function(){return this.end=o.call(this.el[0]),this.diff=s(this.start,this.end),this})),a.attr("class",r),c=c.map((function(){var t=this,i=e.Deferred(),n=jQuery.extend({},l,{queue:!1,complete:function(){i.resolve(t)}});return this.el.animate(this.diff,n),i.promise()})),e.when.apply(e,c.get()).done((function(){n(),e.each(arguments,(function(){var t=this.el;e.each(this.diff,(function(e){t.css(e,"")}))})),l.complete.call(a[0])}))}))},e.fn.extend({_addClass:e.fn.addClass,addClass:function(t,i,n,o){return i?e.effects.animateClass.call(this,{add:t},i,n,o):this._addClass(t)},_removeClass:e.fn.removeClass,removeClass:function(t,i,n,o){return i?e.effects.animateClass.call(this,{remove:t},i,n,o):this._removeClass(t)},_toggleClass:e.fn.toggleClass,toggleClass:function(i,n,o,s,a){return"boolean"===typeof n||n===t?o?e.effects.animateClass.call(this,n?{add:i}:{remove:i},o,s,a):this._toggleClass(i,n):e.effects.animateClass.call(this,{toggle:i},n,o,s)},switchClass:function(t,i,n,o,s){return e.effects.animateClass.call(this,{add:i,remove:t},n,o,s)}})}(),function(){function o(t,i,n,o){return e.isPlainObject(t)&&(i=t,t=t.effect),t={effect:t},null==i&&(i={}),e.isFunction(i)&&(o=i,n=null,i={}),("number"===typeof i||e.fx.speeds[i])&&(o=n,n=i,i={}),e.isFunction(n)&&(o=n,n=null),i&&e.extend(t,i),n=n||i.duration,t.duration=e.fx.off?0:"number"===typeof n?n:n in e.fx.speeds?e.fx.speeds[n]:e.fx.speeds._default,t.complete=o||i.complete,t}function s(t){return!(t&&"number"!==typeof t&&!e.fx.speeds[t])||"string"===typeof t&&!e.effects.effect[t]&&(!i||!e.effects[t])}e.extend(e.effects,{version:"1.9.2",save:function(e,t){for(var i=0;i<t.length;i++)null!==t[i]&&e.data(n+t[i],e[0].style[t[i]])},restore:function(e,i){var o,s;for(s=0;s<i.length;s++)null!==i[s]&&(o=e.data(n+i[s]),o===t&&(o=""),e.css(i[s],o))},setMode:function(e,t){return"toggle"===t&&(t=e.is(":hidden")?"show":"hide"),t},getBaseline:function(e,t){var i,n;switch(e[0]){case"top":i=0;break;case"middle":i=.5;break;case"bottom":i=1;break;default:i=e[0]/t.height}switch(e[1]){case"left":n=0;break;case"center":n=.5;break;case"right":n=1;break;default:n=e[1]/t.width}return{x:n,y:i}},createWrapper:function(t){if(t.parent().is(".ui-effects-wrapper"))return t.parent();var i={width:t.outerWidth(!0),height:t.outerHeight(!0),float:t.css("float")},n=e("<div></div>").addClass("ui-effects-wrapper").css({fontSize:"100%",background:"transparent",border:"none",margin:0,padding:0}),o={width:t.width(),height:t.height()},s=document.activeElement;try{s.id}catch(a){s=document.body}return t.wrap(n),(t[0]===s||e.contains(t[0],s))&&e(s).focus(),n=t.parent(),"static"===t.css("position")?(n.css({position:"relative"}),t.css({position:"relative"})):(e.extend(i,{position:t.css("position"),zIndex:t.css("z-index")}),e.each(["top","left","bottom","right"],(function(e,n){i[n]=t.css(n),isNaN(parseInt(i[n],10))&&(i[n]="auto")})),t.css({position:"relative",top:0,left:0,right:"auto",bottom:"auto"})),t.css(o),n.css(i).show()},removeWrapper:function(t){var i=document.activeElement;return t.parent().is(".ui-effects-wrapper")&&(t.parent().replaceWith(t),(t[0]===i||e.contains(t[0],i))&&e(i).focus()),t},setTransition:function(t,i,n,o){return o=o||{},e.each(i,(function(e,i){var s=t.cssUnit(i);s[0]>0&&(o[i]=s[0]*n+s[1])})),o}}),e.fn.extend({effect:function(){var t=o.apply(this,arguments),n=t.mode,s=t.queue,a=e.effects.effect[t.effect],r=!a&&i&&e.effects[t.effect];if(e.fx.off||!a&&!r)return n?this[n](t.duration,t.complete):this.each((function(){t.complete&&t.complete.call(this)}));function l(i){var n=e(this),o=t.complete,s=t.mode;function r(){e.isFunction(o)&&o.call(n[0]),e.isFunction(i)&&i()}(n.is(":hidden")?"hide"===s:"show"===s)?r():a.call(n[0],t,r)}return a?!1===s?this.each(l):this.queue(s||"fx",l):r.call(this,{options:t,duration:t.duration,callback:t.complete,mode:t.mode})},_show:e.fn.show,show:function(e){if(s(e))return this._show.apply(this,arguments);var t=o.apply(this,arguments);return t.mode="show",this.effect.call(this,t)},_hide:e.fn.hide,hide:function(e){if(s(e))return this._hide.apply(this,arguments);var t=o.apply(this,arguments);return t.mode="hide",this.effect.call(this,t)},__toggle:e.fn.toggle,toggle:function(t){if(s(t)||"boolean"===typeof t||e.isFunction(t))return this.__toggle.apply(this,arguments);var i=o.apply(this,arguments);return i.mode="toggle",this.effect.call(this,i)},cssUnit:function(t){var i=this.css(t),n=[];return e.each(["em","px","%","pt"],(function(e,t){i.indexOf(t)>0&&(n=[parseFloat(i),t])})),n}})}(),function(){var t={};e.each(["Quad","Cubic","Quart","Quint","Expo"],(function(e,i){t[i]=function(t){return Math.pow(t,e+2)}})),e.extend(t,{Sine:function(e){return 1-Math.cos(e*Math.PI/2)},Circ:function(e){return 1-Math.sqrt(1-e*e)},Elastic:function(e){return 0===e||1===e?e:-Math.pow(2,8*(e-1))*Math.sin((80*(e-1)-7.5)*Math.PI/15)},Back:function(e){return e*e*(3*e-2)},Bounce:function(e){var t,i=4;while(e<((t=Math.pow(2,--i))-1)/11);return 1/Math.pow(4,3-i)-7.5625*Math.pow((3*t-2)/22-e,2)}}),e.each(t,(function(t,i){e.easing["easeIn"+t]=i,e.easing["easeOut"+t]=function(e){return 1-i(1-e)},e.easing["easeInOut"+t]=function(e){return e<.5?i(2*e)/2:1-i(-2*e+2)/2}}))}()}(jQuery),function(e,t){var i=0,n={},o={};n.height=n.paddingTop=n.paddingBottom=n.borderTopWidth=n.borderBottomWidth="hide",o.height=o.paddingTop=o.paddingBottom=o.borderTopWidth=o.borderBottomWidth="show",e.widget("ui.accordion",{version:"1.9.2",options:{active:0,animate:{},collapsible:!1,event:"click",header:"> li > :first-child,> :not(li):even",heightStyle:"auto",icons:{activeHeader:"ui-icon-triangle-1-s",header:"ui-icon-triangle-1-e"},activate:null,beforeActivate:null},_create:function(){var t=this.accordionId="ui-accordion-"+(this.element.attr("id")||++i),n=this.options;this.prevShow=this.prevHide=e(),this.element.addClass("ui-accordion ui-widget ui-helper-reset"),this.headers=this.element.find(n.header).addClass("ui-accordion-header ui-helper-reset ui-state-default ui-corner-all"),this._hoverable(this.headers),this._focusable(this.headers),this.headers.next().addClass("ui-accordion-content ui-helper-reset ui-widget-content ui-corner-bottom").hide(),n.collapsible||!1!==n.active&&null!=n.active||(n.active=0),n.active<0&&(n.active+=this.headers.length),this.active=this._findActive(n.active).addClass("ui-accordion-header-active ui-state-active").toggleClass("ui-corner-all ui-corner-top"),this.active.next().addClass("ui-accordion-content-active").show(),this._createIcons(),this.refresh(),this.element.attr("role","tablist"),this.headers.attr("role","tab").each((function(i){var n=e(this),o=n.attr("id"),s=n.next(),a=s.attr("id");o||(o=t+"-header-"+i,n.attr("id",o)),a||(a=t+"-panel-"+i,s.attr("id",a)),n.attr("aria-controls",a),s.attr("aria-labelledby",o)})).next().attr("role","tabpanel"),this.headers.not(this.active).attr({"aria-selected":"false",tabIndex:-1}).next().attr({"aria-expanded":"false","aria-hidden":"true"}).hide(),this.active.length?this.active.attr({"aria-selected":"true",tabIndex:0}).next().attr({"aria-expanded":"true","aria-hidden":"false"}):this.headers.eq(0).attr("tabIndex",0),this._on(this.headers,{keydown:"_keydown"}),this._on(this.headers.next(),{keydown:"_panelKeyDown"}),this._setupEvents(n.event)},_getCreateEventData:function(){return{header:this.active,content:this.active.length?this.active.next():e()}},_createIcons:function(){var t=this.options.icons;t&&(e("<span>").addClass("ui-accordion-header-icon ui-icon "+t.header).prependTo(this.headers),this.active.children(".ui-accordion-header-icon").removeClass(t.header).addClass(t.activeHeader),this.headers.addClass("ui-accordion-icons"))},_destroyIcons:function(){this.headers.removeClass("ui-accordion-icons").children(".ui-accordion-header-icon").remove()},_destroy:function(){var e;this.element.removeClass("ui-accordion ui-widget ui-helper-reset").removeAttr("role"),this.headers.removeClass("ui-accordion-header ui-accordion-header-active ui-helper-reset ui-state-default ui-corner-all ui-state-active ui-state-disabled ui-corner-top").removeAttr("role").removeAttr("aria-selected").removeAttr("aria-controls").removeAttr("tabIndex").each((function(){/^ui-accordion/.test(this.id)&&this.removeAttribute("id")})),this._destroyIcons(),e=this.headers.next().css("display","").removeAttr("role").removeAttr("aria-expanded").removeAttr("aria-hidden").removeAttr("aria-labelledby").removeClass("ui-helper-reset ui-widget-content ui-corner-bottom ui-accordion-content ui-accordion-content-active ui-state-disabled").each((function(){/^ui-accordion/.test(this.id)&&this.removeAttribute("id")})),"content"!==this.options.heightStyle&&e.css("height","")},_setOption:function(e,t){"active"!==e?("event"===e&&(this.options.event&&this._off(this.headers,this.options.event),this._setupEvents(t)),this._super(e,t),"collapsible"!==e||t||!1!==this.options.active||this._activate(0),"icons"===e&&(this._destroyIcons(),t&&this._createIcons()),"disabled"===e&&this.headers.add(this.headers.next()).toggleClass("ui-state-disabled",!!t)):this._activate(t)},_keydown:function(t){if(!t.altKey&&!t.ctrlKey){var i=e.ui.keyCode,n=this.headers.length,o=this.headers.index(t.target),s=!1;switch(t.keyCode){case i.RIGHT:case i.DOWN:s=this.headers[(o+1)%n];break;case i.LEFT:case i.UP:s=this.headers[(o-1+n)%n];break;case i.SPACE:case i.ENTER:this._eventHandler(t);break;case i.HOME:s=this.headers[0];break;case i.END:s=this.headers[n-1];break}s&&(e(t.target).attr("tabIndex",-1),e(s).attr("tabIndex",0),s.focus(),t.preventDefault())}},_panelKeyDown:function(t){t.keyCode===e.ui.keyCode.UP&&t.ctrlKey&&e(t.currentTarget).prev().focus()},refresh:function(){var t,i,n=this.options.heightStyle,o=this.element.parent();"fill"===n?(e.support.minHeight||(i=o.css("overflow"),o.css("overflow","hidden")),t=o.height(),this.element.siblings(":visible").each((function(){var i=e(this),n=i.css("position");"absolute"!==n&&"fixed"!==n&&(t-=i.outerHeight(!0))})),i&&o.css("overflow",i),this.headers.each((function(){t-=e(this).outerHeight(!0)})),this.headers.next().each((function(){e(this).height(Math.max(0,t-e(this).innerHeight()+e(this).height()))})).css("overflow","auto")):"auto"===n&&(t=0,this.headers.next().each((function(){t=Math.max(t,e(this).css("height","").height())})).height(t))},_activate:function(t){var i=this._findActive(t)[0];i!==this.active[0]&&(i=i||this.active[0],this._eventHandler({target:i,currentTarget:i,preventDefault:e.noop}))},_findActive:function(t){return"number"===typeof t?this.headers.eq(t):e()},_setupEvents:function(t){var i={};t&&(e.each(t.split(" "),(function(e,t){i[t]="_eventHandler"})),this._on(this.headers,i))},_eventHandler:function(t){var i=this.options,n=this.active,o=e(t.currentTarget),s=o[0]===n[0],a=s&&i.collapsible,r=a?e():o.next(),l=n.next(),c={oldHeader:n,oldPanel:l,newHeader:a?e():o,newPanel:r};t.preventDefault(),s&&!i.collapsible||!1===this._trigger("beforeActivate",t,c)||(i.active=!a&&this.headers.index(o),this.active=s?e():o,this._toggle(c),n.removeClass("ui-accordion-header-active ui-state-active"),i.icons&&n.children(".ui-accordion-header-icon").removeClass(i.icons.activeHeader).addClass(i.icons.header),s||(o.removeClass("ui-corner-all").addClass("ui-accordion-header-active ui-state-active ui-corner-top"),i.icons&&o.children(".ui-accordion-header-icon").removeClass(i.icons.header).addClass(i.icons.activeHeader),o.next().addClass("ui-accordion-content-active")))},_toggle:function(t){var i=t.newPanel,n=this.prevShow.length?this.prevShow:t.oldPanel;this.prevShow.add(this.prevHide).stop(!0,!0),this.prevShow=i,this.prevHide=n,this.options.animate?this._animate(i,n,t):(n.hide(),i.show(),this._toggleComplete(t)),n.attr({"aria-expanded":"false","aria-hidden":"true"}),n.prev().attr("aria-selected","false"),i.length&&n.length?n.prev().attr("tabIndex",-1):i.length&&this.headers.filter((function(){return 0===e(this).attr("tabIndex")})).attr("tabIndex",-1),i.attr({"aria-expanded":"true","aria-hidden":"false"}).prev().attr({"aria-selected":"true",tabIndex:0})},_animate:function(e,t,i){var s,a,r,l=this,c=0,h=e.length&&(!t.length||e.index()<t.index()),u=this.options.animate||{},d=h&&u.down||u,p=function(){l._toggleComplete(i)};return"number"===typeof d&&(r=d),"string"===typeof d&&(a=d),a=a||d.easing||u.easing,r=r||d.duration||u.duration,t.length?e.length?(s=e.show().outerHeight(),t.animate(n,{duration:r,easing:a,step:function(e,t){t.now=Math.round(e)}}),void e.hide().animate(o,{duration:r,easing:a,complete:p,step:function(e,i){i.now=Math.round(e),"height"!==i.prop?c+=i.now:"content"!==l.options.heightStyle&&(i.now=Math.round(s-t.outerHeight()-c),c=0)}})):t.animate(n,r,a,p):e.animate(o,r,a,p)},_toggleComplete:function(e){var t=e.oldPanel;t.removeClass("ui-accordion-content-active").prev().removeClass("ui-corner-top").addClass("ui-corner-all"),t.length&&(t.parent()[0].className=t.parent()[0].className),this._trigger("activate",null,e)}}),!1!==e.uiBackCompat&&(function(e,t){e.extend(t.options,{navigation:!1,navigationFilter:function(){return this.href.toLowerCase()===location.href.toLowerCase()}});var i=t._create;t._create=function(){if(this.options.navigation){var t=this,n=this.element.find(this.options.header),o=n.next(),s=n.add(o).find("a").filter(this.options.navigationFilter)[0];s&&n.add(o).each((function(i){if(e.contains(this,s))return t.options.active=Math.floor(i/2),!1}))}i.call(this)}}(jQuery,jQuery.ui.accordion.prototype),function(e,t){e.extend(t.options,{heightStyle:null,autoHeight:!0,clearStyle:!1,fillSpace:!1});var i=t._create,n=t._setOption;e.extend(t,{_create:function(){this.options.heightStyle=this.options.heightStyle||this._mergeHeightStyle(),i.call(this)},_setOption:function(e){"autoHeight"!==e&&"clearStyle"!==e&&"fillSpace"!==e||(this.options.heightStyle=this._mergeHeightStyle()),n.apply(this,arguments)},_mergeHeightStyle:function(){var e=this.options;return e.fillSpace?"fill":e.clearStyle?"content":e.autoHeight?"auto":void 0}})}(jQuery,jQuery.ui.accordion.prototype),function(e,t){e.extend(t.options.icons,{activeHeader:null,headerSelected:"ui-icon-triangle-1-s"});var i=t._createIcons;t._createIcons=function(){this.options.icons&&(this.options.icons.activeHeader=this.options.icons.activeHeader||this.options.icons.headerSelected),i.call(this)}}(jQuery,jQuery.ui.accordion.prototype),function(e,t){t.activate=t._activate;var i=t._findActive;t._findActive=function(e){return-1===e&&(e=!1),e&&"number"!==typeof e&&(e=this.headers.index(this.headers.filter(e)),-1===e&&(e=!1)),i.call(this,e)}}(jQuery,jQuery.ui.accordion.prototype),jQuery.ui.accordion.prototype.resize=jQuery.ui.accordion.prototype.refresh,function(e,t){e.extend(t.options,{change:null,changestart:null});var i=t._trigger;t._trigger=function(e,t,n){var o=i.apply(this,arguments);return!!o&&("beforeActivate"===e?o=i.call(this,"changestart",t,{oldHeader:n.oldHeader,oldContent:n.oldPanel,newHeader:n.newHeader,newContent:n.newPanel}):"activate"===e&&(o=i.call(this,"change",t,{oldHeader:n.oldHeader,oldContent:n.oldPanel,newHeader:n.newHeader,newContent:n.newPanel})),o)}}(jQuery,jQuery.ui.accordion.prototype),function(e,t){e.extend(t.options,{animate:null,animated:"slide"});var i=t._create;t._create=function(){var e=this.options;null===e.animate&&(e.animated?"slide"===e.animated?e.animate=300:"bounceslide"===e.animated?e.animate={duration:200,down:{easing:"easeOutBounce",duration:1e3}}:e.animate=e.animated:e.animate=!1),i.call(this)}}(jQuery,jQuery.ui.accordion.prototype))}(jQuery),function(e,t){var i=0;e.widget("ui.autocomplete",{version:"1.9.2",defaultElement:"<input>",options:{appendTo:"body",autoFocus:!1,delay:300,minLength:1,position:{my:"left top",at:"left bottom",collision:"none"},source:null,change:null,close:null,focus:null,open:null,response:null,search:null,select:null},pending:0,_create:function(){var t,i,n;this.isMultiLine=this._isMultiLine(),this.valueMethod=this.element[this.element.is("input,textarea")?"val":"text"],this.isNewMenu=!0,this.element.addClass("ui-autocomplete-input").attr("autocomplete","off"),this._on(this.element,{keydown:function(o){if(this.element.prop("readOnly"))return t=!0,n=!0,void(i=!0);t=!1,n=!1,i=!1;var s=e.ui.keyCode;switch(o.keyCode){case s.PAGE_UP:t=!0,this._move("previousPage",o);break;case s.PAGE_DOWN:t=!0,this._move("nextPage",o);break;case s.UP:t=!0,this._keyEvent("previous",o);break;case s.DOWN:t=!0,this._keyEvent("next",o);break;case s.ENTER:case s.NUMPAD_ENTER:this.menu.active&&(t=!0,o.preventDefault(),this.menu.select(o));break;case s.TAB:this.menu.active&&this.menu.select(o);break;case s.ESCAPE:this.menu.element.is(":visible")&&(this._value(this.term),this.close(o),o.preventDefault());break;default:i=!0,this._searchTimeout(o);break}},keypress:function(n){if(t)return t=!1,void n.preventDefault();if(!i){var o=e.ui.keyCode;switch(n.keyCode){case o.PAGE_UP:this._move("previousPage",n);break;case o.PAGE_DOWN:this._move("nextPage",n);break;case o.UP:this._keyEvent("previous",n);break;case o.DOWN:this._keyEvent("next",n);break}}},input:function(e){if(n)return n=!1,void e.preventDefault();this._searchTimeout(e)},focus:function(){this.selectedItem=null,this.previous=this._value()},blur:function(e){this.cancelBlur?delete this.cancelBlur:(clearTimeout(this.searching),this.close(e),this._change(e))}}),this._initSource(),this.menu=e("<ul>").addClass("ui-autocomplete").appendTo(this.document.find(this.options.appendTo||"body")[0]).menu({input:e(),role:null}).zIndex(this.element.zIndex()+1).hide().data("menu"),this._on(this.menu.element,{mousedown:function(t){t.preventDefault(),this.cancelBlur=!0,this._delay((function(){delete this.cancelBlur}));var i=this.menu.element[0];e(t.target).closest(".ui-menu-item").length||this._delay((function(){var t=this;this.document.one("mousedown",(function(n){n.target===t.element[0]||n.target===i||e.contains(i,n.target)||t.close()}))}))},menufocus:function(t,i){if(this.isNewMenu&&(this.isNewMenu=!1,t.originalEvent&&/^mouse/.test(t.originalEvent.type)))return this.menu.blur(),void this.document.one("mousemove",(function(){e(t.target).trigger(t.originalEvent)}));var n=i.item.data("ui-autocomplete-item")||i.item.data("item.autocomplete");!1!==this._trigger("focus",t,{item:n})?t.originalEvent&&/^key/.test(t.originalEvent.type)&&this._value(n.value):this.liveRegion.text(n.value)},menuselect:function(e,t){var i=t.item.data("ui-autocomplete-item")||t.item.data("item.autocomplete"),n=this.previous;this.element[0]!==this.document[0].activeElement&&(this.element.focus(),this.previous=n,this._delay((function(){this.previous=n,this.selectedItem=i}))),!1!==this._trigger("select",e,{item:i})&&this._value(i.value),this.term=this._value(),this.close(e),this.selectedItem=i}}),this.liveRegion=e("<span>",{role:"status","aria-live":"polite"}).addClass("ui-helper-hidden-accessible").insertAfter(this.element),e.fn.bgiframe&&this.menu.element.bgiframe(),this._on(this.window,{beforeunload:function(){this.element.removeAttr("autocomplete")}})},_destroy:function(){clearTimeout(this.searching),this.element.removeClass("ui-autocomplete-input").removeAttr("autocomplete"),this.menu.element.remove(),this.liveRegion.remove()},_setOption:function(e,t){this._super(e,t),"source"===e&&this._initSource(),"appendTo"===e&&this.menu.element.appendTo(this.document.find(t||"body")[0]),"disabled"===e&&t&&this.xhr&&this.xhr.abort()},_isMultiLine:function(){return!!this.element.is("textarea")||!this.element.is("input")&&this.element.prop("isContentEditable")},_initSource:function(){var t,i,n=this;e.isArray(this.options.source)?(t=this.options.source,this.source=function(i,n){n(e.ui.autocomplete.filter(t,i.term))}):"string"===typeof this.options.source?(i=this.options.source,this.source=function(t,o){n.xhr&&n.xhr.abort(),n.xhr=e.ajax({url:i,data:t,dataType:"json",success:function(e){o(e)},error:function(){o([])}})}):this.source=this.options.source},_searchTimeout:function(e){clearTimeout(this.searching),this.searching=this._delay((function(){this.term!==this._value()&&(this.selectedItem=null,this.search(null,e))}),this.options.delay)},search:function(e,t){return e=null!=e?e:this._value(),this.term=this._value(),e.length<this.options.minLength?this.close(t):!1!==this._trigger("search",t)?this._search(e):void 0},_search:function(e){this.pending++,this.element.addClass("ui-autocomplete-loading"),this.cancelSearch=!1,this.source({term:e},this._response())},_response:function(){var e=this,t=++i;return function(n){t===i&&e.__response(n),e.pending--,e.pending||e.element.removeClass("ui-autocomplete-loading")}},__response:function(e){e&&(e=this._normalize(e)),this._trigger("response",null,{content:e}),!this.options.disabled&&e&&e.length&&!this.cancelSearch?(this._suggest(e),this._trigger("open")):this._close()},close:function(e){this.cancelSearch=!0,this._close(e)},_close:function(e){this.menu.element.is(":visible")&&(this.menu.element.hide(),this.menu.blur(),this.isNewMenu=!0,this._trigger("close",e))},_change:function(e){this.previous!==this._value()&&this._trigger("change",e,{item:this.selectedItem})},_normalize:function(t){return t.length&&t[0].label&&t[0].value?t:e.map(t,(function(t){return"string"===typeof t?{label:t,value:t}:e.extend({label:t.label||t.value,value:t.value||t.label},t)}))},_suggest:function(t){var i=this.menu.element.empty().zIndex(this.element.zIndex()+1);this._renderMenu(i,t),this.menu.refresh(),i.show(),this._resizeMenu(),i.position(e.extend({of:this.element},this.options.position)),this.options.autoFocus&&this.menu.next()},_resizeMenu:function(){var e=this.menu.element;e.outerWidth(Math.max(e.width("").outerWidth()+1,this.element.outerWidth()))},_renderMenu:function(t,i){var n=this;e.each(i,(function(e,i){n._renderItemData(t,i)}))},_renderItemData:function(e,t){return this._renderItem(e,t).data("ui-autocomplete-item",t)},_renderItem:function(t,i){return e("<li>").append(e("<a>").text(i.label)).appendTo(t)},_move:function(e,t){if(this.menu.element.is(":visible"))return this.menu.isFirstItem()&&/^previous/.test(e)||this.menu.isLastItem()&&/^next/.test(e)?(this._value(this.term),void this.menu.blur()):void this.menu[e](t);this.search(null,t)},widget:function(){return this.menu.element},_value:function(){return this.valueMethod.apply(this.element,arguments)},_keyEvent:function(e,t){this.isMultiLine&&!this.menu.element.is(":visible")||(this._move(e,t),t.preventDefault())}}),e.extend(e.ui.autocomplete,{escapeRegex:function(e){return e.replace(/[\-\[\]{}()*+?.,\\\^$|#\s]/g,"\\$&")},filter:function(t,i){var n=new RegExp(e.ui.autocomplete.escapeRegex(i),"i");return e.grep(t,(function(e){return n.test(e.label||e.value||e)}))}}),e.widget("ui.autocomplete",e.ui.autocomplete,{options:{messages:{noResults:"No search results.",results:function(e){return e+(e>1?" results are":" result is")+" available, use up and down arrow keys to navigate."}}},__response:function(e){var t;this._superApply(arguments),this.options.disabled||this.cancelSearch||(t=e&&e.length?this.options.messages.results(e.length):this.options.messages.noResults,this.liveRegion.text(t))}})}(jQuery),function(e,t){var i,n,o,s,a="ui-button ui-widget ui-state-default ui-corner-all",r="ui-state-hover ui-state-active ",l="ui-button-icons-only ui-button-icon-only ui-button-text-icons ui-button-text-icon-primary ui-button-text-icon-secondary ui-button-text-only",c=function(){var t=e(this).find(":ui-button");setTimeout((function(){t.button("refresh")}),1)},h=function(t){var i=t.name,n=t.form,o=e([]);return i&&(o=n?e(n).find("[name='"+i+"']"):e("[name='"+i+"']",t.ownerDocument).filter((function(){return!this.form}))),o};e.widget("ui.button",{version:"1.9.2",defaultElement:"<button>",options:{disabled:null,text:!0,label:null,icons:{primary:null,secondary:null}},_create:function(){this.element.closest("form").unbind("reset"+this.eventNamespace).bind("reset"+this.eventNamespace,c),"boolean"!==typeof this.options.disabled?this.options.disabled=!!this.element.prop("disabled"):this.element.prop("disabled",this.options.disabled),this._determineButtonType(),this.hasTitle=!!this.buttonElement.attr("title");var t=this,r=this.options,l="checkbox"===this.type||"radio"===this.type,u=l?"":"ui-state-active",d="ui-state-focus";null===r.label&&(r.label="input"===this.type?this.buttonElement.val():this.buttonElement.html()),this._hoverable(this.buttonElement),this.buttonElement.addClass(a).attr("role","button").bind("mouseenter"+this.eventNamespace,(function(){r.disabled||this===i&&e(this).addClass("ui-state-active")})).bind("mouseleave"+this.eventNamespace,(function(){r.disabled||e(this).removeClass(u)})).bind("click"+this.eventNamespace,(function(e){r.disabled&&(e.preventDefault(),e.stopImmediatePropagation())})),this.element.bind("focus"+this.eventNamespace,(function(){t.buttonElement.addClass(d)})).bind("blur"+this.eventNamespace,(function(){t.buttonElement.removeClass(d)})),l&&(this.element.bind("change"+this.eventNamespace,(function(){s||t.refresh()})),this.buttonElement.bind("mousedown"+this.eventNamespace,(function(e){r.disabled||(s=!1,n=e.pageX,o=e.pageY)})).bind("mouseup"+this.eventNamespace,(function(e){r.disabled||n===e.pageX&&o===e.pageY||(s=!0)}))),"checkbox"===this.type?this.buttonElement.bind("click"+this.eventNamespace,(function(){if(r.disabled||s)return!1;e(this).toggleClass("ui-state-active"),t.buttonElement.attr("aria-pressed",t.element[0].checked)})):"radio"===this.type?this.buttonElement.bind("click"+this.eventNamespace,(function(){if(r.disabled||s)return!1;e(this).addClass("ui-state-active"),t.buttonElement.attr("aria-pressed","true");var i=t.element[0];h(i).not(i).map((function(){return e(this).button("widget")[0]})).removeClass("ui-state-active").attr("aria-pressed","false")})):(this.buttonElement.bind("mousedown"+this.eventNamespace,(function(){if(r.disabled)return!1;e(this).addClass("ui-state-active"),i=this,t.document.one("mouseup",(function(){i=null}))})).bind("mouseup"+this.eventNamespace,(function(){if(r.disabled)return!1;e(this).removeClass("ui-state-active")})).bind("keydown"+this.eventNamespace,(function(t){if(r.disabled)return!1;t.keyCode!==e.ui.keyCode.SPACE&&t.keyCode!==e.ui.keyCode.ENTER||e(this).addClass("ui-state-active")})).bind("keyup"+this.eventNamespace,(function(){e(this).removeClass("ui-state-active")})),this.buttonElement.is("a")&&this.buttonElement.keyup((function(t){t.keyCode===e.ui.keyCode.SPACE&&e(this).click()}))),this._setOption("disabled",r.disabled),this._resetButton()},_determineButtonType:function(){var e,t,i;this.element.is("[type=checkbox]")?this.type="checkbox":this.element.is("[type=radio]")?this.type="radio":this.element.is("input")?this.type="input":this.type="button","checkbox"===this.type||"radio"===this.type?(e=this.element.parents().last(),t="label[for='"+this.element.attr("id")+"']",this.buttonElement=e.find(t),this.buttonElement.length||(e=e.length?e.siblings():this.element.siblings(),this.buttonElement=e.filter(t),this.buttonElement.length||(this.buttonElement=e.find(t))),this.element.addClass("ui-helper-hidden-accessible"),i=this.element.is(":checked"),i&&this.buttonElement.addClass("ui-state-active"),this.buttonElement.prop("aria-pressed",i)):this.buttonElement=this.element},widget:function(){return this.buttonElement},_destroy:function(){this.element.removeClass("ui-helper-hidden-accessible"),this.buttonElement.removeClass(a+" "+r+" "+l).removeAttr("role").removeAttr("aria-pressed").html(this.buttonElement.find(".ui-button-text").html()),this.hasTitle||this.buttonElement.removeAttr("title")},_setOption:function(e,t){this._super(e,t),"disabled"!==e?this._resetButton():t?this.element.prop("disabled",!0):this.element.prop("disabled",!1)},refresh:function(){var t=this.element.is("input, button")?this.element.is(":disabled"):this.element.hasClass("ui-button-disabled");t!==this.options.disabled&&this._setOption("disabled",t),"radio"===this.type?h(this.element[0]).each((function(){e(this).is(":checked")?e(this).button("widget").addClass("ui-state-active").attr("aria-pressed","true"):e(this).button("widget").removeClass("ui-state-active").attr("aria-pressed","false")})):"checkbox"===this.type&&(this.element.is(":checked")?this.buttonElement.addClass("ui-state-active").attr("aria-pressed","true"):this.buttonElement.removeClass("ui-state-active").attr("aria-pressed","false"))},_resetButton:function(){if("input"!==this.type){var t=this.buttonElement.removeClass(l),i=e("<span></span>",this.document[0]).addClass("ui-button-text").html(this.options.label).appendTo(t.empty()).text(),n=this.options.icons,o=n.primary&&n.secondary,s=[];n.primary||n.secondary?(this.options.text&&s.push("ui-button-text-icon"+(o?"s":n.primary?"-primary":"-secondary")),n.primary&&t.prepend("<span class='ui-button-icon-primary ui-icon "+n.primary+"'></span>"),n.secondary&&t.append("<span class='ui-button-icon-secondary ui-icon "+n.secondary+"'></span>"),this.options.text||(s.push(o?"ui-button-icons-only":"ui-button-icon-only"),this.hasTitle||t.attr("title",e.trim(i)))):s.push("ui-button-text-only"),t.addClass(s.join(" "))}else this.options.label&&this.element.val(this.options.label)}}),e.widget("ui.buttonset",{version:"1.9.2",options:{items:"button, input[type=button], input[type=submit], input[type=reset], input[type=checkbox], input[type=radio], a, :data(button)"},_create:function(){this.element.addClass("ui-buttonset")},_init:function(){this.refresh()},_setOption:function(e,t){"disabled"===e&&this.buttons.button("option",e,t),this._super(e,t)},refresh:function(){var t="rtl"===this.element.css("direction");this.buttons=this.element.find(this.options.items).filter(":ui-button").button("refresh").end().not(":ui-button").button().end().map((function(){return e(this).button("widget")[0]})).removeClass("ui-corner-all ui-corner-left ui-corner-right").filter(":first").addClass(t?"ui-corner-right":"ui-corner-left").end().filter(":last").addClass(t?"ui-corner-left":"ui-corner-right").end().end()},_destroy:function(){this.element.removeClass("ui-buttonset"),this.buttons.map((function(){return e(this).button("widget")[0]})).removeClass("ui-corner-left ui-corner-right").end().button("destroy")}})}(jQuery),function($,undefined){$.extend($.ui,{datepicker:{version:"1.9.2"}});var PROP_NAME="datepicker",dpuuid=(new Date).getTime(),instActive;function Datepicker(){this.debug=!1,this._curInst=null,this._keyEvent=!1,this._disabledInputs=[],this._datepickerShowing=!1,this._inDialog=!1,this._mainDivId="ui-datepicker-div",this._inlineClass="ui-datepicker-inline",this._appendClass="ui-datepicker-append",this._triggerClass="ui-datepicker-trigger",this._dialogClass="ui-datepicker-dialog",this._disableClass="ui-datepicker-disabled",this._unselectableClass="ui-datepicker-unselectable",this._currentClass="ui-datepicker-current-day",this._dayOverClass="ui-datepicker-days-cell-over",this.regional=[],this.regional[""]={closeText:"Done",prevText:"Prev",nextText:"Next",currentText:"Today",monthNames:["January","February","March","April","May","June","July","August","September","October","November","December"],monthNamesShort:["Jan","Feb","Mar","Apr","May","Jun","Jul","Aug","Sep","Oct","Nov","Dec"],dayNames:["Sunday","Monday","Tuesday","Wednesday","Thursday","Friday","Saturday"],dayNamesShort:["Sun","Mon","Tue","Wed","Thu","Fri","Sat"],dayNamesMin:["Su","Mo","Tu","We","Th","Fr","Sa"],weekHeader:"Wk",dateFormat:"mm/dd/yy",firstDay:0,isRTL:!1,showMonthAfterYear:!1,yearSuffix:""},this._defaults={showOn:"focus",showAnim:"fadeIn",showOptions:{},defaultDate:null,appendText:"",buttonText:"...",buttonImage:"",buttonImageOnly:!1,hideIfNoPrevNext:!1,navigationAsDateFormat:!1,gotoCurrent:!1,changeMonth:!1,changeYear:!1,yearRange:"c-10:c+10",showOtherMonths:!1,selectOtherMonths:!1,showWeek:!1,calculateWeek:this.iso8601Week,shortYearCutoff:"+10",minDate:null,maxDate:null,duration:"fast",beforeShowDay:null,beforeShow:null,onSelect:null,onChangeMonthYear:null,onClose:null,numberOfMonths:1,showCurrentAtPos:0,stepMonths:1,stepBigMonths:12,altField:"",altFormat:"",constrainInput:!0,showButtonPanel:!1,autoSize:!1,disabled:!1},$.extend(this._defaults,this.regional[""]),this.dpDiv=bindHover($('<div id="'+this._mainDivId+'" class="ui-datepicker ui-widget ui-widget-content ui-helper-clearfix ui-corner-all"></div>'))}function bindHover(e){var t="button, .ui-datepicker-prev, .ui-datepicker-next, .ui-datepicker-calendar td a";return e.delegate(t,"mouseout",(function(){$(this).removeClass("ui-state-hover"),-1!=this.className.indexOf("ui-datepicker-prev")&&$(this).removeClass("ui-datepicker-prev-hover"),-1!=this.className.indexOf("ui-datepicker-next")&&$(this).removeClass("ui-datepicker-next-hover")})).delegate(t,"mouseover",(function(){$.datepicker._isDisabledDatepicker(instActive.inline?e.parent()[0]:instActive.input[0])||($(this).parents(".ui-datepicker-calendar").find("a").removeClass("ui-state-hover"),$(this).addClass("ui-state-hover"),-1!=this.className.indexOf("ui-datepicker-prev")&&$(this).addClass("ui-datepicker-prev-hover"),-1!=this.className.indexOf("ui-datepicker-next")&&$(this).addClass("ui-datepicker-next-hover"))}))}function extendRemove(e,t){for(var i in $.extend(e,t),t)null!=t[i]&&t[i]!=undefined||(e[i]=t[i]);return e}$.extend(Datepicker.prototype,{markerClassName:"hasDatepicker",maxRows:4,log:function(){this.debug&&console.log.apply("",arguments)},_widgetDatepicker:function(){return this.dpDiv},setDefaults:function(e){return extendRemove(this._defaults,e||{}),this},_attachDatepicker:function(target,settings){var inlineSettings=null;for(var attrName in this._defaults){var attrValue=target.getAttribute("date:"+attrName);if(attrValue){inlineSettings=inlineSettings||{};try{inlineSettings[attrName]=eval(attrValue)}catch(err){inlineSettings[attrName]=attrValue}}}var nodeName=target.nodeName.toLowerCase(),inline="div"==nodeName||"span"==nodeName;target.id||(this.uuid+=1,target.id="dp"+this.uuid);var inst=this._newInst($(target),inline);inst.settings=$.extend({},settings||{},inlineSettings||{}),"input"==nodeName?this._connectDatepicker(target,inst):inline&&this._inlineDatepicker(target,inst)},_newInst:function(e,t){var i=e[0].id.replace(/([^A-Za-z0-9_-])/g,"\\\\$1");return{id:i,input:e,selectedDay:0,selectedMonth:0,selectedYear:0,drawMonth:0,drawYear:0,inline:t,dpDiv:t?bindHover($('<div class="'+this._inlineClass+' ui-datepicker ui-widget ui-widget-content ui-helper-clearfix ui-corner-all"></div>')):this.dpDiv}},_connectDatepicker:function(e,t){var i=$(e);t.append=$([]),t.trigger=$([]),i.hasClass(this.markerClassName)||(this._attachments(i,t),i.addClass(this.markerClassName).keydown(this._doKeyDown).keypress(this._doKeyPress).keyup(this._doKeyUp).bind("setData.datepicker",(function(e,i,n){t.settings[i]=n})).bind("getData.datepicker",(function(e,i){return this._get(t,i)})),this._autoSize(t),$.data(e,PROP_NAME,t),t.settings.disabled&&this._disableDatepicker(e))},_attachments:function(e,t){var i=this._get(t,"appendText"),n=this._get(t,"isRTL");t.append&&t.append.remove(),i&&(t.append=$('<span class="'+this._appendClass+'">'+i+"</span>"),e[n?"before":"after"](t.append)),e.unbind("focus",this._showDatepicker),t.trigger&&t.trigger.remove();var o=this._get(t,"showOn");if("focus"!=o&&"both"!=o||e.focus(this._showDatepicker),"button"==o||"both"==o){var s=this._get(t,"buttonText"),a=this._get(t,"buttonImage");t.trigger=$(this._get(t,"buttonImageOnly")?$("<img/>").addClass(this._triggerClass).attr({src:a,alt:s,title:s}):$('<button type="button"></button>').addClass(this._triggerClass).html(""==a?s:$("<img/>").attr({src:a,alt:s,title:s}))),e[n?"before":"after"](t.trigger),t.trigger.click((function(){return $.datepicker._datepickerShowing&&$.datepicker._lastInput==e[0]?$.datepicker._hideDatepicker():$.datepicker._datepickerShowing&&$.datepicker._lastInput!=e[0]?($.datepicker._hideDatepicker(),$.datepicker._showDatepicker(e[0])):$.datepicker._showDatepicker(e[0]),!1}))}},_autoSize:function(e){if(this._get(e,"autoSize")&&!e.inline){var t=new Date(2009,11,20),i=this._get(e,"dateFormat");if(i.match(/[DM]/)){var n=function(e){for(var t=0,i=0,n=0;n<e.length;n++)e[n].length>t&&(t=e[n].length,i=n);return i};t.setMonth(n(this._get(e,i.match(/MM/)?"monthNames":"monthNamesShort"))),t.setDate(n(this._get(e,i.match(/DD/)?"dayNames":"dayNamesShort"))+20-t.getDay())}e.input.attr("size",this._formatDate(e,t).length)}},_inlineDatepicker:function(e,t){var i=$(e);i.hasClass(this.markerClassName)||(i.addClass(this.markerClassName).append(t.dpDiv).bind("setData.datepicker",(function(e,i,n){t.settings[i]=n})).bind("getData.datepicker",(function(e,i){return this._get(t,i)})),$.data(e,PROP_NAME,t),this._setDate(t,this._getDefaultDate(t),!0),this._updateDatepicker(t),this._updateAlternate(t),t.settings.disabled&&this._disableDatepicker(e),t.dpDiv.css("display","block"))},_dialogDatepicker:function(e,t,i,n,o){var s=this._dialogInst;if(!s){this.uuid+=1;var a="dp"+this.uuid;this._dialogInput=$('<input type="text" id="'+a+'" style="position: absolute; top: -100px; width: 0px;"/>'),this._dialogInput.keydown(this._doKeyDown),$("body").append(this._dialogInput),s=this._dialogInst=this._newInst(this._dialogInput,!1),s.settings={},$.data(this._dialogInput[0],PROP_NAME,s)}if(extendRemove(s.settings,n||{}),t=t&&t.constructor==Date?this._formatDate(s,t):t,this._dialogInput.val(t),this._pos=o?o.length?o:[o.pageX,o.pageY]:null,!this._pos){var r=document.documentElement.clientWidth,l=document.documentElement.clientHeight,c=document.documentElement.scrollLeft||document.body.scrollLeft,h=document.documentElement.scrollTop||document.body.scrollTop;this._pos=[r/2-100+c,l/2-150+h]}return this._dialogInput.css("left",this._pos[0]+20+"px").css("top",this._pos[1]+"px"),s.settings.onSelect=i,this._inDialog=!0,this.dpDiv.addClass(this._dialogClass),this._showDatepicker(this._dialogInput[0]),$.blockUI&&$.blockUI(this.dpDiv),$.data(this._dialogInput[0],PROP_NAME,s),this},_destroyDatepicker:function(e){var t=$(e),i=$.data(e,PROP_NAME);if(t.hasClass(this.markerClassName)){var n=e.nodeName.toLowerCase();$.removeData(e,PROP_NAME),"input"==n?(i.append.remove(),i.trigger.remove(),t.removeClass(this.markerClassName).unbind("focus",this._showDatepicker).unbind("keydown",this._doKeyDown).unbind("keypress",this._doKeyPress).unbind("keyup",this._doKeyUp)):"div"!=n&&"span"!=n||t.removeClass(this.markerClassName).empty()}},_enableDatepicker:function(e){var t=$(e),i=$.data(e,PROP_NAME);if(t.hasClass(this.markerClassName)){var n=e.nodeName.toLowerCase();if("input"==n)e.disabled=!1,i.trigger.filter("button").each((function(){this.disabled=!1})).end().filter("img").css({opacity:"1.0",cursor:""});else if("div"==n||"span"==n){var o=t.children("."+this._inlineClass);o.children().removeClass("ui-state-disabled"),o.find("select.ui-datepicker-month, select.ui-datepicker-year").prop("disabled",!1)}this._disabledInputs=$.map(this._disabledInputs,(function(t){return t==e?null:t}))}},_disableDatepicker:function(e){var t=$(e),i=$.data(e,PROP_NAME);if(t.hasClass(this.markerClassName)){var n=e.nodeName.toLowerCase();if("input"==n)e.disabled=!0,i.trigger.filter("button").each((function(){this.disabled=!0})).end().filter("img").css({opacity:"0.5",cursor:"default"});else if("div"==n||"span"==n){var o=t.children("."+this._inlineClass);o.children().addClass("ui-state-disabled"),o.find("select.ui-datepicker-month, select.ui-datepicker-year").prop("disabled",!0)}this._disabledInputs=$.map(this._disabledInputs,(function(t){return t==e?null:t})),this._disabledInputs[this._disabledInputs.length]=e}},_isDisabledDatepicker:function(e){if(!e)return!1;for(var t=0;t<this._disabledInputs.length;t++)if(this._disabledInputs[t]==e)return!0;return!1},_getInst:function(e){try{return $.data(e,PROP_NAME)}catch(t){throw"Missing instance data for this datepicker"}},_optionDatepicker:function(e,t,i){var n=this._getInst(e);if(2==arguments.length&&"string"==typeof t)return"defaults"==t?$.extend({},$.datepicker._defaults):n?"all"==t?$.extend({},n.settings):this._get(n,t):null;var o=t||{};if("string"==typeof t&&(o={},o[t]=i),n){this._curInst==n&&this._hideDatepicker();var s=this._getDateDatepicker(e,!0),a=this._getMinMaxDate(n,"min"),r=this._getMinMaxDate(n,"max");extendRemove(n.settings,o),null!==a&&o["dateFormat"]!==undefined&&o["minDate"]===undefined&&(n.settings.minDate=this._formatDate(n,a)),null!==r&&o["dateFormat"]!==undefined&&o["maxDate"]===undefined&&(n.settings.maxDate=this._formatDate(n,r)),this._attachments($(e),n),this._autoSize(n),this._setDate(n,s),this._updateAlternate(n),this._updateDatepicker(n)}},_changeDatepicker:function(e,t,i){this._optionDatepicker(e,t,i)},_refreshDatepicker:function(e){var t=this._getInst(e);t&&this._updateDatepicker(t)},_setDateDatepicker:function(e,t){var i=this._getInst(e);i&&(this._setDate(i,t),this._updateDatepicker(i),this._updateAlternate(i))},_getDateDatepicker:function(e,t){var i=this._getInst(e);return i&&!i.inline&&this._setDateFromField(i,t),i?this._getDate(i):null},_doKeyDown:function(e){var t=$.datepicker._getInst(e.target),i=!0,n=t.dpDiv.is(".ui-datepicker-rtl");if(t._keyEvent=!0,$.datepicker._datepickerShowing)switch(e.keyCode){case 9:$.datepicker._hideDatepicker(),i=!1;break;case 13:var o=$("td."+$.datepicker._dayOverClass+":not(."+$.datepicker._currentClass+")",t.dpDiv);o[0]&&$.datepicker._selectDay(e.target,t.selectedMonth,t.selectedYear,o[0]);var s=$.datepicker._get(t,"onSelect");if(s){var a=$.datepicker._formatDate(t);s.apply(t.input?t.input[0]:null,[a,t])}else $.datepicker._hideDatepicker();return!1;case 27:$.datepicker._hideDatepicker();break;case 33:$.datepicker._adjustDate(e.target,e.ctrlKey?-$.datepicker._get(t,"stepBigMonths"):-$.datepicker._get(t,"stepMonths"),"M");break;case 34:$.datepicker._adjustDate(e.target,e.ctrlKey?+$.datepicker._get(t,"stepBigMonths"):+$.datepicker._get(t,"stepMonths"),"M");break;case 35:(e.ctrlKey||e.metaKey)&&$.datepicker._clearDate(e.target),i=e.ctrlKey||e.metaKey;break;case 36:(e.ctrlKey||e.metaKey)&&$.datepicker._gotoToday(e.target),i=e.ctrlKey||e.metaKey;break;case 37:(e.ctrlKey||e.metaKey)&&$.datepicker._adjustDate(e.target,n?1:-1,"D"),i=e.ctrlKey||e.metaKey,e.originalEvent.altKey&&$.datepicker._adjustDate(e.target,e.ctrlKey?-$.datepicker._get(t,"stepBigMonths"):-$.datepicker._get(t,"stepMonths"),"M");break;case 38:(e.ctrlKey||e.metaKey)&&$.datepicker._adjustDate(e.target,-7,"D"),i=e.ctrlKey||e.metaKey;break;case 39:(e.ctrlKey||e.metaKey)&&$.datepicker._adjustDate(e.target,n?-1:1,"D"),i=e.ctrlKey||e.metaKey,e.originalEvent.altKey&&$.datepicker._adjustDate(e.target,e.ctrlKey?+$.datepicker._get(t,"stepBigMonths"):+$.datepicker._get(t,"stepMonths"),"M");break;case 40:(e.ctrlKey||e.metaKey)&&$.datepicker._adjustDate(e.target,7,"D"),i=e.ctrlKey||e.metaKey;break;default:i=!1}else 36==e.keyCode&&e.ctrlKey?$.datepicker._showDatepicker(this):i=!1;i&&(e.preventDefault(),e.stopPropagation())},_doKeyPress:function(e){var t=$.datepicker._getInst(e.target);if($.datepicker._get(t,"constrainInput")){var i=$.datepicker._possibleChars($.datepicker._get(t,"dateFormat")),n=String.fromCharCode(e.charCode==undefined?e.keyCode:e.charCode);return e.ctrlKey||e.metaKey||n<" "||!i||i.indexOf(n)>-1}},_doKeyUp:function(e){var t=$.datepicker._getInst(e.target);if(t.input.val()!=t.lastVal)try{var i=$.datepicker.parseDate($.datepicker._get(t,"dateFormat"),t.input?t.input.val():null,$.datepicker._getFormatConfig(t));i&&($.datepicker._setDateFromField(t),$.datepicker._updateAlternate(t),$.datepicker._updateDatepicker(t))}catch(n){$.datepicker.log(n)}return!0},_showDatepicker:function(e){if(e=e.target||e,"input"!=e.nodeName.toLowerCase()&&(e=$("input",e.parentNode)[0]),!$.datepicker._isDisabledDatepicker(e)&&$.datepicker._lastInput!=e){var t=$.datepicker._getInst(e);$.datepicker._curInst&&$.datepicker._curInst!=t&&($.datepicker._curInst.dpDiv.stop(!0,!0),t&&$.datepicker._datepickerShowing&&$.datepicker._hideDatepicker($.datepicker._curInst.input[0]));var i=$.datepicker._get(t,"beforeShow"),n=i?i.apply(e,[e,t]):{};if(!1!==n){extendRemove(t.settings,n),t.lastVal=null,$.datepicker._lastInput=e,$.datepicker._setDateFromField(t),$.datepicker._inDialog&&(e.value=""),$.datepicker._pos||($.datepicker._pos=$.datepicker._findPos(e),$.datepicker._pos[1]+=e.offsetHeight);var o=!1;$(e).parents().each((function(){return o|="fixed"==$(this).css("position"),!o}));var s={left:$.datepicker._pos[0],top:$.datepicker._pos[1]};if($.datepicker._pos=null,t.dpDiv.empty(),t.dpDiv.css({position:"absolute",display:"block",top:"-1000px"}),$.datepicker._updateDatepicker(t),s=$.datepicker._checkOffset(t,s,o),t.dpDiv.css({position:$.datepicker._inDialog&&$.blockUI?"static":o?"fixed":"absolute",display:"none",left:s.left+"px",top:s.top+"px"}),!t.inline){var a=$.datepicker._get(t,"showAnim"),r=$.datepicker._get(t,"duration"),l=function(){var e=t.dpDiv.find("iframe.ui-datepicker-cover");if(e.length){var i=$.datepicker._getBorders(t.dpDiv);e.css({left:-i[0],top:-i[1],width:t.dpDiv.outerWidth(),height:t.dpDiv.outerHeight()})}};t.dpDiv.zIndex($(e).zIndex()+1),$.datepicker._datepickerShowing=!0,$.effects&&($.effects.effect[a]||$.effects[a])?t.dpDiv.show(a,$.datepicker._get(t,"showOptions"),r,l):t.dpDiv[a||"show"](a?r:null,l),a&&r||l(),t.input.is(":visible")&&!t.input.is(":disabled")&&t.input.focus(),$.datepicker._curInst=t}}}},_updateDatepicker:function(e){this.maxRows=4;var t=$.datepicker._getBorders(e.dpDiv);instActive=e,e.dpDiv.empty().append(this._generateHTML(e)),this._attachHandlers(e);var i=e.dpDiv.find("iframe.ui-datepicker-cover");i.length&&i.css({left:-t[0],top:-t[1],width:e.dpDiv.outerWidth(),height:e.dpDiv.outerHeight()}),e.dpDiv.find("."+this._dayOverClass+" a").mouseover();var n=this._getNumberOfMonths(e),o=n[1],s=17;if(e.dpDiv.removeClass("ui-datepicker-multi-2 ui-datepicker-multi-3 ui-datepicker-multi-4").width(""),o>1&&e.dpDiv.addClass("ui-datepicker-multi-"+o).css("width",s*o+"em"),e.dpDiv[(1!=n[0]||1!=n[1]?"add":"remove")+"Class"]("ui-datepicker-multi"),e.dpDiv[(this._get(e,"isRTL")?"add":"remove")+"Class"]("ui-datepicker-rtl"),e==$.datepicker._curInst&&$.datepicker._datepickerShowing&&e.input&&e.input.is(":visible")&&!e.input.is(":disabled")&&e.input[0]!=document.activeElement&&e.input.focus(),e.yearshtml){var a=e.yearshtml;setTimeout((function(){a===e.yearshtml&&e.yearshtml&&e.dpDiv.find("select.ui-datepicker-year:first").replaceWith(e.yearshtml),a=e.yearshtml=null}),0)}},_getBorders:function(e){var t=function(e){return{thin:1,medium:2,thick:3}[e]||e};return[parseFloat(t(e.css("border-left-width"))),parseFloat(t(e.css("border-top-width")))]},_checkOffset:function(e,t,i){var n=e.dpDiv.outerWidth(),o=e.dpDiv.outerHeight(),s=e.input?e.input.outerWidth():0,a=e.input?e.input.outerHeight():0,r=document.documentElement.clientWidth+(i?0:$(document).scrollLeft()),l=document.documentElement.clientHeight+(i?0:$(document).scrollTop());return t.left-=this._get(e,"isRTL")?n-s:0,t.left-=i&&t.left==e.input.offset().left?$(document).scrollLeft():0,t.top-=i&&t.top==e.input.offset().top+a?$(document).scrollTop():0,t.left-=Math.min(t.left,t.left+n>r&&r>n?Math.abs(t.left+n-r):0),t.top-=Math.min(t.top,t.top+o>l&&l>o?Math.abs(o+a):0),t},_findPos:function(e){var t=this._getInst(e),i=this._get(t,"isRTL");while(e&&("hidden"==e.type||1!=e.nodeType||$.expr.filters.hidden(e)))e=e[i?"previousSibling":"nextSibling"];var n=$(e).offset();return[n.left,n.top]},_hideDatepicker:function(e){var t=this._curInst;if(t&&(!e||t==$.data(e,PROP_NAME))&&this._datepickerShowing){var i=this._get(t,"showAnim"),n=this._get(t,"duration"),o=function(){$.datepicker._tidyDialog(t)};$.effects&&($.effects.effect[i]||$.effects[i])?t.dpDiv.hide(i,$.datepicker._get(t,"showOptions"),n,o):t.dpDiv["slideDown"==i?"slideUp":"fadeIn"==i?"fadeOut":"hide"](i?n:null,o),i||o(),this._datepickerShowing=!1;var s=this._get(t,"onClose");s&&s.apply(t.input?t.input[0]:null,[t.input?t.input.val():"",t]),this._lastInput=null,this._inDialog&&(this._dialogInput.css({position:"absolute",left:"0",top:"-100px"}),$.blockUI&&($.unblockUI(),$("body").append(this.dpDiv))),this._inDialog=!1}},_tidyDialog:function(e){e.dpDiv.removeClass(this._dialogClass).unbind(".ui-datepicker-calendar")},_checkExternalClick:function(e){if($.datepicker._curInst){var t=$(e.target),i=$.datepicker._getInst(t[0]);(t[0].id==$.datepicker._mainDivId||0!=t.parents("#"+$.datepicker._mainDivId).length||t.hasClass($.datepicker.markerClassName)||t.closest("."+$.datepicker._triggerClass).length||!$.datepicker._datepickerShowing||$.datepicker._inDialog&&$.blockUI)&&(!t.hasClass($.datepicker.markerClassName)||$.datepicker._curInst==i)||$.datepicker._hideDatepicker()}},_adjustDate:function(e,t,i){var n=$(e),o=this._getInst(n[0]);this._isDisabledDatepicker(n[0])||(this._adjustInstDate(o,t+("M"==i?this._get(o,"showCurrentAtPos"):0),i),this._updateDatepicker(o))},_gotoToday:function(e){var t=$(e),i=this._getInst(t[0]);if(this._get(i,"gotoCurrent")&&i.currentDay)i.selectedDay=i.currentDay,i.drawMonth=i.selectedMonth=i.currentMonth,i.drawYear=i.selectedYear=i.currentYear;else{var n=new Date;i.selectedDay=n.getDate(),i.drawMonth=i.selectedMonth=n.getMonth(),i.drawYear=i.selectedYear=n.getFullYear()}this._notifyChange(i),this._adjustDate(t)},_selectMonthYear:function(e,t,i){var n=$(e),o=this._getInst(n[0]);o["selected"+("M"==i?"Month":"Year")]=o["draw"+("M"==i?"Month":"Year")]=parseInt(t.options[t.selectedIndex].value,10),this._notifyChange(o),this._adjustDate(n)},_selectDay:function(e,t,i,n){var o=$(e);if(!$(n).hasClass(this._unselectableClass)&&!this._isDisabledDatepicker(o[0])){var s=this._getInst(o[0]);s.selectedDay=s.currentDay=$("a",n).html(),s.selectedMonth=s.currentMonth=t,s.selectedYear=s.currentYear=i,this._selectDate(e,this._formatDate(s,s.currentDay,s.currentMonth,s.currentYear))}},_clearDate:function(e){var t=$(e);this._getInst(t[0]);this._selectDate(t,"")},_selectDate:function(e,t){var i=$(e),n=this._getInst(i[0]);t=null!=t?t:this._formatDate(n),n.input&&n.input.val(t),this._updateAlternate(n);var o=this._get(n,"onSelect");o?o.apply(n.input?n.input[0]:null,[t,n]):n.input&&n.input.trigger("change"),n.inline?this._updateDatepicker(n):(this._hideDatepicker(),this._lastInput=n.input[0],"object"!=typeof n.input[0]&&n.input.focus(),this._lastInput=null)},_updateAlternate:function(e){var t=this._get(e,"altField");if(t){var i=this._get(e,"altFormat")||this._get(e,"dateFormat"),n=this._getDate(e),o=this.formatDate(i,n,this._getFormatConfig(e));$(t).each((function(){$(this).val(o)}))}},noWeekends:function(e){var t=e.getDay();return[t>0&&t<6,""]},iso8601Week:function(e){var t=new Date(e.getTime());t.setDate(t.getDate()+4-(t.getDay()||7));var i=t.getTime();return t.setMonth(0),t.setDate(1),Math.floor(Math.round((i-t)/864e5)/7)+1},parseDate:function(e,t,i){if(null==e||null==t)throw"Invalid arguments";if(t="object"==typeof t?t.toString():t+"",""==t)return null;var n=(i?i.shortYearCutoff:null)||this._defaults.shortYearCutoff;n="string"!=typeof n?n:(new Date).getFullYear()%100+parseInt(n,10);for(var o=(i?i.dayNamesShort:null)||this._defaults.dayNamesShort,s=(i?i.dayNames:null)||this._defaults.dayNames,a=(i?i.monthNamesShort:null)||this._defaults.monthNamesShort,r=(i?i.monthNames:null)||this._defaults.monthNames,l=-1,c=-1,h=-1,u=-1,d=!1,p=function(t){var i=w+1<e.length&&e.charAt(w+1)==t;return i&&w++,i},f=function(e){var i=p(e),n="@"==e?14:"!"==e?20:"y"==e&&i?4:"o"==e?3:2,o=new RegExp("^\\d{1,"+n+"}"),s=t.substring(v).match(o);if(!s)throw"Missing number at position "+v;return v+=s[0].length,parseInt(s[0],10)},g=function(e,i,n){var o=$.map(p(e)?n:i,(function(e,t){return[[t,e]]})).sort((function(e,t){return-(e[1].length-t[1].length)})),s=-1;if($.each(o,(function(e,i){var n=i[1];if(t.substr(v,n.length).toLowerCase()==n.toLowerCase())return s=i[0],v+=n.length,!1})),-1!=s)return s+1;throw"Unknown name at position "+v},m=function(){if(t.charAt(v)!=e.charAt(w))throw"Unexpected literal at position "+v;v++},v=0,w=0;w<e.length;w++)if(d)"'"!=e.charAt(w)||p("'")?m():d=!1;else switch(e.charAt(w)){case"d":h=f("d");break;case"D":g("D",o,s);break;case"o":u=f("o");break;case"m":c=f("m");break;case"M":c=g("M",a,r);break;case"y":l=f("y");break;case"@":var y=new Date(f("@"));l=y.getFullYear(),c=y.getMonth()+1,h=y.getDate();break;case"!":y=new Date((f("!")-this._ticksTo1970)/1e4);l=y.getFullYear(),c=y.getMonth()+1,h=y.getDate();break;case"'":p("'")?m():d=!0;break;default:m()}if(v<t.length){var b=t.substr(v);if(!/^\s+/.test(b))throw"Extra/unparsed characters found in date: "+b}if(-1==l?l=(new Date).getFullYear():l<100&&(l+=(new Date).getFullYear()-(new Date).getFullYear()%100+(l<=n?0:-100)),u>-1){c=1,h=u;do{var C=this._getDaysInMonth(l,c-1);if(h<=C)break;c++,h-=C}while(1)}y=this._daylightSavingAdjust(new Date(l,c-1,h));if(y.getFullYear()!=l||y.getMonth()+1!=c||y.getDate()!=h)throw"Invalid date";return y},ATOM:"yy-mm-dd",COOKIE:"D, dd M yy",ISO_8601:"yy-mm-dd",RFC_822:"D, d M y",RFC_850:"DD, dd-M-y",RFC_1036:"D, d M y",RFC_1123:"D, d M yy",RFC_2822:"D, d M yy",RSS:"D, d M y",TICKS:"!",TIMESTAMP:"@",W3C:"yy-mm-dd",_ticksTo1970:24*(718685+Math.floor(492.5)-Math.floor(19.7)+Math.floor(4.925))*60*60*1e7,formatDate:function(e,t,i){if(!t)return"";var n=(i?i.dayNamesShort:null)||this._defaults.dayNamesShort,o=(i?i.dayNames:null)||this._defaults.dayNames,s=(i?i.monthNamesShort:null)||this._defaults.monthNamesShort,a=(i?i.monthNames:null)||this._defaults.monthNames,r=function(t){var i=d+1<e.length&&e.charAt(d+1)==t;return i&&d++,i},l=function(e,t,i){var n=""+t;if(r(e))while(n.length<i)n="0"+n;return n},c=function(e,t,i,n){return r(e)?n[t]:i[t]},h="",u=!1;if(t)for(var d=0;d<e.length;d++)if(u)"'"!=e.charAt(d)||r("'")?h+=e.charAt(d):u=!1;else switch(e.charAt(d)){case"d":h+=l("d",t.getDate(),2);break;case"D":h+=c("D",t.getDay(),n,o);break;case"o":h+=l("o",Math.round((new Date(t.getFullYear(),t.getMonth(),t.getDate()).getTime()-new Date(t.getFullYear(),0,0).getTime())/864e5),3);break;case"m":h+=l("m",t.getMonth()+1,2);break;case"M":h+=c("M",t.getMonth(),s,a);break;case"y":h+=r("y")?t.getFullYear():(t.getYear()%100<10?"0":"")+t.getYear()%100;break;case"@":h+=t.getTime();break;case"!":h+=1e4*t.getTime()+this._ticksTo1970;break;case"'":r("'")?h+="'":u=!0;break;default:h+=e.charAt(d)}return h},_possibleChars:function(e){for(var t="",i=!1,n=function(t){var i=o+1<e.length&&e.charAt(o+1)==t;return i&&o++,i},o=0;o<e.length;o++)if(i)"'"!=e.charAt(o)||n("'")?t+=e.charAt(o):i=!1;else switch(e.charAt(o)){case"d":case"m":case"y":case"@":t+="0123456789";break;case"D":case"M":return null;case"'":n("'")?t+="'":i=!0;break;default:t+=e.charAt(o)}return t},_get:function(e,t){return e.settings[t]!==undefined?e.settings[t]:this._defaults[t]},_setDateFromField:function(e,t){if(e.input.val()!=e.lastVal){var i,n,o=this._get(e,"dateFormat"),s=e.lastVal=e.input?e.input.val():null;i=n=this._getDefaultDate(e);var a=this._getFormatConfig(e);try{i=this.parseDate(o,s,a)||n}catch(r){this.log(r),s=t?"":s}e.selectedDay=i.getDate(),e.drawMonth=e.selectedMonth=i.getMonth(),e.drawYear=e.selectedYear=i.getFullYear(),e.currentDay=s?i.getDate():0,e.currentMonth=s?i.getMonth():0,e.currentYear=s?i.getFullYear():0,this._adjustInstDate(e)}},_getDefaultDate:function(e){return this._restrictMinMax(e,this._determineDate(e,this._get(e,"defaultDate"),new Date))},_determineDate:function(e,t,i){var n=function(e){var t=new Date;return t.setDate(t.getDate()+e),t},o=function(t){try{return $.datepicker.parseDate($.datepicker._get(e,"dateFormat"),t,$.datepicker._getFormatConfig(e))}catch(l){}var i=(t.toLowerCase().match(/^c/)?$.datepicker._getDate(e):null)||new Date,n=i.getFullYear(),o=i.getMonth(),s=i.getDate(),a=/([+-]?[0-9]+)\s*(d|D|w|W|m|M|y|Y)?/g,r=a.exec(t);while(r){switch(r[2]||"d"){case"d":case"D":s+=parseInt(r[1],10);break;case"w":case"W":s+=7*parseInt(r[1],10);break;case"m":case"M":o+=parseInt(r[1],10),s=Math.min(s,$.datepicker._getDaysInMonth(n,o));break;case"y":case"Y":n+=parseInt(r[1],10),s=Math.min(s,$.datepicker._getDaysInMonth(n,o));break}r=a.exec(t)}return new Date(n,o,s)},s=null==t||""===t?i:"string"==typeof t?o(t):"number"==typeof t?isNaN(t)?i:n(t):new Date(t.getTime());return s=s&&"Invalid Date"==s.toString()?i:s,s&&(s.setHours(0),s.setMinutes(0),s.setSeconds(0),s.setMilliseconds(0)),this._daylightSavingAdjust(s)},_daylightSavingAdjust:function(e){return e?(e.setHours(e.getHours()>12?e.getHours()+2:0),e):null},_setDate:function(e,t,i){var n=!t,o=e.selectedMonth,s=e.selectedYear,a=this._restrictMinMax(e,this._determineDate(e,t,new Date));e.selectedDay=e.currentDay=a.getDate(),e.drawMonth=e.selectedMonth=e.currentMonth=a.getMonth(),e.drawYear=e.selectedYear=e.currentYear=a.getFullYear(),o==e.selectedMonth&&s==e.selectedYear||i||this._notifyChange(e),this._adjustInstDate(e),e.input&&e.input.val(n?"":this._formatDate(e))},_getDate:function(e){var t=!e.currentYear||e.input&&""==e.input.val()?null:this._daylightSavingAdjust(new Date(e.currentYear,e.currentMonth,e.currentDay));return t},_attachHandlers:function(e){var t=this._get(e,"stepMonths"),i="#"+e.id.replace(/\\\\/g,"\\");e.dpDiv.find("[data-handler]").map((function(){var e={prev:function(){window["DP_jQuery_"+dpuuid].datepicker._adjustDate(i,-t,"M")},next:function(){window["DP_jQuery_"+dpuuid].datepicker._adjustDate(i,+t,"M")},hide:function(){window["DP_jQuery_"+dpuuid].datepicker._hideDatepicker()},today:function(){window["DP_jQuery_"+dpuuid].datepicker._gotoToday(i)},selectDay:function(){return window["DP_jQuery_"+dpuuid].datepicker._selectDay(i,+this.getAttribute("data-month"),+this.getAttribute("data-year"),this),!1},selectMonth:function(){return window["DP_jQuery_"+dpuuid].datepicker._selectMonthYear(i,this,"M"),!1},selectYear:function(){return window["DP_jQuery_"+dpuuid].datepicker._selectMonthYear(i,this,"Y"),!1}};$(this).bind(this.getAttribute("data-event"),e[this.getAttribute("data-handler")])}))},_generateHTML:function(e){var t=new Date;t=this._daylightSavingAdjust(new Date(t.getFullYear(),t.getMonth(),t.getDate()));var i=this._get(e,"isRTL"),n=this._get(e,"showButtonPanel"),o=this._get(e,"hideIfNoPrevNext"),s=this._get(e,"navigationAsDateFormat"),a=this._getNumberOfMonths(e),r=this._get(e,"showCurrentAtPos"),l=this._get(e,"stepMonths"),c=1!=a[0]||1!=a[1],h=this._daylightSavingAdjust(e.currentDay?new Date(e.currentYear,e.currentMonth,e.currentDay):new Date(9999,9,9)),u=this._getMinMaxDate(e,"min"),d=this._getMinMaxDate(e,"max"),p=e.drawMonth-r,f=e.drawYear;if(p<0&&(p+=12,f--),d){var g=this._daylightSavingAdjust(new Date(d.getFullYear(),d.getMonth()-a[0]*a[1]+1,d.getDate()));g=u&&g<u?u:g;while(this._daylightSavingAdjust(new Date(f,p,1))>g)p--,p<0&&(p=11,f--)}e.drawMonth=p,e.drawYear=f;var m=this._get(e,"prevText");m=s?this.formatDate(m,this._daylightSavingAdjust(new Date(f,p-l,1)),this._getFormatConfig(e)):m;var v=this._canAdjustMonth(e,-1,f,p)?'<a class="ui-datepicker-prev ui-corner-all" data-handler="prev" data-event="click" title="'+m+'"><span class="ui-icon ui-icon-circle-triangle-'+(i?"e":"w")+'">'+m+"</span></a>":o?"":'<a class="ui-datepicker-prev ui-corner-all ui-state-disabled" title="'+m+'"><span class="ui-icon ui-icon-circle-triangle-'+(i?"e":"w")+'">'+m+"</span></a>",w=this._get(e,"nextText");w=s?this.formatDate(w,this._daylightSavingAdjust(new Date(f,p+l,1)),this._getFormatConfig(e)):w;var y=this._canAdjustMonth(e,1,f,p)?'<a class="ui-datepicker-next ui-corner-all" data-handler="next" data-event="click" title="'+w+'"><span class="ui-icon ui-icon-circle-triangle-'+(i?"w":"e")+'">'+w+"</span></a>":o?"":'<a class="ui-datepicker-next ui-corner-all ui-state-disabled" title="'+w+'"><span class="ui-icon ui-icon-circle-triangle-'+(i?"w":"e")+'">'+w+"</span></a>",b=this._get(e,"currentText"),C=this._get(e,"gotoCurrent")&&e.currentDay?h:t;b=s?this.formatDate(b,C,this._getFormatConfig(e)):b;var _=e.inline?"":'<button type="button" class="ui-datepicker-close ui-state-default ui-priority-primary ui-corner-all" data-handler="hide" data-event="click">'+this._get(e,"closeText")+"</button>",x=n?'<div class="ui-datepicker-buttonpane ui-widget-content">'+(i?_:"")+(this._isInRange(e,C)?'<button type="button" class="ui-datepicker-current ui-state-default ui-priority-secondary ui-corner-all" data-handler="today" data-event="click">'+b+"</button>":"")+(i?"":_)+"</div>":"",k=parseInt(this._get(e,"firstDay"),10);k=isNaN(k)?0:k;for(var S=this._get(e,"showWeek"),D=this._get(e,"dayNames"),R=(this._get(e,"dayNamesShort"),this._get(e,"dayNamesMin")),T=this._get(e,"monthNames"),P=this._get(e,"monthNamesShort"),I=this._get(e,"beforeShowDay"),M=this._get(e,"showOtherMonths"),A=this._get(e,"selectOtherMonths"),E=(this._get(e,"calculateWeek")||this.iso8601Week,this._getDefaultDate(e)),N="",z=0;z<a[0];z++){var H="";this.maxRows=4;for(var L=0;L<a[1];L++){var F=this._daylightSavingAdjust(new Date(f,p,e.selectedDay)),W=" ui-corner-all",O="";if(c){if(O+='<div class="ui-datepicker-group',a[1]>1)switch(L){case 0:O+=" ui-datepicker-group-first",W=" ui-corner-"+(i?"right":"left");break;case a[1]-1:O+=" ui-datepicker-group-last",W=" ui-corner-"+(i?"left":"right");break;default:O+=" ui-datepicker-group-middle",W="";break}O+='">'}O+='<div class="ui-datepicker-header ui-widget-header ui-helper-clearfix'+W+'">'+(/all|left/.test(W)&&0==z?i?y:v:"")+(/all|right/.test(W)&&0==z?i?v:y:"")+this._generateMonthYearHeader(e,p,f,u,d,z>0||L>0,T,P)+'</div><table class="ui-datepicker-calendar"><thead><tr>';for(var j=S?'<th class="ui-datepicker-week-col">'+this._get(e,"weekHeader")+"</th>":"",B=0;B<7;B++){var q=(B+k)%7;j+="<th"+((B+k+6)%7>=5?' class="ui-datepicker-week-end"':"")+'><span title="'+D[q]+'">'+R[q]+"</span></th>"}O+=j+"</tr></thead><tbody>";var V=this._getDaysInMonth(f,p);f==e.selectedYear&&p==e.selectedMonth&&(e.selectedDay=Math.min(e.selectedDay,V));var G=(this._getFirstDayOfMonth(f,p)-k+7)%7,Y=Math.ceil((G+V)/7),K=c&&this.maxRows>Y?this.maxRows:Y;this.maxRows=K;for(var U=this._daylightSavingAdjust(new Date(f,p,1-G)),X=0;X<K;X++){O+="<tr>";var Q=S?'<td class="ui-datepicker-week-col">'+this._get(e,"calculateWeek")(U)+"</td>":"";for(B=0;B<7;B++){var J=I?I.apply(e.input?e.input[0]:null,[U]):[!0,""],Z=U.getMonth()!=p,ee=Z&&!A||!J[0]||u&&U<u||d&&U>d;Q+='<td class="'+((B+k+6)%7>=5?" ui-datepicker-week-end":"")+(Z?" ui-datepicker-other-month":"")+(U.getTime()==F.getTime()&&p==e.selectedMonth&&e._keyEvent||E.getTime()==U.getTime()&&E.getTime()==F.getTime()?" "+this._dayOverClass:"")+(ee?" "+this._unselectableClass+" ui-state-disabled":"")+(Z&&!M?"":" "+J[1]+(U.getTime()==h.getTime()?" "+this._currentClass:"")+(U.getTime()==t.getTime()?" ui-datepicker-today":""))+'"'+(Z&&!M||!J[2]?"":' title="'+J[2]+'"')+(ee?"":' data-handler="selectDay" data-event="click" data-month="'+U.getMonth()+'" data-year="'+U.getFullYear()+'"')+">"+(Z&&!M?"&#xa0;":ee?'<span class="ui-state-default">'+U.getDate()+"</span>":'<a class="ui-state-default'+(U.getTime()==t.getTime()?" ui-state-highlight":"")+(U.getTime()==h.getTime()?" ui-state-active":"")+(Z?" ui-priority-secondary":"")+'" href="#">'+U.getDate()+"</a>")+"</td>",U.setDate(U.getDate()+1),U=this._daylightSavingAdjust(U)}O+=Q+"</tr>"}p++,p>11&&(p=0,f++),O+="</tbody></table>"+(c?"</div>"+(a[0]>0&&L==a[1]-1?'<div class="ui-datepicker-row-break"></div>':""):""),H+=O}N+=H}return N+=x+($.ui.ie6&&!e.inline?'<iframe src="javascript:false;" class="ui-datepicker-cover" frameborder="0"></iframe>':""),e._keyEvent=!1,N},_generateMonthYearHeader:function(e,t,i,n,o,s,a,r){var l=this._get(e,"changeMonth"),c=this._get(e,"changeYear"),h=this._get(e,"showMonthAfterYear"),u='<div class="ui-datepicker-title">',d="";if(s||!l)d+='<span class="ui-datepicker-month">'+a[t]+"</span>";else{var p=n&&n.getFullYear()==i,f=o&&o.getFullYear()==i;d+='<select class="ui-datepicker-month" data-handler="selectMonth" data-event="change">';for(var g=0;g<12;g++)(!p||g>=n.getMonth())&&(!f||g<=o.getMonth())&&(d+='<option value="'+g+'"'+(g==t?' selected="selected"':"")+">"+r[g]+"</option>");d+="</select>"}if(h||(u+=d+(!s&&l&&c?"":"&#xa0;")),!e.yearshtml)if(e.yearshtml="",s||!c)u+='<span class="ui-datepicker-year">'+i+"</span>";else{var m=this._get(e,"yearRange").split(":"),v=(new Date).getFullYear(),w=function(e){var t=e.match(/c[+-].*/)?i+parseInt(e.substring(1),10):e.match(/[+-].*/)?v+parseInt(e,10):parseInt(e,10);return isNaN(t)?v:t},y=w(m[0]),b=Math.max(y,w(m[1]||""));for(y=n?Math.max(y,n.getFullYear()):y,b=o?Math.min(b,o.getFullYear()):b,e.yearshtml+='<select class="ui-datepicker-year" data-handler="selectYear" data-event="change">';y<=b;y++)e.yearshtml+='<option value="'+y+'"'+(y==i?' selected="selected"':"")+">"+y+"</option>";e.yearshtml+="</select>",u+=e.yearshtml,e.yearshtml=null}return u+=this._get(e,"yearSuffix"),h&&(u+=(!s&&l&&c?"":"&#xa0;")+d),u+="</div>",u},_adjustInstDate:function(e,t,i){var n=e.drawYear+("Y"==i?t:0),o=e.drawMonth+("M"==i?t:0),s=Math.min(e.selectedDay,this._getDaysInMonth(n,o))+("D"==i?t:0),a=this._restrictMinMax(e,this._daylightSavingAdjust(new Date(n,o,s)));e.selectedDay=a.getDate(),e.drawMonth=e.selectedMonth=a.getMonth(),e.drawYear=e.selectedYear=a.getFullYear(),"M"!=i&&"Y"!=i||this._notifyChange(e)},_restrictMinMax:function(e,t){var i=this._getMinMaxDate(e,"min"),n=this._getMinMaxDate(e,"max"),o=i&&t<i?i:t;return o=n&&o>n?n:o,o},_notifyChange:function(e){var t=this._get(e,"onChangeMonthYear");t&&t.apply(e.input?e.input[0]:null,[e.selectedYear,e.selectedMonth+1,e])},_getNumberOfMonths:function(e){var t=this._get(e,"numberOfMonths");return null==t?[1,1]:"number"==typeof t?[1,t]:t},_getMinMaxDate:function(e,t){return this._determineDate(e,this._get(e,t+"Date"),null)},_getDaysInMonth:function(e,t){return 32-this._daylightSavingAdjust(new Date(e,t,32)).getDate()},_getFirstDayOfMonth:function(e,t){return new Date(e,t,1).getDay()},_canAdjustMonth:function(e,t,i,n){var o=this._getNumberOfMonths(e),s=this._daylightSavingAdjust(new Date(i,n+(t<0?t:o[0]*o[1]),1));return t<0&&s.setDate(this._getDaysInMonth(s.getFullYear(),s.getMonth())),this._isInRange(e,s)},_isInRange:function(e,t){var i=this._getMinMaxDate(e,"min"),n=this._getMinMaxDate(e,"max");return(!i||t.getTime()>=i.getTime())&&(!n||t.getTime()<=n.getTime())},_getFormatConfig:function(e){var t=this._get(e,"shortYearCutoff");return t="string"!=typeof t?t:(new Date).getFullYear()%100+parseInt(t,10),{shortYearCutoff:t,dayNamesShort:this._get(e,"dayNamesShort"),dayNames:this._get(e,"dayNames"),monthNamesShort:this._get(e,"monthNamesShort"),monthNames:this._get(e,"monthNames")}},_formatDate:function(e,t,i,n){t||(e.currentDay=e.selectedDay,e.currentMonth=e.selectedMonth,e.currentYear=e.selectedYear);var o=t?"object"==typeof t?t:this._daylightSavingAdjust(new Date(n,i,t)):this._daylightSavingAdjust(new Date(e.currentYear,e.currentMonth,e.currentDay));return this.formatDate(this._get(e,"dateFormat"),o,this._getFormatConfig(e))}}),$.fn.datepicker=function(e){if(!this.length)return this;$.datepicker.initialized||($(document).mousedown($.datepicker._checkExternalClick).find(document.body).append($.datepicker.dpDiv),$.datepicker.initialized=!0);var t=Array.prototype.slice.call(arguments,1);return"string"!=typeof e||"isDisabled"!=e&&"getDate"!=e&&"widget"!=e?"option"==e&&2==arguments.length&&"string"==typeof arguments[1]?$.datepicker["_"+e+"Datepicker"].apply($.datepicker,[this[0]].concat(t)):this.each((function(){"string"==typeof e?$.datepicker["_"+e+"Datepicker"].apply($.datepicker,[this].concat(t)):$.datepicker._attachDatepicker(this,e)})):$.datepicker["_"+e+"Datepicker"].apply($.datepicker,[this[0]].concat(t))},$.datepicker=new Datepicker,$.datepicker.initialized=!1,$.datepicker.uuid=(new Date).getTime(),$.datepicker.version="1.9.2",window["DP_jQuery_"+dpuuid]=$}(jQuery),function(e,t){var i="ui-dialog ui-widget ui-widget-content ui-corner-all ",n={buttons:!0,height:!0,maxHeight:!0,maxWidth:!0,minHeight:!0,minWidth:!0,width:!0},o={maxHeight:!0,maxWidth:!0,minHeight:!0,minWidth:!0};e.widget("ui.dialog",{version:"1.9.2",options:{autoOpen:!0,buttons:{},closeOnEscape:!0,closeText:"close",dialogClass:"",draggable:!0,hide:null,height:"auto",maxHeight:!1,maxWidth:!1,minHeight:150,minWidth:150,modal:!1,position:{my:"center",at:"center",of:window,collision:"fit",using:function(t){var i=e(this).css(t).offset().top;i<0&&e(this).css("top",t.top-i)}},resizable:!0,show:null,stack:!0,title:"",width:300,zIndex:1e3},_create:function(){this.originalTitle=this.element.attr("title"),"string"!==typeof this.originalTitle&&(this.originalTitle=""),this.oldPosition={parent:this.element.parent(),index:this.element.parent().children().index(this.element)},this.options.title=this.options.title||this.originalTitle;var t,n,o,s,a,r=this,l=this.options,c=l.title||"&#160;";t=(this.uiDialog=e("<div>")).addClass(i+l.dialogClass).css({display:"none",outline:0,zIndex:l.zIndex}).attr("tabIndex",-1).keydown((function(t){l.closeOnEscape&&!t.isDefaultPrevented()&&t.keyCode&&t.keyCode===e.ui.keyCode.ESCAPE&&(r.close(t),t.preventDefault())})).mousedown((function(e){r.moveToTop(!1,e)})).appendTo("body"),this.element.show().removeAttr("title").addClass("ui-dialog-content ui-widget-content").appendTo(t),n=(this.uiDialogTitlebar=e("<div>")).addClass("ui-dialog-titlebar  ui-widget-header  ui-corner-all  ui-helper-clearfix").bind("mousedown",(function(){t.focus()})).prependTo(t),o=e("<a href='#'></a>").addClass("ui-dialog-titlebar-close  ui-corner-all").attr("role","button").click((function(e){e.preventDefault(),r.close(e)})).appendTo(n),(this.uiDialogTitlebarCloseText=e("<span>")).addClass("ui-icon ui-icon-closethick").text(l.closeText).appendTo(o),s=e("<span>").uniqueId().addClass("ui-dialog-title").html(c).prependTo(n),a=(this.uiDialogButtonPane=e("<div>")).addClass("ui-dialog-buttonpane ui-widget-content ui-helper-clearfix"),(this.uiButtonSet=e("<div>")).addClass("ui-dialog-buttonset").appendTo(a),t.attr({role:"dialog","aria-labelledby":s.attr("id")}),n.find("*").add(n).disableSelection(),this._hoverable(o),this._focusable(o),l.draggable&&e.fn.draggable&&this._makeDraggable(),l.resizable&&e.fn.resizable&&this._makeResizable(),this._createButtons(l.buttons),this._isOpen=!1,e.fn.bgiframe&&t.bgiframe(),this._on(t,{keydown:function(i){if(l.modal&&i.keyCode===e.ui.keyCode.TAB){var n=e(":tabbable",t),o=n.filter(":first"),s=n.filter(":last");return i.target!==s[0]||i.shiftKey?i.target===o[0]&&i.shiftKey?(s.focus(1),!1):void 0:(o.focus(1),!1)}}})},_init:function(){this.options.autoOpen&&this.open()},_destroy:function(){var e,t=this.oldPosition;this.overlay&&this.overlay.destroy(),this.uiDialog.hide(),this.element.removeClass("ui-dialog-content ui-widget-content").hide().appendTo("body"),this.uiDialog.remove(),this.originalTitle&&this.element.attr("title",this.originalTitle),e=t.parent.children().eq(t.index),e.length&&e[0]!==this.element[0]?e.before(this.element):t.parent.append(this.element)},widget:function(){return this.uiDialog},close:function(t){var i,n,o=this;if(this._isOpen&&!1!==this._trigger("beforeClose",t))return this._isOpen=!1,this.overlay&&this.overlay.destroy(),this.options.hide?this._hide(this.uiDialog,this.options.hide,(function(){o._trigger("close",t)})):(this.uiDialog.hide(),this._trigger("close",t)),e.ui.dialog.overlay.resize(),this.options.modal&&(i=0,e(".ui-dialog").each((function(){this!==o.uiDialog[0]&&(n=e(this).css("z-index"),isNaN(n)||(i=Math.max(i,n)))})),e.ui.dialog.maxZ=i),this},isOpen:function(){return this._isOpen},moveToTop:function(t,i){var n,o=this.options;return o.modal&&!t||!o.stack&&!o.modal?this._trigger("focus",i):(o.zIndex>e.ui.dialog.maxZ&&(e.ui.dialog.maxZ=o.zIndex),this.overlay&&(e.ui.dialog.maxZ+=1,e.ui.dialog.overlay.maxZ=e.ui.dialog.maxZ,this.overlay.$el.css("z-index",e.ui.dialog.overlay.maxZ)),n={scrollTop:this.element.scrollTop(),scrollLeft:this.element.scrollLeft()},e.ui.dialog.maxZ+=1,this.uiDialog.css("z-index",e.ui.dialog.maxZ),this.element.attr(n),this._trigger("focus",i),this)},open:function(){if(!this._isOpen){var t,i=this.options,n=this.uiDialog;return this._size(),this._position(i.position),n.show(i.show),this.overlay=i.modal?new e.ui.dialog.overlay(this):null,this.moveToTop(!0),t=this.element.find(":tabbable"),t.length||(t=this.uiDialogButtonPane.find(":tabbable"),t.length||(t=n)),t.eq(0).focus(),this._isOpen=!0,this._trigger("open"),this}},_createButtons:function(t){var i=this,n=!1;this.uiDialogButtonPane.remove(),this.uiButtonSet.empty(),"object"===typeof t&&null!==t&&e.each(t,(function(){return!(n=!0)})),n?(e.each(t,(function(t,n){var o,s;n=e.isFunction(n)?{click:n,text:t}:n,n=e.extend({type:"button"},n),s=n.click,n.click=function(){s.apply(i.element[0],arguments)},o=e("<button></button>",n).appendTo(i.uiButtonSet),e.fn.button&&o.button()})),this.uiDialog.addClass("ui-dialog-buttons"),this.uiDialogButtonPane.appendTo(this.uiDialog)):this.uiDialog.removeClass("ui-dialog-buttons")},_makeDraggable:function(){var t=this,i=this.options;function n(e){return{position:e.position,offset:e.offset}}this.uiDialog.draggable({cancel:".ui-dialog-content, .ui-dialog-titlebar-close",handle:".ui-dialog-titlebar",containment:"document",start:function(i,o){e(this).addClass("ui-dialog-dragging"),t._trigger("dragStart",i,n(o))},drag:function(e,i){t._trigger("drag",e,n(i))},stop:function(o,s){i.position=[s.position.left-t.document.scrollLeft(),s.position.top-t.document.scrollTop()],e(this).removeClass("ui-dialog-dragging"),t._trigger("dragStop",o,n(s)),e.ui.dialog.overlay.resize()}})},_makeResizable:function(i){i=i===t?this.options.resizable:i;var n=this,o=this.options,s=this.uiDialog.css("position"),a="string"===typeof i?i:"n,e,s,w,se,sw,ne,nw";function r(e){return{originalPosition:e.originalPosition,originalSize:e.originalSize,position:e.position,size:e.size}}this.uiDialog.resizable({cancel:".ui-dialog-content",containment:"document",alsoResize:this.element,maxWidth:o.maxWidth,maxHeight:o.maxHeight,minWidth:o.minWidth,minHeight:this._minHeight(),handles:a,start:function(t,i){e(this).addClass("ui-dialog-resizing"),n._trigger("resizeStart",t,r(i))},resize:function(e,t){n._trigger("resize",e,r(t))},stop:function(t,i){e(this).removeClass("ui-dialog-resizing"),o.height=e(this).height(),o.width=e(this).width(),n._trigger("resizeStop",t,r(i)),e.ui.dialog.overlay.resize()}}).css("position",s).find(".ui-resizable-se").addClass("ui-icon ui-icon-grip-diagonal-se")},_minHeight:function(){var e=this.options;return"auto"===e.height?e.minHeight:Math.min(e.minHeight,e.height)},_position:function(t){var i,n=[],o=[0,0];t?(("string"===typeof t||"object"===typeof t&&"0"in t)&&(n=t.split?t.split(" "):[t[0],t[1]],1===n.length&&(n[1]=n[0]),e.each(["left","top"],(function(e,t){+n[e]===n[e]&&(o[e]=n[e],n[e]=t)})),t={my:n[0]+(o[0]<0?o[0]:"+"+o[0])+" "+n[1]+(o[1]<0?o[1]:"+"+o[1]),at:n.join(" ")}),t=e.extend({},e.ui.dialog.prototype.options.position,t)):t=e.ui.dialog.prototype.options.position,i=this.uiDialog.is(":visible"),i||this.uiDialog.show(),this.uiDialog.position(t),i||this.uiDialog.hide()},_setOptions:function(t){var i=this,s={},a=!1;e.each(t,(function(e,t){i._setOption(e,t),e in n&&(a=!0),e in o&&(s[e]=t)})),a&&this._size(),this.uiDialog.is(":data(resizable)")&&this.uiDialog.resizable("option",s)},_setOption:function(t,n){var o,s,a=this.uiDialog;switch(t){case"buttons":this._createButtons(n);break;case"closeText":this.uiDialogTitlebarCloseText.text(""+n);break;case"dialogClass":a.removeClass(this.options.dialogClass).addClass(i+n);break;case"disabled":n?a.addClass("ui-dialog-disabled"):a.removeClass("ui-dialog-disabled");break;case"draggable":o=a.is(":data(draggable)"),o&&!n&&a.draggable("destroy"),!o&&n&&this._makeDraggable();break;case"position":this._position(n);break;case"resizable":s=a.is(":data(resizable)"),s&&!n&&a.resizable("destroy"),s&&"string"===typeof n&&a.resizable("option","handles",n),s||!1===n||this._makeResizable(n);break;case"title":e(".ui-dialog-title",this.uiDialogTitlebar).html(""+(n||"&#160;"));break}this._super(t,n)},_size:function(){var t,i,n,o=this.options,s=this.uiDialog.is(":visible");this.element.show().css({width:"auto",minHeight:0,height:0}),o.minWidth>o.width&&(o.width=o.minWidth),t=this.uiDialog.css({height:"auto",width:o.width}).outerHeight(),i=Math.max(0,o.minHeight-t),"auto"===o.height?e.support.minHeight?this.element.css({minHeight:i,height:"auto"}):(this.uiDialog.show(),n=this.element.css("height","auto").height(),s||this.uiDialog.hide(),this.element.height(Math.max(n,i))):this.element.height(Math.max(o.height-t,0)),this.uiDialog.is(":data(resizable)")&&this.uiDialog.resizable("option","minHeight",this._minHeight())}}),e.extend(e.ui.dialog,{uuid:0,maxZ:0,getTitleId:function(e){var t=e.attr("id");return t||(this.uuid+=1,t=this.uuid),"ui-dialog-title-"+t},overlay:function(t){this.$el=e.ui.dialog.overlay.create(t)}}),e.extend(e.ui.dialog.overlay,{instances:[],oldInstances:[],maxZ:0,events:e.map("focus,mousedown,mouseup,keydown,keypress,click".split(","),(function(e){return e+".dialog-overlay"})).join(" "),create:function(t){0===this.instances.length&&(setTimeout((function(){e.ui.dialog.overlay.instances.length&&e(document).bind(e.ui.dialog.overlay.events,(function(t){if(e(t.target).zIndex()<e.ui.dialog.overlay.maxZ)return!1}))}),1),e(window).bind("resize.dialog-overlay",e.ui.dialog.overlay.resize));var i=this.oldInstances.pop()||e("<div>").addClass("ui-widget-overlay");return e(document).bind("keydown.dialog-overlay",(function(n){var o=e.ui.dialog.overlay.instances;0!==o.length&&o[o.length-1]===i&&t.options.closeOnEscape&&!n.isDefaultPrevented()&&n.keyCode&&n.keyCode===e.ui.keyCode.ESCAPE&&(t.close(n),n.preventDefault())})),i.appendTo(document.body).css({width:this.width(),height:this.height()}),e.fn.bgiframe&&i.bgiframe(),this.instances.push(i),i},destroy:function(t){var i=e.inArray(t,this.instances),n=0;-1!==i&&this.oldInstances.push(this.instances.splice(i,1)[0]),0===this.instances.length&&e([document,window]).unbind(".dialog-overlay"),t.height(0).width(0).remove(),e.each(this.instances,(function(){n=Math.max(n,this.css("z-index"))})),this.maxZ=n},height:function(){var t,i;return e.ui.ie?(t=Math.max(document.documentElement.scrollHeight,document.body.scrollHeight),i=Math.max(document.documentElement.offsetHeight,document.body.offsetHeight),t<i?e(window).height()+"px":t+"px"):e(document).height()+"px"},width:function(){var t,i;return e.ui.ie?(t=Math.max(document.documentElement.scrollWidth,document.body.scrollWidth),i=Math.max(document.documentElement.offsetWidth,document.body.offsetWidth),t<i?e(window).width()+"px":t+"px"):e(document).width()+"px"},resize:function(){var t=e([]);e.each(e.ui.dialog.overlay.instances,(function(){t=t.add(this)})),t.css({width:0,height:0}).css({width:e.ui.dialog.overlay.width(),height:e.ui.dialog.overlay.height()})}}),e.extend(e.ui.dialog.overlay.prototype,{destroy:function(){e.ui.dialog.overlay.destroy(this.$el)}})}(jQuery),function(e,t){var i=/up|down|vertical/,n=/up|left|vertical|horizontal/;e.effects.effect.blind=function(t,o){var s,a,r,l=e(this),c=["position","top","bottom","left","right","height","width"],h=e.effects.setMode(l,t.mode||"hide"),u=t.direction||"up",d=i.test(u),p=d?"height":"width",f=d?"top":"left",g=n.test(u),m={},v="show"===h;l.parent().is(".ui-effects-wrapper")?e.effects.save(l.parent(),c):e.effects.save(l,c),l.show(),s=e.effects.createWrapper(l).css({overflow:"hidden"}),a=s[p](),r=parseFloat(s.css(f))||0,m[p]=v?a:0,g||(l.css(d?"bottom":"right",0).css(d?"top":"left","auto").css({position:"absolute"}),m[f]=v?r:a+r),v&&(s.css(p,0),g||s.css(f,r+a)),s.animate(m,{duration:t.duration,easing:t.easing,queue:!1,complete:function(){"hide"===h&&l.hide(),e.effects.restore(l,c),e.effects.removeWrapper(l),o()}})}}(jQuery),function(e,t){e.effects.effect.bounce=function(t,i){var n,o,s,a=e(this),r=["position","top","bottom","left","right","height","width"],l=e.effects.setMode(a,t.mode||"effect"),c="hide"===l,h="show"===l,u=t.direction||"up",d=t.distance,p=t.times||5,f=2*p+(h||c?1:0),g=t.duration/f,m=t.easing,v="up"===u||"down"===u?"top":"left",w="up"===u||"left"===u,y=a.queue(),b=y.length;for((h||c)&&r.push("opacity"),e.effects.save(a,r),a.show(),e.effects.createWrapper(a),d||(d=a["top"===v?"outerHeight":"outerWidth"]()/3),h&&(s={opacity:1},s[v]=0,a.css("opacity",0).css(v,w?2*-d:2*d).animate(s,g,m)),c&&(d/=Math.pow(2,p-1)),s={},s[v]=0,n=0;n<p;n++)o={},o[v]=(w?"-=":"+=")+d,a.animate(o,g,m).animate(s,g,m),d=c?2*d:d/2;c&&(o={opacity:0},o[v]=(w?"-=":"+=")+d,a.animate(o,g,m)),a.queue((function(){c&&a.hide(),e.effects.restore(a,r),e.effects.removeWrapper(a),i()})),b>1&&y.splice.apply(y,[1,0].concat(y.splice(b,f+1))),a.dequeue()}}(jQuery),function(e,t){e.effects.effect.clip=function(t,i){var n,o,s,a=e(this),r=["position","top","bottom","left","right","height","width"],l=e.effects.setMode(a,t.mode||"hide"),c="show"===l,h=t.direction||"vertical",u="vertical"===h,d=u?"height":"width",p=u?"top":"left",f={};e.effects.save(a,r),a.show(),n=e.effects.createWrapper(a).css({overflow:"hidden"}),o="IMG"===a[0].tagName?n:a,s=o[d](),c&&(o.css(d,0),o.css(p,s/2)),f[d]=c?s:0,f[p]=c?0:s/2,o.animate(f,{queue:!1,duration:t.duration,easing:t.easing,complete:function(){c||a.hide(),e.effects.restore(a,r),e.effects.removeWrapper(a),i()}})}}(jQuery),function(e,t){e.effects.effect.drop=function(t,i){var n,o=e(this),s=["position","top","bottom","left","right","opacity","height","width"],a=e.effects.setMode(o,t.mode||"hide"),r="show"===a,l=t.direction||"left",c="up"===l||"down"===l?"top":"left",h="up"===l||"left"===l?"pos":"neg",u={opacity:r?1:0};e.effects.save(o,s),o.show(),e.effects.createWrapper(o),n=t.distance||o["top"===c?"outerHeight":"outerWidth"](!0)/2,r&&o.css("opacity",0).css(c,"pos"===h?-n:n),u[c]=(r?"pos"===h?"+=":"-=":"pos"===h?"-=":"+=")+n,o.animate(u,{queue:!1,duration:t.duration,easing:t.easing,complete:function(){"hide"===a&&o.hide(),e.effects.restore(o,s),e.effects.removeWrapper(o),i()}})}}(jQuery),function(e,t){e.effects.effect.explode=function(t,i){var n,o,s,a,r,l,c=t.pieces?Math.round(Math.sqrt(t.pieces)):3,h=c,u=e(this),d=e.effects.setMode(u,t.mode||"hide"),p="show"===d,f=u.show().css("visibility","hidden").offset(),g=Math.ceil(u.outerWidth()/h),m=Math.ceil(u.outerHeight()/c),v=[];function w(){v.push(this),v.length===c*h&&y()}for(n=0;n<c;n++)for(a=f.top+n*m,l=n-(c-1)/2,o=0;o<h;o++)s=f.left+o*g,r=o-(h-1)/2,u.clone().appendTo("body").wrap("<div></div>").css({position:"absolute",visibility:"visible",left:-o*g,top:-n*m}).parent().addClass("ui-effects-explode").css({position:"absolute",overflow:"hidden",width:g,height:m,left:s+(p?r*g:0),top:a+(p?l*m:0),opacity:p?0:1}).animate({left:s+(p?0:r*g),top:a+(p?0:l*m),opacity:p?1:0},t.duration||500,t.easing,w);function y(){u.css({visibility:"visible"}),e(v).remove(),p||u.hide(),i()}}}(jQuery),function(e,t){e.effects.effect.fade=function(t,i){var n=e(this),o=e.effects.setMode(n,t.mode||"toggle");n.animate({opacity:o},{queue:!1,duration:t.duration,easing:t.easing,complete:i})}}(jQuery),function(e,t){e.effects.effect.fold=function(t,i){var n,o,s=e(this),a=["position","top","bottom","left","right","height","width"],r=e.effects.setMode(s,t.mode||"hide"),l="show"===r,c="hide"===r,h=t.size||15,u=/([0-9]+)%/.exec(h),d=!!t.horizFirst,p=l!==d,f=p?["width","height"]:["height","width"],g=t.duration/2,m={},v={};e.effects.save(s,a),s.show(),n=e.effects.createWrapper(s).css({overflow:"hidden"}),o=p?[n.width(),n.height()]:[n.height(),n.width()],u&&(h=parseInt(u[1],10)/100*o[c?0:1]),l&&n.css(d?{height:0,width:h}:{height:h,width:0}),m[f[0]]=l?o[0]:h,v[f[1]]=l?o[1]:0,n.animate(m,g,t.easing).animate(v,g,t.easing,(function(){c&&s.hide(),e.effects.restore(s,a),e.effects.removeWrapper(s),i()}))}}(jQuery),function(e,t){e.effects.effect.highlight=function(t,i){var n=e(this),o=["backgroundImage","backgroundColor","opacity"],s=e.effects.setMode(n,t.mode||"show"),a={backgroundColor:n.css("backgroundColor")};"hide"===s&&(a.opacity=0),e.effects.save(n,o),n.show().css({backgroundImage:"none",backgroundColor:t.color||"#ffff99"}).animate(a,{queue:!1,duration:t.duration,easing:t.easing,complete:function(){"hide"===s&&n.hide(),e.effects.restore(n,o),i()}})}}(jQuery),function(e,t){e.effects.effect.pulsate=function(t,i){var n,o=e(this),s=e.effects.setMode(o,t.mode||"show"),a="show"===s,r="hide"===s,l=a||"hide"===s,c=2*(t.times||5)+(l?1:0),h=t.duration/c,u=0,d=o.queue(),p=d.length;for(!a&&o.is(":visible")||(o.css("opacity",0).show(),u=1),n=1;n<c;n++)o.animate({opacity:u},h,t.easing),u=1-u;o.animate({opacity:u},h,t.easing),o.queue((function(){r&&o.hide(),i()})),p>1&&d.splice.apply(d,[1,0].concat(d.splice(p,c+1))),o.dequeue()}}(jQuery),function(e,t){e.effects.effect.puff=function(t,i){var n=e(this),o=e.effects.setMode(n,t.mode||"hide"),s="hide"===o,a=parseInt(t.percent,10)||150,r=a/100,l={height:n.height(),width:n.width(),outerHeight:n.outerHeight(),outerWidth:n.outerWidth()};e.extend(t,{effect:"scale",queue:!1,fade:!0,mode:o,complete:i,percent:s?a:100,from:s?l:{height:l.height*r,width:l.width*r,outerHeight:l.outerHeight*r,outerWidth:l.outerWidth*r}}),n.effect(t)},e.effects.effect.scale=function(t,i){var n=e(this),o=e.extend(!0,{},t),s=e.effects.setMode(n,t.mode||"effect"),a=parseInt(t.percent,10)||(0===parseInt(t.percent,10)||"hide"===s?0:100),r=t.direction||"both",l=t.origin,c={height:n.height(),width:n.width(),outerHeight:n.outerHeight(),outerWidth:n.outerWidth()},h={y:"horizontal"!==r?a/100:1,x:"vertical"!==r?a/100:1};o.effect="size",o.queue=!1,o.complete=i,"effect"!==s&&(o.origin=l||["middle","center"],o.restore=!0),o.from=t.from||("show"===s?{height:0,width:0,outerHeight:0,outerWidth:0}:c),o.to={height:c.height*h.y,width:c.width*h.x,outerHeight:c.outerHeight*h.y,outerWidth:c.outerWidth*h.x},o.fade&&("show"===s&&(o.from.opacity=0,o.to.opacity=1),"hide"===s&&(o.from.opacity=1,o.to.opacity=0)),n.effect(o)},e.effects.effect.size=function(t,i){var n,o,s,a=e(this),r=["position","top","bottom","left","right","width","height","overflow","opacity"],l=["position","top","bottom","left","right","overflow","opacity"],c=["width","height","overflow"],h=["fontSize"],u=["borderTopWidth","borderBottomWidth","paddingTop","paddingBottom"],d=["borderLeftWidth","borderRightWidth","paddingLeft","paddingRight"],p=e.effects.setMode(a,t.mode||"effect"),f=t.restore||"effect"!==p,g=t.scale||"both",m=t.origin||["middle","center"],v=a.css("position"),w=f?r:l,y={height:0,width:0,outerHeight:0,outerWidth:0};"show"===p&&a.show(),n={height:a.height(),width:a.width(),outerHeight:a.outerHeight(),outerWidth:a.outerWidth()},"toggle"===t.mode&&"show"===p?(a.from=t.to||y,a.to=t.from||n):(a.from=t.from||("show"===p?y:n),a.to=t.to||("hide"===p?y:n)),s={from:{y:a.from.height/n.height,x:a.from.width/n.width},to:{y:a.to.height/n.height,x:a.to.width/n.width}},"box"!==g&&"both"!==g||(s.from.y!==s.to.y&&(w=w.concat(u),a.from=e.effects.setTransition(a,u,s.from.y,a.from),a.to=e.effects.setTransition(a,u,s.to.y,a.to)),s.from.x!==s.to.x&&(w=w.concat(d),a.from=e.effects.setTransition(a,d,s.from.x,a.from),a.to=e.effects.setTransition(a,d,s.to.x,a.to))),"content"!==g&&"both"!==g||s.from.y!==s.to.y&&(w=w.concat(h).concat(c),a.from=e.effects.setTransition(a,h,s.from.y,a.from),a.to=e.effects.setTransition(a,h,s.to.y,a.to)),e.effects.save(a,w),a.show(),e.effects.createWrapper(a),a.css("overflow","hidden").css(a.from),m&&(o=e.effects.getBaseline(m,n),a.from.top=(n.outerHeight-a.outerHeight())*o.y,a.from.left=(n.outerWidth-a.outerWidth())*o.x,a.to.top=(n.outerHeight-a.to.outerHeight)*o.y,a.to.left=(n.outerWidth-a.to.outerWidth)*o.x),a.css(a.from),"content"!==g&&"both"!==g||(u=u.concat(["marginTop","marginBottom"]).concat(h),d=d.concat(["marginLeft","marginRight"]),c=r.concat(u).concat(d),a.find("*[width]").each((function(){var i=e(this),n={height:i.height(),width:i.width(),outerHeight:i.outerHeight(),outerWidth:i.outerWidth()};f&&e.effects.save(i,c),i.from={height:n.height*s.from.y,width:n.width*s.from.x,outerHeight:n.outerHeight*s.from.y,outerWidth:n.outerWidth*s.from.x},i.to={height:n.height*s.to.y,width:n.width*s.to.x,outerHeight:n.height*s.to.y,outerWidth:n.width*s.to.x},s.from.y!==s.to.y&&(i.from=e.effects.setTransition(i,u,s.from.y,i.from),i.to=e.effects.setTransition(i,u,s.to.y,i.to)),s.from.x!==s.to.x&&(i.from=e.effects.setTransition(i,d,s.from.x,i.from),i.to=e.effects.setTransition(i,d,s.to.x,i.to)),i.css(i.from),i.animate(i.to,t.duration,t.easing,(function(){f&&e.effects.restore(i,c)}))}))),a.animate(a.to,{queue:!1,duration:t.duration,easing:t.easing,complete:function(){0===a.to.opacity&&a.css("opacity",a.from.opacity),"hide"===p&&a.hide(),e.effects.restore(a,w),f||("static"===v?a.css({position:"relative",top:a.to.top,left:a.to.left}):e.each(["top","left"],(function(e,t){a.css(t,(function(t,i){var n=parseInt(i,10),o=e?a.to.left:a.to.top;return"auto"===i?o+"px":n+o+"px"}))}))),e.effects.removeWrapper(a),i()}})}}(jQuery),function(e,t){e.effects.effect.shake=function(t,i){var n,o=e(this),s=["position","top","bottom","left","right","height","width"],a=e.effects.setMode(o,t.mode||"effect"),r=t.direction||"left",l=t.distance||20,c=t.times||3,h=2*c+1,u=Math.round(t.duration/h),d="up"===r||"down"===r?"top":"left",p="up"===r||"left"===r,f={},g={},m={},v=o.queue(),w=v.length;for(e.effects.save(o,s),o.show(),e.effects.createWrapper(o),f[d]=(p?"-=":"+=")+l,g[d]=(p?"+=":"-=")+2*l,m[d]=(p?"-=":"+=")+2*l,o.animate(f,u,t.easing),n=1;n<c;n++)o.animate(g,u,t.easing).animate(m,u,t.easing);o.animate(g,u,t.easing).animate(f,u/2,t.easing).queue((function(){"hide"===a&&o.hide(),e.effects.restore(o,s),e.effects.removeWrapper(o),i()})),w>1&&v.splice.apply(v,[1,0].concat(v.splice(w,h+1))),o.dequeue()}}(jQuery),function(e,t){e.effects.effect.slide=function(t,i){var n,o=e(this),s=["position","top","bottom","left","right","width","height"],a=e.effects.setMode(o,t.mode||"show"),r="show"===a,l=t.direction||"left",c="up"===l||"down"===l?"top":"left",h="up"===l||"left"===l,u={};e.effects.save(o,s),o.show(),n=t.distance||o["top"===c?"outerHeight":"outerWidth"](!0),e.effects.createWrapper(o).css({overflow:"hidden"}),r&&o.css(c,h?isNaN(n)?"-"+n:-n:n),u[c]=(r?h?"+=":"-=":h?"-=":"+=")+n,o.animate(u,{queue:!1,duration:t.duration,easing:t.easing,complete:function(){"hide"===a&&o.hide(),e.effects.restore(o,s),e.effects.removeWrapper(o),i()}})}}(jQuery),function(e,t){e.effects.effect.transfer=function(t,i){var n=e(this),o=e(t.to),s="fixed"===o.css("position"),a=e("body"),r=s?a.scrollTop():0,l=s?a.scrollLeft():0,c=o.offset(),h={top:c.top-r,left:c.left-l,height:o.innerHeight(),width:o.innerWidth()},u=n.offset(),d=e('<div class="ui-effects-transfer"></div>').appendTo(document.body).addClass(t.className).css({top:u.top-r,left:u.left-l,height:n.innerHeight(),width:n.innerWidth(),position:s?"fixed":"absolute"}).animate(h,t.duration,t.easing,(function(){d.remove(),i()}))}}(jQuery),function(e,t){var i=!1;e.widget("ui.menu",{version:"1.9.2",defaultElement:"<ul>",delay:300,options:{icons:{submenu:"ui-icon-carat-1-e"},menus:"ul",position:{my:"left top",at:"right top"},role:"menu",blur:null,focus:null,select:null},_create:function(){this.activeMenu=this.element,this.element.uniqueId().addClass("ui-menu ui-widget ui-widget-content ui-corner-all").toggleClass("ui-menu-icons",!!this.element.find(".ui-icon").length).attr({role:this.options.role,tabIndex:0}).bind("click"+this.eventNamespace,e.proxy((function(e){this.options.disabled&&e.preventDefault()}),this)),this.options.disabled&&this.element.addClass("ui-state-disabled").attr("aria-disabled","true"),this._on({"mousedown .ui-menu-item > a":function(e){e.preventDefault()},"click .ui-state-disabled > a":function(e){e.preventDefault()},"click .ui-menu-item:has(a)":function(t){var n=e(t.target).closest(".ui-menu-item");!i&&n.not(".ui-state-disabled").length&&(i=!0,this.select(t),n.has(".ui-menu").length?this.expand(t):this.element.is(":focus")||(this.element.trigger("focus",[!0]),this.active&&1===this.active.parents(".ui-menu").length&&clearTimeout(this.timer)))},"mouseenter .ui-menu-item":function(t){var i=e(t.currentTarget);i.siblings().children(".ui-state-active").removeClass("ui-state-active"),this.focus(t,i)},mouseleave:"collapseAll","mouseleave .ui-menu":"collapseAll",focus:function(e,t){var i=this.active||this.element.children(".ui-menu-item").eq(0);t||this.focus(e,i)},blur:function(t){this._delay((function(){e.contains(this.element[0],this.document[0].activeElement)||this.collapseAll(t)}))},keydown:"_keydown"}),this.refresh(),this._on(this.document,{click:function(t){e(t.target).closest(".ui-menu").length||this.collapseAll(t),i=!1}})},_destroy:function(){this.element.removeAttr("aria-activedescendant").find(".ui-menu").andSelf().removeClass("ui-menu ui-widget ui-widget-content ui-corner-all ui-menu-icons").removeAttr("role").removeAttr("tabIndex").removeAttr("aria-labelledby").removeAttr("aria-expanded").removeAttr("aria-hidden").removeAttr("aria-disabled").removeUniqueId().show(),this.element.find(".ui-menu-item").removeClass("ui-menu-item").removeAttr("role").removeAttr("aria-disabled").children("a").removeUniqueId().removeClass("ui-corner-all ui-state-hover").removeAttr("tabIndex").removeAttr("role").removeAttr("aria-haspopup").children().each((function(){var t=e(this);t.data("ui-menu-submenu-carat")&&t.remove()})),this.element.find(".ui-menu-divider").removeClass("ui-menu-divider ui-widget-content")},_keydown:function(t){var i,n,o,s,a,r=!0;function l(e){return e.replace(/[\-\[\]{}()*+?.,\\\^$|#\s]/g,"\\$&")}switch(t.keyCode){case e.ui.keyCode.PAGE_UP:this.previousPage(t);break;case e.ui.keyCode.PAGE_DOWN:this.nextPage(t);break;case e.ui.keyCode.HOME:this._move("first","first",t);break;case e.ui.keyCode.END:this._move("last","last",t);break;case e.ui.keyCode.UP:this.previous(t);break;case e.ui.keyCode.DOWN:this.next(t);break;case e.ui.keyCode.LEFT:this.collapse(t);break;case e.ui.keyCode.RIGHT:this.active&&!this.active.is(".ui-state-disabled")&&this.expand(t);break;case e.ui.keyCode.ENTER:case e.ui.keyCode.SPACE:this._activate(t);break;case e.ui.keyCode.ESCAPE:this.collapse(t);break;default:r=!1,n=this.previousFilter||"",o=String.fromCharCode(t.keyCode),s=!1,clearTimeout(this.filterTimer),o===n?s=!0:o=n+o,a=new RegExp("^"+l(o),"i"),i=this.activeMenu.children(".ui-menu-item").filter((function(){return a.test(e(this).children("a").text())})),i=s&&-1!==i.index(this.active.next())?this.active.nextAll(".ui-menu-item"):i,i.length||(o=String.fromCharCode(t.keyCode),a=new RegExp("^"+l(o),"i"),i=this.activeMenu.children(".ui-menu-item").filter((function(){return a.test(e(this).children("a").text())}))),i.length?(this.focus(t,i),i.length>1?(this.previousFilter=o,this.filterTimer=this._delay((function(){delete this.previousFilter}),1e3)):delete this.previousFilter):delete this.previousFilter}r&&t.preventDefault()},_activate:function(e){this.active.is(".ui-state-disabled")||(this.active.children("a[aria-haspopup='true']").length?this.expand(e):this.select(e))},refresh:function(){var t,i=this.options.icons.submenu,n=this.element.find(this.options.menus);n.filter(":not(.ui-menu)").addClass("ui-menu ui-widget ui-widget-content ui-corner-all").hide().attr({role:this.options.role,"aria-hidden":"true","aria-expanded":"false"}).each((function(){var t=e(this),n=t.prev("a"),o=e("<span>").addClass("ui-menu-icon ui-icon "+i).data("ui-menu-submenu-carat",!0);n.attr("aria-haspopup","true").prepend(o),t.attr("aria-labelledby",n.attr("id"))})),t=n.add(this.element),t.children(":not(.ui-menu-item):has(a)").addClass("ui-menu-item").attr("role","presentation").children("a").uniqueId().addClass("ui-corner-all").attr({tabIndex:-1,role:this._itemRole()}),t.children(":not(.ui-menu-item)").each((function(){var t=e(this);/[^\-—–\s]/.test(t.text())||t.addClass("ui-widget-content ui-menu-divider")})),t.children(".ui-state-disabled").attr("aria-disabled","true"),this.active&&!e.contains(this.element[0],this.active[0])&&this.blur()},_itemRole:function(){return{menu:"menuitem",listbox:"option"}[this.options.role]},focus:function(e,t){var i,n;this.blur(e,e&&"focus"===e.type),this._scrollIntoView(t),this.active=t.first(),n=this.active.children("a").addClass("ui-state-focus"),this.options.role&&this.element.attr("aria-activedescendant",n.attr("id")),this.active.parent().closest(".ui-menu-item").children("a:first").addClass("ui-state-active"),e&&"keydown"===e.type?this._close():this.timer=this._delay((function(){this._close()}),this.delay),i=t.children(".ui-menu"),i.length&&/^mouse/.test(e.type)&&this._startOpening(i),this.activeMenu=t.parent(),this._trigger("focus",e,{item:t})},_scrollIntoView:function(t){var i,n,o,s,a,r;this._hasScroll()&&(i=parseFloat(e.css(this.activeMenu[0],"borderTopWidth"))||0,n=parseFloat(e.css(this.activeMenu[0],"paddingTop"))||0,o=t.offset().top-this.activeMenu.offset().top-i-n,s=this.activeMenu.scrollTop(),a=this.activeMenu.height(),r=t.height(),o<0?this.activeMenu.scrollTop(s+o):o+r>a&&this.activeMenu.scrollTop(s+o-a+r))},blur:function(e,t){t||clearTimeout(this.timer),this.active&&(this.active.children("a").removeClass("ui-state-focus"),this.active=null,this._trigger("blur",e,{item:this.active}))},_startOpening:function(e){clearTimeout(this.timer),"true"===e.attr("aria-hidden")&&(this.timer=this._delay((function(){this._close(),this._open(e)}),this.delay))},_open:function(t){var i=e.extend({of:this.active},this.options.position);clearTimeout(this.timer),this.element.find(".ui-menu").not(t.parents(".ui-menu")).hide().attr("aria-hidden","true"),t.show().removeAttr("aria-hidden").attr("aria-expanded","true").position(i)},collapseAll:function(t,i){clearTimeout(this.timer),this.timer=this._delay((function(){var n=i?this.element:e(t&&t.target).closest(this.element.find(".ui-menu"));n.length||(n=this.element),this._close(n),this.blur(t),this.activeMenu=n}),this.delay)},_close:function(e){e||(e=this.active?this.active.parent():this.element),e.find(".ui-menu").hide().attr("aria-hidden","true").attr("aria-expanded","false").end().find("a.ui-state-active").removeClass("ui-state-active")},collapse:function(e){var t=this.active&&this.active.parent().closest(".ui-menu-item",this.element);t&&t.length&&(this._close(),this.focus(e,t))},expand:function(e){var t=this.active&&this.active.children(".ui-menu ").children(".ui-menu-item").first();t&&t.length&&(this._open(t.parent()),this._delay((function(){this.focus(e,t)})))},next:function(e){this._move("next","first",e)},previous:function(e){this._move("prev","last",e)},isFirstItem:function(){return this.active&&!this.active.prevAll(".ui-menu-item").length},isLastItem:function(){return this.active&&!this.active.nextAll(".ui-menu-item").length},_move:function(e,t,i){var n;this.active&&(n="first"===e||"last"===e?this.active["first"===e?"prevAll":"nextAll"](".ui-menu-item").eq(-1):this.active[e+"All"](".ui-menu-item").eq(0)),n&&n.length&&this.active||(n=this.activeMenu.children(".ui-menu-item")[t]()),this.focus(i,n)},nextPage:function(t){var i,n,o;this.active?this.isLastItem()||(this._hasScroll()?(n=this.active.offset().top,o=this.element.height(),this.active.nextAll(".ui-menu-item").each((function(){return i=e(this),i.offset().top-n-o<0})),this.focus(t,i)):this.focus(t,this.activeMenu.children(".ui-menu-item")[this.active?"last":"first"]())):this.next(t)},previousPage:function(t){var i,n,o;this.active?this.isFirstItem()||(this._hasScroll()?(n=this.active.offset().top,o=this.element.height(),this.active.prevAll(".ui-menu-item").each((function(){return i=e(this),i.offset().top-n+o>0})),this.focus(t,i)):this.focus(t,this.activeMenu.children(".ui-menu-item").first())):this.next(t)},_hasScroll:function(){return this.element.outerHeight()<this.element.prop("scrollHeight")},select:function(t){this.active=this.active||e(t.target).closest(".ui-menu-item");var i={item:this.active};this.active.has(".ui-menu").length||this.collapseAll(t,!0),this._trigger("select",t,i)}})}(jQuery),function(e,t){e.ui=e.ui||{};var i,n=Math.max,o=Math.abs,s=Math.round,a=/left|center|right/,r=/top|center|bottom/,l=/[\+\-]\d+%?/,c=/^\w+/,h=/%$/,u=e.fn.position;function d(e,t,i){return[parseInt(e[0],10)*(h.test(e[0])?t/100:1),parseInt(e[1],10)*(h.test(e[1])?i/100:1)]}function p(t,i){return parseInt(e.css(t,i),10)||0}e.position={scrollbarWidth:function(){if(i!==t)return i;var n,o,s=e("<div style='display:block;width:50px;height:50px;overflow:hidden;'><div style='height:100px;width:auto;'></div></div>"),a=s.children()[0];return e("body").append(s),n=a.offsetWidth,s.css("overflow","scroll"),o=a.offsetWidth,n===o&&(o=s[0].clientWidth),s.remove(),i=n-o},getScrollInfo:function(t){var i=t.isWindow?"":t.element.css("overflow-x"),n=t.isWindow?"":t.element.css("overflow-y"),o="scroll"===i||"auto"===i&&t.width<t.element[0].scrollWidth,s="scroll"===n||"auto"===n&&t.height<t.element[0].scrollHeight;return{width:o?e.position.scrollbarWidth():0,height:s?e.position.scrollbarWidth():0}},getWithinInfo:function(t){var i=e(t||window),n=e.isWindow(i[0]);return{element:i,isWindow:n,offset:i.offset()||{left:0,top:0},scrollLeft:i.scrollLeft(),scrollTop:i.scrollTop(),width:n?i.width():i.outerWidth(),height:n?i.height():i.outerHeight()}}},e.fn.position=function(t){if(!t||!t.of)return u.apply(this,arguments);t=e.extend({},t);var i,h,f,g,m,v=e(t.of),w=e.position.getWithinInfo(t.within),y=e.position.getScrollInfo(w),b=v[0],C=(t.collision||"flip").split(" "),_={};return 9===b.nodeType?(h=v.width(),f=v.height(),g={top:0,left:0}):e.isWindow(b)?(h=v.width(),f=v.height(),g={top:v.scrollTop(),left:v.scrollLeft()}):b.preventDefault?(t.at="left top",h=f=0,g={top:b.pageY,left:b.pageX}):(h=v.outerWidth(),f=v.outerHeight(),g=v.offset()),m=e.extend({},g),e.each(["my","at"],(function(){var e,i,n=(t[this]||"").split(" ");1===n.length&&(n=a.test(n[0])?n.concat(["center"]):r.test(n[0])?["center"].concat(n):["center","center"]),n[0]=a.test(n[0])?n[0]:"center",n[1]=r.test(n[1])?n[1]:"center",e=l.exec(n[0]),i=l.exec(n[1]),_[this]=[e?e[0]:0,i?i[0]:0],t[this]=[c.exec(n[0])[0],c.exec(n[1])[0]]})),1===C.length&&(C[1]=C[0]),"right"===t.at[0]?m.left+=h:"center"===t.at[0]&&(m.left+=h/2),"bottom"===t.at[1]?m.top+=f:"center"===t.at[1]&&(m.top+=f/2),i=d(_.at,h,f),m.left+=i[0],m.top+=i[1],this.each((function(){var a,r,l=e(this),c=l.outerWidth(),u=l.outerHeight(),b=p(this,"marginLeft"),x=p(this,"marginTop"),k=c+b+p(this,"marginRight")+y.width,S=u+x+p(this,"marginBottom")+y.height,D=e.extend({},m),R=d(_.my,l.outerWidth(),l.outerHeight());"right"===t.my[0]?D.left-=c:"center"===t.my[0]&&(D.left-=c/2),"bottom"===t.my[1]?D.top-=u:"center"===t.my[1]&&(D.top-=u/2),D.left+=R[0],D.top+=R[1],e.support.offsetFractions||(D.left=s(D.left),D.top=s(D.top)),a={marginLeft:b,marginTop:x},e.each(["left","top"],(function(n,o){e.ui.position[C[n]]&&e.ui.position[C[n]][o](D,{targetWidth:h,targetHeight:f,elemWidth:c,elemHeight:u,collisionPosition:a,collisionWidth:k,collisionHeight:S,offset:[i[0]+R[0],i[1]+R[1]],my:t.my,at:t.at,within:w,elem:l})})),e.fn.bgiframe&&l.bgiframe(),t.using&&(r=function(e){var i=g.left-D.left,s=i+h-c,a=g.top-D.top,r=a+f-u,d={target:{element:v,left:g.left,top:g.top,width:h,height:f},element:{element:l,left:D.left,top:D.top,width:c,height:u},horizontal:s<0?"left":i>0?"right":"center",vertical:r<0?"top":a>0?"bottom":"middle"};h<c&&o(i+s)<h&&(d.horizontal="center"),f<u&&o(a+r)<f&&(d.vertical="middle"),n(o(i),o(s))>n(o(a),o(r))?d.important="horizontal":d.important="vertical",t.using.call(this,e,d)}),l.offset(e.extend(D,{using:r}))}))},e.ui.position={fit:{left:function(e,t){var i,o=t.within,s=o.isWindow?o.scrollLeft:o.offset.left,a=o.width,r=e.left-t.collisionPosition.marginLeft,l=s-r,c=r+t.collisionWidth-a-s;t.collisionWidth>a?l>0&&c<=0?(i=e.left+l+t.collisionWidth-a-s,e.left+=l-i):e.left=c>0&&l<=0?s:l>c?s+a-t.collisionWidth:s:l>0?e.left+=l:c>0?e.left-=c:e.left=n(e.left-r,e.left)},top:function(e,t){var i,o=t.within,s=o.isWindow?o.scrollTop:o.offset.top,a=t.within.height,r=e.top-t.collisionPosition.marginTop,l=s-r,c=r+t.collisionHeight-a-s;t.collisionHeight>a?l>0&&c<=0?(i=e.top+l+t.collisionHeight-a-s,e.top+=l-i):e.top=c>0&&l<=0?s:l>c?s+a-t.collisionHeight:s:l>0?e.top+=l:c>0?e.top-=c:e.top=n(e.top-r,e.top)}},flip:{left:function(e,t){var i,n,s=t.within,a=s.offset.left+s.scrollLeft,r=s.width,l=s.isWindow?s.scrollLeft:s.offset.left,c=e.left-t.collisionPosition.marginLeft,h=c-l,u=c+t.collisionWidth-r-l,d="left"===t.my[0]?-t.elemWidth:"right"===t.my[0]?t.elemWidth:0,p="left"===t.at[0]?t.targetWidth:"right"===t.at[0]?-t.targetWidth:0,f=-2*t.offset[0];h<0?(i=e.left+d+p+f+t.collisionWidth-r-a,(i<0||i<o(h))&&(e.left+=d+p+f)):u>0&&(n=e.left-t.collisionPosition.marginLeft+d+p+f-l,(n>0||o(n)<u)&&(e.left+=d+p+f))},top:function(e,t){var i,n,s=t.within,a=s.offset.top+s.scrollTop,r=s.height,l=s.isWindow?s.scrollTop:s.offset.top,c=e.top-t.collisionPosition.marginTop,h=c-l,u=c+t.collisionHeight-r-l,d="top"===t.my[1],p=d?-t.elemHeight:"bottom"===t.my[1]?t.elemHeight:0,f="top"===t.at[1]?t.targetHeight:"bottom"===t.at[1]?-t.targetHeight:0,g=-2*t.offset[1];h<0?(n=e.top+p+f+g+t.collisionHeight-r-a,e.top+p+f+g>h&&(n<0||n<o(h))&&(e.top+=p+f+g)):u>0&&(i=e.top-t.collisionPosition.marginTop+p+f+g-l,e.top+p+f+g>u&&(i>0||o(i)<u)&&(e.top+=p+f+g))}},flipfit:{left:function(){e.ui.position.flip.left.apply(this,arguments),e.ui.position.fit.left.apply(this,arguments)},top:function(){e.ui.position.flip.top.apply(this,arguments),e.ui.position.fit.top.apply(this,arguments)}}},function(){var t,i,n,o,s,a=document.getElementsByTagName("body")[0],r=document.createElement("div");for(s in t=document.createElement(a?"div":"body"),n={visibility:"hidden",width:0,height:0,border:0,margin:0,background:"none"},a&&e.extend(n,{position:"absolute",left:"-1000px",top:"-1000px"}),n)t.style[s]=n[s];t.appendChild(r),i=a||document.documentElement,i.insertBefore(t,i.firstChild),r.style.cssText="position: absolute; left: 10.7432222px;",o=e(r).offset().left,e.support.offsetFractions=o>10&&o<11,t.innerHTML="",i.removeChild(t)}(),!1!==e.uiBackCompat&&function(e){var i=e.fn.position;e.fn.position=function(n){if(!n||!n.offset)return i.call(this,n);var o=n.offset.split(" "),s=n.at.split(" ");return 1===o.length&&(o[1]=o[0]),/^\d/.test(o[0])&&(o[0]="+"+o[0]),/^\d/.test(o[1])&&(o[1]="+"+o[1]),1===s.length&&(/left|center|right/.test(s[0])?s[1]="center":(s[1]=s[0],s[0]="center")),i.call(this,e.extend(n,{at:s[0]+o[0]+" "+s[1]+o[1],offset:t}))}}(jQuery)}(jQuery),function(e,t){e.widget("ui.progressbar",{version:"1.9.2",options:{value:0,max:100},min:0,_create:function(){this.element.addClass("ui-progressbar ui-widget ui-widget-content ui-corner-all").attr({role:"progressbar","aria-valuemin":this.min,"aria-valuemax":this.options.max,"aria-valuenow":this._value()}),this.valueDiv=e("<div class='ui-progressbar-value ui-widget-header ui-corner-left'></div>").appendTo(this.element),this.oldValue=this._value(),this._refreshValue()},_destroy:function(){this.element.removeClass("ui-progressbar ui-widget ui-widget-content ui-corner-all").removeAttr("role").removeAttr("aria-valuemin").removeAttr("aria-valuemax").removeAttr("aria-valuenow"),this.valueDiv.remove()},value:function(e){return e===t?this._value():(this._setOption("value",e),this)},_setOption:function(e,t){"value"===e&&(this.options.value=t,this._refreshValue(),this._value()===this.options.max&&this._trigger("complete")),this._super(e,t)},_value:function(){var e=this.options.value;return"number"!==typeof e&&(e=0),Math.min(this.options.max,Math.max(this.min,e))},_percentage:function(){return 100*this._value()/this.options.max},_refreshValue:function(){var e=this.value(),t=this._percentage();this.oldValue!==e&&(this.oldValue=e,this._trigger("change")),this.valueDiv.toggle(e>this.min).toggleClass("ui-corner-right",e===this.options.max).width(t.toFixed(0)+"%"),this.element.attr("aria-valuenow",e)}})}(jQuery),function(e,t){var i=5;e.widget("ui.slider",e.ui.mouse,{version:"1.9.2",widgetEventPrefix:"slide",options:{animate:!1,distance:0,max:100,min:0,orientation:"horizontal",range:!1,step:1,value:0,values:null},_create:function(){var t,n,o=this.options,s=this.element.find(".ui-slider-handle").addClass("ui-state-default ui-corner-all"),a="<a class='ui-slider-handle ui-state-default ui-corner-all' href='#'></a>",r=[];for(this._keySliding=!1,this._mouseSliding=!1,this._animateOff=!0,this._handleIndex=null,this._detectOrientation(),this._mouseInit(),this.element.addClass("ui-slider ui-slider-"+this.orientation+" ui-widget ui-widget-content ui-corner-all"+(o.disabled?" ui-slider-disabled ui-disabled":"")),this.range=e([]),o.range&&(!0===o.range&&(o.values||(o.values=[this._valueMin(),this._valueMin()]),o.values.length&&2!==o.values.length&&(o.values=[o.values[0],o.values[0]])),this.range=e("<div></div>").appendTo(this.element).addClass("ui-slider-range ui-widget-header"+("min"===o.range||"max"===o.range?" ui-slider-range-"+o.range:""))),n=o.values&&o.values.length||1,t=s.length;t<n;t++)r.push(a);this.handles=s.add(e(r.join("")).appendTo(this.element)),this.handle=this.handles.eq(0),this.handles.add(this.range).filter("a").click((function(e){e.preventDefault()})).mouseenter((function(){o.disabled||e(this).addClass("ui-state-hover")})).mouseleave((function(){e(this).removeClass("ui-state-hover")})).focus((function(){o.disabled?e(this).blur():(e(".ui-slider .ui-state-focus").removeClass("ui-state-focus"),e(this).addClass("ui-state-focus"))})).blur((function(){e(this).removeClass("ui-state-focus")})),this.handles.each((function(t){e(this).data("ui-slider-handle-index",t)})),this._on(this.handles,{keydown:function(t){var n,o,s,a,r=e(t.target).data("ui-slider-handle-index");switch(t.keyCode){case e.ui.keyCode.HOME:case e.ui.keyCode.END:case e.ui.keyCode.PAGE_UP:case e.ui.keyCode.PAGE_DOWN:case e.ui.keyCode.UP:case e.ui.keyCode.RIGHT:case e.ui.keyCode.DOWN:case e.ui.keyCode.LEFT:if(t.preventDefault(),!this._keySliding&&(this._keySliding=!0,e(t.target).addClass("ui-state-active"),n=this._start(t,r),!1===n))return;break}switch(a=this.options.step,o=s=this.options.values&&this.options.values.length?this.values(r):this.value(),t.keyCode){case e.ui.keyCode.HOME:s=this._valueMin();break;case e.ui.keyCode.END:s=this._valueMax();break;case e.ui.keyCode.PAGE_UP:s=this._trimAlignValue(o+(this._valueMax()-this._valueMin())/i);break;case e.ui.keyCode.PAGE_DOWN:s=this._trimAlignValue(o-(this._valueMax()-this._valueMin())/i);break;case e.ui.keyCode.UP:case e.ui.keyCode.RIGHT:if(o===this._valueMax())return;s=this._trimAlignValue(o+a);break;case e.ui.keyCode.DOWN:case e.ui.keyCode.LEFT:if(o===this._valueMin())return;s=this._trimAlignValue(o-a);break}this._slide(t,r,s)},keyup:function(t){var i=e(t.target).data("ui-slider-handle-index");this._keySliding&&(this._keySliding=!1,this._stop(t,i),this._change(t,i),e(t.target).removeClass("ui-state-active"))}}),this._refreshValue(),this._animateOff=!1},_destroy:function(){this.handles.remove(),this.range.remove(),this.element.removeClass("ui-slider ui-slider-horizontal ui-slider-vertical ui-slider-disabled ui-widget ui-widget-content ui-corner-all"),this._mouseDestroy()},_mouseCapture:function(t){var i,n,o,s,a,r,l,c,h=this,u=this.options;return!u.disabled&&(this.elementSize={width:this.element.outerWidth(),height:this.element.outerHeight()},this.elementOffset=this.element.offset(),i={x:t.pageX,y:t.pageY},n=this._normValueFromMouse(i),o=this._valueMax()-this._valueMin()+1,this.handles.each((function(t){var i=Math.abs(n-h.values(t));o>i&&(o=i,s=e(this),a=t)})),!0===u.range&&this.values(1)===u.min&&(a+=1,s=e(this.handles[a])),r=this._start(t,a),!1!==r&&(this._mouseSliding=!0,this._handleIndex=a,s.addClass("ui-state-active").focus(),l=s.offset(),c=!e(t.target).parents().andSelf().is(".ui-slider-handle"),this._clickOffset=c?{left:0,top:0}:{left:t.pageX-l.left-s.width()/2,top:t.pageY-l.top-s.height()/2-(parseInt(s.css("borderTopWidth"),10)||0)-(parseInt(s.css("borderBottomWidth"),10)||0)+(parseInt(s.css("marginTop"),10)||0)},this.handles.hasClass("ui-state-hover")||this._slide(t,a,n),this._animateOff=!0,!0))},_mouseStart:function(){return!0},_mouseDrag:function(e){var t={x:e.pageX,y:e.pageY},i=this._normValueFromMouse(t);return this._slide(e,this._handleIndex,i),!1},_mouseStop:function(e){return this.handles.removeClass("ui-state-active"),this._mouseSliding=!1,this._stop(e,this._handleIndex),this._change(e,this._handleIndex),this._handleIndex=null,this._clickOffset=null,this._animateOff=!1,!1},_detectOrientation:function(){this.orientation="vertical"===this.options.orientation?"vertical":"horizontal"},_normValueFromMouse:function(e){var t,i,n,o,s;return"horizontal"===this.orientation?(t=this.elementSize.width,i=e.x-this.elementOffset.left-(this._clickOffset?this._clickOffset.left:0)):(t=this.elementSize.height,i=e.y-this.elementOffset.top-(this._clickOffset?this._clickOffset.top:0)),n=i/t,n>1&&(n=1),n<0&&(n=0),"vertical"===this.orientation&&(n=1-n),o=this._valueMax()-this._valueMin(),s=this._valueMin()+n*o,this._trimAlignValue(s)},_start:function(e,t){var i={handle:this.handles[t],value:this.value()};return this.options.values&&this.options.values.length&&(i.value=this.values(t),i.values=this.values()),this._trigger("start",e,i)},_slide:function(e,t,i){var n,o,s;this.options.values&&this.options.values.length?(n=this.values(t?0:1),2===this.options.values.length&&!0===this.options.range&&(0===t&&i>n||1===t&&i<n)&&(i=n),i!==this.values(t)&&(o=this.values(),o[t]=i,s=this._trigger("slide",e,{handle:this.handles[t],value:i,values:o}),n=this.values(t?0:1),!1!==s&&this.values(t,i,!0))):i!==this.value()&&(s=this._trigger("slide",e,{handle:this.handles[t],value:i}),!1!==s&&this.value(i))},_stop:function(e,t){var i={handle:this.handles[t],value:this.value()};this.options.values&&this.options.values.length&&(i.value=this.values(t),i.values=this.values()),this._trigger("stop",e,i)},_change:function(e,t){if(!this._keySliding&&!this._mouseSliding){var i={handle:this.handles[t],value:this.value()};this.options.values&&this.options.values.length&&(i.value=this.values(t),i.values=this.values()),this._trigger("change",e,i)}},value:function(e){return arguments.length?(this.options.value=this._trimAlignValue(e),this._refreshValue(),void this._change(null,0)):this._value()},values:function(t,i){var n,o,s;if(arguments.length>1)return this.options.values[t]=this._trimAlignValue(i),this._refreshValue(),void this._change(null,t);if(!arguments.length)return this._values();if(!e.isArray(arguments[0]))return this.options.values&&this.options.values.length?this._values(t):this.value();for(n=this.options.values,o=arguments[0],s=0;s<n.length;s+=1)n[s]=this._trimAlignValue(o[s]),this._change(null,s);this._refreshValue()},_setOption:function(t,i){var n,o=0;switch(e.isArray(this.options.values)&&(o=this.options.values.length),e.Widget.prototype._setOption.apply(this,arguments),t){case"disabled":i?(this.handles.filter(".ui-state-focus").blur(),this.handles.removeClass("ui-state-hover"),this.handles.prop("disabled",!0),this.element.addClass("ui-disabled")):(this.handles.prop("disabled",!1),this.element.removeClass("ui-disabled"));break;case"orientation":this._detectOrientation(),this.element.removeClass("ui-slider-horizontal ui-slider-vertical").addClass("ui-slider-"+this.orientation),this._refreshValue();break;case"value":this._animateOff=!0,this._refreshValue(),this._change(null,0),this._animateOff=!1;break;case"values":for(this._animateOff=!0,this._refreshValue(),n=0;n<o;n+=1)this._change(null,n);this._animateOff=!1;break;case"min":case"max":this._animateOff=!0,this._refreshValue(),this._animateOff=!1;break}},_value:function(){var e=this.options.value;return e=this._trimAlignValue(e),e},_values:function(e){var t,i,n;if(arguments.length)return t=this.options.values[e],t=this._trimAlignValue(t),t;for(i=this.options.values.slice(),n=0;n<i.length;n+=1)i[n]=this._trimAlignValue(i[n]);return i},_trimAlignValue:function(e){if(e<=this._valueMin())return this._valueMin();if(e>=this._valueMax())return this._valueMax();var t=this.options.step>0?this.options.step:1,i=(e-this._valueMin())%t,n=e-i;return 2*Math.abs(i)>=t&&(n+=i>0?t:-t),parseFloat(n.toFixed(5))},_valueMin:function(){return this.options.min},_valueMax:function(){return this.options.max},_refreshValue:function(){var t,i,n,o,s,a=this.options.range,r=this.options,l=this,c=!this._animateOff&&r.animate,h={};this.options.values&&this.options.values.length?this.handles.each((function(n){i=(l.values(n)-l._valueMin())/(l._valueMax()-l._valueMin())*100,h["horizontal"===l.orientation?"left":"bottom"]=i+"%",e(this).stop(1,1)[c?"animate":"css"](h,r.animate),!0===l.options.range&&("horizontal"===l.orientation?(0===n&&l.range.stop(1,1)[c?"animate":"css"]({left:i+"%"},r.animate),1===n&&l.range[c?"animate":"css"]({width:i-t+"%"},{queue:!1,duration:r.animate})):(0===n&&l.range.stop(1,1)[c?"animate":"css"]({bottom:i+"%"},r.animate),1===n&&l.range[c?"animate":"css"]({height:i-t+"%"},{queue:!1,duration:r.animate}))),t=i})):(n=this.value(),o=this._valueMin(),s=this._valueMax(),i=s!==o?(n-o)/(s-o)*100:0,h["horizontal"===this.orientation?"left":"bottom"]=i+"%",this.handle.stop(1,1)[c?"animate":"css"](h,r.animate),"min"===a&&"horizontal"===this.orientation&&this.range.stop(1,1)[c?"animate":"css"]({width:i+"%"},r.animate),"max"===a&&"horizontal"===this.orientation&&this.range[c?"animate":"css"]({width:100-i+"%"},{queue:!1,duration:r.animate}),"min"===a&&"vertical"===this.orientation&&this.range.stop(1,1)[c?"animate":"css"]({height:i+"%"},r.animate),"max"===a&&"vertical"===this.orientation&&this.range[c?"animate":"css"]({height:100-i+"%"},{queue:!1,duration:r.animate}))}})}(jQuery),function(e){function t(e){return function(){var t=this.element.val();e.apply(this,arguments),this._refresh(),t!==this.element.val()&&this._trigger("change")}}e.widget("ui.spinner",{version:"1.9.2",defaultElement:"<input>",widgetEventPrefix:"spin",options:{culture:null,icons:{down:"ui-icon-triangle-1-s",up:"ui-icon-triangle-1-n"},incremental:!0,max:null,min:null,numberFormat:null,page:10,step:1,change:null,spin:null,start:null,stop:null},_create:function(){this._setOption("max",this.options.max),this._setOption("min",this.options.min),this._setOption("step",this.options.step),this._value(this.element.val(),!0),this._draw(),this._on(this._events),this._refresh(),this._on(this.window,{beforeunload:function(){this.element.removeAttr("autocomplete")}})},_getCreateOptions:function(){var t={},i=this.element;return e.each(["min","max","step"],(function(e,n){var o=i.attr(n);void 0!==o&&o.length&&(t[n]=o)})),t},_events:{keydown:function(e){this._start(e)&&this._keydown(e)&&e.preventDefault()},keyup:"_stop",focus:function(){this.previous=this.element.val()},blur:function(e){this.cancelBlur?delete this.cancelBlur:(this._refresh(),this.previous!==this.element.val()&&this._trigger("change",e))},mousewheel:function(e,t){if(t){if(!this.spinning&&!this._start(e))return!1;this._spin((t>0?1:-1)*this.options.step,e),clearTimeout(this.mousewheelTimer),this.mousewheelTimer=this._delay((function(){this.spinning&&this._stop(e)}),100),e.preventDefault()}},"mousedown .ui-spinner-button":function(t){var i;function n(){var e=this.element[0]===this.document[0].activeElement;e||(this.element.focus(),this.previous=i,this._delay((function(){this.previous=i})))}i=this.element[0]===this.document[0].activeElement?this.previous:this.element.val(),t.preventDefault(),n.call(this),this.cancelBlur=!0,this._delay((function(){delete this.cancelBlur,n.call(this)})),!1!==this._start(t)&&this._repeat(null,e(t.currentTarget).hasClass("ui-spinner-up")?1:-1,t)},"mouseup .ui-spinner-button":"_stop","mouseenter .ui-spinner-button":function(t){if(e(t.currentTarget).hasClass("ui-state-active"))return!1!==this._start(t)&&void this._repeat(null,e(t.currentTarget).hasClass("ui-spinner-up")?1:-1,t)},"mouseleave .ui-spinner-button":"_stop"},_draw:function(){var e=this.uiSpinner=this.element.addClass("ui-spinner-input").attr("autocomplete","off").wrap(this._uiSpinnerHtml()).parent().append(this._buttonHtml());this.element.attr("role","spinbutton"),this.buttons=e.find(".ui-spinner-button").attr("tabIndex",-1).button().removeClass("ui-corner-all"),this.buttons.height()>Math.ceil(.5*e.height())&&e.height()>0&&e.height(e.height()),this.options.disabled&&this.disable()},_keydown:function(t){var i=this.options,n=e.ui.keyCode;switch(t.keyCode){case n.UP:return this._repeat(null,1,t),!0;case n.DOWN:return this._repeat(null,-1,t),!0;case n.PAGE_UP:return this._repeat(null,i.page,t),!0;case n.PAGE_DOWN:return this._repeat(null,-i.page,t),!0}return!1},_uiSpinnerHtml:function(){return"<span class='ui-spinner ui-widget ui-widget-content ui-corner-all'></span>"},_buttonHtml:function(){return"<a class='ui-spinner-button ui-spinner-up ui-corner-tr'><span class='ui-icon "+this.options.icons.up+"'>&#9650;</span></a><a class='ui-spinner-button ui-spinner-down ui-corner-br'><span class='ui-icon "+this.options.icons.down+"'>&#9660;</span></a>"},_start:function(e){return!(!this.spinning&&!1===this._trigger("start",e))&&(this.counter||(this.counter=1),this.spinning=!0,!0)},_repeat:function(e,t,i){e=e||500,clearTimeout(this.timer),this.timer=this._delay((function(){this._repeat(40,t,i)}),e),this._spin(t*this.options.step,i)},_spin:function(e,t){var i=this.value()||0;this.counter||(this.counter=1),i=this._adjustValue(i+e*this._increment(this.counter)),this.spinning&&!1===this._trigger("spin",t,{value:i})||(this._value(i),this.counter++)},_increment:function(t){var i=this.options.incremental;return i?e.isFunction(i)?i(t):Math.floor(t*t*t/5e4-t*t/500+17*t/200+1):1},_precision:function(){var e=this._precisionOf(this.options.step);return null!==this.options.min&&(e=Math.max(e,this._precisionOf(this.options.min))),e},_precisionOf:function(e){var t=e.toString(),i=t.indexOf(".");return-1===i?0:t.length-i-1},_adjustValue:function(e){var t,i,n=this.options;return t=null!==n.min?n.min:0,i=e-t,i=Math.round(i/n.step)*n.step,e=t+i,e=parseFloat(e.toFixed(this._precision())),null!==n.max&&e>n.max?n.max:null!==n.min&&e<n.min?n.min:e},_stop:function(e){this.spinning&&(clearTimeout(this.timer),clearTimeout(this.mousewheelTimer),this.counter=0,this.spinning=!1,this._trigger("stop",e))},_setOption:function(e,t){if("culture"===e||"numberFormat"===e){var i=this._parse(this.element.val());return this.options[e]=t,void this.element.val(this._format(i))}"max"!==e&&"min"!==e&&"step"!==e||"string"===typeof t&&(t=this._parse(t)),this._super(e,t),"disabled"===e&&(t?(this.element.prop("disabled",!0),this.buttons.button("disable")):(this.element.prop("disabled",!1),this.buttons.button("enable")))},_setOptions:t((function(e){this._super(e),this._value(this.element.val())})),_parse:function(e){return"string"===typeof e&&""!==e&&(e=window.Globalize&&this.options.numberFormat?Globalize.parseFloat(e,10,this.options.culture):+e),""===e||isNaN(e)?null:e},_format:function(e){return""===e?"":window.Globalize&&this.options.numberFormat?Globalize.format(e,this.options.numberFormat,this.options.culture):e},_refresh:function(){this.element.attr({"aria-valuemin":this.options.min,"aria-valuemax":this.options.max,"aria-valuenow":this._parse(this.element.val())})},_value:function(e,t){var i;""!==e&&(i=this._parse(e),null!==i&&(t||(i=this._adjustValue(i)),e=this._format(i))),this.element.val(e),this._refresh()},_destroy:function(){this.element.removeClass("ui-spinner-input").prop("disabled",!1).removeAttr("autocomplete").removeAttr("role").removeAttr("aria-valuemin").removeAttr("aria-valuemax").removeAttr("aria-valuenow"),this.uiSpinner.replaceWith(this.element)},stepUp:t((function(e){this._stepUp(e)})),_stepUp:function(e){this._spin((e||1)*this.options.step)},stepDown:t((function(e){this._stepDown(e)})),_stepDown:function(e){this._spin((e||1)*-this.options.step)},pageUp:t((function(e){this._stepUp((e||1)*this.options.page)})),pageDown:t((function(e){this._stepDown((e||1)*this.options.page)})),value:function(e){if(!arguments.length)return this._parse(this.element.val());t(this._value).call(this,e)},widget:function(){return this.uiSpinner}})}(jQuery),function(e,t){var i=0,n=/#.*$/;function o(){return++i}function s(e){return e.hash.length>1&&e.href.replace(n,"")===location.href.replace(n,"").replace(/\s/g,"%20")}e.widget("ui.tabs",{version:"1.9.2",delay:300,options:{active:null,collapsible:!1,event:"click",heightStyle:"content",hide:null,show:null,activate:null,beforeActivate:null,beforeLoad:null,load:null},_create:function(){var t=this,i=this.options,n=i.active,o=location.hash.substring(1);this.running=!1,this.element.addClass("ui-tabs ui-widget ui-widget-content ui-corner-all").toggleClass("ui-tabs-collapsible",i.collapsible).delegate(".ui-tabs-nav > li","mousedown"+this.eventNamespace,(function(t){e(this).is(".ui-state-disabled")&&t.preventDefault()})).delegate(".ui-tabs-anchor","focus"+this.eventNamespace,(function(){e(this).closest("li").is(".ui-state-disabled")&&this.blur()})),this._processTabs(),null===n&&(o&&this.tabs.each((function(t,i){if(e(i).attr("aria-controls")===o)return n=t,!1})),null===n&&(n=this.tabs.index(this.tabs.filter(".ui-tabs-active"))),null!==n&&-1!==n||(n=!!this.tabs.length&&0)),!1!==n&&(n=this.tabs.index(this.tabs.eq(n)),-1===n&&(n=!i.collapsible&&0)),i.active=n,!i.collapsible&&!1===i.active&&this.anchors.length&&(i.active=0),e.isArray(i.disabled)&&(i.disabled=e.unique(i.disabled.concat(e.map(this.tabs.filter(".ui-state-disabled"),(function(e){return t.tabs.index(e)})))).sort()),!1!==this.options.active&&this.anchors.length?this.active=this._findActive(this.options.active):this.active=e(),this._refresh(),this.active.length&&this.load(i.active)},_getCreateEventData:function(){return{tab:this.active,panel:this.active.length?this._getPanelForTab(this.active):e()}},_tabKeydown:function(t){var i=e(this.document[0].activeElement).closest("li"),n=this.tabs.index(i),o=!0;if(!this._handlePageNav(t)){switch(t.keyCode){case e.ui.keyCode.RIGHT:case e.ui.keyCode.DOWN:n++;break;case e.ui.keyCode.UP:case e.ui.keyCode.LEFT:o=!1,n--;break;case e.ui.keyCode.END:n=this.anchors.length-1;break;case e.ui.keyCode.HOME:n=0;break;case e.ui.keyCode.SPACE:return t.preventDefault(),clearTimeout(this.activating),void this._activate(n);case e.ui.keyCode.ENTER:return t.preventDefault(),clearTimeout(this.activating),void this._activate(n!==this.options.active&&n);default:return}t.preventDefault(),clearTimeout(this.activating),n=this._focusNextTab(n,o),t.ctrlKey||(i.attr("aria-selected","false"),this.tabs.eq(n).attr("aria-selected","true"),this.activating=this._delay((function(){this.option("active",n)}),this.delay))}},_panelKeydown:function(t){this._handlePageNav(t)||t.ctrlKey&&t.keyCode===e.ui.keyCode.UP&&(t.preventDefault(),this.active.focus())},_handlePageNav:function(t){return t.altKey&&t.keyCode===e.ui.keyCode.PAGE_UP?(this._activate(this._focusNextTab(this.options.active-1,!1)),!0):t.altKey&&t.keyCode===e.ui.keyCode.PAGE_DOWN?(this._activate(this._focusNextTab(this.options.active+1,!0)),!0):void 0},_findNextTab:function(t,i){var n=this.tabs.length-1;function o(){return t>n&&(t=0),t<0&&(t=n),t}while(-1!==e.inArray(o(),this.options.disabled))t=i?t+1:t-1;return t},_focusNextTab:function(e,t){return e=this._findNextTab(e,t),this.tabs.eq(e).focus(),e},_setOption:function(e,t){"active"!==e?"disabled"!==e?(this._super(e,t),"collapsible"===e&&(this.element.toggleClass("ui-tabs-collapsible",t),t||!1!==this.options.active||this._activate(0)),"event"===e&&this._setupEvents(t),"heightStyle"===e&&this._setupHeightStyle(t)):this._setupDisabled(t):this._activate(t)},_tabId:function(e){return e.attr("aria-controls")||"ui-tabs-"+o()},_sanitizeSelector:function(e){return e?e.replace(/[!"$%&'()*+,.\/:;<=>?@\[\]\^`{|}~]/g,"\\$&"):""},refresh:function(){var t=this.options,i=this.tablist.children(":has(a[href])");t.disabled=e.map(i.filter(".ui-state-disabled"),(function(e){return i.index(e)})),this._processTabs(),!1!==t.active&&this.anchors.length?this.active.length&&!e.contains(this.tablist[0],this.active[0])?this.tabs.length===t.disabled.length?(t.active=!1,this.active=e()):this._activate(this._findNextTab(Math.max(0,t.active-1),!1)):t.active=this.tabs.index(this.active):(t.active=!1,this.active=e()),this._refresh()},_refresh:function(){this._setupDisabled(this.options.disabled),this._setupEvents(this.options.event),this._setupHeightStyle(this.options.heightStyle),this.tabs.not(this.active).attr({"aria-selected":"false",tabIndex:-1}),this.panels.not(this._getPanelForTab(this.active)).hide().attr({"aria-expanded":"false","aria-hidden":"true"}),this.active.length?(this.active.addClass("ui-tabs-active ui-state-active").attr({"aria-selected":"true",tabIndex:0}),this._getPanelForTab(this.active).show().attr({"aria-expanded":"true","aria-hidden":"false"})):this.tabs.eq(0).attr("tabIndex",0)},_processTabs:function(){var t=this;this.tablist=this._getList().addClass("ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all").attr("role","tablist"),this.tabs=this.tablist.find("> li:has(a[href])").addClass("ui-state-default ui-corner-top").attr({role:"tab",tabIndex:-1}),this.anchors=this.tabs.map((function(){return e("a",this)[0]})).addClass("ui-tabs-anchor").attr({role:"presentation",tabIndex:-1}),this.panels=e(),this.anchors.each((function(i,n){var o,a,r,l=e(n).uniqueId().attr("id"),c=e(n).closest("li"),h=c.attr("aria-controls");s(n)?(o=n.hash,a=t.element.find(t._sanitizeSelector(o))):(r=t._tabId(c),o="#"+r,a=t.element.find(o),a.length||(a=t._createPanel(r),a.insertAfter(t.panels[i-1]||t.tablist)),a.attr("aria-live","polite")),a.length&&(t.panels=t.panels.add(a)),h&&c.data("ui-tabs-aria-controls",h),c.attr({"aria-controls":o.substring(1),"aria-labelledby":l}),a.attr("aria-labelledby",l)})),this.panels.addClass("ui-tabs-panel ui-widget-content ui-corner-bottom").attr("role","tabpanel")},_getList:function(){return this.element.find("ol,ul").eq(0)},_createPanel:function(t){return e("<div>").attr("id",t).addClass("ui-tabs-panel ui-widget-content ui-corner-bottom").data("ui-tabs-destroy",!0)},_setupDisabled:function(t){e.isArray(t)&&(t.length?t.length===this.anchors.length&&(t=!0):t=!1);for(var i,n=0;i=this.tabs[n];n++)!0===t||-1!==e.inArray(n,t)?e(i).addClass("ui-state-disabled").attr("aria-disabled","true"):e(i).removeClass("ui-state-disabled").removeAttr("aria-disabled");this.options.disabled=t},_setupEvents:function(t){var i={click:function(e){e.preventDefault()}};t&&e.each(t.split(" "),(function(e,t){i[t]="_eventHandler"})),this._off(this.anchors.add(this.tabs).add(this.panels)),this._on(this.anchors,i),this._on(this.tabs,{keydown:"_tabKeydown"}),this._on(this.panels,{keydown:"_panelKeydown"}),this._focusable(this.tabs),this._hoverable(this.tabs)},_setupHeightStyle:function(t){var i,n,o=this.element.parent();"fill"===t?(e.support.minHeight||(n=o.css("overflow"),o.css("overflow","hidden")),i=o.height(),this.element.siblings(":visible").each((function(){var t=e(this),n=t.css("position");"absolute"!==n&&"fixed"!==n&&(i-=t.outerHeight(!0))})),n&&o.css("overflow",n),this.element.children().not(this.panels).each((function(){i-=e(this).outerHeight(!0)})),this.panels.each((function(){e(this).height(Math.max(0,i-e(this).innerHeight()+e(this).height()))})).css("overflow","auto")):"auto"===t&&(i=0,this.panels.each((function(){i=Math.max(i,e(this).height("").height())})).height(i))},_eventHandler:function(t){var i=this.options,n=this.active,o=e(t.currentTarget),s=o.closest("li"),a=s[0]===n[0],r=a&&i.collapsible,l=r?e():this._getPanelForTab(s),c=n.length?this._getPanelForTab(n):e(),h={oldTab:n,oldPanel:c,newTab:r?e():s,newPanel:l};t.preventDefault(),s.hasClass("ui-state-disabled")||s.hasClass("ui-tabs-loading")||this.running||a&&!i.collapsible||!1===this._trigger("beforeActivate",t,h)||(i.active=!r&&this.tabs.index(s),this.active=a?e():s,this.xhr&&this.xhr.abort(),c.length||l.length||e.error("jQuery UI Tabs: Mismatching fragment identifier."),l.length&&this.load(this.tabs.index(s),t),this._toggle(t,h))},_toggle:function(t,i){var n=this,o=i.newPanel,s=i.oldPanel;function a(){n.running=!1,n._trigger("activate",t,i)}function r(){i.newTab.closest("li").addClass("ui-tabs-active ui-state-active"),o.length&&n.options.show?n._show(o,n.options.show,a):(o.show(),a())}this.running=!0,s.length&&this.options.hide?this._hide(s,this.options.hide,(function(){i.oldTab.closest("li").removeClass("ui-tabs-active ui-state-active"),r()})):(i.oldTab.closest("li").removeClass("ui-tabs-active ui-state-active"),s.hide(),r()),s.attr({"aria-expanded":"false","aria-hidden":"true"}),i.oldTab.attr("aria-selected","false"),o.length&&s.length?i.oldTab.attr("tabIndex",-1):o.length&&this.tabs.filter((function(){return 0===e(this).attr("tabIndex")})).attr("tabIndex",-1),o.attr({"aria-expanded":"true","aria-hidden":"false"}),i.newTab.attr({"aria-selected":"true",tabIndex:0})},_activate:function(t){var i,n=this._findActive(t);n[0]!==this.active[0]&&(n.length||(n=this.active),i=n.find(".ui-tabs-anchor")[0],this._eventHandler({target:i,currentTarget:i,preventDefault:e.noop}))},_findActive:function(t){return!1===t?e():this.tabs.eq(t)},_getIndex:function(e){return"string"===typeof e&&(e=this.anchors.index(this.anchors.filter("[href$='"+e+"']"))),e},_destroy:function(){this.xhr&&this.xhr.abort(),this.element.removeClass("ui-tabs ui-widget ui-widget-content ui-corner-all ui-tabs-collapsible"),this.tablist.removeClass("ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all").removeAttr("role"),this.anchors.removeClass("ui-tabs-anchor").removeAttr("role").removeAttr("tabIndex").removeData("href.tabs").removeData("load.tabs").removeUniqueId(),this.tabs.add(this.panels).each((function(){e.data(this,"ui-tabs-destroy")?e(this).remove():e(this).removeClass("ui-state-default ui-state-active ui-state-disabled ui-corner-top ui-corner-bottom ui-widget-content ui-tabs-active ui-tabs-panel").removeAttr("tabIndex").removeAttr("aria-live").removeAttr("aria-busy").removeAttr("aria-selected").removeAttr("aria-labelledby").removeAttr("aria-hidden").removeAttr("aria-expanded").removeAttr("role")})),this.tabs.each((function(){var t=e(this),i=t.data("ui-tabs-aria-controls");i?t.attr("aria-controls",i):t.removeAttr("aria-controls")})),this.panels.show(),"content"!==this.options.heightStyle&&this.panels.css("height","")},enable:function(i){var n=this.options.disabled;!1!==n&&(i===t?n=!1:(i=this._getIndex(i),n=e.isArray(n)?e.map(n,(function(e){return e!==i?e:null})):e.map(this.tabs,(function(e,t){return t!==i?t:null}))),this._setupDisabled(n))},disable:function(i){var n=this.options.disabled;if(!0!==n){if(i===t)n=!0;else{if(i=this._getIndex(i),-1!==e.inArray(i,n))return;n=e.isArray(n)?e.merge([i],n).sort():[i]}this._setupDisabled(n)}},load:function(t,i){t=this._getIndex(t);var n=this,o=this.tabs.eq(t),a=o.find(".ui-tabs-anchor"),r=this._getPanelForTab(o),l={tab:o,panel:r};s(a[0])||(this.xhr=e.ajax(this._ajaxSettings(a,i,l)),this.xhr&&"canceled"!==this.xhr.statusText&&(o.addClass("ui-tabs-loading"),r.attr("aria-busy","true"),this.xhr.success((function(e){setTimeout((function(){r.html(e),n._trigger("load",i,l)}),1)})).complete((function(e,t){setTimeout((function(){"abort"===t&&n.panels.stop(!1,!0),o.removeClass("ui-tabs-loading"),r.removeAttr("aria-busy"),e===n.xhr&&delete n.xhr}),1)}))))},_ajaxSettings:function(t,i,n){var o=this;return{url:t.attr("href"),beforeSend:function(t,s){return o._trigger("beforeLoad",i,e.extend({jqXHR:t,ajaxSettings:s},n))}}},_getPanelForTab:function(t){var i=e(t).attr("aria-controls");return this.element.find(this._sanitizeSelector("#"+i))}}),!1!==e.uiBackCompat&&(e.ui.tabs.prototype._ui=function(e,t){return{tab:e,panel:t,index:this.anchors.index(e)}},e.widget("ui.tabs",e.ui.tabs,{url:function(e,t){this.anchors.eq(e).attr("href",t)}}),e.widget("ui.tabs",e.ui.tabs,{options:{ajaxOptions:null,cache:!1},_create:function(){this._super();var t=this;this._on({tabsbeforeload:function(i,n){e.data(n.tab[0],"cache.tabs")?i.preventDefault():n.jqXHR.success((function(){t.options.cache&&e.data(n.tab[0],"cache.tabs",!0)}))}})},_ajaxSettings:function(t,i,n){var o=this.options.ajaxOptions;return e.extend({},o,{error:function(e,t){try{o.error(e,t,n.tab.closest("li").index(),n.tab[0])}catch(i){}}},this._superApply(arguments))},_setOption:function(e,t){"cache"===e&&!1===t&&this.anchors.removeData("cache.tabs"),this._super(e,t)},_destroy:function(){this.anchors.removeData("cache.tabs"),this._super()},url:function(e){this.anchors.eq(e).removeData("cache.tabs"),this._superApply(arguments)}}),e.widget("ui.tabs",e.ui.tabs,{abort:function(){this.xhr&&this.xhr.abort()}}),e.widget("ui.tabs",e.ui.tabs,{options:{spinner:"<em>Loading&#8230;</em>"},_create:function(){this._super(),this._on({tabsbeforeload:function(e,t){if(e.target===this.element[0]&&this.options.spinner){var i=t.tab.find("span"),n=i.html();i.html(this.options.spinner),t.jqXHR.complete((function(){i.html(n)}))}}})}}),e.widget("ui.tabs",e.ui.tabs,{options:{enable:null,disable:null},enable:function(t){var i,n=this.options;(t&&!0===n.disabled||e.isArray(n.disabled)&&-1!==e.inArray(t,n.disabled))&&(i=!0),this._superApply(arguments),i&&this._trigger("enable",null,this._ui(this.anchors[t],this.panels[t]))},disable:function(t){var i,n=this.options;(t&&!1===n.disabled||e.isArray(n.disabled)&&-1===e.inArray(t,n.disabled))&&(i=!0),this._superApply(arguments),i&&this._trigger("disable",null,this._ui(this.anchors[t],this.panels[t]))}}),e.widget("ui.tabs",e.ui.tabs,{options:{add:null,remove:null,tabTemplate:"<li><a href='#{href}'><span>#{label}</span></a></li>"},add:function(i,n,o){o===t&&(o=this.anchors.length);var s,a,r=this.options,l=e(r.tabTemplate.replace(/#\{href\}/g,i).replace(/#\{label\}/g,n)),c=i.indexOf("#")?this._tabId(l):i.replace("#","");return l.addClass("ui-state-default ui-corner-top").data("ui-tabs-destroy",!0),l.attr("aria-controls",c),s=o>=this.tabs.length,a=this.element.find("#"+c),a.length||(a=this._createPanel(c),s?o>0?a.insertAfter(this.panels.eq(-1)):a.appendTo(this.element):a.insertBefore(this.panels[o])),a.addClass("ui-tabs-panel ui-widget-content ui-corner-bottom").hide(),s?l.appendTo(this.tablist):l.insertBefore(this.tabs[o]),r.disabled=e.map(r.disabled,(function(e){return e>=o?++e:e})),this.refresh(),1===this.tabs.length&&!1===r.active&&this.option("active",0),this._trigger("add",null,this._ui(this.anchors[o],this.panels[o])),this},remove:function(t){t=this._getIndex(t);var i=this.options,n=this.tabs.eq(t).remove(),o=this._getPanelForTab(n).remove();return n.hasClass("ui-tabs-active")&&this.anchors.length>2&&this._activate(t+(t+1<this.anchors.length?1:-1)),i.disabled=e.map(e.grep(i.disabled,(function(e){return e!==t})),(function(e){return e>=t?--e:e})),this.refresh(),this._trigger("remove",null,this._ui(n.find("a")[0],o[0])),this}}),e.widget("ui.tabs",e.ui.tabs,{length:function(){return this.anchors.length}}),e.widget("ui.tabs",e.ui.tabs,{options:{idPrefix:"ui-tabs-"},_tabId:function(t){var i=t.is("li")?t.find("a[href]"):t;return i=i[0],e(i).closest("li").attr("aria-controls")||i.title&&i.title.replace(/\s/g,"_").replace(/[^\w\u00c0-\uFFFF\-]/g,"")||this.options.idPrefix+o()}}),e.widget("ui.tabs",e.ui.tabs,{options:{panelTemplate:"<div></div>"},_createPanel:function(t){return e(this.options.panelTemplate).attr("id",t).addClass("ui-tabs-panel ui-widget-content ui-corner-bottom").data("ui-tabs-destroy",!0)}}),e.widget("ui.tabs",e.ui.tabs,{_create:function(){var e=this.options;null===e.active&&e.selected!==t&&(e.active=-1!==e.selected&&e.selected),this._super(),e.selected=e.active,!1===e.selected&&(e.selected=-1)},_setOption:function(e,t){if("selected"!==e)return this._super(e,t);var i=this.options;this._super("active",-1!==t&&t),i.selected=i.active,!1===i.selected&&(i.selected=-1)},_eventHandler:function(){this._superApply(arguments),this.options.selected=this.options.active,!1===this.options.selected&&(this.options.selected=-1)}}),e.widget("ui.tabs",e.ui.tabs,{options:{show:null,select:null},_create:function(){this._super(),!1!==this.options.active&&this._trigger("show",null,this._ui(this.active.find(".ui-tabs-anchor")[0],this._getPanelForTab(this.active)[0]))},_trigger:function(e,t,i){var n,o,s=this._superApply(arguments);return!!s&&("beforeActivate"===e?(n=i.newTab.length?i.newTab:i.oldTab,o=i.newPanel.length?i.newPanel:i.oldPanel,s=this._super("select",t,{tab:n.find(".ui-tabs-anchor")[0],panel:o[0],index:n.closest("li").index()})):"activate"===e&&i.newTab.length&&(s=this._super("show",t,{tab:i.newTab.find(".ui-tabs-anchor")[0],panel:i.newPanel[0],index:i.newTab.closest("li").index()})),s)}}),e.widget("ui.tabs",e.ui.tabs,{select:function(e){if(e=this._getIndex(e),-1===e){if(!this.options.collapsible||-1===this.options.selected)return;e=this.options.selected}this.anchors.eq(e).trigger(this.options.event+this.eventNamespace)}}),function(){var t=0;e.widget("ui.tabs",e.ui.tabs,{options:{cookie:null},_create:function(){var e,t=this.options;null==t.active&&t.cookie&&(e=parseInt(this._cookie(),10),-1===e&&(e=!1),t.active=e),this._super()},_cookie:function(i){var n=[this.cookie||(this.cookie=this.options.cookie.name||"ui-tabs-"+ ++t)];return arguments.length&&(n.push(!1===i?-1:i),n.push(this.options.cookie)),e.cookie.apply(null,n)},_refresh:function(){this._super(),this.options.cookie&&this._cookie(this.options.active,this.options.cookie)},_eventHandler:function(){this._superApply(arguments),this.options.cookie&&this._cookie(this.options.active,this.options.cookie)},_destroy:function(){this._super(),this.options.cookie&&this._cookie(null,this.options.cookie)}})}(),e.widget("ui.tabs",e.ui.tabs,{_trigger:function(t,i,n){var o=e.extend({},n);return"load"===t&&(o.panel=o.panel[0],o.tab=o.tab.find(".ui-tabs-anchor")[0]),this._super(t,i,o)}}),e.widget("ui.tabs",e.ui.tabs,{options:{fx:null},_getFx:function(){var t,i,n=this.options.fx;return n&&(e.isArray(n)?(t=n[0],i=n[1]):t=i=n),n?{show:i,hide:t}:null},_toggle:function(e,t){var i=this,n=t.newPanel,o=t.oldPanel,s=this._getFx();if(!s)return this._super(e,t);function a(){i.running=!1,i._trigger("activate",e,t)}function r(){t.newTab.closest("li").addClass("ui-tabs-active ui-state-active"),n.length&&s.show?n.animate(s.show,s.show.duration,(function(){a()})):(n.show(),a())}i.running=!0,o.length&&s.hide?o.animate(s.hide,s.hide.duration,(function(){t.oldTab.closest("li").removeClass("ui-tabs-active ui-state-active"),r()})):(t.oldTab.closest("li").removeClass("ui-tabs-active ui-state-active"),o.hide(),r())}}))}(jQuery),function(e){var t=0;function i(t,i){var n=(t.attr("aria-describedby")||"").split(/\s+/);n.push(i),t.data("ui-tooltip-id",i).attr("aria-describedby",e.trim(n.join(" ")))}function n(t){var i=t.data("ui-tooltip-id"),n=(t.attr("aria-describedby")||"").split(/\s+/),o=e.inArray(i,n);-1!==o&&n.splice(o,1),t.removeData("ui-tooltip-id"),n=e.trim(n.join(" ")),n?t.attr("aria-describedby",n):t.removeAttr("aria-describedby")}e.widget("ui.tooltip",{version:"1.9.2",options:{content:function(){return e(this).attr("title")},hide:!0,items:"[title]:not([disabled])",position:{my:"left top+15",at:"left bottom",collision:"flipfit flip"},show:!0,tooltipClass:null,track:!1,close:null,open:null},_create:function(){this._on({mouseover:"open",focusin:"open"}),this.tooltips={},this.parents={},this.options.disabled&&this._disable()},_setOption:function(t,i){var n=this;if("disabled"===t)return this[i?"_disable":"_enable"](),void(this.options[t]=i);this._super(t,i),"content"===t&&e.each(this.tooltips,(function(e,t){n._updateContent(t)}))},_disable:function(){var t=this;e.each(this.tooltips,(function(i,n){var o=e.Event("blur");o.target=o.currentTarget=n[0],t.close(o,!0)})),this.element.find(this.options.items).andSelf().each((function(){var t=e(this);t.is("[title]")&&t.data("ui-tooltip-title",t.attr("title")).attr("title","")}))},_enable:function(){this.element.find(this.options.items).andSelf().each((function(){var t=e(this);t.data("ui-tooltip-title")&&t.attr("title",t.data("ui-tooltip-title"))}))},open:function(t){var i=this,n=e(t?t.target:this.element).closest(this.options.items);n.length&&!n.data("ui-tooltip-id")&&(n.attr("title")&&n.data("ui-tooltip-title",n.attr("title")),n.data("ui-tooltip-open",!0),t&&"mouseover"===t.type&&n.parents().each((function(){var t,n=e(this);n.data("ui-tooltip-open")&&(t=e.Event("blur"),t.target=t.currentTarget=this,i.close(t,!0)),n.attr("title")&&(n.uniqueId(),i.parents[this.id]={element:this,title:n.attr("title")},n.attr("title",""))})),this._updateContent(n,t))},_updateContent:function(e,t){var i,n=this.options.content,o=this,s=t?t.type:null;if("string"===typeof n)return this._open(t,e,n);i=n.call(e[0],(function(i){e.data("ui-tooltip-open")&&o._delay((function(){t&&(t.type=s),this._open(t,e,i)}))})),i&&this._open(t,e,i)},_open:function(t,n,o){var s,a,r,l=e.extend({},this.options.position);function c(e){l.of=e,s.is(":hidden")||s.position(l)}o&&(s=this._find(n),s.length?s.find(".ui-tooltip-content").html(o):(n.is("[title]")&&(t&&"mouseover"===t.type?n.attr("title",""):n.removeAttr("title")),s=this._tooltip(n),i(n,s.attr("id")),s.find(".ui-tooltip-content").html(o),this.options.track&&t&&/^mouse/.test(t.type)?(this._on(this.document,{mousemove:c}),c(t)):s.position(e.extend({of:n},this.options.position)),s.hide(),this._show(s,this.options.show),this.options.show&&this.options.show.delay&&(r=setInterval((function(){s.is(":visible")&&(c(l.of),clearInterval(r))}),e.fx.interval)),this._trigger("open",t,{tooltip:s}),a={keyup:function(t){if(t.keyCode===e.ui.keyCode.ESCAPE){var i=e.Event(t);i.currentTarget=n[0],this.close(i,!0)}},remove:function(){this._removeTooltip(s)}},t&&"mouseover"!==t.type||(a.mouseleave="close"),t&&"focusin"!==t.type||(a.focusout="close"),this._on(!0,n,a)))},close:function(t){var i=this,o=e(t?t.currentTarget:this.element),s=this._find(o);this.closing||(o.data("ui-tooltip-title")&&o.attr("title",o.data("ui-tooltip-title")),n(o),s.stop(!0),this._hide(s,this.options.hide,(function(){i._removeTooltip(e(this))})),o.removeData("ui-tooltip-open"),this._off(o,"mouseleave focusout keyup"),o[0]!==this.element[0]&&this._off(o,"remove"),this._off(this.document,"mousemove"),t&&"mouseleave"===t.type&&e.each(this.parents,(function(t,n){e(n.element).attr("title",n.title),delete i.parents[t]})),this.closing=!0,this._trigger("close",t,{tooltip:s}),this.closing=!1)},_tooltip:function(i){var n="ui-tooltip-"+t++,o=e("<div>").attr({id:n,role:"tooltip"}).addClass("ui-tooltip ui-widget ui-corner-all ui-widget-content "+(this.options.tooltipClass||""));return e("<div>").addClass("ui-tooltip-content").appendTo(o),o.appendTo(this.document[0].body),e.fn.bgiframe&&o.bgiframe(),this.tooltips[n]=i,o},_find:function(t){var i=t.data("ui-tooltip-id");return i?e("#"+i):e()},_removeTooltip:function(e){e.remove(),delete this.tooltips[e.attr("id")]},_destroy:function(){var t=this;e.each(this.tooltips,(function(i,n){var o=e.Event("blur");o.target=o.currentTarget=n[0],t.close(o,!0),e("#"+i).remove(),n.data("ui-tooltip-title")&&(n.attr("title",n.data("ui-tooltip-title")),n.removeData("ui-tooltip-title"))}))}})}(jQuery)},f411:function(e,t,i){}}]);